<?php
/**
 * List all Action API error codes:
 *
 * 1000	Invalid API Action
 *
 * 1001	Fatal error
 *
 * 2000	Supplied client cannot be found
 * 2001	Session did not have client information associated
 * 2002	Username is already taken.
 * 2003	Client supplied differes from session record
 * 2004	Supplied user credentials not found.
 *
 * 3000	Could not validate input
 * 3001	Locale not provided
 *
 * 4000	Invalid language passed with locale.
 * 4001	Failed to record session.
 *
 * 5000	Failed to retrieve domain product information
 * 5001	Failed to retrieve product information
 *
 * 6000	Failed to build invoice.
 *
 */

/**
 * Clear out any conflicting session variables since the API_APP
 * does not require a continuous session.
 */
$_REQUEST["sid"] = null;
$_REQUEST["mb_shopping_session"] = null;
$_REQUEST["PHPSESSID"] = null;
$_COOKIE = null;
$_SESSION = null;
$CONF = array();

/**
 * Define the available actions.
 */
$apiActions = array(
	"createInvoice" => "CreateInvoice",
	"createPackage" => "CreatePackage",
	"getBillingAccounts" => "GetBillingAccounts",
	"getClients" => "GetClients",
	"getClientStatus" => "GetClientStatus",
	"getLogEnabledCommands" => "GetLogEnabledCommands",
	"getMBAPIMap" => "GetMBAPIMap",
	"getOrderForms" => "GetOrderForms",
	"getProducts" => "GetProducts",
	"updatePackage" => "UpdatePackage",

	"apiRunEvents" => "APIRunEvents",

	// reports
	"getReportDailySignups" => "GetReportDailySignups",
	"getReportDailyInvoices" => "GetReportDailyInvoices",

	"getReportCustomerPaymentsByDate" => "GetReportCustomerPaymentsByDate",
	"getReportActivePerServerGroup" => "GetReportActivePerServerGroup",
	"getReportPaymentsByPaymentType" => "GetReportPaymentsByPaymentType",
	"getReportActivePerServer" => "GetReportActivePerServer",
	"getReportPackagesByStatus" => "GetReportPackagesByStatus",
	"getReportMostPopularPackages" => "GetReportMostPopularPackages",
	"getReportActivePackagesByCycle" => "GetReportActivePackagesByCycle",
	"getReportBilledServerRevenue" => "GetReportBilledServerRevenue",

	"getReportRecurringServerRevenue" => "GetReportRecurringServerRevenue",
	"getReportRevenuePerServer" => "GetReportRevenuePerServer",

	// misc marketplace api actions
	"createVendorUser" => "CreateVendorUser",

	// marketplace api actions that the vendor hits on the sellers mb
	"getCompanyInformation" => "GetCompanyInformation",
	"updateVendorInformation" => "UpdateVendorInformation",

	// need to be finished:
	"updateSellerStatus" => "UpdateSellerStatus",
	"syncSellerInventory" => "SyncSellerInventory",
	"sendSellerMessage" => "SendSellerMessage",

	"manageSellerProduct" => "ManageSellerProduct",
	"addVendorInvoices" => "AddVendorInvoices",

	// marketplace api actions that the seller hits on the vendors mb
	"getVendorInformation" => "GetVendorInformation",
	"createMarketplaceSeller" => "CreateMarketplaceSeller",
	"updateSellerInformation" => "UpdateSellerInformation",
	"forwardToClientInterface" => "ForwardToClientInterface",

	// need to be finished:
	"requestProductSync" => "RequestProductSync",
	"getVendorProducts" => "GetVendorProducts",
	"requestVendorInvoiceSync" => "RequestVendorInvoiceSync",

	// marketplace vendor provisioning actions (seller hits on vendors mb)
	"createVendorPackage" => "CreateVendorPackage",
	"suspendVendorPackage" => "SuspendVendorPackage",
	"unsuspendVendorPackage" => "UnsuspendVendorPackage",
	"deleteVendorPackage" => "DeleteVendorPackage",
	"cancelVendorPackage" => "CancelVendorPackage",

	"renewV4License" => "RenewV4License",
	"getMMResaleLink" => "GetMMResaleLink",
	"resellerLicenseRegistration" => "ResellerLicenseRegistration",
	"getMGBACustomerInfo" => "GetMGBACustomerInfo",
	"getClientBalances" => "GetClientBalances",

	// client contacts management API
	"createClientContact" => "CreateClientContact",
	"updateClientContact" => "UpdateClientContact",
	"deleteClientContact" => "DeleteClientContact",

	// Forgot customer password (for Plesk UI integration)
	"forgotPassword" => "ForgotPassword",

	"createSession" => "CreateSession",
);

$CONF["actionName"] = $apiActions[$_REQUEST["apiAction"]];

/**
 * Setup the path for this API_APP
 */
chdir(dirname(__FILE__));
require_once(dirname(__FILE__)."/path.php");

/**
 * The API_APP is ALWAYS local
 */
$CONF["installType"] = "local";

/**
 * Define that this is an API based APP
 */
define("ACCESS_TYPE", API_APP);
define("DEBUG", 0);

/**
 * Setup common for this API_APP
 */
require_once(MB_LIBPATH."/lib-tk/include/common.php");

/**
 * Dispatch the action for this API_APP
 */
$apiAction = bgpc("apiAction");
if ( !isset($apiActions[$apiAction]) )
	Error::api(1000, "Invalid API Action: {$apiAction}");

try {
	dispatchAction("api", $apiActions[$apiAction]);
} catch ( InvalidAttributes $e ) {
	if ( "" == $e->getCustomMessage() ) {
		$invalid_attributes = $e->getInvalidAttributes();
		$invalid_attrubute = reset($invalid_attributes);
		$error_message = "Invalid parameters: ".(false === getDBXMLCol($invalid_attrubute) ? $invalid_attrubute : getDBXMLCol($invalid_attrubute));
		while ( false !== ($invalid_attrubute = next($invalid_attributes)) )
			$error_message .= ", ".(false === getDBXMLCol($invalid_attrubute) ? $invalid_attrubute : getDBXMLCol($invalid_attrubute));
	} else {
		$error_message = $e->getMessage();
	}
	$error_message .= "\n{$e->getTraceAsString()}";
	Error::api($e->getCode(), utf8_xml_entity_encode($error_message));
} catch ( ProductException $e ) {
	$error_message = "{$e->getMessage()}\n{$e->getTraceAsString()}";
	Error::api($e->getCode(), utf8_xml_entity_encode($error_message));
} catch ( Exception $e ) {
	$error_message = "{$e->getMessage()}\n{$e->getTraceAsString()}";
	Error::api(1001, utf8_xml_entity_encode($error_message));
}
