<?php
require_once("lib-tk/shield.php");
require_once("lib-api-action/apiaction.php");

class UpdateClientContact extends MB_AdminAPIAction
{
	public function execute()
	{
		parent::execute();

		if ( !is_null($clientContactID = bgpc("clientContactID")) )
			$contact = Factory()->ClientContact($clientContactID);
		else if ( !is_null($clientContactGUID = bgpc("clientContactGUID")) )
			$contact = Factory()->ClientContact($clientContactGUID, "guid");
		else if ( !is_null($clientID = bgpc("clientID")) )
			$contact = Factory()->Client($clientID)->PrimaryContact;
		else if ( !is_null($clientGUID = bgpc("clientGUID")) )
			$contact = Factory()->Client($clientGUID, "guid")->PrimaryContact;
		else
			throw new Exception("Missing parameter 'clientContactID' or 'clientContactGUID'");

		$this->updateContact($contact);

		return $this->output();
	}

	protected function updateContact(ClientContact $contact)
	{
		foreach ( ClientContact::$attrsDescriptor as $attr_name => $attr_desc )
		{
			switch ($attr_name)
			{
			case "client_contact_id":
			case "client_id":
				continue;
			case "language_id":
				if ( !is_null($language = bgpc("language")) ) {
					require_once("object_managers/LanguageManager.php");
					$contact->language_id = LanguageManager::getID($language);
				}
				continue;
			case "countries_id":
				if ( !is_null($country = bgpc("country")) ) {
					require_once("object_managers/CountryManager.php");
					$contact->countries_id = CountryManager::getID($country);
				}
				continue;
			case "contact_password":
				if ( !is_null($plain_password = bgpc("contactPassword")) )
					$contact->contact_password = getPasswordHash($plain_password);
				continue;
			default:
				if ( false === ($param_name = getDBXMLCol($attr_name)) )
					continue;
				if ( is_null($param_value = bgpc($param_name)) )
					continue;
				$contact->$attr_name = $param_value;
			}
		}
		if ( !is_null($emailActions = bgpc("emailActions")) )
		{
			$email_action_ids = array();
			if ( is_array($emailActions) )
				foreach ( $emailActions as $email_action_name )
					$email_action_ids[] = getAPITypeID("emailAction", $email_action_name);
			$contact->emailActionIDs = $email_action_ids;
		}

		require_once("object_managers/ClientContactManager.php");
		ClientContactManager::update($contact, $plain_password);
	}
}