<?php

require_once("lib-action/action.php");

class EditAdminGroupResponse extends MB_Action
{
	public $params = array("adminGroupID", "adminGroupName");

	public function execute()
	{
		parent::execute();

		$adminGroupID = bgpc("adminGroupID");
		if(!$adminGroupID) {
			$this->addError(TRANS_ACTIONERROR, "No AdminGroup ID given.");
		} else {
			if (!Billing\validate_forgery_protection_token()) {
				redirect("admins.php", "ShowAdminGroupDetails", array("adminGroupID" => $adminGroupID));
			}

			$params = array();

			// --------------------------------------
			// Data: GetAdminGroups
			// --------------------------------------
			$getParams = array
			(
				"getAdminData" => 1,
				"getActionData" => 1,
			);
			$query = $this->getMBAPIQuery("GetAdminGroups","",$getParams);
			require_once("lib-mbapi/mbapi.php");
			$data = dispatchMBAPI($query, 1);

			// set adminGroup smarty var (only one group returned)
			$adminGroup =& $data["mbapi"][0]["results"][0]["adminGroups"][0]["adminGroup"][0];

			// --------------------------------------
			// Relations: adminID diffs
			// --------------------------------------
			$adminIDsInGroup = $adminRelParams = array();

			if ($_REQUEST["relKey"]=="adminID") {
				// assign all adminIDs in the group to $adminIDsInGroup for later use
				if ($adminGroup["admins"][0]["admin"]) {
					foreach($adminGroup["admins"][0]["admin"] as $admin) {
						$adminIDsInGroup[] = $admin["adminID"][0];
					}
				}
				// we need to test to see if no adminIDs were passed in from the combo box.
				// if none exist, this means we must be removing all admins from this group
				if (!is_array($_REQUEST["addAdminIDs"])) {
					$_REQUEST["addAdminIDs"] = array();
				}
				// find ADD diffs & add to params
				$adminIDs_to_add = array_diff($_REQUEST["addAdminIDs"],$adminIDsInGroup);
				if (count($adminIDs_to_add)>0) {
					$adminRelParams["adminIDs"]["adminID"] = $adminIDs_to_add;
				}
				// find REMOVE diffs & add to params
				$adminIDs_to_remove = array_diff($adminIDsInGroup,$_REQUEST["addAdminIDs"]);
				if (count($adminIDs_to_remove)>0) {
					$adminRelParams["removeAdminIDs"]["adminID"] = $adminIDs_to_remove;
				}
			}

			// --------------------------------------
			// Relations: actionName diffs
			// --------------------------------------
			$actionNamesInGroup = $addActionNames = $actionRelParams = array();

			if ($_REQUEST["relKey"]=="actionName") {
				// assign all actionNames in the group to $actionNamesInGroup for later use
				if (is_array($adminGroup["actions"][0]["actionName"])) {
					$actionNamesInGroup =& $adminGroup["actions"][0]["actionName"];
				}
				// we have to pull out multiple comboAdd_X values from $_REQUEST
				foreach($_REQUEST as $key => $value) {
					if (ereg("comboAdd",$key)) {
						foreach($value as $actionName) {
							$addActionNames[] = $actionName;
						}
						unset($_REQUEST[$key]);
					}
				}
				// find ADD diffs & add to params
				$actionNames_to_add = array_diff($addActionNames,$actionNamesInGroup);
				if (count($actionNames_to_add)>0) {
					$actionRelParams["actionNames"]["actionName"] = $actionNames_to_add;
				}
				// find REMOVE diffs & add to params
				$actionNames_to_remove = array_diff($actionNamesInGroup,$addActionNames);
				if (count($actionNames_to_remove)>0) {
					$actionRelParams["removeActionNames"]["actionName"] = $actionNames_to_remove;
				}
			}

			// --------------------------------------
			// Update relations only
			// --------------------------------------
			$checkRelUpdate = $relUpdateSuccess = false;

			if (count($adminRelParams)>0 || count($actionRelParams)>0) {
				$checkRelUpdate = true;
				$query = $this->getMBAPIQuery("SetAdminGroups", "update", array_merge($adminRelParams,$actionRelParams));
				$data = dispatchMBAPI($query, 1);
				if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
					$relUpdateSuccess = true;
				}
			}

			// --------------------------------------
			// Update group data only
			// --------------------------------------
			$checkEditUpdate = $editUpdateSuccess = false;

			// check to see if we are only changing the name of this group
			// if yes, do it now
			if ( /*count($params)==0 && */$_REQUEST["adminGroupName"]!="") {
				$checkEditUpdate = true;
				$query = $this->getMBAPIQuery("SetAdminGroups", "update");
				$data = dispatchMBAPI($query, 1);
				if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
					$editUpdateSuccess = true;
				}
			}

			// --------------------------------------
			// All updates and relation changes have been made
			// --------------------------------------
			if ( ($checkRelUpdate && $relUpdateSuccess) ||
				 (!$checkEditUpdate || ($checkEditUpdate && $editUpdateSuccess)) ) {

				if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
					redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
				} else {
                    if (!$adminGroupID && (int) $data["mbapi"][0]["header"][0]["lastInsertID"][0]) {
					    $adminGroupID = ((int) $data["mbapi"][0]["header"][0]["lastInsertID"][0]);
                    }
					redirect("admins.php",getActionID("ShowAdminGroupDetails"), array("adminGroupID"=>$adminGroupID));
				}

			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}