<?php
require_once("lib-action/action.php");
require_once("UI/Page.php");

class DownloadBatchFilePage extends UIPage
{
	public function createControls()
	{
		require_once("object_managers/CryptManager.php");
		if (!CryptManager::isEncryptionEnabled()) {
			$this->addInputPanel(UIFactory()->InputPanel());
			$this->addWarningMessage(UIFactory()->WarningMessage(blmsg("TRANS_BATCH_FILE_NO_ENCRYPTION_ERROR")));
			$this->submitButtons[] = UIFactory()->SubmitButton("cancel", blmsg("TRANS_CANCEL"), UIPageController::CMD_CANCEL);
			return;
		}
		$inputPanel = $this->addInputPanel(UIFactory()->InputPanel(blmsg("TRANS_ENCRYPTION_KEY_TITLE")));
		$inputPanel->addInputControl(UIFactory()->InputPassword("encryptionKey",
			CryptManager::MODE_PASSPHRASE == CryptManager::getEncryptionMode() ?
				blmsg("TRANS_ENCRYPTION_KEY_PASSPHRASE_OPTION") :
				blmsg("TRANS_ENCRYPTION_KEY_LEKPIN_OPTION"),
			true
		));
		$this->submitButtons[] = UIFactory()->SubmitButton("download", blmsg("TRANS_DOWNLOAD"), UIPageController::CMD_OK);
		$this->submitButtons[] = UIFactory()->SubmitButton("back", blmsg("TRANS_BACK"), UIPageController::CMD_CANCEL);
	}

	public function processInput()
	{
		try {
			$batchFile = Factory()->BatchFile($this->getValue("batchFileID"));
			$encryptionKey = bgpc("encryptionKey");
			require_once("object_managers/CryptManager.php");
			switch (CryptManager::getEncryptionMode()) {
			case CryptManager::MODE_LEK:
				CryptManager::checkLekPin($encryptionKey, getCurrentSession()->user_id);
			case CryptManager::MODE_PASSPHRASE:
				CryptManager::setKey($encryptionKey);
				break;
			default:
				throw new ProductException(ProductException::E_FORBIDDEN_OPERATION, blmsg("TRANS_BATCH_FILE_NO_ENCRYPTION_ERROR"));
			}
			header("HTTP/1.1 200 OK");
			header('Content-Type:  text/plain');
			header('Content-Length: ' . (filesize($batchFile->batch_file_content)));
			header("Content-Disposition: attachment; filename=\"{$batchFile->batch_file_name}\"");
			if (isset($_SERVER['HTTP_USER_AGENT']) and strpos($_SERVER['HTTP_USER_AGENT'],'MSIE')) {
				header('Content-Type: application/force-download');
			} else {
				header('Content-Type: application/octet-stream');
			}
			echo $batchFile->batch_file_content;
			exit;
		} catch (ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
			return false;
		}
	}

	protected function getSubmitButtons()
	{
		return $this->submitButtons;
	}

	private $submitButtons = array();
}

class DownloadBatchFile extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$batchFile = Factory()->BatchFile(bgpc("id"));

		$backUrl = action_url("batchfiles", "ShowBatchFiles");
		$controller  = UIFactory()->PageController($backUrl);
		$page = $controller->addPage(new DownloadBatchFilePage($controller));
		$page->title = blmsg("TRANS_BATCH_FILE_DOWNLOAD_TITLE", array("NAME" => $batchFile->batch_file_name));
		$page->parent_title = blmsg("TRANS_BATCH_FILES_MANAGER");
		$page->parent_url = $backUrl;
		$controller->setValue("batchFileID", $batchFile->id);
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
