<?php
require_once("lib-action/action.php");
require_once("UI/Page/UIBatchFileConfirmationPage.php");

class ProcessBatchFileConfirmationPage extends UIBatchFileConfirmationPage
{
	public function createControls()
	{
		parent::createControls();
		require_once("object_managers/CryptManager.php");
		if (CryptManager::isEncryptionEnabled()) {
			$encryption_panel = $this->addInputPanel(UIFactory()->InputPanel(blmsg("TRANS_ENCRYPTION_KEY_TITLE")));
			$encryption_panel->addInputControl(UIFactory()->InputPassword("encryption_key",
				CryptManager::MODE_PASSPHRASE == CryptManager::getEncryptionMode()
					? blmsg("TRANS_ENCRYPTION_KEY_PASSPHRASE_OPTION")
					: blmsg("TRANS_ENCRYPTION_KEY_LEKPIN_OPTION"),
				true
			));
		}
	}

	protected function executeAction($id)
	{
		require_once("object_managers/CryptManager.php");
		if (CryptManager::isEncryptionEnabled()) {
			if (CryptManager::isLekEnabled()) {
				CryptManager::checkLekPin(bgpc("encryption_key"), getCurrentSession()->user_id);
			}
			CryptManager::setKey(bgpc("encryption_key"));
		}
		require_once("object_managers/BatchFileManager.php");
		BatchFileManager::processBatchFile($id);
	}
}

class ProcessBatchFile extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$confirmation_page_strings = array (
			"title" => blmsg("TRANS_PROCESS_BATCH_FILE"),
			"list_title" => blmsg("TRANS_PROCESS"),
			"confirm_messages" => array( "confirm" => blmsg("TRANS_CONFIRM_PROCESSING"), "confirm_check" => blmsg("TRANS_CONFIRM_PROCESSING_MESSAGE")),
			"error_title" => blmsg("TRANS_PROCESSED_BATCH_FILES"),
			"successfully_processed_list_title" => blmsg("TRANS_BATCH_FILES_SUCCESSFULLY_PROCESSED_LIST_TITLE"),
			"empty_processed_list_title" => blmsg("TRANS_BATCH_FILES_EMPTY_PROCESSED_LIST_TITLE"),
		);

		$back_url = action_url("batchfiles", "ShowBatchFiles");
		$controller  = UIFactory()->PageController($back_url);
		$page = $controller->addPage(new ProcessBatchFileConfirmationPage( $controller, $confirmation_page_strings ));
		$page->parent_title = blmsg("TRANS_BATCH_FILES_MANAGER");
		$page->parent_url = $back_url;
		$controller->setValue("id", bgpc("id"));
		$controller->processRequest();
		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");

		return $this->hasExecutedSuccessfully();
	}
}