<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/action.php");
require_once("UI/Page.php");

class SelectClientContactPage extends UIPage
{
	public function createControls()
	{
		require_once("UI/PageController.php");

		$input_type = UIFactory()->InputDropDown("clientContactID",	blmsg("TRANS_CLIENT_CONTACT_NAME"));
		$client_contacts = Factory()->ClientContactList(array('client_id' =>  $this->getValue("clientID")));;
		foreach ( $client_contacts as $client_contact )
			$input_type->addOption(	$client_contact->id, $client_contact->contact_fullname );

		$input_panel = UIFactory()->InputPanel(blmsg("TRANS_CLIENT_CONTACT_CHOOSING"));
		$input_panel->addInputControl($input_type);
		$this->addInputPanel($input_panel);
	}

	public function processInput()
	{
		dispatchAction("clients", getActionID("AddBillingAccount"));
	}
}

class AddBillingAccount extends MB_Action
{
	public $params = array("clientID", "clientContactID");

	public function execute()
	{
		parent::execute();
		$clientID = getParam('clientID');
		$clientContactID = getParam('clientContactID');
		$command = bgpc("command");
		if(!$clientID) {
			$this->addError(TRANS_ACTIONERROR, "No client ID given.");
		} else if(!$clientContactID || "cancel" == $command) {
			$this->createSelectClientContactPage($clientID);
			return $this->hasExecutedSuccessfully();
		} else {
			global $CONF;
			$query = $this->getMBAPIQuery("GetClients", "", array("getContactData"=> "4"));
			$CONF["debug"]->add("Action Command", htmlentities($query), __FILE__, __LINE__);
			$data = dispatchMBAPI($query, 1);
			$CONF["debug"]->add("Action Result", $data, __FILE__, __LINE__);

			$results =& $data["mbapi"][0]["results"][0];
			if ($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors())
			{
				$this->smartyVars["client"] = collapseArrayElements($results["clients"][0]["client"][0]);
				$this->smartyVars["contact"] = collapseArrayElements($results["clients"][0]["client"][0]["contacts"][0]["contact"][0]);
				require_once("object_managers/ClientBillingAccountManager.php");
				$this->smartyVars["billingAccountTypes"] = ClientBillingAccountManager::getAllowedBillingAccountTypes(true);
				$this->smartyVars["bankAccountTypes"] = $CONF["types"]["bankAccount"];
				$this->smartyVars["cardTypes"] = $CONF["types"]["card"];
				$this->smartyVars["cardExtraTypes"] = $CONF["types"]["cardExtraIDs"];

				require_once("objects/PaymentGateway.php");
				$bt_gateways = Factory()->PaymentGatewayList(array("payment_gateway_type" => PaymentGateway::TYPE_HSPC_BANK_TRANSFER));
				if (0 < count($bt_gateways)) {
					$fake_billing_account = Factory()->ClientBillingAccount();
					foreach ($bt_gateways as $bt_gateway) {
						require_once('ModuleTool.php');
						$plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array("gateway_id" => $bt_gateway->id));
						$fake_billing_account->payment_gateway_id = $bt_gateway->id;
						$adapter_input_fields = $plugin->getPaymentMethodAdapter($fake_billing_account)->getInputFields(true);
						$input_fields = array();
						foreach ($adapter_input_fields as $field_name => $field_opts) {
							$input_fields[] = array(
								"field_name" => "{$field_name}_{$bt_gateway->id}",
								"field_title" => $field_opts[0],
								"field_desc" => $field_opts[2],
							);
						}
						$this->smartyVars["ddbp_accounts"][] = array(
							"gateway_id" => $bt_gateway->id,
							"gateway_name" => $bt_gateway->payment_gateway_name,
							"gateway_desc" => $bt_gateway->payment_gateway_description,
							"fields" => $input_fields,
						);
					}
					$this->smartyVars["selected_ddbp_gateway_id"] = bgpc("selected_ddbp_gateway_id");
				}

				$formDefaults = array(
					"billingAccountOrder" => "0",
					"billingAccountPercent" => "100.0",
				);
				if ($_REQUEST["comboAction"] == "AddNewClient") {
					$formDefaults["comboAction"] = "AddNewClient";
				}

				$this->mergeFormState($formDefaults);
				
				// do we have an lek stored?
				require_once("object_managers/CryptManager.php");
				$this->smartyVars["needKey"] = (CryptManager::isLekEnabled() ? 0 : 1);
				$this->addTemplate("clients/AddBillingAccount", $this->smartyVars);
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();

	}

	private function createSelectClientContactPage ( $clientID )
    {
    	$accounts_list_url = action_url("clients", "ShowBillingAccounts", array("clientID" => $clientID));
		$controller  = UIFactory()->PageController($accounts_list_url);
		$page1 = new SelectClientContactPage($controller);

		$page1->title = blmsg("TRANS_ADD_BILLING_ACCOUNT");
		$page1->parent_title = blmsg("TRANS_MANAGE_BILLING_ACCOUNTS");
		$page1->parent_url = $accounts_list_url;
		$page1->setValue( "clientID", $clientID );

		$controller->addPage($page1);
		$controller->processRequest();
		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return;
    }
}
