<?php

require_once(dirname(__FILE__)."/../../../../lib-tk/shield.php");
require_once("lib-action/genericactions.php");

class EditBillingAccountResponse extends EditResponseAction
{
	public function __construct()
	{
		if(!$billingAccount = oneOf("billingAccounts", array("billingAccountID" => (int) $_REQUEST["billingAccountID"]))) {
			$this->addError(MBAPI_ACTIONERROR, "No valid match.");
			return false;
		}

		$this->d = new DataSource("billingAccounts");
		$description = $this->d->getDescription();
		parent::__construct();

		require_once("object_managers/CryptManager.php");
		if (CryptManager::MODE_PASSPHRASE == CryptManager::getEncryptionMode()) {
			CryptManager::setKey($encKey = bgpc("encKey"));
			$this->d->mergeParams(array("encKey" => $encKey));
		} else if (CryptManager::MODE_LEK == CryptManager::getEncryptionMode()) {
			CryptManager::setDefaultKey();
		}

		unset($description['columns']['clientContactID']);

		$success_validation = true;

		$billingAccountType = (int) $_REQUEST["billingAccountType"];
		if($billingAccountType == getAPITypeID("billingAccount", "manual") || $billingAccountType == getAPITypeID("billingAccount", "secondary")) {
			unset($description["columns"]['billingBankType']);
			unset($description["columns"]['billingBankName']);
			unset($description["columns"]['billingBankState']);
			unset($description["columns"]['billingBankABACode']);
			unset($description["columns"]['billingBankAccountNum']);
			unset($description["columns"]['billingBankLicenseNum']);
			unset($description["columns"]['billingBankLicenseState']);
			unset($description["columns"]['billingBankDob']);

			unset($description["columns"]['billingCardNameOnCard']);
			unset($description["columns"]['billingCardType']);
			unset($description["columns"]['billingCardNum']);
			unset($description["columns"]['billingCardExpMonth']);
			unset($description["columns"]['billingCardExpYear']);
			unset($description["columns"]['billingCardIssueNum']);
			unset($description["columns"]['billingCardStartDate']);
		} else if($billingAccountType == getAPITypeID("billingAccount", "card")) {
			unset($description["columns"]['billingBankType']);
			unset($description["columns"]['billingBankName']);
			unset($description["columns"]['billingBankState']);
			unset($description["columns"]['billingBankABACode']);
			unset($description["columns"]['billingBankAccountNum']);
			unset($description["columns"]['billingBankLicenseNum']);
			unset($description["columns"]['billingBankLicenseState']);
			unset($description["columns"]['billingBankDob']);

			$billingCardType = (int) $_REQUEST["billingCardType"];
			$cardExtraTypes = getAPITypes("cardExtraIDs");
			if(!in_array($billingCardType, $cardExtraTypes)) {
				unset($description["columns"]['billingCardIssueNum']);
				unset($description["columns"]['billingCardStartDate']);
			}
			if (!$_REQUEST["billingCardNum"] || $_REQUEST["billingCardNum"]==""){
				unset($description["columns"]['billingCardNum']);
			}
		} else {
			unset($description["columns"]['billingCardNameOnCard']);
			unset($description["columns"]['billingCardType']);
			unset($description["columns"]['billingCardNum']);
			unset($description["columns"]['billingCardExpMonth']);
			unset($description["columns"]['billingCardExpYear']);
			unset($description["columns"]['billingCardIssueNum']);
			unset($description["columns"]['billingCardStartDate']);
		}
		$this->d->setDescription($description);
		switch ($billingAccountType)
		{
			case getAPITypeID("billingAccount", "card"):
				$validation_fields = array(
					"billingCardNameOnCard" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"billingCardExpMonth" => INPUT_RESTRICTION_MONTH|INPUT_RESTRICTION_REQUIRED,
					"billingCardExpYear" => INPUT_RESTRICTION_YEAR|INPUT_RESTRICTION_REQUIRED,
					"billingCardNum" => array(
						INPUT_RESTRICTION_INT|INPUT_RESTRICTION_MAXLEN|INPUT_RESTRICTION_MINLEN|INPUT_RESTRICTION_CARD,
						array(
							'maxLength' => 19,
							'minLength' => 13,
							'allowedCardTypes' => array(getAPITypeName("card", bgpc('billingCardType'))),
						),
					),
                );
				break;
			case getAPITypeID("billingAccount", "echeck"):
			case getAPITypeID("billingAccount", "bank"):
			case getAPITypeID("billingAccount", "wire"):
				$validation_fields = array(
					"billingBankType" => INPUT_RESTRICTION_UNSIGNED_INT,
					"billingBankName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"billingBankState" => INPUT_RESTRICTION_STRING,
					"billingBankAbaCode" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
					"billingBankAccountNum" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
					"billingBankLicenseNum" => INPUT_RESTRICTION_STRING,
					"billingBankLicenseState" => INPUT_RESTRICTION_STRING,
                );
				break;
			case getAPITypeID("billingAccount", "hspcbanktransfer"):
				require_once("ModuleTool.php");
				$gateway_id = bgpc("selected_ddbp_gateway_id");
				$plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array("gateway_id" => $gateway_id));
				$billing_account = Factory()->ClientBillingAccount(bgpc("billingAccountID"));
				$billing_account->payment_gateway_id = $gateway_id;
				$adapter = $plugin->getPaymentMethodAdapter($billing_account);
				$adapter->collectInputFields("_{$gateway_id}");
				$adapter_input_fields = $adapter->getInputFields();
				$validation_fields = array();
				foreach (array_keys($adapter_input_fields) as $field_name) {
					$validation_fields["{$field_name}_{$gateway_id}"] = INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED;
				}
				$adapter_validation = $adapter->validate();
				if (HSPC_PM_Adapter::VALIDATION_OK != $adapter_validation) {
					$success_validation = false;
					billing_put_error($adapter_validation);
				}
				break;
		}
		$this->setReferralActionID(getActionID("EditBillingAccount"));
		$this->setReferralActionFile("clients.php");
		$args = array(
			"billingAccountID" => bgpc("billingAccountID"),
			"clientID" => bgpc("clientID"),
			"clientContactID" => bgpc("clientContactID"),
			"billingAccountType" => $billingAccountType,
			"selected_ddbp_gateway_id" => $gateway_id,
		);
		$success_validation = $this->validateMultiInputs( $validation_fields, array(), array(), 1, $args ) && $success_validation;
		if (! $success_validation) {
			$this->validationRedirect($args, array(), true); //Force redirect
			exit;
		}
	}

	public function editResponseHandler()
	{
		global $CONF;
		// valid match assumed due to earlier check
		$billingAccount = Factory()->ClientBillingAccount(bgpc("billingAccountID"));
		$clientID = $billingAccount->ClientContact->Client->id;

		if ((int)$_REQUEST['defaultBillingAccount']){
			updateForKey(":clientsGeneric", $clientID, array(
				"defaultBillingAccountID" => $billingAccount->id,
			));
		}

		loadConfigs("payment", "paymentQueue");

		if (0 == Settings()->paymentQueue_enableAdvancedShareEditing) {
			// we need to remove shares for other billing accounts
			$shares = grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetBillingAccountShares",
					"params" => array(
						"billingAccountID" => $billingAccount->id,
					),
				),
			));

			$existingShares = array();
			foreach ($shares as $share){
				$existingShares[$share["packageID"]] = $share;
			}

			$newShares = array();
			$updateShares = array();
			foreach ((array)$_REQUEST["packages"] as $packageID){
				if ($existingShares[$packageID]){
					$updateShares[$existingShares[$packageID]["billingAccountShareID"]] = 1;
					unset($existingShares[$packageID]);
				} else {
					$newShares[$packageID] = 1;
				}
			}

			if ((int)$_REQUEST["packages"][0]){
				$oldShares = arrayFor(":clientBillingAccountShares", array(
					"packageID" => $_REQUEST["packages"],
				));
				foreach ((array)$oldShares as $os){
					updateForKey(":clientBillingAccountShares", $os["billingAccountShareID"], array(
						"billingAccountShareValue" => 0,
					));
				}
			}

			foreach ($newShares as $packageID=>$shareValue){
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetBillingAccountShare",
						"subCommand" => "insert",
						"params" => array(
							"billingAccountID" => $billingAccount->id,
							"packageID" => $packageID,
							"billingAccountFallbackGroup" => 1,
							"billingAccountShareGroup" => 0,
							"billingAccountShareValue" => $shareValue,
						),
					),
				)));
			}

			foreach ($updateShares as $shareID=>$shareValue){
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetBillingAccountShare",
						"subCommand" => "update",
						"params" => array(
							"billingAccountShareID" => $shareID,
							"billingAccountShareValue" => $shareValue,
						),
					),
				)));
			}

			foreach ($existingShares as $packageID=>$share){
				$shareID = $share["billingAccountShareID"];
				$shareValue = 0;
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetBillingAccountShare",
						"subCommand" => "update",
						"params" => array(
							"billingAccountShareID" => $shareID,
							"billingAccountShareValue" => $shareValue,
						),
					),
				)));
			}
		} else {
			// we need to preserve shares for other billing accounts
			$shares = grabResultsFromMBAPIArray(array(
				"mbapi" => array(
					"command" => "GetBillingAccountShares",
					"params" => array(
						"billingAccountID" => $billingAccount->id,
					),
				),
			));

			$existingShares = array();
			foreach ($shares as $share){
				$existingShares[$share["packageID"]] = $share;
			}

			$newShares = array();
			$updateShares = array();
			foreach ((array)$_REQUEST["packages"] as $packageID){
				if ($existingShares[$packageID]){
					$updateShares[$existingShares[$packageID]["billingAccountShareID"]] = 1;
					unset($existingShares[$packageID]);
				} else {
					$newShares[$packageID] = 1;
				}
			}

			foreach ($newShares as $packageID=>$shareValue){
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetBillingAccountShare",
						"subCommand" => "insert",
						"params" => array(
							"billingAccountID" => $billingAccount->id,
							"packageID" => $packageID,
							"billingAccountFallbackGroup" => 1,
							"billingAccountShareGroup" => 0,
							"billingAccountShareValue" => $shareValue,
						),
					),
				)));
			}

			foreach ($updateShares as $shareID=>$shareValue){
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetBillingAccountShare",
						"subCommand" => "update",
						"params" => array(
							"billingAccountShareID" => $shareID,
							"billingAccountShareValue" => $shareValue,
						),
					),
				)));
			}

			foreach ($existingShares as $packageID=>$share){
				$shareID = $share["billingAccountShareID"];
				$shareValue = 0;
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetBillingAccountShare",
						"subCommand" => "update",
						"params" => array(
							"billingAccountShareID" => $shareID,
							"billingAccountShareValue" => $shareValue,
						),
					),
				)));
			}

		}

		if (bgpc("billingAccountType") == ClientBillingAccount::TYPE_HSPC_BANK_TRANSFER) {
			require_once("ModuleTool.php");
			$gateway_id = bgpc("selected_ddbp_gateway_id");
			$plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array("gateway_id" => $gateway_id));
			$billingAccount->payment_gateway_id = $gateway_id;
			$adapter = $plugin->getPaymentMethodAdapter($billingAccount);
			$adapter->collectInputFields("_{$gateway_id}");
			$adapter->save();
		}

		//////////////////////////
		return true;
	}

	public function overrideValidateColumnParamType($column, $columnType, $mode, $paramType, $isRequired)
	{
		switch($column) {
			case "billingCardNum":
				if(empty($_REQUEST["billingCardNum"])) {
					$columnType = "";
					break;
				}
				$paramType |= INPUT_RESTRICTION_MAXLEN|INPUT_RESTRICTION_MINLEN|INPUT_RESTRICTION_CARD;
				$args = array(
					'maxLength' => 19,
					'minLength' => 13,
					'allowedCardTypes' => array(getAPITypeName("card", $_REQUEST['billingCardType'])),
				);
				break;
			case "billingBankAccountNum":
				if(empty($_REQUEST["billingBankAccountNum"])) {
					$columnType = "";
					break;
				}
				break;
			default:
				$args = array();
				break;
		}

		return array($paramType, $isRequired, $args);
	}

	public function redirect()
	{
		if(!$this->goBack()) {
			$billingAccounts = new DataSource("billingAccounts", array("billingAccountID" => (int) $_REQUEST["billingAccountID"]));
			$contacts = new DataSource("contacts");
			$clients = new DataSource("clients");

			$clientID = $contacts->getIDByID($clients, $contactID = $billingAccounts->getIDFor(new DataSource("contacts")));

			redirect("clients.php", getActionID("ShowBillingAccounts"), array("clientID" => $clientID, "contact" => $contactID));
		}
	}
}
