<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/genericactions.php");

class EditClientContactResponse extends EditResponseAction
{
	public function __construct()
	{
		if ( is_null($clientContactID = bgpc("clientContactID")) )
			throw new Exception("Missing parameter 'clientContactID'");

		$this->contact = Factory()->ClientContact($clientContactID);

		$validation_fields = array(
			"contactFirstName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactLastName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactEmail" => INPUT_RESTRICTION_EMAIL|INPUT_RESTRICTION_REQUIRED,
			"contactEmailFormat" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
			"contactCompany" => INPUT_RESTRICTION_STRING,
			"contactAddress1" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactAddress2" => INPUT_RESTRICTION_STRING,
			"contactCity" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactState" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"contactZip" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
			"countriesID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"languageID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
			"contactPhone1" => INPUT_RESTRICTION_PHONE|INPUT_RESTRICTION_REQUIRED,
			"contactPhone2" => INPUT_RESTRICTION_PHONE,
			"contactFax" => INPUT_RESTRICTION_PHONE,
			"contactPrimary" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
			"contactMobilePhone" => INPUT_RESTRICTION_PHONE,
			"contactCounty" => INPUT_RESTRICTION_STRING,
			"contactUseSpecifiedEmailRulesOnly" => INPUT_RESTRICTION_BOOL,
		);

		if (!is_null($contactUsername = bgpc("contactUsername")) && $contactUsername != $this->contact->contact_username ) {
			if (false !== $this->validateParam("contactUsername", array(INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED|INPUT_RESTRICTION_NOTINCOL, array("source" => "contacts", "col" => "contactUsername"))))
			$this->validateParam("contactUsername", array(INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED|INPUT_RESTRICTION_NOTINCOL, array("source" => "admins", "col" => "adminUsername")));
			$validation_fields["contactUsername"] = INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED;
		}

		$this->d = new DataSource("contacts");
		$this->setReferral();
		$this->validateMultiInputs($validation_fields, array(), array(), 1);

		parent::EditResponseAction();
	}

	public function setReferral()
	{
		$this->setReferralActionID(getActionID("EditClientContact"));
		$this->setReferralActionFile($this->d->getEntry("names", "file") . ".php");
	}

	public function editResponseHandler()
	{
		// valid match assumed due to earlier check
		$contactID = $this->contact->id;
		$emailActionIDs = (array) $_REQUEST["emailActionIDs"];

		deleteFor("contactEmailActionRels", array("clientContactID" => $contactID));
		foreach($emailActionIDs as $emailActionID) {
			insertFor("contactEmailActionRels", array("clientContactID" => $contactID, "emailActionID" => $emailActionID));
		}

		return true;
	}

	public function overrideValidateColumnParamType($column, $columnType, $mode, $paramType, $isRequired)
	{
		$args = array();
		switch($column)
		{
		case "contactEmail":
			$contactEmail = bgpc("contactEmail");
			if ( $contactEmail == $this->contact->contact_email && 1 == count(Factory()->ClientContactList(array("contact_email" => $contactEmail))) )
				break;
			$paramType |= INPUT_RESTRICTION_NOTINCOL;
			$args = array("source" => "contacts", "col" => "contactEmail");
			break;
		}
		return array($paramType, $isRequired, $args);
	}

	public function redirect()
	{
		if(!$this->goBack()) {
			redirect("clients.php", getActionID("ShowClientContacts"), array("clientID" => $this->contact->client_id));
		}
	}

	protected function validationRedirect($extraArgs = array())
	{
		$extraArgs = array_merge($extraArgs, array("clientID" => bgpc("clientID"), "clientContactID" => bgpc("clientContactID")));
		return parent::validationRedirect($extraArgs);
	}

	private $contact;
}
