<?php

require_once(dirname(__FILE__)."/../../../../lib-action/action.php");
require_once("UI/Page.php");
require_once("UI/InputPanel.php");
require_once("UI/Panel/UIConfirmationListPanel.php");

class ClientRemoveOptionsPanel extends UIInputPanel
{
	protected function getControlHTML(UIInputControl $control)
	{
		$class_error = ($control->isValid()) ? "" : 'class="error"';
		$title = billing_safetyhtml($control->title);
		return <<<HTML
			<tr {$class_error}>
				<td width="100%">{$control->getHTML()}</td>
			</tr>
HTML;
	}
}

class RemoveClientPage extends UIPage
{
	public function createControls()
	{
		$this->title = blmsg("TRANS_REMOVAL_CONFIRMAITON");
		$this->parent_title = blmsg("TRANS_MANAGE_CLIENTS");
		$this->parent_url = action_url("clients", "ShowClients");

		$removeform = UIFactory()->Form();
		$clients_panel = new UIConfirmationListPanel(blmsg("TRANS_REMOVE_CLIENT_LEGEND"), false, array("confirm" => blmsg("TRANS_CONFIRM_REMOVAL"), "confirm_check" => blmsg("TRANS_CONFIRM_REMOVAL_MESSAGE")));
		$options_panel = new ClientRemoveOptionsPanel(blmsg("TRANS_REMOVE_OPTIONS"));
		$removeform->addInputPanel($clients_panel);
		$removeform->addInputPanel($options_panel);
		$removal_list = $clients_panel->addListControl(UIFactory()->List());

		$clients = $this->getClients($this->getValue("client_ids"));

		foreach ($clients as $client)
		{
			$contact_fullname = "";
			try {
				$primary_contact = $client->PrimaryContact;
				$contact_fullname = $primary_contact->contact_fullname;
			} catch (ProductException $e) {
				if ( ProductException::E_OBJECT_NOT_FOUND != $e->getCode()  ) {
					throw $e;
				}
			}
			if ("" == $contact_fullname )
				$contact_fullname = "Client #{$client->client_id}";
			$client_fullname = UIFactory()->InputLabel();
			$client_fullname->value = $contact_fullname;
			$hidden = UIFactory()->InputHidden("id[]");
			$hidden->value = $client->client_id;
			$removal_list->addInputControl($client_fullname);
			$removeform->addHiddenControl($hidden);
		}

		$input_remove_type = UIFactory()->InputRadio("client_remove_type", blmsg("TRANS_REMOVE_OPTIONS"), false,
			array($this, "checkRemoveOption")
		);
		$client_active = $this->getValue("client_active");
		$hidden = UIFactory()->InputHidden("client_active");
		$hidden->value = $client_active;
		$removeform->addHiddenControl($hidden);
		if ("1" == $client_active)
		{
			$input_remove_type->addOption(
				"archive",
				blmsg("TRANS_ARCHIVE_THIS_CLIENT_AND_SAVE"),
				blmsg("TRANS_ARCHIVE_NOTICE_ACTIVE_PACKAGES")
			);
		}
		$input_remove_type->addOption(
			"purge",
			blmsg("TRANS_PURGE_THIS_CLIENT_AND_DELETE"),
			blmsg("TRANS_PURGE_WARNING")
		);
		$options_panel->addInputControl($input_remove_type);

		$this->addForm($removeform);
		$this->form = $removeform;
	}

	public function checkRemoveOption($value, &$message)
	{
		if ( is_null($value)) {
				$message = blmsg("TRANS_REMOVE_OPTION_EMPTY");
				return false;
		}
		return true;
	}

	public function processInput()
	{
		dispatchAction("clients", getActionID("RemoveClientResponse"));
	}

	private function getClients(array $client_ids)
	{
		$clients = array();
		foreach ($client_ids as $client_id)
			$clients[] = Factory()->Client($client_id);

		return $clients;
	}

	private $form;	//
}

class RemoveClient extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$client_ids = bgpc("id");
		if ( empty($client_ids) )
			throw new Exception("Missing parameter 'id'");

		$controller  = UIFactory()->PageController(action_url("clients", "ShowClients"));
		$controller->addPage(new RemoveClientPage($controller));
		$controller->setValue("client_ids", $client_ids);
		$controller->setValue("client_active", bgpc("client_active"));
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}

?>
