<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/action.php");

class SendClientEmailResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		// if the client isn't specified we can't do much with it
		if(!notZero( $_REQUEST["clientID"] )) {
			$this->addError( TRANS_ACTIONERROR, "Client ID is not given." );
			$this->addErrorTemplates();

			return $this->hasExecutedSuccessfully();
		}
		$clientID = (int)$_REQUEST["clientID"];
		$body = bgpc("emailBody");

		$contacts =array();	// list of destinations
		$contactPrimary =0;	// who be the primary

		$customBodies =array();	// if we have variables to replace in the email text, this is the custom
		$customVars =array();	// what vars we're looking for

		// check if we have custom vars to replace
		if (preg_match_all( '[{\w+}]', (string)$_REQUEST['emailBody'], $matches, PREG_SET_ORDER )) {
			// if I'm here I must have matches
			$matches = array_values( $matches );
			for( $i=0, $n=count( $matches ); $i<=$n; $i++ ) {
				if (notNull( $matches[ $i ][0] ) && !in_array( (string)$matches[ $i ][0], $customVars ))
					$customVars[] = (string)$matches[ $i ][0];
			}
		}

		if(is_array($_REQUEST["addClientContactIDs"])) {
			$clientContactIDs = $_REQUEST["addClientContactIDs"];

			$query = new MBQuery("GetClients", "", array(
				"clientID" => (int)$_REQUEST["clientID"],
				"getContactData" => 1,
				"mergePrimaryContact" => 1));
			$data = $query->getResult();
			unset( $query );

			if (!$data->getErrorCount()) {
				$result = grabSimpleMBAPIResults( $data->getStructure() );
				foreach( (array)$result as $clientInfo ) {

					foreach( (array)$clientInfo['contacts'][0]['contact'] as $key => $contact) {

						if (notNull( $customVars )) {	// do the replacement
							/// this is ugly and slow, but will do the trick
							$customBodies[ $contact['contactEmail'] ] = $body;

							foreach ( (array)$customVars  as $var ) {
								$frmVar = ( stripos( $var, 'contact' ) ? 'contact' : 'clientInfo' );
								$customBodies[ $contact['contactEmail'] ] =
									str_replace(
										$var,
										${$frmVar}[ str_replace( array('{','}'), null, $var ) ],
										$customBodies[ $contact['contactEmail'] ] );
							}	unset( $frmVar, $var );
						}

						if(in_array($contact["clientContactID"], $clientContactIDs)) {
							$contacts[ $contact["contactEmail"] ] = array(
								'name' => billing_get_full_name($contact["contactFirstName"], $contact["contactLastName"]),
								'address' => $contact["contactEmail"]);
						}
						if(notZero($contact["contactPrimary"])) {
							$contactPrimary = $contact["clientContactID"];
						}
					}		unset( $key, $contact, $result );
				}	unset( $clientInfo );
			} else {
			 	list( $title, $msg )= each( $data->getErrors() );
				$this->addError( $title, $msg );
				unset( $title, $msg );
			}

			unset( $data );
		} else {
			//No contacts found
		}

		if(!$contactPrimary) {
			$contactPrimary = $clientContactIDs[0];
		}

		//Look for CC and BCC
		$ccAddresses = $bccAddresses = null;
		if($_REQUEST["emailCC"]) {
			$ccAddresses =explode(",",$_REQUEST["emailCC"]);
		}
		if($_REQUEST["emailBCC"]) {
			$bccAddresses =explode(",",$_REQUEST["emailBCC"]);
		}
		$subject = bgpc("emailSubject");

		//This will only allow messages to go out to e-mail addresses inside our system
		if(notNull($contacts) && ($body || $subject)) {
			if (notNull( $customVars )) {
				foreach( (array)$customBodies as $dest => $body ) {
					$this->sendEmail( $subject, $body, $ccAddresses, $bccAddresses, array($contacts[ $dest ]), $contactPrimary );
				}
			} else {
				$this->sendEmail( $subject, $body, $ccAddresses, $bccAddresses, $contacts, $contactPrimary );
			}

			if (!$this->getNumErrors()) {
				redirect("clients.php", "ShowClientDetails", array("clientID"=>$clientID));
			}
		} else {
			$this->addError(TRANS_ACTIONERROR, "No Subject or Body given.");
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	private function sendEmail( $subject, $body, $ccAddresses, $bccAddresses, $contacts, $contactPrimary )
	{
		global $CONF;

		$message = array();
		$message["emailSubject"] = $subject;

		$message["emailBody"] = $body;
		$message["emailTextBody"] = $body;

		$message["emailFromAddress"] = $CONF["user"]["email"];
		$message["emailFromName"] = $CONF["user"]["firstName"] ." " .$CONF["user"]["lastName"];
		$message["emailDateCreated"] = mktime();
		$message["emailDateSent"] = mktime();
		$message["emailActive"] = 1;
		$message["emailContactID"] = $contactPrimary;

		if (utf8_html_entity_decode( $_REQUEST["emailBody"] ) != $_REQUEST["emailBody"]) {	// has html special chars!
			$message["emailSentType"] = 1;
			$message["emailTextBody"] = strip_tags($message["emailTextBody"]);
		}


		$message['emailToAddresses']['emailAddress'] = array_values($contacts);

		if($ccAddresses) {
			$message['emailCCAddresses']['emailAddress'] =array();
			foreach( (array)$ccAddresses as $value) {
				$message['emailCCAddresses']['emailAddress'][] = array('address' => $value);
			}	unset( $value );
		}

		if($bccAddresses) {
			$message['emailBCCAddresses']['emailAddress'] =array();

			foreach( (array)$bccAddresses as $value) {
				$message['emailBCCAddresses']['emailAddress'][] = array('address' => $value);
			}
		}

		if ( !is_null($invoiceID = bgpc("invoiceID")) ) {
			$message["invoiceID"] = $invoiceID;
		}

		if ( !is_null($emailAttachmentIDs = bgpc("emailAttachmentIDs")) ) {
			$message["emailAttachments"]["emailAttachmentID"] = $emailAttachmentIDs;
		}

		$sendEmails = new MBQuery("ProcessEmail", "", $message);
		$sendEmails = $sendEmails->getResult();

		if (!$sendEmails->getErrorCount()) {
			return true;
		} else {
			list( $title, $msg ) = each( $sendEmails->getErrors() );
			$this->addError( $title, $msg );
			return false;
		}
	}
}

