<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/action.php");

/**
 * This class is a child class of MB_Action. It displays a form to allow immediate charging of
 * a client for due invoices.
 *
 * @author  Kris Bailey
 */
class TakeClientPaymentResponse extends MB_Action
{
	public $params = array("clientID", "invoiceIDs");

	public function execute()
	{
		global $CONF;
		parent::execute();
		loadConfigs("payment");

		$clientID = getParam('clientID');
		$invoiceIDs = (array)$_REQUEST["invoiceIDs"];
		if(!$clientID) {
			$this->addError(TRANS_ACTIONERROR, "No client ID given.");
		} elseif (!$invoiceIDs || !(int)$invoiceIDs[0]) {
			$this->addError(TRANS_ACTIONERROR, "No invoice ID's given.");
		} else {
			require_once("object_managers/CryptManager.php");
			if (CryptManager::isEncryptionEnabled() && CryptManager::isLekEnabled()) {
				try {
					CryptManager::checkLekPin(bgpc("encKey"), getCurrentSession()->user_id);
				} catch(ProductException $e) {
					billing_put_error($e->getMessage());
					redirect("clients.php", getActionID("TakeClientPayment"), array("clientID" => $clientID));
				}
			}

			$this->smartyVars["client"] = oneByID("clients", $clientID, array(
				"mergePrimaryContact" => 1,
			));

			$this->paymentAction = Settings()->paymentQueue_transactionAction;
			if ($this->paymentAction!="auth"){
				$this->paymentAction = "charge";
			}
			$runCmd = array(
				"mbapi" => array(
					"command" => "ProcessPaymentQueue",
					"params" => array(
						"invoiceID" =>  $invoiceIDs,
						"generateNew" => 1,
						"runPaymentQueue" => 1,
						"getTransactionData" => 1,
						"transactionAction" => $this->paymentAction,
						"key" => $_REQUEST["encKey"],
						"ignoreRetryTime" => (int)$_REQUEST["ignoreRetryTime"]
					),
				),
			);
			
			$results = dispatchMBAPI(toXML($runCmd), 1);
			
			if ($this->addMBAPIErrors($results) === 0 && !$this->getNumErrors()) {
			
				$this->smartyVars["transactionInfo"] = correctArray($results["mbapi"][0]["results"][0]);
	
				$billingAccountIDs = array();
				foreach ((array)$this->smartyVars["transactionInfo"]["transaction"] as $tr){
					$billingAccountIDs[] = $tr["billingAccountID"];
				}
				$billingAccounts = arrayFor("billingAccounts", array(
					"billingAccountID" => $billingAccountIDs,
				));
				$this->smartyVars["billingAccounts"] = array();
				foreach ($billingAccounts as $x=>$ba){
					$this->smartyVars["billingAccounts"][$ba["billingAccountID"]] =& $billingAccounts[$x];
				}
				$this->smartyVars["paymentStatuses"] = $CONF["statuses"]["transactions"];
				if (!$this->smartyVars["transactionInfo"]){
					$this->smartyVars["noTransactions"] = 1;
				}

				require_once("objects/ClientBillingAccount.php");
				$this->smartyVars["billingAccountTypes"] = ClientBillingAccount::types();
			} else {
				$this->smartyVars["errors"] = correctArray($results["mbapi"][0]["header"][0]["errors"][0]["error"]);
			}

			$this->addTemplate("clients/TakeClientPaymentResponse", $this->smartyVars);
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
