<?php
require_once("lib-action/action.php");
require_once("UI/Page.php");

class EditConfigParamsPage extends UIPage
{
	public function createControls()
	{
		$inputPanel = UIFactory()->InputPanel();

		$configGroupDescription = Factory()->ConfigGroup(bgpc("configGroupID"))->config_group_description;
		if(!is_null($configGroupDescription)) {
			$descriptionPanel = UIFactory()->InputHTML("configGroupDescription");
			$descriptionPanel->value = "<p>".blmsg($configGroupDescription)."</p>";
			$inputPanel->addInputControl($descriptionPanel);
		}

		foreach(Factory()->ConfigGroup(bgpc("configGroupID"))->ConfigParams as $configParam) {
			$description = array(
				"id"		=> $configParam->config_param_name,
				"title"		=> $configParam->config_param_trans_key,
				"desc"		=> $configParam->config_param_description,
				"type"		=> $configParam->config_param_input_type,
				"required"	=> (1 == $configParam->config_param_required),
				"checker"	=> $this->getChecker($configParam->config_param_name),
				"extention"	=> ("f" == $configParam->config_param_input_type && "" != $configParam->config_param_value) ? blmsg("TRANS_FILE_UPLOADED_MSG") : NULL,
				"items"		=> $this->getParamValues($configParam->config_param_id)
			);
			$inputControl = UIFactory()->InputConfig($description);
			$inputControl->value = $configParam->config_param_value;
			$inputPanel->addInputControl($inputControl);
			$this->inputControls[$configParam->config_param_id] = $inputControl;
		}
		$this->addInputPanel($inputPanel);
		$this->getDefaultForm()->setMultipartEncryption();
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function processInput()
	{
		try {
			$conn = DatabaseManager::getConnection();
			require_once("DBTransaction.php");
			$transaction = new DBTransaction($conn);

			foreach($this->inputControls as $id => $control) {
				$this->saveValue($id, $control->value, $conn);
			}

			$transaction->commit();
			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SETTINGS_SAVED_SUCCESSFULLY");
		} catch (ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
		}

		return !$this->hasWarnings() && !$this->hasErrors();
	}

	protected function getSubmitButtons()
	{
		return array(UIFactory()->SubmitButton("ok", blmsg("TRANS_SAVE"), UIPageController::CMD_OK));
	}

	private function saveValue($configParamID, $configParamValue, $conn)
	{
		$invoiceLogoParam = Factory()->ConfigGroup(0)->getChild("billing")->getChild("pdfInvoice")->getConfigParam("invoice_logo");
		if ($configParamID == $invoiceLogoParam->config_param_id) {
			$file = $_FILES[$invoiceLogoParam->config_param_name];
			if ("" == $file["name"]) {
				return;
			}

			require_once('FileDownloadClass.php');
			$ext = FileDownloadClass::getFileExt($invoiceLogoParam->config_param_name);
			$defaultLogoName = "logo.".$ext;
			require_once('object_managers/InvoiceManager.php');
			if (!in_array($ext, InvoiceManager::getPdfImageTypes())) {
				throw new ProductException(ProductException::E_FORBIDDEN_OPERATION, blmsg("TRANS_INCORRECT_IMAGE_FILE_TYPE", array("IMAGE_TYPE" => $ext)));
			}
			FileDownloadClass::downloadFile(Config()->invoiceLogoDirectory, $invoiceLogoParam->config_param_name, $defaultLogoName);
			$configParamValue = $defaultLogoName;
			if ($configParamValue != $invoiceLogoParam->config_param_value && InvoiceManager::DEFAULT_INVOICE_LOGO_FILE != $invoiceLogoParam->config_param_value) {
				unlink(Config()->invoiceLogoDirectory."/".$invoiceLogoParam->config_param_value);
			}
		}

		$configParamValue = (!is_array($configParamValue)) ? $configParamValue : $configParamValue[0];
		$conn->exec("UPDATE config_params SET config_param_value = ".$conn->quote($configParamValue)." WHERE config_param_id = {$configParamID}");
	}

	private function getChecker($paramName)
	{
		$checkers["sync"] = array(
			"count" => Billing_Checker::P_INT_NULL
		);

		$checkers["invoice"] = array(
			"billingDayOfMonth" => Billing_Checker::P_UINT_NOT_NULL,
			"billingProrationThreshold" => Billing_Checker::P_UINT_NOT_NULL,
			"invoice_sequence" => Billing_Checker::P_UINT_NOT_NULL,
			"invoice_generate_offset" => Billing_Checker::P_UINT_NOT_NULL,
			"invoice_domain_generate_offset" => Billing_Checker::P_UINT_NOT_NULL,
			"invoice_due_offset" => Billing_Checker::P_UINT_NOT_NULL,
		);

		$checkers["systemQueue"] = array(
			"max_system_queue_events" => Billing_Checker::P_UINT_NOT_NULL,
			"system_queue_push_time" => Billing_Checker::P_UINT_NOT_NULL
		);

		$checkers["autoSuspension"] = array(
			"autoSuspendFirstNotice" => Billing_Checker::P_UINT_NOT_NULL,
			"autoSuspendSecondNotice" => Billing_Checker::P_UINT_NOT_NULL,
			"autoSuspendThirdNotice" => Billing_Checker::P_UINT_NOT_NULL,
			"autoSuspendClean" => Billing_Checker::P_UINT_NOT_NULL
		);

		$checkers["pdfInvoice"] = array(
			"horizontal_indent" => Billing_Checker::P_INT_NULL,
			"vertical_indent" => Billing_Checker::P_INT_NULL,
			"horizontal_size" => Billing_Checker::P_INT_NULL,
			"vertical_size" => Billing_Checker::P_INT_NULL,
			"font_size" => Billing_Checker::P_INT_NULL
		);

		$configGroupName = Factory()->ConfigGroup(bgpc("configGroupID"))->config_group_name;
		return isset($checkers[$configGroupName][$paramName]) ? $checkers[$configGroupName][$paramName] : NULL;
	}

	private function getParamValues($configParamID)
	{
		$query = "SELECT config_param_value_name as value, config_param_value_trans_key as title
			FROM config_param_values
			WHERE config_param_id = {$configParamID} ORDER BY config_param_value_order";

		$values = array();
		foreach(DatabaseManager::getConnection()->getAll($query) as $value) {
			$values[] = array($value["value"], $value["title"]);
		}
		return $values;
	}
}

class EditConfigParams extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if(!isset($_REQUEST["configGroupID"])) {
			throw new Exception("Missing parameter 'configGroupID'");
		}

		$configGroup = bgpc("configGroupID");

		$editableConfigGroups = array("versions", "clientSidePaymentOptions", "managementrules", "sync", "invoice", "systemQueue", "autoSuspension", "display", "pdfInvoice");
		if(!in_array(Factory()->ConfigGroup($configGroup)->config_group_name, $editableConfigGroups)) {
			throw new Exception("Config group '{$configGroup}' is not editable");
		}

		$controller  = UIFactory()->PageController();
		$controller->addPage($edit_page = new EditConfigParamsPage($controller));

		$edit_page->title = Factory()->ConfigGroup($configGroup)->config_group_trans_key;
		$edit_page->parent_title = "TRANS_MANAGE_SYSTEM";
		$edit_page->parent_url = admin_action_url("configs", "system", "show");

		$controller->processRequest();
		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}