<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/action.php");

class EditEncryptionConfigsConfirm extends MB_Action
{
	public function execute()
	{
		parent::execute();

		try {
			if ("backup" == bgpc("command")) {
				$this->backupLek();
				return $this->hasExecutedSuccessfully();
			}

			if ("restore" == bgpc("command")) {
				$this->restoreLek();
				return $this->hasExecutedSuccessfully();
			}

			$this->validateRequest();

			require_once("object_managers/CryptManager.php");
			CryptManager::update(array("encryption_key_expiration_interval" => bgpc("encryption_key_expiration_interval")));

			require_once("object_managers/AdminManager.php");
			if (AdminManager::getAdmin()->isMasterAdmin()) {
				CryptManager::updateLekBackupOperator(1, "" == bgpc("firstLekBackupOperatorID") ? null : bgpc("firstLekBackupOperatorID"));
				CryptManager::updateLekBackupOperator(2, "" == bgpc("secondLekBackupOperatorID") ? null : bgpc("secondLekBackupOperatorID"));
			}

			if ( $this->noEncryptionChanged() ) {
				redirect(admin_action_url("configs", "system", "show"));
			}
		} catch (ProductException $e) {
			$trans_keys = array(
				"encryption_agree" => "TRANS_ENCRYPTION_AGREE",
				"encryption_mode" => "TRANS_ENCRYPTION_MODE_OPTION",
				"old_pass_phrase" => "TRANS_OLD_PASSPHRASE",
				"old_pass_phrase_confirm" => "TRANS_OLD_PASSPHRASE_CONFIRM",
				"old_lek" => "TRANS_OLD_LEK_HASH",
				"lek_pin" => "TRANS_LEK_PIN",
				"lek_pin_confirm" => "TRANS_LEK_PIN_CONFIRM",
				"pass_phrase" => "TRANS_PASSPHRASE",
				"pass_phrase_confirm" => "TRANS_PASSPHRASE_CONFIRM",
				"lekBackupFile" => "TRANS_ENCRYPTION_LEK_RESTORE_LEK_OPTION",
			);
			if ( "InvalidAttributes" == get_class($e) && "" == $e->getCustomMessage() ) {
				foreach ( $e->getInvalidAttributes() as $attr_name ) {
					$field_name = isset($trans_keys[$attr_name]) ? blmsg($trans_keys[$attr_name]) : $attr_name;
					billing_put_error(blmsg("TRANS_INVALID_FIELD_MESSAGE", array("FIELD_NAME" => $field_name)));
				}
			} else {
				billing_put_error($e->getMessage());
			}
			redirect("configs.php", getActionID("EditEncryptionConfigs"));
		}

		foreach(
			array(
				"encryption_agree",
				"encryption_mode",
				"old_pass_phrase",
				"old_pass_phrase_confirm",
				"old_lek",
				"lek_pin",
				"lek_pin_confirm",
				"pass_phrase",
				"pass_phrase_confirm",
				"priorCypherType",
				"neededCypherType",
			) as $param
		) {
			$this->smartyVars["propagate"][$param] = bgpc($param);
		}

		$neededCypherType = bgpc("neededCypherType");
		$priorCypherType = bgpc("priorCypherType");
		if ( "" == $priorCypherType ) {
			$priorCypherType = $neededCypherType;
		}

		$this->smartyVars["from"] = array(
			"type" => getTypeName("encryption", getAPITypeID("encryption", $priorCypherType)),
		);
		require_once("object_managers/CryptManager.php");
		if ( !CryptManager::isEncryptionEnabled() ) {
			$this->smartyVars["from"]["mode"] = "none";
		} else if( CryptManager::isLekEnabled() ) {
			$this->smartyVars["from"]["mode"] = "lek";
			$this->smartyVars["from"]["date"] = BillingLocale::getDateTimeMedium(strtotime(CryptManager::getEncryptionKeyLastRenewalDate()));
		} else {
			$this->smartyVars["from"]["mode"] = "passphrase";
			$this->smartyVars["from"]["date"] = BillingLocale::getDateTimeMedium(strtotime(CryptManager::getEncryptionKeyLastRenewalDate()));
		}

		$mode = ("agree" == bgpc("encryption_agree") ? bgpc("encryption_mode") : CryptManager::MODE_NONE);
		$this->smartyVars["to"] = array(
			"type" => getTypeName("encryption", getAPITypeID("encryption", $neededCypherType)),
			"mode" => $mode,
		);

		if ( CryptManager::isEncryptionEnabled() && CryptManager::MODE_NONE != $mode ) {
			$this->smartyVars["warningMessages"][] = blmsg("TRANS_ENCRYPTION_UPDATE_WARNING");
		}

		$this->addTemplate("configs/EditEncryptionConfigsConfirm", $this->smartyVars, 1);
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	protected function validateRequest()
	{
		if ( "agree" != ($encryption_agree = bgpc("encryption_agree")) && "" != $encryption_agree ) {
			throw new InvalidAttributes(array("encryption_agree"));
		}
		require_once("object_managers/CryptManager.php");
		if ( "agree" != $encryption_agree && !CryptManager::isEncryptionEnabled() ) {
			throw new ProductException(ProductException::E_FORBIDDEN_OPERATION, blmsg("TRANS_ENCRYPTION_IS_NOT_ENAMBLED_MSG"));
		}

		if ( $this->noEncryptionChanged() || "agree" != $encryption_agree ) {
			return;
		}

		$invalid_attributes = array();
		if ( CryptManager::isEncryptionEnabled() && !CryptManager::isLekEnabled() ) {
			if ( "" == ($old_pass_phrase = bgpc("old_pass_phrase")) || bgpc("old_pass_phrase_confirm") != $old_pass_phrase ) {
				$invalid_attributes[] = "old_pass_phrase";
				$invalid_attributes[] = "old_pass_phrase_confirm";
			}
		}

		if ( CryptManager::isEncryptionEnabled() && CryptManager::isLekEnabled() ) {
			CryptManager::checkLekPin(bgpc("old_lek"), getCurrentSession()->user_id);
		}

		switch ( $encryption_mode = bgpc("encryption_mode") ) {
			case "lek":
				if ( !CryptManager::isEncryptionEnabled() &&
					 CryptManager::isLekEnabled() &&
					 "" == bgpc("lek_pin") && "" == bgpc("lek_pin_confirm")
				) {
					break;
				}
				if ( !Billing_Checker::checkRegexp(Billing_Checker::P_PIN_CODE, $lek_pin = bgpc("lek_pin")) ||
					 bgpc("lek_pin_confirm") != $lek_pin
				) {
					$invalid_attributes[] = "lek_pin";
					$invalid_attributes[] = "lek_pin_confirm";
				}
				break;
			case "passphrase":
				if ( CryptManager::isEncryptionEnabled() &&
					 ("" == ($pass_phrase = bgpc("pass_phrase")) || bgpc("pass_phrase_confirm") != $pass_phrase)
				) {
					$invalid_attributes[] = "pass_phrase";
					$invalid_attributes[] = "pass_phrase_confirm";
				}
				break;
			default:
				$invalid_attributes[] = "encryption_mode";
		}

		if ( !empty($invalid_attributes) ) {
			throw new InvalidAttributes($invalid_attributes);
		}
	}

	protected function noEncryptionChanged()
	{
		require_once("object_managers/CryptManager.php");
		$encryption_mode = ("agree" == bgpc("encryption_agree") ? bgpc("encryption_mode") : CryptManager::MODE_NONE);
		return (CryptManager::getEncryptionMode() == $encryption_mode && "" == bgpc("lek_pin") && "" == bgpc("pass_phrase"));
	}

	private function backupLek()
	{
		require_once("object_managers/CryptManager.php");
		$backup = CryptManager::getLekPartBackup();

		header("HTTP/1.1 200 OK");
		header("Content-Type:  text/plain");
		header("Content-Length: ".strlen($backup));
		header("Content-Disposition: attachment; filename=\"lek.backup\"");
		echo $backup;
		exit;
	}

	private function restoreLek()
	{
        require_once("FileDownloadClass.php");
		if (!FileDownloadClass::isSpecified("lekBackupFile")) {
			throw new InvalidAttributes(array("lekBackupFile"));
		}
		$lekPartBackup = FileDownloadClass::getContents("lekBackupFile");

		require_once("object_managers/CryptManager.php");
		CryptManager::update(array("encryption_agree" => bgpc("encryption_agree")));

		getCurrentSession()->Storage->infoMessages[] = CryptManager::restoreLek(bgpc("lekBackupPin"), $lekPartBackup) ?
			blmsg("TRANS_ENCRYPTION_LEK_RESTORE_SUCCESSFULLY_MSG") : blmsg("TRANS_ENCRYPTION_LEK_RESTORE_PART_SUCCESSFULLY_MSG");

		redirect("configs.php?sid=".bgpc("sid")."&action=".getActionID("EditEncryptionConfigs"));
	}
}
