<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/action.php");

class ShowSyncStatistics extends MB_Action
{
	public function execute()
	{
		parent::execute();
		$objectsDescriptor = array(
			"panels" => array(
				"name" => "TRANS_SYNC_STATISTICS_PANELS",
				"manager_class_name" => "ControlPanelManager",
				"get_sync_statistics_method_name" => "getSyncStatistics",
				"list_controller" => "servergroups",
				"list_action" => "ShowServerGroups"
			),
			"administrator" => array(
				"name" => "TRANS_SYNC_STATISTICS_ADMINISTRATORS",
				"manager_class_name" => "AdminManager",
				"get_sync_statistics_method_name" => "getSyncStatistics",
				"list_controller" => "admins",
				"list_action" => "ShowAdmins"
			),
			"customer" => array(
				"name" => "TRANS_SYNC_STATISTICS_CUSTOMERS",
				"manager_class_name" => "ClientManager",
				"get_sync_statistics_method_name" => "getSyncStatistics",
				"list_controller" => "clients",
				"list_action" => "ShowClients"
			),
			"customer_contact" => array(
				"name" => "TRANS_SYNC_STATISTICS_CUSTOMER_CONTACTS",
				"manager_class_name" => "ClientContactManager",
				"get_sync_statistics_method_name" => "getSyncStatistics",
				"list_controller" => "clients",
				"list_action" => "ShowClientContacts"
			),
			"plans" => array(
				"name" => "TRANS_SYNC_STATISTICS_PLANS",
				"manager_class_name" => "ProductManager",
				"get_sync_statistics_method_name" => "getPlansSyncStatistics",
				"list_controller" => "products",
				"list_action" => "ShowProducts",
                "filter" => array("filter[product_is_addon]" => 0),
			),
			"addons" => array(
				"name" => "TRANS_SYNC_STATISTICS_ADDONS",
				"manager_class_name" => "ProductManager",
				"get_sync_statistics_method_name" => "getAddonsSyncStatistics",
				"list_controller" => "products",
				"list_action" => "ShowProducts",
                "filter" => array("filter[product_is_addon]" => 1),
			),
			"subscriptions" => array(
				"name" => "TRANS_SYNC_STATISTICS_SUBSCRIPTIONS",
				"manager_class_name" => "PackageManager",
				"get_sync_statistics_method_name" => "getSyncStatistics",
				"list_controller" => "packages",
				"list_action" => "ShowPackages"
			),
		);
		
		$syncStatistics = array();
		
		foreach ($objectsDescriptor as $objectName => $objectDescriptor) {
			require_once "object_managers/" . $objectDescriptor['manager_class_name'] . ".php";
			$syncStatistics[$objectName] = array();
			$syncStatistics[$objectName]["statistics"] = call_user_func($objectDescriptor['manager_class_name'] . '::' . $objectDescriptor['get_sync_statistics_method_name']);
			$syncStatistics[$objectName]["label"] = blmsg($objectDescriptor["name"]);
            $filter = isset($objectDescriptor["filter"]) ? $objectDescriptor["filter"] : array();
			$syncStatistics[$objectName]["synced_url"] = action_url($objectDescriptor["list_controller"], $objectDescriptor["list_action"], array_merge($filter, array("filter[sync_status]" => "synced")));
			$syncStatistics[$objectName]["error_url"] = action_url($objectDescriptor["list_controller"], $objectDescriptor["list_action"], array_merge($filter, array("filter[sync_status]" => "error")));
			$syncStatistics[$objectName]["syncing_url"] = action_url($objectDescriptor["list_controller"], $objectDescriptor["list_action"], array_merge($filter, array("filter[sync_status]" => "syncing")));
		}
		$this->smartyVars["syncStatistics"] = $syncStatistics;
		
		require_once("object_managers/SSOManager.php");
		$ssoSyncInfo = array(
			'status' => SSOManager::getSSOServerSyncStatus(),
			'sync_url' => action_url("sso", "SyncSSO"),
		);
		$this->smartyVars["ssoSyncInfo"] = $ssoSyncInfo;

		require_once("object_managers/SyncManager.php");
		$this->smartyVars["syncTaskInfo"] = SyncManager::getSyncTaskInfo();
		
		$this->addTemplate("dashboard/ShowSyncStatistics");
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
