<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/DomainAction.php");
require_once("lib-mbapi/mbapi.php");

/**
 * This Action object takes information provided by the EditDomainAccessPassword action, and sends
 * it to the registrar class for processing.
 *
 * @author  Michael Fountain
 */
class EditDomainAccessPasswordResponse extends DomainAction {
	/**
	 * The constructor for an EditDomainAccessPasswordResponse action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. It also sets up $this->params.
	 */
	public function __construct()
	{
		parent::__construct();
		$this->params = array
		(
			"domainID",
			"domainAccessPassword1",
			"domainAccessPassword2",
		);
	}

	/**
	 * The main method of this class that executes the EditDomainAccessPasswordResponse action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		$domainAccessPassword1 = trim(bgpc("domainAccessPassword1"));
		$domainAccessPassword2 = trim(bgpc("domainAccessPassword2"));

		if(!$domainAccessPassword1 || !$domainAccessPassword2) {
			$this->redirectWithErrorMessage("Domain Access Password is Empty.");
		}
		if($domainAccessPassword1 != $domainAccessPassword2) {
			$this->redirectWithErrorMessage("Domain Access Passwords Do Not Match.");
		}

	 	$mbapiData = $this->getDomain();

		// fetching domain data from the registrar
		$domainSLD = $mbapiData["mbapi"][0]["results"][0]["domains"][0]["domain"][0]["domainSLD"][0];
		$domainTLD = $mbapiData["mbapi"][0]["results"][0]["domains"][0]["domain"][0]["domainTLD"][0];
		$domainID = $mbapiData["mbapi"][0]["results"][0]["domains"][0]["domain"][0]["domainID"][0];

		$xml = "
		<mbapi>
		  <command>ProcessRegistrar</command>
		  <showXMLHeader>0</showXMLHeader>
		  <params>
		    <subCommand>setDomainAccessPassword</subCommand>
		    <configID>0</configID>
			<domainID>".$domainID."</domainID>
			<domainSLD>".$domainSLD."</domainSLD>
			<domainTLD>".$domainTLD."</domainTLD>
			<domainAccessPassword>".$domainAccessPassword1."</domainAccessPassword>
		  </params>
		</mbapi>";
		$data = dispatchMBAPI($xml,1);
		$this->processRegistrarResponse($data);


		return $this->hasExecutedSuccessfully();

	}
} // EditDomainAccessPasswordResponse
?>