<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/DomainAction.php");
require_once("lib-mbapi/mbapi.php");
require_once("lib-tk/include/xml/xml.php");

/**
 * This Action object collects information on domains that match the given parameters, and
 * generates the display of that information.
 *
 * @author  Michael Fountain
 */
class SyncDomainDates extends DomainAction {

	/**
	 * The main method of this class that executes the SyncDomainDates action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute() {
		global $CONF;
		parent::execute();

		// get domain info
		$domain = oneOf('domains', array('domainID' => $this->domainID));
		if (isset($domain['domainID'])) {
			// run the command
			$cmd = new MBQuery('ProcessRegistrar', 'getDomainExpDate', array(
				'configID' => 0,
				'domainID' => $domain["domainID"],
				'domainSLD' => $domain["domainSLD"],
				"domainTLD" => $domain["domainTLD"]
				));

			$cmd = $cmd->getResult();

			if (!$cmd->getErrorCount()) {
				$result = grabSimpleMBAPIResults( $cmd->getStructure() );

				$this->smartyVars['domain'] =& $domain;
				$this->smartyVars['domainExpTimestamp'] = $result['DomainExpTimestamp'];

				unset($result);
				$this->addTemplate('domains/SyncDomainDates');
			} else {
				$subject = $cmd->getSubject();
				$subject = $subject->getEssential();

				if (isset($subject['mbapi']) && isset($subject['mbapi'][0]['header'][0]['errors'][0])) {
					foreach( (array)$subject['mbapi'][0]['header'][0]['errors'] as $err ) {
						$this->addError( $err['title'], $err['message'] );
					}
				}

				unset($subject);
			}
		} else {
			$this->addError(blmsg("TRANS_ACTIONERROR"), blmsg("TRANS_DOMAIN_ID_INVALID", array("DOMAIN_ID" => $this->domainID)));
		}
		if (0 != $this->getNumErrors()) {
			$this->redirectWithErrorMessage(billing_safetyhtml($this->getErrorMessage()));
		}
		return $this->hasExecutedSuccessfully();
	}
} // SyncDomainDates
?>