<?php
require_once(dirname(__FILE__)."/../../../../lib-tk/shield.php");
require_once(dirname(__FILE__)."/../../../../lib-action/action.php");
require_once(dirname(__FILE__)."/../../../../lib-billing/include/config/internal/emailactions.php");

/**
 * This action resend an email from the queue.
 *
 * @package  modernbill
 * @subpackage  action
 * @author  Brian Smith
 */
class PreviewEmailTemplate extends MB_Action {
	/**
	 * Call the parent constructor, setup a string representation for ourselves, and setup the allowed parameters.
	 */
	function PreviewEmailTemplate() {
		parent::MB_Action();
		$this->params = array("emailTplID", "emailActionID", "preview");
	}

	function execute() {
		parent::execute();

		$emailTplID = (int)$_REQUEST["emailTplID"];
		$emailActionID = (int)$_REQUEST["emailActionID"];

		if (!notZero( $emailTplID ) || !notZero( $emailActionID )) {
			$this->addError( TRANS_ACTIONERROR, "No Email ID or No Email Action ID Given" );
		} elseif (in_array((string)$_REQUEST['preview'], array('html', 'text'))) {

			// we need the locale specific info
			$template = Factory()->EmailTemplate($emailTplID);

			// get the email action name so we can grab the file
			$actionName = getAPITypeName( "emailAction", $emailActionID );

			$fileName = mb_path("/lib-mbapi/include/mail/emailTemplates/$actionName.php");

			if (!is_readable( $fileName )) {
				$this->addError( TRANS_ACTIONERROR, "The requested email action [$actionName] does not exist." );
			} else {
				require_once($fileName);

				if (!class_exists( $actionName )) {
					$this->addError( TRANS_ACTIONERROR, "$actionName does not exists." );
				} else {
					if ( isset($_REQUEST["emailTplBody"]) ) {
						$template->email_tpl_body = utf8_html_entity_encode(bgpc("emailTplBody"));
					}
					if ( isset($_REQUEST["emailTplTextBody"]) ) {
						$template->email_tpl_text_body = utf8_html_entity_encode(bgpc("emailTplTextBody"));
					}

					$action = new $actionName();
					$action->enablePreviewMode();
					$action->setTemplate($template);
					$previewData = $action->execute();

					if ( $action->getErrorCount() ) {
						show($action->getErrors());
						exit;
					}
					if ( isset($previewData[$_REQUEST['preview']]) ) {
						mbHeader('Content-Type: text/html; charset=UTF-8');
						if ( "html" == $_REQUEST['preview'] ) {
							echo $previewData[$_REQUEST['preview']];
						} else {
							echo nl2br(utf8_html_entity_encode($previewData[$_REQUEST['preview']]));
						}
						exit;
					}
					$this->addError(TRANS_ACTIONERROR, "Invalid preview type requested.");
				}
			}

		} else {
			$this->smartyVars['emailTplID'] = $_REQUEST['emailTplID'];
			$this->smartyVars['emailActionID'] = $_REQUEST['emailActionID'];
			$this->addTemplate("emails/PreviewEmailTemplate");
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
