<?php

require_once("lib-action/action.php");
require_once("lib-mbapi/mbapi.php");
require_once("lib-action/include/url/url.php");
require_once("lib-tk/persist.php");
require_once("lib-tk/include/system/system.php");

class RunEvents extends MB_Action
{
	public function execute()
	{
		global $CONF;

		$tempSid = $_REQUEST["sid"];

		require_once("objects/SystemQueue.php");
		//If a queue id isn't passed in, limit the run
		if(is_null($systemQueueID = bgpc("systemQueueID"))) {
			$_REQUEST["systemQueueStatus"] = array(SystemQueue::STATUS_NEW, SystemQueue::STATUS_RULE_VIOLATION);
		} else {
			$_REQUEST["systemQueueStatus"] = secureInput($_REQUEST["systemQueueStatus"]);
		}

		$redirectURL = bgpc("redirectURL");
		$redirectAction = bgpc("redirectAction");
		$redirectArgs = bgpc("redirectArgs");
		if (!is_null($systemQueueID)){
			require_once("objects/BillingSession.php");
			$session = getCurrentSession();
			if (is_null(bgpc("forceRun"))) {
				updateForKey(":systemQueue", $systemQueueID, array(
					"systemQueueStatus" => SystemQueue::STATUS_NEW,
					"systemQueueDateFinished" => 0,
				));
				$session->Storage->infoMessages[] = blmsg("TRANS_EVENT_RUN_NEXT_TIME_DESC", array("EVENT_NUMBER" => $systemQueueID));
				if (!is_null($redirectURL) && !is_null($redirectAction)) {
					redirect($redirectURL."?action=".getActionID($redirectAction)."&".$redirectArgs);
				} else {
				 	redirect("events.php", "ShowEvents", array("systemQueueStatus",SystemQueue::STATUS_NEW));
				}
			} elseif (is_null(bgpc("ajax"))) {
				$session->Storage->infoMessages[] = blmsg("TRANS_EVENT_RUN_NOW_DESC", array("EVENT_NUMBER" => $systemQueueID));
			}
		}

		dbg("At top of execute call.");
		ignore_user_abort();
		set_time_limit(0);

		// ------------------------------------------------------------------
		// Are we being forced to reset the counters?
		// ------------------------------------------------------------------
		if ($_REQUEST["forceClear"] || (!$_REQUEST["continue"] && !$CONF["system"]["internal"]["systemQueueRunning"])) {
			dbg("did forceClear or not continue or its not running in conf.");
			$this->resetInternalCounters();
		}

		// ------------------------------------------------------------------
		// Somone wants to stop the queue, kill this process now.
		// ------------------------------------------------------------------
		if ($CONF["system"]["internal"]["systemQueueStop"]) {
			dbg("was told to stop it 1.");
			$this->resetInternalCounters();

			dbg("redirecting due to lack of continue");
			if (!is_null($redirectURL) && !is_null($redirectAction)) {
				redirect($redirectURL."?action=".getActionID($redirectAction)."&".$redirectArgs);
			} else {
			 	redirect("events.php", "ShowEvents", array("systemQueueStatus",SystemQueue::STATUS_NEW));
			}

		// ------------------------------------------------------------------
		// We aren't going to kill the queue, so go....
		// ------------------------------------------------------------------
		} else {
			dbg("we dont need to stop, so what do we need to do?");

			// if there is a system queue id being passed lets allow it through to the command
			// (by adding the systemQueueID param to the params array)
			$params = array();
			if( !is_null($systemQueueID)) {
				dbg("added system queue id into params");
				$params["systemQueueID"] = $systemQueueID;
			}
			// When was this chain of processes started?
			$this->started = (int)$_REQUEST["started"];

			// if there's no continue or no started timestamp then make one
			if(!$_REQUEST["continue"] || !$this->started) {
				dbg("no continue, so create out start timestamp");
				// We're the parent of all upcoming processes, tell them when the family was started
				$this->started = mktime();
			} else {
				dbg("we got the started command");
			}

			// ------------------------------------------------------------------
			// Did the queue stall or timeout for more than 60 minutes?
			// If yes, reset the counters so it can start again
			// ------------------------------------------------------------------
			if ((int)$CONF["system"]["internal"]["systemQueueRunning"] &&
				((mktime()-$CONF["system"]["internal"]["systemQueueStarted"]) > (60*60))) {

				dbg("systemQueueRunning && systemQueueStarted > 60 minutes ago, possible the queue died, not continuing");
				$this->resetInternalCounters();

			// ------------------------------------------------------------------
			// Did the original thread start X or more seconds ago? If yes, kill it to
			// prevent overloading the server with parent-child processes!
			// ------------------------------------------------------------------
			} else if ((int)$CONF["system"]["internal"]["systemQueueRunning"] &&
				(int)$CONF["killOldQueueThreads"] &&
				((mktime()-$CONF["system"]["internal"]["systemQueueStarted"]) > (int)$CONF["killOldQueueThreadsLimit"])) {

				dbg("killOldQueueThreads was true, it ran too long, not continuing");
				$this->resetInternalCounters();
				if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
					redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
				} else {
					redirect("events.php", "ShowEvents", array("systemQueueStatus",getStatusID("new","queue")));
				}

			// ------------------------------------------------------------------
			// The queue is running fin, let it go.
			// ------------------------------------------------------------------
			} else {

				dbg("we dont need to kill the thread, what do we need to do?");

				////////////////////////////////////////// running stuff

				// Are we a child thread? If so, run some events...
				if ($_REQUEST["continue"]){

					dbg("we need to continue");

					if ($_REQUEST["sid"]==$CONF["system"]["internal"]["systemQueueSessionID"]){

						dbg("we are a child");
						loadConfigs("automation", "systemQueue");
						// Remember that if a systemQueueID is given, we will run twice, but only run the event for that ID once
						// We run twice in that this process we have our spawning process, and the process that literally runs the event
						//if((int) $_REQUEST["systemQueueID"]) {
						//	$params = array("runMax" => 1, "systemQueueDateFinished" => 0);
						//} else {
						$params["runMax"] = $CONF["automation"]["systemQueue"]["max_system_queue_events"];
						if (is_null($systemQueueID)) {
							$params["systemQueueDateToRunLessThan"] = $_REQUEST["started"];
							$params["systemQueueDateFinished"] = 0;
						}
						dbg($params);
						//}

						dbg(PERSISTABLE ? "process should last forever, running all events at once" : "process has a timeout, running events incrementally");

						do {
							dbg("running a set of events");

							$query = $this->getMBAPIQuery("ProcessSystemQueue", "", $params);
							$data = dispatchMBAPI($query, 1);
							dbg("RUN query we are running: ".$query);
							dbg($data);
							if($this->addMBAPIErrors($data) === 0) {
								dbg("we dont have errors");
								// Have we run all commands we were supposed to?
								// -- If yes, stop the queue & die gracefully.
								// -- If no, tell the system the queue is still running and continue.
								if(!((int) $data["mbapi"][0]["results"][0]["numExecutedCommands"][0]) || conf("systemQueueStop") || !is_null($systemQueueID)) {
									dbg("setting running as 0 and stop as 1");
									$CONF["system"]["internal"]["systemQueueRunning"] = 0;
									$CONF["system"]["internal"]["systemQueueStop"] = 1;
									$CONF["system"]["internal"]["systemQueueSessionID"] = "";
									storeConfigs("system","internal");
									return 1;
								} else {
									dbg("setting running as 1");
									$CONF["system"]["internal"]["systemQueueRunning"] = 1;
									$CONF["system"]["internal"]["systemQueueStarted"] = $this->started;
									storeConfigs("system","internal");
								}

								if(!PERSISTABLE) {
									// Don't overload the server
									// This would ideally be < 1 second, but PHP on Windows doesn't support usleep() until PHP 5

									dbg("sleeping 3");
									sleep(3);
									$this->startNewThread();
								}
							}

						} while(PERSISTABLE && $CONF["system"]["internal"]["systemQueueRunning"]);

					} else {
						dbg("out sid didnt match the one in conf: ".$_REQUEST["sid"]. " != ". $CONF["system"]["internal"]["systemQueueSessionID"]);
					}

				// No, this is not a child thread, run me as the parent thread!
				} else {

					dbg("we are a parent.");

					loadConfigs("system", "internal");
					// another process running?
					if (strlen($CONF["system"]["internal"]["systemQueueSessionID"])==0 && !(int)$CONF["system"]["internal"]["systemQueueRunning"]){

						// Inform the system of the total number of events that will be run and tell it
						// it's ok for the system queue to run.
						dbg("running a parent run");
						if (is_null($systemQueueID)) {
							$params["systemQueueDateToRunLessThan"] = $_REQUEST["started"];
							$params["systemQueueDateFinished"] = 0;
						}

						if(is_numeric($_REQUEST["systemQueueStatus"]) || is_array($_REQUEST["systemQueueStatus"])) {
							$params["systemQueueStatus"] = $_REQUEST["systemQueueStatus"];
						}

						$query = toXML(array(
							"mbapi" => array(
								"command" => "GetSystemQueue",
								"params" => $params,
							),
						));

						$data = dispatchMBAPI($query, 1);
						dbg("parent query: ".$query);
						dbg($data);
						// errors?
						if($this->addMBAPIErrors($data) === 0) {
							loadConfigs("system", "internal");
							// another process running now?
							if (strlen($CONF["system"]["internal"]["systemQueueSessionID"])==0 && !(int)$CONF["system"]["internal"]["systemQueueRunning"]){
								dbg("no errors, setting num events to whatever and running to 1");
								$_REQUEST["sid"] = $this->cloneSession($_REQUEST["sid"]);
								$CONF["system"]["internal"]["systemQueueNumEvents"] = (int) $data["mbapi"][0]["header"][0]["numResults"][0];
								$CONF["system"]["internal"]["systemQueueRunning"] = 1;
								$CONF["system"]["internal"]["systemQueueStarted"] = $this->started;
								$CONF["system"]["internal"]["systemQueueSessionID"] = $_REQUEST["sid"];
								storeConfigs("system","internal");
								$this->startNewThread();
							} else { // is there another process after we went through all this? check one more time...
								dbg("3 out sid didnt match the one in conf: ".$_REQUEST["sid"]. " != ". $CONF["system"]["internal"]["systemQueueSessionID"]);
							}
						} // end errors
					} else { // end other process check
						dbg("2 out sid didnt match the one in conf: ".$_REQUEST["sid"]. " != ". $CONF["system"]["internal"]["systemQueueSessionID"]);
					}
				} // end parent exec
			}
		} // END IF: Not stopping the queue

		// ------------------------------------------------------------------
		// If there is no request variable for "continue", then we need to just
		// redirect the user or process, else just return.
		// ------------------------------------------------------------------
		if (!$_REQUEST["continue"] && !$_REQUEST["isAPIHit"]){
			dbg("the user has dispatched us, display the run events template for them");
			$_REQUEST["sid"] = $tempSid;
			if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
				redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
			} else {
			 	redirect("events.php", "ShowEvents", array("systemQueueStatus",getStatusID("new","queue")));
			}
			$this->addErrorTemplates();
		}

		return $this->hasExecutedSuccessfully();

	}

	/**
	 * This function clones the session of the sessionid you give it and returns the new session id.
	 *
	 * @param sid The session id to clone.
	 * @return sid The new session id.
	 */
	private function cloneSession($sid)
	{
		dbg("cloning session");
		// It is important to clone our session in case we logout. This will ensure that our child
		// processes live and continue until they are completed or stopped by the system.
		$sessGetXML = "
			<mbapi>
				<command>GetSessions</command>
				<showXMLHeader></showXMLHeader>
				<params>
					<sessionID>".$sid."</sessionID>
				</params>
			</mbapi>
			";
		$sessionInfo = dispatchMBAPI($sessGetXML, 1);
		dbg("got session info");
		if (isset($sessionInfo["mbapi"][0]["results"][0]["sessions"][0]["session"])){
			dbg("have valid session info");
			$sessionInfo = correctArray($sessionInfo["mbapi"][0]["results"][0]["sessions"][0]["session"][0]);
			$sid = sha1(strval(mt_rand(0, 200000000)).strval(microtime()).strval(mt_rand(0, 200000000)));
			$setSessXML = "
				<mbapi>
					<command>SetSession</command>
					<subCommand>insert</subCommand>
					<showXMLHeader></showXMLHeader>
					<params>
						<sessionID>$sid</sessionID>
						<userID>".$sessionInfo["userID"]."</userID>
						<userType>".$sessionInfo["userType"]."</userType>
						<sessionDateCreated>".$sessionInfo["sessionDateCreated"]."</sessionDateCreated>
						<sessionDateAccessed>".$sessionInfo["sessionDateAccessed"]."</sessionDateAccessed>
						<sessionDateExpired>".$sessionInfo["sessionDateExpired"]."</sessionDateExpired>
						<sessionIP>".$sessionInfo["sessionIP"]."</sessionIP>
						<sessionActive>".$sessionInfo["sessionActive"]."</sessionActive>
					</params>
				</mbapi>
				";
			$result = dispatchMBAPI($setSessXML, 1);
			dbg("set session: ".$setSessXML);
			dbg($result);
			if ((int)$result["mbapi"][0]["header"][0]["success"][0]){
				dbg("got result sid");
			}
		}
		return $sid;
	}

	/**
	 * This method resets the internal systemQueue counters
	 */
	private function resetInternalCounters($mungeSessionID=0)
	{
		global $CONF;
		dbg("resetting system queue internals");
		$CONF["system"]["internal"]["systemQueueParentID"] = 0;
		$CONF["system"]["internal"]["systemQueueCommand"] = "";
		$CONF["system"]["internal"]["systemQueueCount"] = 0;
		$CONF["system"]["internal"]["systemQueueNumEvents"] = 0;
		$CONF["system"]["internal"]["systemQueueRunning"] = 0;
		$CONF["system"]["internal"]["systemQueueStarted"] = 0;
		$CONF["system"]["internal"]["systemQueueStop"] = 0;
		$CONF["system"]["internal"]["systemQueueSessionID"] = "";
		storeConfigs("system","internal");
		loadConfigs("system");
	}

	/**
	 * start a new thread
	 */
	private function startNewThread()
	{
		dbg("starting new thread");
		switch(PERSIST_STYLE) {
			case PERSIST_NULL:
				dbg("starting new hit");
				$this->startNewHit();
				break;
			case PERSIST_CLI:
				dbg("starting new process");
				$this->startNewProc();
				break;
			case PERSIST_DAEMON:
				dbg("starting new daemon");
				$this->startNewDaemon();
				break;
		}
	}

	private function getPHPCommand()
	{
		if($php = $_SERVER["_"]) {
			// some shells set this variable -- not required by POSIX
			return $php;
		} else if ( file_exists(PLESK_PHP_PATH) ) {
			return PLESK_PHP_PATH;
		} else {
			return "php";
		}
	}

	private function getSelfCommand()
	{
		$php = $this->getPHPCommand();
		$script = $_SERVER["argv"][0];
		return "${php} ${script}";
	}

	private function startNewProc()
	{
		$self_command = $this->getSelfCommand();
		$sid = $_REQUEST["sid"];
		$actionID = getActionID("RunEvents");
		$started = $this->started;

		$command = "${self_command} ${sid} ${actionID} continue ${started}";

		dbg("event process command");
		dbg($command);
		dbg("event process output");
		dbg(shell_exec($command));
	}

	/**
	 * start new thread via a forked daemon process
	 */
	private function startNewDaemon()
	{
		list($status, $pid, $id) = daemonize();
		switch($status) {
			case DAEMON_PARENT:
				$child_pid = $id;

				// nothing needs to be done
				//syslog(LOG_ERR, "RunEvents parent pid ${pid} just forked child pid ${child_pid}.");

				break;
			case DAEMON_CHILD:
				$session_id = $id;

				// need to start working, now
				//syslog(LOG_ERR, "RunEvents child pid ${pid} just got session id ${session_id}.");

				// set child state
				$_REQUEST["ajax"] = 1;
				$_REQUEST["continue"] = 1;
				$_REQUEST["started"] =  $this->started;

				// force a new connection to the database for the child to avoid concurrency issues with two processes using the same mysql connection; cannot use dbg() on this side between this and the fork
				global $CONF, $db, $remoteDaemonConnection;
				if($CONF["installType"] == "local") {
					$db->NConnect(DatabaseManager::getHost(), DatabaseManager::getUser(), DatabaseManager::getPassword(), DatabaseManager::getName());
				} else {
					unset($remoteDaemonConnection);
					// dispatchMBAPI() will reconnect automatically
				}

				// restart RunEvents
				dispatchAction("events", "RunEvents");

				break;
		}
	}

	/**
	 * This method starts another thread via an HTTP hit back to ModernBill.
	 *
	 * @param none
	 * @return none
	 */
	private function startNewHit()
	{
		global $CONF;

		$urlProtocol = $CONF["secureURLProtocol"];
		$urlHostnameSecure = $CONF["secureURL"];
		$urlHostnameStandard = $CONF["standardURL"];

		// override if using a local IP
		if ($CONF["localURL"]) {
			$urlProtocol = "http";
			$urlHostnameStandard = $CONF["localURL"];
		}

		$parameters = array(
			'ajax' => 1,
			'continue' => 1,
			'started' => $this->started
		);

		if( !is_null($systemQueueID = bgpc("systemQueueID"))) {
			$parameters['systemQueueID'] = $systemQueueID;
		}
		if( !is_null($forceRun = bgpc("forceRun"))) {
			$parameters['forceRun'] = $forceRun;
		}

		$path_query = action_url('events', 'RunEvents', $parameters);

		// Time to start a child process with the new session and let it do its thing!
		if ($urlProtocol=="http") {

			dbg("going to open a connection to: $urlHostnameStandard");
			$url = parse_url($urlHostnameStandard);
			$fp = fsockopen($url["host"], $url["port"]);
			if(!$fp) {
				dbg("failed to connect");
				$this->addError(TRANS_ACTIONERROR, "Failed to connect to host to continue running system queue.");
			} else {
				dbg("making get string");
				dbg("STRING WE START WITH: ".$url["path"]);
				$file = Config()->system_standardPanelUrl . $path_query;
				$out = "GET $file HTTP/1.1\r\n";
				$out .= "Host: ".$url["host"]."\r\n";
				$out .= "Connection: close\r\n\r\n";
				dbg($out);
				if(fwrite($fp, $out) === false) {
					dbg("failed to write to socket.");
					$this->addError(TRANS_ACTIONERROR, "Failed to write data to socket when running system queue.");
				}
				fclose($fp);
			}

		} else {

			dbg("going to open a connection to: $urlHostnameSecure");

			$file = Config()->system_securePanelUrl . $path_query;

			$fp = fopen($file, "r");
			if(!$fp) {
				//Try using the curl class as a backup.  Not having openssl will prevent fopen from connecting
				$c = new Curl();
				$c->setURL($file);
				$c->setFieldMethod(0);
				$c->addHeader("Content-Type: text/xml");
				$result = $c->toString();
				if(!$c->hasExecutedSuccessfully) {
					dbg("failed to connect");
					$this->addError(TRANS_ACTIONERROR, "Failed to connect to host to continue running system queue.");
				}
			} else {
				dbg("fread: ".fread($fp, 10));
				fclose($fp);
			}
		}
	}
}
