<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/action.php");

/**
 * This action collects info for sending mass emails to clients contacts.
 */
class SendEmails extends MB_Action
{
	const CONTACT_TYPE_PRIMARY = 1;
	const CONTACT_TYPE_ALL = 2;

	const EMAIL_FORMAT_HTML = 1;
	const EMAIL_FORMAT_TEXT = 2;

	public function execute()
	{
		parent::execute();

		$this->settings = $this->getDefaultSettings();

		$command = isset($_REQUEST["command"]) ? $_REQUEST["command"] : "";
		if ( "cancel" == $command ) {
			redirect(action_url("events", "ShowEmailQueue"));
		}

		$step = isset($_REQUEST["step"]) ? $_REQUEST["step"] : "";
		switch ( $step ) {
		case "select_recipients":
			if ( $this->processSelectRecipientsStep() ) {
				$this->showClientContactsStep();
			} else {
				$this->showSelectRecipientsStep();
			}
			break;
		case "client_contacts":
			if ( $this->processClientContactsStep() ) {
				if ( "back" == $command ) {
					$this->showSelectRecipientsStep();
				} else {
					if ( "" == $this->settings["template_group_id"] ) {
						$this->showEditEmailStep();
					} else {
						$this->showSendEmailsStep();
					}
				}
			} else {
				$this->showClientContactsStep();
			}
			break;
		case "edit_email":
			if ( $this->processEditEmailStep() ) {
				if ( "back" == $command ) {
					$this->showClientContactsStep();
				} else if ( "reload" == $command ) {
					$this->showEditEmailStep();
				} else {
					$this->showSendEmailsStep();
				}
			} else {
				$this->showEditEmailStep();
			}
			break;
		case "send_emails":
			if ( $this->processSendEmailsStep() ) {
				if ( "back" == $command ) {
					if ( "" == $this->settings["template_group_id"] ) {
						$this->showEditEmailStep();
					} else {
						$this->showClientContactsStep();
					}
				} else {
                    redirect(action_url("events", "ShowEmailQueue"));
				}
			} else {
				$this->showSendEmailsStep();
			}
			break;
		default:
			$this->showSelectRecipientsStep();
		}

		return $this->hasExecutedSuccessfully();
	}

	private function showSelectRecipientsStep()
	{
		$this->smartyVars["client_ids"] = $this->settings["client_ids"];
		$this->smartyVars["client_statuses"] = array("" => TRANS_ALL) + getStatuses("client");
		$this->smartyVars["client_status"] = $this->settings["client_status"];
		$this->smartyVars["client_groups"] = array("" => TRANS_ALL, "none" => TRANS_NONE) + $this->getClientGroups();
		$this->smartyVars["client_group_id"] = $this->settings["client_group_id"];
		$this->smartyVars["servers"] = array("" => TRANS_ALL) + $this->getServers();
		$this->smartyVars["server_id"] = $this->settings["server_id"];
		$this->smartyVars["server_groups"] = array("" => TRANS_ALL) + $this->getServerGroups();
		$this->smartyVars["server_group_id"] = $this->settings["server_group_id"];
		$this->smartyVars["products"] = array("" => TRANS_ALL) + $this->getProducts();
		$this->smartyVars["product_id"] = $this->settings["product_id"];
		$this->smartyVars["contact_types"] = array(
			self::CONTACT_TYPE_PRIMARY => TRANS_SEND_EMAILS_PRIMARY_CONTACT_ONLY,
			self::CONTACT_TYPE_ALL => TRANS_SEND_EMAILS_ALL_CONTACTS,
		);
		$this->smartyVars["contact_type"] = $this->settings["contact_type"];
		$this->smartyVars["template_groups"] = array("" => TRANS_SEND_EMAILS_OWN_TEMPLATE_NAME) + $this->getTemplateGroups();
		$this->smartyVars["template_group_id"] = $this->settings["template_group_id"];

		$this->saveSettings();
		$this->addMessages();
		$this->addTemplate("events/SendEmailsSelectRecipients");
	}

	private function processSelectRecipientsStep()
	{
		$this->loadSettings();

		$this->settings["client_ids"] = trim($this->settings["client_ids"]);
		if ( "" != $this->settings["client_ids"] && !preg_match('/^\d+(\s*,\s*\d+)*$/', $this->settings["client_ids"]) ) {
			$this->addError( TRANS_ACTIONERROR,
				blmsg("TRANS_INVALID_FIELD_MESSAGE", array("FIELD_NAME" => blmsg("TRANS_CLIENT_ID")))
			);
		}
		if ( "" != $this->settings["template_group_id"] ) {
			$template_group = Factory()->EmailTemplateGroup($this->settings["template_group_id"]);
			if ( count($template_group->EmailTemplateList()) == 0 ) {
				$this->addWarning(new Warning(
					"There are no templates in '".$template_group->email_tpl_group_name."' group. Please add templates and try again.", ""
				));
			}
		}
		if ( $this->getNumErrors() || $this->getNumWarnings() ) {
			return false;
		}

		$conn = DatabaseManager::getConnection();
		$query = "SELECT DISTINCT clients.client_id FROM clients";
		if ( "" != $this->settings["server_id"] || "" != $this->settings["server_group_id"] || "" != $this->settings["product_id"] ) {
			$query .= " INNER JOIN packages ON packages.client_id = clients.client_id";
			if ( "" != $this->settings["product_id"] ) {
				$query .= " AND packages.product_id = ".$conn->quote($this->settings["product_id"], DBConnection::PARAM_INT);
			}
			if ( "" != $this->settings["server_id"] || "" != $this->settings["server_group_id"] ) {
				$query .= " INNER JOIN package_attributes ON package_attributes.package_id = packages.package_id";
				if ( "" != $this->settings["server_id"] ) {
					$query .= " AND package_attributes.config_group_id = ".$conn->quote($this->settings["server_id"], DBConnection::PARAM_INT);
				}
				if ( "" != $this->settings["server_group_id"] ) {
					$query .= " AND package_attributes.server_group_id = ".$conn->quote($this->settings["server_group_id"], DBConnection::PARAM_INT);
				}
			}
		}
		if ( "" != $this->settings["client_ids"] || "" != $this->settings["client_status"] || "" != $this->settings["client_group_id"] ) {
			$conditions = array();
			if ( "" != $this->settings["client_ids"] ) {
				$client_ids = split(",", $this->settings["client_ids"]);
				$client_id = reset($client_ids);
				$condition = " clients.client_id IN(".$conn->quote(trim($client_id), DBConnection::PARAM_INT);
				while( false !== ($client_id = next($client_ids)) ) {
					$condition .= ",".$conn->quote(trim($client_id), DBConnection::PARAM_INT);
				}
				$condition .= ")";
				$conditions[] = $condition;
			}
			if ( "" != $this->settings["client_status"] ) {
				$conditions[] = " clients.client_status = ".$conn->quote($this->settings["client_status"], DBConnection::PARAM_INT);
			}
			if ( "" != $this->settings["client_group_id"] ) {
				if ( "none" == $this->settings["client_group_id"] ) {
					$conditions[] = " clients.group_id IS NULL";
				} else {
					$conditions[] = " clients.group_id = ".$conn->quote($this->settings["client_group_id"], DBConnection::PARAM_INT);
				}
			}
			$query .= " WHERE".implode(" AND", $conditions);
		}

		$rows = $conn->getAll($query);
		$client_ids = array();
		foreach ( $rows as $row ) {
			$client_ids[] = $row["client_id"];
		}

		$contact_ids = array();
		if ( count($client_ids) ) {
			$conditions = array("client_id" => $client_ids);
			if ( self::CONTACT_TYPE_PRIMARY == $this->settings["contact_type"] ) {
				$conditions["contact_primary"] = 1;
			}
			$contacts = Factory()->ClientContactList($conditions);

			require_once("object_managers/ClientContactManager.php");
			foreach ( $contacts as $contact ) {
				if ( ClientContactManager::canSendEmail($contact->id, getAPITypeID("emailAction", "userDefinedEmail")) ) {
					$contact_ids[] = $contact->id;
				}
			}
		}
		$this->settings["contact_ids"] = $contact_ids;

		if ( !count($contact_ids) ) {
			$this->addWarning(new Warning(TRANS_SEND_EMAILS_FOUND_NO_CONTACTS_WARNING, ""));
			return false;
		}

		return true;
	}

	private function showClientContactsStep()
	{
		$this->smartyVars["contacts"] = $this->getContacts();
		$this->saveSettings();
		$this->addMessages();
		$this->addTemplate("events/SendEmailsClientContacts");
	}

	private function processClientContactsStep()
	{
		$this->loadSettings();
		return true;
	}

	private function showEditEmailStep()
	{
		$this->smartyVars["email_formats"] = array(
			"" => TRANS_SEND_EMAILS_EMAIL_FORMAT_HTML_AND_TEXT,
			self::EMAIL_FORMAT_HTML => TRANS_SEND_EMAILS_EMAIL_FORMAT_HTML_ONLY,
			self::EMAIL_FORMAT_TEXT => TRANS_SEND_EMAILS_EMAIL_FORMAT_TEXT_ONLY,
		);
		$this->smartyVars["email_format"] = $this->settings["email_format"];
		$this->smartyVars["email_format_html_only"] = self::EMAIL_FORMAT_HTML;
		$this->smartyVars["email_format_text_only"] = self::EMAIL_FORMAT_TEXT;
		$this->smartyVars["email_priorities"] = array(3 => TRANS_NORMAL, 1 => TRANS_HIGH, 5 => TRANS_LOW);
		$this->smartyVars["email_priority"] = $this->settings["email_priority"];
		$this->smartyVars["from_name"] = $this->settings["from_name"];
		$this->smartyVars["from_address"] = $this->settings["from_address"];
		$this->smartyVars["reply_to_name"] = $this->settings["reply_to_name"];
		$this->smartyVars["reply_to_address"] = $this->settings["reply_to_address"];
		$this->smartyVars["cc_address"] = $this->settings["cc_address"];
		$this->smartyVars["bcc_address"] = $this->settings["bcc_address"];
		$this->smartyVars["subject"] = utf8_html_entity_encode($this->settings["subject"]);
		$html_body = utf8_html_entity_encode($this->settings["html_body"]);
		$this->smartyVars["html_body"] = $html_body;
		$text_body = utf8_html_entity_encode($this->settings["text_body"]);
		$this->smartyVars["text_body"] = $text_body;

		$this->smartyVars["html_preview"] = "";
		$this->smartyVars["text_preview"] = "";
		require_once("objects/EmailTemplate.php");
		$template = new EmailTemplate();
		$template->email_tpl_body = $html_body;
		$template->email_tpl_text_body = $text_body;
		require_once("objects/EmailTemplateGroup.php");
		require_once(dirname(__FILE__)."/../../../../lib-mbapi/include/mail/emailTemplates/GenericEmailTemplates.php");
		$action = new MBGenericEmailTemplates(new EmailTemplateGroup(), $template);
		$action->enablePreviewMode();
		if ( "" != $this->settings["html_body"] || "" != $this->settings["text_body"] ) {
			$email_preview = $action->execute();
			if ( "" != $this->settings["html_body"] ) {
				$this->smartyVars["html_preview"] = $email_preview["html"];
			}
			if ( "" != $this->settings["text_body"] ) {
				$this->smartyVars["text_preview"] = utf8_html_entity_encode($email_preview["text"]);
			}
		}

		$placeholders = array();
		foreach ( $action->getContactPreviewInfo() as $name => $value ) {
			if ( !is_array($value) ) {
				$placeholders["$name"] = $value;
			}
		}
		ksort($placeholders);
		$this->smartyVars["placeholders"] = $placeholders;

		$this->saveSettings();
		$this->addMessages();
		$this->addTemplate("events/SendEmailsEditEmail");
	}

	private function processEditEmailStep()
	{
		$this->loadSettings();

		$this->settings["from_name"] = trim($this->settings["from_name"]);
		$this->settings["from_address"] = trim($this->settings["from_address"]);
		$this->settings["reply_to_name"] = trim($this->settings["reply_to_name"]);
		$this->settings["reply_to_address"] = trim($this->settings["reply_to_address"]);
		$this->settings["cc_address"] = trim($this->settings["cc_address"]);
		$this->settings["bcc_address"] = trim($this->settings["bcc_address"]);
		$this->settings["subject"] = trim($this->settings["subject"]);

		if ( isset($_REQUEST["command"]) && "next" == $_REQUEST["command"] && "" == $this->settings["template_group_id"] ) {
			if ( "" == $this->settings["from_name"] ) {
				$this->addWarning(new Warning(
					blmsg("TRANS_EMPTY_FIELD_MESSAGE", array("FIELD_NAME" => blmsg("TRANS_SEND_EMAILS_FROM_NAME"))), ""
				));
			}
			require_once("Billing_Checker.php");
			if ( "" == $this->settings["from_address"] ) {
				$this->addWarning(new Warning(
					blmsg("TRANS_EMPTY_FIELD_MESSAGE", array("FIELD_NAME" => blmsg("TRANS_SEND_EMAILS_FROM_ADDRESS"))), ""
				));
			} else if ( !Billing_Checker::emailAddress($this->settings["from_address"]) ) {
				$this->addWarning(new Warning(
					blmsg("TRANS_INVALID_FIELD_MESSAGE", array("FIELD_NAME" => blmsg("TRANS_SEND_EMAILS_FROM_ADDRESS"))), ""
				));
			}
			if ( "" != $this->settings["reply_to_address"] && !Billing_Checker::emailAddress($this->settings["reply_to_address"]) ) {
				$this->addWarning(new Warning(
					blmsg("TRANS_INVALID_FIELD_MESSAGE", array("FIELD_NAME" => blmsg("TRANS_SEND_EMAILS_REPLY_TO_ADDRESS"))), ""
				));
			}
			if ( "" != $this->settings["cc_address"] && !Billing_Checker::emailAddresses($this->settings["cc_address"]) ) {
				$this->addWarning(new Warning(
					blmsg("TRANS_INVALID_FIELD_MESSAGE", array("FIELD_NAME" => blmsg("TRANS_SEND_EMAILS_CC_ADDRESS"))), ""
				));
			}
			if ( "" != $this->settings["bcc_address"] && !Billing_Checker::emailAddresses($this->settings["bcc_address"]) ) {
				$this->addWarning(new Warning(
					blmsg("TRANS_INVALID_FIELD_MESSAGE", array("FIELD_NAME" => blmsg("TRANS_SEND_EMAILS_BCC_ADDRESS"))), ""
				));
			}
			if ( "" == $this->settings["subject"] ) {
				$this->addWarning(new Warning(
					blmsg("TRANS_EMPTY_FIELD_MESSAGE", array("FIELD_NAME" => blmsg("TRANS_EMAIL_MESSAGE_SUBJECT"))), ""
				));
			}
			if ( self::EMAIL_FORMAT_TEXT != $this->settings["email_format"] && "" == trim($this->settings["html_body"]) ) {
				$this->addWarning(new Warning(
					blmsg("TRANS_EMPTY_FIELD_MESSAGE", array("FIELD_NAME" => blmsg("TRANS_SEND_EMAILS_HTML_MESSAGE"))), ""
				));
			}
			if ( self::EMAIL_FORMAT_HTML != $this->settings["email_format"] && "" == trim($this->settings["text_body"]) ) {
				$this->addWarning(new Warning(
					blmsg("TRANS_EMPTY_FIELD_MESSAGE", array("FIELD_NAME" => blmsg("TRANS_SEND_EMAILS_TEXT_MESSAGE"))), ""
				));
			}
		}
		if ( $this->getNumErrors() || $this->getNumWarnings() ) {
			return false;
		}
		return true;
	}

	private function showSendEmailsStep()
	{
		$this->smartyVars["contacts"] = $this->getContacts();

		if ( "" != $this->settings["template_group_id"] ) {
			$template_group = Factory()->EmailTemplateGroup($this->settings["template_group_id"]);
			require_once(dirname(__FILE__)."/../../../../lib-mbapi/include/mail/emailTemplates/GenericEmailTemplates.php");
			$action = new MBGenericEmailTemplates($template_group);
			$this->smartyVars["email_format"] = "";
			$this->smartyVars["email_priority"] = $this->getEmailPriorityName($template_group->email_tpl_group_priority);
			$this->smartyVars["from_name"] = $template_group->email_tpl_group_from_name;
			$this->smartyVars["from_address"] = $template_group->email_tpl_group_from_address;
			$this->smartyVars["reply_to_name"] = $template_group->email_tpl_group_reply_name;
			$this->smartyVars["reply_to_address"] = $template_group->email_tpl_group_reply_address;
			$this->smartyVars["cc_address"] = $template_group->email_tpl_group_cc_address;
			$this->smartyVars["bcc_address"] = $template_group->email_tpl_group_bcc_address;
		} else {
			require_once("objects/EmailTemplate.php");
			$template = new EmailTemplate();
			$template->email_tpl_subject = $this->settings["subject"];
			$template->email_tpl_body = utf8_html_entity_encode($this->settings["html_body"]);
			$template->email_tpl_text_body = utf8_html_entity_encode($this->settings["text_body"]);
			require_once(dirname(__FILE__)."/../../../../lib-mbapi/include/mail/emailTemplates/GenericEmailTemplates.php");
			require_once("objects/EmailTemplateGroup.php");
			$action = new MBGenericEmailTemplates(new EmailTemplateGroup(), $template);
			$this->smartyVars["email_format"] = $this->settings["email_format"];
			$this->smartyVars["email_priority"] = $this->getEmailPriorityName($this->settings["email_priority"]);
			$this->smartyVars["from_name"] = $this->settings["from_name"];
			$this->smartyVars["from_address"] = $this->settings["from_address"];
			$this->smartyVars["reply_to_name"] = $this->settings["reply_to_name"];
			$this->smartyVars["reply_to_address"] = $this->settings["reply_to_address"];
			$this->smartyVars["cc_address"] = $this->settings["cc_address"];
			$this->smartyVars["bcc_address"] = $this->settings["bcc_address"];
		}
		$this->smartyVars["email_format_html_only"] = self::EMAIL_FORMAT_HTML;
		$this->smartyVars["email_format_text_only"] = self::EMAIL_FORMAT_TEXT;

		$action->enablePreviewMode();
		$email_preview = $action->execute();
		$this->smartyVars["subject"] = utf8_html_entity_encode($email_preview["subject"]);
		$this->smartyVars["html_preview"] = $email_preview["html"];
		$this->smartyVars["text_preview"] = utf8_html_entity_encode($email_preview["text"]);

		$this->saveSettings();
		$this->addMessages();
		$this->addTemplate("events/SendEmails");
	}

	private function processSendEmailsStep()
	{
		$this->loadSettings();

		if ( isset($_REQUEST["command"]) && "ok" == $_REQUEST["command"] ) {
			$contact_id_tags = "";
			foreach ( $this->settings["contact_ids"] as $contact_id ) {
				$contact_id_tags .= "<clientContactID>$contact_id</clientContactID>";
			}
			$email_tags = "";
			if ( "" != $this->settings["template_group_id"] ) {
				$email_tags .= "<emailTemplateGroupID>".$this->settings["template_group_id"]."</emailTemplateGroupID>";
			} else {
				$email_tags .= "<emailSubject>".utf8_xml_entity_encode($this->settings["subject"])."</emailSubject>";
				$email_tags .= "<emailPriority>".$this->settings["email_priority"]."</emailPriority>";
				if ( self::EMAIL_FORMAT_TEXT != $this->settings["email_format"] ) {
					$email_tags .= "<emailBody>".utf8_xml_entity_encode($this->settings["html_body"])."</emailBody>";
				}
				if ( self::EMAIL_FORMAT_HTML != $this->settings["email_format"] ) {
					$email_tags .= "<emailTextBody>".utf8_xml_entity_encode($this->settings["text_body"])."</emailTextBody>";
				}
				$email_tags .= "<emailFromAddress>".$this->settings["from_address"]."</emailFromAddress>";
				$email_tags .= "<emailFromName>".$this->settings["from_name"]."</emailFromName>";
				if ( "" != $this->settings["reply_to_address"] ) {
					$email_tags .= "<emailReplyToAddress>".$this->settings["reply_to_address"]."</emailReplyToAddress>";
				}
				if ( "" != $this->settings["reply_to_name"] ) {
					$email_tags .= "<emailReplyToName>".$this->settings["reply_to_name"]."</emailReplyToName>";
				}
				if ( "" != $this->settings["cc_address"] ) {
					$email_tags .= "<emailCCAddress>".$this->settings["cc_address"]."</emailCCAddress>";
				}
				if ( "" != $this->settings["bcc_address"] ) {
					$email_tags .= "<emailBCCAddress>".$this->settings["bcc_address"]."</emailBCCAddress>";
				}
			}
			$process_mass_email_query = "
				<mbapi>
					<command>ProcessMassEmail</command>
					<params>
						$contact_id_tags
						$email_tags
					</params>
				</mbapi>
			";

			$result = dispatchMBAPI(
				array("mbapi" => array(
					"command" => "SetSystemQueue",
					"subCommand" => "insert",
					"params" => array(
						"systemQueueCommand" => "ProcessMassEmail",
						"systemQueueCommandInput" => utf8_xml_entity_encode($process_mass_email_query),
						"systemQueueDateToRun" => mktime()
					)
				)),
				1
			);
			if ( $result["mbapi"][0]["header"][0]["errorCount"][0] ) {
				foreach ( $result["mbapi"][0]["header"][0]["errors"][0]["error"] as $error ) {
					$this->addError($error["title"][0], $error["message"][0]);
				}
			}
			if ( $this->getNumErrors() ) {
				return false;
			}
		}

		return true;
	}

	private function getDefaultSettings()
	{
		$admin = Factory()->Admin(getCurrentSession()->user_id);
		return array (
			"client_ids" => "",
			"client_status" => getStatusID("client", TRANS_ACTIVE),
			"client_group_id" => "",
			"server_id" => "",
			"server_group_id" => "",
			"product_id" => "",
			"contact_type" => TRANS_SEND_EMAILS_PRIMARY_CONTACT_ONLY,
			"template_group_id" => "",
			"contact_ids" => array(),
			"email_format" => self::EMAIL_FORMAT_HTML,
			"email_priority" => 3,
			"from_name" => $admin->admin_first_name." ".$admin->admin_last_name,
			"from_address" => $admin->admin_email,
			"reply_to_name" => "",
			"reply_to_address" => "",
			"cc_address" => "",
			"bcc_address" => "",
			"subject" => "",
			"html_body" => "",
			"text_body" => "",
		);
	}

	private function loadSettings()
	{
		if ( isset($_REQUEST["data"]) ) {
			$this->settings = array_merge($this->settings, unserialize(base64_decode($_REQUEST["data"])));
		}
		if ( isset($_REQUEST["send_emails_settings"]) ) {
			$this->settings = array_merge($this->settings, bgpc("send_emails_settings"));
		}
	}

	private function saveSettings()
	{
		$this->smartyVars["data"] = base64_encode(serialize($this->settings));
	}

	private function getClientGroups()
	{
		$attribute_id_name_map = array();
		$attribute_groups = Factory()->AttributeGroupList();
		foreach ( $attribute_groups as $attribute_group ) {
			$attribute_id_name_map[$attribute_group->id] = $attribute_group->group_name;
		}
		asort($attribute_id_name_map);
		return $attribute_id_name_map;
	}

	private function getServers()
	{
		$server_id_name_map = array();
		$server_groups = Factory()->ServerGroupList();
		foreach ( $server_groups as $server_group ) {
			$servers = $server_group->ControlPanelList();
			foreach ( $servers as $server ) {
				$server_params = Factory()->ConfigParamList(array("config_group_id" => $server->id, "config_param_name" => "moduleDisplayName"));
				foreach ( $server_params as $server_param ) {
					$display_name = $server_param->config_param_value;
					if ( "" != $display_name ) {
						$server_id_name_map[$server->id] = $server_param->config_param_value;
					}
				}
			}
		}
		asort($server_id_name_map);
		return $server_id_name_map;
	}

	private function getServerGroups()
	{
		$server_group_id_name_map = array();
		$server_groups = Factory()->ServerGroupList();
		foreach ( $server_groups as $server_group ) {
			$server_group_id_name_map[$server_group->id] = $server_group->server_group_name;
		}
		asort($server_group_id_name_map);
		return $server_group_id_name_map;
	}

	private function getProducts()
	{
		$product_id_name_map = array();
		$products = Factory()->ProductList(array("product_is_addon" => 0));
		foreach ( $products as $product ) {
			$product_id_name_map[$product->id] = $product->product_name;
		}
		asort($product_id_name_map);
		return $product_id_name_map;
	}

	private function getTemplateGroups()
	{
		$template_group_id_name_map = array();
		$template_groups = Factory()->EmailTemplateGroupList(array("email_action_id" => getAPITypeID("emailAction", "userDefinedEmail")));
		foreach ( $template_groups as $template_group ) {
			$template_group_id_name_map[$template_group->id] = $template_group->email_tpl_group_name;
		}
		asort($template_group_id_name_map);
		return $template_group_id_name_map;
	}

	private function getContacts()
	{
		$contacts = array();
		foreach ( $this->settings["contact_ids"] as $contact_id ) {
			$contact = Factory()->ClientContact($contact_id);
			$contact_key = $contact->contact_first_name.$contact->contact_last_name.$contact->contact_email;
			$contacts[$contact_key] = array (
				"contact_id" => $contact->id,
				"contact_first_name" => $contact->contact_first_name,
				"contact_last_name" => $contact->contact_last_name,
				"contact_email" => $contact->contact_email,
			);
		}
		ksort($contacts);
		return array_values($contacts);
	}

	private function getEmailPriorityName($priority)
	{
		if ( $priority < 3 ) {
			return TRANS_HIGH;
		}
		if ( $priority > 3 ) {
			return TRANS_LOW;
		}
		return TRANS_NORMAL;
	}

	private function addMessages()
	{
		foreach ( $this->errors as $error  ) {
			$this->smartyVars["errorMessages"][] = $error[1];
		}
		foreach ( $this->warnings as $warning  ) {
			$this->smartyVars["warningMessages"][] = $warning->message;
		}
	}

	private $settings;
}
