<?php

require_once("lib-action/action.php");

/**
 * This action displays a list of events, and the links to actions that act upon those events, to the screen. All items
 * displayed have already been added to the system queue. From this action, the user should be able to add, edit, or
 * remove items from the event queue. To get our list of events, we call GetSystemQueue.
 */
class ShowEvents extends MB_Action
{
	public function __construct()
	{
		parent::__construct();

		$this->params = array(
            "systemQueueID",
            "systemParentQueueID",
            "systemQueueCommand",
            "systemQueueCommandInput",
			"configGroupID",
			"serverGroupID",
			"systemQueueStatus",
			"systemQueueIsRecurring",
            "systemQueueDateCreated", "systemQueueDateCreatedLeast", "systemQueueDateCreatedGreatest",
            "systemQueueDateStarted", "systemQueueDateStartedLeast", "systemQueueDateStartedGreatest",
            "systemQueueDateFinished", "systemQueueDateFinishedLeast", "systemQueueDateFinishedGreatest"
        );
		$this->sortColumns = array(
			"systemQueueID",
			"systemQueueDateToRun",
			"systemQueueCommand",
			"systemQueueCommandInput",
			"systemQueueDateCreated",
					"systemQueueDateStarted",
			"systemQueueDateFinished",
			"systemQueueIsRecurring"
		);
		// search in a specific column
		if (notNull($_REQUEST["queryColumn"])){
			if (notNull($_REQUEST["query"])) {
				$_REQUEST[$_REQUEST["queryColumn"]] = $_REQUEST["query"];
			}
		}
	}

	public function execute()
	{
		parent::execute();

		$this->initListParams();
		$registrars = $this->getRegistrarOptions();
		$panels = $this->getPanelOptions();

		if(!(isset($_REQUEST['serverGroupID']) && $_REQUEST['serverGroupID'])) {
			unset($_REQUEST['serverGroupID']);
		}
		if(!(isset($_REQUEST['configGroupID']) && $_REQUEST['configGroupID'])) {
			unset($_REQUEST['configGroupID']);
		}
		if(!(isset($_REQUEST['systemQueueStatus']) && $_REQUEST['systemQueueStatus'] !== "")) {
			unset($_REQUEST['systemQueueStatus']);
		}
		if(!(isset($_REQUEST['systemQueueIsRecurring']) && $_REQUEST['systemQueueIsRecurring'] !== "")) {
			unset($_REQUEST['systemQueueIsRecurring']);
		}

		$c = count($_REQUEST["systemQueueIDs"]);
		if ($c > 0) {
			$xml = array("systemQueueIDs" => array("systemQueueID" => $_REQUEST["systemQueueIDs"]));
			$systemQueueIDsArgs = "";
			for($i=0;$i<$c;$i++){
				$systemQueueIDsArgs .= "&systemQueueIDs[]=".$_REQUEST["systemQueueIDs"][$i];
			}
		} else {
			$xml = array();
		}
		$query = $this->getMBAPIQuery("GetSystemQueue", "", $xml, 1, 1, 1, 1);
		require_once("lib-mbapi/mbapi.php");
		$data = dispatchMBAPI($query, 1);
		$results =& $data["mbapi"][0]["results"][0];
		if($this->addMBAPIErrors($data) === 0) {

			// set the sorting args for display
			$this->smartyVars["displaySortColumnName"] = $data["mbapi"][0]["header"][0]["sortColumnName"][0];
			$this->smartyVars["displaySortColumn"] = $data["mbapi"][0]["header"][0]["sortColumn"][0];
			$this->smartyVars["displaySortDir"] = $data["mbapi"][0]["header"][0]["sortDir"][0];

			$events = collapseArrayElements(protectArray($results["systemQueue"][0]["systemQueueEvent"]));
			if($events){
				foreach($events as $i => $event) {
					if(isset($events[$i]["systemQueueLogs"]["systemQueueLog"])) {
						$events[$i]["systemQueueLogs"]["systemQueueLog"] = (array) $events[$i]["systemQueueLogs"]["systemQueueLog"];
					}
				}
			}
			$this->smartyVars["events"] = $events;
			$this->smartyVars["registrars"] = $registrars;
			$this->smartyVars["panels"] = $panels;
			$this->smartyVars["statuses"] = getStatuses("queue");
			$this->smartyVars["paginationLinks"] = collapseArrayElements($results["paginationLinks"]);
			$this->smartyVars["sortColumns"] = array();
			$this->smartyVars["systemQueueIDsArgs"] = $systemQueueIDsArgs;


			$this->mergeServerGroupData();
			$this->addTemplate("events/ShowEvents");
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	private function initListParams()
	{
		$listName = "UIEventList";
		$session = getCurrentSession();
		$eventListParams = reset(Factory()->ListParamList(array(
			"user_id" => $session->user_id,
			"user_type" => $session->user_type,
			"list_name" => $listName,
		)));
		if (false === $eventListParams) {
			$eventListParams = Factory()->ListParam();
			// Default parameters
			$eventListParams->user_id = $session->user_id;
			$eventListParams->user_type = $session->user_type;
			$eventListParams->list_name = $listName;
			$eventListParams->sort = "systemQueueDateToRun";
			$eventListParams->reverse = 0;
			$eventListParams->page_num = 0;
			$eventListParams->update();
		}

		$sort = bgpc('sortColumn');
		if (!is_null($sort))
		{
			$sortDir = bgpc('sortDir');
			// if new command with the same sorting, then reverse sorting; else reset reverse
			$eventListParams->reverse = ("asc" == $sortDir) ? 0 : 1;
			$eventListParams->sort = $sort;
		}
		$_REQUEST["sortColumn"] = $eventListParams->sort;
		$_REQUEST["sortDir"] = (0 == $eventListParams->reverse) ? "asc" : "desc";

		$recordOffset = bgpc('recordOffset');
		$defaultNumRows = MBACTION_DEFAULT_NUM_RECORDS;
		if (!is_null($recordOffset)) {
			$eventListParams->page_num = $recordOffset/$defaultNumRows;
		}
		$_REQUEST["recordOffset"] = $eventListParams->page_num*$defaultNumRows;
		$eventListParams->update();
	}
}
