<?php

if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");

require_once mb_path("/lib-action/include/graph/core/php5/jpgraph.php");
require_once mb_path("/lib-action/include/graph/core/php5/jpgraph_bar.php");

/**
 * Generate and return a graph image to the user. If a graph for the given credentials has already been generated, and the 
 * cached image is within its lifetime, the cached image will be returned.
 *
 * @author  Brian Smith
 */
class ShowGenericBar extends MB_Action {

	/**
	 * The main method of this class that executes the ShowGenericBar action, and returns success or failure.
	 *
	 * @return  int  1 if this action executed successfully, or 0 if otherwise.
	 */
	function execute()
	{
		parent::execute();
		
		define("DEFAULT_GFORMAT", "auto");

		// Setup data
		// What width & height should our graph have?
		$width = (int) $_REQUEST["width"];
		if(!notZero($width)) {
			$width = 300;
		}
		$height = (int) $_REQUEST["height"];
		if(!notZero($height)) {
			$height = 40;
		}

		$limit = (int) $_REQUEST["limit"];
		if (!notZero($limit)) {
			$limit = 100;
		}

		$amount = (int) $_REQUEST["amount"];
		if (!notZero($amount)) {
			$amount = 0;
		}

		$tickCount = (int) $_REQUEST["tickCount"];
		if (!notZero( $tickCount )) {
			$tickCount = 5;
		}

		$tickInc = (int) $_REQUEST["tickInc"];

		// Setup graph
		$graph = new Graph( $width, $height );
		//$graph->SetFrame(false);
		$graph->SetColor("gray");
		$graph->legend->Hide();
		
		if ($amount > $limit) {
			$under = $limit;
			$over = $amount - $limit;
			
			$graph->SetScale("int", 0, $amount);
			
			$bar1 = new BarPlot( array($under) );
			$bar1->SetLineWeight( 3 );
			$bar1->SetWidth( 1 );
			$bar1->SetFillGradient( "orange", "steelblue", GRAD_HOR );
			

			$bar2 = new BarPlot( array($over) );
			$bar2->SetLineWeight( 3 );
			$bar2->SetWidth( 1 );
			$bar2->SetFillGradient( "red", "orange", GRAD_HOR );

			$abPlot = new AccBarPlot( array($bar1, $bar2) );
			$abPlot->SetLineWeight( 3 );
			
			$plot = new GroupBarPlot(array($abPlot));
			$plot->SetWidth( 1 );

			$percentText = (!$limit ? 0 : ($amount / $limit)*100);
			$caption = new Text((string)sprintf("%01.1f%%", $percentText), $height/2, ($width/2)-20);
			$caption->SetAlign("right");
			$caption->SetColor("navy");
			$caption->show();
			$graph->AddText( $caption );
			unset( $percentText );
		} else {
			$graph->SetScale("int", 0, $limit);

			$tmpArray = (array) $amount;
			$plot = new BarPlot( $tmpArray );
			unset($tmpArray);
			$plot->SetAlign("left");
			$plot->SetLineWeight( 3 );
			$plot->SetValuePos("bottom");
			$plot->value->SetFormat('%01.1f%%', '-%01.1f%%');
			
			$plot->SetFillGradient( "steelblue", "lightsteelblue", GRAD_HOR );
			$plot->SetWidth( 1 );

			$percentText = (!$limit ? 0 : ($amount / $limit)*100);
			$caption = new Text((string)sprintf("%01.1f%%", $percentText), $height/2.6, ($width/2)-15);
			$caption->SetAlign("right");
			$caption->SetColor("navy");
			$caption->show();
			$graph->AddText( $caption );
			unset( $percentText );
		}
		
		$graph->Add( $plot );
		
		$graph->yaxis->scale->ticks->Set( ( notZero($tickInc) ? $tickInc : ($limit/$tickCount) ) );
		
		$graph->SetAxisStyle(AXSTYLE_BOXIN);
		
		$adj = ($height - $width)/2;
		$graph->img->setMargin(10, 10, $adj, $adj);

		// set the center and rotate
		$graph->img->SetCenter(floor($width/2), floor($height/2));
		$graph->setAngle(90);

		$graph->xaxis->img->SetTranslation(0, -($adj+10));
		$graph->xaxis->img->right_margin = 5;
		$graph->xaxis->img->width = $height+( ($_REQUEST["count"]=="false") ? 15 : 0);
		
		// Display the graph
		$graph->Stroke();
				
		return $this->hasExecutedSuccessfully();
	}
} // ShowGenericBar

$a = new ShowGenericBar();
$a->execute();
