<?php
require_once("lib-action/action.php");
require_once("UI/Page.php");

class InvoiceFacturaSettingsEditPage extends UIPage
{
	public function storeJS(UIJavaScriptStorage $storage)
	{
		parent::storeJS($storage);
		if ($storage->hasJS(get_class()))
			return;

		$js = "
		function disableControls(el)
		{
			f = document.forms['{$this->getDefaultForm()->name}'];
			for (i = f.elements.length; i--;) {
				var item = f.elements[i];
				if ('checkbox' == item.type && el != item) {
					if (el.checked == true) {
						item.disabled = false;
					} else {
						item.disabled = true;
					}
				}
			}
		}
		";
		$storage->addJS(get_class(), $js);
	}

	public function createControls()
	{
		$controls = $this->getControls();
		foreach($controls as $panel) {
			$inpitPanel = UIFactory()->InputPanel(blmsg($panel["title"]));
			foreach($panel["controls"] as $control) {
				$description = array(
					"id"		=> $control[0],
					"title"		=> $control[1],
					"type"		=> $control[2],
				);

				$inputControl = UIFactory()->InputConfig($description);
				$inputControl->value = Settings()->$description["id"];
				if ("invoiceFactura_enableGeneration" == $control[0]) {
					$inputControl->onchange = "disableControls(this);";
				}
				$inpitPanel->addInputControl($inputControl);
				$this->module_input_controls[] = $inputControl;
			}
			$this->addInputPanel($inpitPanel);
		}
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function processInput()
	{
		try {
			foreach ($this->module_input_controls as $control) {
				$name = $control->name;
				Settings()->$name = $control->value;
			}
			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SETTINGS_SAVED_SUCCESSFULLY");
		} catch (ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
		}

		return !$this->hasWarnings() && !$this->hasErrors();
	}

	protected function getSubmitButtons()
	{
		return array(UIFactory()->SubmitButton("ok", blmsg("TRANS_SAVE"), UIPageController::CMD_OK));
	}

	protected function getControls()
	{
		return array(
		    "commonSettings" => array(
			"title" => "TRANS_INVOICE_FACTURA_SETTINGS__COMMON_SETTINGS",
			"controls" => array(
			    array("invoiceFactura_enableGeneration", "TRANS_INVOICE_FACTURA_SETTINGS__ENABLE_GENERATION", "b"),
			)
		    ),
		    "personsSettings" => array(
			"title" => "TRANS_INVOICE_FACTURA_SETTINGS__PERSONS_SETTINGS",
			"controls" => array(
			    array("invoiceFactura_persons_generateInvoiceFacturas", "TRANS_INVOICE_FACTURA_SETTINGS__GENERATE_INVOICE_FACTURAS", "b"),
			    array("invoiceFactura_persons_sendEmail", "TRANS_INVOICE_FACTURA_SETTINGS__SEND_EMAIL", "b"),
			)
		    ),
		    "companiesSettings" => array(
			"title" => "TRANS_INVOICE_FACTURA_SETTINGS__COMPANIES_SETTINGS",
			"controls" => array(
			    array("invoiceFactura_companies_generateInvoiceFacturas", "TRANS_INVOICE_FACTURA_SETTINGS__GENERATE_INVOICE_FACTURAS", "b"),
			    array("invoiceFactura_companies_sendEmail", "TRANS_INVOICE_FACTURA_SETTINGS__SEND_EMAIL", "b"),
			)
		    ),
		    "individuals" => array(
			"title" => "TRANS_INVOICE_FACTURA_SETTINGS__INDIVIDUALS_SETTINGS",
			"controls" => array(
			    array("invoiceFactura_individuals_generateInvoiceFacturas", "TRANS_INVOICE_FACTURA_SETTINGS__GENERATE_INVOICE_FACTURAS", "b"),
			    array("invoiceFactura_individuals_sendEmail", "TRANS_INVOICE_FACTURA_SETTINGS__SEND_EMAIL", "b"),
			)
		    ),
		);
	}
}

class EditInvoiceFacturaSettings extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$controller  = UIFactory()->PageController();
		$controller->addPage($edit_page = new InvoiceFacturaSettingsEditPage($controller));

		$edit_page->title = blmsg("TRANS_INVOICE_FACTURA_SETTINGS");
		$edit_page->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$edit_page->parent_url = admin_action_url("configs", "system", "show");

		$controller->processRequest();
		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}