<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-billing/include/accounting/Refund.php");

/**
 * This Action object takes information provided by the ProcessTransactionRefund action,
 * and uses it to refund the transaction amount to the customer.
 */
class ProcessTransactionRefundResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if (is_null($transactionID = bgpc("transactionID"))) {
			throw new ProductException(ProductException::E_OBJECT_NOT_FOUND, "Transaction not found: id={$transactionID}");
		}
		require_once("object_managers/CryptManager.php");
		if ("" == ($key = bgpc("key"))) {
			$this->addError(blmsg("TRANS_ACTIONERROR"), "No encryption key given.");
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		} else if (CryptManager::isEncryptionEnabled() && CryptManager::isLekEnabled()) {
			try {
				CryptManager::checkLekPin($key, getCurrentSession()->user_id);
			} catch(ProductException $e) {
				billing_put_error($e->getMessage());
				redirect("invoices.php", getActionID("ProcessTransactionRefund"), array("invoiceID" => bgpc("invoiceID"), "transactionIDs" => array($transactionID)));
			}
		}

		$invoiceID = ((int) $_REQUEST["invoiceID"]);
		$markUnpaid = (int)$_REQUEST["markUnpaid"];

		if ($_REQUEST["refundType"]=="partial"){
			$_REQUEST["refundAmount"] = $this->getNumberFromInput($_REQUEST["lesserAmount"]);
			$markUnpaid = 0;
		}

		require_once("object_managers/CryptManager.php");
		CryptManager::setKey($key);

        $gatewayAction = "void" == bgpc("gatewayAction") ? "void" : "refund";

		$transaction = Factory()->Transaction($transactionID);

		if (is_null($billingAccount = $transaction->ClientBillingAccount)) {
			$this->addError(blmsg("TRANS_ACTIONERROR"), "Sorry, the billing account this refund is for was deleted.  To perform this refund please go to your merchant account provider.");
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		$transactionAmountRemained = round(doubleval($transaction->transaction_amount) - doubleval($transaction->transaction_amount_refunded), 2);
		$refundAmount = doubleval(bgpc("refundAmount")) ? doubleval(bgpc("refundAmount")) : $transactionAmountRemained;
		if ($refundAmount > $transactionAmountRemained) {
			$refundAmount = $transactionAmountRemained;
		}

		try {
			require_once("object_managers/PaymentManager.php");
			if ("void" == bgpc("gatewayAction")) {
				$success = PaymentManager::void($transaction, $refundAmount);
			} else {
				$success = PaymentManager::refund($transaction, $refundAmount);
			}
			if (!$success) {
				$this->addError(blmsg("TRANS_ACTIONERROR"), $transaction->transaction_notes);
				$this->addErrorTemplates();
				return $this->hasExecutedSuccessfully();
			}
		} catch (ProductException $e) {
			$this->addOperationErrors($e);
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		$invoiceInfo = correctArray(grabFirstResultFromMBAPIArray(array(
			"mbapi" => array(
				"command" => "GetInvoices",
				"params" => array(
					"invoiceID" => $invoiceID,
					"getInvoiceLineItemsData" => 1,
				),
			),
		)));

		$lineItemIDs = (array)$_REQUEST["lineItemIDs"];

		if (array_sum($lineItemIDs)){

			if ($markUnpaid){
				dbg(dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetInvoices",
						"subCommand" => "update",
						"params" => array(
							"invoiceID" => $invoiceID,
							"invoiceStatus" => getAPIStatusID("invoice", "due"),
							"invoiceAmountPaid" => $invoiceInfo["invoiceAmount"]-$refundAmount,
						),
					),
				)), 1));

				foreach ((array)$lineItemIDs as $liID){
					dbg(dispatchMBAPI(toXML(array(
						"mbapi" => array(
							"command" => "SetTransactionSuccessRel",
							"subCommand" => "delete",
							"params" => array(
								"lineItemID" => $liID,
								"transactionID" => $transactionID,
							),
						),
					)), 1));
				}
			}

			$return = dispatchMBAPI(toXML(array(
				"mbapi" => array(
					"command" => "ProcessRefundAccounting",
					"params" => array(
						"transactionID" => $transactionID,
						"refundAmount" => $refundAmount,
						"ledgerAccountName" => ($markUnpaid)?"accounts_receivable":"refund_expense",
						"invoiceID" => $invoiceID,
					),
				),
			)), 1);
			if (!$return["mabpi"][0]["header"][0]["success"][0]){
				dbg($return);
			}

		} else {

			$clientID = $invoiceData["lineItems"]["lineItem"][0]["clientID"];

			if ($markUnpaid){
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetTransactionSuccessRel",
						"subCommand" => "delete",
						"params" => array(
							"transactionID" => $transactionID,
						),
					),
				)));
				$amountPaid = $invoiceInfo["invoiceAmountPaid"] - $transaction->transaction_amount;
				$status = getAPIStatusID("invoice", "due");
				dispatchMBAPI(toXML(array(
					"mbapi" => array(
						"command" => "SetInvoices",
						"subCommand" => "update",
						"params" => array(
							"invoiceID" => $invoiceID,
							"invoiceStatus" => $status,
							"invoiceAmountPaid" => $amountPaid,
						),
					),
				)));
			}

			$merge = array();
			if ($refundAmount != $transaction->transaction_amount) {
				$merge = array(
					"refundAmount" => $refundAmount,
				);
			}

			$return = dispatchMBAPI(toXML(array(
				"mbapi" => array(
					"command" => "ProcessRefundAccounting",
					"params" => array(
						"transactionID" => $transactionID,
						"ledgerAccountName" => ($markUnpaid)?"accounts_receivable":"refund_expense",
						"invoiceID" => $invoiceID,
					) + $merge,
				),
			)), 1);
			if (!$return["mabpi"][0]["header"][0]["success"][0]){
				dbg($return);
			}

		}

		if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
			redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
		} else {
			redirect("invoices.php?action=".getActionID("ShowInvoices")."&sid=".secureInput($_REQUEST["sid"]));
		}
	}
}