<?php
require_once("lib-action/action.php");

/**
 * This class is a child class of MB_Action. Using $_REQUEST["invoiceID"] it displays all
 * the details for the found invoice, and only displays the details of a single invoice.
 */
class ShowInvoiceDetails extends MB_Action
{
	public $params = array("invoiceID");
    public function __construct()
    {
        unset($_GET["ctr"]);
	    unset($_GET["act"]);
        parent::__construct();
    }


	public function execute()
	{
		global $CONF;
		parent::execute();
		set_time_limit(0);

		$invoiceID = ((int) $_REQUEST["invoiceID"]);
		if(!$invoiceID) {
			$this->addError(TRANS_ACTIONERROR, "No Invoice ID given.");
		} else {
			if (strlen($_REQUEST["query"])){
				$packageIDs = grabResultsFromMBAPIArray(array(
					"mbapi" => array(
						"command" => "GetMatchingLineItems",
						"params" => array(
							"invoiceID" => (int)$_REQUEST["invoiceID"],
							"query" => secureInput($_REQUEST["query"]),
						),
					),
				));
			} else {
				$packageIDs = false;
			}

			$query = array(
				"mbapi" => array(
					"command" => "GetInvoices",
					"params" => array(
						"invoiceID" => (int)$_REQUEST["invoiceID"],
						"getInvoiceLineItemsData"=>1,
						"getInvoiceLineItemsPaymentsData"=>1,
						"getTaxZonesData"=>1,
						"lineItemOffset"=>(int)$_REQUEST['page']*10
					) + (($packageIDs)?array("packageID" => array_unique($packageIDs)):array()),
				),
			);

			$data = dispatchMBAPI(toXML($query), 1);

			if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
				if (((int) $data["mbapi"][0]["header"][0]["numResults"][0]) > 0){

					$this->smartyVars["invoice"] = correctArray($data["mbapi"][0]["results"][0]["invoices"][0]["invoice"][0]);

					if ((int)$this->smartyVars["invoice"]["creditNoteForInvoiceID"]){
						$this->smartyVars["sourceInvoice"] = grabFirstResultFromMBAPIArray(array(
							"mbapi" => array(
								"command" => "GetInvoices",
								"params" => array(
									"invoiceID" => (int)$this->smartyVars["invoice"]["creditNoteForInvoiceID"],
								),
							),
						));
					} else {
						$this->smartyVars["creditNote"] = grabFirstResultFromMBAPIArray(array(
							"mbapi" => array(
								"command" => "GetInvoices",
								"params" => array(
									"creditNoteForInvoiceID" => (int)$this->smartyVars["invoice"]["invoiceID"],
								),
							),
						));
					}

					$this->smartyVars["totalTax"] = $this->smartyVars["invoice"]["totalTax"];

					$this->smartyVars["totalPages"] = $pages = ceil($this->smartyVars["invoice"]["totalLineItems"]/10);
					$this->smartyVars["page"] = (int)$_REQUEST["page"];
					$this->smartyVars["invoicePageLinks"] = array();
					if ($pages>1){
						for ($i=0; $i<$pages; $i++){
							$this->smartyVars["invoicePageLinks"][$i] = $i;
						}
					}

					$this->smartyVars["taxZones"] = array();
					$this->smartyVars["addonIDs"] = array();
					$this->smartyVars["addonTotals"] = array();
					$this->smartyVars["lineItemIDs"] = array();
					$lastMainLineItem = 0;
					$packageIDsForDomainGetting = array();
					foreach ($this->smartyVars["invoice"]["lineItems"] as $k=>$v){
						$this->smartyVars["lineItemIDs"][] = $v["lineItemID"];
						$packageIDsForDomainGetting[] = $v["packageID"];
						if ($v["packageAddonID"]){
							$this->smartyVars["addonIDs"][] = $v["packageAddonID"];
						} else {
							$lastMainLineItem = $v["lineItemID"];
						}
						$this->smartyVars["addonTotals"]["total"][$lastMainLineItem] += $v["lineItemTotalAmount"];
						$this->smartyVars["addonTotals"]["setup"][$lastMainLineItem] += $v["lineItemSetupAmount"];
						$this->smartyVars["addonTotals"]["tax"][$lastMainLineItem] += $v["lineItemTaxAmount"];

						$this->smartyVars["invoice"]["lineItems"][$k]["packageComments"] = utf8_html_entity_decode($this->smartyVars["invoice"]["lineItems"][$k]["packageComments"]);
						$this->smartyVars["invoice"]["lineItems"][$k]["lineItemAmountDue"] = $v["lineItemTotalAmount"]-$v["lineItemPaymentAmount"];
						$taxZones = (array)$this->smartyVars["invoice"]["lineItems"][$k]["taxZones"];
						dbg("tax zones: ");
						dbg($taxZones);
						$this->smartyVars["invoice"]["lineItems"][$k]["taxZones"] = $taxZones;
						foreach ($taxZones as $zone){
							if (!$this->smartyVars["taxZones"][$zone["taxZoneID"]]){
								$this->smartyVars["taxZones"][$zone["taxZoneID"]] = $zone;
							}
							$this->smartyVars["taxZones"][$zone["taxZoneID"]]["taxTotal"] += $zone["lineItemTaxTotal"];
						}
					}

					$this->smartyVars["taxZones"] = array_values($this->smartyVars["taxZones"]);

					$clientID = (int)$this->smartyVars["invoice"]["clientID"];
					$clientContactID = (int)$this->smartyVars["invoice"]["clientContactID"];
					$this->smartyVars["clientID"] = $clientID;

					if ((int)$clientID){
						$clientInfoCmd = "
							<mbapi>
								<command>GetClients</command>
								<showXMLHeader>0</showXMLHeader>
								<params>
									<mergePrimaryContact>1</mergePrimaryContact>
									<clientID>$clientID</clientID>
									<mergeDefaultBillingAccount>1</mergeDefaultBillingAccount>
									<getContactData>3</getContactData>
									<getBillingData>1</getBillingData>
									<getCompanyData>1</getCompanyData>
									<getAccountingData>1</getAccountingData>
								</params>
							</mbapi>";

						$clientData = dispatchMBAPI($clientInfoCmd, 1);
						$clientData = $clientData["mbapi"][0]["results"][0]["clients"][0]["client"][0];
						$contactData = $clientData["contacts"][0]["contact"][0];
						$companyData = $clientData;
					}

					if ((int)$clientContactID){
						$contactData = oneByID("contacts", $clientContactID);
					}

					$countryData = oneByID("countries", $contactData["countriesID"]);
					$contactData["countriesISO2"] = $countryData["countriesISO2"];

					$dpids = array_values(array_unique($packageIDsForDomainGetting));

					if (count($dpids)){
						$domainPackages = correctArray(grabResultsFromMBAPIArray($xx = array(
							"mbapi" => array(
								"command" => "GetPackages",
								"params" => array(
									"packageID" => $dpids,
									"getDomainData" => 1,
								),
							),
						)));

						$this->smartyVars["domains"] = array();
						foreach ((array)$domainPackages as $dpkg){
							$this->smartyVars["domains"][$dpkg["packageID"]] = $dpkg["domains"];
						}
					}

					$this->smartyVars["billingAccounts"] = (array)correctArray($clientData["billingAccounts"][0]["billingAccount"]);

					$this->smartyVars["billingAccountOptions"] = array();
					foreach ($this->smartyVars["billingAccounts"] as $row){
						if ($row["billingCardType"]==3){
							$thisID = $row["billingAccountID"];
							if (defined($CONF["types"]["billingAccount"][$row["billingAccountType"]])) {
								$thisDesc = constant($CONF["types"]["billingAccount"][$row["billingAccountType"]]);
							} else {
								$thisDesc = $CONF["types"]["billingAccount"][$row["billingAccountType"]];
							}
							$thisDesc .= " - ".billing_get_full_name($row["contactFirstName"], $row["contactLastName"]);
							$this->smartyVars["billingAccountOptions"][$thisID] = $thisDesc;
						}
					}

					$this->smartyVars["clientData"] = correctArray($clientData);
					$this->smartyVars["clientData"]["billingCardNum"] = "";
					$tmp = correctArray($clientData["accountingGeneralLedgers"][0]["accountingGeneralLedger"]);
					if ($tmp["currencyID"]){
						$tmp = array($tmp);
					}
					$this->smartyVars["clientCredits"] = $this->getAccountingCurrencyTotals($tmp, "customer_credits");
					ksort($this->smartyVars["clientCredits"]);
					$this->smartyVars["contactData"] = correctArray($contactData);
					$this->smartyVars["companyData"] = correctArray($companyData);

					$this->smartyVars["transactions"] = correctArray(grabResultsFromMBAPIArray(array(
						"mbapi" => array(
							"command" => "GetTransactions",
							"params" => array(
								"invoiceIDs" => array(
									"invoiceID" => $invoiceID,
								),
							),
						),
					)));
					dbg($this->smartyVars["transactions"]);

					$refundStatusList = array(
						getAPIStatusID("transactions", "successful"),
						getAPIStatusID("transactions", "captured"),
						getAPIStatusID("transactions", "partially_refunded"),
					);

					$getGatewayCapsStatusList = array_merge($refundStatusList, array(
						getAPIStatusID("transactions", "authorized")
					));

					$this->smartyVars["transactionStatuses"] = getStatuses("transactions");
					foreach ((array)$this->smartyVars["transactions"] as $k => $v){
						if ( "" == $v["paymentGatewayID"] ) {
							continue;
						}
						$payment_gateway = Factory()->PaymentGateway($v["paymentGatewayID"]);
						$this->smartyVars["transactions"][$k]["moduleName"] = $payment_gateway->moduleName;

						if (0 == $v["billingAccountID"] || 0 == Factory()->ClientBillingAccount($v["billingAccountID"])->billing_account_active) {
							continue;
						}
						if ( !in_array($v["authReturn"], $getGatewayCapsStatusList) ) {
							continue;
						}
						require_once("object_managers/PaymentGatewayManager.php");
						if (in_array($v["authReturn"], $refundStatusList) &&
							(PaymentGatewayManager::isOperationSupported($payment_gateway, PaymentGatewayManager::OPERATION_REFUND) ||
							 PaymentGatewayManager::isOperationSupported($payment_gateway, PaymentGatewayManager::OPERATION_VOID))
						) {
							$this->smartyVars["transactions"][$k]["canRefund"] = 1;
							$this->smartyVars["transactionActions"][$v["transactionID"]]["canRefund"] = 1;
						}
						if ($v["authReturn"]==getAPIStatusID("transactions", "authorized") &&
							PaymentGatewayManager::isOperationSupported($payment_gateway, PaymentGatewayManager::OPERATION_CAPTURE)
						) {
							$this->smartyVars["transactions"][$k]["canFinalize"] = 1;
							$this->smartyVars["transactionActions"][$v["transactionID"]]["canFinalize"] = 1;
						}
						if (in_array($v["authReturn"], $refundStatusList) &&
							(PaymentGatewayManager::isOperationSupported($payment_gateway, PaymentGatewayManager::OPERATION_PARTIAL_REFUND) ||
							 PaymentGatewayManager::isOperationSupported($payment_gateway, PaymentGatewayManager::OPERATION_PARTIAL_VOID))
						) {
							$this->smartyVars["transactions"][$k]["canPartialRefund"] = 1;
							$this->smartyVars["transactionActions"][$v["transactionID"]]["canPartialRefund"] = 1;
						}
					}

					$successRel = correctArray(grabResultsFromMBAPIArray(array(
						"mbapi" => array(
							"command" => "GetTransactionSuccessRel",
							"params" => array(
								"lineItemID" => $this->smartyVars["lineItemIDs"],
							),
						),
					)));
					dbg($successRel);
					$this->smartyVars["lineItemsPartialRefund"] = array();
					foreach ((array)$successRel as $relrow){
						if ($this->smartyVars["transactionActions"][$relrow["transactionID"]]["canPartialRefund"]){
							$this->smartyVars["lineItemsPartialRefund"][$relrow["lineItemID"]][$relrow["transactionID"]] = 1;
						}
					}

					$this->smartyVars["paymentMethods"] = blmsg_array(getTypes("payments"));
					$tmp = explode(";", $CONF["user"]["locale"]->localeArray["locale_pattern_currency"]);
					$tmp[0] = preg_replace("^¤#0.00", "¤#,##0.00", $tmp[0]);
					$tmp[1] = preg_replace("^¤#0.00", "¤#,##0.00", $tmp[1]);
					$CONF["user"]["locale"]->localeArray["locale_pattern_currency"] = $tmp[0].";".$tmp[1];
					$this->smartyVars["localeArray"] =  $CONF["user"]["locale"]->localeArray;

					// set up for using the date selector widget

					$this->formDefaults["transactionDateCreated"] = time();
					$this->setupDateSelector("transactionDateCreated");
					$this->mergeFormState();

					require_once("objects/Invoice.php");
					$this->smartyVars["invoiceStatuses"] = Invoice::statuses();

					// just one last thing, find out exactly how much is due.
					$this->smartyVars["invoice"]["invoiceAmountDue"] = $this->smartyVars["invoice"]["invoiceAmount"] - $this->smartyVars["invoice"]["invoiceAmountPaid"];
					$this->smartyVars["invoice"]["invoiceComments"] = utf8_html_entity_decode($this->smartyVars["invoice"]["invoiceComments"]);

					$this->addTemplate("invoices/ShowInvoiceDetails", $this->smartyVars);

				} else {
					$this->addError(TRANS_ACTIONERROR, "Invoice does not exist.");
				}
			}
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
