<?php

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This action displays the form to enter an activation key for KA
 */
class RetrieveKeysResponse extends MB_Action
{
	public function execute()
	{
		$keyHistory = array();
		$keys = array_merge(arrayFor(":configParams", array(
			"configGroupID" => internalConfigGroupID,
			"configParamName" => array("license"),
			"sortDir" => "DESC",
			"sortColumn" => "configParamID",
		)), arrayFor(":configParams", array(
			"configGroupID" => internalConfigGroupID,
			"configParamName" => array("license_archived"),
			"sortDir" => "DESC",
			"sortColumn" => "configParamID",
		)));
		
		$current = new OPEN_FUSION_LICENSE(CURRENT_LICENSE_KEY);
		if ($current->licenseParsed){
			$keyNumber = $current->keyNumber;
			$keyDomain = $current->licenseDomain;
			$licenseServerURL = $current->licenseServerURL;
		}
		
		foreach ((array)$keys as $k){
			$x = new OPEN_FUSION_LICENSE($k['configParamValue']);
			if ($x->licenseParsed){ // not interested in licenses that don't parse :-)
				$keyHistory[] = array(
					"keyNumber" => $x->keyNumber.".".$x->keyVersion,
					"updatePassword" => $x->updateTicket,
				);
			}
		}
		
		if (!$keyHistory){
			$this->smartyVars['success'] = false;
			$this->smartyVars['errorText'] = "You must first activate a license or upload a license before you can update it.";
		} else {
		
			dbg("INITIATING KA PKP REQUEST");
			$response = KA_PKP_Request::getKey(
				KAPRODUCTU,
				KAPRODUCTP,
				$keyHistory,
				createPPBInfoStruct(),
				array(
					'targetProductInfo' => (object)array(
						'productName' => 'plesk-billing',
						'productVersion' => KA_LICENSE_VERSION,
					),
					'guid' => $keyDomain,
				),
				$licenseServerURL
			);
			
			if ($response->cleanKeysHistory){
				dbg("was told to clean keys history, removing all old keys...");
				foreach ((array)$keys as $k){
					if ($k["configParamID"]!=CURRENT_KEY_CONFIG_PARAM_ID){
						dbg("removing key: ".$k["configParamID"]);
						deleteForKey(":configParams", $k["configParamID"]);
					}
				}
			}
	
			$key = $response->getKey();
			if ($response->detailResultCode==11){
				$this->smartyVars['upToDate'] = 1;
				$this->smartyVars['success'] = true;
			} elseif ($key->licenseParsed){
				if ($response->cleanKeysHistory){
					deleteForKey(":configParams", CURRENT_KEY_CONFIG_PARAM_ID);
				}
				$key->store();
				$this->smartyVars['success'] = true;
			} else {
				$this->smartyVars['success'] = false;
				switch ($response->resultCode){
					case 0:
					case 1:
						$this->smartyVars['errorText'] = $response->resultDesc;
						break;
					case 2:
						$this->smartyVars['errorText'] = "There has been an internal error.  Please try again later.  If you continue to have problems please contact support and give them error code <strong>".$response->detailResultCode."</strong>.";
						break;
					default:
						$this->smartyVars['errorText'] = "There has been an internal error.  Please contact support.";
						break;
				}
			}
		}
		
		$this->addTemplate("modules/RetrieveKeysResponse");

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();

	}
}