<?php

require_once("lib-action/action.php");
require_once("UI/Page.php");
require_once("objects/ControlPanel.php");

class EditPanelConfigPage extends UIPage
{
	public function createControls()
	{
		require_once("ModuleTool.php");
		require_once("DataObject.php");

		$this->module_input_controls = array();

		if (!is_null($panelID = $this->getValue("configGroupID"))) {
			$controlPanel =  Factory()->ControlPanel($panelID);
		} else {
			$controlPanel =  Factory()->ControlPanel();
			$controlPanel->module_name = Factory()->ConfigGroup($this->getValue("parentConfigGroupID"))->config_group_name;
			$controlPanel->server_group_id = $this->getValue("serverGroupID");
		}

		foreach (ModuleTool::getModuleControls("panel", $controlPanel->module_name, $controlPanel->id) as $inputPanel) {
			$inputPanel->initFromArray(DataObject::getMapOf($controlPanel->ConfigParams, "name", "value"));
			$this->module_input_controls = array_merge($this->module_input_controls, $inputPanel->getInputControls());
			if ( "" == $inputPanel->title ) {
				$inputPanel->title = blmsg("TRANS_CONFIGURATION_PARAMETERS");
			}
			$this->addInputPanel($inputPanel);
		}
		if (is_null(bgpc("command")) && ! is_null($controlPanel->id)) { // Check Panel Connection here in view only mode
			$this->checkPanelConnection($controlPanel);
		}
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function initControl()
	{
		if (!is_null($panelID = $this->getValue("configGroupID"))) {
			$panel = Factory()->ControlPanel($panelID);
			if ($panel->isSyncable() && !is_null($panel->sync_error)) {
                $this->addErrorMessage(UIFactory()->ErrorMessage(
					blmsg("TRANS_SYNC_THIS_PANEL_ERROR_MSG", array("ERROR" => nl2br(billing_safetyhtml($panel->sync_error))))
				));
			}
		}
		parent::initControl();
	}

	public function processInput()
	{
		$controlPanel = is_null($panelID = $this->getValue("configGroupID")) ? Factory()->ControlPanel() : Factory()->ControlPanel($panelID);
		$controlPanel->ConfigParams->clear();
		foreach ($this->module_input_controls as $control)
		{
			switch ($control->name)
			{
				case "modulePassword":
					require_once("lib-tk/include/crypt/crypt.php");
					if ("" != $control->value)
					{
						require_once("lib-tk/include/crypt/crypt.php");
						$controlPanel->ConfigParams[$control->name]->value = getEncryptedText(getStuff($control->value));
					}
					break;
				case "api":
					if ("" != $control->value)
						$controlPanel->ConfigParams[$control->name]->value = rtrim($control->value, "/");
					break;
				default:
					$controlPanel->ConfigParams[$control->name]->value = $control->value;
			}
		}

		if (! $this->checkPanelConnection($controlPanel, true))
			return false;

		try {
			require_once("object_managers/ControlPanelManager.php");
			ControlPanelManager::update($controlPanel);
			if(ControlPanel::SYNC_STATUS_ERROR == $controlPanel->sync_status) {
				getCurrentSession()->Storage->warningMessages[] = blmsg(
					"TRANS_PANEL_UPDATED_WARNING_MSG",
					array("ERROR" => $controlPanel->sync_error)
				);
			} else {
				getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_PANEL_UPDATED_SUCCESSFULLY_MSG");
			}
		} catch (ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
		}

		return ! $this->hasWarnings() && ! $this->hasErrors();
	}

	private function checkPanelConnection(ControlPanel $controlPanel, $dont_show_version_warning = false)
	{
		if ($controlPanel->isPlesk()) {
			try {
				require_once("PleskAPIFactory.php");
				PleskAPIFactory::getInstance(array("1.6.3.0"), $controlPanel);
			} catch (PleskProtocolNotFound $e) {
				if (true !== $dont_show_version_warning)
					$this->addWarningMessage(UIFactory()->WarningMessage(blmsg("TRANS_PLESK_PANEL_DOES_NOT_SUPPORTED_LESS_VERSION_10_MSG")));
				return true;
			} catch (PleskAPIException $e) {
				$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
				return false;
			} catch (ProductException $e) {
				$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
				return false;
			}
		}
		return true;
	}

	private $module_input_controls;
	private $plesk_sso_status;
}

class EditPanelConfig extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$config_group_id = bgpc("configGroupID");
		$parent_config_group_id = bgpc("parentConfigGroupID");
		$server_group_id = bgpc("serverGroupID");

		if (1 == bgpc("sync")) {
			try {
				$controlPanel = Factory()->ControlPanel($config_group_id);
				$controlPanel->sync_tasks |= ControlPanel::SYNC_NEED_UPDATE | ControlPanel::SYNC_NEED_REPAIR_SSO;
				require_once("object_managers/ControlPanelManager.php");
				ControlPanelManager::sync($controlPanel);
				if (ControlPanel::SYNC_NO_TASKS == $controlPanel->sync_tasks) {
					getCurrentSession()->Storage->infoMessages[] = blmsg(
						"TRANS_PANEL_SYNC_SUCCESS_MSG",
						array("NAME" => $controlPanel->name, "PANEL_ID" => $controlPanel->id)
					);
				} else {
					getCurrentSession()->Storage->errorMessages[] = blmsg(
						"TRANS_PANEL_SYNC_FAILURE_MSG",
						array("NAME" => $controlPanel->name, "PANEL_ID" => $controlPanel->id, "ERROR" => $controlPanel->sync_error)
					);
				}
			} catch (ProductException $e) {
				getCurrentSession()->Storage->errorMessages[] = blmsg(
					"TRANS_PANEL_SYNC_FAILURE_MSG",
					array("NAME" => $controlPanel->name, "PANEL_ID" => $controlPanel->id, "ERROR" => $e->getMessage())
				);
			}
			redirect(action_url("servergroups", "ShowServerGroups"));
		}
		if (1 == bgpc("syncAll")) {
			try {
				$controlPanel = Factory()->ControlPanel($config_group_id);
				require_once("object_managers/ControlPanelManager.php");
				ControlPanelManager::asyncAll($controlPanel);
				getCurrentSession()->Storage->infoMessages[] = blmsg(
					"TRANS_PANEL_SYNC_ALL_SUCCESS_MSG",
					array("NAME" => $controlPanel->name, "PANEL_ID" => $controlPanel->id)
				);
			} catch (ProductException $e) {
				getCurrentSession()->Storage->errorMessages[] = blmsg(
					"TRANS_PANEL_SYNC_ALL_FAILURE_MSG",
					array("NAME" => $controlPanel->name, "PANEL_ID" => $controlPanel->id, "ERROR" => $e->getMessage())
				);
			}
			redirect(action_url("servergroups", "ShowServerGroups"));
		}

		$back_url = action_url("servergroups", "ShowServerGroups");
		$controller  = UIFactory()->PageController($back_url);
		$controller->addPage($edit_page = new EditPanelConfigPage($controller));
		$edit_page->title = blmsg("TRANS_EDIT_PANEL_OPTIONS");
		$edit_page->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$edit_page->parent_url = $back_url;
		$controller->setValue("configGroupID", $config_group_id);
		$controller->setValue("parentConfigGroupID", $parent_config_group_id);
		$controller->setValue("serverGroupID", $server_group_id);
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
