<?php

require_once("UI/Page.php");

class SSLPluginSelectModulePage extends UIPage
{
	public function createControls()
	{
		require_once("UI/PageController.php");
		require_once("ModuleTool.php");

		$this->subtitle = blmsg("TRANS_SSL_PLUGIN_CHOOSE_MODULE");
		$this->subtitle_description = blmsg("TRANS_SSL_PLUGIN_CHOOSE_MODULE_DESCRIPTION");
		$this->subtitle_icon_id = "bid-cert-add";

		$input_module = UIFactory()->InputDropDown(
			"ssl_plugin_module",
			blmsg("TRANS_SSL_PLUGIN_MODULE")
		);
		$input_module->value = $this->getValue("ssl_plugin_module");

		foreach ( ModuleTool::getModules("sslmodule") as $module_name ) {
			$module_info = ModuleTool::getModuleInfo("sslmodule", $module_name);
			$input_module->addOption(
				$module_name, $module_info["displayName"]
			);
		}

		$input_name = UIFactory()->InputString(
			"ssl_plugin_name",
			blmsg("TRANS_SSL_PLUGIN_NAME"),
			true,
			array($this, "checkSSLPluginName")
		);
		$input_name->value = $this->getValue("ssl_plugin_name");

		$input_panel = UIFactory()->InputPanel(blmsg("TRANS_SSL_PLUGIN_MODULE_CHOOSING"));
		$input_panel->addInputControl($input_module);
		$input_panel->addInputControl($input_name);

		$this->addInputPanel($input_panel);
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function checkSSLPluginName($name, &$message)
	{
		if ( Factory()->SSLPluginList(array("ssl_plugin_name" => $name)) ) {
			$message = blmsg("TRANS_SSL_PLUGIN_ALREADY_EXISTS", array("PLUGIN_NAME" => $name));
			return false;
		}
		return true;
	}
}

class SSLPluginModuleParamsPage extends UIPage
{
	public function createControls()
	{
		$this->subtitle = blmsg("TRANS_SSL_PLUGIN_EDIT_CONFIG");
		$this->subtitle_description = blmsg("TRANS_SSL_PLUGIN_EDIT_CONFIG_DESCRIPTION");
		$this->subtitle_icon_id = "bid-cert-add";
		$this->form = $this->getDefaultForm();

		// Plugin Configuration
		require_once("ModuleTool.php");
		$this->plugin = ModuleTool::getModuleInstance("sslmodule", $this->getValue("ssl_plugin_module"), array());
		$this->plugin->addConfigParams($this->form);
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function processInput()
	{
		$plugin_info = Factory()->SSLPlugin();
		$plugin_info->initFromArray($this->getValues());

		foreach ($this->plugin->collectConfigParams($this->form) as $param_name => $param_value)
			$plugin_info->SSLPluginParams[$param_name]->value = $param_value;

		require_once("object_managers/SSLPluginManager.php");
		SSLPluginManager::update($plugin_info);
		return true;
	}

	private $plugin;
	private $form;
}


class AddSSLPlugin extends MB_Action
{
	public function execute()
	{
		parent::execute();
		$plugins_list_url = action_url("modules", "ShowSSLPlugins");

		$controller  = UIFactory()->WizardController($plugins_list_url);

		$page1 = new SSLPluginSelectModulePage($controller);
		$page2 = new SSLPluginModuleParamsPage($controller);

		$page2->title = $page1->title = blmsg("TRANS_ADD_NEW_SSL_PLUGIN");
		$page2->parent_title = $page1->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$page2->parent_url = $page1->parent_url = $plugins_list_url;

		$controller->addPage($page1);
		$controller->addPage($page2);
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
