<?php

require_once("UI/Page.php");

class SSLPluginEditPage extends UIPage
{
	public function __construct($controller)
	{
		parent::__construct($controller);

		require_once("ModuleTool.php");
		$this->plugin_info = Factory()->SSLPlugin(bgpc("id"));
		$this->plugin = ModuleTool::getModuleInstance(
			"sslmodule",
			$this->plugin_info->ssl_plugin_module,
			array("pluginID" => $this->plugin_info->ssl_plugin_id)
		);
		$this->form = $this->getDefaultForm();
	}

	public function createControls()
	{

		$this->title = blmsg("TRANS_SSL_PLUGIN_EDIT");
		$this->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$this->parent_url = action_url("modules", "ShowSSLPlugins");

		$this->form->addInputPanel(
			$general_panel = UIFactory()->InputPanel(blmsg("TRANS_SSL_PLUGIN_INFORMATION"))
		);

		// Module Information
		require_once("ModuleTool.php");
		$input_module = UIFactory()->InputLabel("ssl_plugin_module", blmsg("TRANS_SSL_PLUGIN_MODULE"));
		$module_info = ModuleTool::getModuleInfo("sslmodule", $this->plugin_info->ssl_plugin_module);
		$input_module->value = $module_info["displayName"];
		$general_panel->addInputControl($input_module);

		// Plugin Name
		$input_plugin_name = UIFactory()->InputString(
			"ssl_plugin_name",
			blmsg("TRANS_SSL_PLUGIN_NAME"),
			true,
			array($this, "checkSSLPluginName", $this->plugin_info->ssl_plugin_id)
		);
		$input_plugin_name->value = $this->getValue("ssl_plugin_name", $this->plugin_info->ssl_plugin_name);
		$general_panel->addInputControl($input_plugin_name);

		// Plugin Configuration
		$this->plugin->addConfigParams($this->form);
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function processReload()
	{
		foreach ($this->plugin->collectConfigParams($this->form) as $param_name => $param_value)
			$this->plugin_info->SSLPluginParams[$param_name]->value = $param_value;

		require_once("object_managers/SSLPluginManager.php");
		SSLPluginManager::update($this->plugin_info);

		redirect(action_url("modules", "EditSSLPluginConfig", array("id" => $this->plugin_info->ssl_plugin_id)));
	}

	public function processInput()
	{
		$this->plugin_info->initFromArray($this->getValues());

		foreach ($this->plugin->collectConfigParams($this->form) as $param_name => $param_value)
			$this->plugin_info->SSLPluginParams[$param_name]->value = $param_value;

		require_once("object_managers/SSLPluginManager.php");
		SSLPluginManager::update($this->plugin_info);

		getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SSL_MODULE_CONFIG_SAVED_OK_MSG");
		redirect(action_url("modules", "EditSSLPluginConfig", array("id" => $this->plugin_info->ssl_plugin_id)));
	}

	public function checkSSLPluginName($name, & $message, $ssl_plugin_id)
	{
		foreach (Factory()->SSLPluginList(array("ssl_plugin_name" => $name)) as $plugin_info)
		{
			if ($ssl_plugin_id != $plugin_info->ssl_plugin_id)
			{
				$message = blmsg("TRANS_SSL_PLUGIN_ALREADY_EXISTS", array("PLUGIN_NAME" => $name));
				return false;
			}
		}
		return true;
	}

	private $plugin;
	private $plugin_info;
	private $form;
}

class EditSSLPluginConfig extends MB_Action
{
	public function execute()
	{
		parent::execute();
		if ( is_null($ssl_plugin_id = bgpc("id")) ) {
             throw new Exception("Missing parameter 'id'");
		}
		$controller  = UIFactory()->PageController(action_url("modules", "ShowSSLPlugins"), blmsg("TRANS_SSL_MODULE_CONFIG_SAVE"), blmsg("TRANS_SSL_MODULE_CONFIG_GO_BACK"));
		$controller->addPage(new SSLPluginEditPage($controller));
		$controller->setValue("ssl_plugin_id", $ssl_plugin_id);
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
