<?php
require_once mb_path("/lib-action/action.php");

class EditOrderForm extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$orderFormID = bgpc("orderFormID");
		if ( is_null($orderFormID) ) {
			throw new Exception("Missing parameter 'orderFormID'");
		}

		$getOptions = array(
			"getOrderFormParams" => 1,
			"getOrderFormItems" => 1,
			"getCouponData" => 1,
			"orderFormID" => $orderFormID
		);
		$query = $this->getMBAPIQuery("GetOrderForms", "", $getOptions, 1, 1, 1, 1);
		require_once mb_path("/lib-mbapi/mbapi.php");
		$ofdata = dispatchMBAPI($query, 1);
		if ( 0 !== $this->addMBAPIErrors($ofdata) || $this->getNumErrors() ) {
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		$smartyVars = array();
		$this->mergeCountryData();
		$smartyVars["countryList"] = $this->smartyVars["countryList"];

		$this->mergeLanguageData();
		$smartyVars["languageList"] = $this->smartyVars["languageList"];

		require_once("object_managers/PaymentGatewayManager.php");
		$smartyVars["currencyList"] = PaymentGatewayManager::getAcceptedCurrencies();

		require_once("object_managers/OrderFormManager.php");
		$smartyVars["widgetTemplateList"] = OrderFormManager::getWidgetTemplates();
		$smartyVars["defaultWidgetTemplateDesc"] = blmsg(
			"TRANS_STORE_DEFAULT_WIDGET_TEMPLATE_DESC",
			array("URL" => $this->orderForm->getUrl())
		);

		$smartyVars["defaultStoreDesc"] = blmsg(
			"TRANS_STORE_DEFAULT_DESC",
			array("DEFAULT_URL" => $this->orderForm->getUrl())
		);

		require_once("objects/PaymentGateway.php");
		$primary_gateways = Factory()->PaymentGatewayList(array("payment_gateway_type" => array(PaymentGateway::TYPE_PRIMARY, PaymentGateway::TYPE_HSPC_CREDIT_CARD)));
		require_once("DataObject.php");
		$primary_gateway_list = DataObject::getMapOf($primary_gateways, "id", "payment_gateway_name");
		if ( false === asort($primary_gateway_list) ) {
        	    global $php_errormsg;
		    throw new Exception("Failed to sort primary gateways: {$php_errormsg}");
		}
		$smartyVars["primaryGatewayList"] = $primary_gateway_list;

		$tax_zones = Factory()->TaxZoneList(array("tax_zone_active" => 1));
		$tax_zone_options = array("" =>  blmsg("TRANS_SHOW_NET_PRICE"));
		foreach ($tax_zones as $tax_zone){
			$percent = BillingLocale::getPercentNumber(floatval($tax_zone->tax_zone_amount));
			$tax_zone_options[$tax_zone->tax_zone_id] = blmsg("TRANS_ISO_3166_".$tax_zone->countries_iso_2)." ({$percent})";
		}
		$smartyVars["taxZoneList"] = $tax_zone_options;

		$hspc_bt_gateways = Factory()->PaymentGatewayList(array("payment_gateway_type" => PaymentGateway::TYPE_HSPC_BANK_TRANSFER));
		$hspc_bt_gateway_list = DataObject::getMapOf($hspc_bt_gateways, "id", "payment_gateway_name");
		if ( false === asort($hspc_bt_gateway_list) ) {
        	    global $php_errormsg;
		    throw new Exception("Failed to sort HSPC Bank Transfer gateways: {$php_errormsg}");
		}
		$smartyVars["hspcBankTransferGatewayList"] = $hspc_bt_gateway_list;

		$order_form = Factory()->OrderForm($orderFormID);

		foreach ($order_form->ThirdPartyGatewaysList() as $gateway)
		{
			$third_party_gateways_in_form[] = $gateway->id;
		}
		$third_party_gateways = DataObject::getMapOf(
			Factory()->PaymentGatewayList(array("payment_gateway_type" => array(PaymentGateway::TYPE_THIRD_PARTY, PaymentGateway::TYPE_HSPC_THIRD_PARTY))),
			"id", "payment_gateway_name"
		);
		if ( false === asort($third_party_gateways) ) {
        	    global $php_errormsg;
		    throw new Exception("Failed to sort third party gateways: {$php_errormsg}");
		}
		$smartyVars["thirdPartyGatewaysInForm"] = $third_party_gateways_in_form;
		$smartyVars["thirdPartyGateways"] = $third_party_gateways;

		$query = $this->getMBAPIQuery("GetCoupons", "", array());
		$coupondata = dispatchMBAPI($query, 1);
		dbg($coupondata);
		if (isset($coupondata["mbapi"][0]["results"][0]["coupons"]) && isset($coupondata["mbapi"][0]["results"][0]["coupons"][0]["coupon"]) && is_array($coupondata["mbapi"][0]["results"][0]["coupons"][0]["coupon"])){
			foreach ($coupondata["mbapi"][0]["results"][0]["coupons"][0]["coupon"] as $coup){
				$allcouponIDs[] = $coup["couponID"][0];
				$smartyVars["coupons"][$coup["couponID"][0]] = $coup;
			}
		}

		$query = $this->getMBAPIQuery("GetCompanies", "", array());
		$companydata = dispatchMBAPI($query, 1);
		$allcomps = $companydata["mbapi"][0]["results"][0]["companies"][0]["company"];

		$getOptions = array(
			"productActive" => 1,
			"getNonAddons" => 1,
		);
		$query = $this->getMBAPIQuery("GetProducts", "", $getOptions);
		$productdata = dispatchMBAPI($query, 1);
		//Get all productIDs
		if (is_array($productdata["mbapi"][0]["results"][0]["products"][0]["product"])){
			foreach ($productdata["mbapi"][0]["results"][0]["products"][0]["product"] as $prod){
				$allproductIDs[] = $prod["productID"][0];
				$smartyVars["products"][$prod["productID"][0]] = $prod;
			}
		}

		// set orderForm smarty var (only one group returned)
		$smartyVars["orderForm"] =& $ofdata["mbapi"][0]["results"][0]["orderForms"][0]["orderForm"][0];

		// set productsInForm smarty var & correct offset if only 1 orderFormItem is returned
		// this will be an array of all orderFormItem data where the orderFormItem's are assigned to this group
		$smartyVars["productsInForm"] = array();
		if(is_array($smartyVars["orderForm"]["orderFormItems"][0]["orderFormItem"])) {
			$smartyVars["orderFormItemsInForm"] = $smartyVars["orderForm"]["orderFormItems"][0]["orderFormItem"];
			if(count($smartyVars["orderForm"]["orderFormItems"][0]["orderFormItem"])==1) {
				$smartyVars["orderFormItemsInForm"] = array($smartyVars["orderForm"]["orderFormItems"][0]["orderFormItem"][0]);
			}
		}

		// assign all orderFormItemIDs in the group to $productIDsInForm for later use
		$productIDsInForm = array();
		if(is_array($smartyVars["orderFormItemsInForm"])){
			foreach($smartyVars["orderFormItemsInForm"] as $orderFormItem) {
				if(is_array($orderFormItem["productID"]) && count($orderFormItem["productID"])) {
					$smartyVars["productsInForm"][$orderFormItem["productID"][0]] = $smartyVars["products"][$orderFormItem["productID"][0]]["productName"][0]." - ". getTypeName("products", $smartyVars["products"][$orderFormItem["productID"][0]]["productType"][0]);
				}
			}
		}


		// assign all orderFormItemIDs NOT in the group to $orderFormItemIDsNotInForm for later use
		$productIDsNotInForm = array();
		if(is_array($productdata["mbapi"][0]["results"][0]["products"][0]["product"])){
			foreach($productdata["mbapi"][0]["results"][0]["products"][0]["product"] as $product) {
				if (!in_array($product["productID"][0],array_flip($smartyVars["productsInForm"]))) {
					$smartyVars["productsNotInForm"][$product["productID"][0]] = $smartyVars["products"][$product["productID"][0]]["productName"][0]." - ". getTypeName("products", $smartyVars["products"][$product["productID"][0]]["productType"][0]);
				}
			}
		}

		$orderFormParamsTemp = $smartyVars["orderForm"]["orderFormParams"][0]["orderFormParam"];
		//Check for an array in order_form_param_value as that is how we store checkbox values
		$i = 0;
		if(is_array($orderFormParamsTemp)){
			foreach ($orderFormParamsTemp as $paramcheck){

				switch ($paramcheck["orderFormParamName"][0])
				{
					case "payment_type_check":
					case "payment_type_invoice":
					case "payment_type_draft":
						$smartyVars["manualPaymentMethods"][] = $paramcheck;
						break;
					default:
						$smartyVars["orderFormParams"][] = $paramcheck;

						if ($paramcheck["orderFormParamInputType"][0] == "c") {
							$smartyVars["orderFormParams"][$i]["orderFormParamValue"][0] = unserialize($paramcheck["orderFormParamValue"][0]);
						}
						$i++;
				}
			}
		}

		$smartyVars["orderForm"]["coupons"][0]["coupon"] = protectArray($smartyVars["orderForm"]["coupons"][0]["coupon"]);
		$smartyVars = collapseArrayElements($smartyVars);

		if(is_array($smartyVars["orderForm"]["orderFormValues"]["orderFormValue"])){
			foreach ($smartyVars["orderForm"]["orderFormValues"]["orderFormValue"] as $value) {
				$smartyVars["orderFormValues"][$value["orderFormParamID"]][$value["orderFormValueName"]] = (defined($value["orderFormValueTransKey"])) ? constant($value["orderFormValueTransKey"]) : $value["orderFormValueTransKey"];
			}
		}

		if(is_array($allcomps)){
			foreach ($allcomps as $comp) {
				$smartyVars["companies"][$comp["companyID"][0]] = $comp["companyName"][0];
			}
		}

		if(is_array($smartyVars["orderForm"]["coupons"]["coupon"])) {
			$coupons = $smartyVars["orderForm"]["coupons"]["coupon"];
		}

		if(is_array($coupons)){
			foreach($coupons as $coupon){
				$smartyVars["couponsInForm"][$coupon["couponID"]] = $coupon["couponCode"];
			}
		}else{
			$smartyVars["couponsInForm"] = array();
		}

		if(is_array($allcouponIDs)){
			foreach($allcouponIDs as $key=>$id){
				if (!in_array($id, array_flip($smartyVars["couponsInForm"]))) {
					$smartyVars["couponsNotInForm"][$id] = $smartyVars["coupons"][$id]["couponCode"];
				}
			}
		}

		// nuke nested orderFormItems data from orderForm param, this makes the smarty data easier to deal with
		unset($smartyVars["orderForm"]["orderFormItems"]);
		unset($smartyVars["orderForm"]["orderFormParams"]);
		unset($smartyVars["orderForm"]["orderFormValues"]);
		//unset($smartyVars["orderFormItemsInForm"]);

		$this->addTemplate("orderforms/EditOrderForm", $smartyVars, 1);
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}