<?php
require_once("lib-action/action.php");
require_once("UI/Page.php");

class EditOrderFormHeaderAndFooterPage extends UIPage
{
	private $orderFormID;

	public function __construct(UIPageController $controller, $orderFormID)
	{
		parent::__construct($controller);
		$this->orderFormID = $orderFormID;
	}

	public function createControls()
	{
		$orderForm = Factory()->OrderForm($this->orderFormID);

		$this->module_input_controls = array();
		$panels = array(
			array(
				"id" => "orderFormHeader",
				"title" => blmsg("TRANS_ONLINE_STORE_HEADER"),
				"value" => $orderForm->header,
				"desc" => blmsg("TRANS_ONLINE_STORE_HEADER_DESC")
			),
			array(
				"id" => "orderFormFooter",
				"title" => blmsg("TRANS_ONLINE_STORE_FOOTER"),
				"value" => $orderForm->footer,
				"desc" => blmsg("TRANS_ONLINE_STORE_FOOTER_DESC")
			)
		);
		foreach($panels as $panel) {
			$inputPanel = UIFactory()->InputPanel($panel["title"]);

			$inputControl = UIFactory()->InputLabel($panel["id"]."_desc", "");
			$inputControl->value = $panel["desc"];
			$inputPanel->addInputControl($inputControl);

			$inputControl = UIFactory()->InputWYSIWYG($panel["id"], "", false, null, "", array("elements" => $panel["id"]));
			$inputControl->value = $panel["value"];
			$this->module_input_controls[] = $inputControl;

			$inputPanel->addInputControl($inputControl);
			$this->addInputPanel($inputPanel);
		}
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function processInput()
	{
		$orderForm = Factory()->OrderForm($this->orderFormID);
		$orderForm->header = bgpc("orderFormHeader");
		$orderForm->footer = bgpc("orderFormFooter");
		$orderForm->OrderFormParams->update();
		return true;
	}

	protected function getSubmitButtons()
	{
		return array(
			UIFactory()->SubmitButton("ok", blmsg("TRANS_SAVE"), UIPageController::CMD_OK),
			UIFactory()->SubmitButton("apply", blmsg("TRANS_APPLY"), UIPageController::CMD_APPLY),
			UIFactory()->SubmitButton("cancel", blmsg("TRANS_CANCEL"), UIPageController::CMD_CANCEL)
		);
	}
}

class EditOrderFormHeaderAndFooter extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$orderFormID = bgpc("orderFormID");
		if ( is_null($orderFormID) ) {
			throw new Exception("Missing parameter 'orderFormID'");
		}

		$orderFormPageURL = action_url("orderforms", "ShowOrderFormDetails", array("orderFormID" => $orderFormID));
		$controller = UIFactory()->PageController($orderFormPageURL);
		$controller->addPage($page = new EditOrderFormHeaderAndFooterPage($controller, $orderFormID));

		$orderFormName = Factory()->OrderForm($orderFormID)->order_form_name;
		$page->title = blmsg("TRANS_ONLINE_STORE_EDIT_HEADER_AND_FOOTER_PAGE_TITLE", array("store_name" => $orderFormName));
		$page->parent_title = $orderFormName;
		$page->parent_url = $orderFormPageURL;

		$controller->processRequest();
		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}