<?php
require_once mb_path("/lib-action/action.php");

class EditOrderFormResponse extends MB_Action
{
	public $params = array(
		"orderFormID",
		"orderFormName",
		"orderFormCode",
		"currencyID",
		"paymentGatewayID",
		"hspcBankTransferGatewayID",
		"companyID",
		"languageID",
		"countriesID",
		"orderFormVerifyVat",
		"orderFormDisableCharging",
		"defaultWidgetTemplate",
		"isDefault",
		"requestTaxVatID",
		"defaultTaxZoneID",
		"enableIndividualEnterpreneurType",
		"useTermsUrl",
	);

	public function execute()
	{
		parent::execute();

		$updateOrderFormParams = array();
		$_REQUEST["requestTaxVatID"] = billing_get_param("requestTaxVatID", $_REQUEST, 0);
		$_REQUEST["enableIndividualEnterpreneurType"] = billing_get_param("enableIndividualEnterpreneurType", $_REQUEST,  0);
		$_REQUEST["orderFormVerifyVat"] = billing_get_param("orderFormVerifyVat", $_REQUEST, 0);
		$_REQUEST["orderFormDisableCharging"] = billing_get_param("orderFormDisableCharging", $_REQUEST, 0);
		$_REQUEST["paymentGatewayID"] = billing_get_param("paymentGatewayID", $_REQUEST, "");
		$_REQUEST["defaultTaxZoneID"] = billing_get_param("defaultTaxZoneID", $_REQUEST, "");
		$_REQUEST["hspcBankTransferGatewayID"] = billing_get_param("hspcBankTransferGatewayID", $_REQUEST, "");

		$orderFormParams = bgpc("orderFormParams");
		foreach ($orderFormParams as $orderFormParam) {
			// find the orderFormID for these params

			$orderFormParamID = ((int) $orderFormParam["orderFormParamID"]);
			$orderFormParamValue = (!is_array($orderFormParam["orderFormParamValue"])) ?
				$orderFormParam["orderFormParamValue"] : serialize($orderFormParam["orderFormParamValue"]);

			$updateOrderFormParams["orderFormParam"][] = array(
				"orderFormParamID" => $orderFormParamID,
				"orderFormParamValue" => $orderFormParamValue,
			);
		}

		if ( 0 == count($updateOrderFormParams) ) {
			$this->addError(blmsg("TRANS_ACTIONERROR"), "No order form params given.");
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}
		if("1" == bgpc("useTermsUrl") && "" == $orderFormParams['terms_url']['orderFormParamValue'])
		{
			$this->addError(blmsg("TRANS_ACTIONERROR"), blmsg("TRANS_ORDERFORM_TERMS_URL_REQUIRED_MSG"));
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		$getOptions = array(
			"getOrderFormItems" => 1,
			"getCouponData" => 1,
			"orderFormID" => bgpc("orderFormID")
		);
		require_once("lib-mbapi/mbapi.php");
		$getquery = getMBAPIQuery("GetOrderForms", "", $getOptions);
		$getdata = dispatchMBAPI($getquery, 1);
		$ofdata =& $getdata["mbapi"][0]["results"][0]["orderForms"][0]["orderForm"][0];

		$ofitemsdata =& $ofdata["orderFormItems"][0]["orderFormItem"];
		$ofcoupondata =& $ofdata["coupons"][0]["coupon"];

		$allproductIDs = $alladdonIDs = $allcouponIDs = $ofProductIDs = $ofProductAddonIDs = $addCouponIDs =
			$updateOrderFormItems = $removeItems = $addCoupons = $removeCoupons = $addProductAddons	= array();


		//Get the order form inputs assigned to variables
		if(is_array($_REQUEST["addProductIDs"])){
			$ofProductIDs =& $_REQUEST["addProductIDs"];
		}else{
			$ofProductIDs = array();
		}

		if(is_array($_REQUEST["addCouponIDs"])){
			$ofCouponIDs =& $_REQUEST["addCouponIDs"];
		}else{
			$ofCouponIDs = array();
		}

		// Get existing items
		if (is_array($ofitemsdata)){
			foreach($ofitemsdata as $formitem) {
				$allproductIDs[$formitem["orderFormItemID"][0]] = $formitem["productID"][0];
			}
		}


		// Get the product data for all coupons in the system. This is needed so that we don't add coupons that have
		// no products associated with the order form.
		$getOptions = array(
			"getProductData" => 1
		);
		$cquery = getMBAPIQuery("GetCoupons", "", $getOptions);

		$cdata = dispatchMBAPI($cquery, 1);
		$couponData =& $cdata["mbapi"][0]["results"][0]["coupons"][0]["coupon"];

		if(is_array($couponData)){
			foreach ($couponData as $coupprod){
				if(is_array($coupprod["products"][0]["product"])){
					foreach($coupprod["products"][0]["product"] as $relprod){
						$possibleCouponProducts[$coupprod["couponID"][0]][] = $relprod["productID"][0];
					}
				}
			}
		}

		// Get existing coupons that are related to this OF
		if (is_array($ofcoupondata)){
			foreach($ofcoupondata as $coupon) {
				$allcouponIDs[] = $coupon["couponID"][0];
			}
		}


		// Update and remove products
		foreach ($ofProductIDs as $key=>$id){
			$orderFormItem = array(
				"productID" => $id,
				"orderFormItemOrder" => $key,
			);
			if (in_array($id, $allproductIDs)){
				$ofitemids = array_flip($allproductIDs);
				$orderFormItem["orderFormItemID"] = $ofitemids[$id];
			}
			$updateOrderFormItems["orderFormItem"][] = $orderFormItem;
		}

		foreach (array_diff($allproductIDs, $ofProductIDs) as $key=>$id){
			$removeItems["productID"][] = $id;
		}

		// Add and remove coupons - check to see if the coupon to be added relates to any of the products on the form first
		foreach (array_diff($ofCouponIDs, $allcouponIDs) as $key=>$id){
			if(in_array($id, array_keys($possibleCouponProducts)) && count(array_intersect($possibleCouponProducts[$id], $ofProductIDs))){
				$addCoupons["couponID"][] = $id;
			}
		}

		foreach (array_diff($allcouponIDs, $ofCouponIDs) as $key=>$id){
			$removeCoupons["couponID"][] = $id;
		}

		$thirdPartyGatewayIDs = array();
		foreach (bgpc("addThirdPartyGatewayIDs", array()) as $third_party_gateway_id) {
			$thirdPartyGatewayIDs["paymentGatewayID"][] = $third_party_gateway_id;
		}

		// Generate the XML for the SetOrderForms call
		$xmlParams = array (
			"thirdPartyGatewayIDs"  => $thirdPartyGatewayIDs,
			"orderFormParams"  => $updateOrderFormParams,
			"orderFormItems"  => $updateOrderFormItems,
			"removeOrderFormItems"  => $removeItems,
			"couponIDs"  => $addCoupons,
			"removeCouponIDs"  => $removeCoupons,
			"productAddons"  => $addProductAddons,
		);

		$query = $this->getMBAPIQuery("SetOrderForms", "update", $xmlParams);

		$data = dispatchMBAPI($query, 1);
		if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
			if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
				redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"]).
				"&".$_REQUEST["redirectArgs"]);
			} else {
				redirect("orderforms.php", getActionID("ShowOrderForms"));
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
