<?php
require_once("lib-action/action.php");
require_once("UI/Page.php");

class EditOrderFormSettingsPage extends UIPage
{
	public function setVars($orderFormID, $languages, $countries)
	{
		if(is_null($orderFormID)) {
			require_once("object_managers/OrderFormManager.php");
			$this->orderForm = OrderFormManager::create();
		} else {
			$this->orderForm = Factory()->OrderForm($orderFormID);
		}
		$this->orderFormID = $orderFormID;
		$this->orderForm = Factory()->OrderForm($this->orderFormID);
		$this->languages = array();
		foreach($languages as $key => $value) {
			$this->languages[] = array($key, $value);
		}

		$this->countries = array();
		foreach($countries as $key => $value) {
			$this->countries[] = array($key, $value);
		}
	}

	public function createControls()
	{
		require_once("UI/Tabs/UIOrderFormSettingsTabs.php");
		$tabs = new UIOrderFormSettingsTabs();
		$this->options = $tabs->setCurrent(bgpc("options"));
		$this->setScreenTabs($tabs);

		$domainTips = UIFactory()->TipsPanel("TRANS_ORDER_DOMAIN_OPTIONS_TIP", "TRANS_ORDER_DOMAIN_OPTIONS_DESC", "domains" == $this->options);
		$domainTips->id = "panel-domainTips";
		$this->addTipsPanel($domainTips);
		$this->addInputPanelToForm("panel-general", NULL, $this->loadGeneralOptions(), "general" == $this->options);

		$this->addInputPanelToForm("panel-payment", NULL, $this->loadPaymentOptions(), "payment" == $this->options);

		$this->addInputPanelToForm("panel-domains", NULL, $this->loadDomainOptions(), "domains" == $this->options);

		$this->addInputPanelToForm("panel-taxes", NULL, $this->loadTaxesOptions(), "taxes" == $this->options);

		$securityOptions = $this->loadSecurityOptions();
		$this->addInputPanelToForm("panel-security_email_banning", blmsg("TRANS_ORDER_EMAIL_BANNING_PANEL"), $securityOptions["email_banning"], "security" == $this->options);
		$this->addInputPanelToForm("panel-security_ip_banning", blmsg("TRANS_ORDER_IP_BANNING_PANEL"), $securityOptions["ip_banning"], "security" == $this->options);
		$this->addInputPanelToForm("panel-security_customer_credentials", blmsg("TRANS_ORDER_CUSTOMER_CREDENTIALS_PANEL"), $securityOptions["customer_credentials"], "security" == $this->options);

		$this->addInputPanelToForm("panel-license", NULL, $this->loadLicenseOptions(), "license" == $this->options);

		$this->addInputPanelToForm("panel-products", NULL, $this->loadProductsOptions(), "products" == $this->options, true);

		$this->getDefaultForm()->id = "settings_form";

		$optionControl = UIFactory()->InputHidden("options");
		$optionControl->value = $this->options;
		$this->getDefaultForm()->addHiddenControl($optionControl);
        $this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function initControl()
	{
		parent::initControl();
		if(isset($_REQUEST['is_default'])) {
			$this->module_input_controls["panel-general"]["is_default"]->value = 1;
			$this->module_input_controls["panel-general"]["is_default"]->read_only = true;
		}
	}

	public function processInput()
	{
		try {
			$this->saveGeneralOptions();
			$this->savePaymentOptions();
			$this->saveDomainOptions();
			$this->saveTaxesOptions();
			$this->saveSecurityOptions();
			$this->saveLicenseOptions();
			$this->saveProductsOptions();

			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SETTINGS_SAVED_SUCCESSFULLY");
		} catch (ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
		}

		if(!$this->hasWarnings() && !$this->hasErrors()) {
			if(is_null(bgpc("orderFormID")))
			{
				redirect("orderforms.php", getActionID("ShowOrderFormDetails"), array("orderFormID" => $this->orderForm->order_form_id));
			}
			return true;
		}
		return false;

		return !$this->hasWarnings() && !$this->hasErrors();
	}

	protected function getSubmitButtons()
	{
		return array(
			UIFactory()->SubmitButton("ok", blmsg("TRANS_SAVE"), UIPageController::CMD_OK),
			UIFactory()->SubmitButton("cancel", blmsg("TRANS_CANCEL"), UIPageController::CMD_CANCEL)
		);
	}

	public function storeJS(UIJavaScriptStorage $storage)
	{
		parent::storeJS($storage);
		if ($storage->hasJS(get_class()))
			return;

		$js = "
		function toggleLicenseVariants()
		{
			var types = $('settings_form').use_terms_url;
			for(var i = 0; i < types.length; i++) {
				if('fid-use_terms_url_1' == types[i].id) {
					if(types[i].checked) {
						$('row-terms_url').show();
						$('row-terms_text').hide();
					} else {
						$('row-terms_url').hide();
						$('row-terms_text').show();
					}
					break;
				}
			}
		}
		";

		$storage->addJS(get_class(), $js);
	}

	private function addInputPanelToForm($panelID, $title, $controls, $isVisible, $hideTitle = false)
	{
		$title = ("" != $title) ? $title : NULL;
		$inputPanel = UIFactory()->InputPanel($title, $hideTitle, $isVisible);
		$inputPanel->id = $panelID;
		foreach($controls as $id => $control) {
			$description = array(
				"id" => $id,
				"title"	=> $control["title"],
				"desc" => isset($control["desc"]) ? $control["desc"] : "",
				"type" => $control["type"],
				"required" => isset($control["required"]) ? $control["required"] : false,
				"read_only" => isset($control["read_only"]) ? $control["read_only"] : false,
				"disabled" => isset($control["disabled"]) ? $control["disabled"] : false,
				"onclick" => isset($control["onclick"]) ? $control["onclick"] : "",
				"items"	=> isset($control["items"]) ? $control["items"] : array(),
				"checker" => isset($control["checker"]) ? $control["checker"] : NULL,
				"extention" => isset($control["extention"]) ? $control["extention"] : NULL,
				"can_be_disabled" => isset($control["can_be_disabled"]) ? $control["can_be_disabled"] : NULL,
				"left_column_title" => isset($control["left_column_title"]) ? $control["left_column_title"] : NULL,
				"right_column_title" => isset($control["right_column_title"]) ? $control["right_column_title"] : NULL,
			);
			$isControlVisible = (!isset($control['isVisible']) || true === $control['isVisible']);

			$inputControl = UIFactory()->InputConfig($description);
			$inputControl->value = $control["value"];
			$inputPanel->addInputControl($inputControl, $isControlVisible);
			$this->module_input_controls[$panelID][$id] = $inputControl;
		}
		$this->addInputPanel($inputPanel);
	}

	private function loadGeneralOptions()
	{
		$generalOptions = array();
		$generalOptions["order_form_name"] = array(
			"title" => blmsg("TRANS_ORDERFORM_NAME"),
			"desc" => blmsg("TRANS_ORDERFORM_NAME_DESC"),
			"value" => $this->orderForm->order_form_name,
			"type" => "t",
			"required" => true
		);

		$generalOptions["order_form_code"] = array(
			"title" => blmsg("TRANS_ORDERFORM_CODE"),
			"desc" => blmsg("TRANS_ORDERFORM_CODE_DESC"),
			"value" => $this->orderForm->order_form_code,
			"type" => "t",
			"required" => true
		);

		require_once("object_managers/PaymentGatewayManager.php");
		$currenciesList = array();
		foreach(PaymentGatewayManager::getAcceptedCurrencies() as $key => $value) {
			$currenciesList[] = array($key, $value);
		}
		require_once("object_managers/OrderFormManager.php");
		$generalOptions["currency_id"] = array(
			"title" => blmsg("TRANS_CURRENCY"),
			"desc" => blmsg("TRANS_ORDERFORM_CURRENCY_DESC"),
			"value" => $this->orderForm->currency_id,
			"type" => "s",
			"read_only" => OrderFormManager::canChangeCurrency($this->orderForm) ? 0 : 1,
			"items" => $currenciesList
		);

		$generalOptions["language_id"] = array(
			"title" => blmsg("TRANS_LANGUAGE"),
			"desc" => blmsg("TRANS_ORDERFORM_LANGUAGE_DESC"),
			"value" => $this->orderForm->language_id,
			"type" => "s",
			"items" => $this->languages
		);

		$generalOptions["countries_id"] = array(
			"title" => blmsg("TRANS_COUNTRY"),
			"desc" => blmsg("TRANS_ORDERFORM_COUNTRY_DESC"),
			"value" => $this->orderForm->countries_id,
			"type" => "s",
			"items" => $this->countries
		);

		$generalOptions["enable_individual_enterpreneur_type"] = array(
			"title" => blmsg("TRANS_ENABLE_INDIVIDUAL_ENTERPRENEUR_TYPE_OPTION"),
			"desc" => blmsg("TRANS_ENABLE_INDIVIDUAL_ENTERPRENEUR_TYPE_DESC"),
			"value" => $this->orderForm->enable_individual_enterpreneur_type,
			"type" => "b"
		);

		$generalOptions["is_default"] = array(
			"title" => blmsg("TRANS_STORE_DEFAULT"),
			"desc" => blmsg("TRANS_STORE_DEFAULT_DESC", array("DEFAULT_URL" => Config()->system_secureStoreUrl)),
			"value" => $this->orderForm->is_default,
			"read_only" => $this->orderForm->is_default,
			"type" => "b"
		);

		require_once("object_managers/OrderFormManager.php");
		$widgetTemplatesList = array();
		foreach(OrderFormManager::getWidgetTemplates() as $key => $value) {
			$widgetTemplatesList[] = array($key, $value);
		}
		$generalOptions["default_widget_template"] = array(
			"title" => blmsg("TRANS_STORE_DEFAULT_WIDGET_TEMPLATE"),
			"desc" => blmsg("TRANS_STORE_DEFAULT_WIDGET_TEMPLATE_DESC", array("URL" => Config()->system_secureStoreUrl)),
			"value" => $this->orderForm->default_widget_template,
			"type" => "s",
			"items" => $widgetTemplatesList
		);

		$generalOptions["continue_shopping_url"] = array(
			"title" => blmsg("TRANS_ORDERFORM_CONTINUE_SHOPPING_URL"),
			"desc" => is_null($this->orderForm->id) ? blmsg("TRANS_ORDERFORM_CONTINUE_SHOPPING_URL_DISABLED_DESC") : blmsg("TRANS_ORDERFORM_CONTINUE_SHOPPING_URL_DESC"),
			"value" => isset($this->orderForm->OrderFormParams["continue_shopping_url"]) ?
					$this->orderForm->OrderFormParams["continue_shopping_url"]->order_form_param_value :
					OrderFormManager::getDefaultContinueShoppingUrl($this->orderForm->order_form_code),
			"type" => "t",
			"disabled" => is_null($this->orderForm->id)
		);

		return $generalOptions;
	}

	private function loadPaymentOptions()
	{
		// load tax zones
		$taxZonesList = array();
		$taxZonesList[] = array("", blmsg("TRANS_SHOW_NET_PRICE"));
		foreach( Factory()->TaxZoneGroupList(array("tax_zone_group_active" => 1)) as $taxZoneGroup) {
			foreach ($taxZoneGroup->taxZoneList() as $taxZone) {
				$percent = BillingLocale::getPercentNumber(floatval($taxZone->tax_zone_amount));
				$taxZonesList[] = array(
					$taxZone->tax_zone_id,
					blmsg("TRANS_SHOW_GROSS_PRICE", array(
						"tax_group" => $taxZoneGroup->tax_zone_group_name,
						"country" => blmsg("TRANS_ISO_3166_".$taxZone->countries_iso_2),
						"percent" => $percent
					))
				);

			}
		}

		// load primary gateways list
		require_once("objects/PaymentGateway.php");
		$primaryGateways = Factory()->PaymentGatewayList(array("payment_gateway_type" => array(PaymentGateway::TYPE_PRIMARY, PaymentGateway::TYPE_HSPC_CREDIT_CARD)));
		$primaryGateways = DataObject::getMapOf($primaryGateways, "id", "payment_gateway_name");
		if ( false === asort($primaryGateways) ) {
        	    global $php_errormsg;
		    throw new Exception("Failed to sort primary gateways: {$php_errormsg}");
		}
		$primaryGatewaysList = array();
		foreach($primaryGateways as $key => $value) {
			$primaryGatewaysList[] = array($key, $value);
		}

		// load HSPC gateways list
		$hspcBankTransferGateways = Factory()->PaymentGatewayList(array("payment_gateway_type" => PaymentGateway::TYPE_HSPC_BANK_TRANSFER));
		$hspcBankTransferGateways = DataObject::getMapOf($hspcBankTransferGateways, "id", "payment_gateway_name");
		if ( false === asort($hspcBankTransferGateways) ) {
        	    global $php_errormsg;
		    throw new Exception("Failed to sort HSPC Bank Transfer gateways: {$php_errormsg}");
		}
		$hspcBankTransferGatewaysList = array();
		foreach($hspcBankTransferGateways as $key => $value) {
			$hspcBankTransferGatewaysList[] = array($key, $value);
		}

		// load third party gateways list
		$thirdPartyGateways = DataObject::getMapOf(
			Factory()->PaymentGatewayList(array("payment_gateway_type" => array(PaymentGateway::TYPE_THIRD_PARTY, PaymentGateway::TYPE_HSPC_THIRD_PARTY))),
			"id", "payment_gateway_name"
		);
		if ( false === asort($thirdPartyGateways) ) {
        	    global $php_errormsg;
		    throw new Exception("Failed to sort third party gateways: {$php_errormsg}");
		}
		$thirdPartyGatewaysList = array();
		foreach($thirdPartyGateways as $key => $value) {
			$thirdPartyGatewaysList[] = array($key, $value);
		}

		$selectedThirdPartyGateways = array();
		foreach ($this->orderForm->ThirdPartyGatewaysList() as $gateway) {
			$selectedThirdPartyGateways[] = $gateway->id;
		}

		// configure payment options
		$paymentOptions = array();
		$paymentOptions["default_tax_zone_id"] = array(
			"title" => blmsg("TRANS_INCLUDE_DEFAULT_TAX_ZONE_OPTION"),
			"desc" => blmsg("TRANS_INCLUDE_DEFAULT_TAX_ZONE_OPTION_DESC"),
			"value" => $this->orderForm->default_tax_zone_id,
			"type" => "s",
			"items" => $taxZonesList
		);
		$paymentOptions["order_form_disable_charging"] = array(
			"title" => blmsg("TRANS_DISABLE_CHARGING"),
			"desc" => blmsg("TRANS_DISABLE_CHARGING_DESC"),
			"value" => $this->orderForm->order_form_disable_charging,
			"type" => "b"
		);
		if(0 != count($primaryGatewaysList)) {
			$paymentOptions["payment_gateway_id"] = array(
				"title" => blmsg("TRANS_PRIMARY_PAYMENT_GATEWAY"),
				"desc" => blmsg("TRANS_PRIMARY_GATEWAY_DESCRIPTION"),
				"value" => $this->orderForm->payment_gateway_id,
				"type" => "s",
				"items" => $primaryGatewaysList,
				"can_be_disabled" => true
			);
		} else {
			$paymentOptions["payment_gateway_id"] = array(
				"title" => blmsg("TRANS_PRIMARY_PAYMENT_GATEWAY"),
				"value" => blmsg("TRANS_ONLINE_STORE_NO_PRIMARY_GATEWAY_MSG"),
				"type" => "d"
			);
		}
		if(0 != count($hspcBankTransferGatewaysList)) {
			$paymentOptions["hspc_bank_transfer_gateway_id"] = array(
				"title" => blmsg("TRANS_HSPC_BANK_TRANSFER_GATEWAY"),
				"desc" => blmsg("TRANS_HSPC_BANK_TRANSFER_GATEWAY_DESCRIPTION"),
				"value" => $this->orderForm->hspc_bank_transfer_gateway_id,
				"type" => "s",
				"items" => $hspcBankTransferGatewaysList,
				"can_be_disabled" => true
			);
		} else {
			$paymentOptions["hspc_bank_transfer_gateway_id"] = array(
				"title" => blmsg("TRANS_HSPC_BANK_TRANSFER_GATEWAY"),
				"value" => blmsg("TRANS_ONLINE_STORE_NO_BANK_TRANSFER_PLUGIN_MSG"),
				"type" => "d"
			);
		}
		if(0 != count($thirdPartyGatewaysList)) {
			$paymentOptions["third_party_gateway_ids"] = array(
				"title" => blmsg("TRANS_THIRD_PARTY_GATEWAYS"),
				"desc" => blmsg("TRANS_THIRD_PARTY_GATEWAY_DESCRIPTION"),
				"value" => $selectedThirdPartyGateways,
				"type" => "bl",
				"items" => $thirdPartyGatewaysList
			);
		} else {
			$paymentOptions["third_party_gateway_ids"] = array(
				"title" => blmsg("TRANS_THIRD_PARTY_GATEWAYS"),
				"value" => blmsg("TRANS_ONLINE_STORE_NO_THIRD_PARTY_GATEWAY_MSG"),
				"type" => "d"
			);
		}

		$additionalPaymentOptions = array(
			"payment_type_check" => array("TRANS_ORDER_ENABLE_CHECK", "TRANS_ORDER_ENABLE_CHECK_DESC"),
			"payment_type_invoice" => array("TRANS_ORDER_ENABLE_INVOICE", "TRANS_ORDER_ENABLE_INVOICE_DESC"),
			"payment_type_draft" => array("TRANS_ORDER_ENABLE_BANK_DRAFT", "TRANS_ORDER_ENABLE_BANK_DRAFT_DESC")
		);

		foreach($additionalPaymentOptions as $optionName => $optionParams) {
			$paymentOptions[$optionName] = array(
				"title" => blmsg($optionParams[0]),
				"desc" => blmsg($optionParams[1]),
				"value" => $this->orderForm->OrderFormParams[$optionName]->order_form_param_value,
				"type" => "b"
			);
		}

		return $paymentOptions;
	}

	private function loadDomainOptions()
	{
		$selectedOptions = unserialize($this->orderForm->OrderFormParams["dom_options"]->order_form_param_value);
		$domainOptions = array();
		require_once("object_managers/OrderFormManager.php");
		foreach(OrderFormManager::getDomainOptions() as $optionName => $optionTitle) {
			$domainOptions[$optionName] = array(
				"title" => blmsg($optionTitle),
				"value" => in_array($optionName, $selectedOptions) ? 1 : 0,
				"type" => "b"
			);
		}

		return $domainOptions;
	}

	private function loadTaxesOptions()
	{
		$taxesOptions = array();
		$taxesOptions["request_tax_vat_id"] = array(
			"title" => blmsg("TRANS_REQUEST_TAX_VAT_ID_OPTION"),
			"desc" => blmsg("TRANS_REQUEST_TAX_VAT_ID_DESC"),
			"value" => $this->orderForm->request_tax_vat_id,
			"type" => "b"
		);
		$taxesOptions["order_form_verify_vat"] = array(
			"title" => blmsg("TRANS_VERIFY_VAT_NUMBERS"),
			"desc" => blmsg("TRANS_VERIFY_VAT_NUMBERS_DESC"),
			"value" => $this->orderForm->order_form_verify_vat,
			"type" => "b"
		);

		return $taxesOptions;
	}

	private function loadSecurityOptions()
	{
		$securityOptions = array(
		    "email_banning" => array(),
		    "ip_banning" => array(),
		    "customer_credentials" => array()
		);

		$panels = array(
			"email_banning" => array(
				"email_banning" => array("TRANS_ORDER_EMAIL_BANNING", "TRANS_ORDER_EMAIL_BANNING_DESC", "b", 0),
				"email_banning_text" => array("TRANS_ORDER_EMAIL_BANNING_TEXT", "TRANS_ORDER_EMAIL_BANNING_TEXT_DESC", "a", 0)
			),
			"ip_banning" => array(
				"ip_banning_text" => array("TRANS_ORDER_IP_BANNING_TEXT", "TRANS_ORDER_IP_BANNING_TEXT_DESC", "a", 0)
			),
			"customer_credentials" => array(
				"username_min_length" => array("TRANS_ORDERFORM_MIN_USERNAME", "TRANS_ORDERFORM_MIN_USERNAME_DESC", "t", 1),
				"username_max_length" => array("TRANS_ORDERFORM_MAX_USERNAME", "TRANS_ORDERFORM_MAX_USERNAME_DESC", "t", 1),
				"password_min_length" => array("TRANS_ORDERFORM_MIN_PASSWORD", "TRANS_ORDERFORM_MIN_PASSWORD_DESC", "t", 1),
				"password_max_length" => array("TRANS_ORDERFORM_MAX_PASSWORD", "TRANS_ORDERFORM_MAX_PASSWORD_DESC", "t", 1)
			)
		);

		$orderFormParams = $this->orderForm->OrderFormParams;
		foreach($panels as $panelName => $panelOptions) {
			foreach($panelOptions as $optionName => $optionParams) {
				$optionDescription = array(
					"title" => blmsg($optionParams[0]),
					"desc" => blmsg($optionParams[1]),
					"value" => $orderFormParams[$optionName]->order_form_param_value,
					"type" => $optionParams[2],
					"required" => $optionParams[3]
				);
				$securityOptions[$panelName] = array_merge(
					$securityOptions[$panelName],
					array($optionName => $optionDescription)
				);
			}
		}

		return $securityOptions;
	}

	private function loadLicenseOptions()
	{
		require_once("object_managers/OrderFormManager.php");
		$licenseOptions = array();
		$licenseOptions["use_terms_url"] = array(
			"title" => blmsg("TRANS_ORDER_FORM_TERMS_TEXT"),
			"value" => $this->orderForm->use_terms_url,
			"type" => "r",
			"onclick" => "toggleLicenseVariants()",
			"items" => array(
				array("1", blmsg("TRANS_ORDERFORM_TERMS_URL_FORWARD_CUSTOMERS")),
				array("0", blmsg("TRANS_ORDERFORM_TERMS_URL_NO"))
			)
		);

		$licenseOptions["terms_url"] = array(
			"title" => blmsg("TRANS_ORDERFORM_TERMS_URL_YES"),
			"value" => $this->orderForm->OrderFormParams["terms_url"]->order_form_param_value,
			"type" => "t",
			"isVisible" => isset($_REQUEST["use_terms_url"]) ? (1 == bgpc("use_terms_url")) : (1 == $this->orderForm->use_terms_url)
		);

		$licenseOptions["terms_text"] = array(
			"title" => blmsg("TRANS_EDIT_ORDER_FORM_TERMS_TEXT_OPTION"),
			"value" => OrderFormManager::getTermsText($this->orderForm),
			"type" => "w",
			"extention" => array(
				"elements" => "terms_text",
			),
			"isVisible" => isset($_REQUEST["use_terms_url"]) ? (0 == bgpc("use_terms_url")) : (0 == $this->orderForm->use_terms_url)
		);

		return $licenseOptions;
	}

	private function loadProductsOptions()
	{
		$productsOptions = array();

		$availableProducts = array();
		foreach(Factory()->ProductList(array("product_active" => 1, "product_is_addon" => 0)) as $product) {
			$availableProducts[] = array($product->product_id, $product->product_name." - ".blmsg(Product::typeName($product->product_type)));
		}

		$selectedProducts = array();
		foreach($this->orderForm->ProductList() as $product) {
			$selectedProducts[] = $product->product_id;
		}

		$productsOptions["products"] = array(
			"title" => blmsg("TRANS_ASSIGNED_PRODUCTS"),
			"desc" => blmsg("TRANS_ORDER_FORM_PRODUCT_SETTINGS_DESC"),
			"value" => $selectedProducts,
			"type" => "dsl",
			"items" => $availableProducts,
			"left_column_title" => blmsg("TRANS_ASSIGNED_PRODUCTS"),
			"right_column_title" => blmsg("TRANS_UNASSIGNED_PRODUCTS"),
		);

		return $productsOptions;
	}

	private function saveGeneralOptions()
	{
		foreach ($this->module_input_controls["panel-general"] as $control) {
			$name = $control->name;
			switch($name) {
				case "continue_shopping_url" :
					if("" == $control->value || $control->value == OrderFormManager::getDefaultContinueShoppingUrl($this->orderForm->order_form_code)) {
						unset($this->orderForm->OrderFormParams["continue_shopping_url"]);
					} else {
						$this->orderForm->OrderFormParams["continue_shopping_url"]->order_form_param_value = $control->value;
					}
					break;
				default:
					$this->orderForm->$name = $control->value;
			}
		}
		require_once("object_managers/OrderFormManager.php");
		OrderFormManager::update($this->orderForm);
	}

	private function savePaymentOptions()
	{
		foreach ($this->module_input_controls["panel-payment"] as $control) {
			$name = $control->name;
			switch($name) {
				case "payment_gateway_id" :
				case "hspc_bank_transfer_gateway_id" :
					if(0 == bgpc("enable".$name)) {
						$this->orderForm->$name = NULL;
					} else {
						$this->orderForm->$name = $control->value;
					}
					break;
				case "third_party_gateway_ids" :
					$this->orderForm->thirdPartyGatewayIDs = is_array($control->value) ? $control->value : array();
					break;
				case "default_tax_zone_id" :
					$this->orderForm->default_tax_zone_id = ((int)$control->value) ? $control->value : NULL;
					break;
				default :
					if(in_array($name, array("payment_type_check", "payment_type_invoice", "payment_type_draft"))) {
						$this->orderForm->OrderFormParams[$name]->order_form_param_value = $control->value;
					} else {
						$this->orderForm->$name = $control->value;
					}
			}
        	}

		//One payment method must be enabled
		if (!$this->orderForm->isAvailablePaymentMethods()) {
			throw new ProductException(ProductException::E_OBJECT_HAS_INVALID_ATTRS, blmsg("TRANS_ORDERFORM_PAYMENT_METHODS_ERROR"));
		}

		require_once("object_managers/OrderFormManager.php");
		OrderFormManager::update($this->orderForm);
	}

	private function saveDomainOptions()
	{
		$selectedOptions = array();
		require_once("object_managers/OrderFormManager.php");
		foreach(OrderFormManager::getDomainOptions() as $optionName => $optionTitle) {
			if(isset($_REQUEST[$optionName])) {
				$selectedOptions[] = $optionName;
			}
		}

		$this->orderForm->OrderFormParams["dom_options"]->order_form_param_value = serialize($selectedOptions);
		OrderFormManager::update($this->orderForm);
	}

	private function saveTaxesOptions()
	{
		foreach ($this->module_input_controls["panel-taxes"] as $control) {
			$name = $control->name;
			$this->orderForm->$name = $control->value;
		}
		require_once("object_managers/OrderFormManager.php");
		OrderFormManager::update($this->orderForm);
	}

	private function saveSecurityOptions()
	{
		$panels = array("panel-security_email_banning", "panel-security_ip_banning", "panel-security_customer_credentials");
		foreach($panels as $panelID) {
			foreach ($this->module_input_controls[$panelID] as $control) {
				$this->orderForm->OrderFormParams[$control->name]->order_form_param_value = $control->value;
			}
		}
		OrderFormManager::update($this->orderForm);
	}

	private function saveLicenseOptions()
	{
		$this->orderForm->use_terms_url = bgpc("use_terms_url");
		if($this->orderForm->use_terms_url) {
			$termsURL = bgpc("terms_url");
			if("" == $termsURL) {
				$this->addErrorMessage(UIFactory()->ErrorMessage(blmsg("TRANS_ORDERFORM_TERMS_URL_REQUIRED_MSG")));
				return;
			}
			$this->orderForm->OrderFormParams["terms_url"]->order_form_param_value = $termsURL;
		} else {
			$this->orderForm->terms_text = bgpc("terms_text");
		}

		require_once("object_managers/OrderFormManager.php");
		OrderFormManager::update($this->orderForm);
	}

	private function saveProductsOptions()
	{
		$products = bgpc("products");
		require_once("object_managers/OrderFormManager.php");
		OrderFormManager::updatePlans($this->orderForm, $products ? $products : array());
	}

	private $orderForm;
	private $languages;
	private $countries;
	private $options;
}

class EditOrderFormSettings extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$orderFormID = bgpc("orderFormID");

		$parent_url = is_null($orderFormID) ? action_url("orderforms", "ShowOrderForms") :
			action_url("orderforms", "ShowOrderFormDetails", array("orderFormID" => $orderFormID));

		$controller  = UIFactory()->PageController($parent_url);
		$controller->addPage($page = new EditOrderFormSettingsPage($controller));

		$this->mergeLanguageData();
		$this->mergeCountryData();
		$page->setVars($orderFormID, $this->smartyVars["languageList"], $this->smartyVars["countryList"]);

		$page->title = blmsg("TRANS_ONLINE_STORE_SETTINGS");
		$page->parent_title = is_null($orderFormID) ? blmsg("TRANS_ONLINE_STORES") : Factory()->OrderForm($orderFormID)->order_form_name;
		$page->parent_url = $parent_url;

		$controller->processRequest();
		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}