<?php
require_once mb_path("/lib-action/action.php");

class ShowOrderFormDetails extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$orderFormID = bgpc("orderFormID");
		if ( is_null($orderFormID) ) {
			throw new Exception("Missing parameter 'orderFormID'");
		}

		$orderForm = Factory()->OrderForm($orderFormID);

		$this->smartyVars["orderForm"] = array();
		$this->smartyVars["orderForm"]["orderFormID"] = $orderFormID;
		$this->smartyVars["orderForm"]["orderFormName"] = $orderForm->order_form_name;
		$this->smartyVars["orderForm"]["orderFormDisableCharging"] = $orderForm->order_form_disable_charging;
		$this->smartyVars["orderForm"]["requestTaxVatID"] = $orderForm->request_tax_vat_id;
		$this->smartyVars["orderForm"]["enableIndividualEnterpreneurType"] = $orderForm->enable_individual_enterpreneur_type;
		$this->smartyVars["orderForm"]["orderFormVerifyVat"] = $orderForm->order_form_verify_vat;

		if(is_null($orderForm->default_tax_zone_id)) {
			$this->smartyVars["orderForm"]["defaultTaxZone"] = blmsg("TRANS_SHOW_NET_PRICE");
		} else {
			$taxZone = Factory()->TaxZone($orderForm->default_tax_zone_id);
			$this->smartyVars["orderForm"]["defaultTaxZone"] = blmsg("TRANS_SHOW_GROSS_PRICE", array(
                "tax_group" => $taxZone->getTaxZoneGroup()->tax_zone_group_name,
				"country" => blmsg("TRANS_ISO_3166_".$taxZone->countries_iso_2),
				"percent" => BillingLocale::getPercentNumber(floatval($taxZone->tax_zone_amount))
			));
		}

		$this->smartyVars["orderForm"]["primaryPaymentGateway"] = (!is_null($orderForm->payment_gateway_id)) ? $orderForm->PrimaryGateway->payment_gateway_name : "-";
		$this->smartyVars["orderForm"]["hspcBankTransferGateway"] = (!is_null($orderForm->hspc_bank_transfer_gateway_id)) ? $orderForm->HSPCBankTransferGateway->payment_gateway_name : "-";

		$this->smartyVars["orderForm"]["thirdPartyGatewaysList"] = array();
		if(0 != count($orderForm->thirdPartyGatewayIDs)) {
			foreach($orderForm->thirdPartyGatewayIDs as $gatewayID) {
				$this->smartyVars["orderForm"]["thirdPartyGatewaysList"][] = Factory()->PaymentGateway($gatewayID)->payment_gateway_name;
			}
		}

		$additionalPaymentOptions = array(
			"payment_type_check" => "TRANS_ORDER_ENABLE_CHECK",
			"payment_type_invoice" => "TRANS_ORDER_ENABLE_INVOICE",
			"payment_type_draft" => "TRANS_ORDER_ENABLE_BANK_DRAFT"
		);

		$this->smartyVars["manualPaymentMethods"] = array();
		foreach($additionalPaymentOptions as $optionName => $optionTitle) {
			$this->smartyVars["manualPaymentMethods"][] = array(
				"title" => blmsg($optionTitle),
				"value" => $orderForm->OrderFormParams[$optionName]->order_form_param_value
			);
		}

		$this->smartyVars["domainOptions"] = array();
		$domainOptionParamsSelected = unserialize($orderForm->OrderFormParams["dom_options"]->order_form_param_value);
		
		require_once("object_managers/OrderFormManager.php");
		foreach(OrderFormManager::getDomainOptions() as $optionName => $optionTitle) {
			$this->smartyVars["domainOptions"][] = array(
				"title" => blmsg($optionTitle),
				"value" => in_array($optionName, $domainOptionParamsSelected)
			);
		}

		$this->smartyVars["emailBanningParamValue"] = $orderForm->OrderFormParams["email_banning"]->order_form_param_value;
		
		$query = $this->getMBAPIQuery("GetWebsiteWidgets", "", array("orderFormID" => $orderFormID));
		$data = dispatchMBAPI($query, 1);
		$this->smartyVars['widgets'] = grabSimpleMBAPIResults($data);

		$query = $this->getMBAPIQuery("GetCoupons", "", array("getOrderFormData" => 1, "getProductData" => 1));
		$data = dispatchMBAPI($query, 1);
		$coupons = grabSimpleMBAPIResults($data);

		$this->smartyVars["coupons"] = array();
		foreach($coupons as $coupon) {
			if("1" != $coupon["couponActive"] || !isset($coupon["orderForms"])) {
				continue;
			}

			foreach($coupon["orderForms"][0]["orderForm"] as $orderFormData) {
				if($orderFormData["orderFormID"] == $orderFormID) {
					$products = $coupon["products"][0];
					$coupon["products"] = array();
					foreach($products["product"] as $product) {
						$product = Factory()->Product($product["productID"]);
						$productOrderForms = $product->OrderFormList();
						foreach($productOrderForms as $productOrderForm) {
							if($productOrderForm->order_form_id == $orderFormID) {
								$coupon["products"][] = array(
								    "productID" => $product->product_id,
								    "productName" => $product->product_name
								);
							}
						}
					}
					$this->smartyVars["coupons"][] = $coupon;
					break;
				}
			}
		}

		$this->smartyVars['backURL'] = action_url("orderforms", "ShowOrderFormDetails", array("orderFormID" => $orderFormID));
		$this->smartyVars['websiteWidgetPreviewURL'] = Config()->system_secureStoreUrl;

		$this->smartyVars['storeButtons'] = array(
			"storeSettings" => array(
				"id" => "storeSettings",
				"title" => blmsg("TRANS_ONLINE_STORE_SETTINGS"),
				"href" => action_url("orderforms", "EditOrderFormSettings", array("orderFormID" => $orderFormID))
			),
			"goToStore" => array(
				"id" => "goToStore",
				"title" => blmsg("TRANS_ONLINE_STORES_GO_TO"),
				"href" => Config()->system_secureStoreUrl."{$orderForm->order_form_code}.html",
				"target" => "_blank"
			),
			"removeStore" => array(
				"id" => "removeStore",
				"title" => blmsg("TRANS_REMOVE"),
				"href" => action_url("orderforms", "RemoveOrderForm", array("orderFormID" => $orderFormID)),
			)
		);

		$this->smartyVars['storeDesignButtons'] = array(
			"addWidget" => array(
				"id" => "addWidget",
				"title" => blmsg("TRANS_ADD_WEBSITE_WIDGET"),
				"href" => action_url("websitewidgets", "AddWebsiteWidget", array("orderFormID" => $orderFormID, "backURL" => $smartyVars['backURL']))
			),
			"removeWidget" => array(
				"id" => "removeWidget",
				"title" => blmsg("TRANS_REMOVE"),
				"href" => "",
				"onclick" => "if(0 == getSelectedWidgetsCount()) { alert('".blmsg('TRANS_LIST_NO_SELECTED_ITEMS')."'); } else { $('widgets_form').submit(); return false; }"
			)
		);

		$this->smartyVars['tabs'] = array(
			"design" => array(
				"url" => "javascript: setTab('design');",
				"title" => blmsg("TRANS_ONLINE_STORE_DESIGN"),
				"current" => true
			),
			"coupons" => array(
				"url" => "javascript: setTab('coupons');",
				"title" => blmsg("TRANS_COUPONS"),
				"current" => false
			)
		);
		
		$this->smartyVars['standardStoreURL'] = Config()->system_standardStoreUrl;

		$this->addTemplate("orderforms/ShowOrderFormDetails");
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}