<?php

require_once("lib-action/action.php");
require_once("UI/Page.php");

class PaymentGatewaySelectModulePage extends UIPage
{
	public function createControls()
	{
		$showTestingGateways = UIFactory()->InputHidden("showTestingGateways");
		$showTestingGateways->value = $this->getValue("showTestingGateways", 0);
		$this->getDefaultForm()->addHiddenControl($showTestingGateways);
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;

		$inputPanel = $this->addInputPanel(UIFactory()->InputPanel());

		$this->primaryGatewayModule = UIFactory()->InputRadio("gatewayModule", blmsg("TRANS_PAYMENT_GATEWAY_MODULE_PRIMARY"));
		$this->primaryGatewayModule->multicolumn = true;
		$this->primaryGatewayModule->column_width = "256px";
		$this->primaryGatewayModule->optionsDescription = blmsg("TRANS_PRIMARY_GATEWAY_DESCRIPTION");
		$inputPanel->addInputControl($this->primaryGatewayModule);

		$this->thirdPartyGatewayModule = UIFactory()->InputRadio("gatewayModule", blmsg("TRANS_PAYMENT_GATEWAY_MODULE_THIRD_PARTY"));
		$this->thirdPartyGatewayModule->multicolumn = true;
		$this->thirdPartyGatewayModule->column_width = "256px";
		$this->thirdPartyGatewayModule->optionsDescription = blmsg("TRANS_THIRD_PARTY_GATEWAY_DESCRIPTION");
		$inputPanel->addInputControl($this->thirdPartyGatewayModule);

		if ($this->isHSPCAvailable()) {
			$this->bankTransferModule = UIFactory()->InputRadio("gatewayModule", blmsg("TRANS_PAYMENT_GATEWAY_MODULE_BANK_TRANSFER"));
			$this->bankTransferModule->multicolumn = true;
			$this->bankTransferModule->column_width = "256px";
			$this->bankTransferModule->optionsDescription = blmsg("TRANS_HSPC_BANK_TRANSFER_GATEWAY_DESCRIPTION");
			$inputPanel->addInputControl($this->bankTransferModule);
		}
	}

	public function initControl()
	{
		$showTestingGateways = $this->getValue("showTestingGateways", 0);
		$this->description = blmsg("TRANS_PAYMENT_GATEWAY_ADD_DESC", array(
			"SHOW_TESTING_LINK" => "<a href='#' id='showTestingGatewaysLink' onclick='showTestingGateways();'" . ($showTestingGateways ? "style='display: none;'" : "") . ">" . blmsg("TRANS_PAYMENT_GATEWAY_ADD_SHOW_TESTING_LINK") . "</a>",
			"HIDE_TESTING_LINK" => "<a href='#' id='hideTestingGatewaysLink' onclick='hideTestingGateways();'" . ($showTestingGateways ? "" : "style='display: none;'") . ">" . blmsg("TRANS_PAYMENT_GATEWAY_ADD_HIDE_TESTING_LINK") . "</a>",
		));

		require_once("objects/PaymentGateway.php");
		$moduleStatuses = PaymentGateway::statuses();
		require_once("object_managers/PaymentGatewayManager.php");
		$primaryGatewayOptions = array();
		foreach (PaymentGatewayManager::getModules(PaymentGateway::TYPE_PRIMARY) as $moduleName) {
			$moduleInfo = PaymentGatewayManager::getModuleInfo($moduleName);
			$moduleTitle = $moduleInfo["displayName"];
			$value = PaymentGateway::TYPE_PRIMARY . "%" . $moduleName;
			if (PaymentGateway::STATUS_STABLE != $moduleInfo["status"]) {
				$moduleTitle .= " (" . blmsg($moduleStatuses[$moduleInfo["status"]]) . ")";
				$this->testingGateways[] = "{$this->primaryGatewayModule->name}_{$value}";
			}
			$primaryGatewayOptions[] = array(
				"value" => $value,
				"title" => $moduleTitle,
				"hint" => isset($moduleInfo["countryCodes"]) ? $this->getCountries($moduleInfo["countryCodes"]) : "",
				"isVisible" => $showTestingGateways || PaymentGateway::STATUS_STABLE == $moduleInfo["status"],
			);
		}
		if ($this->isHSPCAvailable()) {
			foreach (PaymentGatewayManager::getHSPCModules(PaymentGateway::TYPE_HSPC_CREDIT_CARD) as $moduleInfo) {
				$primaryGatewayOptions[] = array(
					"value" => PaymentGateway::TYPE_HSPC_CREDIT_CARD . "%" . $moduleInfo["name"],
					"title" => $moduleInfo["name"],
					"hint" => isset($moduleInfo["countryCodes"]) ? $this->getCountries($moduleInfo["countryCodes"]) : "",
					"isVisible" => true,
				);
			}
		}
		usort($primaryGatewayOptions, array($this, "compareTitle"));
		foreach ($primaryGatewayOptions as $moduleInfo) {
			$this->primaryGatewayModule->addOption($moduleInfo["value"], $moduleInfo["title"], $moduleInfo["hint"], $moduleInfo["isVisible"]);
		}
		$thirdPartyGatewayOptions = array();
		foreach (PaymentGatewayManager::getModules(PaymentGateway::TYPE_THIRD_PARTY) as $moduleName) {
			$moduleInfo = PaymentGatewayManager::getModuleInfo($moduleName);
			$moduleTitle = $moduleInfo["displayName"];
			$value = PaymentGateway::TYPE_THIRD_PARTY . "%" . $moduleName;
			if (PaymentGateway::STATUS_STABLE != $moduleInfo["status"]) {
				$moduleTitle .= " (" . blmsg($moduleStatuses[$moduleInfo["status"]]) . ")";
				$this->testingGateways[] = "{$this->thirdPartyGatewayModule->name}_{$value}";
			}
			$thirdPartyGatewayOptions[] = array(
				"value" => $value,
				"title" => $moduleTitle,
				"hint" => isset($moduleInfo["countryCodes"]) ? $this->getCountries($moduleInfo["countryCodes"]) : "",
				"isVisible" => $showTestingGateways || PaymentGateway::STATUS_STABLE == $moduleInfo["status"],
			);
		}
		if ($this->isHSPCAvailable()) {
			foreach (PaymentGatewayManager::getHSPCModules(PaymentGateway::TYPE_HSPC_THIRD_PARTY) as $moduleInfo) {
				$thirdPartyGatewayOptions[] = array(
					"value" => PaymentGateway::TYPE_HSPC_THIRD_PARTY . "%" . $moduleInfo["name"],
					"title" => $moduleInfo["name"],
					"hint" => isset($moduleInfo["countryCodes"]) ? $this->getCountries($moduleInfo["countryCodes"]) : "",
					"isVisible" => true,
				);
			}
		}
		usort($thirdPartyGatewayOptions, array($this, "compareTitle"));
		foreach ($thirdPartyGatewayOptions as $moduleInfo) {
			$this->thirdPartyGatewayModule->addOption($moduleInfo["value"], $moduleInfo["title"], $moduleInfo["hint"], $moduleInfo["isVisible"]);
		}
		if ($this->isHSPCAvailable()) {
			$bankTransferOptions = array();
			foreach (PaymentGatewayManager::getHSPCModules(PaymentGateway::TYPE_HSPC_BANK_TRANSFER) as $moduleInfo) {
				$bankTransferOptions[] = array(
					"value" => PaymentGateway::TYPE_HSPC_BANK_TRANSFER . "%" . $moduleInfo["name"],
					"title" => $moduleInfo["name"],
					"hint" => isset($moduleInfo["countryCodes"]) ? $this->getCountries($moduleInfo["countryCodes"]) : "",
					"isVisible" => true,
				);
			}
			usort($bankTransferOptions, array($this, "compareTitle"));
			foreach ($bankTransferOptions as $moduleInfo) {
				$this->bankTransferModule->addOption($moduleInfo["value"], $moduleInfo["title"], $moduleInfo["hint"], $moduleInfo["isVisible"]);
			}
		}

		parent::initControl();
	}

	public function storeJS(UIJavaScriptStorage $storage)
	{
		parent::storeJS($storage);
		if ($storage->hasJS(get_class())) {
			return;
		}
		$js = "
			var testingGateways = " . json_encode($this->testingGateways) . ";

			function showTestingGateways()
			{
				for (var i = 0; i < testingGateways.length; i++) {
					jQuery('#' + testingGateways[i].replace('%', '\\\\%')).css('display', 'inline-block');
				}
				jQuery('#showTestingGatewaysLink').css('display', 'none');
				jQuery('#hideTestingGatewaysLink').css('display', 'inline');
				jQuery('#fid-showTestingGateways').val(1);
			}

			function hideTestingGateways()
			{
				for (var i = 0; i < testingGateways.length; i++) {
					jQuery('#' + testingGateways[i].replace('%', '\\\\%')).css('display', 'none');
				}
				jQuery('#hideTestingGatewaysLink').css('display', 'none');
				jQuery('#showTestingGatewaysLink').css('display', 'inline');
				jQuery('#fid-showTestingGateways').val(0);
			}
		";
		$storage->addJS(get_class(), $js);
	}

	public function processInput()
	{
		if (is_null($this->getValue("gatewayModule"))) {
			$this->addWarningMessage(UIFactory()->WarningMessage(blmsg("TRANS_PAYMENT_GATEWAY_NO_SELECTED_MSG")));
			return false;
		}
		list($paymentGatewayType, $paymentGatewayModule) = explode("%", $this->getValue("gatewayModule"));
		$this->setValue("paymentGatewayType", $paymentGatewayType);
		$this->setValue("paymentGatewayModule", $paymentGatewayModule);
		return true;
	}

	protected function getSubmitButtons()
	{
		require_once("UI/WizardController.php");
		return array(
			UIFactory()->SubmitButton("wizard-next", blmsg("TRANS_SELECT"), UIWizardController::CMD_NEXT),
			UIFactory()->SubmitButton("cancel", blmsg("TRANS_CANCEL"), UIWizardController::CMD_CANCEL),
		);
	}

	private function isHSPCAvailable()
	{
		if (is_null($this->getValue("hspcAvailable"))) {
			require_once("HSPC/Tools.php");
			$status = HSPCTools::getConnectionStatus();
			$this->setValue("hspcAvailable", $status["available"]);
		}
		return $this->getValue("hspcAvailable");
	}

	private function getCountries(array $codes)
	{
		$countries = array();
		foreach ($codes as $code) {
			$countries[] = blmsg("TRANS_ISO_3166_{$code}");
		}
		return implode(", ", $countries);
	}

	private function compareTitle($moduleLeft, $moduleRight)
	{
		return strcasecmp($moduleLeft["title"], $moduleRight["title"]);
	}

	private $primaryGatewayModule;
	private $thirdPartyGatewayModule;
	private $bankTransferModule;
	private $testingGateways = array();
}

class AddPaymentGateway extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$back_url = action_url("paymentgateways", "ShowPaymentGateways");
		$controller  = UIFactory()->WizardController($back_url);
		$controller->addPage($select_module_page = new PaymentGatewaySelectModulePage($controller));
		require_once("UI/Page/UIPaymentGatewayEditPage.php");
		$controller->addPage($module_params_page = new UIPaymentGatewayEditPage($controller));
		$select_module_page->title = $module_params_page->title = blmsg("TRANS_ADD_NEW_PAYMENT_GATEWAY");
		$select_module_page->parent_title = $module_params_page->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$select_module_page->parent_url = $module_params_page->parent_url = $back_url;
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
