<?php
require_once(dirname(__FILE__)."/../../../../lib-action/action.php");
require_once("UI/Page.php");

class RemovePaymentGatewaysPage extends UIPage
{
	public function createControls()
	{
		$this->title = blmsg("TRANS_REMOVE_PAYMENT_GATEWAYS");
		require_once("UI/Panel/UIConfirmationListPanel.php");
		$this->remove_panel = $this->addInputPanel(
			new UIConfirmationListPanel(blmsg("TRANS_REMOVE"), !$this->getValue("operation_complete", false),
			array("confirm" => blmsg("TRANS_CONFIRM_REMOVAL"), "confirm_check" => blmsg("TRANS_CONFIRM_REMOVAL_MESSAGE"))
			)
		);
	}

	public function initControl()
	{
		if ( !$this->getValue("operation_complete", false) ) {
			if ( is_null($payment_gateway_ids = $this->getValue("payment_gateway_ids")) ) {
				throw new Exception("Missing parameter 'payment_gateway_ids'");
			}
			$removal_list = $this->remove_panel->addListControl(
				UIFactory()->List(blmsg("TRANS_PAYMENT_GATEWAYS_REMOVAL_LIST_TITLE"))
			);
			foreach ( $payment_gateway_ids as $payment_gateway_id ) {
				$payment_gateway = Factory()->PaymentGateway($payment_gateway_id);
				$input_label = UIFactory()->InputLabel();
				$input_label->value = $payment_gateway->payment_gateway_name;
				$removal_list->addInputControl($input_label);
			}
		} else {
			$this->remove_panel->title = blmsg("TRANS_REMOVED_GATEWAYS");
			$this->remove_panel->display_confirm_option = false;
			if ( !empty($this->removed_gateways) ) {
				$removal_list = $this->remove_panel->addListControl(
					UIFactory()->List(blmsg("TRANS_PAYMENT_GATEWAYS_SUCCESSFULLY_REMOVAL_LIST_TITLE"))
				);
				foreach ( $this->removed_gateways as $payment_gateway_name ) {
					$input_label = UIFactory()->InputLabel();
					$input_label->value = $payment_gateway_name;
					$removal_list->addInputControl($input_label);
				}
			} else {
				$this->remove_panel->addListControl(
					UIFactory()->List(blmsg("TRANS_PAYMENT_GATEWAYS_EMPTY_REMOVAL_LIST_TITLE"))
				);
			}
		}
		parent::initControl();
	}

	public function processInput()
	{
		if ( $this->getValue("operation_complete", false) ) {
			return true;
		}
		require_once("object_managers/PaymentGatewayManager.php");
		$errors = array();
		foreach ( $this->getValue("payment_gateway_ids") as $payment_gateway_id ) {
			try {
				$payment_gateway = Factory()->PaymentGateway($payment_gateway_id);
				$payment_gateway_name = $payment_gateway->payment_gateway_name;
				PaymentGatewayManager::delete($payment_gateway);
				$this->removed_gateways[] = $payment_gateway_name;
			} catch ( ProductException $e ) {
				$errors[] = blmsg(
					"TRANS_REMOVE_PAYMENT_GATEWAY_ERROR_MESSAGE",
					array("GATEWAY_NAME" => $payment_gateway_name, "ERROR_MESSAGE" => $e->getMessage())
				);
			}
		}
		$this->setValue("operation_complete", true);
		if ( !empty($errors) ) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($errors));
			return false;
		}
		return true;
	}

	protected function getSubmitButtons()
	{
		require_once("UI/PageController.php");
		$submit_buttons[] = UIFactory()->SubmitButton("ok", blmsg("TRANS_OK"), UIPageController::CMD_OK);
		if ( !$this->getValue("operation_complete", false) ) {
			$submit_buttons[] = UIFactory()->SubmitButton("cancel", blmsg("TRANS_CANCEL"), UIPageController::CMD_CANCEL);
		}
		return $submit_buttons;
	}

	private $remove_panel;
	private $removed_gateways = array();
}

class RemovePaymentGateways extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$back_url = action_url("paymentgateways", "ShowPaymentGateways");
		$controller  = UIFactory()->PageController($back_url);
		$remove_page = $controller->addPage(new RemovePaymentGatewaysPage($controller));
		$remove_page->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$remove_page->parent_url = $back_url;
		$controller->setValue("payment_gateway_ids", bgpc("id"));
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
