<?php
require_once("lib-action/action.php");
require_once("lib-tk/include/creditcard/validate.php");
require_once("lib-billing/include/accounting/General.php");
require_once("lib-billing/include/accounting/Payment.php");
require_once("lib-mbapi/include/modules/gateway/gateway.php");

class ChargeVirtualTerminalResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$invoiceID = bgpc("invoiceID");
		$this->smartyVars["formValues"] = $_REQUEST;
		if ( is_null($paymentGatewayID = bgpc("paymentGatewayID")) ) {
			throw new Exception("Missing parameter 'paymentGatewayID'");
		}
		$this->paymentGateway = Factory()->PaymentGateway($paymentGatewayID);
		$this->smartyVars["paymentGateway"] = $this->paymentGateway;
		$this->mergeCountryData(billing_get_user_locale()->country);

		$this->smartyVars["doaccounting"] = bgpc("doaccounting");

		$this->smartyVars["transactionAction"] = array(
			"charge" => TRANS_CHARGE,
			"auth" => TRANS_AUTH_ONLY
		);
		$this->smartyVars["selectedTransactionAction"] = bgpc("transactionAction");

		if (bgpc("applyPaymentTo")=="lineitems"){
			$this->chargeAmount = 0;
			foreach ((array)$_REQUEST["lineItemPayments"] as $xy=>$pymt){
				$this->chargeAmount += ($ta=$this->getNumberFromInput($pymt));
				$this->smartyVars["formValues"]["lineItemPayments"][$xy] = $ta;
			}
		} else {
			$this->chargeAmount = $this->getNumberFromInput($_REQUEST["paymentAmount"]);
			$this->smartyVars["formValues"]["paymentAmount"] = $this->chargeAmount;
		}

		$this->smartyVars["cardTypes"] = $this->paymentGateway->acceptedCardTypes;
		$this->smartyVars["accepteCheck"] = $this->paymentGateway->acceptECheck;

		$this->smartyVars["billingCardExpYear"] = array();
		for ($i=date("Y"); $i<date("Y")+21; $i++){
			$this->smartyVars["billingCardExpYear"][$i] = $i;
		}
		$this->smartyVars["billingCardExpMonth"] = array();
		for ($i=1; $i<13; $i++){
			$this->smartyVars["billingCardExpMonth"][$i] = $i;
		}
		$this->smartyVars["accounttypes"] = getTypes("bankAccount");

		$success = ("" != $invoiceID) ? $this->processInvoicePayment($invoiceID) : $this->processNoInvoicePayment();
		if (1 != $success){
			$this->addTemplate("modules/ChargeVirtualTerminal");
		} else {
			if ( "" != $invoiceID ) {
				redirect("invoices.php?action=".getActionID("ShowInvoiceDetails")."&sid=".bgpc("sid")."&invoiceID=".$invoiceID);
			} else {
				$this->addTemplate("modules/ChargeVirtualTerminalResponse");
			}
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	private function processInvoicePayment($invoiceID)
	{
		global $CONF;
		$invoiceIDs = array();

		$invoiceInfo = correctArray(grabFirstResultFromMBAPIArray(array(
			"mbapi" => array(
				"command" => "GetInvoices",
				"params" => array(
					"invoiceID" => $invoiceID,
					"getInvoiceLineItemsData" => 1,
					"getInvoiceLineItemsPaymentsData" => 1,
					"getTaxZonesData" => 1,
				),
			),
		)));
		$invoiceIDs[] = $invoiceID;
		$this->smartyVars["invoiceInfo"] = $invoiceInfo;
		$this->smartyVars["invoiceInfo"]["amountDue"] = $invoiceInfo["invoiceAmount"]-$this->smartyVars["invoiceInfo"]["invoiceAmountPaid"];
		$currency = Factory()->Currency($invoiceInfo["currencyID"]);
		$this->smartyVars["currencyID"] = $currency->id;

		foreach ((array)$_REQUEST["lineItemPayments"] as $k=>$pm){
			$_REQUEST["lineItemPayments"][$k] = $this->getNumberFromInput($pm);
		}

		require_once("object_managers/CryptManager.php");
		CryptManager::setDefaultKey();
		$cardNum = preg_replace("/[^0-9]/", "", bgpc("billingCardNum"));
		if (!$this->isCardOrCodeValid($cardNum)) {
			return 0;
		}

		loadConfigs("payment");
		$transactionAction = ($_REQUEST["transactionAction"]) ? $_REQUEST["transactionAction"] : Settings()->paymentQueue_transactionAction;

		$clientID = $invoiceInfo["clientID"];
		$clientInfo = oneByID("clients", $clientID, array(
			"mergePrimaryContact" => 1,
		));

		$cardCVV2 = preg_replace("/[^0-9]/", "", bgpc("cardCVV2"));
		require_once("objects/ClientBillingAccount.php");
		///////////// create the account to store the card in...
		$billingAccountID = insertFor("billingAccounts", array(
			"billingAccountType" => ClientBillingAccount::accountTypeByName(bgpc("paymentMethod")),
			"clientContactID" => $clientInfo["clientContactID"],
			"billingCardNameOnCard" =>  bgpc("billingCardNameOnCard"),
			"billingCardType" =>  bgpc("billingCardType"),
			"billingCardNum" => $cardNum,
			"billingCardExpMonth" => bgpc("billingCardExpMonth"),
			"billingCardExpYear" => bgpc("billingCardExpYear"),
			"billingCardIssueNum" => bgpc("billingCardIssueNum"),
			"billingCardStartDate" => bgpc("billingCardStartDate"),
			"billingBankName" => bgpc("billingBankName"),
			"billingBankState" => bgpc("billingBankState"),
			"billingBankType" => bgpc("billingBankType"),
			"billingBankAbaCode" => bgpc("billingBankAbaCode"),
			"billingBankAccountNum" => bgpc("billingBankAccountNum"),
			"billingBankLicenseNum" => bgpc("billingBankLicenseNum"),
			"billingBankLicenseState" => bgpc("billingBankLicenseState"),
			"billingBankDob" => bgpc("billingBankDob"),
		));
		$billingAccount = Factory()->ClientBillingAccount($billingAccountID);

		// create transaction
		$transaction = Factory()->Transaction();
		require_once("objects/Transaction.php");
		$transaction->auth_return = Transaction::STATUS_PROCESSING;
		$transaction->transaction_amount = $this->chargeAmount;
		$transaction->currency_id = $currency->id;
		$transaction->billing_account_id = $billingAccountID;
		$transaction->invoiceIDs = $invoiceIDs;
		require_once("objects/ClientBillingAccount.php");
		switch ($billingAccount->billing_account_type) {
			case ClientBillingAccount::TYPE_CREDIT_CARD:
				$transaction->transaction_payment_method = Transaction::PAYMENT_CCSINGLE;
				$transaction->payment_gateway_id = $this->paymentGateway->id;
				$transaction->transaction_notes = blmsg("TRANS_PAYMENT_TRANSACTION_PROCESSING_NOTE");
				if (!is_null($billingAccountID)) {
					$billingAccount->billing_card_num = $cardNum;
					$billingAccount->cardCVV2 = $cardCVV2;
				}
				break;
			case ClientBillingAccount::TYPE_ECHECK:
				$transaction->transaction_payment_method = Transaction::PAYMENT_ECHECK;
				$transaction->payment_gateway_id = $this->paymentGateway->id;
				$transaction->transaction_notes = blmsg("TRANS_PAYMENT_TRANSACTION_PROCESSING_NOTE");
				break;
			default:
				throw new Exception("Unsuitable billing account type: {$billingAccount->billing_account_type}");
		}

		require_once("object_managers/TransactionManager.php");
		TransactionManager::update($transaction);

		// process transaction
		require_once("objects/PaymentGateway.php");
		switch ($transaction->PaymentGateway->payment_gateway_type) {
			case PaymentGateway::TYPE_PRIMARY:
			case PaymentGateway::TYPE_HSPC_CREDIT_CARD:
				require_once("object_managers/PaymentManager.php");
				("charge" == $transactionAction) ? PaymentManager::charge($transaction) : PaymentManager::auth($transaction);
				break;
			default:
				throw new Exception("Unsuitable payment gateway type: {$transaction->PaymentGateway->payment_gateway_type}");
		}

		$this->smartyVars["transaction"]["transactionID"] = $transaction->id;
		$this->smartyVars["transaction"]["authorizationCode"] = $transaction->auth_code;
		$this->smartyVars["transaction"]["message"] = $transaction->transaction_notes;

		switch ($transaction->auth_return) {
			case Transaction::STATUS_SUCCESSFUL_CHARGE:
				PaymentManager::takePayment($transaction);
				$success = 1;
				break;
			case Transaction::STATUS_PROCESSING:
				$success = 1;
				break;
			case Transaction::STATUS_DECLINED_CHARGE:
				$success = 0;
				$this->smartyVars["errorMessage"] = blmsg("TRANS_PAYMENT_DECLINED");
				break;
			default:
				$success = 0;
				$this->smartyVars["errorMessage"] = blmsg("TRANS_GATEWAY_FAILURE");
				break;
		}
		return $success;
	}

	private function processNoInvoicePayment()
	{
		global $CONF;
		if ( is_null($currencyID = bgpc("currencyID")) ) {
			throw new Exception("Missing parameter 'currencyID'");
		}
		$currency = Factory()->Currency($currencyID);
		$this->smartyVars["currencyID"] = $currency->id;

		foreach ((array)$_REQUEST["lineItemPayments"] as $k=>$pm){
			$_REQUEST["lineItemPayments"][$k] = $this->getNumberFromInput($pm);
		}

		$cardNum = preg_replace("/[^0-9]/", "", bgpc("billingCardNum"));
		if (!$this->isCardOrCodeValid($cardNum)) {
			return 0;
		}

		$this->chargeAmount = $this->getNumberFromInput($_REQUEST["paymentAmount"]);
		$this->smartyVars["formValues"]["paymentAmount"] = $this->chargeAmount;

		$mergeParams = array(
			"contactFirstName",
			"contactLastName",
			"contactAddress1",
			"contactAddress2",
			"contactCity",
			"contactState",
			"contactZip",
			"countriesID",
			"contactPhone1",
			"contactFax",
			"contactEmail",
			"billingCardNameOnCard",
			"billingCardType",
			"billingCardNum",
			"billingCardExpMonth",
			"billingCardExpYear",
			"billingCardIssueNum",
			"billingCardStartDate",
			"billingBankName",
			"billingBankState",
			"billingBankType",
			"billingBankAbaCode",
			"billingBankAccountNum",
			"billingBankLicenseNum",
			"billingBankLicenseState",
			"billingBankDob",
		);

		require_once("object_managers/CryptManager.php");
		CryptManager::setDefaultKey();
		$gatewayParams = array();
		foreach ($mergeParams as $param){
			$gatewayParams[$param] = bgpc($param);
		}
		loadConfigs("payment");
		$transactionAction = ($_REQUEST["transactionAction"]) ? $_REQUEST["transactionAction"] : Settings()->paymentQueue_transactionAction;

		$packageComments = bgpc("paymentComments");
		$chargeCommand = array(
			"mbapi" => array(
				"command" => "ProcessGateway",
				"subCommand" => $transactionAction,
				"params" => array(
					"paymentGatewayID" => $this->paymentGateway->id,
					"currencyID" => $currency->id,
					"paymentDescription" => $packageComments,
					"paymentAmount" => $this->chargeAmount,
					"paymentMethod" => (bgpc("paymentMethod")!="card")?"ECHECK":"CC",
					"cardNum" => bgpc("billingCardNum"),
					"bankAccountNum" => bgpc("billingBankAccountNum"),
					"cardCVV2" => bgpc("cardCVV2"),
					"invoiceName" => "none",
				) + $gatewayParams,
			),
		);

		$chargeResults = dispatchMBAPI($chargeCommand, 1);
		if ($chargeResults["mbapi"][0]["results"] && $chargeResults["mbapi"][0]["results"][0]["transactions"] && $chargeResults["mbapi"][0]["results"][0]["transactions"][0]["transaction"]){
			$chargeResults = correctArray($chargeResults["mbapi"][0]["results"][0]["transactions"][0]["transaction"][0]);
			if ( GATEWAY_SUCCESS == $chargeResults["status"] ) {
				$this->smartyVars["transaction"] = $chargeResults;
				$this->smartyVars["transactionAmount"] = $this->chargeAmount;
				$success = 1;
				if ((int)$_REQUEST["doaccounting"]){ // make income entry
					$entries = array();
					$entry = array();
					$entry["ledgerAccountID"] = getGeneralLedgerAccountID("cash");
					$entry["generalLedgerDescription"] = $packageComments;
					$entry["generalLedgerDebitAmount"] = $this->chargeAmount;
					$entry["generalLedgerCreditAmount"] = 0;
					$entry["invoiceID"] = 0;
					$entry["clientID"] = 0;
					$entries[] = $entry;

					$entry = array();
					$entry["ledgerAccountID"] = getGeneralLedgerAccountID("revenue");
					$entry["generalLedgerDescription"] = $packageComments;
					$entry["generalLedgerDebitAmount"] = 0;
					$entry["generalLedgerCreditAmount"] = $this->chargeAmount;
					$entry["invoiceID"] = 0;
					$entry["clientID"] = 0;
					$entries[] = $entry;

					$gen = new General();
					$gen->setClientID(0);
					$gen->setInvoiceID(0);
					$gen->setDescription(TRANS_CHARGED_CARD);
					$gen->setTotalDebits($this->chargeAmount);
					$gen->setTotalCredits($this->chargeAmount);
					$gen->setCurrencyID($this->smartyVars["currencyID"]);
					$gen->setLedgerEntries($entries);
					$gen->execute();
				}
			} else {
				$success = 0;
				if ( GATEWAY_ERROR == $chargeResults["status"] ) {
					$this->smartyVars["errorMessage"] = blmsg("TRANS_GATEWAY_ERROR");
				} else {
					$this->smartyVars["errorMessage"] = blmsg("TRANS_GATEWAY_RETURNED_ERROR");
				}
				$this->smartyVars["errorMessage"] .= ": {$chargeResults['statusMessage']}";
				if ( isset($chargeResults["message"]) ) {
					$this->smartyVars["errorMessage"] .= " - {$chargeResults['message']}";
				}
			}
		} else {
			$success = 0;
			$this->smartyVars["errorMessage"] = blmsg("TRANS_GATEWAY_ERROR");
		}
		return $success;
	}

	private function isCardOrCodeValid($cardNum)
	{
		$result = true;
		if ($_REQUEST["paymentMethod"]=="card"){
			$c = new CreditCard($cardNum, array());
			if (!$c->isValid()){
				$this->smartyVars["errorMessage"] = TRANS_INVALID_CREDIT_CARD_NUMBER;
				$result = false;
			}
		} else {
			if ($_REQUEST["billingBankAbaCode"]==""){
				$this->smartyVars["errorMessage"] = TRANS_INVALID_BANK_ROUTING_NUMBER;
				$result = false;
			} elseif ($_REQUEST["billingBankAccountNum"]==""){
				$this->smartyVars["errorMessage"] = TRANS_INVALID_BANK_ACCOUNT_NUMBER;
				$result = false;
			}
		}
		return $result;
	}

	private $paymentGateway;
	private $chargeAmount;
}
