<?php
require_once("lib-action/action.php");
require_once("UI/Page.php");

class PaymentQueueSettingsEditPage extends UIPage
{
	public function createControls()
	{
		$controls = $this->getControls();
		foreach($controls as $panel) {
			$inpitPanel = UIFactory()->InputPanel(blmsg($panel["title"]));
			foreach($panel["controls"] as $control) {
				$description = array(
					"id"		=> $control[0],
					"title"		=> $control[1],
					"desc"		=> $control[2],
					"type"		=> $control[3],
					"required"	=> $control[4],
					"items"		=> $control[5],
					"checker"	=> isset($control[6]) ? $control[6] : NULL
				);
				if("batchFileManager_ResponsibleAdmin" == $description["id"]) {
					foreach ( Factory()->AdminList() as $admin ) {
						$description["items"][] = array($admin->id, "{$admin->admin_first_name} {$admin->admin_last_name} ({$admin->admin_username})");
					}
				}

				$inputControl = UIFactory()->InputConfig($description);
				$inputControl->value = Settings()->$description["id"];
				$inpitPanel->addInputControl($inputControl);
				$this->module_input_controls[] = $inputControl;
			}
			$this->addInputPanel($inpitPanel);
		}
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function processInput()
	{
		try {
			foreach ($this->module_input_controls as $control) {
				$name = $control->name;
				Settings()->$name = $control->value;
			}
			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SETTINGS_SAVED_SUCCESSFULLY");
		} catch (ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
		}

		return !$this->hasWarnings() && !$this->hasErrors();
	}

	protected function getSubmitButtons()
	{
		return array(UIFactory()->SubmitButton("ok", blmsg("TRANS_SAVE"), UIPageController::CMD_OK));
	}

	protected function getControls()
	{
		return array(
		    "commonSettings" => array(
			"title" => "TRANS_PAYMENT_QUEUE_COMMON_SETTINGS",
			"controls" => array(
			    array("paymentQueue_generateNew", "TRANS_PAYMENT_QUEUE_NEW", "", "b", 1, array()),
			    array("paymentQueue_generateDeclined", "TRANS_PAYMENT_QUEUE_DECLINED", "", "b", 1, array()),
			    array("paymentQueue_generateError", "TRANS_PAYMENT_QUEUE_ERROR", "", "b", 1, array()),
			    array("paymentQueue_generateDueDateOnly", "TRANS_PAYMENT_QUEUE_DUE_DATE_ONLY", "", "b", 1, array()),
			    array("paymentQueue_clearPaymentQueue", "TRANS_PAYMENT_QUEUE_CLEAR", "TRANS_PAYMENT_QUEUE_CLEAR_DESC", "b", 1, array()),
			    array("paymentQueue_tryLesserAmounts", "TRANS_PAYMENT_QUEUE_TRY_LESSER_AMOUNTS", "TRANS_PAYMENT_QUEUE_TRY_LESSER_AMOUNTS_DESC", "b", 1, array()),
			    array("paymentQueue_enableAdvancedShareEditing", "TRANS_ENABLE_ADVANCED_PAYMENT_SHARE_EDITING", "TRANS_ENABLE_ADVANCED_PAYMENT_SHARE_EDITING_DESC", "b", 1, array()),
			    array("paymentQueue_batchRunOffset", "TRANS_PAYMENT_QUEUE_BATCH_DAYS_BEFORE_DUE", "TRANS_PAYMENT_QUEUE_BATCH_DAYS_BEFORE_DUE_DESC", "t", 1, array(), Billing_Checker::P_UINT_NOT_NULL),
			)
		    ),
		    "creditCardSettings" => array(
			"title" => "TRANS_PAYMENT_QUEUE_CREDIT_CARD_SETTINGS",
			"controls" => array(
			    array("paymentQueue_transactionAction", "TRANS_PAYMENT_QUEUE_ACTION", "TRANS_PAYMENT_QUEUE_ACTION_DESC", "s", 1, array(array("charge", "TRANS_CHARGE"), array("auth", "TRANS_AUTH_ONLY"))),
			)
		    ),
		    "bankTransferSettings" => array(
			"title" => "TRANS_PAYMENT_QUEUE_BANK_TRANSFER_SETTINGS",
			"controls" => array(
			    array("batchFileManager_ProcessDaysCount", "TRANS_BATCH_FILES_PROCESS_DAYS_COUNT", "", "t", 1, array(), Billing_Checker::P_UINT_NOT_NULL),
			    array("batchFileManager_ProcessTransactionsCount", "TRANS_BATCH_FILES_PROCESS_TRANSACTION_COUNT", "", "t", 1, array(), Billing_Checker::P_UINT_NOT_NULL),
			    array("batchFileManager_ResponsibleAdmin", "TRANS_RESPONSIBLE_ADMIN", "", "s", 1, array()),
			)
		    ),
		);
	}
}

class EditPaymentQueueSettings extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$controller  = UIFactory()->PageController();
		$controller->addPage($edit_page = new PaymentQueueSettingsEditPage($controller));

		$edit_page->title = blmsg("TRANS_PAYMENT_QUEUE");
		$edit_page->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$edit_page->parent_url = admin_action_url("configs", "system", "show");

		$controller->processRequest();
		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}