<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("admin/include/lib-action/products/ShowProductDetailsAction.php");
require_once("lib-billing/include/config/internal/emailactions.php");
require_once("lib-tk/toolkit.php");
require_once("lib-action/include/wizard/Wizard.php");

class EditProductAttributes extends MB_ProductAction
{
	public $params = array("productID");

	public function __construct()
	{
		parent::__construct("attributes");
	}

	public function execute()
	{
		parent::execute();

		$productID = getParam("productID");
		if(!$productID) {
			$this->addError(TRANS_ACTIONERROR, "No product ID given.");
		} else {
			$q = new MBQuery("GetProducts", "", array("productID" => $productID, "getProductAttributeData" => 1, "getServerGroupGhostData" => 1));
			$r =& $q->getResult();
			if($r->getSuccess()) {
				$gs =& $r->getSubjectFrom(
					array("mbapi", "results", "products", "product", "ghostServerGroups", "ghostServerGroup"),
					"serverGroupID", "serverGroupName"
				);
				$serverGroups = $gs->getAssociated();
				$s =& $r->getSubjectFrom(
					array("mbapi", "results", "products", "product", "productAttributes", "productAttribute"),
					"productAttributeID", "productAttributeName"
				);
				$ps =& $r->getSubjectFrom(
					array("mbapi", "results", "products", "product"),
					"productID", "productName"
				);
				$pt =& $ps->getTable();
				$product = $pt->getOne();
				$product["productAddonGroupID"] = (int)$product["productAddonGroupID"];

				$this->smartyVars["productVariants"] = arrayFor(":productVariants", array(
					"productID" => $productID,
				));

				$this->setupTypeButtons($product["productType"]);
				$this->smartyVars["productButtons"] = $this->productButtons->getButtons();

				$s->setKey("serverGroupID");
				$attrsTableByServerGroup = $s->getAssociatedSets();
				$genericAttributes = $attrsTableByServerGroup[NULL];
				$attrsByServerGroup = $s->getAssociated();

				$wantedServerGroups = $gs->getKeys();
				$wantedServerGroups[] = 0;
				$serverGroupAttributes = array();
				$multiSelects = array();

				foreach($wantedServerGroups as $serverGroup) {
					$panelAttributes = $this->getAttributesForServerGroupID($serverGroup, $product, false, true);
					// here we test if what we got back from the panel object
					// is a numerically indexed array, meaning we are going to
					// use a custom attributes template provided by the panel,
					// or a keyed array of wizard questions
					if ($panelAttributes && !is_numeric(key($panelAttributes))){
						$panelStep = new WizardStep($serverGroups[$serverGroup]['serverGroupName'], TRANS_ATTRIBUTES);
						$panelStep->setID($serverGroup);

						if(isset($attrsTableByServerGroup[$serverGroup])) {
							$panelAttributeValues = associateArrayRowByID($attrsTableByServerGroup[$serverGroup], "productAttributeName");
						} else {
							$panelAttributeValues = array();
						}

						$reverseValMap = array();
						if (is_array( $panelAttributes ))
						foreach($panelAttributes as $attrName => $attrData) {
							$attr =& $panelAttributes[$attrName];

							if (isset($attr['valueMap'])){
								$reverseValMap[$attrName] = array();
								foreach($attr['valueMap'] as $value => $map) {
									foreach($map as $name => $bool) {
										list($trueVal, $falseVal) = $bool;

										if(isset($panelAttributeValues[$name])) {
											if($panelAttributeValues[$name]["productAttributeValue"] === $trueVal) {
												$reverseValMap[$attrName][$value] = true;
											} else if(
													$panelAttributeValues[$name]["productAttributeValue"] === $falseVal
												&&	isset($reverseValMap[$attrName][$value])
											) {
												unset($reverseValMap[$attrName][$value]);
											}
										}
									}
								}
							}
						}

						$i = 0;
						if (is_array( $panelAttributes ))
						foreach($panelAttributes as $attrName => $attrData) {
							if (isset($panelAttributes[$attrName])){
								$attr =& $panelAttributes[$attrName];
								if(isset($panelAttributeValues[$attrName])) {
									$attrSettings = $panelAttributeValues[$attrName];
								} else {
									$attrSettings = false;
								}

								$panelQuestion =& $attr['question'];
								$panelQuestion->setID(++$i);

								$questionType = $panelQuestion->getType();
								if($questionType === QUESTION_INPUT_MULTISELECT) {
									$multiSelects[$attrName] = true;
								}
								$isMulti = $panelQuestion->hasOptions();

								$userVal = $this->getUserDataFor('question_' . $serverGroup . '_' . $i);
								if($userVal === NULL) {
									if($isMulti) {
										if(isset($attr['valueMap'])) {
											$options = array_keys($reverseValMap[$attrName]);
											$panelQuestion->setSelected($options);
										} else if($attrSettings) {
											$panelQuestion->setSelected($attrSettings["productAttributeValue"]);
										}
									} else {
										if(isset($attr['valueMap'])) {
											$options = array_keys($reverseValMap[$attrName]);
											$panelQuestion->setAnswers(array());
											$panelQuestion->addAnswer($options[0]);
										} else if($attrSettings) {
											$panelQuestion->setAnswers(array());
											$panelQuestion->addAnswer($attrSettings["productAttributeValue"]);
											if ($panelQuestion->type=='b'){
												$panelQuestion->addAnswer($panelAttributeValues[$attrName.'UNIT']["productAttributeValue"]);
											}
										}
									}
								} else {
									if($isMulti) {
										$panelQuestion->setSelected($userVal);
									} else if($attrSettings) {
										$panelQuestion->setAnswers(array());
										if (is_array($userVal)) {
											foreach ($userVal as $answer) {
												$panelQuestion->addAnswer($answer);
											}
										} else {
											$panelQuestion->addAnswer($userVal);
										}
									}

									$errors = $panelQuestion->validateFor($userVal);
									$panelStep->erroredQuestions[$i] = $errors;
									$panelStep->invalidQuestions[$i] = errorMesgsFor($errors);
									if ($errors) {
                                        $this->smartyVars['inputInvalid'] = 1;
									}
								}
								$panelStep->addQuestion($panelQuestion);
							} else {
								dbg("got an attribute that wasn't there...");
								dbg($attrName);
								dbg($panelAttributes);
							}

						}
						$serverGroupAttributes[] = array(
							"type" => "content",
							"data" => $panelStep->display(false, true),
						);
					} elseif ($panelAttributes){
						$attributes = arrayFor(":productAttributes", array(
							"productID" => $productID,
							"serverGroupID" => $serverGroup,
						));
						$keyedAttributes = array();
						foreach ($attributes as $a){
							$keyedAttributes[$a["productAttributeName"]][] = $a;
						}
						$serverGroupAttributes[] = array(
							"type" => "include",
							"file" => $panelAttributes[0],
							"serverGroupID" => $serverGroup,
							"serverGroupName" => $serverGroups[$serverGroup]['serverGroupName'],
							"attributes" => $keyedAttributes,
							"panelVariables" => $panelAttributes[1],
						);
					}
				}

				$javascript = "";
				foreach($multiSelects as $multiSelect => $blah) {
					$javascript .= "selectAllOptions(document.EditProductAttributeResponse.elements['" . $multiSelect . "[]']);";
				}

				$plan = Factory()->Product($productID);
				if (Product::TYPE_HOSTING == $plan->product_type && !$plan->isAddon()) {
					// check ipv4Address and ipv6Address properties for PP plans
					foreach ($plan->ProductAttributesList() as $planAttributes) {
						if (0 == $planAttributes->server_group_id) {
							continue;
						}
						if (false === ($controlPanel = reset(Factory()->ServerGroup($planAttributes->server_group_id)->ControlPanelList()))) {
							continue;
						}
						if (!$controlPanel->isPlesk()) {
							continue;
						}
						if ("none" == $planAttributes["ipv4Address"]->value && "none" == $planAttributes["ipv6Address"]->value) {
							$this->smartyVars["warningMessages"][] = blmsg("TRANS_PANEL_PLESK_IP_ADDRESS_NO_SELECTED_MSG");
						}
					}
				}

				$this->smartyVars['productID'] = $productID;
				$this->smartyVars['product'] = $product;
				$this->smartyVars['javascript'] = $javascript;
				$this->smartyVars['productAttributes'] = $productAttributes;
				$this->smartyVars['genericAttributes'] = $genericAttributes;
				$this->smartyVars['serverGroupAttributes'] = $serverGroupAttributes;
				$this->smartyVars['cycles'] = getCyclesArray();

				$this->addTemplate("widgets/products/EditProductAttributes");
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
