<?php

require_once("lib-tk/shield.php");
require_once("admin/include/lib-action/products/ShowProductDetailsAction.php");
require_once("UI/Form.php");

class EditProductSSLPlugin extends MB_ProductAction
{
	public function __construct()
	{
		parent::__construct("sslplugin");
	}

	public function execute()
	{
		parent::execute();

		if (is_null(bgpc("productID")))
		{
			$this->addError(TRANS_ACTIONERROR, "No product ID given.");
		}
		else
		{
			$this->product_id = bgpc("productID");
			$this->prepareOldSmartyVars();
			$this->prepareNewSmartyVars();
			$this->addTemplate("widgets/products/EditProductSSLPlugin", $this->smartyVars);
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	private function prepareOldSmartyVars()
	{
		global $CONF;

		$q = new MBQuery("GetProducts", "", array("productID" => $this->product_id));
		$r = $q->getResult();
		$subject = $r->getSubjectFrom(
			array("mbapi", "results", "products", "product", 0), "productID", "productName"
		);
		$product = $subject->getCollapsed();
		$this->smartyVars['product'] = $product;
		$types = $CONF["types"]["products"];
		$this->smartyVars['types'] = $types;

		$this->smartyVars["productVariants"] = arrayFor(":productVariants", array(
			"productID" => $this->product_id,
		));

		$this->setupTypeButtons($product["productType"]);
		$this->smartyVars["productButtons"] = $this->productButtons->getButtons();
	}

	private function prepareNewSmartyVars()
	{
		$this->product_attributes = Factory()->ProductAttributes(array($this->product_id, 0));

		if ("save" == bgpc("command") && 0 < (int)bgpc("ssl_plugin_id"))
		{
			$this->plugin_id = bgpc("ssl_plugin_id");
		}
		else
		{
			$this->plugin_id = isset($this->product_attributes["ssl_plugin_id"])
				? $this->product_attributes["ssl_plugin_id"]->value
				: NULL;
		}

		// Plugin Information
		$plugins_info = Factory()->SSLPluginList(array("ssl_plugin_enabled" => 1));

		$general_panel = UIFactory()->InputPanel(blmsg("TRANS_SSL_PLUGIN_INFORMATION"));
		$general_panel->addInputControl($input_plugin = UIFactory()->InputDropDown("ssl_plugin_id", blmsg("TRANS_SSL_PLUGIN")));
		$input_plugin->value = $this->plugin_id;
		$input_plugin->addOption("", "");

		foreach ($plugins_info as $plugin_info)
		{
			$input_plugin->addOption(
				$plugin_info->ssl_plugin_id, $plugin_info->ssl_plugin_name
			);
		}
		$input_plugin->required = true;

		// Input Form
		$form = UIFactory()->Form(action_url("products", "EditProductSSLPlugin", array(
			"productID" => $this->product_id,
		)), "EditProduct");
		$form->addInputPanel($general_panel);

		// Plugin Configuration
		$plugin = NULL;
		if (isset($this->product_attributes["ssl_plugin_id"])
			&& $this->product_attributes["ssl_plugin_id"]->value == $this->plugin_id)
		{
			require_once mb_path("/lib-mbapi/include/modules/sslmodule/sslmodule.php");
			$plugin = SSLModule::getPluginInstance($this->plugin_id, array("productID" => $this->product_id));
			$plugin->addProductParams($form);
		}

		$form->addSubmitButton(UIFactory()->SubmitButton("ok", blmsg("TRANS_SAVE"), "save"));

		$page = UIFactory()->Page();
		$page->addForm($form);

		// Save and get output
		if ("save" == bgpc("command"))
		{
			$page->initFromRequest();
			if ($page->validateInput())
			{
				$this->saveSSLPluginInfo();

				if (! is_null($plugin))
					$this->saveSSLPluginParams($form, $plugin);

				redirect(action_url("products", "EditProductSSLPlugin", array(
					"productID" => $this->product_id,
				)));
			}
		}
		$this->smartyVars["content"] = $page->getHTML();
	}

	private function saveSSLPluginInfo()
	{
		$this->product_attributes["ssl_plugin_id"]->value = $this->plugin_id;
		$this->product_attributes->update();
	}

	private function saveSSLPluginParams(UIForm $form, SSLModule $plugin)
	{
		foreach ($plugin->collectProductParams($form) as $name => $value)
			$this->product_attributes[$name]->value = $value;

		$this->product_attributes->update();
	}

	private $product_id = 0;
	private $plugin_id = 0;
	private $product_attributes = NULL;
}
