<?php

require_once(dirname(__FILE__)."/../../../../lib-tk/shield.php");
require_once("lib-action/genericactions.php");

class ShowProductDetailsAction extends ShowDetailsAction
{
	// holds the menu items for each different type
	public $menuItems;

	function ShowProductDetailsAction($where="details") {
		parent::ShowDetailsAction();

		$this->menuItems = array(
			"advanced" => array(
				"details",
				"orderforms",
				"cycles",
				"servergroups",
				"attributes",
				"pricing",
				"options", // it will be shown only for type = "domain"
				"optionvalues",
				//"coupons",
				//"relations",
				"miscellaneous",
				"children",
				//"emails",
				//"notifications",
				//"usage",
				"ledgergroups",
				"clone",
				"simple",
			),
			"unknown" => array(
				"details",
				//"servergroups",
				"attributes",
				"orderforms",
				"cycles",
				"optionvalues",
				"pricing",
				//"coupons",
				//"relations",
				"miscellaneous",
				"children",
				//"emails",
				//"notifications",
				//"usage",
				"ledgergroups",
				"clone",
				//"advanced", // all items are the same as in advanced menu
			),
			"hosting" => array(
				"details",
				"orderforms",
				"cycles",
				"servergroups",
				"attributes",
				"applications",
				"pricing",
				"optionvalues",
				//"coupons",
				//"relations",
				"miscellaneous",
				//"usage",
				"children",
				//"emails",
				//"notifications",
				"ledgergroups",
				"clone",
				//"advanced", // all items are the same as in advanced menu
			),
			"addon" => array(
				"details",
				"orderforms",
				"cycles",
				"servergroups",
				"attributes",
				"pricing",
				"optionvalues",
				//"coupons",
				//"relations",
				"miscellaneous",
				//"emails",
				"ledgergroups",
				"clone",
				"advanced",
			),
			"domain" => array(
				"details",
				"orderforms",
				"cycles",
				"options",
				"pricing",
				//"coupons",
				//"relations",
				"miscellaneous",
				"children",
				//"emails",
				//"notifications",
				"ledgergroups",
				"clone",
				"advanced",
			),
			"cert" => array(
				"details",
				"orderforms",
				"cycles",
				"attributes",
				"optionvalues",
				"pricing",
				//"coupons",
				//"relations",
				"miscellaneous",
				//"emails",
				//"notifications",
				"ledgergroups",
				"clone",
				"advanced",
			),
			"ssl-cert" => array(
				"details",
				"orderforms",
				"cycles",
				"sslplugin",
				"pricing",
				"optionvalues",
				"miscellaneous",
				"ledgergroups",
				"clone",
			),

		);

		dbg("in constructor for details action");
		$this->setConfigurePlanMessage();
		$this->selectedButton = $where;
		$this->productButtons = new ProductButtons();
	}


	function getProductInfo(){
		$this->productInfo = (!$this->productInfo)?correctArray($this->productButtons->getProductInfo()):$this->productInfo;
	}

	function setupTypeButtons($type){
		dbg("adding type: ".$type);
		$this->productInfo = (!$this->productInfo)?correctArray($this->productButtons->getProductInfo()):$this->productInfo;

		if (! bgpc("advanced", 0)){
			switch ($type){
				case getAPITypeID("products", "hosting"):
					$this->setupHostingButtons();
				break;
				case getAPITypeID("products", "domain"):
					$this->setupDomainButtons();
				break;
				case getAPITypeID("products", "cert"):
					$this->setupCertButtons();
					$this->smartyVars['warningMessages'][] = blmsg('TRANS_CERT_PRODUCT_CYCLE_DESCRIPTION'); 
				break;
				case getAPITypeID("products", "ssl-cert"):
					$this->setupSSLCertButtons();
				break;
				default:
					$this->setupButtons($this->menuItems["unknown"]);
				break;
			}
		} else {
			$menu_items = $this->menuItems["advanced"];
			if(getAPITypeID("products", "domain") != $type)
			{
				$key = array_search("options", $menu_items);
				if($key)
				{
					unset($menu_items[$key]);
				}
			}
			$this->setupButtons($menu_items);
		}
		if ((int)$this->productInfo["productAddonGroupID"]){
			$this->setupAddonOptions();
		} else {
			switch ($type){
				case getAPITypeID("products", "hosting"):
					$this->setupHostingOptions();
				break;
				case getAPITypeID("products", "domain"):
					$this->setupDomainOptions();
				break;
				case getAPITypeID("products", "cert"):
					$this->setupCertOptions();
				case getAPITypeID("products", "ssl-cert"):
					$this->setupSSLCertOptions();
				break;
			}
		}
	}

	function setupHostingButtons(){
		dbg("doing hosting buttons setup");
		$this->setupButtons($this->menuItems["hosting"]);
	}

	function setupDomainButtons(){
		dbg("doing domain buttons setup");
		$this->setupButtons($this->menuItems["domain"]);
	}

	function setupCertButtons(){
		dbg("doing certs buttons setup");
		$this->setupButtons($this->menuItems["cert"]);

		$disableButtons = array();
		dbg("product info for testing buttons...");
		dbg($this->productInfo);
		if (!(int)$this->productInfo["orderForms"][0]["orderFormID"]){
			$disableButtons = array("cycles", "options", "pricing", "ledgergroups", "options", "usage", "servergroups", "attributes", "children", "emails", "relations");
		} elseif (!(int)$this->productInfo["ghostCycles"][0]["cycleID"]){
			$disableButtons = array("options", "pricing", "usage", "ledgergroups", "servergroups", "attributes", "children", "emails", "relations");
		}

		foreach ((array)$disableButtons as $bttn){
			$this->productButtons->setEnabled($bttn, false);
		}
	}

	protected function setupSSLCertButtons()
	{
		dbg("doing SSL certs buttons setup");
		$this->setupButtons($this->menuItems["ssl-cert"]);
	}

	// handle hosting specific options for buttons, like which ones are enabled and whatnot
	function setupHostingOptions()
	{
		$this->productButtons->setName("options", TRANS_OPTIONS);

		$disableButtons = array();
		dbg("product info for testing buttons...");
		dbg($this->productInfo);

		if(!isset($this->productInfo["orderForms"])) {
			$disableButtons = array("cycles", "optionvalues", "pricing", "ledgergroups", "usage", "servergroups", "attributes", "children", "emails", "coupons", "relations");
		} elseif (!isset($this->productInfo["ghostCycles"])) {
			$disableButtons = array("optionvalues", "pricing", "usage", "ledgergroups", "servergroups", "attributes", "children", "emails", "coupons", "relations");
		} elseif (!isset($this->productInfo["productVariants"])) {
			$disableButtons = array("servergroups", "attributes");
		} else {
			$disableButtons = array("attributes");
			$serverGroups = Factory()->Product($this->productInfo['productID'])->ServerGroupList();
			foreach($serverGroups as $serverGroup) {
				$panels = $serverGroup->ControlPanelList();
				if(0 != count($panels))	{
					$disableButtons = array();
					break;
				}
			}
		}

		foreach ($disableButtons as $bttn) {
			$this->productButtons->setEnabled($bttn, false);
		}
	}

	function setupDomainOptions(){
		$this->productButtons->setName("options", TRANS_TLDS);

		$disableButtons = array();
		dbg("product info for testing buttons...");
		dbg($this->productInfo);
		if (!(int)$this->productInfo["orderForms"][0]["orderFormID"]){
			$disableButtons = array("cycles", "options", "pricing", "ledgergroups", "usage", "servergroups", "attributes", "children", "emails", "coupons", "relations");
		} elseif (!(int)$this->productInfo["ghostCycles"][0]["cycleID"]){
			$disableButtons = array("options", "pricing", "usage", "ledgergroups", "servergroups", "attributes", "children", "emails", "coupons", "relations");
		} else {
/*			$foundone = 0;
			foreach ($this->productInfo["TLDs"] as $tld){
				dbg($tld);
				if ((int)$tld["productVariantActive"]==1){
					dbg("found one");
					$foundone = 1;
				}
			}
			if (!(int)$foundone){*/
			if (!count($this->productInfo["productVariants"])){
				$disableButtons = array("pricing", "children", "ledgergroups", "emails", "relations");
			}
		}

		foreach ((array)$disableButtons as $bttn){
			$this->productButtons->setEnabled($bttn, false);
		}

	}

	function setupAddonOptions(){
	}

	function setupCertOptions(){
	}

	protected function setupSSLCertOptions()
	{
		if (0 == (int)$this->productInfo["orderForms"][0]["orderFormID"]
			|| 0 == (int)$this->productInfo["ghostCycles"][0]["cycleID"])
		{
			$this->productButtons->setEnabled("pricing", false);
			$this->productButtons->setEnabled("ledgergroups", false);
		}
	}

	function setupButtons($buttons=array()){

		$this->productButtons->setupButtons($buttons);

		$this->addSSLPluginButton();
		$this->addOrderFormsButton();
		$this->addCyclesButton();
		$this->addOptionValuesButton();
		$this->addOptionsButton();
		$this->addPricingButton();
		$this->addUsageButton();
		$this->addServerGroupsButton();
		$this->addAttributesButton();
		$this->addApplicationsButton();
		$this->addAddonsButton();
		$this->addEmailsButton();
		$this->addNotificationsButton();
		$this->addCouponsButton();
		$this->addMiscellaneousButton();
		$this->addAddonAttributesButton();
		$this->addRelationsButton();
		$this->addLedgerGroupsButton();
		$this->addCloneButton();
		$this->addAdvancedButton();
		$this->addSimpleButton();

		$this->productButtons->setSelected($this->selectedButton);

	}
	
	protected function getLinkUrl($action, $aditionalParameters = array()) {
		$basicParameters = array(
			'productID'	=> (int)bgpc("productID"),
			'sid'		=> bgpc("sid"),
			'advanced'	=> (int)bgpc("advanced")
		);
		$parameters = array_merge($basicParameters, $aditionalParameters);
		return billing_safetyhtml(action_url('products', $action, $parameters));
	}
	
	protected function addSSLPluginButton(){

		$this->productButtons->setName("sslplugin", TRANS_SSL_PLUGIN);
		$this->productButtons->setEnabled("sslplugin");
		$this->productButtons->setIcon("sslplugin", "sslplugin.gif");
		$this->productButtons->setLink("sslplugin", $this->getLinkUrl('EditProductSSLPlugin'));
		$this->productButtons->setWidget("sslplugin", "EditProductSSLPlugin");

	}

	function addOrderFormsButton(){

		$this->productButtons->setName("orderforms", TRANS_ORDER_FORMS);
		$this->productButtons->setEnabled("orderforms");
		$this->productButtons->setIcon("orderforms", "orderform.gif");
		$this->productButtons->setLink("orderforms", $this->getLinkUrl('EditProductOrderForms'));
		$this->productButtons->setWidget("orderforms", "EditProductOrderForms");

	}


	function addOptionValuesButton(){

		$this->productButtons->setName("optionvalues", TRANS_OPTIONS);
		if(0 != $this->productInfo["productVariableID"]) {
			$this->productButtons->setEnabled("optionvalues");
		}
		$this->productButtons->setIcon("optionvalues", "options.gif");
		$this->productButtons->setLink("optionvalues", $this->getLinkUrl('EditAppliedValues'));
		$this->productButtons->setWidget("optionvalues", "EditAppliedValues");

	}


	function addCyclesButton(){

		$this->productButtons->setName("cycles", TRANS_BILLING_CYCLES);
		$this->productButtons->setEnabled("cycles");
		$this->productButtons->setIcon("cycles", "cycles.gif");
		$this->productButtons->setLink("cycles", $this->getLinkUrl('EditProductCycles'));
		$this->productButtons->setWidget("cycles", "EditProductCycles");

	}

	function addOptionsButton(){

		$this->productButtons->setName("options", TRANS_TLDS);
		$this->productButtons->setEnabled("options");
		$this->productButtons->setIcon("options", "options.gif");
		$this->productButtons->setLink("options", $this->getLinkUrl('EditProductTLDs'));
		$this->productButtons->setWidget("options", "EditProductTLDs");

	}

	function addPricingButton(){

		$this->productButtons->setName("pricing", TRANS_PRICES);
		$this->productButtons->setEnabled("pricing");
		$this->productButtons->setIcon("pricing", "pricing.gif");
		if (count($this->smartyVars["productVariants"])==1){
			$this->productButtons->setLink("pricing", $this->getLinkUrl('EditProductVariantPrices', array('productVariantID' => (int)$this->smartyVars["productVariants"][0]["productVariantID"])));
			$this->productButtons->setWidget("pricing", "EditProductVariantPrices");
		} else {
			$this->productButtons->setLink("pricing", $this->getLinkUrl('ShowProductVariants', array('productVariantActive' => 1)));
			$this->productButtons->setWidget("pricing", "ShowProductVariants");
		}

	}

	function addUsageButton(){

		$this->productButtons->setName("usage", TRANS_USAGE_PRICING);
		$this->productButtons->setEnabled("usage");
		$this->productButtons->setIcon("usage", "usage.gif");
		$this->productButtons->setLink("usage", $this->getLinkUrl('EditUsagePricing'));
		$this->productButtons->setWidget("usage", "EditUsagePricing");

	}

	function addServerGroupsButton(){

		$this->productButtons->setName("servergroups", TRANS_SERVER_GROUPS);
		$this->productButtons->setEnabled("servergroups");
		$this->productButtons->setIcon("servergroups", "servergroups.gif");
		$this->productButtons->setLink("servergroups", $this->getLinkUrl('EditProductServerGroups'));
		$this->productButtons->setWidget("servergroups", "EditProductServerGroups");

	}

	function addAttributesButton(){

		$this->productButtons->setName("attributes", TRANS_ATTRIBUTES);
		$this->productButtons->setEnabled("attributes");
		$this->productButtons->setIcon("attributes", "attributes.gif");
		$this->productButtons->setLink("attributes", $this->getLinkUrl('EditProductAttributes'));
		$this->productButtons->setWidget("attributes", "EditProductAttributes");
	}
	
	function addApplicationsButton()
	{
		$this->productButtons->setName("applications", TRANS_APPLICATIONS);
		$this->productButtons->setEnabled("applications");
		$this->productButtons->setIcon("applications", "applications.png");
		$this->productButtons->setLink("applications", $this->getLinkUrl('EditProductApplications'));
		$this->productButtons->setWidget("applications", "EditProductApplications");
	}

	function addAddonAttributesButton(){

		$this->productButtons->setName("addonattributes", TRANS_ATTRIBUTES);
		$this->productButtons->setEnabled("addonattributes");
		$this->productButtons->setIcon("addonattributes", "attributes.gif");
		$this->productButtons->setLink("addonattributes", $this->getLinkUrl('EditProductAddonAttributes'));
		$this->productButtons->setWidget("addonattributes", "EditProductAddonAttributes");
	}

	function addAddonsButton(){

		$this->productButtons->setName("children", TRANS_ADDONS);
		$this->productButtons->setEnabled("children");
		$this->productButtons->setIcon("children", "addons.gif");
		$this->productButtons->setLink("children", $this->getLinkUrl('ShowProductAddons'));
		$this->productButtons->setWidget("children", "ShowProductAddons");

	}

	function addEmailsButton(){

		$this->productButtons->setName("emails", TRANS_EMAILS);
		$this->productButtons->setEnabled("emails");
		$this->productButtons->setIcon("emails", "emails.gif");
		$this->productButtons->setLink("emails", $this->getLinkUrl('EditProductEmails'));
		$this->productButtons->setWidget("emails", "EditProductEmails");

	}

	function addNotificationsButton() {

		$this->productButtons->setName("notifications", TRANS_NOTIFICATIONS);
		$this->productButtons->setEnabled("notifications");
		$this->productButtons->setIcon("notifications", "emails.gif");
		$this->productButtons->setLink("notifications", $this->getLinkUrl('EditProductNotifications'));
		$this->productButtons->setWidget("notifications", "EditProductNotifications");

	}

	function addCouponsButton(){

		$this->productButtons->setName("coupons", TRANS_COUPONS);
		$this->productButtons->setEnabled("coupons");
		$this->productButtons->setIcon("coupons", "coupon.gif");
		$this->productButtons->setLink("coupons", $this->getLinkUrl('EditProductCoupons'));
		$this->productButtons->setWidget("coupons", "EditProductCoupons");

	}

	function addMiscellaneousButton(){

		$this->productButtons->setName("miscellaneous", TRANS_MISCELLANEOUS_OPTIONS);
		$this->productButtons->setEnabled("miscellaneous");
		$this->productButtons->setIcon("miscellaneous", "options.gif");
		$this->productButtons->setLink("miscellaneous", $this->getLinkUrl('EditProductMiscellaneous'));
		$this->productButtons->setWidget("miscellaneous", "EditProductMiscellaneous");

	}

	function addRelationsButton(){

		$this->productButtons->setName("relations", TRANS_UPGRADE_DOWNGRADE);
		$this->productButtons->setEnabled("relations");
		$this->productButtons->setIcon("relations", "cycles.gif");
		$this->productButtons->setLink("relations", $this->getLinkUrl('EditOneProductRelations'));
		$this->productButtons->setWidget("relations", "EditOneProductRelations");

	}

	function addLedgerGroupsButton(){

		$this->productButtons->setName("ledgergroups", TRANS_LEDGER_ENTRIES);
		$this->productButtons->setEnabled("ledgergroups");
		$this->productButtons->setIcon("ledgergroups", "pricing.gif");
		$this->productButtons->setLink("ledgergroups", $this->getLinkUrl('EditLedgerGroups'));
		$this->productButtons->setWidget("ledgergroups", "EditLedgerGroups");

	}

	function addCloneButton(){

		$this->productButtons->setName("clone", TRANS_PLUS_CLONE_PRODUCT);
		$this->productButtons->setEnabled("clone");
		$this->productButtons->setIcon("clone", "clone.gif");
		$this->productButtons->setLink("clone", $this->getLinkUrl('CloneProductResponse'));
		$this->productButtons->setWidget("clone", "CloneProductResponse");

	}

	function addAdvancedButton(){

		$this->productButtons->setName("advanced", TRANS_PLUS_ADVANCED);
		$this->productButtons->setEnabled("advanced");
		$this->productButtons->setIcon("advanced", "");
		$this->productButtons->setLink("advanced", $this->getLinkUrl('ShowProductDetails', array('advanced' => 1)));
		$this->productButtons->setWidget("advanced", "ShowProductDetails");

	}

	function addSimpleButton(){

		$this->productButtons->setName("simple", TRANS_MINUS_SIMPLE);
		$this->productButtons->setEnabled("simple");
		$this->productButtons->setIcon("simple", "");
		$this->productButtons->setLink("simple", $this->getLinkUrl('ShowProductDetails', array('advanced' => 0)));
		$this->productButtons->setWidget("simple", "ShowProductDetails");

	}

}


class ProductButtons
{
	// the button array we are operating on
	public $structure;

	function ProductButtons($advanced = 0){
		if ((int)$advanced){
			$this->setupAdvancedButtons();
		}
	}



	function getProductInfo(){
		return (array)grabFirstResultFromMBAPIArray(
			array(
				"mbapi" => array(
					"command" => "GetProducts",
					"params" => array(
						"productID" => (int)$_REQUEST["productID"],
						"getProductAttributeIDs" => 1,
						"getProductChildIDs" => 1,
						"getProductVariantPriceIDs" => 1,
						"getEmailTemplateIDs" => 1,
						"getEmailActionGroupIDs" => 1,
						"getCouponIDs" => 1,
						"getOrderFormIDs" => 1,
						"getProductVariantIDs" => 1,
						"getTLDIDs" => 1,
						"getCycleGhostIDs" => 1,
						"getServerGroupGhostIDs" => 1
					)
				)
			)
		);
	}


	/**
	 * function to setup the button structure from an array of button ids
	 */
	function setupButtons($buttons = array()){
		foreach ((array)$buttons as $button){
			$this->structure[$button] = array(
				"id"=>$button,
				"disabled"=>1,
				"name"=>$button,
				"link"=>"",
				"class"=>"productHeaderButtonOff",
				"icon"=>"data_edit.gif",
				"selected"=>0,
			);
		}

		$this->setButtonOption("details", "disabled", 0);
		$this->setButtonOption("details", "name", TRANS_DETAILS);
		$this->setButtonOption("details", "widget", "EditDetails");
		$this->setButtonOption("details", "link", billing_safetyhtml(action_url('products', 'ShowProductDetails', array('productID' => (int)bgpc("productID"), 'sid' => bgpc("sid"), 'advanced' => (int)bgpc("advanced")))));
		$this->setButtonOption("details", "class", "productHeaderButton");
		$this->setButtonOption("details", "icon", "details.gif");

	}

	/**
	 * function to activate a button
	 */
	function setEnabled($button, $enable=true){
		if ((int)$_REQUEST["advanced"]){
			$enable = true;
		}
		$this->setButtonOption($button, "disabled", ($enable)?0:1);
		$this->setButtonOption($button, "class", ($enable)?"productHeaderButton":"productHeaderButtonOff");
	}

	/**
	 * function to set a button as selected
	 */
	function setSelected($button){
		dbg("ran with: ".$button);
		$this->setButtonOption($button, "selected", 1);
	}

	/**
	 * function to set a link for a specific button
	 */
	function setLink($button, $link){
		$this->setButtonOption($button, "link", $link);
	}

	/**
	 * function to set widget that displays the edit form for a button
	 */
	function setWidget($button, $widget){
		$this->setButtonOption($button, "widget", $widget);
	}

	/**
	 * function to set a button name
	 */
	function setName($button, $name){
		$this->setButtonOption($button, "name", $name);
	}

	/**
	 * function to set a button class
	 */
	function setClass($button, $class){
		$this->setButtonOption($button, "class", $class);
	}

	/**
	 * function to set a button icon
	 */
	function setIcon($button, $icon){
		$this->setButtonOption($button, "icon", $icon);
	}

	/**
	 * function to set a button icon
	 */
	function removeButton($button, $icon){
		$this->setButtonOption($button, "icon", $icon);
	}

	/**
	 * function to set a generic key on a button
	 */
	function setButtonOption($button, $option, $value=""){
		if ($this->structure[$button]){
			$this->structure[$button][$option] = $value;
		}
	}

	/**
	 * get the structure to send to smarty
	 */
	function getButtons(){
		return (array)$this->structure;
	}
}





// for compatabilities sake right now, i am doing this, i will change it later:
class MB_ProductAction extends MB_Action
{
	// holds the menu items for each different type
	public $menuItems;

	function MB_ProductAction($where="details") {
		$this->menuItems = array(
			"advanced" => array(
				"details",
				"orderforms",
				"cycles",
				"servergroups",
				"attributes",
				"pricing",
				"options", // it will be shown only for type = "domain"
				"optionvalues",
				//"coupons",
				//"relations",
				"miscellaneous",
				"children",
				//"emails",
				//"notifications",
				//"usage",
				"ledgergroups",
				"clone",
				"simple",
			),
			"unknown" => array(
				"details",
				//"servergroups",
				"attributes",
				"orderforms",
				"cycles",
				"optionvalues",
				"pricing",
				//"coupons",
				//"relations",
				"miscellaneous",
				"children",
				//"emails",
				//"notifications",
				//"usage",
				"ledgergroups",
				"clone",
				//"advanced", // all items are the same as in advanced menu
			),
			"hosting" => array(
				"details",
				"orderforms",
				"cycles",
				"servergroups",
				"attributes",
				"applications",
				"pricing",
				"optionvalues",
				//"coupons",
				//"relations",
				"miscellaneous",
				//"usage",
				"children",
				//"emails",
				//"notifications",
				"ledgergroups",
				"clone",
				//"advanced", // all items are the same as in advanced menu
			),
			"addon" => array(
				"details",
				"orderforms",
				"cycles",
				"pricing",
				"servergroups",
				"attributes",
				"optionvalues",
				//"coupons",
				//"relations",
				"miscellaneous",
				//"emails",
				"ledgergroups",
				"clone",
				"advanced",
			),
			"domain" => array(
				"details",
				"orderforms",
				"cycles",
				"options",
				"pricing",
				//"coupons",
				//"relations",
				"miscellaneous",
				"children",
				//"emails",
				//"notifications",
				"ledgergroups",
				"clone",
				"advanced",
			),
			"cert" => array(
				"details",
				"orderforms",
				"cycles",
				"attributes",
				"pricing",
				"optionvalues",
				//"coupons",
				//"relations",
				"miscellaneous",
				//"emails",
				//"notifications",
				"ledgergroups",
				"clone",
				"advanced",
			),
			"ssl-cert" => array(
				"details",
				"orderforms",
				"cycles",
				"sslplugin",
				"pricing",
				"optionvalues",
				"miscellaneous",
				"ledgergroups",
				"clone",
			),
		);


		dbg("in constructor for details action");
		$this->setConfigurePlanMessage();
		$this->selectedButton = $where;
		$this->productButtons = new ProductButtons();
	}


	function getProductInfo(){
		$this->productInfo = (!$this->productInfo)?correctArray($this->productButtons->getProductInfo()):$this->productInfo;
	}

	function setupTypeButtons($type){
		dbg("adding type: ".$type);
		$this->productInfo = (!$this->productInfo)?correctArray($this->productButtons->getProductInfo()):$this->productInfo;
		
		if ((int)$this->productInfo["productAddonGroupID"] && $type != getAPITypeID("products", "ssl-cert")){
			$this->setupButtons($this->menuItems["addon"]);
		} elseif (!(int)$_REQUEST["advanced"]){
			switch ($type){
				case getAPITypeID("products", "hosting"):
					$this->setupHostingButtons();
				break;
				case getAPITypeID("products", "domain"):
					$this->setupDomainButtons();
				break;
				case getAPITypeID("products", "cert"):
					$this->setupCertButtons();
					$this->smartyVars['warningMessages'][] = blmsg('TRANS_CERT_PRODUCT_CYCLE_DESCRIPTION'); 
				break;
				case getAPITypeID("products", "ssl-cert"):
					$this->setupSSLCertButtons();
				break;
				default:
					$this->setupButtons($this->menuItems["unknown"]);
				break;
			}
		} else {
			$menu_items = $this->menuItems["advanced"];
			if(getAPITypeID("products", "domain") != $type)
			{
				$key = array_search("options", $menu_items);
				if($key)
				{
					unset($menu_items[$key]);
				}
			}
			$this->setupButtons($menu_items);
		}
		if ((int)$this->productInfo["productAddonGroupID"]){
			$this->setupAddonOptions();
		} else {
			switch ($type){
				case getAPITypeID("products", "hosting"):
					$this->setupHostingOptions();
				break;
				case getAPITypeID("products", "domain"):
					$this->setupDomainOptions();
				break;
				case getAPITypeID("products", "cert"):
					$this->setupCertOptions();
				case getAPITypeID("products", "ssl-cert"):
					$this->setupSSLCertOptions();
				break;
			}
		}
	}

	function setupHostingButtons(){
		dbg("doing hosting buttons setup");
		$this->setupButtons($this->menuItems["hosting"]);
	}

	function setupDomainButtons(){
		dbg("doing domain buttons setup");
		$this->setupButtons($this->menuItems["domain"]);
	}

	function setupCertButtons(){
		dbg("doing certs buttons setup");
		$this->setupButtons($this->menuItems["cert"]);

		$disableButtons = array();
		dbg("product info for testing buttons...");
		dbg($this->productInfo);
		if (!(int)$this->productInfo["orderForms"][0]["orderFormID"]){
			$disableButtons = array("cycles", "options", "pricing", "options", "usage", "servergroups", "attributes", "children", "ledgergroups", "emails", "notifications", "relations");
		} elseif (!(int)$this->productInfo["ghostCycles"][0]["cycleID"]){
			$disableButtons = array("options", "pricing", "usage", "servergroups", "attributes", "children", "emails", "ledgergroups", "notifications", "relations");
		}

		foreach ((array)$disableButtons as $bttn){
			$this->productButtons->setEnabled($bttn, false);
		}
	}

	protected function setupSSLCertButtons()
	{
		dbg("doing SSL Cert buttons setup");
		$this->setupButtons($this->menuItems["ssl-cert"]);
	}

	// handle hosting specific options for buttons, like which ones are enabled and whatnot
	function setupHostingOptions()
	{
		$this->productButtons->setName("options", TRANS_OPTIONS);

		$disableButtons = array();
		dbg("product info for testing buttons...");
		dbg($this->productInfo);
		if(!isset($this->productInfo["orderForms"])) {
			$disableButtons = array("cycles", "options", "pricing", "usage", "ledgergroups", "servergroups", "attributes", "children", "emails", "notifications", "coupons", "relations");
		} elseif (!isset($this->productInfo["ghostCycles"])) {
			$disableButtons = array("options", "pricing", "usage", "servergroups", "ledgergroups", "attributes", "children", "emails", "notifications", "coupons", "relations");
		} elseif (!isset($this->productInfo["productVariants"])) {
			$disableButtons = array("servergroups", "emails", "relations");
		} else {
			$disableButtons = array("attributes");
			$serverGroups = Factory()->Product($this->productInfo['productID'])->ServerGroupList();
			foreach($serverGroups as $serverGroup) {
				$panels = $serverGroup->ControlPanelList();
				if(0 != count($panels)) {
					$disableButtons = array();
					break;
				}
			}
		}

		foreach ($disableButtons as $bttn) {
			$this->productButtons->setEnabled($bttn, false);
		}
	}

	function setupDomainOptions(){
		$this->productButtons->setName("options", TRANS_TLDS);

		$disableButtons = array();
		dbg("product info for testing buttons...");
		dbg($this->productInfo);
		if (!(int)$this->productInfo["orderForms"][0]["orderFormID"]){
			$disableButtons = array("cycles", "options", "pricing", "usage", "servergroups", "ledgergroups", "attributes", "children", "emails", "notifications", "coupons", "relations");
		} elseif (!(int)$this->productInfo["ghostCycles"][0]["cycleID"]){
			$disableButtons = array("options", "pricing", "usage", "servergroups", "attributes", "ledgergroups", "children", "emails", "notifications", "coupons", "relations");
		} else {
/*			$foundone = 0;
			dbg($this->productInfo);
			foreach ($this->productInfo["TLDs"] as $tld){
				dbg($tld);
				if ((int)$tld["productVariantActive"]==1){
					dbg("found one");
					$foundone = 1;
				}
			}*/
			//if (!(int)$foundone){
			if (!count($this->productInfo["productVariants"])){
				$disableButtons = array("pricing", "children", "emails", "notifications", "ledgergroups", "relations");
			}
		}

		foreach ((array)$disableButtons as $bttn){
			$this->productButtons->setEnabled($bttn, false);
		}

	}

	function setupAddonOptions(){
	}

	function setupCertOptions(){
	}

	protected function setupSSLCertOptions()
	{
		if (0 == (int)$this->productInfo["orderForms"][0]["orderFormID"]
			|| 0 == (int)$this->productInfo["ghostCycles"][0]["cycleID"])
		{
			$this->productButtons->setEnabled("pricing", false);
			$this->productButtons->setEnabled("ledgergroups", false);
		}
	}

	function setupButtons($buttons=array()){

		$this->productButtons->setupButtons($buttons);

		$this->addSSLPluginButton();
		$this->addOrderFormsButton();
		$this->addCyclesButton();
		$this->addOptionsButton();
		$this->addOptionValuesButton();
		$this->addPricingButton();
		$this->addUsageButton();
		$this->addServerGroupsButton();
		$this->addAttributesButton();
		$this->addApplicationsButton();
		$this->addAddonsButton();
		$this->addEmailsButton();
		$this->addNotificationsButton();
		$this->addCouponsButton();
		$this->addMiscellaneousButton();
		$this->addAddonAttributesButton();
		$this->addRelationsButton();
		$this->addLedgerGroupsButton();
		$this->addCloneButton();
		$this->addAdvancedButton();
		$this->addSimpleButton();

		$this->productButtons->setSelected($this->selectedButton);
		
	}
	
	protected function getLinkUrl($action, $aditionalParameters = array()) {
		$basicParameters = array(
			'productID'	=> (int)bgpc("productID"),
			'sid'		=> bgpc("sid"),
			'advanced'	=> (int)bgpc("advanced")
		);
		$parameters = array_merge($basicParameters, $aditionalParameters);
		return billing_safetyhtml(action_url('products', $action, $parameters));
	}

	protected function addSSLPluginButton(){

		$this->productButtons->setName("sslplugin", TRANS_SSL_PLUGIN);
		$this->productButtons->setEnabled("sslplugin");
		$this->productButtons->setIcon("sslplugin", "sslplugin.gif");
		$this->productButtons->setLink("sslplugin", $this->getLinkUrl('EditProductSSLPlugin'));
		$this->productButtons->setWidget("sslplugin", "EditProductSSLPlugin");

	}

	function addOrderFormsButton(){

		$this->productButtons->setName("orderforms", TRANS_ORDER_FORMS);
		$this->productButtons->setEnabled("orderforms");
		$this->productButtons->setIcon("orderforms", "orderform.gif");
		$this->productButtons->setLink("orderforms", $this->getLinkUrl('EditProductOrderForms'));
		$this->productButtons->setWidget("orderforms", "EditProductOrderForms");

	}


	function addOptionValuesButton(){

		$this->productButtons->setName("optionvalues", TRANS_OPTIONS);
		if(0 != $this->productInfo["productVariableID"]) {
			$this->productButtons->setEnabled("optionvalues");
		}
		$this->productButtons->setIcon("optionvalues", "options.gif");
		$this->productButtons->setLink("optionvalues", $this->getLinkUrl('EditAppliedValues'));
		$this->productButtons->setWidget("optionvalues", "EditAppliedValues");

	}


	function addCyclesButton(){

		$this->productButtons->setName("cycles", TRANS_BILLING_CYCLES);
		$this->productButtons->setEnabled("cycles");
		$this->productButtons->setIcon("cycles", "cycles.gif");
		$this->productButtons->setLink("cycles", $this->getLinkUrl('EditProductCycles'));
		$this->productButtons->setWidget("cycles", "EditProductCycles");

	}

	function addOptionsButton(){

		$this->productButtons->setName("options", TRANS_TLDS);
		$this->productButtons->setEnabled("options");
		$this->productButtons->setIcon("options", "options.gif");
		$this->productButtons->setLink("options", $this->getLinkUrl('EditProductTLDs'));
		$this->productButtons->setWidget("options", "EditProductTLDs");

	}

	function addPricingButton(){

		$this->productButtons->setName("pricing", TRANS_PRICES);
		$this->productButtons->setEnabled("pricing");
		$this->productButtons->setIcon("pricing", "pricing.gif");
		if (count($this->smartyVars["productVariants"])==1){
			$this->productButtons->setLink("pricing", $this->getLinkUrl('EditProductVariantPrices', array('productVariantID' => (int)$this->smartyVars["productVariants"][0]["productVariantID"])));
			$this->productButtons->setWidget("pricing", "EditProductVariantPrices");
		} else {
			$this->productButtons->setLink("pricing", $this->getLinkUrl('ShowProductVariants', array('productVariantActive' => 1)));
			$this->productButtons->setWidget("pricing", "ShowProductVariants");
		}

	}

	function addUsageButton(){

		$this->productButtons->setName("usage", TRANS_USAGE_PRICING);
		$this->productButtons->setEnabled("usage");
		$this->productButtons->setIcon("usage", "usage.gif");
		$this->productButtons->setLink("usage", $this->getLinkUrl('EditUsagePricing'));
		$this->productButtons->setWidget("usage", "EditUsagePricing");
	}

	function addServerGroupsButton(){

		$this->productButtons->setName("servergroups", TRANS_SERVER_GROUPS);
		$this->productButtons->setEnabled("servergroups");
		$this->productButtons->setIcon("servergroups", "servergroups.gif");
		$this->productButtons->setLink("servergroups", $this->getLinkUrl('EditProductServerGroups'));
		$this->productButtons->setWidget("servergroups", "EditProductServerGroups");
	}

	function addAttributesButton()
	{
		$this->productButtons->setName("attributes", TRANS_ATTRIBUTES);
		$this->productButtons->setEnabled("attributes");
		$this->productButtons->setIcon("attributes", "attributes.gif");
		$this->productButtons->setLink("attributes", $this->getLinkUrl('EditProductAttributes'));
		$this->productButtons->setWidget("attributes", "EditProductAttributes");
	}
	
	function addApplicationsButton()
	{
		$this->productButtons->setName("applications", TRANS_APPLICATIONS);
		$this->productButtons->setEnabled("applications");
		$this->productButtons->setIcon("applications", "applications.png");
		$this->productButtons->setLink("applications", $this->getLinkUrl('EditProductApplications'));
		$this->productButtons->setWidget("applications", "EditProductApplications");
	}

	function addAddonAttributesButton()
	{
		$this->productButtons->setName("addonattributes", TRANS_ATTRIBUTES);
		$this->productButtons->setEnabled("addonattributes");
		$this->productButtons->setIcon("addonattributes", "attributes.gif");
		$this->productButtons->setLink("addonattributes", $this->getLinkUrl('EditProductAddonAttributes'));
		$this->productButtons->setWidget("addonattributes", "EditProductAddonAttributes");
	}

	function addAddonsButton()
	{
		$this->productButtons->setName("children", TRANS_ADDONS);
		$this->productButtons->setEnabled("children");
		$this->productButtons->setIcon("children", "addons.gif");
		$this->productButtons->setLink("children", $this->getLinkUrl('ShowProductAddons'));
		$this->productButtons->setWidget("children", "ShowProductAddons");
	}

	function addEmailsButton(){

		$this->productButtons->setName("emails", TRANS_EMAILS);
		$this->productButtons->setEnabled("emails");
		$this->productButtons->setIcon("emails", "emails.gif");
		$this->productButtons->setLink("emails", $this->getLinkUrl('EditProductEmails'));
		$this->productButtons->setWidget("emails", "EditProductEmails");

	}

	function addNotificationsButton() {

		$this->productButtons->setName("notifications", TRANS_NOTIFICATIONS);
		$this->productButtons->setEnabled("notifications");
		$this->productButtons->setIcon("notifications", "emails.gif");
		$this->productButtons->setLink("notifications", $this->getLinkUrl('EditProductNotifications'));
		$this->productButtons->setWidget("notifications", "EditProductNotifications");

	}

	function addCouponsButton(){

		$this->productButtons->setName("coupons", TRANS_COUPONS);
		$this->productButtons->setEnabled("coupons");
		$this->productButtons->setIcon("coupons", "coupon.gif");
		$this->productButtons->setLink("coupons", $this->getLinkUrl('EditProductCoupons'));
		$this->productButtons->setWidget("coupons", "EditProductCoupons");

	}

	function addMiscellaneousButton(){

		$this->productButtons->setName("miscellaneous", TRANS_MISCELLANEOUS_OPTIONS);
		$this->productButtons->setEnabled("miscellaneous");
		$this->productButtons->setIcon("miscellaneous", "options.gif");
		$this->productButtons->setLink("miscellaneous", $this->getLinkUrl('EditProductMiscellaneous'));
		$this->productButtons->setWidget("miscellaneous", "EditProductMiscellaneous");

	}

	function addRelationsButton(){

		$this->productButtons->setName("relations", TRANS_UPGRADE_DOWNGRADE);
		$this->productButtons->setEnabled("relations");
		$this->productButtons->setIcon("relations", "cycles.gif");
		$this->productButtons->setLink("relations", $this->getLinkUrl('EditOneProductRelations'));
		$this->productButtons->setWidget("relations", "EditOneProductRelations");

	}

	function addLedgerGroupsButton(){

		$this->productButtons->setName("ledgergroups", TRANS_LEDGER_ENTRIES);
		$this->productButtons->setEnabled("ledgergroups");
		$this->productButtons->setIcon("ledgergroups", "pricing.gif");
		$this->productButtons->setLink("ledgergroups", $this->getLinkUrl('EditLedgerGroups'));
		$this->productButtons->setWidget("ledgergroups", "EditLedgerGroups");

	}

	function addCloneButton(){

		$this->productButtons->setName("clone", TRANS_PLUS_CLONE_PRODUCT);
		$this->productButtons->setEnabled("clone");
		$this->productButtons->setIcon("clone", "clone.gif");
		$this->productButtons->setLink("clone", $this->getLinkUrl('CloneProductResponse'));
		$this->productButtons->setWidget("clone", "CloneProductResponse");

	}

	function addAdvancedButton(){

		$this->productButtons->setName("advanced", TRANS_PLUS_ADVANCED);
		$this->productButtons->setEnabled("advanced");
		$this->productButtons->setIcon("advanced", "");
		$this->productButtons->setLink("advanced", $this->getLinkUrl('ShowProductDetails', array('advanced' => 1)));
		$this->productButtons->setWidget("advanced", "ShowProductDetails");

	}

	function addSimpleButton(){

		$this->productButtons->setName("simple", TRANS_MINUS_SIMPLE);
		$this->productButtons->setEnabled("simple");
		$this->productButtons->setIcon("simple", "");
		$this->productButtons->setLink("simple", $this->getLinkUrl('ShowProductDetails', array('advanced' => 0)));
		$this->productButtons->setWidget("simple", "ShowProductDetails");

	}
}
