<?php

require_once("lib-action/action.php");
require_once("UI/Page.php");

class ShowProducts extends MB_Action
{
	public function execute()
	{
		parent::execute();

		if (1 == bgpc("sync")) {
			$this->processSync();
		}

		$page = new ShowProductsPage();
		$page->description = blmsg('TRANS_PLAN_LIST_DESCRIPTION');
		$page->createControls();

		$this->smartyVars["page"] = $page;
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}

	private function processSync()
	{
		$planIDs = bgpc("id", array());
		if (empty($planIDs)) {
			return;
		}
		$errors = array();
		foreach ($planIDs as $planID) {
			try {
				$plan = Factory()->Product($planID);
				$plan->sync_tasks |= Product::SYNC_NEED_REPAIR;
				require_once("object_managers/ProductManager.php");
				ProductManager::sync($plan);
				if (!is_null($plan->sync_error)) {
					$errors[] = blmsg("TRANS_SYNC_PLAN_ERROR_MSG", array("PLAN_ID" => $plan->id, "ERROR" => $plan->sync_error));
				}
			} catch (ProductException $e) {
				$errors[] = blmsg("TRANS_SYNC_PLAN_ERROR_MSG", array("PLAN_ID" => $planID, "ERROR" => $e->getMessage()));
			}
		}
		if (empty($errors)) {
			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SYNC_PLANS_SUCCESS_MSG");
		} else {
			getCurrentSession()->Storage->errorMessages[] = blmsg(
				"TRANS_SYNC_PLANS_FAILURE_MSG",
				array("ERROR" => billing_safetyhtml(implode("\n", $errors)))
			);
		}
	}
}

class ShowProductsPage extends UIPage
{
	public function createControls()
	{
		// Prepare presentation of page
		$this->title = blmsg("TRANS_SHOW_PRODUCTS");
		$this->parent_url = action_url("dashboard", "ShowDashboard");
		// Create controls
		// Tunnel Vars for selection ability (used for import clients), ConfigGroupID, ServerGroupID

		$form = $this->getDefaultForm();
		$form->addHiddenControl($hidden = UIFactory()->InputHidden("sid")); $hidden->value = bgpc("sid");
		$form->addHiddenControl($hidden = UIFactory()->InputHidden("action")); $hidden->value = bgpc("act");

		require_once("UI/List/UIProductList.php");

		if (! is_null($tunnelVars = bgpc("tunnelVars")))
		{
			$this->subtitle = blmsg("TRANS_SELECT");
			$this->subtitle_icon_id = "bid-domain-templates";
			$this->subtitle_description = blmsg("TRANS_PLEASE_SELECT_THE_PRODUCT_THIS_CLIENT_HAS");

			$this->addListPanel(new UIProductList(UIProductList::MODE_FOR_CLIENT_IMPORT, array(
				"serverGroupID" => bgpc("serverGroupID"),
				"configGroupID" => bgpc("configGroupID"),
				"tunnelVars" => $tunnelVars,
			)));
		}
		elseif (! is_null($clientID = bgpc("clientID")))
		{
			$this->subtitle = blmsg("TRANS_SELECT");
			$this->subtitle_icon_id = "bid-domain-templates";
			$this->subtitle_description = blmsg("TRANS_PLEASE_SELECT_THE_PRODUCT_FOR_NEW_CLIENTS_PACKAGE", array(
				"CLIENT_NAME" => "#{$clientID} " . Factory()->Client($clientID)->PrimaryContact->contact_fullname,
			));

			$this->addListPanel(new UIProductList(UIProductList::MODE_FOR_NEW_CLIENTS_PACKAGE, array(
				"clientID" => $clientID,
			)));
		}
		else
		{
			/*$this->addToolsPanel($tools_panel = UIFactory()->ToolsPanel(blmsg("TRANS_TOOLS")));
			$tools_panel->addButton(UIFactory()->ToolButton(
				"new-product",
				blmsg("TRANS_CREATE_NEW_PRODUCT"),
				action_url("products", "AddProduct")
			));
			$tools_panel->addButton(UIFactory()->ToolButton(
				"plan-addons",
				blmsg("TRANS_SHOW_PLAN_ADDONS_BUTTON"),
				action_url("products", "ShowPlanAddons")
			));*/
			require_once("UI/List/UIProductList.php");
			$this->addListPanel(new UIProductList());
		}

		parent::createControls();
	}
}
