<?php

class Products_PlanOverviewController extends Billing_Zend_Controller_Action_AdminController
{
	public function init()
	{
		parent::init();
	}

	public function indexAction()
	{
		$problems = array();


        try {
            if (!is_null($id = $this->getRequest()->getParam('id'))) {
                $plan = Factory()->Product($id);
            } elseif(!is_null($productGUID = $this->getRequest()->getParam('productGUID'))) {
                $plan = Factory()->Product($productGUID, 'guid');
            } else {
                throw new ProductException(ProductException::E_OBJECT_NOT_FOUND, $this->_translator->translate("TRANS_NO_PLAN_ID_OR_GUID"));
            }
        } catch (ProductException $e) {
            $this->_redirectWithMessage($this->view->ActionUrl('products', 'ShowProducts'), $e->getMessage(), "error");
        }

		$this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_PLAN_OVERVIEW_TITLE', array('NAME' => $plan->product_name)));

		if (!$plan->product_active) {
			$problems[] = 'TRANS_PLAN_OVERVIEW_OVERVIEW_PROBLEM_INACTIVE';
		}

		$activeProductVariants = array();
		foreach ($plan->ProductVariantList() as $productVariant) {
			if ($productVariant->product_variant_active) {
				$activeProductVariants[$productVariant->id] = $productVariant;
			}
		}

		$activeCycles = array();
		foreach ($plan->cycleIDs as $cycleId) {
			$activeCycles[$cycleId] = true;
		}

		if (!$activeCycles) {
			$problems[] = 'TRANS_PLAN_OVERVIEW_OVERVIEW_PROBLEM_CYCLES';
		}

		$this->view->type = 'TRANS_PRODUCT_TYPE_NAME_' . $plan->product_type;
		$this->view->subscribers = $plan->getPackagesCount();

		require_once 'object_managers/ProductManager.php';
		require_once 'objects/ProductVariantPrice.php';
		$pricesData = ProductManager::getPricesData($plan->id);
		$activeCurrencies = array();
		foreach ($pricesData as $cycleId => $variants) {
			if (!isset($activeCycles[$cycleId])) {
				continue;
			}
			foreach ($variants as $variantId => $currencies) {
				if (!isset($activeProductVariants[$variantId])) {
					continue;
				}
				foreach ($currencies as $code => $prices) {
					if (isset($prices[ProductVariantPrice::TYPE_GENERAL])) {
						$activeCurrencies[$code] = true;
					}
				}
			}
		}
		$stores = $plan->OrderFormList();
		$actualCurrencies = array();
		$missedCurrencies = array();
		foreach ($stores as $store) {
			$actualCurrencies[$store->currency_id] = true;
			if (!isset($activeCurrencies[$store->currency_id]) && !isset($missedCurrencies[$store->currency_id])) {
				$problems[] = $this->_translator->translate('TRANS_PLAN_OVERVIEW_OVERVIEW_PROBLEM_CURRENCY') . $store->currency_id;
				$missedCurrencies[$store->currency_id] = true;
			}
		}
		$this->view->stores = $stores;

		require_once 'objects/Product.php';
		if ($plan->product_type == Product::TYPE_HOSTING) {
			$serverGroupIds = $plan->serverGroupIDs;
			if (!$serverGroupIds) {
				$problems[] = 'TRANS_PLAN_OVERVIEW_OVERVIEW_PROBLEM_SERVER_GROUPS';
			}
			if (sizeof($serverGroupIds) == 1) {
				$serverGroupId = $serverGroupIds[0];
				$attributes = array();
				$planAttributesList = $plan->ProductAttributesList(array('server_group_id' => $serverGroupId));
				if ($planAttributes = array_shift($planAttributesList)) {
					$displayableAttributes = array(
						'domainDiskSpace',
						'domainMaxTraffic',
						'domainMaxSites',
						'domainMaxMailbox',
						'domainMaxMaillists',
						'domainMaxMYSQLDB'
					);
					foreach ($displayableAttributes as $name) {
						if (isset($planAttributes[$name]) && $planAttributes[$name]->product_attribute_value) {
							$item = array('value' => $planAttributes[$name]->product_attribute_value);
							if (isset($planAttributes[$name . 'UNIT'])) {
								$item['unit'] = $planAttributes[$name . 'UNIT']->product_attribute_value;
							}
							$attributes[$name] = $item;
						}
					}
				}
				$this->view->attributes = $attributes;
				$this->view->attributesCount = sizeof($planAttributes);
				$this->view->displayAttributes = sizeof($attributes) ? true : false;
			} else {
				$this->view->displayAttributes = false;
			}

		} else {
			$this->view->displayAttributes = false;
		}

		if ($plan->product_type == Product::TYPE_DOMAIN) {
			if (!$activeProductVariants) {
				$problems[] = 'TRANS_PLAN_OVERVIEW_OVERVIEW_PROBLEM_TLDS';
			}
		}

		if ($plan->product_type == Product::TYPE_SSL_CERT) {
			$attributes = array_shift($plan->ProductAttributesList(array('server_group_id' => 0)));
			if (!isset($attributes['ssl_plugin_id']) || !$attributes['ssl_plugin_id']->value) {
				$problems[] = 'TRANS_PLAN_OVERVIEW_OVERVIEW_PROBLEM_SSL_PROVIDER';
			}
            $plugin = Factory()->SSLPlugin($attributes['ssl_plugin_id']->value);
            if (1 != $plugin->ssl_plugin_enabled) {
                $problems[] = $this->_translator->translate('TRANS_PLAN_SETTINGS_TAB_RESOURCES_SECTION_SSL_MODULE_DISABLED_WARNING', array('SSL_MODULE_NAME' => $plugin->ssl_plugin_name));
            }
		}

		if ($plan->isAddon()) {
			$this->view->parents = $plan->ParentList();
		} else {
			$children = $plan->ChildrenList();
			usort($children, array($this, '_sortAddons'));
			$this->view->children = $children;
		}

		$prices = array();
		require_once 'objects/ProductVariantPrice.php';
		$productVariantList = $plan->ProductVariantList();
		if ($variant = array_shift($productVariantList)) {
			foreach ($plan->CycleList() as $cycle) {
				$tmp = $variant->ProductVariantPriceList(
					array(
						'cycle_id' => $cycle->id,
						'product_variant_price_type' => ProductVariantPrice::TYPE_GENERAL
					)
				);
				$variantPrices = array();
				foreach ($tmp as $price) {
					if (isset($actualCurrencies[$price->currency_id])) {
						$variantPrices[] = $price;
					}
				}
				if ($variantPrices) {
					$prices[$cycle->cycle_display_name] = $variantPrices;
				}
			}
		}
		$this->view->prices = $prices;
		$this->view->displayPrices = $variant && !sizeof($productVariantList) ? true : false;

		if ($variant && $discount = $variant->CycleDiscountGroup) {
			$this->view->discount = $discount;
		}

		$this->view->coupons = $plan->CouponList();

		require_once 'UI/Page/UISelectProductTypePage.php';
		$toolButtons = array(
			'settings' => array(
				'label' => $plan->isAddon() ? 'TRANS_ADDON_OVERVIEW_TOOLBAR_SETTINGS' : 'TRANS_PLAN_OVERVIEW_TOOLBAR_SETTINGS',
				'class' => 'sb-plan-settings',
				'href' => $this->view->adminActionUrl('products', 'plan-settings', 'edit', array('id' => $plan->id))
			),
			'convert' => array(
				'label' => 'TRANS_PLAN_OVERVIEW_TOOLBAR_CONVERT',
				'class' => 'sb-plan-convert',
				'href' => $this->view->adminActionUrl('products', 'plan-settings', 'edit', array('id' => $plan->id, 'mode' => UISelectProductTypePage::MODE_ADDON)),
				'separate' => true
			),
			'ledger' => array(
				'label' => 'TRANS_PLAN_OVERVIEW_TOOLBAR_LEDGER',
				'class' => 'sb-plan-ledger',
				'href' => $this->view->actionUrl('products', 'EditLedgerGroups', array('productID' => $plan->id)),
				'separate' => true
			),
			'clone' => array(
				'label' => 'TRANS_PLAN_OVERVIEW_TOOLBAR_CLONE',
				'class' => 'sb-plan-clone',
				'href' => $this->view->adminActionUrl('products', 'plan-settings', 'clone', array('id' => $plan->id)),
				'separate' => true
			),
			'remove' => array(
				'label' => 'TRANS_PLAN_OVERVIEW_TOOLBAR_REMOVE',
				'class' => 'sb-plan-remove',
				'href' => $this->view->actionUrl('products', 'RemoveProduct', array('productID' => $plan->id)),
				'separate' => true
			)
		);
		require_once 'objects/Product.php';
		if (!$plan->isAddon() && $plan->product_type == Product::TYPE_SSL_CERT) {
		} else if ($plan->isAddon() && $plan->product_type == Product::TYPE_SSL_CERT) {
			$toolButtons['convert']['label'] = 'TRANS_ADDON_OVERVIEW_TOOLBAR_CONVERT';
			$toolButtons['convert']['href'] = $this->view->adminActionUrl('products', 'plan-settings', 'edit', array('id' => $plan->id, 'mode' => UISelectProductTypePage::MODE_PRODUCT));
		} else {
			unset($toolButtons['convert']);
		}

		$this->view->problems = $problems;

		$this->view->toolButtons = $toolButtons;

		$this->view->isAddon = $plan->isAddon();

		if ($plan->isSyncable() && !is_null($plan->sync_error)) {
			$transKey = $plan->isAddon() ? "TRANS_SYNC_THIS_PLAN_ADDON_ERROR_MSG" : "TRANS_SYNC_THIS_PLAN_ERROR_MSG";
			$this->_status->addError(blmsg($transKey, array("ERROR" => nl2br(billing_safetyhtml($plan->sync_error)))));
		}
    }

    private function _sortAddons($a, $b)
    {
    	if ($a->product_addon_group_id > $b->product_addon_group_id) {
    		return 1;
    	} else if ($a->product_addon_group_id < $b->product_addon_group_id) {
    		return -1;
    	} else {
    		return 0;
    	}
    }

}