<?php
require_once 'UI/Page/UISelectProductTypePage.php';

class Products_PlanSettingsController extends Billing_Zend_Controller_Action_AdminController
{
	
	const OPERATION_EDIT = 'edit';
	const OPERATION_CREATE = 'create';
	const OPERATION_CLONE = 'clone';
	
	public function init()
	{
		parent::init();
        $this->view->placeholder('pageHasTabs')->set(true);
	}

	public function editAction()
	{
		$plan = Factory()->Product($this->getRequest()->getParam('id'));
		$this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_PLAN_SETTINGS_TITLE', array('NAME' => $plan->product_name)));
		$this->view->placeholder('pagePath')->set(array(
			'TRANS_PLAN_OVERVIEW_TITLE' => $this->view->translate('TRANS_PLAN_OVERVIEW_TITLE', array('NAME' => $plan->product_name))
		));
		$mode = $this->getRequest()->getParam('mode');
		$this->process($plan, self::OPERATION_EDIT, $mode);
	}
	
	public function createAction()
	{
		$plan = Factory()->Product();
		
		require_once 'objects/Product.php';
		$availableTypes = Product::types();
		$requestedType = $this->getRequest()->getParam('type');
		$plan->product_type = isset($availableTypes[$requestedType]) ? $requestedType : Product::TYPE_HOSTING;
		
		$mode = $this->getRequest()->getParam('mode');
		if ($mode == UISelectProductTypePage::MODE_ADDON) {
			$plan->product_is_addon = 1;
			$this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_ADDON_SETTINGS_CREATE_TITLE'));
		} else {
			$this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_PLAN_SETTINGS_CREATE_TITLE'));
		}
		
		$this->process($plan, self::OPERATION_CREATE, $mode);
	}
	
	public function cloneAction()
	{
		$plan = Factory()->Product($this->getRequest()->getParam('id'));
		$this->view->placeholder('pageTitle')->set($this->view->translate('TRANS_PLAN_SETTINGS_CLONE_TITLE', array('NAME' => $plan->product_name)));
		$this->view->placeholder('pagePath')->set(array(
			'TRANS_PLAN_OVERVIEW_TITLE' => $this->view->translate('TRANS_PLAN_OVERVIEW_TITLE', array('NAME' => $plan->product_name))
		));
		$mode = $this->getRequest()->getParam('mode');
		$this->process($plan, self::OPERATION_CLONE, $mode);
	}
	
	private function process($plan, $operation, $mode = UISelectProductTypePage::MODE_PRODUCT)
	{
		$post = $this->_request->isPost() ? $this->_request->getPost() : null;
		switch ($operation) {
			case self::OPERATION_CREATE:
				$backUrl = $this->view->actionUrl('products', 'ShowProducts');
				$submitUrl = $this->view->adminActionUrl('products', 'plan-settings', 'create', array('type' => $plan->product_type, 'mode' => $mode));
				break;
			case self::OPERATION_CLONE:
				$backUrl = $this->view->adminActionUrl('products', 'plan-overview', 'index', array('id' => $plan->id));
				$submitUrl = $this->view->adminActionUrl('products', 'plan-settings', 'clone', array('id' => $plan->id, 'mode' => $mode));
				break;
			default:
				$backUrl = $this->view->adminActionUrl('products', 'plan-overview', 'index', array('id' => $plan->id));
				$submitUrl = $this->view->adminActionUrl('products', 'plan-settings', 'edit', array('id' => $plan->id, 'mode' => $mode));
		}
		$attributes = array(
			'data' => $post,
			'operation' => $operation,
			'cancelUrl' => $backUrl,
			'submitUrl' => $submitUrl
		);
		if ($mode == UISelectProductTypePage::MODE_ADDON || (!$mode && $plan->product_addon_group_id)) {
			$isAddon = true;
			$attributes['isAddon'] = true;
		} else {
			$isAddon = false;
		}
		if ($operation == self::OPERATION_CLONE) {
			$target = Factory()->Product();
			$target->product_type = $plan->product_type;
			$target->product_is_addon = $plan->product_is_addon;
			$attributes['target'] = $target;
		}
		$form = new Billing_Zend_Form_Final_PlanSettingsForm($plan, $attributes);
		
		if ($this->_request->isPost()) {
			try {
				$form->process();
				switch ($operation) {
					case self::OPERATION_CREATE:
						$redirectUrl = $this->view->adminActionUrl('products', 'plan-overview', 'index', array('id' => $plan->id));
						$message = $this->view->translate($isAddon ? 'TRANS_PLAN_SETTINGS_MESSAGE_SUCCESS_CREATE_ADDON' : 'TRANS_PLAN_SETTINGS_MESSAGE_SUCCESS_CREATE', array('NAME' => $plan->product_name));
						break;
					case self::OPERATION_CLONE:
						$redirectUrl = $this->view->adminActionUrl('products', 'plan-overview', 'index', array('id' => $target->id));
						$message = $this->view->translate($isAddon ? 'TRANS_PLAN_SETTINGS_MESSAGE_SUCCESS_CLONE_ADDON' : 'TRANS_PLAN_SETTINGS_MESSAGE_SUCCESS_CLONE', array('NAME' => $target->product_name));
						break;
					default:
						$redirectUrl = $backUrl;
						$message = $this->view->translate($isAddon ? 'TRANS_PLAN_SETTINGS_MESSAGE_SUCCESS_EDIT_ADDON' : 'TRANS_PLAN_SETTINGS_MESSAGE_SUCCESS_EDIT', array('NAME' => $plan->product_name));
				}
				$this->_redirectWithMessage($redirectUrl, $message);
				$this->_redirect($redirectUrl);
			} catch (ProductException $e) {
				$this->_status->addError($e->getMessage());
			}

		}
		$this->view->form = $form;
		$this->view->serverGroups = Factory()->ServerGroupList();
		$this->view->selectedServerGroups = $plan->ServerGroupList();
	}
	
}