<?php
require_once("lib-action/action.php");
require_once("UI/Page.php");

class AddDomainRegistrarSelectModulePage extends UIPage
{
	public function createControls()
	{
		$showTestingRegistrars = UIFactory()->InputHidden("showTestingRegistrars");
		$showTestingRegistrars->value = $this->getValue("showTestingRegistrars", 0);
		$this->getDefaultForm()->addHiddenControl($showTestingRegistrars);
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;

		$this->input_module = UIFactory()->InputRadio("module", blmsg("TRANS_DOMAIN_REGISTRAR_SELECT_OPTION"));
		$this->input_module->multicolumn = true;
		$this->input_module->column_width = "256px";

		$input_panel = UIFactory()->InputPanel();
		$input_panel->addInputControl($this->input_module);

		$this->addInputPanel($input_panel);
	}

	public function initControl()
	{
		$showTestingRegistrars = $this->getValue("showTestingRegistrars", 0);
		$this->description = blmsg("TRANS_ADD_DOMAIN_REGISTRAR_DESC", array(
			"SHOW_TESTING_LINK" => "<a href='#' id='showTestingRegistrarsLink' onclick='showTestingRegistrars();'" . ($showTestingRegistrars ? "style='display: none;'" : "") . ">" . blmsg("TRANS_ADD_DOMAIN_REGISTRAR_SHOW_TESTING_LINK") . "</a>",
			"HIDE_TESTING_LINK" => "<a href='#' id='hideTestingRegistrarsLink' onclick='hideTestingRegistrars();'" . ($showTestingRegistrars ? "" : "style='display: none;'") . ">" . blmsg("TRANS_ADD_DOMAIN_REGISTRAR_HIDE_TESTING_LINK") . "</a>",
		));

		require_once("object_managers/BillingRegistrarManager.php");
		$moduleOptions = array();
		foreach (BillingRegistrarManager::getNotInstalledModules() as $module) {
			$moduleInfo = BillingRegistrarManager::getModuleInfo($module);
			$moduleTitle = $moduleInfo["displayName"];
			if (Registrar::STATUS_STABLE != $moduleInfo["status"]) {
				$moduleTitle .= " (" . blmsg("TRANS_TESTING") . ")";
				$this->testingRegistrars[] = "{$this->input_module->name}_{$module}";
			}
			$moduleOptions[] = array(
				"value" => $module,
				"title" => $moduleTitle,
				"hint" => isset($moduleInfo["countryCodes"]) ? $this->getCountries($moduleInfo["countryCodes"]) : "",
				"isVisible" => $showTestingRegistrars || Registrar::STATUS_STABLE == $moduleInfo["status"],
			);
		}
		usort($moduleOptions, array($this, "compareTitle"));
		foreach ($moduleOptions as $option) {
			$this->input_module->addOption($option["value"], $option["title"], $option["hint"], $option["isVisible"]);
		}
		parent::initControl();
	}

	public function storeJS(UIJavaScriptStorage $storage)
	{
		parent::storeJS($storage);
		if ($storage->hasJS(get_class())) {
			return;
		}
		$js = "
			var testingRegistrars = " . json_encode($this->testingRegistrars) . ";

			function showTestingRegistrars()
			{
				for (var i = 0; i < testingRegistrars.length; i++) {
					jQuery('#' + testingRegistrars[i].replace('%', '\\\\%')).css('display', 'inline-block');
				}
				jQuery('#showTestingRegistrarsLink').css('display', 'none');
				jQuery('#hideTestingRegistrarsLink').css('display', 'inline');
				jQuery('#fid-showTestingRegistrars').val(1);
			}

			function hideTestingRegistrars()
			{
				for (var i = 0; i < testingRegistrars.length; i++) {
					jQuery('#' + testingRegistrars[i].replace('%', '\\\\%')).css('display', 'none');
				}
				jQuery('#hideTestingRegistrarsLink').css('display', 'none');
				jQuery('#showTestingRegistrarsLink').css('display', 'inline');
				jQuery('#fid-showTestingRegistrars').val(0);
			}
		";
		$storage->addJS(get_class(), $js);
	}

	public function processInput()
	{
		if (is_null($this->getValue("module"))) {
			$this->addWarningMessage(UIFactory()->WarningMessage(blmsg("TRANS_DOMAIN_REGISTRAR_NO_SELECTED_MSG")));
			return false;
		}
		return true;
	}

	protected function getSubmitButtons()
	{
		require_once("UI/WizardController.php");
		return array(
			UIFactory()->SubmitButton("wizard-next", blmsg("TRANS_SELECT"), UIWizardController::CMD_NEXT),
			UIFactory()->SubmitButton("cancel", blmsg("TRANS_CANCEL"), UIWizardController::CMD_CANCEL),
		);
	}

	private function compareTitle($moduleLeft, $moduleRight)
	{
		return strcasecmp($moduleLeft["title"], $moduleRight["title"]);
	}

	private function getCountries(array $codes)
	{
		$countries = array();
		foreach ($codes as $code) {
			$countries[] = blmsg("TRANS_ISO_3166_{$code}");
		}
		return implode(", ", $countries);
	}

	private $input_module;
	private $testingRegistrars = array();
}

class AddDomainRegistrar extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$back_url = action_url("registrars", "ShowRegistrars");
		$controller  = UIFactory()->WizardController($back_url);
		$controller->addPage($select_module_page = new AddDomainRegistrarSelectModulePage($controller));
		require_once("UI/Page/UIDomainRegistrarEditPage.php");
		$controller->addPage($module_params_page = new UIDomainRegistrarEditPage($controller));
		$select_module_page->title = $module_params_page->title = blmsg("TRANS_ADD_DOMAIN_REGISTRAR");
		$select_module_page->parent_title = $module_params_page->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$select_module_page->parent_url = $module_params_page->parent_url = $back_url;
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
