<?php

require_once("lib-action/action.php");
require_once("UI/Page.php");

class EditActionLogSettingsPage extends UIPage
{
	public function __construct($controller)
	{
		parent::__construct($controller);
		$this->form = $this->getDefaultForm();
	}

	public function createControls()
	{
		$this->title = blmsg("TRANS_ACTION_LOG_SETTINGS");
		$this->parent_title = blmsg("TRANS_ACTION_LOG");
		$this->parent_url = action_url("security", "ShowActionLog");

		require_once("UI/Tabs/UIActionLogTabs.php");
		$tabs = new UIActionLogTabs();
		$tabs->setCurrent("settings");
		$this->setScreenTabs($tabs);

		switch (getCurrentSession()->Storage->EditActionLogSettings_ProcessStatus)
		{
		case "ok":
			$this->addInfoMessage(UIFactory()->InfoMessage(blmsg("TRANS_ACTION_LOG_SETTINGS_SAVED_OK")));
			break;
		case "cancelled":
			$this->addWarningMessage(UIFactory()->WarningMessage(blmsg("TRANS_ACTION_LOG_SETTINGS_CANCELLED")));
			break;
		}
		unset(getCurrentSession()->Storage->EditActionLogSettings_ProcessStatus);

		$this->form->addInputPanel(
			$general_panel = UIFactory()->InputPanel(blmsg("TRANS_ACTION_LOG_SETTINGS_PANEL"))
		);

		$input_rotation_interval = UIFactory()->InputString(
			"security_ActionLogRotationInterval",
			blmsg("TRANS_ACTION_LOG_ROTATION_INTERVAL"),
			true,
			Billing_Checker::P_UINT_NOT_NULL,
			blmsg("TRANS_ACTION_LOG_ROTATION_INTERVAL_DESC")
		);
		$input_rotation_interval->value = $this->getValue("security_ActionLogRotationInterval", Settings()->security_ActionLogRotationInterval);
		$general_panel->addInputControl($input_rotation_interval);
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function processInput()
	{
		if (0 == count($action_log_tasks = Factory()->SystemQueueList(array("system_queue_command" => "ProcessActionLog", "system_queue_is_recurring" => "1"))))
			throw new Exception("ProcessActionLog event in event list is absent");

		$action_log_tasks[0]->system_queue_month = Settings()->security_ActionLogRotationInterval = $this->getValue("security_ActionLogRotationInterval");
		$action_log_tasks[0]->update();

		getCurrentSession()->Storage->EditActionLogSettings_ProcessStatus = "ok";
		return true;
	}

	public function processCancel()
	{
		getCurrentSession()->Storage->EditActionLogSettings_ProcessStatus = "cancelled";
	}
}

class EditActionLogSettings extends MB_Action
{
	public function execute()
	{
		parent::execute();
		$controller = UIFactory()->PageController(action_url("security", "EditActionLogSettings"));
		$controller->addPage(new EditActionLogSettingsPage($controller));
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
