<?php

require_once("lib-action/action.php");
require_once("UI/Page.php");

class AuthSettingsPage extends UIPage
{
	public function __construct($controller)
	{
		parent::__construct($controller);
		$this->form = $this->getDefaultForm();
	}

	public function createControls()
	{

		$this->title = blmsg("TRANS_MANAGE_AUTH_SECURITY_SETTINGS");
		$this->parent_title = blmsg("TRANS_MANAGE_SYSTEM");
		$this->parent_url = admin_action_url("configs", "system", "show");

		$this->form->addInputPanel(
			$general_panel = UIFactory()->InputPanel(blmsg("TRANS_AUTH_SECURITY_SETTINGS"))
		);

		require_once("Billing_Checker.php");

		// Password expiration interval in days
		$input_pass_expiration_interval = UIFactory()->InputOptionalString(
			"security_PasswordExpirationInterval",
			blmsg("TRANS_PASSWORD_EXPIRATION_INTERVAL"),
			true,
			Billing_Checker::P_UINT_NOT_NULL,
			blmsg("TRANS_PASSWORD_EXPIRATION_INTERVAL_DESC")
		);
		$input_pass_expiration_interval->offValue = "0";
		$input_pass_expiration_interval->optionTitle = blmsg("TRANS_NEVER_EXPIRE_PASSWORD");
		$input_pass_expiration_interval->value = $this->getValue("security_PasswordExpirationInterval");
		$general_panel->addInputControl($input_pass_expiration_interval);

		// Number of the login failure attempts
		$input_login_failure_attempts = UIFactory()->InputOptionalString(
			"security_LoginFailureAttempts",
			blmsg("TRANS_LOGIN_FAILURE_ATTEMPTS"),
			true,
			Billing_Checker::P_UINT_NOT_NULL,
			blmsg("TRANS_LOGIN_FAILURE_ATTEMPTS_DESC")
		);
		$input_login_failure_attempts->offValue = "0";
		$input_login_failure_attempts->optionTitle = blmsg("TRANS_TURN_OFF_LOGIN_ATTEMPTS");
		$input_login_failure_attempts->value = $this->getValue("security_LoginFailureAttempts");
		$general_panel->addInputControl($input_login_failure_attempts);

		// Locking interval in minutes
		$input_login_locking_interval = UIFactory()->InputString(
			"security_LoginLockingInterval",
			blmsg("TRANS_LOGIN_LOCKING_INTERVAL"),
			true,
			Billing_Checker::P_UINT_NOT_NULL,
			blmsg("TRANS_LOGIN_LOCKING_INTERVAL_DESC")
		);
		$input_login_locking_interval->value = $this->getValue("security_LoginLockingInterval");
		$general_panel->addInputControl($input_login_locking_interval);

		// Notify Admin about user's account was locked.
		$input_notify_admin_about_locking = UIFactory()->InputCheckBox(
			"security_NotifyAdminAboutAccountLocking",
			blmsg("TRANS_NOTIFY_ADMIN_ABOUT_ACCOUNT_WAS_LOCKED"),
			false,
			NULL,
			blmsg("TRANS_NOTIFY_ADMIN_ABOUT_ACCOUNT_WAS_LOCKED_DESC")
		);
		$input_notify_admin_about_locking->value = $this->getValue("security_NotifyAdminAboutAccountLocking");
		$general_panel->addInputControl($input_notify_admin_about_locking);

		// Notify the abused account about he was locked.
		$input_notify_abused_account_about_locking = UIFactory()->InputCheckBox(
			"security_NotifyAbusedAccountAboutLocking",
			blmsg("TRANS_NOTIFY_ABUSED_ACCOUNT_ABOUT_ACCOUNT_WAS_LOCKED"),
			false,
			NULL,
			blmsg("TRANS_NOTIFY_ABUSED_ACCOUNT_ABOUT_ACCOUNT_WAS_LOCKED_DESC")
		);
		$input_notify_abused_account_about_locking->value = $this->getValue("security_NotifyAbusedAccountAboutLocking");
		$general_panel->addInputControl($input_notify_abused_account_about_locking);
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function processInput()
	{
		foreach ($this->params as $param_name)
			Settings()->$param_name = $this->getValue($param_name);

		return true;
	}

	public function getValue($param_name, $def_value = NULL)
	{
		$value_from_db = Settings()->$param_name;
		$def_value = is_null($value_from_db) ? $def_value : $value_from_db;

		return parent::getValue($param_name, $def_value);
	}

	private $form;
	private $params = array(
		"security_PasswordExpirationInterval",
		"security_LoginFailureAttempts",
		"security_LoginLockingInterval",
		"security_NotifyAdminAboutAccountLocking",
		"security_NotifyAbusedAccountAboutLocking",
	);
}

class EditAuthSettings extends MB_Action
{
	public function execute()
	{
		parent::execute();
		$controller  = UIFactory()->PageController(admin_action_url("configs", "system", "show"));
		$controller->addPage(new AuthSettingsPage($controller));
		$controller->processRequest();

		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}
