<?php
if(!$CONF || $_REQUEST["CONF"]) {
	if (!defined("MB_LIBPATH")){
		define("MB_LIBPATH", realpath(dirname(__FILE__)."/../../../.."));
		require_once(MB_LIBPATH."/lib-pkg/parsefunction.php");
	}
	require_once mb_path("/lib-tk/include/common.php");
}

require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");
require_once mb_path("/lib-tk/include/xml/xml.php");

/**
 * This Action object collects information on domains that match the given parameters, and
 * generates the display of that information.
 *
 * @author  Andy Christ
 */
class CheckServerLoad extends MB_Action {
	/**
	 * The constructor for a CheckServerLoad action object. This merely calls the parent constructor,
	 * and sets the string representation of this object. This method also sets up $this->params and
	 * $this->sortColumns for use in $this->getMBAPIQuery().
	 */
	public function CheckServerLoad() {
		parent::MB_Action();
		$this->stringRep = "CheckServerLoad";
		$this->params = array
		("configGroupID");
	}

	/**
	 * The main method of this class that executes the getServerLoad action, and returns success or failure.
	 *
	 * @return  1 if this action executed successfully, or 0 if otherwise.
	 */
	public function execute() {
		global $CONF;
		parent::execute();

		$serverGroupId = (int)$_REQUEST["serverGroupID"];
		$controlPanelId = (int)$_REQUEST["configGroupID"];
		
		$serverLoadData = array();
		
		try {
			if(!$serverGroupId) {
				throw new Exception("No ServerGroup ID given.");
			}
			if ($controlPanelId) {
				$controlPanel = Factory()->ControlPanel($controlPanelId);
				$serverLoadData[$controlPanelId] = $this->_getServerLoadData($controlPanel);
			} else {
				$serverGroup = Factory()->ServerGroup($serverGroupId);
				foreach ($serverGroup->ControlPanelList() as $controlPanel) {
					$serverLoadData[$controlPanel->id] = $this->_getServerLoadData($controlPanel);
				}
			}
			
			$smartyVars = array();
			$smartyVars["panelCount"] = count($serverLoadData);
			$smartyVars["serverLoadData"] = $serverLoadData;
			
			$this->addTemplate("widgets/checkServerLoad", $smartyVars);
		} catch (ProductException $exception) {
			$this->addOperationErrors($exception);
			$this->addErrorTemplates();
		}
		
		return $this->hasExecutedSuccessfully();
	}
	
	/**
	 * Retrieve server load data of Panel,
	 * corresponding given ControlPanel
	 * 
	 * @param	ControlPanel	$controlPanel
	 */
	private function _getServerLoadData(ControlPanel $controlPanel) 
	{
		require_once('ModuleTool.php');
		$panel = ModuleTool::getModuleInstance(
			"panel",
			$controlPanel->module_name,
			array(
				"subCommand" => "panelDummy",
				"configGroupID" => $controlPanel->id
			)
		);
		$serverLoadData = array();
		if($panel->capabilities[PANEL_GETSERVERLOAD]) {
			$panel->panelGetServerLoad();
			$serverLoadData = $panel->arrayData;
		} else {
			if($panel->capabilities[PANEL_GETTESTS]){
				$panel->panelGetTest();
				$serverLoadData = $panel->arrayData;
			} else {
				$serverLoadData = "not capable";
			}
		}
		return $serverLoadData;
	}
	
}
?>