<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/action.php");
require_once("lib-mbapi/include/modules/module.php");
require_once("lib-tk/include/sso/SSOBillingFactory.php");
require_once("lib-tk/include/sso/SSOPleskAPIFactory.php");
require_once("objects/ControlPanel.php");

class ShowServerGroups extends MB_Action
{
	public $params = array("serverGroupID", "serverGroupName");

	public $sortColumns = array("serverGroupID", "serverGroupName");

	public function execute()
	{
		parent::execute();

		$query = $this->getMBAPIQuery("GetServerGroups", "", array("getPanels" => 1));
		require_once("lib-mbapi/mbapi.php");
		$data = dispatchMBAPI($query, 1);

		$results =& $data["mbapi"][0]["results"][0];

		//Get the server names
		$query = $this->getMBAPIQuery("GetConfigs", "", array("configParamName" => "moduleDisplayName", "getFormAttributes" => 1));
		$data = dispatchMBAPI($query, 1);

		if($data["mbapi"][0]["header"][0]["numResults"][0]>0) {

			$serverNames = collapseArrayElements($data["mbapi"][0]["results"][0]);

			if(is_array($serverNames["configs"]["config"][0])) {
				foreach($serverNames["configs"]["config"] as $key => $value) {
					$temp[$value["configGroupID"]] = $value;
				}
				$serverNames = $temp;
				$temp = NULL;
			} else {
				$serverNames[$serverNames["configs"]["config"]["configGroupID"]] = $serverNames["configs"]["config"];
			}
		}

		//Get the max accounts for each panel
		$query = $this->getMBAPIQuery("GetConfigs", "", array("configParamName" => "panelMaxAccounts", "getFormAttributes" => 1));
		$data = dispatchMBAPI($query, 1);

		if($data["mbapi"][0]["header"][0]["numResults"][0]>0) {
			$maxAccounts = correctArray($data["mbapi"][0]["results"][0]["configs"][0]["config"]);
			foreach($maxAccounts as $value) {
				$serverNames[$value["configGroupID"]]["panelMaxAccounts"] = (int) $value["configParamValue"];
			}
		}

		//Get the current number of accounts per panel
		$query = $this->getMBAPIQuery("GetConfigs", "", array("configParamName" => "panelCurrentAccounts", "getFormAttributes" => 1));
		$data = dispatchMBAPI($query, 1);

		if($data["mbapi"][0]["header"][0]["numResults"][0]>0) {
			$maxAccounts = correctArray($data["mbapi"][0]["results"][0]["configs"][0]["config"]);
			foreach($maxAccounts as $value) {
				$serverNames[$value["configGroupID"]]["panelCurrentAccounts"] = (int) $value["configParamValue"];
			}
		}

		if($this->addMBAPIErrors($data) === 0) {
			// Get list of files in modules/panel to build panel list
			// Only if a Panel isn't associated yet, otherwise, must keep the same Panel.
			$panels = getAvailableModules("panel");
			ksort($panels);
			foreach ($panels as $panelClass => $panelFile) {
				require_once mb_path("/lib-mbapi/include/modules/panel/".secureInput($panelFile));
				$moduleInfo = call_user_func(array($panelClass, "getModuleInfo"));
				$p = new $panelClass(array("subCommand" => "getCapabilities"));
				$canGetServerLoad = $p->capabilities[PANEL_GETSERVERLOAD];
				$useInternalIPQueue =  $p->capabilities[PANEL_USE_INTERNAL_IP_QUEUE];

				//The smarty var name panels will not work
				$this->smartyVars["controlPanels"][$panelClass] = $moduleInfo["displayName"];
				$this->smartyVars["canGetServerLoad"][$panelClass] = $canGetServerLoad;
				$this->smartyVars["useInternalIPQueue"][$panelClass] = $useInternalIPQueue;
			}

			$this->smartyVars["paginationLinks"] = $results["paginationLinks"][0];
			$this->smartyVars = collapseArrayElements(protectArray($this->smartyVars));
			$ips = array();
			// collapse the outer array first and properly format the sub array if 1 item or more than 1
			if(is_array($results["serverGroups"][0]["serverGroup"])) {
				foreach($results["serverGroups"][0]["serverGroup"] as $key => $value) {
					$panels = $value["panels"][0]["panel"];

					$details = correctArray($panels,"panels");

					foreach((array)$details as $panelInstance) {
						$controlPanel = Factory()->ControlPanel($panelInstance["configGroupID"]);
						if($this->smartyVars["useInternalIPQueue"][$controlPanel->module_name]) {
							//Pull the count of IPs for this panel instance
							$panelDetails = correctArray($panelInstance);

							$query = $this->getMBAPIQuery("GetIPLogs", "", array("configGroupID" => $panelDetails["configGroupID"]));
							$data = correctArray(dispatchMBAPI($query, 1));
							$ips[$panelDetails["configGroupID"]] = $data["mbapi"][0]["header"][0]["numTotalRecords"];
						}
					}
					$configGroups[$key] = collapseArrayElements($value);

					if(1 == count($panels)) {
						$panels = array(collapseArrayElements($panels));
					} else {
						$panels = collapseArrayElements($panels);
					}

					$hasSyncErrors = false;
					foreach($panels as $k => $panel)
					{
						$controlPanel = Factory()->ControlPanel($panel["configGroupID"]);
						$panels[$k]["moduleEnabled"] = $controlPanel->ConfigParams["moduleEnabled"]->value;
						$panels[$k]["syncStatus"] = $controlPanel->sync_status;
						$panelType = $controlPanel->module_name;
						if(ControlPanel::SYNC_STATUS_ERROR == $controlPanel->sync_status) {
							$hasSyncErrors = true;
						}
					}

					if(count($panels)>0) {
						$configGroups[$key]["panels"] = $panels;
						$configGroups[$key]["panelType"] = $panelType;
						$configGroups[$key]["canGetServerLoad"] = $this->smartyVars["canGetServerLoad"][$panelType];
						$configGroups[$key]["useInternalIPQueue"] = $this->smartyVars["useInternalIPQueue"][$panelType];
						$configGroups[$key]["hasSyncErrors"] = $hasSyncErrors;
					} else {
						unset($configGroups[$key]["panels"]);
					}
				}
			}

			if(count($serverNames)>1) {
				$this->smartyVars["serverNames"] = $serverNames;
			} else {
				$this->smartyVars["serverNames"][0] = $serverNames;
			}
			if ($_REQUEST['sso_repairing_started']) {
				$this->smartyVars["sso_repairing_started"] = true;
			}
			$this->smartyVars["ips"] = $ips;
			$this->smartyVars["serverGroups"] = $configGroups;
			$this->mergeSystemNavData();
			$this->addTemplate("servers/ShowServerGroups");
		}
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}