<?php

/**
 * modernbill-admin specific implementation of operations required by ProcessSSOAPIRequest 
 */
class AdminSSOAPI
{
    /**
     * Chceck admin's credentials.
     *
     * @param string $name User name
     * @param string $password User's password. If this parameter is empty then only check that user name is correct. 
     * @return String User name identifier (login name).
     */    
	public function checkCredentials($name, $password)
	{
		$has_true_creds = ! is_null($admin_id = $this->getAdminID($name, $password));
		$ip_address = bgpc("request_origin");

		require_once("object_managers/SecurityManager.php");
		SecurityManager::registerAdminAccess($name, $ip_address, $has_true_creds);

		return $has_true_creds && SecurityManager::isAdminAccessAllowed($name, $ip_address) ? $admin_id : NULL;
    }

    /**
     * Close session by admin's user name.
     *
     * @param string $userName
     */    
	public function closeSessionByName($userName)
	{
        $qh = new QueryMaster('select');
        $qh->addTables('admins');
        $qh->addColumns('admin_id');
        $qh->addAndClause('admin_username', '=', $userName);
        foreach ($qh->getResults() as $row) {
            $adminId = $row['admin_id'];

            $h = new QueryMaster('update');
            $h->addTables('sessions');
            $h->addParam('session_active', 0, 'i');
            $h->addAndClause('user_id', '=', $adminId, 'i');
            $h->addAndClause('user_type', '=', 2, 'i');
            $h->execute();
        }
    }

	private function getAdminID($name, $password)
	{
		try
		{
			$admin = Factory()->Admin($name, 'admin_username');
		}
		catch (ProductException $e)
		{
			if ($e->getCode() == ProductException::E_OBJECT_NOT_FOUND)
				return NULL;

			throw $e;
		}

		$password_hash = getPasswordHash($password);
		require_once('object_managers/AdminManager.php');

		if ($password_hash == $admin->admin_temp_password)
		{
			$admin->admin_password = $admin->admin_temp_password;
			$admin->admin_temp_password = NULL;
			$admin->update();
			return $admin->id;
		}

		if ($password_hash == $admin->admin_password)
		{
			if ("" != $admin->admin_temp_password)
			{
				$admin->admin_temp_password = NULL;
				$admin->update();
			}
			return $admin->id;
		}
		return NULL;
	}
}

