<?php
require_once("lib-action/action.php");
require_once("UI/Page.php");

class AddWebsiteWidgetPage extends UIPage
{
	public function createControls()
	{
		$inputPanel = UIFactory()->InputPanel();
		$this->addInputPanel($inputPanel);

		$websiteWidgetNameInput = UIFactory()->InputString("websiteWidgetName", blmsg("TRANS_WEBSITE_WIDGET_NAME_OPTION"), true);
		$inputPanel->addInputControl($websiteWidgetNameInput);
		$this->input_controls["websiteWidgetName"] = $websiteWidgetNameInput;

		$codeInput = UIFactory()->InputString("code", blmsg("TRANS_WEBSITE_WIDGET_CODE"), true);
		$inputPanel->addInputControl($codeInput);
		$this->input_controls["code"] = $codeInput;

		$continueShoppingUrlInput = UIFactory()->InputString("continue_shopping_url", blmsg("TRANS_WEBSITE_WIDGET_CONTINUE_SHOPPING_URL"));
		$continueShoppingUrlInput->disabled = true;
		$continueShoppingUrlInput->hint = blmsg("TRANS_WEBSITE_WIDGET_CONTINUE_SHOPPING_URL_DISABLED_DESC");
		$inputPanel->addInputControl($continueShoppingUrlInput);
		$this->input_controls["continue_shopping_url"] = $continueShoppingUrlInput;

		$websiteWidgetTplInput = UIFactory()->InputDropDown("websiteWidgetTpl", blmsg("TRANS_WEBSITE_WIDGET_TEMPLATE_OPTION"), true);
		require_once("object_managers/OrderFormManager.php");
		foreach(OrderFormManager::getWidgetTemplates() as $name => $title) {
			$websiteWidgetTplInput->addOption($name, $title);
		}
		$inputPanel->addInputControl($websiteWidgetTplInput);
		$this->input_controls["websiteWidgetTpl"] = $websiteWidgetTplInput;

		$productsList = Factory()->OrderForm(bgpc("orderFormID"))->ProductList();
		if(count($productsList) > 0) {
			$productsInput = UIFactory()->InputDoubleListSelector("products", blmsg("TRANS_WEBSITE_WIDGET_PRODUCTS_OPTION"));
			foreach($productsList as $product) {
				$productsInput->addOption($product->product_id, $product->product_name);
			}
			$inputPanel->addInputControl($productsInput);
			$this->input_controls["products"] = $productsInput;
		} else {
			$this->addWarningMessage(UIFactory()->WarningMessage(blmsg("TRANS_NO_PRODUCTS_TIED_TO_ORDER_FORM")));
		}
		$this->getDefaultForm()->buttons_align = UIForm::BUTTONS_ALIGN_CENTER;
	}

	public function processInput()
	{
		try {
			if(0 != count(Factory()->WebsiteWidgetList(array("code" => $this->input_controls["code"]->value)))) {
				$this->addErrorMessage(UIFactory()->ErrorMessage(blmsg("TRANS_WEBSITE_WIDGET_CODE_ALREADY_EXISTS_MSG", array("CODE" => bgpc("code")))));
				return false;
			}

			$websiteWidget = Factory()->WebsiteWidget();
			$websiteWidget->website_widget_name = $this->input_controls["websiteWidgetName"]->value;
			$websiteWidget->website_widget_tpl = $this->input_controls["websiteWidgetTpl"]->value;
			$websiteWidget->order_form_id = (int)bgpc("orderFormID");
			$websiteWidget->code = $this->input_controls["code"]->value;
			$websiteWidget->planIDs = isset($_REQUEST["products"]) ? $this->input_controls["products"]->value : array();
			$websiteWidget->update();

			getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SETTINGS_SAVED_SUCCESSFULLY");
		} catch (ProductException $e) {
			$this->addErrorMessage(UIFactory()->ErrorMessage($e->getMessage()));
		}

		return !$this->hasWarnings() && !$this->hasErrors();
	}

	protected function getSubmitButtons()
	{
		return array(UIFactory()->SubmitButton("ok", blmsg("TRANS_SAVE"), UIPageController::CMD_OK));
	}
}

class AddWebsiteWidget extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$orderFormID = bgpc("orderFormID");
		if ( is_null($orderFormID) ) {
			throw new Exception("Missing parameter 'orderFormID'");
		}
		$orderForm = Factory()->OrderForm($orderFormID);

		$orderFormPageURL = action_url("orderforms", "ShowOrderFormDetails", array("orderFormID" => $orderFormID));
		$controller  = UIFactory()->PageController($orderFormPageURL);
		$controller->addPage($edit_page = new AddWebsiteWidgetPage($controller));

		$edit_page->title = blmsg("TRANS_ADD_WEBSITE_WIDGET_FOR_STORE", array("store_name" => $orderForm->order_form_name));
		$edit_page->parent_title = $orderForm->order_form_name;
		$edit_page->parent_url = $orderFormPageURL;

		$controller->processRequest();
		$this->smartyVars["page"] = $controller->getPage();
		$this->addTemplate("page");
		return $this->hasExecutedSuccessfully();
	}
}