<?php

require_once("lib-tk/shield.php");
require_once("lib-action/action.php");

class ShowDashboard extends MB_Action
{
	public function execute()
	{
		global $CONF;
		parent::execute();

		$contact = Factory()->ClientContact(getCurrentSession()->user_id);
		$clientID = $contact->Client->id;

		// ---------------------------------------------------
		// Get client data ...
		// ---------------------------------------------------
		$this->mergeClientData();

		// ---------------------------------------------------
		// Get DUE invoices ...
		// ---------------------------------------------------
		$this->smartyVars["invoices"] = array(); // init
		$params = array(
			"clientID" => $clientID,
			"getDueInvoicesOnly" => 1,
			"getInvoiceLineItemsData" => 0
		);
		require_once("lib-mbapi/mbapi.php");
		$query = getMBAPIQuery("GetInvoices", "",  $params, "ASC", "invoiceID", 0, 5);
		$data = dispatchMBAPI($query, 1);
		$results =& $data["mbapi"][0]["results"][0];
		if($this->addMBAPIErrors($data) === 0) {
			// set master invoices data array
			$this->smartyVars["invoices"] = collapseArrayElements(protectArray($results["invoices"][0]["invoice"]));
			// build invoice line item data
			if($this->smartyVars["invoices"]) {
				for ($i = 0; $i < count($this->smartyVars["invoices"]); $i++){
					if (!$this->smartyVars["invoices"][$i]["lineItems"]["lineItem"][0]["clientID"]){
						$this->smartyVars["invoices"][$i]["lineItems"]["lineItem"] = array($this->smartyVars["invoices"][$i]["lineItems"]["lineItem"]);
					}
				}
			}

			// set master invoices line item data array
			$lineItems = array();
			$c = count($results["invoices"][0]["invoice"]);
			for($i = 0; $i < $c; $i++) {
				// leave this uncollapsed so that the template code can work with both singular and plural results;
				// see the comment in the template for details
				$lineItemsData =& $results["invoices"][0]["invoice"][$i]["lineItems"][0]["lineItem"];
				$lineItems[$results["invoices"][0]["invoice"][$i]["invoiceID"][0]] = collapseArrayElements(protectArray($lineItemsData));
			}
			$this->smartyVars["lineItems"] = $lineItems;
		}

		// ---------------------------------------------------
		// Get ACTIVE packages ...
		// ---------------------------------------------------
		$suppressFeePackages = ((int)$_REQUEST["showFees"])?0:1;

		$this->smartyVars["packages"] = correctArray(grabResultsFromMBAPIArray(array(
			"mbapi" => array(
				"command" => "GetPackages",
				"params" => array(
					"clientID" => $clientID,
					"getBillablePackagesOnly" => 1,
					"getDomainData" => 1,
					"suppressFeePackages" => $suppressFeePackages,
				),
			),
		)));

		$this->smartyVars["packageStatuses"] = getStatuses("package");
		require_once("lib-billing/include/config/internal/cycles.php");
		$this->smartyVars["cycles"] = getCyclesArray();

		// ---------------------------------------------------
		// Get OPEN tickets ...
		// ---------------------------------------------------
		$helpdeskEnabled = $CONF["system"]["main"]["helpdesk_enabled"];
		if($helpdeskEnabled) {
			$this->smartyVars["helpdeskEnabled"] = 1;
			$this->smartyVars["helpdeskDepartments"] = $this->getMBAPIData("GetHelpdeskDepartments", "helpdeskDepartments", "helpdeskDepartment");
			$this->smartyVars["helpdeskBuckets"] = $this->getMBAPIData("GetHelpdeskBuckets", "helpdeskBuckets", "helpdeskBucket");
			$query = new MBQuery("GetHelpdeskTickets", "",array(
									"userID" => $clientID, "userType" => 1,
									//	"getHelpdeskDepartmentData" => 1,
									//	"getHelpdeskBucketData" => 1,
									"getUserData" => 1));
			$data = $query->getResult();



			if(!$data->getErrorCount()) {

				$this->smartyVars["helpdeskTickets"] = grabSimpleMBAPIResults( $data->getStructure() );

				for($i=0, $n=count( $this->smartyVars ); $i<$n; $i++) {
					// Below we add an extra array() to avoid the effects of collapseArrayElements().
					$this->smartyVars["helpdeskTickets"][ $i ]["hdDepartmentIDs"] =
						array(arrayParentToChild($this->smartyVars["helpdeskTickets"][$i]["hdDepartmentIDs"],
							"hdDepartmentID"));
					$this->smartyVars["helpdeskTickets"][ $i ]["hdBucketIDs"] =
						array(arrayParentToChild($this->smartyVars["helpdeskTickets"][ $i ]["hdBucketIDs"],
							"hdBucketID"));
				}	unset( $i, $n );

				if(is_array($this->smartyVars["helpdeskTickets"])) {

					$openID = getAPIStatusID("helpdesk", "open");
					$closedID = getAPIStatusID("helpdesk", "closed");

					$this->smartyVars["helpdeskTicketStatus"] = array("open" => $openID, "closed" => $closedID);

					foreach( (array)$this->smartyVars["helpdeskTickets"] as $key => $value) {
						switch ($value["hdTicketStatusID"]) {
						case 0:
							$this->smartyVars["helpdeskTicketsClosed"][ $value["hdTicketID"] ] = $value["hdTicketKey"];
						break;
						case 2:
							$this->smartyVars["helpdeskTicketsOpen"][ $value["hdTicketID"] ] = $value["hdTicketKey"];
							$this->smartyVars["helpdeskTicketsOpenSubject"][ $value["hdTicketID"] ] = substr($value["hdTicketSubject"], 0, 25);
						break;
						}
					}
				}
			}
		} else {
			$this->smartyVars["helpdeskEnabled"] = 0;
		}

		//---------------------------------------------------------------------------------------------
		// Get News
		//---------------------------------------------------------------------------------------------
		$getNews = new MBQuery("GetNewsGroups", "", array("newsGroupActive" => 1));
		$newsGroups = $getNews->getResult();
		$newsGroups = grabSimpleMBAPIResults( $newsGroups->getStructure() );

		if (is_array( $newsGroups ))
		foreach( (array)$newsGroups as $group ) {
			if (notZero( $group["newsGroupID"] )) {
				$this->smartyVars["news"][] = $group;

				$getArticle = new MBQuery("GetNews", "", array("newsGroupID" => (int)$group["newsGroupID"], "newsActive" => 1));
				$articles = $getArticle->getResult();
				$articles = grabSimpleMBAPIResults( $articles->getStructure() );

				if (notNull( $articles )) {
					$this->smartyVars["news"][ count($this->smartyVars["news"])-1 ]["articles"] = $articles;
				}
				unset( $articles, $getArticle );
			}
		}
		unset($newsGroups, $getNews);

		$uir_packages = Factory()->PackageList(array(
			"client_id" => $clientID,
			"package_status" => getAPIStatusID("package", "user_input_required"),
		));
		if (0 < count($uir_packages))
		{
			$messages = array(blmsg("TRANS_UIR_YOU_HAVE_PACKAGES_THAT_REQUIRE_ADDITIONAL_DATA"));
			foreach ($uir_packages as $package)
			{
				$domain = is_null($package->PrimaryDomain)
					? ""
					: " ({$package->PrimaryDomain->domain_sld}.{$package->PrimaryDomain->domain_tld})";

				$message = blmsg("TRANS_UIR_PACKAGE_NAME", array(
					"packageID" => $package->package_id,
					"packageName" => $package->package_name . $domain,
				));
				$message = "<a href=\"".action_url("packages", "ShowClientServiceInputForm", array("packageID" => $package->package_id))."\">{$message}</a>";
				$messages[] = $message;
			}
			$this->smartyVars["messages"] = UIFactory()->WarningMessage(join("<BR/>", $messages))->getHTML();
		}

		if ( 0 == $contact->contact_primary && $contact->contact_username != $contact->contact_email ) {
			$this->smartyVars["warningMessages"][] = blmsg(
				"TRANS_CLIENT_CONTACT_EMAIL_USERNAME_DIFFERENT_CONTACT_WELCOME_MSG",
				array("URL" => action_url("profile", "EditClientContact"))
			);
		}

		$this->hasExecutedSuccessfully = ((int) $this->addTemplate("dashboard/ShowDashboard", $this->smartyVars));

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
