<?php

require_once("lib-action/action.php");

class PayInvoices extends MB_Action
{
	public function execute()
	{
		global $CONF;
		parent::execute();

		require_once("object_managers/CryptManager.php");
		$this->smartyVars["nolek"] = (CryptManager::isLekEnabled() ? 0 : 1);

		$clientID = ((int) $CONF["user"]["id"]);
		if ( !$clientID ) {
			$this->addError(TRANS_ACTIONERROR, "You must not be logged in because you don't have an ID.");
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		// ---------------------------------------------------
		// Get client data ...
		// ---------------------------------------------------
		$this->mergeClientData($clientID, array("getBillingData"=>1));
		if ($this->smartyVars["client"]["billingAccounts"]["billingAccount"]["billingAccountID"]){
			$this->smartyVars["client"]["billingAccounts"]["billingAccount"] = array($this->smartyVars["client"]["billingAccounts"]["billingAccount"]);
		}
		$this->smartyVars["client"]["billingAccounts"]["billingAccount"] = (array)$this->smartyVars["client"]["billingAccounts"]["billingAccount"];
		$this->smartyVars["billingAccounts"] = array();

		require_once("objects/ClientBillingAccount.php");
		foreach ($this->smartyVars["client"]["billingAccounts"]["billingAccount"] as $acct){
			if(0 == $acct["billingAccountActive"]) {
				continue;
			}
			$account_name = Factory()->ClientBillingAccount($acct["billingAccountID"])->getAccountName();
			switch ($acct["billingAccountType"])
			{
			    case ClientBillingAccount::TYPE_CREDIT_CARD:
		            case ClientBillingAccount::TYPE_ECHECK:
		            case ClientBillingAccount::TYPE_HSPC_BANK_TRANSFER:
					$this->smartyVars["billingAccounts"][$acct["billingAccountID"]] = $account_name;
	            	break;
	            default:
	            	break;
			}

		}
		dbg($this->smartyVars);
		for ($i=date("Y"); $i<date("Y")+21; $i++){
			$this->smartyVars["cardYears"][$i] = $i;
		}
		$this->smartyVars["cardTypes"] = $CONF["types"]["card"];

		$this->smartyVars["payTotal"] = 0;

		// ---------------------------------------------------
		// Get DUE invoices ...
		// ---------------------------------------------------
		$this->smartyVars["invoices"] = array(); // init
		$this->smartyVars["invoiceGroups"] = array();
		$params = array(
			"clientID" => $clientID,
			"getDueInvoicesOnly" => 1,
		);
		if ((int)$_REQUEST["invoiceID"]){
			$params["invoiceID"] = $this->smartyVars["invoiceID"] = (int)$_REQUEST["invoiceID"];
		}
		require_once mb_path("/lib-mbapi/mbapi.php");
		$query = getMBAPIQuery("GetInvoices", "",  $params, "ASC", "invoiceID", 0);
		$data = dispatchMBAPI($query, 1);
		if ( 0 !== $this->addMBAPIErrors($data) ) {
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		// set master invoices data array
		$results = $data["mbapi"][0]["results"][0];
		$this->smartyVars["invoices"] = collapseArrayElements(protectArray($results["invoices"][0]["invoice"]));

		if (empty($this->smartyVars["invoices"])) {
			if (isset($params["invoiceID"])) {
				throw new ProductException(
					ProductException::E_FAIL_OPERATION,
					blmsg("TRANS_PAYMENT_INVOICE_ALREADY_PAYED_ERROR_MSG", array("INVOICE_ID" => $params["invoiceID"]))
				);
			} else {
				throw new ProductException(ProductException::E_FAIL_OPERATION, blmsg("TRANS_PAYMENT_NO_INVOICES_TO_PAY_ERROR_MSG"));
			}
		}

		// build invoice line item data
		if($this->smartyVars["invoices"]) {
			for ($i = 0; $i < count($this->smartyVars["invoices"]); $i++){
				$this->smartyVars["invoices"][$i]["invoiceAmountDue"] = $this->smartyVars["invoices"][$i]["invoiceAmount"]-$this->smartyVars["invoices"][$i]["invoiceAmountPaid"];
				$this->smartyVars["invoiceGroups"][$this->smartyVars["invoices"][$i]["orderFormID"]]["invoices"][] =& $this->smartyVars["invoices"][$i];
				$thisdue = $this->smartyVars["invoices"][$i]["invoiceAmount"]-$this->smartyVars["invoices"][$i]["invoiceAmountPaid"];
				if ($thisdue<0)$thisdue = 0;
				$this->smartyVars["invoiceGroups"][$this->smartyVars["invoices"][$i]["orderFormID"]]["payTotal"] += $thisdue;
			}
		}

		$order_form_ids = array_keys($this->smartyVars["invoiceGroups"]);
		$order_forms = Factory()->OrderFormList(array("order_form_id" => $order_form_ids));
		foreach ( $order_forms as $order_form ) {
			if ( !is_null($order_form->PrimaryGateway) ) {
				$this->smartyVars["invoiceGroups"][$order_form->id]["hasPrimary"] = 1;
				$this->smartyVars["invoiceGroups"][$order_form->id]["cardTypes"] = $order_form->PrimaryGateway->acceptedCardTypes;
			}
			if ( !is_null($order_form->HSPCBankTransferGateway) ) {
				$this->smartyVars["invoiceGroups"][$order_form->id]["hasHSPCBankTransfer"] = 1;
			}
			$this->smartyVars["invoiceGroups"][$order_form->id]["secondary"] = $order_form->ThirdPartyGatewaysList();
		}
		$this->smartyVars["secondaryGatewayNames"] = array();
		require_once("object_managers/PaymentGatewayManager.php");
		require_once("object_managers/PaymentManager.php");
		foreach ((array)$this->smartyVars["invoices"] as $k=>$invoice){
			if ( !isset($this->smartyVars["invoiceGroups"][$invoice["orderFormID"]]["secondary"]) ) {
				continue;
			}
			$secondaries = $this->smartyVars["invoiceGroups"][$invoice["orderFormID"]]["secondary"];
			foreach ( $secondaries as $secondary_gateway ) {
				try {
					if (!PaymentGatewayManager::isOperationSupported($secondary_gateway, PaymentGatewayManager::OPERATION_CHARGE)) {
						continue;
					}
					$billingAccount = reset(Factory()->ClientBillingAccountList(array(
						"billing_account_type" => ClientBillingAccount::TYPE_THIRD_PARTY_GATEWAY,
						"client_contact_id" => getCurrentSession()->user_id,
						"payment_gateway_id" => $secondary_gateway->id,
					)));
					if (false === $billingAccount) {
						$billingAccount = Factory()->ClientBillingAccount();
						$billingAccount->billing_account_type = ClientBillingAccount::TYPE_THIRD_PARTY_GATEWAY;
						$billingAccount->client_contact_id = getCurrentSession()->user_id;
						$billingAccount->payment_gateway_id = $secondary_gateway->id;
						$billingAccount->update();
					}
					list($chargeRequest) = PaymentManager::getChargeRequests(Factory()->Invoice($invoice["invoiceID"]), $billingAccount, true);
					$this->smartyVars["invoices"][$k]["secondary"][$secondary_gateway->moduleName][] = $chargeRequest;
				} catch (ProductException $e) {
					dbg($e->getMessage());
					// skip gateway
				}
			}
			if ($this->smartyVars["invoices"] && $this->smartyVars["invoices"][$k] && is_array($this->smartyVars["invoices"][$k])){
				$this->smartyVars["secondaryGatewayNames"][$invoice["orderFormID"]] = array_keys((array)$this->smartyVars["invoices"][$k]["secondary"]);
			}
		}

		$this->hasExecutedSuccessfully = ((int) $this->addTemplate("invoices/PayInvoices", $this->smartyVars));

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
