<?php

require_once(dirname(__FILE__)."/".str_repeat("../",4)."lib-tk/shield.php");
require_once("lib-action/action.php");

class PayInvoicesResponse extends MB_Action
{
	public function execute()
	{
		parent::execute();

		$contact = Factory()->ClientContact(getCurrentSession()->user_id);
		$clientID = $contact->client_id;

		$invoiceIDs = bgpc("invoiceIDs");
		$billingAccountID = bgpc("billingAccountID");
		$paymentAmount = doubleval(bgpc("payTotal"));

		$oid = ((int)$_REQUEST["invoiceID"]) ? "<invoiceID>".(int)$_REQUEST["invoiceID"]."</invoiceID>":"";

		$invoiceXML = "
			<mbapi>
				<command>GetInvoices</command>
				<params>
					<clientID>".$clientID."</clientID>
					<getDueInovicesOnly>1</getDueInovicesOnly>
					<getInvoiceLineItemsData>1</getInvoiceLineItemsData>
					".$oid."
					<orderFormID>".((int)$_REQUEST["orderFormID"])."</orderFormID>
				</params>
			</mbapi>
		";
		require_once("lib-mbapi/mbapi.php");
		$invoices = dispatchMBAPI($invoiceXML, 1);
		$invoices = $invoices["mbapi"][0]["results"][0];
		if (isset($invoices["invoices"][0]["invoice"])){
			$invoices = correctArray($invoices["invoices"][0]["invoice"]);
		}
		$validInvoiceIDs = array();
		foreach ($invoices as $invoice){
			$validInvoiceIDs[] = $invoice["invoiceID"];
		}
		$newInvoiceIDs = array();
		if (isset($invoiceIDs)){
			foreach ($invoiceIDs as $v){
				if (in_array($v, $validInvoiceIDs)){
					$newInvoiceIDs[] = $v;
				}
			}
			$invoiceIDs = $newInvoiceIDs;
		} else {
			dbg("redirected prematurely");
			redirect("invoices.php?action=".getActionID("PayInvoices")."&sid=".secureInput($_REQUEST["sid"]));
			return 1;
		}

		$newInvoices = array();
		foreach ($invoices as $invoice){
			if (@in_array($invoice["invoiceID"], $invoiceIDs)){
				$newInvoices[] = $invoice;
				$lastInvoiceID = $invoice["invoiceID"];
			}
		}
		$invoices = $newInvoices;

		if ( 0 == count($invoices) ) {
			return $this->paymentError(blmsg("TRANS_INVOICES_ALREADY_PAID_FOR"));
		}

		require_once("object_managers/CryptManager.php");
		$cardNum = $cardCVV2 = null;
		if ( 0 == $billingAccountID ) {
			CryptManager::setDefaultKey();

			$cardCVV2 = ereg_replace("[^0-9]", "", $_REQUEST["cardCVV2"]);
			$cardNum = ereg_replace("[^0-9]", "", $_REQUEST["billingCardNum"]);
			$cardExpMonth = ereg_replace("[^0-9]", "", $_REQUEST["billingCardExpMonth"]);
			$cardExpYear = ereg_replace("[^0-9]", "", $_REQUEST["billingCardExpYear"]);
			$nameOnCard = $_REQUEST["billingCardNameOnCard"];
			$accountType = (int)$_REQUEST["billingAccountType"];
			$cardType = (int)$_REQUEST["billingCardType"];
			$cardStartDate = $_REQUEST["billingCardStartDate"];
			$cardIssueNum = $_REQUEST["billingCardIssueNum"];

			$billingAccountID = insertFor("billingAccounts", array(
				"clientContactID" => $contact->id,
				"billingAccountType" => getAPITypeID("billingAccount", "card"),
				"billingCardNameOnCard" => $nameOnCard,
				"billingCardType" => $cardType,
				"billingCardNum" => $cardNum,
				"billingCardExpMonth" => $cardExpMonth,
				"billingCardExpYear" => $cardExpYear,
				"billingCardIssueNum" => $cardIssueNum,
				"billingCardStartDate" => $cardStartDate,
			));
		} else if (CryptManager::MODE_LEK == CryptManager::getEncryptionMode()) {
			CryptManager::setKey(CryptManager::getLekPin());
		}

		if ( !isset($_REQUEST["orderFormID"]) ) {
			return $this->paymentError(blmsg("TRANS_PAYMENT_ERROR_MESSAGE", array("MESSAGE" => "Missing parameter 'orderFormID'")));
		}
		$orderForm = Factory()->OrderForm($_REQUEST["orderFormID"]);
		if ( is_null($orderForm->PrimaryGateway) && (is_null($orderForm->HSPCBankTransferGateway))) {
			return $this->paymentError(blmsg("TRANS_PAYMENT_ERROR_MESSAGE",
				array("MESSAGE" => "Order form #{$orderForm->id} '{$orderForm->order_form_name} has not primary and Bank Transfer payment gateway assigned'"))
			);
		}

		// create transaction
		$billingAccount = Factory()->ClientBillingAccount($billingAccountID);
		$transaction = Factory()->Transaction();
		require_once("objects/Transaction.php");
		$transaction->auth_return = Transaction::STATUS_PROCESSING;
		$transaction->transaction_amount = $paymentAmount;
		$transaction->currency_id = $orderForm->currency_id;
		$transaction->billing_account_id = $billingAccount->id;
		$transaction->invoiceIDs = $invoiceIDs;
		require_once("objects/ClientBillingAccount.php");
		switch ($billingAccount->billing_account_type) {
			case ClientBillingAccount::TYPE_CREDIT_CARD:
				$transaction->transaction_payment_method = Transaction::PAYMENT_CCSINGLE;
				$transaction->payment_gateway_id = $orderForm->PrimaryGateway->id;
				$transaction->transaction_notes = blmsg("TRANS_PAYMENT_TRANSACTION_PROCESSING_NOTE");
				if ("" != $cardNum) {
					$billingAccount->billing_card_num = $cardNum;
				}
				if ("" != $cardCVV2) {
					$billingAccount->cardCVV2 = $cardCVV2;
				}
				break;
			case ClientBillingAccount::TYPE_ECHECK:
				$transaction->transaction_payment_method = Transaction::PAYMENT_ECHECK;
				$transaction->payment_gateway_id = $orderForm->PrimaryGateway->id;
				$transaction->transaction_notes = blmsg("TRANS_PAYMENT_TRANSACTION_PROCESSING_NOTE");
				break;
			case ClientBillingAccount::TYPE_HSPC_BANK_TRANSFER:
				$transaction->transaction_payment_method = Transaction::PAYMENT_HSPCBANKTRANSFER;
				$transaction->payment_gateway_id = $billingAccount->PaymentGateway->id;
				$transaction->transaction_notes = blmsg("TRANS_HSPC_BATCH_FILE_TRANSACTION_INPROGRESS");
				break;
			default:
				throw new Exception("Unsuitable billing account type: {$billingAccount->billing_account_type}");
		}
		require_once("object_managers/TransactionManager.php");
		TransactionManager::update($transaction);

		// process transaction
		require_once("objects/PaymentGateway.php");
		switch ($transaction->PaymentGateway->payment_gateway_type) {
			case PaymentGateway::TYPE_PRIMARY:
			case PaymentGateway::TYPE_HSPC_CREDIT_CARD:
				require_once("object_managers/PaymentManager.php");
				PaymentManager::charge($transaction);
				break;
			case PaymentGateway::TYPE_HSPC_BANK_TRANSFER:
				require_once("object_managers/BatchFileManager.php");
				BatchFileManager::createTransaction($transaction->id);
				break;
			default:
				throw new Exception("Unsuitable payment gateway type: {$transaction->PaymentGateway->payment_gateway_type}");
		}

		$this->smartyVars["currencyID"] = $transaction->currency_id;
		$this->smartyVars["payTotal"] = $transaction->transaction_amount;
		$this->smartyVars["invoices"] = $invoices;
		$this->smartyVars["authorizationCode"] = $transaction->auth_code;
		$this->smartyVars["transactionID"] = $transaction->id;
		$this->smartyVars["paymentNotes"] = $transaction->transaction_notes;

		switch ($transaction->auth_return) {
			case Transaction::STATUS_SUCCESSFUL_CHARGE:
				require_once("object_managers/PaymentManager.php");
				PaymentManager::takePayment($transaction);
				$this->smartyVars["paymentStatus"] = blmsg("TRANS_PAYMENT_STATUS_ACCEPTED");
				break;
			case Transaction::STATUS_PROCESSING:
				$this->smartyVars["paymentStatus"] = blmsg("TRANS_PAYMENT_STATUS_PROCESSING");
				break;
			case Transaction::STATUS_DECLINED_CHARGE:
				$this->smartyVars["paymentStatus"] = blmsg("TRANS_PAYMENT_STATUS_DECLINED");
				break;
			default:
				$this->smartyVars["paymentStatus"] = blmsg("TRANS_PAYMENT_STATUS_ERRORED");
				break;
		}

		$this->addTemplate("invoices/PayInvoicesResponse");
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	private function paymentError($error_text)
	{
		$this->smartyVars["paymentError"] = 1;
		$this->smartyVars["paymentErrorText"] = $error_text;
		$this->hasExecutedSuccessfully = ((int) $this->addTemplate("invoices/PayInvoicesResponse", $this->smartyVars));
		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}
}
