<?php

require_once("lib-action/action.php");
require_once("UI/Page.php");

class ShowClientServiceInputFormPage extends UIPage
{
	public function __construct($controller)
	{
		parent::__construct($controller);

		$this->package = Factory()->Package(bgpc("packageID"));

		switch (getAPITypeName("products", $this->package->package_type))
		{
			case "ssl-cert":
				require_once mb_path("/lib-mbapi/include/modules/sslmodule/sslmodule.php");
				$this->module = SSLModule::getPluginInstance(
					$this->package->PackageAttributes["ssl_plugin_id"]->value,
					array("packageID" => $this->package->package_id)
				);
				break;
			case "hosting":
				$panel = $this->package->ControlPanel;
				require_once("ModuleTool.php");
				$this->module = ModuleTool::getModuleInstance("panel", $panel->module_name, array(
					"serverGroupID" => $panel->server_group_id,
					"configGroupID" => $panel->id,
					"packageID" => $this->package->id,
					"internalPackageID" => $this->package->id,
				));
				break;
			default:
				throw new Exception(
					"User Input for package (packageID: {$this->package->package_id}) " .
					"with the product type \"".getAPITypeName("products", $this->package->package_type)."\" " .
					"is not supported"
				);
		}
		$this->form = $this->getDefaultForm();
	}

	public function createControls()
	{
		$domain = is_null($this->package->PrimaryDomain)
			? ""
			: " ({$this->package->PrimaryDomain->domain_sld}.{$this->package->PrimaryDomain->domain_tld})";

		$this->title = blmsg("TRANS_EDIT_PACKAGE_USER_INPUT", array(
			"packageID" => $this->package->package_id,
			"packageName" => $this->package->package_name . $domain
		));
		$this->parent_title = blmsg("TRANS_PACKAGES");
		$this->parent_url = action_url("packages", "ShowPackages");

		try {
			$this->module->addUserParams($this->form);
		} catch(ProductException $e) {
			$this->handleException($e);
		}
	}

	public function processInput()
	{
		foreach ($this->module->collectUserParams($this->form) as $param_name => $param_value)
		{
			$this->package->PackageAttributes[$param_name]->value = $param_value;
		}
		require_once("object_managers/PackageManager.php");
		$this->package->package_status = getAPIStatusID("package", "pending");
		PackageManager::update($this->package);

		$this->module->reloadAttributes();
		if (0 < count($errors = $this->module->checkAttributes("add")))
		{
			$this->addErrorMessage(UIFactory()->ErrorMessage($errors));
			return false;
		}

		$queue = Factory()->SystemQueueList(array(
			"package_id" => $this->package->package_id,
			"system_queue_status" => array(
				getAPIStatusID("queue", "new"),
				getAPIStatusID("queue", "rule"),
			),
		));
		foreach ($queue as $queue_item)
		{
			$queue_item->system_queue_status = getAPIStatusID("queue", "force");
			$queue_item->update();

			try {
				dispatchMBAPI(array(
					"mbapi" => array(
						"command" => "ProcessSystemQueue",
						"params" => array(
							"runMax" => 1,
							"systemQueueID" => $queue_item->system_queue_id,
						),
					),
				));
			} catch(ProductException $e) {
				$this->handleException($e);
			}
		}
		getCurrentSession()->Storage->infoMessages[] = blmsg("TRANS_SSL_MODULE_CUSTOMER__USER_INPUT_DONE_MSG");
		return true;
	}

	private function handleException(ProductException $e)
	{
		require_once("ErrorHandlersManager.php");
		$err_num = ErrorHandlersManager::storeExceptionAsCrash($e)->number;
		getCurrentSession()->Storage->errorMessages[] = blmsg("TRANS_SSL_MODULE_CUSTOMER__USER_INPUT_ERROR_MSG", array(
			"ERROR_NUMBER" => $err_num,
			"SUBSCRIPTION_ID" => $this->package->id,
		));
		redirect(action_url("dashboard", "ShowDashboard"));
	}

	private $package;
	private $package_attrs;
	private $plugin;
	private $form;
}

class ShowClientServiceInputForm extends MB_Action
{
	public function execute()
	{
		parent::execute();
		if ( is_null($packageID = bgpc("packageID")) ) {
             throw new Exception("Missing parameter 'packageID'");
		}
		$controller  = UIFactory()->PageController(action_url("dashboard", "ShowDashboard"));
		$controller->addPage(new ShowClientServiceInputFormPage($controller));
		$controller->processRequest();

		$this->addTemplate("page", array("page" => $controller->getPage(), "showScreenTitle" => true));
		return $this->hasExecutedSuccessfully();
	}
}
