<?php

require_once(dirname(__FILE__)."/../../../../lib-tk/shield.php");
require_once("lib-action/genericactions.php");

class EditBillingAccountResponse extends EditResponseAction
{
	public function __construct()
	{
		require_once("object_managers/CryptManager.php");
		CryptManager::setDefaultKey();

		global $CONF;
		$clientContactID = $CONF["user"]["clientContactID"];
		$clientID = (int)$CONF["user"]["id"];
		$contacts = arrayFor(":contacts", array(
			"clientID" => $clientID,
		));

		$keyedContacts = array();
		foreach ($contacts as $x=>$c){
			$keyedContacts[$c["clientContactID"]] =& $contacts[$x];
		}

		// if we are the primary contact and we can edit the billing accounts of
		// any contact for this client, else just edit our own
		if ((int)$keyedContacts[$clientContactID]["contactPrimary"] && is_array($keyedContacts[(int)$_REQUEST["clientContactID"]])){
			$clientContactID = (int)$_REQUEST["clientContactID"];
		}
		$_REQUEST["clientContactID"] = $clientContactID;

		if(!$currentBillingAccount = oneOf("billingAccounts", array("clientContactID" => $clientContactID, "billingAccountID" => (int) $_REQUEST["billingAccountID"]))) {
			$this->addError(MBAPI_ACTIONERROR, "No valid match.");
			return false;
		}

		loadConfigs("payment");

		$this->d = new DataSource("billingAccounts");
		$description = $this->d->getDescription();
		$description["names"]["file"] = "profile";
		parent::EditResponseAction();
		unset($description['columns']['clientContactID']);

		$success_validation = true;

		$billingAccountType = (int) $currentBillingAccount["billingAccountType"];
		if($billingAccountType == getAPITypeID("billingAccount", "manual") || $billingAccountType == getAPITypeID("billingAccount", "secondary")) {
			unset($description["columns"]['billingBankType']);
			unset($description["columns"]['billingBankName']);
			unset($description["columns"]['billingBankState']);
			unset($description["columns"]['billingBankABACode']);
			unset($description["columns"]['billingBankAccountNum']);
			unset($description["columns"]['billingBankLicenseNum']);
			unset($description["columns"]['billingBankLicenseState']);
			unset($description["columns"]['billingBankDob']);

			unset($description["columns"]['billingCardNameOnCard']);
			unset($description["columns"]['billingCardType']);
			unset($description["columns"]['billingCardNum']);
			unset($description["columns"]['billingCardExpMonth']);
			unset($description["columns"]['billingCardExpYear']);
			unset($description["columns"]['billingCardIssueNum']);
			unset($description["columns"]['billingCardStartDate']);
		} else if($billingAccountType == getAPITypeID("billingAccount", "card")) {
			unset($description["columns"]['billingBankType']);
			unset($description["columns"]['billingBankName']);
			unset($description["columns"]['billingBankState']);
			unset($description["columns"]['billingBankABACode']);
			unset($description["columns"]['billingBankAccountNum']);
			unset($description["columns"]['billingBankLicenseNum']);
			unset($description["columns"]['billingBankLicenseState']);
			unset($description["columns"]['billingBankDob']);

			$billingCardType = (int) $_REQUEST["billingCardType"];
			$cardExtraTypes = getAPITypes("cardExtraIDs");
			if(!in_array($billingCardType, $cardExtraTypes)) {
				unset($description["columns"]['billingCardIssueNum']);
				unset($description["columns"]['billingCardStartDate']);
			}
			if (!$_REQUEST["billingCardNum"] || $_REQUEST["billingCardNum"]==""){
				unset($description["columns"]['billingCardNum']);
			}
		} else {
			unset($description["columns"]['billingCardNameOnCard']);
			unset($description["columns"]['billingCardType']);
			unset($description["columns"]['billingCardNum']);
			unset($description["columns"]['billingCardExpMonth']);
			unset($description["columns"]['billingCardExpYear']);
			unset($description["columns"]['billingCardIssueNum']);
			unset($description["columns"]['billingCardStartDate']);
		}
		$this->d->setDescription($description);
		switch ($billingAccountType)
		{
			case getAPITypeID("billingAccount", "card"):
				$validation_fields = array(
					"billingCardNameOnCard" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"billingCardExpMonth" => INPUT_RESTRICTION_MONTH|INPUT_RESTRICTION_REQUIRED,
					"billingCardExpYear" => INPUT_RESTRICTION_YEAR|INPUT_RESTRICTION_REQUIRED,
					"billingCardNum" => array(
						INPUT_RESTRICTION_INT|INPUT_RESTRICTION_MAXLEN|INPUT_RESTRICTION_MINLEN|INPUT_RESTRICTION_CARD,
						array(
							'maxLength' => 19,
							'minLength' => 13,
							'allowedCardTypes' => array(getAPITypeName("card", bgpc('billingCardType'))),
						),
					),
                );
				break;
			case getAPITypeID("billingAccount", "echeck"):
			case getAPITypeID("billingAccount", "bank"):
			case getAPITypeID("billingAccount", "wire"):
				$validation_fields = array(
					"billingBankType" => INPUT_RESTRICTION_UNSIGNED_INT,
					"billingBankName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
					"billingBankState" => INPUT_RESTRICTION_STRING,
					"billingBankAbaCode" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
					"billingBankAccountNum" => INPUT_RESTRICTION_UNSIGNED_INT,
					"billingBankLicenseNum" => INPUT_RESTRICTION_STRING,
					"billingBankLicenseState" => INPUT_RESTRICTION_STRING,
                );
				break;
			case getAPITypeID("billingAccount", "hspcbanktransfer"):
				require_once("ModuleTool.php");
				$gateway_id = bgpc("selected_ddbp_gateway_id");
				$plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array("gateway_id" => $gateway_id));
				$billing_account = Factory()->ClientBillingAccount(bgpc("billingAccountID"));
				$billing_account->payment_gateway_id = $gateway_id;
				$adapter = $plugin->getPaymentMethodAdapter($billing_account);
				$adapter->collectInputFields();
				$adapter_input_fields = $adapter->getInputFields();
				$validation_fields = array();
				foreach (array_keys($adapter_input_fields) as $field_name) {
					$validation_fields[$field_name] = INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED;
				}
				$adapter_validation = $adapter->validate();
				if (HSPC_PM_Adapter::VALIDATION_OK != $adapter_validation) {
					$success_validation = false;
					billing_put_error($adapter_validation);
				}
				break;
		}
		$this->setReferralActionID(getActionID("EditBillingAccount"));
		$this->setReferralActionFile("profile.php");
		$args = array("billingAccountID" => bgpc("billingAccountID"), "billingAccountType" => $billingAccountType, "selected_ddbp_gateway_id" => $gateway_id);
		$success_validation = $this->validateMultiInputs( $validation_fields, array(), array(), 1, $args ) && $success_validation;
		if (! $success_validation) {
			$this->validationRedirect($args, array(), true); //Force redirect
			exit;
		}
	}

	public function editResponseHandler()
	{
		global $CONF;
		// valid match assumed due to earlier check
		$billingAccount = Factory()->ClientBillingAccount(bgpc("billingAccountID"));

		$contact = oneByID(":contacts", $CONF["user"]["clientContactID"]);
		if ((int)$contact['contactPrimary'] && (int)$_REQUEST['defaultBillingAccount']){
			updateForKey(":clientsGeneric", $CONF['user']['id'], array(
				"defaultBillingAccountID" => $billingAccount->id,
			));
		}

		if (bgpc("billingAccountType") == ClientBillingAccount::TYPE_HSPC_BANK_TRANSFER) {
			require_once("ModuleTool.php");
			$gateway_id = bgpc("selected_ddbp_gateway_id");
			$plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array("gateway_id" => $gateway_id));
			$billingAccount->payment_gateway_id = $gateway_id;
			$adapter = $plugin->getPaymentMethodAdapter($billingAccount);
			$adapter->collectInputFields();
			$adapter->save();
		}

		$shares = arrayFor(":clientBillingAccountShares", array(
			"billingAccountID" => $billingAccount->id,
		));

		$packageIDs = array();
		foreach ((array)$_REQUEST["packages"] as $pid){
			$packageIDs[$pid] = 1;
		}

		foreach ((array)$shares as $share){
			if ($packageIDs[$share["packageID"]]){
				updateForKey(":clientBillingAccountShares", $share["billingAccountShareID"], array(
					"billingAccountShareValue" => 1,
				));
				unset($packageIDs[$share["packageID"]]);
			} else {
				updateForKey(":clientBillingAccountShares", $share["billingAccountShareID"], array(
					"billingAccountShareValue" => 0,
				));
			}
		}

		$_REQUEST["packages"] = array_keys($packageIDs);

		foreach((array) $_REQUEST["packages"] as $packageID) {
			insertFor(":clientBillingAccountShares", array(
				"billingAccountID" => $billingAccount->id,
				"billingAccountShareGroup" => "0",
				"billingAccountFallbackGroup" => "1",
				"billingAccountShareValue" => "1",
				"packageID" => (int) $packageID,
			));
		}
		return true;
	}

	public function overrideValidateColumnParamType($column, $columnType, $mode, $paramType, $isRequired)
	{
		switch($column) {
			case "billingCardNum":
				if(empty($_REQUEST["billingCardNum"])) {
					$columnType = "";
					break;
				}
				$paramType |= INPUT_RESTRICTION_MAXLEN|INPUT_RESTRICTION_MINLEN|INPUT_RESTRICTION_CARD;
				$args = array(
					'maxLength' => 19,
					'minLength' => 13,
					'allowedCardTypes' => array(getAPITypeName("card", $_REQUEST['billingCardType'])),
				);
				break;
			case "billingBankAccountNum":
				if(empty($_REQUEST["billingBankAccountNum"])) {
					$columnType = "";
					break;
				}
				break;
			default:
				$args = array();
				break;
		}

		return array($paramType, $isRequired, $args);
	}
}