<?php

require_once(dirname(__FILE__)."/../../../../lib-tk/shield.php");
require_once("lib-action/action.php");

class EditClientContact extends MB_Action
{
	public function execute()
	{
		global $CONF;
		parent::execute();

		if ( !$this->limitParams() ) {
			$this->addError(blmsg("TRANS_ACTIONERROR"), "Invalid parameters");
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		require_once("lib-mbapi/mbapi.php");
		$query = $this->getMBAPIQuery("GetClients", "", $this->limitParams + array("getContactData" => "4", "mergePrimaryContact" => 0));
		$data = dispatchMBAPI($query, 1);
		$results =& $data["mbapi"][0]["results"][0];

		if ( 0 !== $this->addMBAPIErrors($data) || 0 < $this->getNumErrors() ) {
			$this->addErrorTemplates();
			return $this->hasExecutedSuccessfully();
		}

		loadConfigs("userSettings");

		$this->smartyVars["userSettings"] =& $CONF["userSettings"]['display'];

		$this->mergeCountryData();
		$this->mergeLanguageData();
		$client = correctArray($results["clients"][0]["client"][0]);
		$contact = correctArray($results["clients"][0]["client"][0]["contacts"][0]["contact"][0]);
		$this->smartyVars["client"] = $client;
		$this->smartyVars["contact"] = $contact;
		$this->smartyVars["clientContactIDEditor"] = $CONF["user"]["clientContactID"];
		$this->smartyVars["clientContactID"] = getParam("clientContactID");

		require_once("object_managers/SecurityManager.php");
		SecurityManager::pushCurrentActionIntoLog(
			SecurityManager::USER_TYPE_CLIENT,
			$client["clientID"],
			Factory()->Client($client["clientID"])->PrimaryContact->contact_username,
			"TRANS_ACTION_LOG_REASON_SHOW_USER_DETAILS",
			blmsg(
				"TRANS_ACTION_LOG_COMMENT_CLIENT_CONTACT",
				array("CLIENT_ID" => $client["clientID"], "CONTACT_ID" => $contact["clientContactID"]),
				SecurityManager::getActionLogLocale()
			)
		);

		$selectedEmailActionIDs = array();
		if ($contact["emailActionIDs"])
		{
			if ($contact["emailActionIDs"][0]
				&& is_array($contact["emailActionIDs"][0])
				&& isset($contact["emailActionIDs"][0]["emailActionID"]))
			{
				$selectedEmailActionIDs = array();
				foreach ($contact["emailActionIDs"] as $emailAction) {
					$selectedEmailActionIDs[] = $emailAction["emailActionID"];
				}
			} else {
				$selectedEmailActionIDs = $contact["emailActionIDs"];
			}
		}
		$this->smartyVars["selectedEmailActionIDs"] = $selectedEmailActionIDs;

		$ead = new DataSource("contactEmailActionRels", array("clientContactID" => $this->smartyVars["clientContactID"]));
		$t =& $ead->getTable();
		$contact["emailActionIDs"] = $t->getColumnDistinct("emailActionID");

		require_once("lib-billing/include/config/internal/emailactions.php");
		$this->smartyVars["emailActions"] = blmsg_array($CONF["types"]["emailAction"]);

		$formDefaults = $contact;
		$this->mergeFormState($formDefaults);

		if (count(arrayFor('clientContacts', array('clientID' => $this->smartyVars['client']['clientID']))) == 1) {
			$this->smartyVars['onlyPrimaryExists'] = 1;
		} else {
			$this->smartyVars['onlyPrimaryExists'] = 0;
		}
		$this->smartyVars["currentLoggedInUser"] =& $CONF["user"];

		if ( 0 == $contact["contactPrimary"] && $contact["contactUsername"] != $contact["contactEmail"] ) {
			$this->smartyVars["warningMessages"][] = blmsg("TRANS_CLIENT_CONTACT_EMAIL_USERNAME_DIFFERENT_CONTACT_MSG");
			$this->smartyVars["highlightContactEmail"] = true;
		}

		require_once("object_managers/SecurityManager.php");
		if (SecurityManager::isPasswordExpired())
			$this->smartyVars["warningMessages"][] = blmsg("TRANS_CLIENT_CONTACT_PASSWORD_WAS_EXPIRED_NEED_TO_CHANGE", array("DAYS"  => Settings()->security_PasswordExpirationInterval));

		$this->addTemplate("profile/EditClientContact", $this->smartyVars);

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();
	}

	private function limitParams()
	{
		global $CONF;
		$clientContact = Factory()->ClientContact(getCurrentSession()->user_id);
		$this->clientContactID = $clientContact->client_contact_id;
		$this->contextContactID = is_null($clientContactID = bgpc("clientContactID")) ? $this->clientContactID : $clientContactID ;
		$this->contact = oneByID("contacts", $this->clientContactID);
		if (1 == $clientContact->contact_primary ) {
			$this->limitParams = array(
				"clientID" => $clientContact->client_id,
				"clientContactID" => $this->contextContactID,
			);
			$_REQUEST["clientID"] = $clientContact->client_id;
			$_REQUEST["clientContactID"] = $this->contextContactID;
		} else {
			$this->limitParams = array(
				"clientContactID" => $this->clientContactID,
			);
			$_REQUEST["clientContactID"] = $this->clientContactID;
		}
		$this->contextContact = oneOf(
			"contacts",
			array(
				"clientID" => $clientContact->client_id,
				"clientContactID" => $this->contextContactID,
			)
		);
		return (bool) $this->contextContact;
    }
}
