<?php

require_once(dirname(__FILE__)."/../../../../lib-tk/shield.php");
require_once("lib-action/action.php");

class EditClientContactResponse extends MB_Action
{
	public $_clientContact;

	public function __construct()
	{
		parent::__construct();

		loadConfigs("userSettings");

		// set all possible xml params here, $_REQUEST is already cleaned
		$this->params = array
		(
			"clientContactID",
			"contactFirstName",
			"contactLastName",
			"contactEmail",
			"contactUsername",
			"contactEmailFormat",
			"contactCompany",
			"contactAddress1",
			"contactAddress2",
			"contactCity",
			"contactState",
			"contactZip",
			"countriesID",
			"languageID",
			"contactPhone1",
			"contactPhone2",
			"contactFax",
			"contactMobilePhone",
			"emailActionIDs",
			"selectableEmailActions",
			"contactActive",
			"contactUseSpecifiedEmailRulesOnly"
		);

		/**
		 *	unset illegal key values that we don't want on the back end
		 */
		$tmp = array_flip($this->params);
		$this->unsetIllegalKeys( $tmp );
		$this->params = array_flip($tmp);
	}

	public function execute()
	{
		global $CONF;
		parent::execute();

		$currentUser = oneByID("contacts", $CONF["user"]["clientContactID"]);
		$clientContactID = bgpc("clientContactID");

		$requestedClientContact = oneByID('contacts',$_REQUEST['clientContactID']);

		if (!$currentUser["contactPrimary"] && $requestedClientContact['clientContactID'] != $currentUser['clientContactID']) {
			$this->addError(TRANS_ACTIONERROR, "Only the primary contact can add/edit others' contacts.");
		} elseif (!notZero($clientContactID)) {
			$this->addError(TRANS_ACTIONERROR, "Client Contact ID is not given.");
		} elseif ($CONF['user']['id'] != $requestedClientContact['clientID']) {
			$this->addError(TRANS_ACTIONERROR, "Requested client Contact ID does not belong to logged in client.");
	    } else {

			$this->setReferralActionID(getActionID("EditClientContact"));
			$this->setReferralActionFile("profile.php");
			$args = array( "clientContactID" => $clientContactID );
			$postedValues = array(
				"clientContactID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
				"contactFirstName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"contactLastName" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"contactPassword" => array(INPUT_RESTRICTION_PASSWORD, array("confirm" => "contactPasswordConfirm")),
				"contactEmail" => INPUT_RESTRICTION_EMAIL|INPUT_RESTRICTION_REQUIRED,
				"contactEmailFormat" => INPUT_RESTRICTION_BOOL|INPUT_RESTRICTION_REQUIRED,
				"contactCompany" => INPUT_RESTRICTION_STRING,
				"contactAddress1" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"contactAddress2" => INPUT_RESTRICTION_STRING,
				"contactCity" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"contactState" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"contactZip" => INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED,
				"countriesID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
				"languageID" => INPUT_RESTRICTION_UNSIGNED_INT|INPUT_RESTRICTION_REQUIRED,
				"contactPhone1" => INPUT_RESTRICTION_PHONE|INPUT_RESTRICTION_REQUIRED,
				"contactPhone2" => INPUT_RESTRICTION_PHONE,
				"contactFax" => INPUT_RESTRICTION_PHONE,
				"contactMobilePhone" => INPUT_RESTRICTION_PHONE,
				"contactUseSpecifiedEmailRulesOnly" => INPUT_RESTRICTION_BOOL
			);
			if ( 1 == $requestedClientContact['contactPrimary'] ) {
				$postedValues["contactUsername"] = INPUT_RESTRICTION_STRING|INPUT_RESTRICTION_REQUIRED;
			}
			/**
			 *	unset illegal key values that we don't want on the back end
			 */
			$this->unsetIllegalKeys( $postedValues );

			$success = $this->validateMultiInputs(
					$postedValues,
					array("contactPassword","contactPasswordConfirm"),
					array(),
					1,
					$args
					);
			if($success) {
				unset($_REQUEST["contactPasswordConfirm"]);

				// make sure no other exists with the same username

				if ($_REQUEST["contactUsername"]) {
					if ($clientContactID != $CONF["user"]["clientContactID"]) {
						if($contactByName = oneByName(":contacts", $_REQUEST["contactUsername"])) {
							if($contactByName["clientContactID"] != $clientContactID) {
								$this->addError(TRANS_ACTIONERROR, "Another user by the same username already exists.");
							}
						}
					}
				}

				if (!$this->getNumErrors()) {
					// only update the password if set
					unset($_REQUEST["contactPasswordConfirm"]);
					if ($_REQUEST["contactPassword"]=="") {
						unset($_REQUEST["contactPassword"]);
					} else {
						$this->params[] = "contactPassword";
					}

					if ( 0 == $requestedClientContact['contactPrimary'] ) {
						// we set username equal to email forcedly for no primary contacts
						$contactEmail = in_array("contactEmail", $this->params) ? bgpc("contactEmail") : $requestedClientContact['contactEmail'];
						$this->params[] = "contactUsername";
						$_REQUEST["contactUsername"] = $contactEmail;
					}

					$xmlParams = "";
					foreach ($this->params as $paramName) {
						if ($paramName == "emailActionIDs"){
							$xmlParams .= "<emailActionIDs>\n";
							if(is_array($_REQUEST[$paramName])) {
								foreach($_REQUEST[$paramName] as $key=>$eaid){
									$xmlParams .= "<emailActionID>".(int)$eaid."</emailActionID>\n";
								}
							}
							$xmlParams .= "</emailActionIDs>\n";
						} else {
							$xmlParams .= "<".$paramName.">".utf8_xml_entity_encode(bgpc($paramName))."</".$paramName.">\n";
						}
					}
					if ((int)$_REQUEST['contactUseSpecifiedEmailRulesOnly']){
						// added to forcefully make SetClients remove all email actions right before setting the selected ones
						$xmlParams .= "<removeAllEmailActionIDs>1</removeAllEmailActionIDs>\n";
					}
					// fetch client ID for this contact first
					$query = "
					<mbapi>
					<command>GetClients</command>
						<params>
							<clientContactID>$clientContactID</clientContactID>
						</params>
					</mbapi>
					";
					require_once("lib-mbapi/mbapi.php");
					$data = dispatchMBAPI($query, 1);

					$clientID = $data["mbapi"][0]["results"][0]["clients"][0]["client"][0]["clientID"][0];

					// create sanatized xml command
					$query = "
					<mbapi>
					<command>SetClients</command>
					<subCommand>update</subCommand>
						<params>
							<clientID>$clientID</clientID>
							<clientContacts>
								<clientContact>
									$xmlParams
								</clientContact>
							</clientContacts>
						</params>
					</mbapi>
					";

					$data = dispatchMBAPI($query, 1);

					$results =& $data["mbapi"][0]["results"][0];
					if($this->addMBAPIErrors($data) === 0 && !$this->getNumErrors()) {
						if ($_REQUEST["redirectURL"]!="" && $_REQUEST["redirectAction"]!="") {
							redirect($_REQUEST["redirectURL"]."?action=".getActionID($_REQUEST["redirectAction"])."&".$_REQUEST["redirectArgs"]);
						} elseif( 1 == Factory()->ClientContact(getCurrentSession()->user_id)->contact_primary ) {
							redirect("profile.php", getActionID("ShowClientContacts"));
						} else {
							redirect("dashboard.php", getActionID("ShowDashboard"));
						}
					}
				}
			}
		}

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();

	}

	private function unsetIllegalKeys( &$array )
	{
		global $CONF;

		if (!notNull( $this->_clientContact )) {
			$this->_clientContact = oneOf('clientContacts', array('clientContactID' => (int)$_REQUEST['clientContactID']));
			$this->_clientContact['clientContactCount'] = count(arrayFor('clientContacts', array('clientID' => (int)$_REQUEST['clientID'])));
		}

		if ($this->_clientContact['clientContactCount'] == 1) {
			unset($array['contactActive']);
		}

		foreach( (array)$CONF["userSettings"]["display"] as $k => $v ) {
			if ($v==1)
				continue;

			switch( $k ) {
			case "editUserName":
				unset($array["contactUsername"]);
				break;
			case "editFirstName":
				unset($array["contactFirstName"]);
				break;
			case "editLastName":
				unset($array["contactLastName"]);
				break;
			case "editEmail":
				unset($array["contactEmail"]);
				break;
			case "editCompanyName":
				unset($array["contactCompany"]);
				break;
			case "editCountry":
				unset($array["countriesID"]);
				break;
			case "editEmailActions":
				unset($array["emailActionIDs"]);
				unset($array["selectableEmailActions"]);
				break;
			}
		}	unset( $k, $v );
	}
}
