<?php
require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";

/**
 * This file maintains all information having to do with client and administrative actions. It is
 * basically just a big set of action names, keyed on a unique integer. This entire setup exists
 * to maintain higher overall security, and although not intentional, some level of security via
 * ambiguity. Always use the functions located at the bottom of this file to access this array,
 * never access it directly.
 */

/**
 * This is the master regex used to split all actions into like groups. The key is the translated
 * name of the action group. The value is the regex argument used to parse and filter actions to
 * their groups.
 *
 * The interface will use this grouping only for DISPLAY, not for functionality.
 */
$actionGroupRegex = array
(
    "TRANS_ACCOUNTING_ACTIONS" => "accounting|journal|ledger|profit|period|finance",
    "TRANS_ADMIN_ACTIONS" => "admin|profile",
    "TRANS_BOOKMARK_ACTIONS" => "bookmark",
    "TRANS_CERT_ACTIONS" => "cert",
    "TRANS_CLIENT_ACTIONS" => "client|billing|manualpayment",
    "TRANS_COMPANIES_ACTIONS" => "compan",
    "TRANS_CONFIG_ACTIONS" => "config|gateway|registrar",
    "TRANS_COUPON_ACTIONS" => "coupon",
    "TRANS_CYCLE_ACTIONS" => "cycle",
    "TRANS_DASHBOARD_ACTIONS" => "dashboard",
    "TRANS_DOMAIN_ACTIONS" => "domain",
    "TRANS_EMAIL_ACTIONS" => "email",
    "TRANS_EVENT_ACTIONS" => "event",
    "TRANS_FAQ_ACTIONS" => "faq",
    "TRANS_FRAUD_ACTIONS" => "fraud",
    "TRANS_GRAPH_ACTIONS" => "graph",
    "TRANS_CRASH_REPORTS" => "crashes",
    "TRANS_HELPDESK_ACTIONS" => "helpdesk",
    "TRANS_INVOICE_ACTIONS" => "invoice",
    "TRANS_LOG_ACTIONS" => "log",
    "TRANS_LOGIN_ACTIONS" => "login|forgot|logout",
    "TRANS_MIGRATION_ACTIONS" => "migrate",
    "TRANS_NEWS_ACTIONS" => "news",
    "TRANS_WIZARD_ACTIONS" => "wizard",
    "TRANS_ORDERFORM_ACTIONS" => "order|cart|question",
    "TRANS_PACKAGE_ACTIONS" => "package",
    "TRANS_PAYMENTGROUP_ACTIONS" => "paymentgroup",
    "TRANS_PAYMENTQUEUE_ACTIONS" => "paymentqueue",
    "TRANS_PPC_ACTIONS" => "ppc",
    "TRANS_PRODUCT_ACTIONS" => "product|usage|price|attribute",
    "TRANS_REPORTS_ACTIONS" => "report",
    "TRANS_SERVER_ACTIONS" => "server|panel|module",
    "TRANS_SYSTEM_ACTIONS" => "system|search",
    "TRANS_TAX_ACTIONS" => "tax",
    "TRANS_THEME_ACTIONS" => "theme",
    "TRANS_TLD_ACTIONS" => "tld",
    "TRANS_TODO_ACTIONS" => "todo",
    "TRANS_TRANSACTION_ACTIONS" => "transaction|check|balance|deposit|credit|fund",
    "TRANS_WEBSITEWIDGETS_ACTIONS" => "websitewidget",
    "TRANS_BATCH_FILES_MANAGER" => "batchfile",
);

/**
 * List all valid client action names. If the action name is not listed here, a client will
 * be denied access.
 *
 * @author Michael Fountain
 */
$CONF["validClientActions"] = array
(
	// client actions from login.php
	"ShowLogin",
	"LoginResponse",
	"LogoutResponse",
	"InitSSOLogin",
	"InitSSOLogout",
	"ProcessSSOAPIRequest",
	"ReceiveSSOLoginStatus",
	"SelectSSOLoginMethod",

	// actions for SSO IdP
	"ShowIDPFIAttach",
	"ShowIDPFIConfirmDelete",
	"ShowIDPFIConfirmDetach",
	"ShowIDPFIManagement",
	"ShowIDPFIUpdate",
	"ShowIDPLogin",
	"ShowIDPSelectName",

	// client actions from dashboard.php
    "ShowDashboard",
    "PayInvoices",
    "PayInvoicesResponse",

	//Helpdesk actions
	"AddHelpdeskTicket",
	"AddHelpdeskTicketResponse",
	"ShowHelpdeskTicketDetails",
	"EditHelpdeskTicketResponse",

);

$CONF["withoutEULAActions"] = array
(
	"ShowLogin",
	"LoginResponse",
	"LogoutResponse",
	"InitSSOLogin",
	"InitSSOLogout",
	"SelectSSOLoginMethod",
	"ShowForgottenPassword",
	"ShowCrashReports",
	"GotoCrashReport",
	"RemoveCrashReports",
	"EditEulaConfigrmation",
	"EditInitialConfiguration"
);

$CONF["actions"] = array(
	0 => "APILoginResponse",
	1 => "ActivatePackage",
	2 => "ActivatePackageAddon",
	3 => "ActivatePackageResponse",
	4 => "AddAdmin",
	5 => "AddAdminGroup",
	6 => "AddAdminGroupResponse",
	7 => "AddAdminResponse",
	8 => "AddAttribute",
	9 => "AddAttributeGroup",
	10 => "AddAttributeGroupResponse",
	11 => "AddAttributeMap",
	12 => "AddAttributeMapResponse",
	13 => "AddAttributeMapping",
	14 => "AddAttributeMappingResponse",
	15 => "AddAttributeResponse",
	16 => "AddBillingAccount",
	17 => "AddBillingAccountResponse",
	18 => "AddBookmark",
	19 => "AddBookmarkResponse",
	20 => "AddCartCoupon",
	21 => "AddCartCouponResponse",
	22 => "AddCartDomain",
	23 => "AddCartDomainResponse",
	24 => "AddCartProduct",
	25 => "AddCartProductResponse",
	26 => "AddCert",
	27 => "AddCertResponse",
	28 => "AddCheck",
	29 => "AddCheckResponse",
	30 => "AddClient",
	31 => "AddClientContact",
	32 => "AddClientContactResponse",
	33 => "AddClientCreditResponse",
	34 => "AddClientResponse",
	35 => "AddCompany",
	36 => "AddCompanyResponse",
	37 => "AddCoupon",
//	38 => "AddCouponResponse",
	39 => "AddCustomClientAttribute",
	40 => "AddCustomClientAttributeResponse",
	41 => "AddCustomProductAttribute",
	42 => "AddCustomProductAttributeResponse",
	43 => "AddCycle",
	44 => "AddCycleDiscount",
	45 => "AddCycleDiscountGroup",
	46 => "AddCycleDiscountGroupResponse",
	47 => "AddCycleDiscountResponse",
	48 => "AddCycleGroup",
	49 => "AddCycleGroupResponse",
	50 => "AddCycleResponse",
	51 => "AddDeposit",
	52 => "AddDepositResponse",
	53 => "AddDomain",
	54 => "AddDomainResponse",
	55 => "AddEmailTemplate",
	56 => "AddEmailTemplateGroup",
//	57 => "AddEmailTemplateGroupResponse",
	58 => "AddEmailTemplateResponse",
	59 => "AddFAQ",
	60 => "AddFAQCategory",
	61 => "AddFAQCategoryResponse",
	62 => "AddFAQResponse",
	63 => "AddFraudCountryThresholdResponse",
	64 => "AddFraudSetResponse",
	65 => "AddGeneralJournalEntry",
	66 => "AddGeneralJournalEntryResponse",
	67 => "AddGenericAttribute",
	68 => "AddGenericAttributeResponse",
	69 => "AddGroupAttribute",
	70 => "AddGroupAttributeResponse",
	71 => "AddHelpdeskBucket",
	72 => "AddHelpdeskBucketResponse",
	73 => "AddHelpdeskDepartment",
	74 => "AddHelpdeskDepartmentResponse",
	75 => "AddHelpdeskResponseTemplate",
	76 => "AddHelpdeskResponseTemplateResponse",
	77 => "AddHelpdeskTicket",
	78 => "AddHelpdeskTicketResponse",
	79 => "AddHistoricalPackage",
	80 => "AddHistoricalPackageMenu",
	81 => "AddHistoricalPackageResponse",
	82 => "AddIPLog",
	83 => "AddIPLogResponse",
	84 => "AddInvalidIpnLog",
	85 => "AddInvalidIpnLogResponse",
	86 => "AddInvoice",
	87 => "AddInvoiceResponse",
	88 => "AddInvoicingGroup",
	89 => "AddInvoicingGroupResponse",
	90 => "AddLedgerAccount",
	91 => "AddMigrationConfig",
	92 => "AddMigrationConfigResponse",
	93 => "AddNews",
	94 => "AddNewsGroup",
	95 => "AddNewsGroupResponse",
	96 => "AddNewsResponse",
//	97 => "AddOrderForm",
//	98 => "AddOrderFormResponse",
	99 => "AddPackage",
	100 => "AddPackageAddon",
	101 => "AddPackageAddonAttribute",
	102 => "AddPackageAddonAttributeResponse",
	103 => "AddPackageAddonResponse",
	104 => "AddPackageAttribute",
	105 => "AddPackageAttributeResponse",
	106 => "AddPackageResponse",
	107 => "AddPanel",
	108 => "AddPanelResponse",
//	109 => "AddPaymentGroup",
//	110 => "AddPaymentGroupResponse",
	111 => "AddPrepayDiscount",
	112 => "AddPrepayDiscountResponse",
	113 => "AddPrice",
	114 => "AddProduct",
	115 => "AddProductAddon",
	116 => "AddProductAddonGroup",
	117 => "AddProductAddonGroupResponse",
	118 => "AddProductAddonResponse",
	119 => "AddProductAttributes",
	120 => "AddProductAttributesResponse",
	121 => "AddProductGroup",
	122 => "AddProductGroupResponse",
	123 => "AddProductPricing",
	124 => "AddProductPricingResponse",
	125 => "AddProductRelation",
	126 => "AddProductRelationResponse",
//	127 => "AddProductResponse",
	128 => "AddProductValues",
	129 => "AddProductValuesResponse",
	130 => "AddProductVariable",
	131 => "AddProductVariableResponse",
	132 => "AddProductVariableValue",
	133 => "AddProductVariableValueResponse",
	134 => "AddProductVariant",
	135 => "AddProductVariantResponse",
	136 => "AddQuestionValueMap",
	137 => "AddQuestionValueMapResponse",
	138 => "AddQuestionValueMapping",
	139 => "AddQuestionValueMappingResponse",
	140 => "AddDomainRegistrar",
//	141 => "AddRegistrarResponse",
//	142 => "AddServer",
	143 => "AddServerGroup",
	144 => "AddServerGroupResponse",
//	145 => "AddServerResponse",
	146 => "AddSkeletonProduct",
	147 => "AddTLD",
	148 => "AddTLDResponse",
	149 => "AddTaxZone",
	150 => "AddTaxZoneGroup",
	151 => "AddTaxZoneGroupResponse",
	152 => "AddTaxZoneResponse",
	153 => "AddTheme",
	154 => "AddThemeResponse",
	155 => "AddTodo",
	156 => "AddTodoResponse",
	157 => "AddVendor",
	158 => "AddVendorInvoices",
	159 => "AddVendorResponse",
	160 => "AddWebsiteWidget",
//	161 => "AddWebsiteWidgetResponse",
	162 => "AddWizardQuestion",
	163 => "AddWizardQuestionResponse",
	164 => "AddWizardStep",
	165 => "AddWizardStepResponse",
	166 => "CancelAuthTransaction",
	167 => "CancelCert",
	168 => "CancelCertResponse",
	169 => "CancelInvoice",
	170 => "CancelInvoiceResponse",
	171 => "CancelPackage",
	172 => "CancelPackageResponse",
	173 => "CancelVendorPackage",
	174 => "CaptureFunds",
	175 => "ChangePackageClient",
	176 => "ChangePackageClientResponse",
	177 => "ChargeVirtualTerminal",
	178 => "ChargeVirtualTerminalResponse",
	179 => "CheckCertStatus",
	180 => "CheckCertStatusResponse",
//	181 => "CheckDomainAvailability",
	182 => "CheckDomainTransferAvailability",
	183 => "CheckMultiDomainWhois",
	184 => "CheckRunPaymentQueue",
	185 => "CheckServerLoad",
	186 => "ClearCart",
	187 => "ClearPaymentQueue",
	188 => "CloneAdminGroup",
	189 => "CloneAdminGroupResponse",
	190 => "CloneBookmark",
	191 => "CloneBookmarkResponse",
	192 => "CloneCoupon",
	193 => "CloneCouponResponse",
	194 => "CloneOrderForm",
	195 => "CloneOrderFormResponse",
	196 => "CloneProduct",
	197 => "CloneProductAddon",
	198 => "CloneProductAddonResponse",
	199 => "CloneProductAttribute",
	200 => "CloneProductAttributeResponse",
	201 => "CloneProductResponse",
	202 => "CloneServerGroup",
	203 => "CloneServerGroupResponse",
	204 => "CloneTheme",
	205 => "CloneThemeResponse",
	206 => "ClosePeriod",
	207 => "ClosePeriodConfirm",
	208 => "ClosePeriodResponse",
	209 => "ConfirmCreditNote",
	210 => "ConfirmCreditNoteResponse",
	211 => "CreateMarketplaceSeller",
	212 => "CreatePackage",
	213 => "CreateVendorPackage",
	214 => "CreateVendorUser",
	215 => "CreateVendorUserResponse",
	216 => "DeletePackage",
	217 => "DeletePackageAddon",
	218 => "DeletePackageResponse",
	219 => "DeleteVendorPackage",
	220 => "DisassociateDomain",
	221 => "DowngradePackage",
	222 => "DowngradePackageAddon",
	223 => "DowngradePackageResponse",
	224 => "EditAdmin",
	225 => "EditAdminGroup",
	226 => "EditAdminGroupResponse",
	227 => "EditAdminResponse",
	228 => "EditAppliedValues",
	229 => "EditAppliedValuesResponse",
	230 => "EditAttribute",
	231 => "EditAttributeGroup",
	232 => "EditAttributeGroupResponse",
	233 => "EditAttributeMap",
	234 => "EditAttributeMapResponse",
	235 => "EditAttributeMapping",
	236 => "EditAttributeMappingResponse",
	237 => "EditAttributeResponse",
	238 => "EditBasicClientAttributes",
	239 => "EditBasicClientAttributesResponse",
	240 => "EditBillingAccount",
	241 => "EditBillingAccountResponse",
	242 => "EditBookmark",
	243 => "EditBookmarkResponse",
	244 => "EditCartDomain",
	245 => "EditCartDomainResponse",
	246 => "EditCartProduct",
	247 => "EditCartProductResponse",
	248 => "EditCert",
	249 => "EditCertResponse",
	250 => "EditCheck",
	251 => "EditCheckResponse",
	252 => "EditClient",
	253 => "EditClientArchiveState",
	254 => "EditClientArchiveStateResponse",
	255 => "EditClientContact",
	256 => "EditClientContactResponse",
	257 => "EditClientResponse",
	258 => "EditCompany",
	259 => "EditCompanyResponse",
	260 => "EditConfigParams",
	261 => "EditCoupon",
//	262 => "EditCouponResponse",
	263 => "EditCycle",
	264 => "EditCycleDiscount",
	265 => "EditCycleDiscountGroup",
	266 => "EditCycleDiscountGroupResponse",
	267 => "EditCycleDiscountResponse",
	268 => "EditCycleGroup",
	269 => "EditCycleGroupResponse",
	270 => "EditCycleResponse",
	271 => "EditDbgPageloadsSettings",
	272 => "EditDbgPageloadsSettingsResponse",
	273 => "EditDeposit",
	274 => "EditDepositResponse",
	275 => "EditDomain",
	276 => "EditDomainAccessPassword",
	277 => "EditDomainAccessPasswordResponse",
	278 => "EditDomainContactData",
	279 => "EditDomainContactDataResponse",
	280 => "EditDomainDNS",
	281 => "EditDomainDNSResponse",
	282 => "EditDomainRegistrarLockOptions",
	283 => "EditDomainRegistrarLockOptionsResponse",
	284 => "EditDomainRenewTypeOptions",
	285 => "EditDomainRenewTypeOptionsResponse",
	286 => "EditDomainResponse",
	287 => "EditEmailConfig",
	288 => "EditEmailConfigResponse",
	289 => "EditEmailQueue",
	290 => "EditEmailQueueResponse",
	291 => "EditEmailTemplate",
	292 => "EditEmailTemplateGroup",
	293 => "EditEmailTemplateGroupResponse",
	294 => "EditEmailTemplateResponse",
	295 => "EditEncryptionConfigs",
	296 => "EditEncryptionConfigsConfirm",
	297 => "EditEncryptionConfigsResponse",
	298 => "EditEvent",
	299 => "EditEventResponse",
	300 => "EditFAQ",
	301 => "EditFAQCategory",
	302 => "EditFAQCategoryResponse",
	303 => "EditFAQResponse",
	304 => "EditFraudCountryThreshold",
	305 => "EditFraudCountryThresholdResponse",
	306 => "EditFraudGuardianConfig",
	307 => "EditFraudGuardianConfigResponse",
	308 => "EditFraudSetOrderFormsResponse",
	309 => "EditFraudSetResponse",
	310 => "EditGeneralJournalEntry",
	311 => "EditGeneralJournalEntryResponse",
	312 => "EditGenericAttributeResponse",
	313 => "EditGroupAttribute",
	314 => "EditGroupAttributeResponse",
	315 => "EditHelpdeskBucket",
	316 => "EditHelpdeskBucketResponse",
	317 => "EditHelpdeskDepartment",
	318 => "EditHelpdeskDepartmentResponse",
	319 => "EditHelpdeskResponseTemplate",
	320 => "EditHelpdeskResponseTemplateResponse",
	321 => "EditHelpdeskTicket",
	322 => "EditHelpdeskTicketResponse",
	323 => "EditIPLog",
	324 => "EditIPLogResponse",
	325 => "EditInvalidIpnLog",
	326 => "EditInvalidIpnLogResponse",
	327 => "EditInvoice",
	328 => "EditInvoiceResponse",
	329 => "EditInvoicingGroup",
	330 => "EditInvoicingGroupResponse",
	331 => "EditLedgerAccount",
	332 => "EditLedgerAccountResponse",
	333 => "EditLedgerGroupEntries",
	334 => "EditLedgerGroupEntriesResponse",
	335 => "EditMigrationConfig",
	336 => "EditMigrationConfigResponse",
	337 => "EditMyProfile",
	338 => "EditMyProfileResponse",
	339 => "EditNewOrdersResponse",
	340 => "EditNews",
	341 => "EditNewsGroup",
	342 => "EditNewsGroupResponse",
	343 => "EditNewsResponse",
	344 => "EditOneProductRelations",
	345 => "EditOneProductRelationsResponse",
	346 => "EditOrderForm",
	347 => "EditOrderFormResponse",
	348 => "EditPPCFactoryConfig",
	349 => "EditPPCFactoryConfigResponse",
	350 => "EditPackage",
	351 => "EditPackageAddon",
	352 => "EditPackageAddonAttribute",
	353 => "EditPackageAddonAttributeResponse",
	354 => "EditPackageAddonResponse",
	355 => "EditPackageAddons",
	356 => "EditPackageAddonsResponse",
	357 => "EditPackageAttribute",
	358 => "EditPackageAttributeResponse",
	359 => "EditPackageResponse",
	360 => "EditPanel",
	361 => "EditPanelAttributeResponse",
	362 => "EditPanelConfig",
	363 => "EditPanelConfigResponse",
	364 => "EditPanelResponse",
//	365 => "EditPaymentGroup",
//	366 => "EditPaymentGroupResponse",
	367 => "EditPaymentQueue",
	368 => "EditPaymentQueueResponse",
	369 => "EditPrepayDiscount",
	370 => "EditPrepayDiscountResponse",
	371 => "EditPrice",
	372 => "EditProduct",
	373 => "EditProductAddon",
	374 => "EditProductAddonAttributes",
	375 => "EditProductAddonAttributesResponse",
	376 => "EditProductAddonGroup",
	377 => "EditProductAddonGroupResponse",
	378 => "EditProductAddonGroups",
	379 => "EditProductAddonGroupsResponse",
	380 => "EditProductAddonResponse",
	381 => "EditProductAddons",
	382 => "EditProductAddonsResponse",
	383 => "EditProductAttribute",
	384 => "EditProductAttributeResponse",
	385 => "EditProductAttributes",
	386 => "EditProductChildren",
	387 => "EditProductChildrenResponse",
	388 => "EditProductCoupons",
	389 => "EditProductCouponsResponse",
	390 => "EditProductCycles",
	391 => "EditProductCyclesResponse",
	392 => "EditProductEmails",
	393 => "EditProductEmailsResponse",
	394 => "EditProductGroup",
	395 => "EditProductGroupResponse",
	396 => "EditProductNotifications",
	397 => "EditProductNotificationsResponse",
	398 => "EditProductOrderForms",
	399 => "EditProductOrderFormsResponse",
	400 => "EditProductPriceResponse",
//	401 => "EditProductPrices",
//	402 => "EditProductPricesResponse",
	403 => "EditProductRelation",
	404 => "EditProductRelationResponse",
	405 => "EditProductResponse",
	406 => "EditProductServerGroups",
	407 => "EditProductServerGroupsResponse",
	408 => "EditProductTLDs",
	409 => "EditProductTLDsResponse",
	410 => "EditProductValues",
	411 => "EditProductValuesResponse",
	412 => "EditProductVariable",
	413 => "EditProductVariableResponse",
	414 => "EditProductVariableValue",
	415 => "EditProductVariableValueResponse",
	416 => "EditProductVariant",
	417 => "EditProductVariantLedgerEntries",
	418 => "EditProductVariantLedgerEntriesResponse",
	419 => "EditProductVariantPrices",
	420 => "EditProductVariantPricesResponse",
	421 => "EditProductVariantResponse",
	422 => "EditProductsGroup",
	423 => "EditProductsGroupResponse",
	424 => "EditQuestionValueMap",
	425 => "EditQuestionValueMapResponse",
	426 => "EditQuestionValueMapping",
	427 => "EditQuestionValueMappingResponse",
	428 => "EditDomainRegistrar",
//	429 => "EditRegistrarConfigResponse",
	430 => "EditRegistrarSettings",
	431 => "EditRegistrarSettingsResponse",
	432 => "EditSSLFactoryConfig",
	433 => "EditSSLFactoryConfigResponse",
	434 => "EditSellerSettings",
	435 => "EditSellerSettingsResponse",
//	436 => "EditServer",
	437 => "EditServerGroup",
	438 => "EditServerGroupResponse",
//	439 => "EditServerResponse",
	440 => "EditTLD",
	441 => "EditTLDResponse",
	442 => "EditTaxZone",
	443 => "EditTaxZoneGroup",
	444 => "EditTaxZoneGroupResponse",
	445 => "EditTaxZoneResponse",
	446 => "EditTheme",
	447 => "EditThemeResponse",
	448 => "EditTodo",
	449 => "EditTodoResponse",
//	450 => "EditUltimateAffiliateConfig",
//	451 => "EditUltimateAffiliateConfigResponse",
	452 => "EditUsagePricing",
	453 => "EditUsagePricingResponse",
	454 => "EditVendor",
	455 => "EditVendorResponse",
	456 => "EditWebsiteWidget",
//	457 => "EditWebsiteWidgetResponse",
	458 => "EditWizardQuestion",
	459 => "EditWizardQuestionResponse",
	460 => "EditWizardStep",
	461 => "EditWizardStepResponse",
//	462 => "EditiDevAffiliateConfig",
//	463 => "EditiDevAffiliateConfigResponse",
	464 => "EnablePackage",
	465 => "EnablePackageAddon",
	466 => "EnablePackageResponse",
	467 => "ExportPaymentQueue",
	468 => "ForgottenPasswordResponse",
	469 => "ForwardToClientInterface",
	470 => "GenerateInvoiceEvents",
	471 => "GeneratePaymentQueue",
	472 => "GeneratePaymentQueueResponse",
	473 => "GetAutocompleteClients",
	474 => "GetAutocompleteDomains",
	475 => "GetAutocompleteProducts",
	476 => "GetBillingAccounts",
	477 => "GetClientStatus",
	478 => "GetCompanyInformation",
	479 => "GetEmailTemplate",
	480 => "GetEmailTemplateVariables",
	481 => "GetLogEnabledCommands",
	482 => "GetMBAPIMap",
	483 => "GetOrderForms",
	484 => "GetReportActivePackagesByCycle",
	485 => "GetReportActivePerServer",
	486 => "GetReportActivePerServerGroup",
	487 => "GetReportBilledServerRevenue",
	488 => "GetReportCustomerPaymentsByDate",
	489 => "GetReportDailyInvoices",
	490 => "GetReportDailySignups",
	491 => "GetReportMostPopularPackages",
	492 => "GetReportPackagesByStatus",
	493 => "GetReportPaymentsByPaymentType",
	494 => "GetReportRecurringServerRevenue",
	495 => "GetReportRevenuePerServer",
	496 => "GetVendorInformation",
	497 => "GetVendorProducts",
	498 => "ImportClientFromPanel",
	499 => "ImportClientFromPanelResponse",
	500 => "InstallSystemLicense",
	501 => "InstallSystemLicenseResponse",
	502 => "LockDomainPrices",
	503 => "LockDomainPricesResponse",
	504 => "LockPackageAddonPrices",
	505 => "LockPackageAddonPricesResponse",
	506 => "LockPackagePrices",
	507 => "LockPackagePricesResponse",
	508 => "LockProductAddonPrices",
	509 => "LockProductAddonPricesResponse",
	510 => "LockProductPrices",
	511 => "LockProductPricesResponse",
	512 => "LoginResponse",
	513 => "LogoutResponse",
	514 => "ManageClientActionActivate",
	515 => "ManageClientActionActivateResponse",
	516 => "ManageClientActionArchive",
	517 => "ManageClientActionArchiveResponse",
	518 => "ManageClientActionCancel",
	519 => "ManageClientActionCancelResponse",
	520 => "ManageClientActionDelete",
	521 => "ManageClientActionDeleteResponse",
	522 => "ManageClientActionPurge",
	523 => "ManageClientActionPurgeResponse",
	524 => "ManageClientActionSuspend",
	525 => "ManageClientActionSuspendResponse",
	526 => "ManageClientActionUnsuspend",
	527 => "ManageClientActionUnsuspendResponse",
	528 => "ManageClientNotes",
	529 => "ManageClientNotesResponse",
	530 => "ManageSellerProduct",
	531 => "MergeClient",
	532 => "MergeClientResponse",
	533 => "MergeCompanies",
	534 => "MergeCompaniesResponse",
//	535 => "MigrateConfig",
//	536 => "MigrateConfigAssociate",
//	537 => "MigrateConfigAssociateResponse",
//	538 => "MigrateConfigResponse",
//	539 => "MigrateDeprocess",
//	540 => "MigrateDisabled",
//	541 => "MigrateError",
//	542 => "MigrateIntro",
//	543 => "MigrateManager",
//	544 => "MigrateModuleDeprocess",
//	545 => "MigrateModuleManager",
//	546 => "MigrateModuleProcess",
//	547 => "MigrateProcess",
//	548 => "MigrateProductDeprocess",
//	549 => "MigrateProductManager",
//	550 => "MigrateProductProcess",
//	551 => "MigrateProductSkip",
//	552 => "MigrateSkip",
	553 => "OrderCertCSRRequest",
	554 => "OrderCertCSRRequestResponse",
	555 => "OrderCertCSRVerify",
	556 => "OrderCertConfirmOrder",
	557 => "OrderCertContactRequest",
	558 => "OrderCertContactRequestResponse",
	559 => "OrderCertEnterCSR",
	560 => "OrderCertGetApproverEula",
	561 => "OrderCertGetApproverEulaResponse",
	562 => "OrderCertGetApproverResponse",
	563 => "OrderCertOrderResponse",
	564 => "OrderCertOrderResults",
	565 => "OrderCertOrderSummary",
	566 => "OrderCertReceipt",
	567 => "OrderCertSelectApprover",
	568 => "OrderCertSelectApproverResponse",
	569 => "OrderCertSubmitOrderResponse",
	570 => "OrderCertVerifyCSRResponse",
	571 => "OrderFormProvisioning",
	572 => "PayInvoice",
	573 => "PayInvoiceResponse",
	574 => "PayInvoices",
	575 => "PayInvoicesResponse",
	576 => "PreviewEmailTemplate",
	577 => "ProcessCartCheckout",
	578 => "ProcessCartCheckoutLoader",
	579 => "ProcessCartLogin",
	580 => "ProcessCartLoginResponse",
	581 => "ProcessCartRegister",
	582 => "ProcessManualPayment",
	583 => "ProcessTransactionRefund",
	584 => "ProcessTransactionRefundResponse",
	585 => "PurgeDbgPageloads",
	586 => "PushProduct",
	587 => "ReassignPackageAttributeResponse",
	588 => "ReauthBillingAccount",
	589 => "ReauthBillingAccountResponse",
	590 => "RecordAdminActions",
//	591 => "ReencryptBillingAccountsResponse",
	592 => "RegisterDomainConfirm",
	593 => "RegisterDomainConfirmResponse",
	594 => "ReloadEmailTemplateResponse",
	595 => "RemoveAdmin",
	596 => "RemoveAdminGroup",
	597 => "RemoveAdminGroupResponse",
	598 => "RemoveAdminResponse",
	599 => "RemoveAttribute",
	600 => "RemoveAttributeGroup",
	601 => "RemoveAttributeGroupResponse",
	602 => "RemoveAttributeMap",
	603 => "RemoveAttributeMapResponse",
	604 => "RemoveAttributeMapping",
	605 => "RemoveAttributeMappingResponse",
	606 => "RemoveAttributeResponse",
	607 => "RemoveBillingAccount",
	608 => "RemoveBillingAccountResponse",
	609 => "RemoveBookmark",
	610 => "RemoveBookmarkResponse",
	611 => "RemoveCartCoupon",
	612 => "RemoveCartCouponResponse",
	613 => "RemoveCartProduct",
	614 => "RemoveCartProductResponse",
	615 => "RemoveCert",
	616 => "RemoveCertResponse",
	617 => "RemoveClient",
	618 => "RemoveClientContact",
//	619 => "RemoveClientContactResponse",
	620 => "RemoveClientNoteResponse",
	621 => "RemoveClientResponse",
	622 => "RemoveCompany",
	623 => "RemoveCompanyResponse",
	624 => "RemoveCoupon",
//	625 => "RemoveCouponResponse",
	626 => "RemoveCycle",
	627 => "RemoveCycleDiscount",
	628 => "RemoveCycleDiscountGroup",
	629 => "RemoveCycleDiscountGroupResponse",
	630 => "RemoveCycleDiscountResponse",
	631 => "RemoveCycleGroup",
	632 => "RemoveCycleGroupResponse",
	633 => "RemoveCycleResponse",
	634 => "RemoveDomain",
	635 => "RemoveDomainResponse",
	636 => "RemoveEmail",
	637 => "RemoveEmailQueue",
	638 => "RemoveEmailQueueResponse",
	639 => "RemoveEmailResponse",
	640 => "RemoveEmailTemplate",
	641 => "RemoveEmailTemplateGroup",
	642 => "RemoveEmailTemplateGroupResponse",
	643 => "RemoveEmailTemplateResponse",
	644 => "RemoveEvent",
	645 => "RemoveEventResponse",
	646 => "RemoveFAQ",
	647 => "RemoveFAQCategory",
	648 => "RemoveFAQCategoryResponse",
	649 => "RemoveFAQResponse",
	650 => "RemoveFraudCountryThresholdResponse",
	651 => "RemoveFraudSetResponse",
//	652 => "RemoveGateway",
//	653 => "RemoveGatewayResponse",
	654 => "RemoveGroupAttribute",
	655 => "RemoveGroupAttributeResponse",
	656 => "RemoveHelpdeskBucket",
	657 => "RemoveHelpdeskBucketResponse",
	658 => "RemoveHelpdeskDepartment",
	659 => "RemoveHelpdeskDepartmentResponse",
	660 => "RemoveHelpdeskResponseTemplate",
	661 => "RemoveHelpdeskResponseTemplateResponse",
	662 => "RemoveHelpdeskTicket",
	663 => "RemoveHelpdeskTicketResponse",
	664 => "RemoveIPLog",
	665 => "RemoveIPLogResponse",
	666 => "RemoveInvalidIpnLog",
	667 => "RemoveInvalidIpnLogResponse",
	668 => "RemoveInvoice",
	669 => "RemoveInvoiceResponse",
	670 => "RemoveInvoiceTransaction",
	671 => "RemoveInvoiceTransactionResponse",
	672 => "RemoveInvoicingGroup",
	673 => "RemoveInvoicingGroupResponse",
	674 => "RemoveLedgerAccount",
	675 => "RemoveLedgerGroup",
	676 => "RemoveLedgerGroupResponse",
	677 => "RemoveMigrationConfig",
	678 => "RemoveMigrationConfigResponse",
	679 => "RemoveNews",
	680 => "RemoveNewsGroup",
	681 => "RemoveNewsGroupResponse",
	682 => "RemoveNewsResponse",
	683 => "RemoveOrderForm",
	684 => "RemoveOrderFormResponse",
	685 => "RemovePackage",
	686 => "RemovePackageAddon",
	687 => "RemovePackageAddonAttribute",
	688 => "RemovePackageAddonAttributeResponse",
	689 => "RemovePackageAddonResponse",
	690 => "RemovePackageAttribute",
	691 => "RemovePackageAttributeResponse",
	692 => "RemovePackageResponse",
	693 => "RemovePanel",
	694 => "RemovePanelResponse",
//	695 => "RemovePaymentGroup",
//	696 => "RemovePaymentGroupGateway",
//	697 => "RemovePaymentGroupGatewayResponse",
//	698 => "RemovePaymentGroupResponse",
	699 => "RemovePaymentQueue",
	700 => "RemovePaymentQueueResponse",
	701 => "RemovePrepayDiscount",
	702 => "RemovePrepayDiscountResponse",
	703 => "RemoveProduct",
	704 => "RemoveProductAddon",
	705 => "RemoveProductAddonGroup",
	706 => "RemoveProductAddonGroupResponse",
	707 => "RemoveProductAddonResponse",
	708 => "RemoveProductAttribute",
	709 => "RemoveProductAttributeResponse",
	710 => "RemoveProductGroup",
	711 => "RemoveProductGroupResponse",
	712 => "RemoveProductResponse",
	713 => "RemoveProductValues",
	714 => "RemoveProductValuesResponse",
	715 => "RemoveProductVariable",
	716 => "RemoveProductVariableResponse",
	717 => "RemoveProductVariableValue",
	718 => "RemoveProductVariableValueResponse",
	719 => "RemoveProductVariant",
	720 => "RemoveProductVariantResponse",
	721 => "RemoveQuestion",
	722 => "RemoveQuestionResponse",
	723 => "RemoveQuestionValueMap",
	724 => "RemoveQuestionValueMapResponse",
	725 => "RemoveQuestionValueMapping",
	726 => "RemoveQuestionValueMappingResponse",
//	727 => "RemoveServer",
	728 => "RemoveServerGroup",
	729 => "RemoveServerGroupResponse",
//	730 => "RemoveServerResponse",
	731 => "RemoveTLD",
	732 => "RemoveTLDResponse",
	733 => "RemoveTaxZone",
	734 => "RemoveTaxZoneGroup",
	735 => "RemoveTaxZoneGroupResponse",
	736 => "RemoveTaxZoneResponse",
	737 => "RemoveTheme",
	738 => "RemoveThemeResponse",
	739 => "RemoveTodo",
	740 => "RemoveTodoResponse",
	741 => "RemoveTransaction",
	742 => "RemoveTransactionResponse",
	743 => "RemoveWebsiteWidget",
	744 => "RemoveWebsiteWidgetResponse",
	745 => "RemoveWizardQuestion",
	746 => "RemoveWizardQuestionResponse",
	747 => "RemoveWizardStep",
	748 => "RemoveWizardStepResponse",
	749 => "RenewDomain",
	750 => "RenewDomainResponse",
	751 => "ReplayCallbackLog",
	752 => "ReplayCallbackLogResponse",
	753 => "ReportClientList",
	754 => "ReportClientListDetailed",
	755 => "ReportClientListGraph",
	756 => "RequestProductSync",
	757 => "RequestVendorInvoiceSync",
	758 => "ResendCertApproverEmail",
	759 => "ResendCertApproverEmailResponse",
	760 => "ResendEmail",
	761 => "ResendEmailResponse",
	762 => "ResendInvoice",
	763 => "ResendInvoiceResponse",
	764 => "ResetContactPassword",
	765 => "ResetContactPasswordResponse",
	766 => "RunDebugEvent",
	767 => "RunEvents",
	768 => "RunPaymentQueue",
	769 => "RunPaymentQueueResponse",
	770 => "SearchDispatchResponse",
	771 => "SendClientEmail",
	772 => "SendClientEmailResponse",
	773 => "SendEmailQueue",
	774 => "SendEmailQueueResponse",
	775 => "SendSellerMessage",
	776 => "ShowAccountingReportsMenu",
	777 => "ShowAccountingStats",
	778 => "ShowAccountsPayableLedger",
	779 => "ShowAccountsReceivableLedger",
	780 => "ShowAdminDetails",
	781 => "ShowAdminGroupDetails",
	782 => "ShowAdminGroups",
	783 => "ShowAdminStats",
	784 => "ShowAdmins",
	785 => "ShowAgingReport",
	786 => "ShowAgingReportContainer",
	787 => "ShowAllTodos",
	788 => "ShowAttributeDetails",
	789 => "ShowAttributeGroups",
	790 => "ShowAttributeMapDetails",
	791 => "ShowAttributeMappingDetails",
	792 => "ShowAttributeMappings",
	793 => "ShowAttributeMaps",
	794 => "ShowAttributes",
	795 => "ShowBalanceSheet",
	796 => "ShowBalanceSheetContainer",
	797 => "ShowBillingAccountDetails",
	798 => "ShowBillingAccounts",
	799 => "ShowBookmarks",
	800 => "ShowCallbackLog",
	801 => "ShowCallbackLogDetails",
	802 => "ShowCart",
	803 => "ShowCartCheckoutConfirmForm",
	804 => "ShowCartCheckoutForm",
	805 => "ShowCartCheckoutSuccess",
	806 => "ShowCartProductAddons",
	807 => "ShowCartProductAddonsStep",
	808 => "ShowCashFlow",
	809 => "ShowCashFlowContainer",
	810 => "ShowCertDetails",
	811 => "ShowCerts",
	812 => "ShowClientARHistory",
	813 => "ShowClientCallbackLog",
	814 => "ShowClientCallbackLogDetails",
	815 => "ShowClientContactDetails",
	816 => "ShowClientContacts",
	817 => "ShowClientCreditHistory",
	818 => "ShowClientDetails",
	819 => "ShowClientGraph",
	820 => "ShowClientStats",
	821 => "ShowClients",
	822 => "ShowCompanies",
	823 => "ShowCompanyDetails",
	824 => "ShowConfigs",
	825 => "ShowCouponDetails",
	826 => "ShowCoupons",
	827 => "ShowCycleDiscountGroups",
	828 => "ShowCycleDiscounts",
	829 => "ShowCycleGroups",
	830 => "ShowCycleGroupsResponse",
	831 => "ShowCycleStats",
	832 => "ShowCycles",
	833 => "ShowCyclesResponse",
	834 => "ShowDashboard",
	835 => "ShowDashboardNewOrders",
	836 => "ShowDashboardTodos",
	837 => "ShowDbgPageloadDetails",
	838 => "ShowDbgPageloads",
	839 => "ShowDomainDetails",
	840 => "ShowDomainEPPCodeStep",
	841 => "ShowDomains",
	842 => "ShowEmailDetails",
	843 => "ShowEmailMenus",
	844 => "ShowEmailQueue",
	845 => "ShowEmailQueueDetails",
	846 => "ShowEmailTemplateDetails",
	847 => "ShowEmailTemplateGroupDetails",
	848 => "ShowEmailTemplateGroups",
	849 => "ShowEmailTemplates",
	850 => "ShowEventInputDetails",
	851 => "ShowEventOutputDetails",
	852 => "ShowEventStatusBar",
	853 => "ShowEvents",
	854 => "ShowEventsStats",
	855 => "ShowFAQs",
	856 => "ShowFinancesMenu",
	857 => "ShowFinancialGraph",
	858 => "ShowForgottenPassword",
	859 => "ShowFraudCountryThresholds",
	860 => "ShowFraudSetDetails",
	861 => "ShowFraudSets",
	862 => "ShowGeneralJournal",
	863 => "ShowGeneralLedger",
	864 => "ShowGenerateInvoiceMenu",
	865 => "ShowGenericBar",
	866 => "ShowGroupAttributes",
	867 => "ShowHelpdesk",
	868 => "ShowHelpdeskBucketDetails",
	869 => "ShowHelpdeskBucketTickets",
	870 => "ShowHelpdeskBuckets",
	871 => "ShowHelpdeskDepartments",
	872 => "ShowHelpdeskGraph",
	873 => "ShowHelpdeskResponseTemplates",
	874 => "ShowHelpdeskTicketDetails",
	875 => "ShowHelpdeskTickets",
	876 => "ShowIPLog",
	877 => "ShowInvalidIpnLogDetails",
	878 => "ShowInvalidIpnLogs",
	879 => "ShowInvoiceDetails",
	880 => "ShowInvoiceSheet",
	881 => "ShowInvoiceSheetContainer",
	882 => "ShowInvoiceStats",
	883 => "ShowInvoices",
	884 => "ShowInvoicingGroups",
	885 => "ShowJournalsMenu",
	886 => "ShowLastFiveClientInvoices",
	887 => "ShowLastFiveClientPackages",
	888 => "ShowLedgerAccountDetails",
	889 => "ShowLedgerAccounts",
	890 => "ShowLedgersMenu",
	891 => "ShowLogDetails",
	892 => "ShowLogMenu",
	893 => "ShowLogin",
	//894 => "ShowMBRequirementsTest",
	895 => "ShowMigrationConfigDetails",
	896 => "ShowMigrationConfigs",
	897 => "ShowMultiDomainWhois",
	898 => "ShowMultiDomainWhoisStep",
	899 => "ShowNewOrders",
	900 => "ShowNews",
	901 => "ShowNewsDetails",
	902 => "ShowNewsGroups",
	903 => "ShowOrderFormProductsStep",
	904 => "ShowOrderForms",
	905 => "ShowPDFInvoice",
	906 => "ShowPHPInfo",
	907 => "ShowPPCCodes",
	908 => "ShowPackageAddonAttributeDetails",
	909 => "ShowPackageAddonAttributes",
	910 => "ShowPackageAddonDetails",
	911 => "ShowPackageAddons",
	912 => "ShowPackageAttributeDetails",
	913 => "ShowPackageAttributes",
	914 => "ShowPackageDetails",
	915 => "ShowPackageGraph",
	916 => "ShowPackages",
	917 => "ShowPanelClientLogin",
	918 => "ShowPanelDetails",
	919 => "ShowPanelOptions",
//	920 => "ShowPaymentGroups",
	921 => "ShowPaymentQueue",
	922 => "ShowPaymentQueueDetails",
	923 => "ShowPaymentQueueStats",
	924 => "ShowPrepayDiscounts",
	925 => "ShowProductAddonDetails",
	926 => "ShowProductAddonGroups",
	927 => "ShowProductAddons",
	928 => "ShowProductAttributeDetails",
	929 => "ShowProductAttributes",
	930 => "ShowProductDetails",
	931 => "ShowProductGraph",
	932 => "ShowProductGroups",
	933 => "ShowProductRelationDetails",
	934 => "ShowProductRelations",
	935 => "ShowProductStats",
	936 => "ShowProductVariableDetails",
	937 => "ShowProductVariableValueDetails",
	938 => "ShowProductVariableValues",
	939 => "ShowProductVariables",
	940 => "ShowProductVariants",
	941 => "ShowProducts",
	942 => "ShowProductsRevenue",
	943 => "ShowProductsRevenueContainer",
	944 => "ShowProfitAndLoss",
	945 => "ShowProfitAndLossContainer",
	946 => "ShowPurchasesJournal",
	947 => "ShowQuestionValueMapDetails",
	948 => "ShowQuestionValueMappingDetails",
	949 => "ShowQuestionValueMappings",
	950 => "ShowQuestionValueMaps",
	951 => "ShowReceiptsJournal",
//	952 => "ShowRegisterDomainForm",
	953 => "ShowRegistrars",
	954 => "ShowReportContainer",
	955 => "ShowReportDetails",
	956 => "ShowReports",
	957 => "ShowResendInvoice",
	958 => "ShowResendInvoiceResponse",
	959 => "ShowSellerStatus",
//	960 => "ShowServerDetails",
	961 => "ShowServerGroupDetails",
	962 => "ShowServerGroups",
	963 => "ShowServerGroupsReport",
//	964 => "ShowServers",
	965 => "ShowSignupSheet",
	966 => "ShowSignupSheetContainer",
	967 => "ShowSystemLibraryInfo",
	968 => "ShowSystemLicenseInfo",
//	969 => "ShowSystemModuleInfo",
	970 => "ShowSystemServerInfo",
	971 => "ShowTLDDetails",
	972 => "ShowTLDLoader",
	973 => "ShowTLDs",
	974 => "ShowTaxPayableLedger",
	975 => "ShowTaxZoneGroups",
	976 => "ShowTaxZones",
	977 => "ShowThemeDetails",
	978 => "ShowThemes",
	979 => "ShowTodoDetails",
	980 => "ShowTodos",
	981 => "ShowTransactionDetails",
	982 => "ShowTransactionLog",
	983 => "ShowTransactions",
	984 => "ShowTransferDomainForm",
	985 => "ShowTrendGraph",
	986 => "ShowUncapturedFunds",
	987 => "ShowVendorDetails",
	988 => "ShowVendors",
	989 => "ShowVirtualTerminals",
	990 => "ShowWebsiteWidget",
	991 => "ShowWebsiteWidgets",
	992 => "ShowWizardQuestions",
	993 => "ShowWizardSteps",
	994 => "StopEvents",
	995 => "SuppressPPCCodeResponse",
	996 => "SuspendPackage",
	997 => "SuspendPackageAddon",
	998 => "SuspendPackageResponse",
	999 => "SuspendVendorPackage",
	1000 => "SyncDomainDates",
	1001 => "SyncDomainDatesResponse",
	1002 => "SyncSellerInventory",
	1003 => "SyncVendorAPIURL",
	1004 => "SyncVendorDetails",
	1005 => "TakeClientPayment",
	1006 => "TakeClientPaymentResponse",
	1007 => "TestSellerApiURL",
	1008 => "TestVendorAPIURL",
	1009 => "TransferDomainConfirm",
	1010 => "TransferDomainConfirmResponse",
	1011 => "UnLockProductAddonPrices",
	1012 => "UnLockProductAddonPricesResponse",
	1013 => "UnLockProductPrices",
	1014 => "UnLockProductPricesResponse",
	1015 => "UnlockDomainPrices",
	1016 => "UnlockDomainPricesResponse",
	1017 => "UnlockPackageAddonPrices",
	1018 => "UnlockPackageAddonPricesResponse",
	1019 => "UnlockPackagePrices",
	1020 => "UnlockPackagePricesResponse",
	1021 => "UnsuspendVendorPackage",
	1022 => "UpdatePackage",
	1023 => "UpdatePackageBillingShares",
	1024 => "UpdatePackageBillingSharesResponse",
	1025 => "UpdateSellerStatus",
	1026 => "UpdateSellerVendor",
	1027 => "UpdateSellerVendorResponse",
	1028 => "UpdateVendorInformation",
	1029 => "UpgradePackage",
	1030 => "UpgradePackageAddon",
	1031 => "UpgradePackageResponse",
	1032 => "UploadReport",
	1033 => "UploadReportResponse",
	1034 => "VoidInvoice",
	1035 => "VoidInvoiceResponse",
	1036 => "ShowClientServiceInputForm",
//	1037 => "ShowClientServiceInputFormResponse",
	1038 => "RemoveProductRelation",
	1039 => "RemoveProductRelationResponse",
	1040 => "ShowBans",
	1041 => "ShowBanDetails",
	1042 => "EditBan",
	1043 => "EditBanResponse",
	1044 => "AddBan",
	1045 => "AddBanResponse",
	1046 => "RemoveBan",
	1047 => "RemoveBanResponse",
	1048 => "ChangePackageCycle",
	1049 => "ChangePackageCycleResponse",
	1050 => "AddSingleInvoice",
	1051 => "AddSingleInvoiceResponse",
	1052 => "EditLedgerGroups",
	1053 => "EditProductMiscellaneous",
	1054 => "EditProductMiscellaneousResponse",
	1055 => "PerformPanelAction",
//	1056 => "FirstTimeIntroduction",
//	1057 => "FirstTimeDisplayStep",
//	1058 => "FirstTimePerformAction",
	1059 => "LoginToPanel",
	1060 => "InitSSOLogin",
	1061 => "InitSSOLogout",
	1062 => "ReceiveSSOLoginStatus",
	1063 => "ProcessSSOAPIRequest",
	1064 => "SelectSSOLoginMethod",
	1065 => "ShowIDPLogin",
	1066 => "ShowIDPFIUpdate",
	1067 => "ShowIDPFIConfirmDetach",
	1068 => "ShowIDPFIAttach",
//	1069 => "ShowIDPFIConfirmDelete",
	1070 => "ShowIDPSelectName",
	1071 => "ShowIDPFIManagement",
//	1072 => "ActivateSystemLicense",
//	1073 => "ActivateSystemLicenseResponse",
//	1074 => "RetrieveKeys",
//	1075 => "RetrieveKeysResponse",
//	1076 => "UploadKey",
//	1077 => "UploadKeyResponse",
//	1078 => "SwitchToBundledLicense",
//	1079 => "SwitchToBundledLicenseResponse",
//	1080 => "SwitchToFreeLicense",
//	1081 => "SwitchToFreeLicenseResponse",
//	1082 => "RollBackLicenseKey",
//	1083 => "RollBackLicenseKeyResponse",
//	1084 => "EditSSOConfig",
//	1085 => "EditSSOConfigResponse",
	1086 => "LogInToStore",
	1087 => "ProcessCartForgotPasswordResponse",
	1088 => "ShowCrashReports",
	1089 => "GotoCrashReport",
	1090 => "RemoveCrashReports",
	1091 => "CertifyGoDaddyAccount",
	1092 => "SendEmails",
	1093 => "ShowPaymentGateways",
	1094 => "AddPaymentGateway",
	1095 => "EditPaymentGateway",
	1096 => "RemovePaymentGateways",
	1097 => "ShowSSLPlugins",
	1098 => "EditSSLPluginConfig",
	1099 => "AddSSLPlugin",
	1100 => "EnableSSLPlugins",
	1101 => "DisableSSLPlugins",
	1102 => "EditProductSSLPlugin",
//	1103 => "RepairSSOClient",
	1104 => "CancelBatchFile",
	1105 => "CompleteBatchFile",
	1106 => "ProcessBatchFile",
	1107 => "ShowBatchFiles",
	1108 => "ShowBatchFileDetails",
	1109 => "CompleteBatchFileTransactions",
	1110 => "AssignTransactionsToBatchFile",
	1111 => "DeclineBatchFileTransactions",
	1112 => "DeassignTransactionsFromBatchFile",
	1114 => "ShowBatchFilesTransactions",
	1115 => "DownloadBatchFile",
	1116 => "EditAuthSettings",
	1117 => "EditOrderFormTermsText",
	1118 => "EditOrderFormTermsTextResponse",
	1119 => "ShowOrderFormTermsText",
	1120 => "ShowActionLog",
	1121 => "EditActionLogSettings",
	1122 => "ShowSSLCertificatePurchase",
//	1123 => "ShowPlanAddons",
	1124 => "SyncSSO",
	1125 => "ShowSyncStatistics",
	1126 => "DomainUploadDocuments",
	1127 => "EditPaymentQueueSettings",
	1128 => "ShowInvoiceFacturas",
	1129 => "ImportPanelPlans",
	1130 => "GetClientBalances",
	1131 => "ForgotPassword",
	1132 => "CreateClientContact",
	1133 => "UpdateClientContact",
	1134 => "DeleteClientContact",
	1135 => "CreateSession",
	1136 => "EditInvoiceFacturaSettings",
	1137 => "DownloadInvoiceFactura",
	1138 => "ImportPanelCustomers",
	1139 => "ShowOrderFormDetails",
	1140 => "EditOrderFormSettings",
	1141 => "EditOrderFormHeaderAndFooter",
	1142 => "EditOrderFormTheme",
	1143 => "EditOrderFormThemeResponse",
	1144 => "EditDomainRegistrationInfo",
	1145 => "ImportPanelSubscriptions",
	1146 => "EditProductApplications",
	1147 => "EditProductApplicationsResponse",
	1148 => "EditEulaConfigrmation",
	1149 => "EditInitialConfiguration",
	1150 => "ShowSystemOverview",
	1151 => "ShowSystemSetup",
);

// path: module -> controller -> action
$CONF["actionPaths"] = array(
    "clients" => array(
		"group" => array(
			"add" => "AddAttributeGroup",
			"edit" => "EditAttributeGroup"
		)
	),
	"configs" => array(
		"system" => array(
			"show" => "ShowConfigs",
		),
		"initial-wizard" => array(
			"eula" => "EditEulaConfigrmation",
			"config" => "EditInitialConfiguration",
		),
	),
	"discounts" => array(
		"index" => array(
			"index" => "ShowCycleDiscountGroups",
			"add" => "AddCycleDiscountGroup",
			"edit" => "EditCycleDiscountGroup",
			"remove" => "RemoveCycleDiscountGroup"
		),
		"coupon" => array(
			"add" => "AddCoupon",
			"edit" => "EditCoupon",
		)
	),
	"taxes" => array(
		"index" => array(
			"add" => "AddTaxZoneGroup",
			"edit" => "EditTaxZoneGroup"
		)
	),
	"products" => array(
        "group" => array(
            "edit" => "EditProductAddonGroup",
        ),
		"plan-overview" => array(
			"index" => "ShowProductDetails"
		),
		"plan-settings" => array(
			"edit" => "ShowProductDetails",
			"create" => "ShowProductDetails",
			"clone" => "ShowProductDetails"
		)
	)
);