<?php

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";
require_once("lib-mbapi/include/modules/gateway/HSPCGateway.php");
require_once("objects/Transaction.php");
require_once("objects/GatewayTransaction.php");

class HSPCCreditCard extends HSPCGateway
{
	public function isTransactionTypeSupported($transaction_type)
	{
		require_once("HSPC/PaymentProcess.php");
		return HSPCPaymentProcess::isTransactionTypeSupported($this->getPluginID(), $transaction_type);
	}

	public function process(Transaction $transaction, GatewayTransaction $gateway_transaction, GatewayTransaction $prev_gateway_transaction = NULL, array $request = NULL, $request_uri = NULL)
	{
		// Generate HSPC request
		require_once("HSPC/Tools.php");
		require_once("HSPC/Struct/Account.php");
		require_once("HSPC/Struct/PP/DocumentInfo.php");
		require_once("HSPC/Struct/PP/PaymentMethod.php");
		require_once("HSPC/Struct/PP/ProcessTransactionRequest.php");
		require_once("HSPC/PaymentProcess.php");

		$transactionAmount = Transaction::COMMAND_REFUND == $transaction->command || Transaction::COMMAND_VOID == $transaction->command ?
			$transaction->command_args["amount"] : $transaction->transaction_amount;

		$document_info = new HSPCStructPPDocumentInfo(array(
			"id"			=> $transaction->id,
			"number"		=> $transaction->id,
			"date"			=> $transaction->transaction_date_created,
			"total"			=> $transactionAmount,
			"subtotal"		=> $transactionAmount,
			"description"	=> "Automatically generated transaction for Credit Card processing",
		));
		$payment_method = new HSPCStructPPPaymentMethod(
			$this->getPaymentMethodAdapter(
				$transaction->ClientBillingAccount,
				$transaction->ClientBillingAccount->payment_gateway_id
			) -> getHSPCData()
		);
		$account_info = HSPCTools::getHSPCContact(
			$transaction->ClientBillingAccount->client_contact_id,
			HSPCTools::USER_CLIENT
		);
		$request_args = array(
			"config"				=> $this->getPluginConfig(),
			"transaction_type"		=> $gateway_transaction->type,
			"transaction_id"		=> $transaction->id,
			"transaction_amount"	=> $transactionAmount,
			"transaction_refnum"	=> $gateway_transaction->transaction_ref,
			"ref_no"				=> $gateway_transaction->transaction_ref,
			"document_info"			=> $document_info,
			"payment_method"		=> $payment_method,
			"account_info"			=> $account_info,
			"currency_iso"			=> $transaction->currency_id,
		);
		if (! is_null($prev_gateway_transaction)) {
			$request_args["previous_transaction_data"] = $prev_gateway_transaction->data;
		}
		if (GatewayTransaction::TYPE_CALLBACK == $gateway_transaction->type) {
			$request_args["request_context"] = $request;
			$request_args["request_uri"] = $request_uri;
		}
		$request = new HSPCStructPPProcessTransactionRequest($request_args);
		dbg(__METHOD__." request:");
		dbg(var_export($request, true));
		$response = HSPCPaymentProcess::processTransaction($this->getPluginID(), $request);
		dbg(__METHOD__." response:");
		dbg(var_export($response, true));
		return $response;
	}
}
