<?php

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";
require_once("lib-mbapi/include/modules/HSPCModule.php");
require_once("HSPC/Struct/PP/ProcessBatchContentRequest.php");
require_once("objects/ClientBillingAccount.php");

class HSPCGateway extends HSPCModule
{
	const PM_FORM_ACTION_VIEW = "view";
	const PM_FORM_ACTION_EDIT = "edit";
	const PM_FORM_ACTION_ADD = "add";

	/**
	 * INPUT:
	 *		gateway
	 *			or
	 *		gateway_id
	 *			or
	 *		billing_account_id
	 *			or
	 *		client_contact_id	- use this parameter when no billing_account_id parameter passed
	 */
	public function __construct(array $params)
	{
		parent::__construct($params);

		if (isset($this->input["gateway"]))
			$this->gateway = $this->input["gateway"];

		if (isset($this->input["billing_account_id"]))
		{
			$this->billing_account = Factory()->ClientBillingAccount($this->input["billing_account_id"]);
			$this->input["client_contact_id"] = $this->billing_account->client_contact_id;
			$this->input["gateway_id"] = $this->billing_account->payment_gateway_id;
		}

		if (is_null($this->gateway) && isset($this->input["gateway_id"]))
			$this->gateway = Factory()->PaymentGateway($this->input["gateway_id"]);

		if (is_null($this->gateway))
			throw new Exception("Can not create instance for object [".get_class($this)."]: No any info about selected gateway");

		if (! in_array($this->getPluginType(), array("BT", "CC", "TP")))
			throw new Exception("Can not create instance for object [".get_class($this)."]: Wrong type of the passed payment gateway type");

		if (isset($this->input["client_contact_id"]))
			$this->client_contact = Factory()->ClientContact($this->input["client_contact_id"]);

		// loadPaymentMethodData can be executer after gateway initialization
		if (isset($this->input["billing_account_id"]))
			$this->loadPaymentMethodData();
	}

	public function process()
	{
		$this->dieMandatoryMethod(__METHOD__);
	}

	public function getPluginType()
	{
		require_once("object_managers/PaymentGatewayManager.php");
		return PaymentGatewayManager::getHSPCModuleType($this->gateway->payment_gateway_type);
	}

	public function getPluginTemplateID()
	{
		return $this->gateway->payment_gateway_module;
	}

	public function getPluginConfig()
	{
		return FALSE === ($config = unserialize($this->decryptData($this->gateway->PaymentGatewayParams["config"]->value))) ? NULL : $config;
	}

	public function collectPluginConfig()
	{
		require_once("HSPC/PaymentProcess.php");

		$hspc_result = HSPCPaymentProcess::getCollectedData($this->getPluginID(), $this->getPluginConfig());

		if ("" != $hspc_result->data->user_error)
			return $hspc_result->data->user_error;

		$this->gateway->PaymentGatewayParams["config"]->value = $this->encryptData(serialize($hspc_result->data->plugin_data));

		return "";
	}

	public function savePluginConfig()
	{
		require_once("object_managers/PaymentGatewayManager.php");
		PaymentGatewayManager::update($this->gateway);
	}

	public function getPluginConfigForm()
	{
		require_once("HSPC/PaymentProcess.php");
		$hspc_result = HSPCPaymentProcess::getEditForm($this->getPluginID(), $this->getPluginConfig());
		return $hspc_result->data;
	}

	public function loadPaymentMethodData()
	{
		$this->payment_method_data = $this->getPaymentMethodAdapter()->getHSPCData();
	}

/*
	public function getPaymentMethodData($billing_account_id = NULL)
	{
		if (is_null($billing_account_id) && is_null($this->billing_account)) {
			throw new Exception("The 'getPaymentMethodData' needs to know billing account");
		}
		$billing_account = is_null($billing_account_id) ? $this->billing_account : Factory()->ClientBillingAccount($billing_account_id);
		$billing_account_params = $billing_account->ClientBillingAccountParams;
		return array(
			"public_data" => (FALSE === ($data = unserialize($billing_account_params["hspc_public_data"]->value)) ? NULL : $data),
			"secure_data" => (FALSE === ($data = unserialize($billing_account_params["hspc_secure_data"]->value)) ? NULL : $data),
		);
	}
*/
	public function getPaymentMethodAdapter(ClientBillingAccount $billing_account = NULL, $gateway_id = NULL)
	{
		if (is_null($billing_account) && is_null($this->billing_account)) {
			throw new Exception("The 'getPaymentMethodAdapter' needs to know billing account");
		}
		$billing_account = is_null($billing_account) ? $this->billing_account : $billing_account;

		$method = $this->getPaymentMethod();
		$method_name = $method["method_name"];
		$adapter_file = dirname(__FILE__)."/HSPCPMAdapters/{$method_name}.php";
		if (! file_exists($adapter_file))
			return NULL;

		require_once($adapter_file);
		$adapter = "HSPC_PM_{$method_name}";
		return new $adapter($billing_account, $gateway_id);
	}

	public function getPaymentMethod()
	{
		try {
			require_once("HSPC/PaymentProcess.php");
			return ! is_null($this->payment_method)
				? $this->payment_method
				: $this->payment_method = HSPCPaymentProcess::getSupportedPaymentMethod($this->getPluginID());
		} catch (SoapFault $e) {
			throw new ProductException(ProductException::E_FAIL_OPERATION, blmsg("TRANS_PBA_ERROR_MSG", array("MESSAGE" => $e->getMessage())));
		}
	}

	public function getSecureData($data)
	{
		require_once("HSPC/PaymentMethod.php");
		$method = $this->getPaymentMethod();
		$result = HSPCPaymentMethod::getSecureData($method["method_name"], $data);
		return $result->data;
	}

	public function getPublicData($data)
	{
		require_once("HSPC/PaymentMethod.php");
		$method = $this->getPaymentMethod();
		$result = HSPCPaymentMethod::getPublicData($method["method_name"], $data);
		return $result->data;
	}

	public function getPublicNumber()
	{
		require_once("HSPC/PaymentMethod.php");
		$method = $this->getPaymentMethod();
		$result = HSPCPaymentMethod::getPublicNumber(
			$method["method_name"],
			$this->payment_method_data["secure_data"],
			$this->payment_method_data["public_data"]
		);
		return $result->data;
	}

	public function encryptData($value)
	{
		return $this->getCryptor()->encrypt($value);
	}

	public function decryptData($value)
	{
		return $this->getCryptor()->decrypt($value);
	}

	public function getPluginID()
	{
		$map = array("CC" => "OP", "TP" => "OP", "BT" => "BT");
		return "{$map[$this->getPluginType()]}_{$this->getPluginTemplateID()}";
	}

	private function getCryptor()
	{
		if (! is_null($this->cryptor))
		{
			$this->cryptor->errors = array();
			return $this->cryptor;
		}

		require_once("lib-tk/include/crypt/crypt.php");
		return $this->cryptor = new Crypt($this->cryptor_secret_key);
	}

	private function dieMandatoryMethod($method)
	{
		throw new Exception("[class ".get_class($this)."] The method \"${method}\" should be overwritten by child class!");
	}

	protected $plugin_type;
	protected $billing_account;
	protected $client_contact;
	protected $payment_method_data = array(
		"public_data" => NULL,
		"secure_data" => NULL,
	);
	protected $gateway;
	protected $cryptor_secret_key = "3356897nf238hjgfjgwy79692hff62kf9qm4cxnm8vt62";

	private $payment_method;
	private $cryptor;
}
