<?php

require_once dirname(__FILE__)."/../../../../../lib-tk/shield.php";
require_once dirname(__FILE__)."/Adapter.php";
require_once "objects/ClientBillingAccount.php";
require_once "objects/PaymentGateway.php";

class HSPC_PM_OP_CCard extends HSPC_PM_Adapter
{
	public function getHSPCData()
	{
		$acc = $this->account;

		$valid_from = NULL;
		if ("" != $acc->billing_card_start_date) {
			list($from_month, $from_year) = split("/", $acc->billing_card_start_date);
			$from_year = date("Y", mktime(0, 0, 0, $from_month, 1, $from_year));
			$from_month = date("m", mktime(0, 0, 0, $from_month, 1, $from_year));
			$valid_from = "{$from_year}-{$from_month}-01 00:00:00";
		}
		$public_data = array("valid_from"	=> $valid_from);

		$contact = $acc->ClientContact;
		list($holder_name_1, $holder_name_2) = split(" ", $acc->billing_card_name_on_card);

		require_once("Phone.php");
		$phone = new Phone($contact->contact_phone1);

		$secure_data = array(
			"card_number"	=> $acc->billing_card_num,
			"holder_name"	=> $holder_name_1,
			"holder_name2"	=> $holder_name_2,
			"cvv"			=> $acc->cardCVV2,
			"issue_number"	=> $acc->billing_card_issue_num,
			"address1"		=> $contact->contact_address_1,
			"address2"		=> $contact->contact_address_2,
			"city"			=> $contact->contact_city,
			"country"		=> $contact->Country->countries_iso_2,
			"zip"			=> $contact->contact_zip,
			"phone_src"		=> (array)$phone,
			"phone"			=> $phone,
		);
		$state_map = array("US" => "state", "CA" => "state_uscanada", $contact->Country->countries_iso_2 => NULL);
		if (! is_null($state_field = $state_map[$contact->Country->countries_iso_2])) {
			$secure_data[$state_field] = $contact->contact_state;
		} else {
			$secure_data["state"] = "";
		}

		return array(
			"public_data"	=> $public_data,
			"secure_data"	=> $secure_data,
			"expire_date"	=> sprintf("%04d-%02d-01 00:00:00", $acc->billing_card_exp_year, $acc->billing_card_exp_month),
			"type"			=> $this->creditCardTypesMap[$acc->billing_card_type],
		);
	}

	public function getSupportedCardTypes()
	{
		$plugin = ModuleTool::getModuleInstance("gateway", "HSPCBankTransfer", array("gateway_id" => $this->gateway_id));
		$payment_method = $plugin->getPaymentMethod();

		$result = array();
		$reverse_map = array_flip($this->creditCardTypesMap);
		foreach ($payment_method["method_types"] as $hspc_method_type) {
			$result[] = $reverse_map[$hspc_method_type];
		}
		return $result;
	}

	protected function getLastFourDigit()
	{
		$hspc_data = $this->getHSPCData();
		return substr($hspc_data["secure_data"]["card_number"], -4);
	}

	private $creditCardTypesMap = array(
		PaymentGateway::TYPE_CARD_VISA			=> "V",
		PaymentGateway::TYPE_CARD_MASTERCARD	=> "M",
		PaymentGateway::TYPE_CARD_AMEX			=> "A",
		PaymentGateway::TYPE_CARD_DISCOVER		=> "D",
		PaymentGateway::TYPE_CARD_JCB			=> "J",
		PaymentGateway::TYPE_CARD_ENROUTE		=> "E",
		PaymentGateway::TYPE_CARD_DINERS		=> "C",
		PaymentGateway::TYPE_CARD_SOLO			=> "S",
		PaymentGateway::TYPE_CARD_MAESTRO		=> "W",
		PaymentGateway::TYPE_CARD_DELTA			=> "T",
		PaymentGateway::TYPE_CARD_BANKCARD		=> "B",
		PaymentGateway::TYPE_CARD_ELECTRON		=> "L",
	);
}