<?php

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";
require_once("lib-mbapi/include/modules/gateway/HSPCGateway.php");
require_once("objects/Transaction.php");
require_once("objects/GatewayTransaction.php");
require_once("objects/Invoice.php");
require_once("objects/ClientBillingAccount.php");

class HSPCThirdParty extends HSPCGateway
{
	public function isTransactionTypeSupported($transaction_type)
	{
		require_once("HSPC/PaymentProcess.php");
		return HSPCPaymentProcess::isTransactionTypeSupported($this->getPluginID(), $transaction_type);
	}

	public function processRequest(Invoice $invoice, ClientBillingAccount $account, $refnum, $amount)
	{
		// Generate HSPC request
		require_once("HSPC/Tools.php");
		require_once("HSPC/Struct/Account.php");
		require_once("HSPC/Struct/PP/DocumentInfo.php");
		require_once("HSPC/Struct/PP/ProcessTransactionRequest.php");
		require_once("HSPC/PaymentProcess.php");

		$document_info = new HSPCStructPPDocumentInfo(array(
			"id"			=> $invoice->id,
			"number"		=> $invoice->invoice_number,
			"date"			=> $invoice->invoice_date_entered,
			"total"			=> $amount,
			"subtotal"		=> $amount,
			"description"	=> "Automatically generated transaction for ThirdParty gateway",
		));
		$account_info = HSPCTools::getHSPCContact(
			$account->client_contact_id,
			HSPCTools::USER_CLIENT
		);
		$request_args = array(
			"config"				=> $this->getPluginConfig(),
			"transaction_type"		=> GatewayTransaction::TYPE_SALE,
			"transaction_id"		=> $invoice->id,
			"transaction_amount"	=> $amount,
			"transaction_refnum"	=> $refnum,
			"ref_no"				=> $refnum,
			"document_info"			=> $document_info,
			"account_info"			=> $account_info,
			"currency_iso"			=> $invoice->currency_id,
		);
		$request = new HSPCStructPPProcessTransactionRequest($request_args);
		dbg(__METHOD__." request:");
		dbg(var_export($request, true));
		$response = HSPCPaymentProcess::processTransaction($this->getPluginID(), $request);
		dbg(__METHOD__." response:");
		dbg(var_export($response, true));
		return $response;
		// TODO
	}

	public function processTransaction(
		Invoice $invoice,
		$amount,
		GatewayTransaction $gateway_transaction,
		GatewayTransaction $prev_gateway_transaction = NULL,
		array $request = NULL,
		$request_uri = NULL
	) {
		// Generate HSPC request
		require_once("HSPC/Tools.php");
		require_once("HSPC/Struct/Account.php");
		require_once("HSPC/Struct/PP/DocumentInfo.php");
		require_once("HSPC/Struct/PP/ProcessTransactionRequest.php");

		$document_info = new HSPCStructPPDocumentInfo(array(
			"id"			=> $invoice->id,
			"number"		=> $invoice->invoice_number,
			"date"			=> $invoice->invoice_date_entered,
			"total"			=> $amount,
			"subtotal"		=> $amount,
			"description"	=> "Automatically generated transaction for ThirdParty gateway",
		));
		$account_info = HSPCTools::getHSPCContact(
			Factory()->ClientBillingAccount($gateway_transaction->billing_account_id)->client_contact_id,
			HSPCTools::USER_CLIENT
		);
		$request_args = array(
			"config"				=> $this->getPluginConfig(),
			"transaction_type"		=> $gateway_transaction->type,
			"transaction_id"		=> $invoice->id,
			"transaction_amount"	=> $amount,
			"transaction_refnum"	=> $gateway_transaction->transaction_ref,
			"ref_no"				=> $gateway_transaction->transaction_ref,
			"document_info"			=> $document_info,
			"account_info"			=> $account_info,
			"currency_iso"			=> $invoice->currency_id,
		);
		if (! is_null($prev_gateway_transaction)) {
			$request_args["previous_transaction_data"] = $prev_gateway_transaction->data;
		}
		if (GatewayTransaction::TYPE_CALLBACK == $gateway_transaction->type) {
			$request_args["request_context"] = $request;
			$request_args["request_uri"] = $request_uri;
		}
		$request = new HSPCStructPPProcessTransactionRequest($request_args);
		dbg(__METHOD__." request:");
		dbg(var_export($request, true));
		$response = HSPCPaymentProcess::processTransaction($this->getPluginID(), $request);
		dbg(__METHOD__." response:");
		dbg(var_export($response, true));
		return $response;
	}

	public function collectReferenceNumber($pluginID, array $request)
	{
		require_once("HSPC/PaymentProcess.php");
		return HSPCPaymentProcess::collectReferenceNumber($pluginID, $request);
	}
}
