<?php

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";

require_once mb_path("/lib-mbapi/include/modules/gateway/gateway.php");

/**
 * This class is an instantion of the Gateway class, that executes a command pertaining
 * to comunicating with a payment gateway, in the default case from the database. Pass in
 * only the array parameters that are set.
 *
 *<module>
 * <gateway>
 *  <authorizenet>
 *     <params>
 *       <configGroupName></configGroupName>						(Required: Configuration instance of AuthorizeNet module)
 *       <subCommand>charge</subCommand>       						(Required: One of charge, auth, finalize, refund)
 *       <invoiceID></invoiceID>
 *       <authCode></authCode>										(Required for finalize)
 *       <transactionCode></transactionCode>						(Required for refund)
 *       <paymentDescription></paymentDescription>					(Required)
 *       <paymentAmount></paymentAmount>							(Required: 0.00)
 *       <paymentMethod></paymentMethod>							(Required: CC, echeck)
 *       <isRecurring></isRecurring>								(0,1)
 *       <billingAccountID></billingAccountID>			(Enter either this or billing info below)
 *       <clientContactID></clientContactID>									(Enter either this or contact info below)
 *       <billingCardNum></billingCardNum>
 *       <billingCardExpYear></billingCardExpYear>
 *       <billingCardExpMonth></billingCardExpMonth>
 *       <cardCVV2>222</cardCVV2>
 *       <billingBankName></billingBankName>
 *       <billingBankAbaCode></billingBankAbaCode>
 *       <billingBankAccountNum></billingBankAccountNum>
 *       <billingBankLicenseNum></billingBankLicenseNum>
 *       <billingBankDOB></billingBankDOB>
 *       <contactFirstName></contactFirstName>
 *       <contactLastName></contactLastName>
 *       <contactCompany></contactCompany>
 *       <contactAddress1></contactAddress1>
 *       <contactAddress2></contactAddress2>
 *       <contactCity></contactCity>
 *       <contactState></contactState>
 *       <contactZip></contactZip>
 *       <countriesID></countriesID>
 *       <contactPhone1></contactPhone1>
 *       <contactFax></contactFax>
 *       <contactEmail></contactEmail>
 *     </params>
 *   </authorizenet>
 * </gateway>
 *<module>
 *
 * @package modernbill
 * @subpackage mbapi
 */

$className = "AuthorizeNet";
$GLOBALS["moduleInfo"][$className] = array
(
    "type" => "gateway",
    "status" => PaymentGateway::STATUS_STABLE,
    "version" => "1.0.2",
    "gatewaytype" => "1",
    "displayName" => "AuthorizeNet",
    "author" => "Parallels",
    "capabilities" => array(GATEWAY_CHARGE, GATEWAY_AUTHORIZE, GATEWAY_PRIOR_AUTH_CHARGE, GATEWAY_REFUND, GATEWAY_PARTIAL_REFUND, GATEWAY_VOID, GATEWAY_IPV6_COMPLIANCE),
    "countryCodes" => array("US", "CA"),
);

class AuthorizeNet extends Gateway {

	/**
	 * Returns the module display for the BluePay module. This includes version number and display name.
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the display info for the module.
	 */
    function getModuleInfo() {
        return $GLOBALS["moduleInfo"]["AuthorizeNet"];
    }

	/**
	 * Execute the command based upon the given information, and return the result.
	 *
	 * @return  The result of the AuthorizeNet gateway command based upon the given information.
	 */
	 function AuthorizeNet($params){
	 $this->stringRep = "authorizenet";
	 	parent::Gateway($params);

        $moduleInfo = $this->getModuleInfo();

        foreach ($moduleInfo["capabilities"] as $cap){
            $this->capabilities[$cap] = 1;
        }
		$this->version = "1.0.0";
		$this->liveURL = "https://secure.authorize.net/gateway/transact.dll";
		$this->testURL = "https://test.authorize.net/gateway/transact.dll";
	 }

	/**
	 * Returns the default configuration parameters for the BluePay module. The number of elements in each array must me identical to
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the parameters the module uses.
	 */
	function addDefaultConfigParams() {
		static $defaultConfigParams = array(
			array(NULL, NULL, "Login_ID", "testdrive", "", "TRANS_AUTHNET_LOGINID", "TRANS_AUTHNET_LOGINID_DESC", "t", 1, 1, 0, null),
			array(NULL, NULL, "Password", "testdrive", "", "TRANS_AUTHNET_PASSWORD", "TRANS_AUTHNET_PASSWORD_DESC", "p", 1, 2, 0, null),
			array(NULL, NULL, "Version", "3.1", "", "TRANS_AUTHNET_VERSION","TRANS_AUTHNET_VERSION_DESC", "t", 1, 3, 0, null),
			array(NULL, NULL, "Invoice_Description", "", "", "TRANS_INVOICE_DESCRIPTION", "TRANS_INVOICE_DESCRIPTION_DESC", "t", 0, 4, 0, null),
			array(NULL, NULL, "Generic_Emails", "0", "", "TRANS_GATEWAY_GEN_EMAIL","TRANS_GATEWAY_GEN_EMAIL_DESC", "b", 0, 5, 0, null),
			array(NULL, NULL, "Batch_Recurring", "1", "", "TRANS_BATCH_RECURRING","TRANS_BATCH_RECURRING_DESC", "b", 1, 6, 0, null),
            array(NULL, NULL, "Testing", "1", "", "TRANS_GATEWAY_TESTING", "TRANS_GATEWAY_TESTING_DESC", "b", 1, 7, 0, null),
            array(NULL, NULL, "Gateway_URL", "1", "", "TRANS_GATEWAY_URL", "TRANS_GATEWAY_URL_DESC", "b", 1, 7, 0, null),
            array(NULL, NULL, "card_accept_0", "1", "", "TRANS_ACCEPT_VISA", "TRANS_ACCEPT_VISA_DESC", "b", 1, 8, 0, null),
            array(NULL, NULL, "card_accept_1", "1", "", "TRANS_ACCEPT_MASTERCARD", "TRANS_ACCEPT_MASTERCARD_DESC", "b", 1, 9, 0, null),
            array(NULL, NULL, "card_accept_2", "1", "", "TRANS_ACCEPT_AMEX", "TRANS_ACCEPT_AMEX_DESC", "b", 1, 10, 0, null),
            array(NULL, NULL, "card_accept_3", "1", "", "TRANS_ACCEPT_DISCOVER", "TRANS_ACCEPT_DISCOVER_DESC", "b", 1, 11, 0, null),
            array(NULL, NULL, "card_accept_4", "1", "", "TRANS_ACCEPT_JCB", "TRANS_ACCEPT_JCB_DESC", "b", 1, 12, 0, null),
            array(NULL, NULL, "accept_echeck", "1", "", "TRANS_ACCEPT_ECHECK", "TRANS_ACCEPT_ECHECK_DESC", "b", 1, 13, 0, null),
            );
		return $defaultConfigParams;
	}

	 /**
	  *
	  * From a child module this function must do the following:
	  *   - Maintain $this->hasExecuted, $this->hasExecutedSuccessfully, $this->timestamp,
	  *     $this->status, $this->accountID
	  *   - Add any queries to the debug list with addDebug()
	  *   - Add any errors to the error list with addError()
	  *   - Return $this->returnData, or the appropriate error constant
	  *
	  * @return  Returns feedback from execution of this Gateway object, or an error constant defined
	  *          at the top of this file.
	  */
	function execute(){
		$this->simulateRefundsForTestURL = 1;
		return $this->runAuthNETProtocol();
	} //execute

} // AuthorizeNet

/* Debug
$debug = "
<module>
  <gateway>
    <authorizenet>
      <params>
        <configGroupName>0</configGroupName>
        <subCommand>charge</subCommand>
        <invoiceID>135</invoiceID>
        <authCode></authCode>
        <transactionCode></transactionCode>
        <paymentDescription>ModernBill Test Service</paymentDescription>
        <paymentAmount>1.54</paymentAmount>
        <paymentMethod>CC</paymentMethod>
        <isRecurring>1</isRecurring>
        <billingAccountID></billingAccountID>
        <clientContactID></clientContactID>
        <billingCardNum>5400000000000005</billingCardNum>
        <billingCardExpYear>2007</billingCardExpYear>
        <billingCardExpMonth>1</billingCardExpMonth>
        <cardCVV2>222</cardCVV2>
        <billingBankName></billingBankName>
        <billingBankAbaCode></billingBankAbaCode>
        <billingBankAccountNum></billingBankAccountNum>
        <billingBankLicenseNum></billingBankLicenseNum>
        <billingBankDOB></billingBankDOB>
        <contactFirstName>Mark</contactFirstName>
        <contactLastName>Garrett</contactLastName>
        <contactCompany>ModernGigabyte</contactCompany>
        <contactAddress1>201 E Jefferson</contactAddress1>
        <contactAddress2>Ste 201</contactAddress2>
        <contactCity>Louisville</contactCity>
        <contactState>KY</contactState>
        <contactZip>40202</contactZip>
        <countriesID>US</countriesID>
        <contactPhone1>502.566.7754</contactPhone1>
        <contactFax>502.566.7724</contactFax>
        <contactEmail>mark@modernbill.com</contactEmail>
      </params>
    </authorizenet>
  </gateway>
</module>
";
$payment = new AuthorizeNet($debug);
$CONF["debug"]->add("MBAPI Result", htmlspecialchars($payment->getData()));
echo "<pre>".$CONF['debug']->show()."</pre>";
/* */
