<?php

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";
require_once mb_path("/lib-mbapi/include/modules/gateway/gateway.php");

class DemoGatewayModule extends Gateway
{
	const status		= PaymentGateway::STATUS_STABLE;
	const version		= '1.0.0';
	const gatewaytype	= '1';
	const displayName	= 'Demo Gateway';
	const author		= 'Parallels';

	private static $moduleCapabilities = array(
		GATEWAY_CHARGE,
		GATEWAY_AUTHORIZE,
		GATEWAY_PRIOR_AUTH_CHARGE,
		GATEWAY_REFUND,
		GATEWAY_ECHECK,
		GATEWAY_PARTIAL_REFUND,
		GATEWAY_VOID,
		GATEWAY_IPV6_COMPLIANCE,
	);

	public static function staticGetModuleInfo()
	{
		return array(
			'type' => 'gateway',
 			'status' => self::status,
			'version' => self::version,
			'gatewaytype' => self::gatewaytype,
			'displayName' => self::displayName,
			'author' => self::author,
			'capabilities' => self::$moduleCapabilities,
		);
	}

	// not static because this is overriding a non-static method in the Gateway class
	public function getModuleInfo()
	{
		return self::staticGetModuleInfo();
	}

	public function addDefaultConfigParams()
	{
		static $defaultConfigParams = array(
			array(NULL, NULL, "result", "0", "", "TRANS_GATEWAY_TESTING", "TRANS_GATEWAY_TEST_RESULT_DESC", "s", 1, 4, 0, array
				(
					array(NULL, NULL, "0", "TRANS_TESTING_RESULT_BASED_ON_NAME", 0),
					array(NULL, NULL, "1", "TRANS_TESTING_ALWAYS_APPROVE", 1),
					array(NULL, NULL, "2", "TRANS_TESTING_ALWAYS_DECLINE", 2),
					array(NULL, NULL, "3", "TRANS_TESTING_ALWAYS_ERROR", 3),
					array(NULL, NULL, "4", "TRANS_TESTING_RANDOM_RESULT", 4),
				)),
			array(NULL, NULL, "card_accept_0", "1", "", "TRANS_ACCEPT_VISA", "TRANS_ACCEPT_VISA_DESC", "b", 1, 8, 0, NULL),
			array(NULL, NULL, "card_accept_1", "1", "", "TRANS_ACCEPT_MASTERCARD", "TRANS_ACCEPT_MASTERCARD_DESC", "b", 1, 9, 0, NULL),
			array(NULL, NULL, "card_accept_2", "1", "", "TRANS_ACCEPT_AMEX", "TRANS_ACCEPT_AMEX_DESC", "b", 1, 10, 0, NULL),
			array(NULL, NULL, "card_accept_3", "1", "", "TRANS_ACCEPT_DISCOVER", "TRANS_ACCEPT_DISCOVER_DESC", "b", 1, 11, 0, NULL),
			array(NULL, NULL, "card_accept_4", "1", "", "TRANS_ACCEPT_JCB", "TRANS_ACCEPT_JCB_DESC", "b", 1, 12, 0, NULL),
			array(NULL, NULL, "card_accept_5", "1", "", "TRANS_ACCEPT_ENROUTE", "TRANS_ACCEPT_ENROUTE_DESC", "b", 1, 13, 0, NULL),
			array(NULL, NULL, "card_accept_6", "1", "", "TRANS_ACCEPT_DINERS", "TRANS_ACCEPT_DINERS_DESC", "b", 1, 14, 0, NULL),
			array(NULL, NULL, "card_accept_7", "1", "", "TRANS_ACCEPT_SOLO", "TRANS_ACCEPT_SOLO_DESC", "b", 1, 15, 0, NULL),
			array(NULL, NULL, "card_accept_8", "1", "", "TRANS_ACCEPT_MAESTRO", "TRANS_ACCEPT_MAESTRO_DESC", "b", 1, 16, 0, NULL),
			array(NULL, NULL, "card_accept_9", "1", "", "TRANS_ACCEPT_DELTA", "TRANS_ACCEPT_DELTA_DESC", "b", 1, 17, 0, NULL),
			array(NULL, NULL, "card_accept_10", "1", "", "TRANS_ACCEPT_BANKCARD", "TRANS_ACCEPT_BANKCARD_DESC", "b", 1, 18, 0, NULL),
			array(NULL, NULL, "card_accept_11", "1", "", "TRANS_ACCEPT_ELECTRON", "TRANS_ACCEPT_ELECTRON_DESC", "b", 1, 19, 0, NULL),
			array(NULL, NULL, "accept_echeck", "1", "", "TRANS_ACCEPT_ECHECK", "TRANS_ACCEPT_ECHECK_DESC", "b", 1, 20, 0, NULL),
		);
		return $defaultConfigParams;
	}

	public function execute()
	{
		if ((int)$this->config['result']==0){ // result based on name
			$this->config['result'] = 3; // lets default to error
			$map = array(
				'success' => 1,
				'approve' => 1,
				'approval' => 1,
				'decline' => 2,
				'declined' => 2,
				'reject' => 2,
				'error' => 3,
				'failure' => 3,
			);
			$name = trim(strtolower($this->params['billingCardNameOnCard'][0]));
			foreach ($map as $term=>$result){
				if (strpos($name, $term)===0){
					$this->config['result'] = $result;
					break;
				}
			}
		} elseif ((int)$this->config['result']==4){ // random result
			$this->config['result'] = rand(1,3);
		}
		$transactionID = $authorizationCode = $this->params['invoiceID'][0];
		switch ((int)$this->config['result']){
			case 1:
				$status = GATEWAY_SUCCESS;
				$message = 'Transaction Approved';
				break;
			case 2:
				$status = GATEWAY_DECLINED;
				$message = 'Transaction Declined';
				break;
			case 3:
				$status = GATEWAY_FAILURE;
				$message = 'Gateway Error';
				break;
		}
		return $this->createOutputXML(array(
			"status"		=> $status,
			"statusName"		=> getAPIStatusName("transactions", $status),
			"statusMessage"		=> getStatusName("transactions", $status),
			"authorizationCode"	=> $authorizationCode,
			"AVSCode"		=> 0,
			"transactionID"		=> $transactionID,
			"message"		=> $message,
		));
	}
}

$GLOBALS["moduleInfo"][$className = "DemoGatewayModule"] = DemoGatewayModule::staticGetModuleInfo();
