<?php

require_once(dirname(__FILE__)."/../../../../lib-tk/shield.php");
require_once("lib-mbapi/include/modules/gateway/gateway.php");

/**
* This class is an instantion of the Gateway class, that executes a command pertaining
* to comunicating with a payment gateway, in the default case from the database. Pass in
* only the array parameters that are set.
*
* <gateway>
*  <echoinc>
*     <params>
*       <configGroupName></configGroupName>                                            (Required: Configuration instance of Echo module)
*       <subCommand>charge</subCommand>                                                (Required: One of charge, auth, finalize, refund)
*       <invoiceID></invoiceID>
*       <authCode></authCode>                                                                          (Required for finalize)
*       <transactionCode></transactionCode>                                            (Required for refund)
*       <paymentDescription></paymentDescription>                                      (Required)
*       <paymentAmount></paymentAmount>                                                        (Required: 0.00)
*       <paymentMethod></paymentMethod>                                                        (Required: CC, echeck)
*       <isRecurring></isRecurring>                                                            (0,1)
*       <billingAccountID></billingAccountID>                  (Enter either this or billing info below)
*       <clientContactID></clientContactID>                                                                    (Enter either this or contact info below)
*     </params>
*   </echoinc>
* </gateway>
*
* @package modernbill
* @subpackage mbapi
*/

$className = "EchoInc";
$GLOBALS["moduleInfo"][$className] = array
(
	"type" => "gateway",
	"status" => PaymentGateway::STATUS_TESTING,
	"version" => "1.0.1",
	"gatewaytype" => "1",
	"displayName" => "EchoInc",
	"author" => "Parallels",
	"capabilities" => array(GATEWAY_CHARGE, GATEWAY_IPV6_COMPLIANCE),
	"countryCodes" => array("US"),
);

class EchoInc extends Gateway
{
		/**
		* Returns the module display for the BluePay module. This includes version number and display name.
		* the columns in config_params. This array will be looped by SetConfigGroups.
		*
		* @return      An array of arrays containing all the display info for the module.
		*/
	function getModuleInfo() {
		return $GLOBALS["moduleInfo"]["EchoInc"];
	}

		/**
		* Execute the command based upon the given information, and return the result.
		*
		* @return  The result of the EchoInc gateway command based upon the given information.
		*/
		function EchoInc($params){
			$this->stringRep = "echoinc";
			global $CONF;
			parent::Gateway($params);

			$moduleInfo = $this->getModuleInfo();

			foreach ($moduleInfo["capabilities"] as $cap){
				$this->capabilities[$cap] = 1;
			}
			$this->version = "1.0.1";
			$this->liveURL = "https://wwws.echo-inc.com/scripts/INR200.EXE";
			$this->testURL = "https://wwws.echo-inc.com/scripts/INR200.EXE";

		}

		/**
		* Returns the default configuration parameters for the BluePay module. The number of elements in each array must me identical to
		* the columns in config_params. This array will be looped by SetConfigGroups.
		*
		* @return      An array of arrays containing all the parameters the module uses.
		*/
		function addDefaultConfigParams() {
				static $defaultConfigParams = array(
						array(NULL, NULL, "Merchant_ID", "", "", "TRANS_ECHOINC_MERCHANTID", "TRANS_ECHOINC_MERCHANTID_DESC", "t", 1, 1, 0, NULL),
						array(NULL, NULL, "Merchant_Pin", "", "", "TRANS_ECHOINC_MERCHANTPIN", "TRANS_ECHOINC_MERCHANTPIN_DESC", "t", 1, 2, 0, NULL),
						array(NULL, NULL, "Testing", "1", "", "TRANS_GATEWAY_TESTING", "TRANS_GATEWAY_TESTING_DESC", "b", 1, 3, 0, NULL),
						array(NULL, NULL, "Invoice_Description", "", "", "TRANS_INVOICE_DESCRIPTION", "TRANS_INVOICE_DESCRIPTION_DESC", "t", 0, 4, 0, NULL),
			array(NULL, NULL, "card_accept_0", "1", "", "TRANS_ACCEPT_VISA", "TRANS_ACCEPT_VISA_DESC", "b", 1, 8, 0, NULL),
			array(NULL, NULL, "card_accept_1", "1", "", "TRANS_ACCEPT_MASTERCARD", "TRANS_ACCEPT_MASTERCARD_DESC", "b", 1, 9, 0, NULL),
			array(NULL, NULL, "card_accept_2", "1", "", "TRANS_ACCEPT_AMEX", "TRANS_ACCEPT_AMEX_DESC", "b", 1, 10, 0, NULL),
			array(NULL, NULL, "card_accept_3", "1", "", "TRANS_ACCEPT_DISCOVER", "TRANS_ACCEPT_DISCOVER_DESC", "b", 1, 11, 0, NULL),
			);
				return $defaultConfigParams;
		}

		/**
		*
		* From a child module this function must do the following:
		*   - Maintain $this->hasExecuted, $this->hasExecutedSuccessfully, $this->timestamp,
		*     $this->status, $this->accountID
		*   - Add any queries to the debug list with addDebug()
		*   - Add any errors to the error list with addError()
		*   - Return $this->returnData, or the appropriate error constant
		*
		* @return  Returns feedback from execution of this Gateway object, or an error constant defined
		*          at the top of this file.
		*/
		function execute(){

			global $db, $CONF;
			$postFields = null;

			dbg($this->config);

			if ($this->status === GATEWAY_PARAM_FAILURE) {
			return $this->status;
			}

			switch ($this->params['subCommand'][0]){
				case "charge":
				break;
				case "auth":
				break;
				case "finalize":
				break;
				case "refund":
				break;
				default:
			}

			// Format postFields
			$postFields  = "billing_ip_address=". $this->getRemoteIP() . "&";
			$postFields .= "product_description=". $this->params['paymentDescription'][0]."&";
			$postFields .= "cc_number=".           $this->params['billingCardNum'][0]."&";
			$postFields .= "ccexp_month=".         $this->params['billingCardExpMonth'][0]."&";
			$postFields .= "ccexp_year=".          $this->params['billingCardExpYear'][0]."&";
			if ($this->params['cardCVV2'][0]) { $postFields .= "cnp_security=".$this->params['cardCVV2'][0]."&"; }
			$postFields .= "grand_total=".         $this->params['paymentAmount'][0]."&";
			$postFields .= "billing_first_name=".  $this->params['contactFirstName'][0]."&";
			$postFields .= "billing_last_name=".   $this->params['contactLastName'][0]."&";
			$postFields .= "billing_company_name=".$this->params['contactCompany'][0]."&";
			$postFields .= "billing_address1=".    $this->params['contactAddress1'][0]."&";
			if ($this->params['contactAddress2'][0]) { $postFields .= "billing_address2=".  $this->params['contactAddress2'][0]."&"; }
			$postFields .= "billing_city=".        $this->params['contactCity'][0]."&";
			$postFields .= "billing_state=".       $this->params['contactState'][0]."&";
			$postFields .= "billing_zip=".         $this->params['contactZip'][0]."&";
			$postFields .= "billing_country=".     $this->params['countriesID'][0]."&";
			$postFields .= "billing_phone=".       $this->params['contactPhone1'][0]."&";
			$postFields .= "billing_fax=".         $this->params['contactFax'][0]."&";
			$postFields .= "billing_email=".       $this->params['contactEmail'][0]."&";

			$transaction_type = "EV";
			$order_type = "S";

			$cnp_recurring = ($this->params['isRecurring'][0]) ? "Y" : "N" ;

			if ($this->config["Testing"]) {
				$url = $this->testURL;
				dbg("Post URL (TEST)", $url);
				$echo_debug = "C";
			} else {
				$url = $this->liveURL;
				dbg("Post URL (LIVE)", $url);
				$echo_debug = "F";
			}

			$merchant_echo_id = $this->config["Merchant_ID"];
			$merchant_pin = $this->config["Merchant_Pin"];

			@mt_srand((double)microtime()*1000000);
			$counter = mt_rand();
			$postFields  = "transaction_type=$transaction_type&cnp_recurring=$cnp_recurring&order_type=$order_type&counter=$counter&debug=$echo_debug&merchant_echo_id=$merchant_echo_id&merchant_pin=$merchant_pin&$postFields";
			$postFields  = str_replace("'","",$postFields);

			dbg("Post Fields String", $postFields);

			if(!$postFields) {
				$this->addError(TRANS_MBAPIERROR, "No parameters given Gateway.");
			} else {
				$status = NULL;
				// Call the cURL class and execute the call
				$curlcall = $this->getCurlObject($url, $postFields);

				$info = $this->getModuleInfo();
				$product = "MB5" . $info["displayName"] . "Module/" . $info["version"];
				$agent = $product . " (cURL " . $curlcall->getCurlVersion() . ")";
				$curlcall->addHeader("User-Agent: " . $agent);

				$curlResult = $curlcall->toString();
				$curlResult = $curlcall->resultString;

				dbg("Raw cURL Object:");
				dbg($curlcall);
				dbg("Raw cURL Return: ".$curlResult);

				$startpos = strpos($curlResult, "<ECHOTYPE3>") + 11;
				$endpos = strpos($curlResult, "</ECHOTYPE3>");
				$echotype3 = substr($curlResult, $startpos, $endpos - $startpos);

				$this->message = $this->GetEchoProp($echotype3, 'decline_code');
				$this->authorizationCode = $this->GetEchoProp($echotype3, 'auth_code');
				$this->AVSCode = $this->GetEchoProp($echotype3, 'avs_result');
				$this->transactionID = $this->GetEchoProp($echotype3, 'order_number');

				switch ($this->GetEchoProp($echotype3, 'status')) {
					case "G": $this->status = GATEWAY_SUCCESS; break;
					case "D": $this->status = GATEWAY_DECLINED; break;
					default: $this->status = GATEWAY_FAILURE; break;
				}

				if ($curlResult !="") {
					$this->hasExecuted = 1;
				} else {
					$this->status = GATEWAY_CONNECTION_FAILURE;
				}
			}
			$this->hasExecuted = 1;

			$retval .= "   <gatewayReturn>\n";
			$retval .= "     <status>".$this->status."</status>\n";
			$retval .= "     <authorizationCode>".$this->authorizationCode."</authorizationCode>\n";
			$retval .= "     <AVSCode>".$this->AVSCode."</AVSCode>\n";
			$retval .= "     <transactionID>".$this->transactionID."</transactionID>\n";
			$retval .= "     <message>".$this->message."</message>\n";
			$retval .= "   </gatewayReturn>\n";
			$this->xmlData = $retval;
			return $retval;
		} //execute

		// function to get stuff from results :-)
		function GetEchoProp($haystack, $prop) {
			dbg("HAYSTACK! ".$haystack);
			if (($start_pos = strpos(strtolower($haystack), "<$prop>"))!==false) {
				$start_pos += strlen("<$prop>");
				$end_pos = strpos(strtolower($haystack), "</$prop", $start_pos);
				$x = substr($haystack, $start_pos, $end_pos - $start_pos);
				dbg($x);
				return htmlentities(strip_tags($x));
			} else {
				return "";
			}
		}
}
