<?php

require_once("lib-mbapi/include/modules/gateway/gateway.php");

$className = "eWAY";
$GLOBALS["moduleInfo"][$className] = array(
    "type" => "gateway",
    "status" => PaymentGateway::STATUS_STABLE,
    "version" => "1.0.1",
    "gatewaytype" => "1",
    "displayName" => "eWAY",
    "author" => "Parallels",
    "capabilities" => array(GATEWAY_CHARGE, GATEWAY_IPV6_COMPLIANCE),
    "countryCodes" => array("AU"),
);

class eWAY extends Gateway
{
	public $liveURLWithCVN = "https://www.eway.com.au/gateway_cvn/xmlpayment.asp";
	public $testURLWithCVN = "https://www.eway.com.au/gateway_cvn/xmltest/testpage.asp";

	public function getModuleInfo()
	{
        return $GLOBALS["moduleInfo"]["eWAY"];
    }

	public function __construct($params)
	{
		parent::Gateway($params);
		$this->liveURL = "https://www.eway.com.au/gateway/xmlpayment.asp";
		$this->testURL = "https://www.eway.com.au/gateway/xmltest/testpage.asp";
	}

	public function addDefaultConfigParams()
	{
		static $defaultConfigParams = array(
			array(NULL, NULL, "Customer_ID", "", "", "TRANS_EWAY_CUSTOMERID", "TRANS_EWAY_CUSTOMERID_DESC", "t", 1, 1, 0, NULL),
			array(NULL, NULL, "Testing", "1", "", "TRANS_GATEWAY_TESTING", "TRANS_GATEWAY_TESTING_DESC", "b", 1, 4, 0, NULL),
			array(NULL, NULL, "Invoice_Description", "", "", "TRANS_INVOICE_DESCRIPTION", "TRANS_INVOICE_DESCRIPTION_DESC", "a", 0, 3, 0, NULL),
            array(NULL, NULL, "card_accept_0", "1", "", "TRANS_ACCEPT_VISA", "TRANS_ACCEPT_VISA_DESC", "b", 1, 4, 0, NULL),
            array(NULL, NULL, "card_accept_1", "1", "", "TRANS_ACCEPT_MASTERCARD", "TRANS_ACCEPT_MASTERCARD_DESC", "b", 1, 5, 0, NULL),
            array(NULL, NULL, "card_accept_2", "1", "", "TRANS_ACCEPT_AMEX", "TRANS_ACCEPT_AMEX_DESC", "b", 1, 6, 0, NULL),
            array(NULL, NULL, "card_accept_3", "1", "", "TRANS_ACCEPT_DISCOVER", "TRANS_ACCEPT_DISCOVER_DESC", "b", 1, 7, 0, NULL),
            array(NULL, NULL, "card_accept_4", "1", "", "TRANS_ACCEPT_JCB", "TRANS_ACCEPT_JCB_DESC", "b", 1, 8, 0, NULL),
            array(NULL, NULL, "card_accept_6", "1", "", "TRANS_ACCEPT_DINERS", "TRANS_ACCEPT_DINERS_DESC", "b", 1, 9, 0, NULL),
            array(NULL, NULL, "card_accept_10", "1", "", "TRANS_ACCEPT_BANKCARD", "TRANS_ACCEPT_BANKCARD_DESC", "b", 1, 10, 0, NULL),
            );
		return $defaultConfigParams;
	}

	public function execute()
	{
	 	$postFields = null;

		if ($this->status === GATEWAY_PARAM_FAILURE) {
			return $this->status;
		}

		dbg($this);

		$exp_year  = date("y",mktime(0,0,0,$this->params['billingCardExpMonth'][0],1,$this->params['billingCardExpYear'][0]));
		$exp_month = date("m",mktime(0,0,0,$this->params['billingCardExpMonth'][0],1,$exp_year));
		if ($this->params['billingCardNameOnCard'][0]) {
			$fullName= $this->params['billingCardNameOnCard'][0];
		} else {
			$fullName= billing_get_full_name($this->params['contactFirstName'][0], $this->params['contactLastName'][0]);
		}

		$postFields  = "<ewaygateway>";
		$postFields .= "<ewayCustomerID>".$this->config["Customer_ID"]."</ewayCustomerID>";
		$postFields .= "<ewayTotalAmount>".((int)floor($this->params['paymentAmount'][0]*100))."</ewayTotalAmount>";
		$postFields .= "<ewayCustomerFirstName>".$this->params['contactFirstName'][0]."</ewayCustomerFirstName>";
		$postFields .= "<ewayCustomerLastName>".$this->params['contactLastName'][0]."</ewayCustomerLastName>";
		$postFields .= "<ewayCustomerEmail>".$this->params['contactEmail'][0]."</ewayCustomerEmail>";
		$postFields .= "<ewayCustomerAddress>".$this->params['contactAddress1'][0]." ".$this->params['contactAddress2'][0]."</ewayCustomerAddress>";
		$postFields .= "<ewayCustomerPostcode>".$this->params['contactZip'][0]."</ewayCustomerPostcode>";
		$postFields .= "<ewayCustomerInvoiceDescription>".$this->params['paymentDescription'][0]."</ewayCustomerInvoiceDescription>";
		$postFields .= "<ewayCustomerInvoiceRef>".$this->params['invoiceID'][0]."</ewayCustomerInvoiceRef>";
		$postFields .= "<ewayCardHoldersName>$fullName</ewayCardHoldersName>";
		$postFields .= "<ewayCardNumber>".$this->params['billingCardNum'][0]."</ewayCardNumber>";
		$postFields .= "<ewayCardExpiryMonth>$exp_month</ewayCardExpiryMonth>";
		$postFields .= "<ewayCardExpiryYear>$exp_year</ewayCardExpiryYear>";
		$postFields .= "<ewayTrxnNumber>".$this->params['invoiceID'][0]."</ewayTrxnNumber>";
		$postFields .= "<ewayOption1></ewayOption1>";
		$postFields .= "<ewayOption2></ewayOption2>";
		$postFields .= "<ewayOption3></ewayOption3>";
		$postFields .= "<ewayCVN>".$this->params['cardCVV2'][0]."</ewayCVN>";
		$postFields .= "</ewaygateway>";

		if ($this->config["Testing"]) {
			$url = $this->params['cardCVV2'][0] ? $this->testURLWithCVN : $this->testURL;
			dbg("Post URL (TEST): ".$url);
		} else {
			$url = $this->params['cardCVV2'][0] ? $this->liveURLWithCVN : $this->liveURL;
			dbg("Post URL (LIVE)".$url);
		}

		dbg($postFields);
	 	foreach($this->params as $pm => $val) {
	 		dbg("Transaction Parameter: $pm".$val[0]);
	 	}

        if(!$postFields) {
			$this->addError(TRANS_MBAPIERROR, "No parameters given Gateway.");
		} else {
			$status = NULL;
			// Call the cURL class and execute the call
			$curlcall = $this->getCurlObject($url);

			$info = $this->getModuleInfo();
			$product = "MB5" . $info["displayName"] . "Module/" . $info["version"];
			$agent = $product . " (cURL " . $curlcall->getCurlVersion() . ")";
			$curlcall->addHeader("User-Agent: " . $agent);

			$curlcall->addField("", $postFields, 0);
			$curlResult = trim($curlcall->toString());

			dbg("Raw cURL Object:");
			dbg($curlcall);

			l1dbg("Raw cURL Return: ".$curlResult);

			$x = new MBXMLLite($curlResult);
			$charge = correctArray($x->toArray());

			l1dbg($charge);
			$charge = $charge["ewayResponse"][0];

			$this->message = trim($charge['ewayTrxnStatus']." ".$charge['ewayTrxnError']);
			$this->authorizationCode = $charge['ewayAuthCode'];
			$this->transactionID = $charge['ewayTrxnNumber'];

			switch (trim(rtrim(strtolower($charge['ewayTrxnStatus'])))) {
				case "true": $this->status = GATEWAY_SUCCESS; break;
				case "false": $this->status = GATEWAY_DECLINED; break;
				default: $this->status = GATEWAY_FAILURE; break;
			}

			if ($curlResult != "") {
				$this->hasExecutedSuccessfully = 1;
			} else {
				$this->status = GATEWAY_CONNECTION_FAILURE;
			}
		}
		$this->hasExecuted = 1;

		return $this->createOutputXML(array(
			"status" => $this->status,
			"statusName" => getAPIStatusName("transactions", $this->status),
			"statusMessage" => getStatusName("transactions", $this->status),
			"authorizationCode" => $this->authorizationCode,
			"AVSCode" => $this->AVSCode,
			"transactionID" => $this->transactionID,
			"message" => $this->message,
		));
	 }
}
