<?php

require_once("lib-tk/include/xml/xml.php");
require_once("lib-mbapi/include/modules/module.php");
require_once("lib-tk/include/creditcard/validate.php");
require_once("lib-tk/include/crypt/crypt.php");
require_once("lib-tk/functions.php");
require_once("objects/PaymentGateway.php");

$i = 1;
/**
 * The constant to represent a successful gateway execution.
 */
define("GATEWAY_SUCCESS", $i);
$i++;

/**
 * The constant to represent a declined transaction by the last gateway execution.
 */
define("GATEWAY_DECLINED", $i);
$i++;

/**
 * The constant to represent a failed gateway execution.
 */
define("GATEWAY_FAILURE", $i);
$i++;

/**
 * The constant to represent a failed connection to a physical gateway during execution.
 */
define("GATEWAY_CONNECTION_FAILURE", $i);
$i++;

/**
 * The constant to represent a failed requirement during parameter validation.
 */
define("GATEWAY_PARAM_FAILURE", $i);
$i++;

/**
 * The constant to identify a gateway's charge capability.
 */
define("GATEWAY_CHARGE", $i);
$i++;

/**
 * The constant to identify a gateway's authorize capability.
 */
define("GATEWAY_AUTHORIZE", $i);
$i++;

/**
 * The constant to identify a gateway's ability to apply a charge after an authorization.
 */
define("GATEWAY_PRIOR_AUTH_CHARGE", $i);
$i++;

/**
 * The constant to identify a gateway's refund ability.
 */
define("GATEWAY_REFUND", $i);
$i++;

/**
 * The constant to identify a gateway's refund ability.
 */
define("GATEWAY_PARTIAL_REFUND", $i);
$i++;

/**
 * The constant to identify a gateway's void ability.
 */
define("GATEWAY_VOID", $i);
$i++;

/**
 * The constant to identify a gateway's echeck ability.
 */
define("GATEWAY_ECHECK", $i);
$i++;

/**
 * The constant to represent a failed gateway module execution.
 */
define("GATEWAY_ERROR", $i);
$i++;

define("GATEWAY_RECURRING_CHARGE", $i);
$i++;

define("GATEWAY_IPV6_COMPLIANCE", $i);
$i++;

/**
 * An interface for all Gateway modules to implement. If a Gateway module does not follow this interface
 * exactly, it is not portable among the application, and may fail at some points. This interface is only
 * meant for true gateways, in which payment services like PayPal and WorldPay are not. This interface
 * was written using a PHP4 implementation, but later should be changed for PHP5 implementation in order
 * to create truly abstract methods.
 *
 * <module>
 *  <gateway>
 *   <bluepay>
 *      <params>
 *        <subCommand>charge</subCommand>                   (Required: One of charge, auth, finalize, refund)
 *        <invoiceID></invoiceID>
 *        <authCode></authCode>                             (Required for finalize)
 *        <transactionCode></transactionCode>               (Required for refund)
 *        <paymentDescription></paymentDescription>         (Required)
 *        <paymentAmount></paymentAmount>                   (Required: 0.00)
 *        <paymentMethod></paymentMethod>                   (Required: CC, echeck)
 *        <isRecurring></isRecurring>                       (0,1)
 *        <billingAccountID></billingAccountID>
 *      </params>
 *    </bluepay>
 *  </gateway>
 * <module>
 *
 */
abstract class Gateway extends MB_Module
{
	/**
	 * The status returned by the last execution of this Gateway.
	 */
	public $status;

	/**
	 * The authorization code returned by the last execution of this Gateway.
	 */
	public $authorizationCode;

	/**
	 * The AVS code returned by the last execution of this Gateway.
	 */
	public $AVSCode;

	/**
	 * The transaction ID returned by the last execution of this Gateway.
	 */
	public $transactionID;

	/**
	 * These three variables are for Protx transactions only - they are stored for future transactions
	 */
	public $protxSecurityKey;
	public $protxTransactionCode;
	public $protxCCID;

	/**
	 * The text message returned by the last execution of this Gateway. Usually only set if an error occured
	 */
	public $message;

	/**
	 * The capabilities provided by this Gateway object. The only required capability for a
	 * gateway is the "charge" capability.
	 */
	public $capabilities;

	/**
	 * The version of the instantiated child module.
	 */
	public $version;

	/**
	 * The transaction URL of the instantiated child module that the command is sent to.
	 */
	public $liveURL;

	/**
	 * The transaction URL of the instantiated child module that the command is sent to.
	 */
	public $testURL;

 	/**
	 * Whether or not the instantiation of this object is a valid one, ready for use.
	 */
	public $valid;

	/**
	 * array  config
	 */
	public $config;

	/**
	 * string  request classname
	 */
	public $requestClass;

	/**
	 * The constructor for a Gateway object. For the most part this constructor should not be overridden
	 * by Gateway implementations. This method is designed to take the <params> element of an MBAPI
	 * command's input XML. It uses getDBCol() and getDBXMLCol() to associate array element names with
	 * database column names, so literally when coming from an MBAPI command, no modification to the
	 * <params> element is needed. Be sure to pass in the <params> element as an array, and not raw XML.
	 *
	 * @param  params  The parameters needed by a Gateway object. All of the following parameters are
	 *                 required here, and if they are not provided, all calls to this class' methods will
	 *                 fail. Validity can be checked using the isValid() method. The following parameters
	 *                 are required:
	 *                   invoiceID, paymentDescription, paymentAmount, paymentMethod, paymentType
	 *                 and either
	 *                   billingAccountID
	 *                 or
	 *                   billingCardType, billingCardNum, billingCardExpMonth,
	 *                   billingCardExpYear, billingCardIssueNum, billingCardStartDate
	 *                 or
	 *                   billingBankType, billingBankName, billingBankABACode,
	 *                   billingBankAccountNum, billingBankLicenseNum, billingBankLicenseState,
	 *                   billingBankDOB, billingBankState
	 *                 and either
	 *                   clientContactID
	 *                 or
	 *                   contactFirstName, contactLastName, contactEmail, contactCompany, contactAddress1,
	 *                   contactAddress2, contactCity, contactState, contactZip, countriesID,
	 *                   contactPhone1, contactPhone2, contactFax
	 */
	public function Gateway($input)
	{
		global $CONF;
		parent::__construct($input);
        $this->setConfigID();

		$this->status = GATEWAY_FAILURE;

		// Set default capabilities
		$moduleInfo = $this->getModuleInfo();
		foreach ($moduleInfo["capabilities"] as $cap ) {
			$this->capabilities[$cap] = 1;
		}

		if ( isset($this->params["billingAccountID"]) ) {
			// Get Billing account info and contact info
			$m = NULL;
			$m = new QueryMaster("client_billing_accounts.*", "client_contacts.*", "clients.*", "countries.countries_iso_2");
			$m->addJoin("clients", "client_id", "client_contacts");
			$m->addJoin("client_contacts", "client_contact_id", "client_billing_accounts");
			$m->addJoin("client_contacts", "countries_id", "countries");
			$m->addAndClause("client_billing_accounts.billing_account_id", "=", $this->params["billingAccountID"][0], "i");

			$billingres = $m->getRow();
			dbg("query: ".$m->getQuery());
			foreach ($billingres as $col_key => $col_value) {
				$col_key = getDBXMLCol($col_key);
				$this->params[$col_key][0] = unsecureInput($col_value);
			}
		}

		if ( isset($this->params["paymentAmount"]) ) {
			$this->params["paymentAmount"][0] = unsecureInput($this->params["paymentAmount"][0]);
		}

		if ( isset($this->params["countriesID"]) ) {
			$h = new QueryMaster("countries.countries_iso_2");
			$h->addAndClause("countries_id", "=", $this->params["countriesID"][0], "i");
			$this->params["countriesISO2"][0] = $h->getOne();
		}

		require_once("objects/PaymentGateway.php");
		require_once("object_managers/CryptManager.php");
		if ( isset($this->params["cardNum"]) ) {
			// we are handling a single payment where the card is present
			$this->params["billingCardNum"][0] = $this->params["cardNum"][0];
		} else if ( isset($this->params["bankAccountNum"]) ) {
			$this->params["billingBankAccountNum"][0] = $this->params["bankAccountNum"][0];
		} else if ( PaymentGateway::TYPE_THIRD_PARTY == $moduleInfo["gatewaytype"] ) {
			// we are a secondary gateway and don't need card information
		} else if ( CryptManager::isEncryptionEnabled() ) {
			// Credit card decryption logic
			// set the decryption key to a blank string
			$decKey = "";

			if ( CryptManager::isLekEnabled() ) {
				$decKey = CryptManager::getLek();
			} else {
				// we are doing passphrase, get contact_date_created and make a hash from it and the passphrase
				$decKey = getPasswordHash($this->params["contactDateCreated"][0].$this->params["key"][0]);
			}

			if ( "" != $decKey ) { // if we have a key then...
				// make the blowfish object
				$c = new Crypt($decKey, "orangefish", NULL);

				if ( isset($this->params["billingCardNum"]) ) {
					// decrypt the credit card
					$unEncryptedCC = ereg_replace("[^0-9]+", "", $c->decrypt($this->params["billingCardNum"][0]));

					// if the card number is present but it isn't a valid number then fail.
					if ( !validateCard($unEncryptedCC) ) {
						$this->status = GATEWAY_FAILURE;
					} else { // else we have a valid card, update the param
						$this->params["billingCardNum"][0] = $unEncryptedCC;
					}
				} else if ( isset($this->params["billingBankAccountNum"]) ) {
					// decrypt the bank account number
					$unEncryptedAN = ereg_replace("[^0-9]+", "", $c->decrypt($this->params["billingBankAccountNum"][0]));

					// if the card number is present but it isn't a valid number then fail.
					if ( "" == $unEncryptedAN ) {
						$this->status = GATEWAY_FAILURE;
					} else { // else we have a valid bank account number, update the param
						$this->params["billingBankAccountNum"][0] = $unEncryptedAN;
					}
				}
			} else {
				// we didn't have a decryption key so we fail this one.
				$this->addError(blmsg("TRANS_GATEWAY_ERROR"), "Failed to get a decryption key");
				$this->status = GATEWAY_PARAM_FAILURE;
			}
		}

		if ( isset($this->params["billingCardExpYear"]) ) {
			if ( 1900 > $this->params["billingCardExpYear"][0] ) {
				$this->params["billingCardExpYear"][0] += 2000;
			}
			if ( date("Y") == $this->params["billingCardExpYear"][0] && (int)date("m") > $this->params["billingCardExpMonth"][0] ) {
				$this->cardExpired = true;
			} else if ( (int)date("Y") > $this->params["billingCardExpYear"][0] ) {
				$this->cardExpired = true;
			}
		}

		if ( isset($this->params["currencyID"]) ) {
			$this->params["currency"][0] = $this->params["currencyID"][0];
		}

		if ( "PayPal" == get_class($this) ) {
			$this->params["enableNameBasedMapping"][0] = Config()->paypal["enableNameBasedMapping"];
			$this->params["enableClientAutoCreation"][0] = Config()->paypal["enableClientAutoCreation"];
		}

		dbg($this->params);

		if ( !is_array($input) ) {
			return;
		}

		if ( !isset($this->params["currencyID"]) ) {
			$this->addError(blmsg("TRANS_GATEWAY_ERROR"), "Missing parameter: currencyID");
			$this->status = GATEWAY_PARAM_FAILURE;
		}

		if ( is_null($this->configID) ) {
			$this->addError(blmsg("TRANS_GATEWAY_ERROR"), "Property is not set: configID");
			$this->status = GATEWAY_PARAM_FAILURE;
		}
	}

	/**
	 * Get an array of capabilities that this Gateway object supports. The array is of the form:
	 *   GATEWAY_CAPABILITY_CONSTANT => INTEGER
	 * If the INTEGER does not equal 0 (resolves to true), it can be assumed that this Gateway has
	 * the capability represented by GATEWAY_CAPABILITY_CONSTANT, and if the GATEWAY_CAPABILITY_CONSTANT
	 * array element is not set or is equal to 0, this Gateway object does not have that capability.
	 * The GATEWAY_CAPABILITY_CONSTANT constants are defined at the top of this file.
	 *
	 * @return  An array of capabilities of this Gateway object.
	 */
	public function getCapabilities()
	{
		return $this->capabilities;
	}

	/**
	 * Returns 1 if this instantiation is valid, and 0 if otherwise.
	 *
	 * @return  1 if this instantiation is valid, and 0 if otherwise.
	 */
	public function isValid()
	{
		return $this->valid;
	}

	/**
	 * From a child module this function must do the following:
	 *   - Maintain $this->hasExecuted, $this->hasExecutedSuccessfully, $this->timestamp,
	 *     $this->status, $this->accountID
	 *   - Add any queries to the debug list with addDebug()
	 *   - Add any errors to the error list with addError()
	 *   - Return $this->returnData, or the appropriate error constant
	 *
	 * @return  Returns feedback from execution of this Gateway object, or an error constant defined
	 *          at the top of this file.
	 */
	public function execute()
	{
		parent::execute();
		if ( !is_null($this->requestClass) ) {
			$requestClass = $this->requestClass;
			$request = new $requestClass($this);
			return $request->execute();
		}
	}

	/**
	 * This method can be overrided.
	 */
	public function getCurlObject($url = '', $fields = '', $cli = 0, $config_identity = NULL)
	{
		if (is_null($config_identity)) {
			$config_identity = 'Gateway_'.get_class($this);
		}
		require_once("lib-mbapi/include/curl/curl.php");
		$c = new Curl($url, $fields, $cli, $config_identity);
		$c->setOption("CURLOPT_SSLVERSION", 3);
		return $c;
	}

	protected function setConfigID()
	{
		if ( isset($this->params["paymentGatewayID"]) ) {
			$payment_gateway = Factory()->PaymentGateway($this->params["paymentGatewayID"][0]);
			$this->configID = $payment_gateway->id;
			$this->config = PaymentGateway::getMapOf($payment_gateway->PaymentGatewayParams, "name", "value");
			dbg($this->config);
		}
	}

	protected function getDecryptedText($text)
	{
		$c = new Crypt(getPasswordHash(Crypt::ENCRYPTION_KEY), "orangefish", NULL);
		$passwd = $c->decrypt($text);
		unset( $c );
		//dbg("DECRYPTED: ".$text);
		//dbg("To: ".$passwd);
		return (strlen($passwd)>0)?$passwd:$text;
	}

	protected function makeAccountingCurrenyConversion($beginAmount, $beginCurrency, $endAmount, $endCurrency, $clientID, $invoiceID)
	{
		// lets see if this payment was converted to a different currency, and convert accordingly
		if ($endAmount>0 && $endCurrency!=""){
			if ($endCurrency!=$beginCurrency){
				// make the entry for our original currency
				$entries = array();
				$entries[] = array(
									"clientID" => $clientID,
									"invoiceID" => $invoiceID,
									"generalLedgerType" => 1,
									"ledgerAccountID" => getGeneralLedgerAccountID("currency_expense"),
									"generalLedgerDescription" => TRANS_CURRENCY_CONVERSION." - ".$beginCurrency." => ".$endCurrency,
									"generalLedgerDebitAmount" => doubleval($beginAmount),
									"generalLedgerCreditAmount" => 0,
									"currencyID" => $beginCurrency
									);
				$entries[] = array(
									"clientID" => $clientID,
									"invoiceID" => $invoiceID,
									"generalLedgerType" => 1,
									"ledgerAccountID" => getGeneralLedgerAccountID("cash"),
									"generalLedgerDescription" => TRANS_CURRENCY_CONVERSION." - ".$beginCurrency." => ".$endCurrency,
									"generalLedgerDebitAmount" => 0,
									"generalLedgerCreditAmount" => doubleval($beginAmount),
									"currencyID" => $beginCurrency
									);
			    $gen = new General();
				$gen->setClientID($clientID);
				$gen->setInvoiceID($invoiceID);
				$gen->setDescription(TRANS_CURRENCY_CONVERSION." - ".$beginCurrency." => ".$endCurrency);
				$gen->setTotalDebits(doubleval($beginAmount));
				$gen->setTotalCredits(doubleval($beginAmount));
				$gen->setCurrencyID($beginCurrency);
				$gen->setLedgerEntries($entries);
				$gen->execute();
				// make the entry for our new currency
				$entries = array();
				$entries[] = array(
									"clientID" => $clientID,
									"invoiceID" => $invoiceID,
									"generalLedgerType" => 1,
									"ledgerAccountID" => getGeneralLedgerAccountID("cash"),
									"generalLedgerDescription" => TRANS_CURRENCY_CONVERSION." - ".$endCurrency." <= ".$beginCurrency,
									"generalLedgerDebitAmount" => doubleval($endAmount),
									"generalLedgerCreditAmount" => 0,
									"currencyID" => $endCurrency
									);
				$entries[] = array(
									"clientID" => $clientID,
									"invoiceID" => $invoiceID,
									"generalLedgerType" => 1,
									"ledgerAccountID" => getGeneralLedgerAccountID("currency_income"),
									"generalLedgerDescription" => TRANS_CURRENCY_CONVERSION." - ".$endCurrency." <= ".$beginCurrency,
									"generalLedgerDebitAmount" => 0,
									"generalLedgerCreditAmount" => doubleval($endAmount),
									"currencyID" => $endCurrency
									);
			    $gen = new General();
				$gen->setClientID($clientID);
				$gen->setInvoiceID($invoiceID);
				$gen->setDescription(TRANS_CURRENCY_CONVERSION." - ".$endCurrency." <= ".$beginCurrency);
				$gen->setTotalDebits(doubleval($endAmount));
				$gen->setTotalCredits(doubleval($endAmount));
				$gen->setCurrencyID($endCurrency);
				$gen->setLedgerEntries($entries);
				$gen->execute();
				dbg("made currency conversion entries");
			}
		}
	}


	/**
	 * function to quickly insert a callback record into the callback log
	 */
	protected function insertCallbackLog($data, $attributes=array())
	{
		$id = insertFor("callbackLogs", (array)$data);
		if (count($attributes)){
			$this->insertCallbackLogAttributes($id, (array)$attributes);
		}
		return $id;
	}


	/**
	 * function to quickly insert the callback log attributes (k = v pairs)
	 */
	protected function insertCallbackLogAttributes($callbackLogID, $attributes)
	{
		foreach ((array)$attributes as $name=>$value){
			insertFor("callbackLogAttributes", array(
				"callbackLogID" => $callbackLogID,
				"callbackLogAttributeName" => $name,
				"callbackLogAttributeValue" => $value,
			));
		}
	}

	/**
	 * function to insert a transaction and get the last insert id
	 */
	protected function insertTransaction($data=array())
	{
		if (count($data)){
			$h = new QueryMaster("insert");
			$h->addTables("transaction_log");
			$h->addParam("auth_return", $data["auth_return"], "i");
			$h->addParam("transaction_payment_method", $data["transaction_payment_method"], "i");
			$h->addParam("transaction_code", $data["transaction_code"], "s");
			$h->addParam("transaction_notes", $data["transaction_notes"], "s");
			$h->addParam("transaction_date_created", $data["transaction_date_created"], "i");
			$h->addParam("transaction_amount", $data["transaction_amount"], "d");
			$h->addParam("currency_id", $data["currency_id"], "s");
			if ( $data["payment_gateway_id"] ) {
				$h->addParam("payment_gateway_id", $data["payment_gateway_id"], "i");
			}
			$h->execute();
			$transactionID = $h->getLastInsertID();
			dbg("inserted transaction: ".$transactionID);
			return $transactionID;
		} else {
			return 0;
		}
	}

	/**
	 * function to quickly add an authorizenet
	 */
	protected function runAuthNETProtocol($overrides=array())
	{
		global $CONF;
		dbg($overrides);

		if ($this->cardExpired){
			return $this->createOutputXML(array(
				"status" => GATEWAY_DECLINED,
				"authorizationCode" => '',
				"AVSCode" => '',
				"transactionID" => '',
				"message" => TRANS_CARD_IS_EXPIRED . " (".$this->params["billingCardExpMonth"][0]."/".$this->params["billingCardExpYear"][0].")",
			));
		}

		$postFields = array();

		if ($this->status === GATEWAY_PARAM_FAILURE) {
			return $this->createOutputXML(array(
				"status" => $this->status,
			));
		}

		switch ($this->params['subCommand'][0]){
			case "charge":
				$postFields["x_Type"] = "AUTH_CAPTURE";
			break;
			case "auth":
				$postFields["x_Type"] = "AUTH_ONLY";
			break;
			case "finalize":
				if ($this->params['transactionCode'][0]!=''){
					$postFields["x_Trans_ID"] = $this->params['transactionCode'][0];
				}
				$postFields["x_Type"] = "PRIOR_AUTH_CAPTURE";
			break;
			case "refund":
				if ($this->params['transactionCode'][0]!='') {
					$postFields["x_Trans_ID"] = $this->params['transactionCode'][0];
				}
				if ($this->simulateRefundsForTestURL && (int)$this->config["Gateway_URL"]){
					$forceRefundSuccess = 1;
				}
				$postFields["x_Type"] = "CREDIT";
			break;
			case "void":
				if ($this->params['transactionCode'][0]!='') {
					$postFields["x_Trans_ID"] = $this->params['transactionCode'][0];
				}
				$postFields["x_Type"] = "VOID";
			break;
			default:
				//No Sub Commnad give
				$this->addError(TRANS_MBAPIERROR, "No sub-command was given.");
		}

		$postFields["x_Invoice_Num"] = $this->params['invoiceID'][0];
		$postFields["x_Description"] = $this->params['paymentDescription'][0];
		$postFields["x_Amount"] = sprintf("%01.2f", $this->params['paymentAmount'][0]);
		$postFields["x_Method"] = strtoupper($this->params['paymentMethod'][0]);

		if ($postFields["x_Method"]=="ECHECK"){
			if ($this->params['billingBankName'][0]) {
				$postFields["x_Bank_Name"] = $this->params['billingBankName'][0];
			}
			if ($this->params['billingBankType'][0]) {
				switch ($this->params['billingBankType'][0]){
					case getAPITypeID("bankAccount", "savings"):
					case getAPITypeID("bankAccount", "business-savings"):
						$postFields["x_Bank_Acct_Type"] = "SAVINGS";
					break;
					case getAPITypeID("bankAccount", "checking"):
						$postFields["x_Bank_Acct_Type"] = "CHECKING";
					break;
					case getAPITypeID("bankAccount", "business-checking"):
						$postFields["x_Bank_Acct_Type"] = "BUSINESSCHECKING";
					break;
				}
			}
			$postFields["x_eCheck_Type"] = "WEB";
			if ($this->params['billingBankAbaCode'][0]) {
				$postFields["x_Bank_ABA_Code"] = $this->params['billingBankAbaCode'][0];
			}
			if ($this->params['billingBankAccountNum'][0]) {
				$postFields["x_Bank_Acct_Num"] = $this->params['billingBankAccountNum'][0];
			}
			if ($this->params['billingBankLicenseNum'][0]) {
				$postFields["x_Drivers_License_Num"] = $this->params['billingBankLicenseNum'][0];
			}
			if ($this->params['billingBankLicenseState'][0]) {
				$postFields["x_Drivers_License_State"] = $this->params['billingBankLicenseState'][0];
			}
			if ($this->params['billingBankDOB'][0]) {
				$postFields["x_Drivers_License_DOB"] = $this->params['billingBankDOB'][0];
			}
		} else {
			if ($this->params['billingCardNum'][0]) {
				$postFields["x_Card_Num"] = $this->params['billingCardNum'][0];
			}
			if ($this->params['billingCardExpYear'][0]) {
				$month = (int)$this->params['billingCardExpMonth'][0];
				if ($month<10)$month = "0".$month;
				if ($month>12)$month = "12";
				if ($month<1)$month = "01";
				$year = (int)$this->params['billingCardExpYear'][0];
				if ($year>2000)$year -= 2000;
				if ($year<10)$year = "0".$year;
				$postFields["x_Exp_Date"] = $month.$year;
			}
			if ($this->params['billingCardNameOnCard'][0]) {
				$postFields["x_Card_Name"] = $this->params['billingCardNameOnCard'][0];
			} else {
				$postFields["x_Card_Name"] = billing_get_full_name($this->params['contactFirstName'][0], $this->params['contactLastName'][0]);
			}
			if ($this->params['cardCVV2'][0]) {
				$postFields["x_Card_Code"] = $this->params['cardCVV2'][0];
			}
		}

		$postFields["x_Cust_ID"] = $this->params['clientContactID'][0];
		$postFields["x_First_Name"] = $this->params['contactFirstName'][0];
		$postFields["x_Last_Name"] = $this->params['contactLastName'][0];
		$postFields["x_Company"] = $this->params['clientCompany'][0];
		$postFields["x_Address"] = strip_tags(nl2br($this->params['contactAddress1'][0]));
		if($this->params['contactAddress2'][0]) {
			$postFields["x_Address_2"] = $this->params['contactAddress2'][0];
		}
		$postFields["x_City"] = $this->params['contactCity'][0];
		$postFields["x_State"] = $this->params['contactState'][0];
		$postFields["x_Zip"] = $this->params['contactZip'][0];
		$postFields["x_Country"] = $this->params['countriesISO2'][0];
		$postFields["x_Phone"] = $this->params['contactPhone1'][0];
		$postFields["x_Fax"] = $this->params['contactFax'][0];
		if($this->params['contactEmail'][0]) {
			$postFields["x_Email"] = $this->params['contactEmail'][0];
		} else {
			$postFields["x_Email"] = "";
		}

		$postFields["x_customer_ip"] = $this->getRemoteIP();

		$version = $this->config["Version"];
		$recurringbilling = ($this->params['isRecurring'][0] &&
			$this->config["Batch_Recurring"])
				? "YES" : "NO";
		$testing = (int)$this->config["Testing"];
		$password = $this->config["Password"];
		$login = $this->config["Login_ID"];
		$custemail = $this->params['contactEmail'][0];
		$postFields["x_Login"] = $login;
		$postFields["x_Password"] = $password;
		$postFields["x_Version"] = $version;
		$postFields["x_ADC_URL"] = "FALSE";
		$postFields["x_recurring_billing"] = $recurringbilling;
		$postFields["x_ADC_Delim_Data"] = "TRUE";
		$postFields["x_Test_Request"] = (($testing)?"TRUE":"FALSE");
		$postFields["x_Email_Customer"] = $custemail;

		if((int)$this->config["Gateway_URL"]) {
			$url = $this->testURL;
			$CONF["debug"]->add("Post URL (TEST)", $url);
		} else {
			$url = $this->liveURL;
			$CONF["debug"]->add("Post URL (LIVE)", $url);
		}

		$CONF["debug"]->add("Post Fields Data", $postFields);
		foreach($this->params as $pm => $val) {
			$CONF["debug"]->add("Transaction Parameter: $pm", $val[0]);
		}

		if(!count($postFields)) {
			$this->addError(TRANS_MBAPIERROR, "No parameters given to Gateway.");
		} else {
			$status = NULL;
			// Call the cURL class and execute the call
			$curlcall = $this->getCurlObject($url);

			foreach ($overrides as $k=>$v)$postFields[$k] = $v;

			$curlcall->addMultiFields($postFields);
			dbg($postFields);

			$info = $this->getModuleInfo();
			$product = "MB5" . $info["displayName"] . "Module/" . $info["version"];
			$agent = $product . " (cURL " . $curlcall->getCurlVersion() . ")";
			$curlcall->addHeader("User-Agent: " . $agent);

			$curlResult = $curlcall->toString();

			$CONF["debug"]->add("Raw cURL Object:", $curlcall);
			$CONF["debug"]->add("Raw cURL Return:", $curlResult);

			if (is_numeric($curlResult[0])){
				$delim = $curlResult[1];
			} elseif (is_numeric($curlResult[1])){
				$delim = $curlResult[0].$curlResult[3].$curlResult[0];
				$curlResult = substr($curlResult, 1, -1);
			} else {
				$delim = ",";
			}

			// Process the return value
			$charge = explode($delim, $curlResult);

			$stat = $charge[0];
			$this->message = $charge[3];
			$this->authorizationCode = $charge[4];
			$this->AVSCode = $charge[5];
			$this->transactionID = $charge[6];

			if (strpos($this->message, "DUPLICATE")!==false){
				$stat = 3;
			}

			switch ($stat) {
				case 1:
					$this->status = GATEWAY_SUCCESS;
					break;
				case 2:
					$this->status = GATEWAY_DECLINED;
					break;
				case 3:
					$this->status = GATEWAY_FAILURE;
					break;
			}

			if ($forceRefundSuccess){
				$this->status = GATEWAY_SUCCESS;
			}

			if ($curlResult > 0) {
				$this->hasExecutedSuccessfully = 1;
			} else {
				$this->status = GATEWAY_CONNECTION_FAILURE;
			}
		}

		$this->hasExecuted = 1;

		return $this->createOutputXML(array(
			"status" => $this->status,
			"statusName" => getAPIStatusName("transactions", $this->status),
			"statusMessage" => getStatusName("transactions", $this->status),
			"authorizationCode" => $this->authorizationCode,
			"AVSCode" => $this->AVSCode,
			"transactionID" => $this->transactionID,
			"message" => $this->message,
		));
	}

	protected function createOutputXML($data=array(), $dontSetXMLData=false)
	{
		dbg($data);
		if ($dontSetXMLData){
			return toXML(array(
				"gatewayReturn" => $data,
			));
		} else {
			return $this->xmlData = toXML(array(
				"gatewayReturn" => $data,
			));
		}
	}

	protected function sendSecondaryPaymentEmail($clientID = 0, $transactionID = 0, $invoiceID = 0)
	{
		require_once(dirname(__FILE__)."/../../../../lib-mbapi/include/mail/emailTemplates/paymentSecondaryEmail.php");
		$action = new paymentSecondaryEmail($clientID, $transactionID, $invoiceID);
		return $action->execute();
	}

	protected function getRemoteIP()
	{
		require_once("object_managers/IPAddressManager.php");
		$remoteIP = IPAddressManager::canonicalize($_SERVER["REMOTE_ADDR"]);
		if (false === $remoteIP) {
			return "127.0.0.1";
		}
		if (IPAddressManager::IPv6 == IPAddressManager::getVersion($remoteIP) && !in_array(GATEWAY_IPV6_COMPLIANCE, $this->getCapabilities())) {
			return "127.0.0.1";
		}
		return $remoteIP;
	}
}

class TransactionRecord
{
	public $that;
	public $type;
	public $configID;
	public $billingAccountID;
	public $transactionID;
	public $transactionRecordID;
	public $fields;

	function getTransactionCode() {
		if(!$this->transactionID) $this->getRefForReauth();
		return $this->transactionID;
	}

	function TransactionRecord(&$that, $type = "regular", $transactionID = NULL) {
		$this->type = getAPITypeID("transactionRecord", $type);
		if($transactionID === NULL) $transactionID = $that->reauthRef;
		if($transactionID === NULL) $transactionID = $that->params["transactionCode"][0];
		$this->that =& $that;
		$this->configID = $that->configID;
		$this->billingAccountID = $that->params["billingAccountID"][0];
		$this->transactionID = $transactionID;
		$this->transactionRecordID = NULL;
		$this->fields = NULL;
	}

	function getIDByRef($ref = NULL) {
		if($billingAccountID === NULL) $billingAccountID = $this->billingAccountID;
		if($this->transactionRecordID) return $this->transactionRecordID;
		if($ref === NULL) $ref = $this->transactionID;

		$d = new DataSource("gwTransactions");
		$h =& $d->getSelectQuery(
			array(
				"billingAccountID" => $billingAccountID,
				"gwTransactionRef" => $ref,
				"comparisonOperator" => "=",
				"column" => "gwTransactionID",
			)
		);
		return $this->transactionRecordID = $h->getOne();
	}

	function getRefForReauth($billingAccountID = NULL) {
		if($billingAccountID === NULL) $billingAccountID = $this->billingAccountID;

		$d = new DataSource("gwTransactions");
		$h =& $d->getSelectQuery(
			array(
				"billingAccountID" => $billingAccountID,
				"type" => getAPITypeID("transactionRecord", "reauth"),
				"column" => "gwTransactionRef",
			)
		);
		$h->addOrderByColumns(getDBCol("gwTransactionID"), "desc");

		return $this->transactionID = $h->getOne();
	}

	function getFields($transactionRecordID = NULL) {
		if($transactionRecordID === NULL) $transactionRecordID = $this->getIDByRef();

		$d = new DataSource("gwTransactionAttrs");
		$h =& $d->getSelectQuery(
			array(
				"gwTransactionID" => $transactionRecordID,
			)
		);
		$t =& $h->getTable(NULL, NULL, "gw_transaction_attr_name", "gw_transaction_attr_value");
		return $this->fields = $t->getOptions();
	}

	function getField($field) {
		if($this->fields === NULL) $this->getFields();
		return $this->fields[$field];
	}

	function initRecord($ref = NULL) {
		if($ref === NULL) $ref = $this->transactionID;
		$d = new DataSource("gwTransactions");
		$h =& $d->getInsertQuery(
			array(
				"gwTransactionType" => $this->type,
				"gwTransactionRef" => $ref,
				"billingAccountID" => $this->billingAccountID,
				"paymentGatewayID" => $this->configID,
			)
		);
		if(!$h->execute()) {
			$this->that->addError(TRANS_GATEWAY_ERROR, "Failed to initialize transaction record: " . $h->getLastError());
			return false;
		} else {
			return $this->transactionRecordID = $h->getLastInsertID();
		}
	}

	function addField($field, $value, $skipEmpty = false) {
		if($skipEmpty && (string) $value === "") return true;
		$d = new DataSource("gwTransactionAttrs");
		$h =& $d->getInsertQuery(
			array(
				"gwTransactionID" => $this->transactionRecordID,
				"gwTransactionAttrName" => $field,
				"gwTransactionAttrValue" => $value,
			)
		);
		if(!$h->execute()) {
			$this->that->addError(TRANS_GATEWAY_ERROR, "Failed to add transaction record field $field as $value: " . $h->getLastError());
			return false;
		} else {
			return $h->getLastInsertID();
		}
	}

	function updateField($field, $value) {
		$h = new QueryMaster("update");
		$h->addTables("gw_transactions");
		$h->addParam("gwTransactionAttrValue", $value);
		$h->addAndClause("gwTransactionID", "=", $this->transactionRecordID);
		$h->addAndClause("gwTransactionAttrName", "=", $field, "s");
		if(!$h->execute()) {
			$this->that->addError(TRANS_GATEWAY_ERROR, "Failed to update transaction record field $field to $value: " . $h->getLastError());
			return false;
		} else {
			return true;
		}
	}
}

class TransactionRequest
{
	/**
	 * GateWay  reference to a subclass of the GateWay class
	 */
	public $that;

	/**
	 * array  of module configuration
	 */
	public $config;

	/**
	 * string  subcommand mode ("charge", etc.)
	 */
	public $mode;

	/**
	 * string  name of the response class to use
	 */
	public $responseClass;

	/**
	 * string  how we identify ourself
	 */
	public $agent;

	/**
	 * TransactionRecord  object for handling the transaction record
	 */
	public $t;

	public function TransactionRequest(Gateway &$that)
	{
		$this->that =& $that;
		$this->config = $that->config;
		$this->configID = $that->configID;
		$this->mode = $that->params["subCommand"][0];
		$this->responseClass = "TransactionResponse";
		$this->transactionID = $that->params["transactionCode"][0];

		if(isset($this->that->params["product"][0])) {
			$productParams = $this->that->params["product"];
			$this->products = array();
			foreach($productParams as $productParam) {
				if(isset($productParam["description"][0]) && isset($productParam["quantity"][0]) && isset($productParam["total"][0])) {
					$this->products[] = array(
						"description" => $productParam["description"][0],
						"quantity" => $productParam["quantity"][0],
						"total" => $productParam["total"][0],
					);
				}
			}

			$lines = array();
			foreach($this->products as $product) {
				$lines[] = $product["description"] . " x " . $product["quantity"] . " = " . $product["total"];
			}
			if($lines) {
				$this->description = implode("\n", $lines);
			}
		}
	}

	function initCall() {
		$this->call = $this->that->getCurlObject();

		$info = $this->that->getModuleInfo();
		$product = "MB5" . $info["displayName"] . "Module/" . $info["version"];
		$this->agent = $product . " (cURL " . $this->call->getCurlVersion() . ")";
		$this->call->addHeader("User-Agent: " . $this->agent);
	}


	function buildCall() {
		// stub
	}

	function makeCall() {
		$err = $this->call->execute();
		if($this->call->getNumErrors()) {
			$this->that->addError(GATEWAY_CONNECTION_FAILURE, "cURL error: " . $err);
			return false;
		}
		return $this->call->toString(0);
	}

	function call() {
		$responseClass = $this->responseClass;
		$this->initCall();
		$this->buildCall();
		if(!$response = $this->makeCall()) return false;
		$this->response = new $responseClass($this, $response);
		return $this->response->execute();
	}

	function execute() {
		return $this->call();
	}
}

class TransactionResponse
{
	/**
	 * TransactionRequest  subclass for referring back to the request
	 */
	public $that;

	/**
	 * array  to be returned in the gateway command response
	 */
	public $gatewayResponse;

	/**
	 * int  billing account ID
	 */
	public $billingAccountID;

	/**
	 * TransactionRecord  object for handling the transaction record
	 */
	public $t;

	function TransactionResponse(&$that, $response) {
		$this->that =& $that;
		$this->response = $response;
		$this->transactionID = $that->that->params["transactionCode"][0];
		return true;
	}

	function getGatewayStatus() {
		// stub
	}

	function getTransactionID() {
		// stub
	}

	function getMessage() {
		// stub
	}

	function getAuthorizationCode() {
		// stub
	}

	function getAVSCode() {
		// stub
	}

	function getRejected() {
		// stub
		return $this->getGatewayStatus() == GATEWAY_DECLINED;
	}

	function initXML() {
		$this->gatewayResponse = array();
	}

	function buildXML() {
		$this->gatewayResponse["status"] = quoteXML($this->getGatewayStatus());

		$this->gatewayResponse["statusName"] = getAPIStatusName("transactions", $this->getGatewayStatus());
		$this->gatewayResponse["statusMessage"] = getStatusName("transactions", $this->getGatewayStatus());
		$this->gatewayResponse["transactionID"] = quoteXML($this->getTransactionID());
		$this->gatewayResponse["message"] = quoteXML($this->getMessage());

		$this->gatewayResponse["authorizationCode"] = quoteXML($this->getAuthorizationCode());
		$this->gatewayResponse["AVSCode"] = quoteXML($this->getAVSCode());
	}

	function execute() {
		if($this->getRejected()) {
			$billingAccountID = $this->that->that->params["billingAccountID"][0];
			$h = new QueryMaster("select");
			$h->addColumns("clients.client_id");
			$h->addJoin("clients", "client_id", "client_contacts");
			$h->addJoin("client_contacts", "client_contact_id", "client_billing_accounts");
			$h->addAndClause("billing_account_id", "=", $billingAccountID);
			$clientID = $h->getOne();

			$tr = new TransactionRecord($this->that->that, "reject", $this->getTransactionID());
			$tr->initRecord();
			$tr->addField("subCommand", $this->that->that->params["subCommand"][0]);
			$tr->addField("paymentAmount", $this->that->that->params["paymentAmount"][0]);
			$tr->addField("paymentDescription", $this->that->that->params["paymentDescription"][0]);
			$tr->addField("currency", $this->that->that->params["currency"][0]);
			$tr->addField("Message", $this->getMessage());

			$q = new MBQuery(
				"SetTodos", "insert",
				array(
					"todoPriority" => 0,
					"todoTitle" => "TRANS_BILLING_ACCOUNT_NEEDS_REAUTH",
					"todoDesc" => "TRANS_BILLING_ACCOUNT_NEEDS_REAUTH_DESC",
					"todoLinkFile" => "profile",
					"todoLinkName" => "TRANS_EDIT_BILLING_ACCOUNT",
					"todoLinkPk" => $billingAccountID,
					"todoStatusID" => getAPIStatusID("todo", "open"),
					"todoLinkActionName" => "EditBillingAccount",
					"userType" => getAPITypeID("user", "client"),
					"userID" => $clientID,
				)
			);
			$r =& $q->getResult();
		}

		// update the gateway with the results
		$this->initXML();
		$this->buildXML();
		$this->that->that->arrayData = array("gatewayReturn" => array($this->gatewayResponse));
		$this->that->that->xmlData = NULL;
		//$this->that->that->xmlData = toXML($this->arrayData);
		if($this->gatewayResponse["status"] === GATEWAY_FAILURE) {
			$this->that->that->hasExecutedSuccessfully = false;
		} else {
			if($this->finish()) {
				$this->that->that->hasExecutedSuccessfully = true;
			} else {
				$this->that->that->hasExecutedSuccessfully = false;
			}
		}
		$this->that->that->hasExecuted = 1;
		return $this->that->that->xmlData;
	}

	function finish() {
		$subCommand = $this->that->that->params["subCommand"][0];
		switch($subCommand) {
			case "reauth":
				$type = "reauth";
				break;
			default:
				$type = "regular";
		}
		$this->t = new TransactionRecord($this->that->that, $type, $this->getTransactionID());
		$this->t->initRecord();
		$this->t->addField("subCommand", $subCommand);
		$this->t->addField("paymentAmount", $this->that->that->params["paymentAmount"][0]);
		$this->t->addField("paymentDescription", $this->that->that->params["paymentDescription"][0]);
		$this->t->addField("currency", $this->that->that->params["currency"][0]);
		$this->t->addField("requestType", $this->that->that->requestClass);
		$this->t->addField("responseType", $this->that->responseClass);
		$this->t->addField("Message", $this->getMessage());
		$this->t->addField("Status", $this->getGatewayStatus());
	}

	function updateAccount($params) {
		if(!$params["billingAccountID"] = $this->that->that->params["billingAccountID"][0]) {
			return true;
		}
		require_once mb_path("/lib-tk/toolkit.php");
		$d = new DataSource("billingAccounts");
		$h =& $d->getUpdateQuery($params);
		if(!$h->execute()) {
			$this->that->addError(TRANS_GATEWAY_ERROR, "Failed to update billing account: " . $this->getLastError());
			return false;
		} else {
			return true;
		}
	}
}
