<?php

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";
require_once mb_path("/lib-mbapi/include/modules/gateway/gateway.php");

$className = "InternetSecure";
$GLOBALS["moduleInfo"][$className] = array (
    "type" => "gateway",
    "status" => PaymentGateway::STATUS_STABLE,
    "version" => "1.0.0",
    "gatewaytype" => "1",
    "displayName" => "InternetSecure",
    "author" => "Parallels",
    "capabilities" => array(GATEWAY_CHARGE, GATEWAY_IPV6_COMPLIANCE),
    "countryCodes" => array("US", "CA"),
);

class InternetSecure extends Gateway
{
	public function getModuleInfo()
	{
		return $GLOBALS["moduleInfo"]["InternetSecure"];
	}

	 public function __construct($params)
	 {
	 	parent::Gateway($params);
		$this->liveURL = "https://secure.internetsecure.com/process.cgi";
		$this->testURL = "https://secure.internetsecure.com/process.cgi";
	 }

	public function addDefaultConfigParams()
	{
		static $defaultConfigParams = array(
			array(NULL, NULL, "Merchant_ID", "", "", "TRANS_INTERNETSECURE_MERCHANTID", "TRANS_INTERNETSECURE_MERCHANTID_DESC", "t", 1, 1, 0, NULL),
			array(NULL, NULL, "Testing", "A", "", "TRANS_GATEWAY_TESTING", "TRANS_GATEWAY_TESTING_DESC", "s", 1, 2, 0, array
					(
						array(NULL, NULL, "A", "TRANS_TESTING_ALWAYS_APPROVE", 0),
						array(NULL, NULL, "D", "TRANS_TESTING_ALWAYS_DECLINE", 1),
						array(NULL, NULL, "L", "TRANS_TESTING_LIVE_MODE", 2),
					)),
			array(NULL, NULL, "Invoice_Description", "", "", "TRANS_INVOICE_DESCRIPTION", "TRANS_INVOICE_DESCRIPTION_DESC", "a", 0, 3, 0, NULL),
            array(NULL, NULL, "card_accept_0", "1", "", "TRANS_ACCEPT_VISA", "TRANS_ACCEPT_VISA_DESC", "b", 1, 4, 0, NULL),
            array(NULL, NULL, "card_accept_1", "1", "", "TRANS_ACCEPT_MASTERCARD", "TRANS_ACCEPT_MASTERCARD_DESC", "b", 1, 5, 0, NULL),
            array(NULL, NULL, "card_accept_2", "1", "", "TRANS_ACCEPT_AMEX", "TRANS_ACCEPT_AMEX_DESC", "b", 1, 6, 0, NULL),
            array(NULL, NULL, "card_accept_3", "1", "", "TRANS_ACCEPT_DISCOVER", "TRANS_ACCEPT_DISCOVER_DESC", "b", 1, 7, 0, NULL),
		);
		return $defaultConfigParams;
	}

	public function execute()
	{
	 	$postFields = null;

		if ( GATEWAY_PARAM_FAILURE === $this->status ) {
			return $this->status;
		}


		$this->gatewayInfo = $this->config;


		$exp_year  = date("y",mktime(0,0,0,$this->params['billingCardExpMonth'][0],1,$this->params['billingCardExpYear'][0]));
		$exp_month = date("m",mktime(0,0,0,$this->params['billingCardExpMonth'][0],1,$exp_year));


		$this->postFields = array();
		$this->postFields["xxxRequestMode"] = "X";


		$transactionXML = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n";

		$transactionInfo = array("TranxRequest" => array());
		$transactionInfo["TranxRequest"]["MerchantNumber"] = $this->gatewayInfo["Merchant_ID"];

		$this->testModeFlag = "";
		if ($this->gatewayInfo["Testing"]=="A"){
			$this->testModeFlag = "{TEST}";
		} elseif ($this->gatewayInfo["Testing"]=="D"){
			$this->testModeFlag = "{TESTD}";
		}

		$transactionInfo["TranxRequest"]["products"] = $this->params['paymentAmount'][0]."::1::I".$this->params["invoiceID"][0]."::".htmlentities($this->gatewayInfo["Invoice_Description"])."::".$this->testModeFlag."|";


		if ($this->params['billingCardNameOnCard'][0]) {
			$fullName= $this->params['billingCardNameOnCard'][0];
		} else {
			$fullName= billing_get_full_name($this->params['contactFirstName'][0], $this->params['contactLastName'][0]);
		}


		$transactionInfo["TranxRequest"]["xxxName"] = htmlentities($fullName);
		$transactionInfo["TranxRequest"]["xxxAddress"] = htmlentities($this->params['contactAddress1'][0]."\n".$this->params['contactAddress2'][0]);
		$transactionInfo["TranxRequest"]["xxxCity"] = htmlentities($this->params['contactCity'][0]);
		$transactionInfo["TranxRequest"]["xxxProvince"] = htmlentities($this->params['contactState'][0]);
		$transactionInfo["TranxRequest"]["xxxPostal"] = htmlentities($this->params['contactZip'][0]);
		$transactionInfo["TranxRequest"]["xxxCountry"] = $this->params["countriesISO2"][0];
		$transactionInfo["TranxRequest"]["xxxPhone"] = htmlentities($this->params['contactPhone1'][0]);

		$transactionInfo["TranxRequest"]["xxxEmail"] = htmlentities($this->params['contactEmail'][0]);
		$transactionInfo["TranxRequest"]["xxxCard_Number"] = $this->params['billingCardNum'][0];
		$transactionInfo["TranxRequest"]["xxxCCMonth"] = $exp_month;
		$transactionInfo["TranxRequest"]["xxxCCYear"] = $exp_year;
		$transactionInfo["TranxRequest"]["CVV2Indicator"] = ($this->params["cardCVV2"][0]=="")?0:1;
		$transactionInfo["TranxRequest"]["CVV2"] = $this->params["cardCVV2"][0];


		$transactionXML .= toXML($transactionInfo);


		dbg("XML sent to Internet Secure Gateway: ");
		dbg($transactionXML);

		$this->postFields["xxxRequestData"] = $transactionXML;


		$status = NULL;

		$this->call = $this->getCurlObject($this->liveURL);
		// PPB for Windows doesn't work correctly with SSL 3.0
		$this->call->setOption("CURLOPT_SSLVERSION", 32);

		$info = $this->getModuleInfo();
		$product = "MB5" . $info["displayName"] . "Module/" . $info["version"];
		$agent = $product . " (cURL " . $this->call->getCurlVersion() . ")";

		$this->call->addHeader("User-Agent: " . $agent);

		foreach ($this->postFields as $field=>$value){
			$this->call->addField($field, $value);
		}


		$curlResult = $this->call->toString();

		if ($curlResult != "") {
			$this->hasExecutedSuccessfully = 1;
			dbg("Raw curl result:");
			dbg($curlResult);

			if (strpos($curlResult, "<TranxResponse>") && !strpos($curlResult, "</TranxResponse>")){
				$curlResult .= "</TranxResponse>";
			}

			$xmldoc = new MBXMLLite($curlResult);
			$this->curlResults = $xmldoc->toArray();

			if (isset($this->curlResults["TranxResponse"])){
				$this->curlResults = correctArray($this->curlResults);
				$this->curlResults = $this->curlResults["TranxResponse"][0];
				dbg($this->curlResults);
				if ($this->curlResults["ApprovalCode"]==""){
					$this->status = GATEWAY_DECLINED;
					$this->AVSCode = $this->curlResults["CVV2ResponseCode"];
					$this->transactionID = $this->curlResults["ReceiptNumber"];
					$this->authorizationCode = "";
					$this->message = ($this->curlResults["Verbiage"]!="")?$this->curlResults["TranxResponse"]["Error"]." - ".$this->curlResults["Verbiage"]:$this->curlResults["Error"];
				} else {
					$this->status = GATEWAY_SUCCESS;
					$this->AVSCode = $this->curlResults["CVV2ResponseCode"];
					$this->transactionID = $this->curlResults["ReceiptNumber"];
					$this->authorizationCode = $this->curlResults["ApprovalCode"];
					$this->message = $this->curlResults["Verbiage"];
				}
			} else {
				$this->status = GATEWAY_FAILURE;
			}

		} else {
			$this->status = GATEWAY_CONNECTION_FAILURE;
		}


		$this->hasExecuted = 1;

		$this->returnInfo = array();

		return $this->createOutputXML(array(
			"status" => $this->status,
			"authorizationCode" => $this->authorizationCode,
			"AVSCode" => $this->AVSCode,
			"transactionID" => $this->transactionID,
			"message" => $this->message,
		));
	}
}
