<?php

require_once("lib-mbapi/include/modules/gateway/gateway.php");

$className = "LinkPoint";
$GLOBALS["moduleInfo"][$className] = array(
	"type" => "gateway",
	"status" => PaymentGateway::STATUS_STABLE,
	"version" => "1.0.0",
	"gatewaytype" => "1",
	"displayName" => "LinkPoint",
	"author" => "Parallels",
	"capabilities" => array(GATEWAY_CHARGE, GATEWAY_AUTHORIZE, GATEWAY_PRIOR_AUTH_CHARGE, GATEWAY_REFUND, GATEWAY_VOID, GATEWAY_IPV6_COMPLIANCE),
	"countryCodes" => array("US"),
);

class LinkPoint extends Gateway
{
	public function getModuleInfo()
	{
        return $GLOBALS["moduleInfo"]["LinkPoint"];
    }

	public function __construct($params)
	{
	 	parent::Gateway($params);
		$this->liveURL = "https://secure.linkpt.net:1129/LSGSXML";
		$this->testURL = "https://staging.linkpt.net:1129/LSGSXML";
	}

	public function addDefaultConfigParams()
	{
		$defaultConfigParams = array(
			array(NULL, NULL, "Store_Number", "", "", "TRANS_LINKPOINT_STORENUM", "TRANS_LINKPOINT_STORENUM_DESC", "t", 1, 1, 0, NULL),
			array(NULL, NULL, "Key_File", "", "", "TRANS_LINKPOINT_KEYFILE","TRANS_LINKPOINT_KEYFILE_DESC", "t", 1, 2, 0, NULL),
			array(NULL, NULL, "Testing", "GOOD", "", "TRANS_TEST_MODE", "", "s", 1, 3, 0, array
					(
						array(NULL, NULL, "GOOD", "TRANS_TESTING_ALWAYS_APPROVE", 0),
						array(NULL, NULL, "DECLINE", "TRANS_TESTING_ALWAYS_DECLINE", 1),
						array(NULL, NULL, "DUPLICATE", "TRANS_TESTING_ALWAYS_DUPLICATE", 1),
						array(NULL, NULL, "RANDOM", "TRANS_TESTING_RANDOM", 1),
						array(NULL, NULL, "LIVE", "TRANS_TESTING_LIVE_MODE", 1),
					)),
			array(NULL, NULL, "Recurring", "1", "", "TRANS_RECURRING_MODE", "", "b", 1, 3, 0, NULL),
			array(NULL, NULL, "Invoice_Description", "", "", "TRANS_INVOICE_DESCRIPTION", "TRANS_INVOICE_DESCRIPTION_DESC", "a", 0, 6, 0, NULL),
			array(NULL, NULL, "card_accept_0", "1", "", "TRANS_ACCEPT_VISA", "TRANS_ACCEPT_VISA_DESC", "b", 1, 8, 0, NULL),
			array(NULL, NULL, "card_accept_1", "1", "", "TRANS_ACCEPT_MASTERCARD", "TRANS_ACCEPT_MASTERCARD_DESC", "b", 1, 9, 0, NULL),
			array(NULL, NULL, "card_accept_2", "1", "", "TRANS_ACCEPT_AMEX", "TRANS_ACCEPT_AMEX_DESC", "b", 1, 10, 0, NULL),
			array(NULL, NULL, "card_accept_3", "1", "", "TRANS_ACCEPT_DISCOVER", "TRANS_ACCEPT_DISCOVER_DESC", "b", 1, 11, 0, NULL),
			array(NULL, NULL, "card_accept_4", "1", "", "TRANS_ACCEPT_JCB", "TRANS_ACCEPT_JCB_DESC", "b", 1, 12, 0, NULL),
			array(NULL, NULL, "card_accept_6", "1", "", "TRANS_ACCEPT_DINERS", "TRANS_ACCEPT_DINERS_DESC", "b", 1, 14, 0, NULL),
			);
		return $defaultConfigParams;
	}

	public function execute()
	{
		$postFields = null;

		if ($this->status === GATEWAY_PARAM_FAILURE) {
			return $this->status;
		}

		// ok, we need to format and gather all our info here:

		$exp_year  = date("y",mktime(0,0,0,$this->params['billingCardExpMonth'][0],1,$this->params['billingCardExpYear'][0]));
		$exp_month = date("m",mktime(0,0,0,$this->params['billingCardExpMonth'][0],1,$exp_year));

		if ($this->params['billingCardNameOnCard'][0]) {
			$fullName= $this->params['billingCardNameOnCard'][0];
		} else {
			$fullName= billing_get_full_name($this->params['contactFirstName'][0], $this->params['contactLastName'][0]);
		}

		switch ($this->params['subCommand'][0]){
			case "charge":
				$oid = $this->params["invoiceID"][0]."X".ereg_replace("[^[:alnum:]]", "", uniqid(0));
				$transactionType = "SALE";
			break;
			case "auth":
				$oid = $this->params["invoiceID"][0]."X".ereg_replace("[^[:alnum:]]", "", uniqid(0));
				$transactionType = "PREAUTH";
			break;
			case "finalize":
				if ($this->params['transactionCode'][0]!=''){
					$oid = $this->params['transactionCode'][0];
				}
				$transactionType = "POSTAUTH";
			break;
			case "refund":
				if ($this->params['transactionCode'][0]!=''){
					$oid = $this->params['transactionCode'][0];
				}
				$transactionType = "CREDIT";
			break;
			case "void":
				if ($this->params['transactionCode'][0]!=''){
					$oid = $this->params['transactionCode'][0];
				}
				$transactionType = "VOID";
			break;
			default:
				//No Sub Commnad give
				$this->addError(TRANS_MBAPIERROR, "No sub-command was given.");
		}
		$modes = array("GOOD", "DECLINE", "DUPLICATE");
		$transactionMode = ($this->config["Testing"]=="RANDOM")?array_rand($modes):$this->config["Testing"];
		switch ($this->params["billingBankType"][0]){
			case getAPITypeID("bankAccount", "checking"):
				$accountType = "pc";
			break;
			case getAPITypeID("bankAccount", "savings"):
				$accountType = "ps";
			break;
			case getAPITypeID("bankAccount", "business-checking"):
				$accountType = "bc";
			break;
			case getAPITypeID("bankAccount", "business-savings"):
				$accountType = "bs";
			break;
		}

		$addrNum = explode(" ", $this->params['contactAddress1'][0]);

		// ok, lets put the info together into an array that will be made into xml

		$transactionData = array(
			"order" => array(
				"orderoptions" => array(
					"ordertype" => $transactionType,
					"result" => $transactionMode,
				),
				"merchantinfo" => array(
					"configfile" => $this->config["Store_Number"],
				),
				"transactiondetails" => array(
					"transactionorigin" => "ECI",
					"oid" => $oid,
					"terminaltype" => "UNSPECIFIED",
					"ip" => $this->getRemoteIP(),
					"recurring" => ((int)$this->config["Recurring"])?"Yes":"No",
				),
				"creditcard" => array(
					"cardnumber" => $this->params['billingCardNum'][0],
					"cardexpmonth" => $exp_month,
					"cardexpyear" => $exp_year,
					"cvmvalue" => $this->params["cardCVV2"][0],
					"cvmindicator" => ($this->params["cardCVV2"][0]!="")?"provided":"not_provided",
				),
				"payment" => array(
					"chargetotal" => doubleval($this->params['paymentAmount'][0]),
				),
				"telecheck" => array(
					"routing" => $this->params['billingBankAbaCode'][0],
					"account" => $this->params['billingBankAccountNum'][0],
					"checknumber" => 850,
					"bankname" => $this->params['billingBankName'][0],
					"bankstate" => $this->params['billingBankState'][0],
					"dl" => $this->params['billingBankLicenseNum'][0],
					"dlstate" => $this->params['billingBankLicenseState'][0],
					"accounttype" => $accountType,
				),
				"billing" => array(
					"name" => $fullName,
					"company" => $this->params['contactCompany'][0],
					"address1" => $this->params['contactAddress1'][0],
					"address2" => $this->params['contactAddress2'][0],
					"city" => $this->params['contactCity'][0],
					"state" => $this->params['contactState'][0],
					"zip" => $this->params['contactZip'][0],
					"country" => $this->params['countriesISO2'][0],
					"phone" => $this->params['contactPhone1'][0],
					"fax" => $this->params['contactFax'][0],
					"addrnum" => $addrNum[0],
				),
				"shipping" => array(
					"name" => $fullName,
					"address1" => $this->params['contactAddress1'][0],
					"address2" => $this->params['contactAddress2'][0],
					"city" => $this->params['contactCity'][0],
					"state" => $this->params['contactState'][0],
					"zip" => $this->params['contactZip'][0],
					"country" => $this->params['countriesISO2'][0],
				),
				"notes" => array(
					"comments" => ($this->params["invoiceID"][0])?TRANS_INVOICE_NUMBER.": ".$this->params["invoiceID"][0]:$this->config["Invoice_Description"],
				),
			),
		);

		// unset any info we dont need due to billing account type.

		switch ((int)$this->params["billingAccountType"][0]){
			case getAPITypeID("billingAccount", "card"):
			default:
				unset($transactionData["order"]["telecheck"]);
			break;
			case getAPITypeID("billingAccount", "echeck"):
				unset($transactionData["order"]["creditcard"]);
			break;
		}
		$postFields = str_replace("	", "", str_replace("\n", "", toXML($transactionData)));
		$postFields = toXML($transactionData);
		dbg($postFields);

		if ( "" == $postFields ) {
			$this->status = GATEWAY_PARAM_FAILURE;
			return $this->createOutputXML(array(
				"status" => $this->status,
			));
		}

		$url = ("LIVE" == $this->config["Testing"]) ? $this->liveURL : $this->testURL;

		$status = NULL;
		// Call the cURL class and execute the call
		$curlcall = $this->getCurlObject($url);

		$info = $this->getModuleInfo();
		$product = "MB5LinkPointModule/" . $info["version"];
		$agent = $product . " (cURL " . $curlcall->getCurlVersion() . ")";
		$curlcall->addHeader("User-Agent: " . $agent);

		$curlcall->addField("", $postFields, 0);
		$curlcall->setSSLCertPath(unsecureInput($this->config["Key_File"]));
		$curlResult = $curlcall->toString();
		dbg($curlResult);

		if ( !$curlcall->hasExecutedSuccessfully() ) {
			$this->status = GATEWAY_CONNECTION_FAILURE;
			return $this->createOutputXML(array(
				"status" => $this->status,
				"message" => implode(" ", $curlcall->errors),
			));
		}

		$x = new MBXMLLite("<document>".$curlResult."</document>"); // have to have a document element!
		$results = correctArray($x->toArray());

		dbg($results);

		$results = $results["document"][0];

		switch ($results["r_approved"]){
			case "APPROVED":
				$this->status = GATEWAY_SUCCESS;
				break;
			case "DECLINED":
				$this->status = GATEWAY_DECLINED;
				break;
			default:
				$this->status = GATEWAY_FAILURE;
				break;
		}

		$message_items = array();
		foreach ( array("r_approved", "r_message", "r_error") as $message_item_name )
			if ( "" != $results[$message_item_name] )
				$message_items[] = $results[$message_item_name];

		$this->message = implode(": ", $message_items);
		$this->authorizationCode = $oid;
		$this->AVSCode = $results["r_avs"];
		$this->transactionID = $oid;

		$this->hasExecuted = 1;

		return $this->createOutputXML(array(
			"status" => $this->status,
			"authorizationCode" => $this->authorizationCode,
			"AVSCode" => $this->AVSCode,
			"transactionID" => $this->transactionID,
			"message" => $this->message,
		));
	}
}
