<?php

require_once("lib-tk/shield.php");
require_once("lib-mbapi/include/modules/gateway/gateway.php");
require_once("lib-mbapi/include/modules/gateway/moneris/core/mpgClasses.php");
require_once("lib-tk/include/creditcard/validate.php");

$className = "Moneris";
$GLOBALS["moduleInfo"][$className] = array
(
    "type" => "gateway",
    "status" => PaymentGateway::STATUS_STABLE,
    "version" => "1.0.1",
    "gatewaytype" => "1",
    "displayName" => "Moneris",
    "author" => "Parallels",
    "capabilities" => array(GATEWAY_CHARGE, GATEWAY_AUTHORIZE, GATEWAY_PRIOR_AUTH_CHARGE, GATEWAY_REFUND, GATEWAY_VOID, GATEWAY_IPV6_COMPLIANCE),
    "countryCodes" => array("US", "CA"),
);

function evaluate($string) {
	switch($string) {
		case "null":
		case "NULL":
			return NULL;
		case "true":
		case "TRUE":
			return true;
		case "false":
		case "FALSE":
			return false;
		default:
			return $string;
	}
}

$GLOBALS["CONF"]["types"]["monerisCVDIndicator"] = array();
$GLOBALS["CONF"]["types"]["monerisCVDIndicator"][0] = TRANS_CVD_BYPASSED;
$GLOBALS["CONF"]["types"]["monerisCVDIndicator"][1] = TRANS_CVD_PRESENT;
$GLOBALS["CONF"]["types"]["monerisCVDIndicator"][2] = TRANS_CVD_ILLEGIBLE;
$GLOBALS["CONF"]["types"]["monerisCVDIndicator"][9] = TRANS_CVD_NONE;

$GLOBALS["CONF"]["APItypes"]["monerisCVDIndicator"] = array();
$GLOBALS["CONF"]["APItypes"]["monerisCVDIndicator"][0] = "bypassed";
$GLOBALS["CONF"]["APItypes"]["monerisCVDIndicator"][1] = "present";
$GLOBALS["CONF"]["APItypes"]["monerisCVDIndicator"][2] = "illegible";
$GLOBALS["CONF"]["APItypes"]["monerisCVDIndicator"][9] = "none";

$GLOBALS["CONF"]["types"]["monerisCVDResponseCode"] = array();
$GLOBALS["CONF"]["types"]["monerisCVDResponseCode"]["M"] = TRANS_MATCH;
$GLOBALS["CONF"]["types"]["monerisCVDResponseCode"]["N"] = TRANS_NO_MATCH;
$GLOBALS["CONF"]["types"]["monerisCVDResponseCode"]["P"] = TRANS_NOT_PROCESSED;
$GLOBALS["CONF"]["types"]["monerisCVDResponseCode"]["S"] = TRANS_SHOULD_BE_PRESENT;
$GLOBALS["CONF"]["types"]["monerisCVDResponseCode"]["U"] = TRANS_NONE;

$GLOBALS["CONF"]["APItypes"]["monerisCVDResponseCode"] = array();
$GLOBALS["CONF"]["APItypes"]["monerisCVDResponseCode"]["M"] = "match";
$GLOBALS["CONF"]["APItypes"]["monerisCVDResponseCode"]["N"] = "noMatch";
$GLOBALS["CONF"]["APItypes"]["monerisCVDResponseCode"]["P"] = "notProcessed";
$GLOBALS["CONF"]["APItypes"]["monerisCVDResponseCode"]["S"] = "shouldBePresent";
$GLOBALS["CONF"]["APItypes"]["monerisCVDResponseCode"]["U"] = "none";

class AVSElement {
	public $name;
	public $match;
	public $incompatible;
	public $unavailable;

	function AVSElement($name) {
		$this->name = $name;
	}
}

class AVSResponse {
	public $response;
	public $address;
	public $postCode;
	public $international;

	function AVSResponse($response) {
		$this->address = new AVSElement("address");
		$this->postCode = new AVSElement("postCode");

		switch($response) {
			case "D":
			case "G":
			case "I":
			case "M":
				$this->international = true;
				break;

			case "R":
			case "U":
			case "W":
				$this->international = false;
				break;
		}

		switch($response) {
			case "P":
				$this->postCode->incompatible = true;
			case "Z":
				$this->postCode->match = true;
				break;

			case "B":
				$this->address->incompatible = true;
			case "A":
				$this->address->match = true;
				break;

			case "D":
			case "M":
			case "Y":
				$this->postCode->match =
				$this->address->match = true;
				break;

			case "C":
				$this->address->incompatible =
				$this->postCode->incompatible = true;
				break;

			case "G":
			case "U":
				$this->address->unavailable = true;
				break;

		}

		switch($response) {
			case "A":
				$this->postCode->match = false;
				break;

			case "Z":
				$this->address->match = false;
				break;

			case "N":
				$this->address->match =
				$this->postCode->match = false;
				break;
		}
	}
}

/****************
 * base classes *
 ****************/

class MonerisBaseTransactionRequest extends TransactionRequest {
	public $call;
	public $response;

	public $storeID;
	public $APIToken;

	public $transaction;
	public $custInfo;
	public $avsInfo;
	public $cvdInfo;
	public $recur;

	function MonerisBaseTransactionRequest(&$that) {
		$this->TransactionRequest($that);
		$this->responseClass = "MonerisBaseTransactionResponse";

		$this->storeID = $this->config["storeID"];
		$this->APIToken = $this->config["APIToken"];

		$this->transaction = array();

		$this->setTransactionType(7); // SSL-enabled Merchant
	}

	function setTransactionType($type) {
		$this->transaction["crypt_type"] = $type;
	}

	function getTransactionType() {
		return $this->transaction["crypt_type"];
	}

	function initCall() {
		$t = new mpgTransaction($this->transaction);
		if($this->custInfo) {
			foreach($this->custInfo as $custInfo) {
				$t->setCustInfo($custInfo);
			}
		}
		if($this->avsInfo && !$this->config["disable_avs"]) {
			$t->setAVS(new mpgAVS($this->avsInfo));
		}
		if($this->cvdInfo && !$this->config["disable_cvd"]) {
			$t->setCVD(new mpgCVD($this->cvdInfo));
		}
		if($this->recur) {
			$t->setRecur($this->recur);
		}
		$this->call = new mpgHttpsPost(
			$this->storeID,
			$this->APIToken,
			new mpgRequest($t)
		);
	}

	function makeCall() {
		$mpgResponse =& $this->call->getMpgResponse();
		$complete = evaluate($mpgResponse->getComplete());
		$timedOut = evaluate($mpgResponse->getTimedOut());
		$responseCode = evaluate($mpgResponse->getResponseCode());
		$message = evaluate($mpgResponse->getMessage());
		if(!$complete || $timedOut || $responseCode === NULL) {
			$gateway_response["status"] = GATEWAY_FAILURE;
			$gateway_response["message"] = $message;
			$this->that->arrayData = array("gatewayReturn" => array($gateway_response));
			return false;
		}
		dbg($this);
		return $mpgResponse;
	}
}

class MonerisBaseTransactionResponse extends TransactionResponse {
	public $response;

	public $complete;
	public $timedOut;
	public $responseCode;
	public $message;

	public $status;

	public $date;
	public $time;
	public $timestamp;

	public $transType;
	public $cardType;
	public $transAmount;

	public $txnNumber;
	public $receiptId;
	public $referenceNum;
	public $authCode;

	public $avsResultCode;
	public $cvdResultCode;

	public $recurSuccess;

	function MonerisBaseTransactionResponse(&$that, $response) {
		if(!$this->TransactionResponse($that, $response)) return false;

		$this->complete = evaluate($response->getComplete());
		$this->timedOut = evaluate($response->getTimedOut());
		$this->status = evaluate($response->getResponseCode());
		$this->ISOstatus = evaluate($response->getISO());
		$this->message = evaluate($response->getMessage());

		if($this->status === NULL) {
			$this->gatewayStatus = GATEWAY_FAILURE;
		} else if($this->status < 50) {
			$this->gatewayStatus = GATEWAY_SUCCESS;
		} else if($this->status >= 50) {
			$this->gatewayStatus = GATEWAY_DECLINED;
		}

		$this->recurSuccess = evaluate($response->getRecurSuccess());

		$this->date = $response->getTransDate();
		$this->time = $response->getTransTime();
		list($year, $month, $day) = explode("-", $this->date);
		list($hour, $minute, $second) = explode(":", $this->time);
		$this->timestamp = mktime($hour, $minute, $second, $month, $day, $year);

		$this->transType = evaluate($response->getTransType());
		$this->cardType = evaluate($response->getCardType());
		$this->transAmount = evaluate($response->getTransAmount());

		$this->txnNumber = evaluate($response->getTxnNumber());
		$this->receiptId = evaluate($response->getReceiptId());
		$this->referenceNum = evaluate($response->getReferenceNum());
		$this->authCode = evaluate($response->getAuthCode());

		$this->avsResultCode = evaluate($response->getAvsResultCode());
		$this->cvdResultCode = evaluate($response->getCvdResultCode());

		$this->CVDIndicator = getAPITypeName("monerisCVDIndicator", $this->cvdResultCode{0});
		$this->CVDResponseCode = getAPITypeName("monerisCVDResponseCode", $this->cvdResultCode{1});

		$this->AVSResponse = new AVSResponse($this->avsResultCode);
		return true;
	}

	function getStatus() {
		return $this->status;
	}

	function getGatewayStatus() {
		return $this->gatewayStatus;
	}

	function getMessage() {
		return $this->message . " [" . TRANS_SEQUENCE_NUMBER . ": " . $this->referenceNum . "]";
	}

	function getTransactionID() {
		return $this->txnNumber;
	}

	function getAVSCode() {
		return $this->avsResultCode;
	}

	function getAuthorizationCode() {
		return $this->authCode;
	}

	function finish() {
		parent::finish();
		$this->t->addField("orderID", $this->that->id, true);
		$this->t->addField("complete", $this->complete, true);
		$this->t->addField("timedOut", $this->timedOut, true);
		$this->t->addField("ISOstatus", $this->ISOstatus, true);
		$this->t->addField("timestamp", $this->timestamp, true);
		$this->t->addField("receiptId", $this->receiptId, true);
		$this->t->addField("referenceNum", $this->referenceNum, true);
		$this->t->addField("authCode", $this->authCode, true);
		$this->t->addField("CVDIndicator", $this->CVDIndicator, true);
		$this->t->addField("CVDResponseCode", $this->CVDResponseCode, true);
	}
}

class MonerisCardTransactionRequest extends MonerisBaseTransactionRequest {
	function MonerisCardTransactionRequest(&$that) {
		$this->MonerisBaseTransactionRequest($that);

		$this->transaction["order_id"] = $this->id = sha1(
			  (string) mt_rand(0, 200000000)
			. (string) microtime()
			. (string) mt_rand(0, 200000000)
		);

		$this->transaction["amount"] = number_format($that->params["paymentAmount"][0], 2, ".", "");
		$this->transaction["pan"] = $that->params["billingCardNum"][0];
		$this->transaction["expdate"] = substr($that->params['billingCardExpYear'][0], -2, 2) . sprintf("%02s", $that->params['billingCardExpMonth'][0]);

		list($streetNum, $streetName) = explode(" ", $that->params['contactAddress1'][0] . " " . $that->params['contactAddress2'][0], 2);

		$this->avsInfo = array(
			"avs_street_number" => $streetNum,
			"avs_street_name" => $streetName,
			"avs_zipcode" => $that->params['contactZip'][0],
		);
		if($that->params['cardCVV2'][0]) {
			$this->cvdInfo = array(
				"cvd_indicator" => 1,
				"cvd_value" => $that->params['cardCVV2'][0],
			);
		} else {
			$this->cvdInfo = array(
				"cvd_indicator" => 0,
				"cvd_value" => NULL,
			);
		}

		$custInfo = new mpgCustInfo();

		$h = new QueryMaster("countries.countries_iso_num");
		$h->addAndClause("countries_id", "=", $that->params["countriesID"][0], "i");
		if(!$h->execute()) return false;
		$country = $h->getOne();

		$custInfo->setBilling(
			array(
				 "first_name" => $that->params["contactFirstName"][0],
                 "last_name" => $that->params["contactLastName"][0],
                 "company_name" => $that->params["contactCompany"][0],
                 "address" => $that->params['contactAddress1'][0] . " " . $that->params['contactAddress2'][0],
                 "city" => $that->params['contactCity'][0],
                 "province" => $that->params['contactState'][0],
                 "postal_code" => $that->params['contactZip'][0],
                 "country" => $country,
                 "phone_number" => $that->params['contactPhone1'][0],
                 "fax" => $that->params['contactFax'][0],
                 //"tax1" => NULL,
                 //"tax2" => NULL,
                 //"tax3" => NULL,
                 //"shipping_cost" => NULL
			)
		);

		$custInfo->setEmail($that->params['contactEmail'][0]);

		$this->custInfo = array();
		$this->custInfo[] =& $custInfo;
	}
}

class MonerisReferenceTransactionRequest extends MonerisBaseTransactionRequest {
	function MonerisReferenceTransactionRequest(&$that) {
		$this->MonerisBaseTransactionRequest($that);

		$t = new TransactionRecord($that);
		$this->transaction["order_id"] = $this->id = $t->getField("orderID");
		$this->transaction["txn_number"] = $that->params["transactionCode"][0];
	}
}

/******************************
 * complete transaction types *
 ******************************/

class MonerisPurchaseTransactionRequest extends MonerisCardTransactionRequest {
	function MonerisPurchaseTransactionRequest(&$that) {
		$this->MonerisCardTransactionRequest($that);

		$this->transaction["type"] = "purchase";
		$this->transaction["amount"] = number_format($that->params["paymentAmount"][0], 2, ".", "");
	}
}

class MonerisPreauthTransactionRequest extends MonerisCardTransactionRequest {
	function MonerisPreauthTransactionRequest(&$that) {
		$this->MonerisCardTransactionRequest($that);

		$this->transaction["type"] = "preauth";
		$this->transaction["amount"] = number_format($that->params["paymentAmount"][0], 2, ".", "");
	}
}

class MonerisCaptureTransactionRequest extends MonerisReferenceTransactionRequest {
	function MonerisCaptureTransactionRequest(&$that) {
		$this->MonerisReferenceTransactionRequest($that);

		$this->transaction["type"] = "completion";
		$this->transaction["comp_amount"] = number_format($that->params["paymentAmount"][0], 2, ".", "");
	}
}

class MonerisVoidTransactionRequest extends MonerisReferenceTransactionRequest {
	function MonerisVoidTransactionRequest(&$that) {
		$this->MonerisReferenceTransactionRequest($that);

		$this->transaction["type"] = "purchasecorrection";
	}
}

class MonerisRefundTransactionRequest extends MonerisReferenceTransactionRequest {
	function MonerisRefundTransactionRequest(&$that) {
		$this->MonerisReferenceTransactionRequest($that);

		$this->transaction["type"] = "refund";
		$this->transaction["amount"] = number_format($that->params["paymentAmount"][0], 2, ".", "");
	}
}


/**************
 * Main class *
 **************/

class Moneris extends Gateway
{
	public function getModuleInfo()
	{
        return $GLOBALS["moduleInfo"]["Moneris"];
    }

	public function addDefaultConfigParams()
	{
		static $defaultConfigParams = array(
			array(NULL, NULL, "storeID", "store1", "", "TRANS_MONERIS_STORE_ID", "TRANS_MONERIS_STORE_ID_DESC", "t", 1, 1, 0, NULL),
			array(NULL, NULL, "APIToken", "yesguy", "", "TRANS_MONERIS_API_TOKEN","TRANS_MONERIS_API_TOKEN_DESC", "t", 1, 1, 0, NULL),
            array(NULL, NULL, "card_accept_0", "1", "", "TRANS_ACCEPT_VISA", "TRANS_ACCEPT_VISA_DESC", "b", 1, 8, 0, NULL),
            array(NULL, NULL, "card_accept_1", "1", "", "TRANS_ACCEPT_MASTERCARD", "TRANS_ACCEPT_MASTERCARD_DESC", "b", 1, 9, 0, NULL),
            array(NULL, NULL, "card_accept_2", "1", "", "TRANS_ACCEPT_AMEX", "TRANS_ACCEPT_AMEX_DESC", "b", 1, 10, 0, NULL),
            array(NULL, NULL, "card_accept_3", "1", "", "TRANS_ACCEPT_DISCOVER", "TRANS_ACCEPT_DISCOVER_DESC", "b", 1, 11, 0, NULL),
            array(NULL, NULL, "card_accept_4", "1", "", "TRANS_ACCEPT_JCB", "TRANS_ACCEPT_JCB_DESC", "b", 1, 12, 0, NULL),
            array(NULL, NULL, "card_accept_6", "1", "", "TRANS_ACCEPT_DINERS", "TRANS_ACCEPT_DINERS_DESC", "b", 1, 14, 0, NULL),
            array(NULL, NULL, "disable_avs", "0", "", "TRANS_DISABLE_AVS", "TRANS_DISABLE_AVS_DESC", "b", 1, 14, 0, NULL),
            array(NULL, NULL, "disable_cvd", "0", "", "TRANS_DISABLE_CVS", "TRANS_DISABLE_CVD_DESC", "b", 1, 14, 0, NULL),
		);
		return $defaultConfigParams;
	}

	public function execute()
	{
		switch($this->params["subCommand"][0]) {
			case "charge":
				$this->requestClass = "MonerisPurchaseTransactionRequest";
				break;
			case "auth":
			case "reauth":
				$this->requestClass = "MonerisPreauthTransactionRequest";
				break;
			case "finalize":
				$this->requestClass = "MonerisCaptureTransactionRequest";
				break;
			case "refund":
				$this->requestClass = "MonerisRefundTransactionRequest";
				break;
			case "void":
				$this->requestClass = "MonerisVoidTransactionRequest";
				break;
			default:
				$this->addError(TRANS_GATEWAY_ERROR, "Invalid subCommand.");
				return false;
		}
		return parent::execute();
	 }
}
