<?php

require_once('lib-tk/shield.php');
require_once('lib-mbapi/include/modules/gateway/gateway.php');

$className = 'PayFlowPro';

$GLOBALS['moduleInfo'][$className] = array
(
    'type' => 'gateway',
    'status' => PaymentGateway::STATUS_STABLE,
    'version' => '2.0.0',
    'gatewaytype' => '1',
    'displayName' => $className,
    'author' => 'Parallels',
    'capabilities' => array(
    	GATEWAY_CHARGE,
		GATEWAY_AUTHORIZE,
		GATEWAY_PRIOR_AUTH_CHARGE,
		GATEWAY_REFUND,
		GATEWAY_VOID,
//		GATEWAY_ECHECK,
		GATEWAY_IPV6_COMPLIANCE,
	),
    "countryCodes" => array("US", "CA", "GB"),
);

class PayFlowPro extends Gateway
{
	public function getModuleInfo()
	{
		return $GLOBALS['moduleInfo']['PayFlowPro'];
	}

	public function __construct($input)
	{
		parent::Gateway($input);
		$this->liveURL = 'https://payflowpro.paypal.com';
		$this->testURL = 'https://pilot-payflowpro.paypal.com';
	}

	public function addDefaultConfigParams()
	{
		/*
		 * [0] = option name
		 * [1] = option type (t = text, p = password, r = radio)
		 * [2] = default value
		 * [3] = option label
		 * [4] = option description
		 */
		$adminOptions = array(
			array('partner',            't',        NULL,           'PAYFLOWPRO_PARTNER',       'PAYFLOWPRO_PARTNER_DESC'       ),
			array('vendor',             't',        NULL,           'PAYFLOWPRO_VENDOR',        'PAYFLOWPRO_VENDOR_DESC'        ),
			array('username',           't',        NULL,           'PAYFLOWPRO_USERNAME',      'PAYFLOWPRO_USERNAME_DESC'      ),
			array('password',           'p',        NULL,           'PAYFLOWPRO_PASSWORD',      'PAYFLOWPRO_PASSWORD_DESC'      ),
			array('testMode',           'b',        '0',            'PAYFLOWPRO_TEST_MODE',     'PAYFLOWPRO_TEST_MODE_DESC'     ),
			array('card_accept_0',      'b',        '0',            'ACCEPT_VISA',              'ACCEPT_VISA_DESC'              ),
			array('card_accept_1',		'b',		'0',			'ACCEPT_MASTERCARD',		'ACCEPT_MASTERCARD_DESC'	),
			array('card_accept_2',		'b',		'0',			'ACCEPT_AMEX',					'ACCEPT_AMEX_DESC'					),
			array('card_accept_3',		'b',		'0'	,			'ACCEPT_DISCOVER',			'ACCEPT_DISCOVER_DESC'			),
			array('card_accept_4',		'b',		'0',			'ACCEPT_JCB',						'ACCEPT_JCB_DESC'						),
			array('card_accept_5',		'b',		'0'	,			'ACCEPT_ENROUTE',			'ACCEPT_ENROUTE_DESC'			),
			array('card_accept_6',		'b',		'0',			'ACCEPT_DINERS',				'ACCEPT_DINERS_DESC'				),
			array('card_accept_7',		'b',		'0',			'ACCEPT_SOLO',					'ACCEPT_SOLO_DESC'					),
			array('card_accept_8',		'b',		'0',			'ACCEPT_MAESTRO',			'ACCEPT_MAESTRO_DESC'			),
			array('card_accept_9',		'b',		'0',			'ACCEPT_DELTA',					'ACCEPT_DELTA_DESC'				),
			array('card_accept_10',	'b',		'0',			'ACCEPT_BANKCARD',			'ACCEPT_BANKCARD_DESC'		),
			array('card_accept_11',	'b',		'0',			'ACCEPT_ELECTRON',			'ACCEPT_ELECTRON_DESC'			)
		);

		$i = 1;
		foreach($adminOptions as $adminOption) {
			$defaultConfigParams[] = array(
				NULL,NULL,$adminOption[0],$adminOption[2],'','TRANS_'.$adminOption[3],'TRANS_'.$adminOption[4],$adminOption[1],0,$i++,0,NULL
			);
		}

		// Grab the configs set at the registrar level
		return $defaultConfigParams;
	}

	public function execute()
	{
		// populate $this->params with collapsed array
		// and set the proper URL to hit based on test mode
		$this->params = correctArray($this->params);
		$this->submitURL = ($this->config['testMode']) ? $this->testURL : $this->liveURL;

		switch($this->params['subCommand']) {
			case 'charge':
				$TRXTYPE = 'S';
			break;
			case 'auth':
				$TRXTYPE = 'A';
			break;
			case 'finalize':
				$ORIGID = $this->params['transactionCode'];
				$TRXTYPE = 'D';
			break;
			case 'refund':
				$ORIGID = $this->params['transactionCode'];
				$TRXTYPE = 'C';
			break;
			case 'void':
				$ORIGID = $this->params['transactionCode'];
				$TRXTYPE = 'V';
			break;

			// just in case something funky is going on
			default:
				return $this->createOutputXML(array(
					"status" =>GATEWAY_ERROR,
					"message" => 'Invalid Gateway Action Requested',
					)
				);
		}

		$tempPostValues = array(
			// basic authentication that needs to be a part of any hit on PayFlowPro's API
			'PARTNER'		=> 		$this->config['partner'],
			'PWD'			=>		$this->config['password'],
			'USER'			=> 		$this->config['username'],
			'VENDOR'		=>		$this->config['vendor'],

			// required postfields for all transactions
			'ACCT'			=> 		$this->params['billingCardNum'],
			'AMT'			=>		$this->params['paymentAmount'],
			'EXPDATE'		=>		str_pad($this->params['billingCardExpMonth'],2,'0',STR_PAD_LEFT).substr($this->params['billingCardExpYear'],2,2),
			'TENDER'		=> 		'C',
			'TRXTYPE'		=> 		$TRXTYPE,

			// optional postvalues that do can magic things like lower merchant rates
			// please note: magic things might also include appearance of unicorns and rainbows
			'NAME'			=>		billing_get_full_name($this->params['contactFirstName'], $this->params['contactLastName']),
			'STREET'		=>		(isset($this->params['contactAddress1']) && trim($this->params['contactAddress1']) != '') ? $this->params['contactAddress1'] : '',
			'ZIP'				=>		(isset($this->params['contactZip']) && trim($this->params['contactZip']) != '' ) ? $this->params['contactZip'] : '',
			'CVV2'			=>		(isset($this->params['cardCVV2']) && trim($this->params['cardCVV2']) != '') ? $this->params['cardCVV2'] : '',
			'ORIGID'		=>		(isset($ORIGID) && trim($ORIGID) != '') ? $ORIGID : ''
		);

		// generate final list of postvalues to go through to PayFlowPro's API
		// PayFlowPro requires that a string length be attached to all "names"
		// in the name/value pair, as we do not URL encode what is sent to them
		$postValues = array();
		foreach($tempPostValues as $postElement => $postValue) {
			if (isset($postValue) && trim($postValue) != '')
				$postValues[$postElement.'['.strlen($postValue).']'] = $postValue;
		}

		// unset unneeded variables
		unset($tempPostValues);

		// set a random request ID, i'm not sure why they require this
		$requestID = time();

		// make the call...
		$curlCall = $this->getCurlObject($this->submitURL);
		$curlCall->addHeader('Content-Type: text/namevalue');
		$curlCall->addHeader('X-VPS-CLIENT-TIMEOUT: 45');
		$curlCall->addHeader('X-VPS-REQUEST-ID: '. $requestID);
		$curlCall->addMultiFields($postValues, 0);

		// store the API response into a string
		$curlResult = $curlCall->toString();
		$curlResultArray = array();

		if (!$curlResult) {

			$this->status = GATEWAY_CONNECTION_FAILURE;

		} else {

			// parse the response string into an array
			parse_str($curlResult, $curlResultArray);

			// set an informational message for return from the response
			$this->message = $curlResultArray['RESULT'].': '.$curlResultArray['RESPMSG'];

			// set more return data based on API response
			switch($curlResultArray['RESULT']) {
				case '0':
						$this->transactionID = $curlResultArray['PNREF'];
						$this->authorizationCode = $curlResultArray['AUTHCODE'];
						$this->status = GATEWAY_SUCCESS;
				break;

				default:
					$this->status = GATEWAY_DECLINED;
				break;
			}

		}

		// return necessary information back to ModernBill
		$this->hasExecuted = 1;
		return $this->createOutputXML(array(
			"status" => $this->status,
			"statusName" => getAPIStatusName("transactions", $this->status),
			"statusMessage" => getStatusName("transactions", $this->status),
			"authorizationCode" => $this->authorizationCode,
			"transactionID" => $this->transactionID,
			"message" => $this->message,
		));
	}
}