<?php

require_once("lib-mbapi/include/modules/gateway/gateway.php");

$className = "PayPal";
$GLOBALS["moduleInfo"][$className] = array(
	"type" => "gateway",
	"status" => PaymentGateway::STATUS_STABLE,
	"version" => "1.0.3",
	"gatewaytype" => "2",
	"displayName" => "PayPal",
	"author" => "Parallels",
	"capabilities" => array(GATEWAY_CHARGE, GATEWAY_RECURRING_CHARGE, GATEWAY_IPV6_COMPLIANCE),
    "countryCodes" => array("INT"),
);

class PayPal extends Gateway
{
	public function getModuleInfo()
	{
		return $GLOBALS["moduleInfo"]["PayPal"];
	}

	public function __construct($input = "")
	{
		parent::Gateway($input);

		$this->setURLs($this->config["Testing"]);

		$this->maxTerms = array(
			"d"=>90, // days, max 90
			"w"=>52, // weeks, max 52
			"m"=>24, // months, max 24
			"y"=>5   // years, max 5
		);
	}

	public function addDefaultConfigParams()
	{
		global $CONF;
		$defaultConfigParams = array(
			array(NULL, NULL, "PayPal_ID", $CONF["user"]["email"], "", "TRANS_PAYPAL_ID", "TRANS_PAYPAL_ID_DESC", "t", 1, 1, 0, NULL),
			array(NULL, NULL, "Item_Name", "", "", "TRANS_PAYPAL_ITEMNAME", "TRANS_PAYPAL_ITEMNAME_DESC", "t", 1, 2, 0, NULL),
			// array(NULL, NULL, "Return_URL", $CONF["secureURLProtocol"]."://".$CONF["secureURL"], "", "TRANS_PAYPAL_RETURNURL","TRANS_PAYPAL_RETURNURL_DESC", "t", 1, 3, 0, NULL),
			// array(NULL, NULL, "Cancel_URL", $CONF["secureURLProtocol"]."://".$CONF["secureURL"], "", "TRANS_PAYPAL_CANCELURL","TRANS_PAYPAL_CANCELURL_DESC", "t", 1, 4, 0, NULL),
			array(NULL, NULL, "Display_Logo", $CONF["secureURLProtocol"]."://".$CONF["secureURL"]."/lib-themes/default/lib-billing/images/logos/logo.gif", "", "TRANS_PAYPAL_DISPLAYLOGO","TRANS_PAYPAL_DISPLAYLOGO_DESC", "t", 1, 5, 0, NULL),
			array(NULL, NULL, "Submit_Image", "https://www.paypal.com/en_US/i/btn/x-click-but03.gif", "", "TRANS_PAYPAL_SUBMITIMAGE","TRANS_PAYPAL_SUBMITIMAGE_DESC", "t", 1, 6, 0, NULL),
			array(NULL, NULL, "IPN_URL", $CONF["secureURLProtocol"]."://".$CONF["secureURL"]."/gw/paypal.php", "", "TRANS_PAYPAL_IPNURL","TRANS_PAYPAL_IPNURL_DESC", "t", 1, 7, 0, NULL),
			array(NULL, NULL, "Enable_Subscriptions", "1", "", "TRANS_PAYPAL_SUBENABLE","TRANS_PAYPAL_SUBENABLE_DESC", "b", 0, 8, 0, NULL),
			// array(NULL, NULL, "Subscriptions_Override", "0", "", "TRANS_PAYPAL_SUBOVERRIDE","TRANS_PAYPAL_SUBOVERRIDE_DESC", "b", 0, 9, 0, NULL),
			array(NULL, NULL, "Subscriptions_Image", "https://www.paypal.com/images/x-click-but20.gif", "", "TRANS_PAYPAL_SUBIMAGE","TRANS_PAYPAL_SUBIMAGE_DESC", "t", 1, 10, 0, NULL),
			array(NULL, NULL, "Testing", "1", "", "TRANS_GATEWAY_TESTING", "TRANS_GATEWAY_TESTING_DESC", "b", 1, 4, 0, NULL),
			array(NULL, NULL, "Acceptable_PayPal_Payees", $CONF["user"]["email"], "", "TRANS_PAYPAL_ACCEPT_ADDRESSES", "TRANS_PAYPAL_ACCEPT_ADDRESSES_DESC", "a", 1, 11, 0, NULL),
		);
		return $defaultConfigParams;
	}

	public function execute()
	{
		if ( GATEWAY_PARAM_FAILURE === $this->status ) {
			return $this->status;
		}

		if ( isset($this->params["singlePayment"]) && 1 == $this->params["singlePayment"][0] ) {
			$this->config["Enable_Subscriptions"] = 0;
		}

		// lets get all invoice info
		$h = new QueryMaster("invoices.*");
		$h->addAndClause("invoice_id", "=", $this->params["invoiceID"][0], "i");
		$invoiceInfo = $h->getRow();

		// get the line items
		$h = new QueryMaster("invoice_line_items.*", "packages.*");
		$h->enableStrictMode();
		$h->addAndClause("invoice_id", "=", $this->params["invoiceID"][0], "i");
		$invoiceLineItems = $h->getResults(-1, 0);

		// get package and line item ids
		$lineItemIDs = array();
		$pids = array();
		$clientID = 0;
		$lineItemToPackage = array();
		foreach ($invoiceLineItems as $row){
			$pids[] = $row["package_id"];
			$clientID = $row["client_id"];
			$lineItemIDs[] = $row["line_item_id"];
			$lineItemToPackage[$row["line_item_id"]] = $row["package_id"];
			$lineItemToAddon[$row["line_item_id"]] = $row["package_addon_id"];
		}

		$clientInfo = oneByID(":clients", $clientID);

		// getting all shares for those ids
		$hx = new QueryMaster("client_billing_account_shares.*");
		$hx->addAndClause("package_id", "=", $pids, "i");
		$bas = $hx->getResults(-1, 0);
		// making keyed array of billing accounts
		$billingAccounts = array();
		foreach ($bas as $row){
			$billingAccounts[$row["package_id"]] = $row["billing_account_id"];
		}

		$this->currentTaxZones = array();
		// if we dont have a tax exempt client:
		if ((int)$clientInfo["applyTax"]){

			// gotta get tax zones
			$h = new QueryMaster("line_items_tax_zone_rel.*");
			$h->addColumns("tax_zones.*");
			$h->addJoin("tax_zones", "tax_zone_id", "line_items_tax_zone_rel");
			$h->addAndClause("line_item_id", "=", $lineItemIDs, "i");
			$h->addOrderByColumns("tax_zone_stackable", "DESC");
			$currentTaxZones = $h->getResults(-1, 0);
			dbg($currentTaxZones);

			foreach ($currentTaxZones as $row){
				if ((int)$lineItemToAddon[$row["line_item_id"]]){
					$this->currentAddonTaxZones[$lineItemToAddon[$row["line_item_id"]]][] = $row;
				} else {
					$this->currentTaxZones[$lineItemToPackage[$row["line_item_id"]]][] = $row;
				}
			}
			dbg($this->currentTaxZones);
		}

		// init addon ids we need to get
		$packageAddonIDs = array();

		// run through line items getting any addon ids we need
		foreach ($invoiceLineItems as $lineItem){
			if ($lineItem["package_addon_id"]){
				$packageAddonIDs[] = $lineItem["package_addon_id"];
			}
		}

		// query the db for addon info for the addon ids we need
		$h = new QueryMaster("package_addons.*");
		$h->addAndClause("package_addon_id", "=", $packageAddonIDs, "i");
		$addons = $h->getResults(-1, 0);

		// init keyed addon array
		$keyedAddons = array();

		// populate the keyed addon array (not enough data to worry about passing by reference)
		foreach ($addons as $addon){
			if ($addon["package_addon_id"]){
				$keyedAddons[$addon["package_addon_id"]] = $addon;
			}
		}

		// go through line items and add addon data into line items data where needed
		foreach ($invoiceLineItems as $k=>$lineItem){
			$invoiceLineItems[$k]["numRenewals"] = $this->getNumRenewals($lineItem); // get the number of times this needs to renew
			$invoiceLineItems[$k]["cycleInfo"] = $this->translateCycleInfo($lineItem);
			if ($lineItem["package_addon_id"]){
				$invoiceLineItems[$k]["addonInfo"] = $keyedAddons[$lineItem["package_addon_id"]];
			}
		}
		dbg($invoiceLineItems);


		// initialize the array we will keep track of how many (and for what) subscriptions we will need
		$subscriptions = array();

		$unsupportedCycle = false;

		$needSubscription = 0;

		// loop through the line items of this invoice and group them into subarrays based on their cycle and
		// coupon num cycles
		$oneOffPayments = array();

		foreach ($invoiceLineItems as $lineItem){
			if ( $lineItem["cycleInfo"]===false && 1 == $this->config["Enable_Subscriptions"] ) {
				$unsupportedCycle = true;
			}
			$billingAccountID = $billingAccounts[$lineItem["package_id"]];
			if ($lineItem["package_addon_id"]){
				$cycle_id = $lineItem["cycle_id"];
				$coupon_num_cycles = $lineItem["addonInfo"]["coupon_num_cycles"];
				$num_renewals = $lineItem["numRenewals"];
			} else {
				$cycle_id = $lineItem["cycle_id"];
				$coupon_num_cycles = $lineItem["coupon_num_cycles"];
				$num_renewals = $lineItem["numRenewals"];
			}
			if ($num_renewals!=0){
				$needSubscription = 1;
			}
			if ($num_renewals===0){
				$oneOffPayments[] = $lineItem;
			} else {
				$subscriptions[$cycle_id."-".$num_renewals."-".$coupon_num_cycles."-".$billingAccountID][] = $lineItem;
			}
		}

		dbg($subscriptions);

		$returnHTML = array();


		if ( 0 == $this->config["Enable_Subscriptions"] || 0 == $needSubscription ) {

			if (round($invoiceInfo["invoice_amount"]-$invoiceInfo["invoice_amount_paid"], 2)>0){
				$oneOffPayments = array(); // need to zero out the array if we did a one off payment for the whole invoice
				// make the one time payment stuff here
				$oneTime = array();
				$oneTime["cmd"] = "_xclick";
				$oneTime["lc"] = strtoupper($this->params["countriesISO2"][0]);
				$oneTime["image_url"] = urlencode($this->config["Display_Logo"]);
				$oneTime["notify_url"] = $this->config["IPN_URL"];
				$oneTime["business"] = $this->config["PayPal_ID"];
				$oneTime["item_name"] = $this->config["Item_Name"];
				$oneTime["item_number"] = "I".$this->params["invoiceID"][0]."P".$this->configID;
				$oneTime["amount"] = $invoiceInfo["invoice_amount"]-$invoiceInfo["invoice_amount_paid"];
				$oneTime["no_note"] = "1";
				$oneTime["invoice"] = $this->params["invoiceID"][0]."P".$this->configID;
				$oneTime["currency_code"] = $this->params["currency"][0];

				$returnHTML[] = $this->createOneTimeHTMLOutput($oneTime);

			}

		}
		// ok, added first one time payment, now doing subscriptions

		if ( 1 == $this->config["Enable_Subscriptions"] && 1 == $needSubscription ) {

			// for each subscription group we have, loop through the items and get the prices and the cycle unit
			// and the num promotional period cycles.
			foreach ($subscriptions as $items){

				$initialPeriodPrice = 0;
				$firstprice = 0;
				$regprice = 0;
				$packages = array();
				$addons = array();
				foreach ($oneOffPayments as $item){
					$initialPeriodPrice += $item["line_item_total_amount"];
				}
				$oneOffPayments = array(); // already took care of these so lets get rid of them
				foreach ($items as $item){
					$initialPeriodPrice += $item["line_item_total_amount"];
					if ($item["package_addon_id"]){
						$addons[] = $item["package_addon_id"];
						$couponcycles = $item["addonInfo"]["coupon_num_cycles"];
						if ($couponcycles>1)$firstprice = round($firstprice+$item["line_item_total_amount"], 2);
						$tp = $item["addonInfo"]["package_addon_price"]*$item["line_item_quantity"];
						$tp += $this->calculateTaxes($tp, $item["package_id"], $item["package_addon_id"]);
						$regprice = round($regprice+$tp, 2);
					} else {
						$packages[] = $item["package_id"];
						$couponcycles = $item["coupon_num_cycles"];
						if ($couponcycles>1)$firstprice = round($firstprice+$item["line_item_total_amount"], 2);
						$tp = $item["package_price"]*$item["line_item_quantity"];
						$tp += $this->calculateTaxes($tp, $item["package_id"]);
						$regprice = round($regprice+$tp, 2);
					}
				}

				$h = new QueryMaster("insert");
				$h->addTables("subscriptions");
				$h->addParam("subscription_status", getAPIStatusID("subscription", "inactive"), "i");
				$h->addParam("client_id", $clientID, "i");
				$h->addParam("subscription_date_created", mktime(), "i");
				$h->addParam("subscription_date_modified", mktime(), "i");
				$h->addParam("payment_gateway_id", $this->configID, "i");
				$h->addParam("billing_account_id", $billingAccounts[$item[0]["package_id"]], "i");
				$h->addParam("subscription_account_identifier", trim(rtrim($this->config["PayPal_ID"])), "s");
				$subresult = $h->execute();
				$subscriptionID = $h->getLastInsertID();

				dbg($packages);
				dbg($addons);

				foreach ($packages as $pid){
					$h = new QueryMaster("insert");
					$h->addTables("subscription_package_rel");
					$h->addParam("subscription_id", $subscriptionID, "i");
					$h->addParam("package_id", $pid, "i");
					$h->execute();
				}

				foreach ($addons as $pid){
					$h = new QueryMaster("insert");
					$h->addTables("subscription_package_addon_rel");
					$h->addParam("subscription_id", $subscriptionID, "i");
					$h->addParam("package_addon_id", $pid, "i");
					$h->execute();
				}


				//$initialPeriodPrice = (count($returnHTML)==0)?$invoiceInfo["invoice_amount"]:0;

				$periodLength = $items[0]["cycleInfo"]["cycles"];
				$periodUnit = $items[0]["cycleInfo"]["unit"];

				$subscription = array();
				$subscription["image_url"] = urlencode($this->config["Display_Logo"]);
				$subscription["lc"] = strtoupper($this->params["countriesISO2"][0]);
				$subscription["cmd"] = "_xclick-subscriptions";
				$subscription["notify_url"] = $this->config["IPN_URL"];
				$subscription["business"] = $this->config["PayPal_ID"];
				$subscription["item_name"] = $this->config["Item_Name"];
				$subscription["item_number"] = "SCR".$subscriptionID;
				$subscription["no_shipping"] = "1";
				$subscription["no_note"] = "1";
				$subscription["currency_code"] = $this->params["currency"][0];

				$subscription["a1"] = round($initialPeriodPrice, 2); // trial period 1 price
				$subscription["p1"] = $periodLength; // trial period 1 length
				$subscription["t1"] = $periodUnit; // trial period 1 unit

				if ($firstprice>0){
					$subscription["a2"] = $firstprice; // trial period 2 price
					$subscription["p2"] = $couponcycles; // trial period 2 length
					$subscription["t2"] = $periodUnit; // trial period 2 unit
				}

				$subscription["a3"] = $regprice; // price per cycle
				$subscription["p3"] = $periodLength; // reg cycle length
				$subscription["t3"] = $periodUnit; // reg cycle units

				$subscription["src"] = "1";
				$subscription["sra"] = "1";
				if ($items[0]["numRenewals"]>-1){
					$subscription["srt"] = $items[0]["numRenewals"]; // number of times this payment will happen
				}

				if (0 < $subscription["a1"] + $subscription["a2"] + $subscription["a3"])
					$returnHTML[] = $this->createSubscriptionHTMLOutput($subscription);
			}

			////

		}

		$this->hasExecuted = 1;

		if ($unsupportedCycle){
			$retval .= "   <gatewayReturn>\n";
			$retval .= "     <status>3</status>\n";
			$retval .= "     <message>Unsupported cycle</message>\n";
			$retval .= "	 <unsupportedCycle>1</unsupportedCycle>\n";
			$retval .= "   </gatewayReturn>\n";
		} else {
			$retval .= "   <gatewayReturn>\n";
			$retval .= "     <status>1</status>\n";
			foreach ($returnHTML as $ret){
				$retval .= "     <paymentForm>".utf8_html_entity_encode($ret["paymentForm"])."</paymentForm>\n";
				$retval .= "     <paymentLink>".utf8_html_entity_encode($ret["paymentLink"])."</paymentLink>\n";
				$retval .= "     <paymentLinkURL>".utf8_html_entity_encode($ret["paymentLinkURL"])."</paymentLinkURL>\n";
			}
			$retval .= "   </gatewayReturn>\n";
		}

		$this->xmlData = $retval;
		return $retval;
	} //execute

	public function setOverrideClientID($clientID)
	{
		$this->manualClientID = $clientID;
	}

	public function enableCreateClient()
	{
		$this->createClientOnDemand = 1;
	}

	/**
	*	process an IPN
	*
	*/
	public function processCallbackReplay($callbackLogID)
	{
		if ( is_null($callbackLogID) ) {
			return false;
		}
		$this->callbackLogID = $callbackLogID;

		$h = new QueryMaster("callback_log_attributes.*");
		$h->addAndClause("callback_log_id", "=", $callbackLogID, "i");
		$attrs = $h->getResults();

		$IPNData = array();
		foreach ( $attrs as $attr ) {
			$IPNData[$attr["callback_log_attribute_name"]] = $attr["callback_log_attribute_value"];
		}
		if ( "" == $IPNData["receiver_email"] ) {
			return false;
		}

		dbg("PayPal: processing ipn replay");
		list($callbackLogAction, $callbackLogActionStatus) = $this->getCallbackLogAction($IPNData);
		$mapData = $this->getIPNMapData($IPNData);

		// check for IPN's for paypal accounts we do not recognize
		if ( !$this->authenticate($mapData["paymentGatewayID"], $IPNData) ) {
			$this->logIPN("TRANS_INVALID_RECEIVER_FRAUD_DETECTED");
			return false;
		}

		if ( is_null($mapData["clientID"]) ) {
			$this->logIPN("TRANS_UNABLE_TO_FIND_APPROPRIATE_CLIENT");
			return false;
		}

		updateForKey("callbackLogs", $callbackLogID, $mapData);

		$this->handleIPNActions($mapData, $IPNData, $callbackLogAction);

		return true;
	}

	/**
	*	process an IPN
	*/
	public function processReturn($post)
	{
		$IPNData = array();
		if ( isset($post["V4IPN"]) ) {
			$IPNData["V4IPN"] = $post["V4IPN"];
			$post = unserialize(base64_decode($post["V4IPN"]));
			dbg("got a v4 ipn");
			dbg($post);
		}

		if ( "" == $post["receiver_email"] ) {
			return false;
		}

		dbg("PayPal: processing ipn return");

		// read the post from PayPal system and add 'cmd'
		$postFields = "cmd=_notify-validate&";
		foreach ( $post as $k => $v ) {
			$IPNData[$k] = secureInput(stripslashes($v));
			$postFields .= "$k=".urlencode(stripslashes($v))."&";
		}

		dbg("getting ready to do the curl call:");
		dbg($postFields);
		$hashString = "";
		foreach ( $IPNData as $k => $v ) {
			$hashString .= "<$k>$v";
		}
		$IPNData["MBv5Signiture"] = sha1($hashString);

		$mapData = $this->getIPNMapData($IPNData);
		if ( !isset($mapData["paymentGatewayID"]) ) {
			throw new Exception("Missing parameter 'paymentGatewayID'");
		}
		$payment_gateway = Factory()->PaymentGateway($mapData["paymentGatewayID"]);
		$this->setURLs($payment_gateway->PaymentGatewayParams["Testing"]->value);

		$curlcall = $this->getCurlObject($this->IPNVerifyURL, $postFields);
		$curlResult = $curlcall->toString();
		// uncomment this to fake a good result:
		// $curlResult = " VERIFIED ";
		dbg("did curl call");
		dbg($curlResult);
		if ( !preg_match("/VERIFIED/", $curlResult) ) {
			dbg("IPN is NOT valid");
			$this->logIPN("TRANS_PAYPAL_DIDNT_VERIFY");
			return false;
		}
		dbg("IPN is valid");

		// lets test for duplicate IPN's
		// we take the hash of our stuff and check for a callback log attribute
		// for a paypal callback log record that matches our hash.
		$h = new QueryMaster("callback_log_attributes.*");
		$h->addTables("callback_log");
		$h->addJoin("callback_log", "callback_log_id", "callback_log_attributes");
		$h->addAndClause("callback_log_attributes.callback_log_attribute_name", "=", "MBv5Signiture", "s");
		$h->addAndClause("callback_log.callback_log_type", "=", getAPITypeID("callbackLog", "paypal"), "s");
		$h->addAndClause("callback_log_attributes.callback_log_attribute_value", "=", $IPNData["MBv5Signiture"], "s");
		$testRow = $h->getRow();
		if ($testRow["callback_log_attribute_value"]==$IPNData["MBv5Signiture"]){
			dbg($testRow);
			dbg($IPNData);
			$this->logIPN("TRANS_DUPLICATE_IPN_POSTING");
			return false;
		}

		// this little block of code inserts a callback log record AND its
		// callback log attributes (which are everything in the $IPNData
		// array, which includes our identity hash).
		list($callbackLogAction, $callbackLogActionStatus) = $this->getCallbackLogAction($IPNData);
		$this->callbackLogID = $this->insertCallbackLog(
			array(
				"callbackLogType" => getAPITypeID("callbackLog", "paypal"),
				"callbackLogTransactionID" => $IPNData["txn_id"],
				"callbackLogParentTransactionID" => $IPNData["parent_txn_id"],
				"callbackLogAction" => getAPITypeID("callbackAction", $callbackLogAction),
				"callbackLogActionStatus" => getAPIStatusID("callbackAction", $callbackLogActionStatus),
				"callbackLogDateCreated" => time(),
			) + $mapData,
			$IPNData
		);

		// check for IPN's for paypal accounts we do not recognize
		if ( !$this->authenticate($mapData["paymentGatewayID"], $IPNData) ) {
			$this->logIPN("TRANS_INVALID_RECEIVER_FRAUD_DETECTED");
			return false;
		}

		if ( is_null($mapData["clientID"]) ) {
			$this->logIPN("TRANS_UNABLE_TO_FIND_APPROPRIATE_CLIENT");
			return false;
		}

		$this->handleIPNActions($mapData, $IPNData, $callbackLogAction);

		return true;
	}

	/**
	*	take a fields array and create the html output fields needed
	*
	*/
	private function createSubscriptionHTMLOutput($fields)
	{
		// init return variable
		$ret = array();

		$ret["paymentForm"] = "<form action='".$this->subscriptionURL."' method='post' target='_blank'>
			<input type='image' src='".$this->config["Subscriptions_Image"]."' border='0' name='submit' alt='Pay with Paypal'>";

		$ret["paymentLinkURL"] = $this->subscriptionURL."?";

		foreach ($fields as $k=>$v){
			$ret["paymentLinkURL"] .= $k."=".urlencode($v)."&";
			$ret["paymentForm"] .= "<input type='hidden' name='".$k."' value='".$v."' />";
		}

		$ret["paymentForm"] .= "</form>";

		$ret["paymentLink"] = "<a href='".$ret["paymentLinkURL"]."' target='_blank'>".(($this->params["imageLinks"]) ? "<img src=\"".$this->config["Submit_Image"]."\" />" : TRANS_PAY_WITH_PAYPAL)."</a>";

		return $ret;

	}

	/**
	*	take a fields array and create the html output fields needed
	*
	*/
	private function createOneTimeHTMLOutput($fields)
	{
		// init return variable
		$ret = array();

		$ret["paymentForm"] = "<form action='".$this->paymentURL."' method='post' target='_blank'>
			<input type='image' src='".$this->config["Submit_Image"]."' border='0' name='submit' alt=\"Make payments with PayPal - it's fast, free and secure!\">";

		$ret["paymentLinkURL"] = $this->paymentURL."?";

		foreach ($fields as $k=>$v){
			$ret["paymentLinkURL"] .= $k."=".urlencode($v)."&";
			$ret["paymentForm"] .= "<input type='hidden' name='".$k."' value='".$v."' />";
		}

		$ret["paymentForm"] .= "</form>";

		$ret["paymentLink"] = "<a href='".$ret["paymentLinkURL"]."' target='_blank'>".(($this->params["imageLinks"]) ? "<img src=\"".$this->config["Submit_Image"]."\" border=\"0\" />" : TRANS_PAY_WITH_PAYPAL)."</a>";

		return $ret;

	}

	/**
	 * function to handle things like chargebacks, chargeback reversals, pending echecks, echeck payments
	 */
	private function handleIPNActions($mapData, $data, $callbackLogAction)
	{
		$this->isSubscriptionAction = (strlen($data["subscr_id"])>2)?1:0;
		$this->parentTransactionInformation = $this->findParentInformation($data);
		switch ($callbackLogAction){
			// non-payment related ipn's
			case "subscriptionSignup": // tested works
				$this->handleSubscriptionSignup($mapData, $data);
				break;
			case "subscriptionFailed": // uses same code (so it works)
				$this->handleSubscriptionFailed($mapData);
				break;
			case "subscriptionCancelation": // tested works
				$this->handleSubscriptionCanceled($mapData);
				break;
			case "subscriptionExpired": // uses same code (so it works)
				$this->handleSubscriptionExpired($mapData);
				break;
			case "paymentEcheckPending":
				break;
			case "newDisputeCase":
				break;

			// payment related ipn's
			case "subscriptionPayment": // tested works
				$this->handleSubscriptionPayment($mapData, $data);
				break;
			case "paymentEcheckFinished": // post the echeck
			case "payment": // tested works
				$this->handlePayment($mapData, $data);
				break;

			// repayment (treat these as new payments based on whether they are subscription related or not)
			case "paymentReversalCancelation":
			case "paymentChargebackCancelation":
			case "paymentEcheckReversalCancelation":
				if ($this->isSubscriptionAction){
					$this->handleSubscriptionPayment($mapData, $data);
				} else {
					$this->handlePayment($mapData, $data);
				}
				break;

			// refund related, do refund actions
			case "paymentEcheckDenied":
			case "paymentReversal":
			case "paymentChargeback":
			case "paymentEcheckReversal":
			case "paymentRefundedGateway": // tested works
				$this->handleTransactionRefund($mapData, $data);
				break;
		}
	}

	private function handleTransactionRefund($mapData, $IPNData)
	{
		if (!$this->parentTransactionInformation["callbackLog"]["callback_log_id"]){
			dbg($this->parentTransactionInformation);
			$this->logIPN("TRANS_REFUND_IPN_WITH_NO_PARENT");
			return;
		}

		$parentTransaction = $this->getParentTransaction($IPNData["parent_txn_id"]);

		if ($parentTransaction["transaction_id"]){
			$transactionID = $parentTransaction["transaction_id"];

			// update the transaction status to refunded
			$h = new QueryMaster("update");
			$h->addTables("transaction_log");
			$h->addAndClause("transaction_id", "=", $transactionID, "i");
			$h->addParam("auth_return", getAPIStatusID("transactions", "refunded"), "i");
			$h->addParam("transaction_amount_refunded", abs($IPNData["mc_gross"]), "i");
			$h->execute();


			$h = new QueryMaster("transaction_success_rel.*");
			$h->addColumns("invoices.*");
			$h->addColumns("invoice_line_items.invoice_id");
			$h->addJoin("invoices", "invoice_id", "invoice_line_items");
			$h->addJoin("invoice_line_items", "line_item_id", "transaction_success_rel");
			$h->addAndClause("transaction_id", "=", $transactionID, "i");
			$lineItems = $h->getResults();
			$invoices = array();
			$invoicesData = array();
			foreach ((array)$lineItems as $li){
				$invoicesData[$li["invoice_id"]] = $li;
				$invoices[$li["invoice_id"]] += $li["line_item_payment_amount"];
			}

			// delete the payment relations to line items for it
			$h = new QueryMaster("delete");
			$h->addTables("transaction_success_rel");
			$h->addAndClause("transaction_id", "=", $transactionID, "i");
			dbg($h->getQuery());
			$h->execute();

			foreach ((array)$invoices as $invoiceID=>$amountRefunded){
				$invoiceInfo = $invoicesData[$invoiceID];
				$h = new QueryMaster("update");
				$h->addTables("invoices");
				$h->addAndClause("invoice_id", "=", $invoiceID, "i");
				$h->addParam("invoice_amount_paid", $invoiceInfo["invoice_amount_paid"]-$amountRefunded, "d");
				$h->addParam("invoice_status", getAPIStatusID("invoice", "due"), "i");
				$h->execute();
			}

			dispatchMBAPI(toXML(array(
				"mbapi" => array(
					"command" => "ProcessRefundAccounting",
					"params" => array(
						"transactionID" => $transactionID,
					),
				)
			)));

		} else {
			dbg($this->parentTransactionInformation);
			dbg($parentTransaction);
			$this->logIPN("TRANS_REFUND_IPN_WITH_MISSING_TRANSACTION");
		}
	}

	private function findParentInformation($data)
	{
		if ($data["parent_txn_id"]){
			$h = new QueryMaster("callback_log.*");
			$h->addAndClause("callback_log.callback_log_type", "=", getAPITypeID("callbackLog", "paypal"), "i");
			$h->addAndClause("callback_log.callback_log_transaction_id", "=", $data["parent_txn_id"], "s");
			$callbackLog = $h->getRow();
			dbg($h->getQuery());
			if ($callbackLog["callback_log_id"]){
				$h = new QueryMaster("callback_log_attributes");
				$h->addAndClause("callback_log_id", "=", $callbackLog["callback_log_id"], "i");
				$callbackLogAttributes = $h->getResults();
				return array(
					"callbackLog" => $callbackLog,
					"callbackLogAttributes" => $callbackLogAttributes
				);
			} else {
				dbg("didnt get anything from query");
			}
		}
		dbg("no parent or didnt get anything from query");
		return array();
	}

	private function getCallbackLogAttributeByName($attributes, $name)
	{
		foreach ((array)$attributes as $attr){
			if ($attr["callback_log_attribute_name"]==$name){
				return $attr;
			}
		}
	}

	private function handleSubscriptionExpired($mapData)
	{
		dbg("subscription end of term IPN.");
		if ($mapData["subscriptionID"]){
			$this->updateSubscriptionStatus($mapData["subscriptionID"], "expired");
		} else {
			$this->logIPN("TRANS_UNMATCHED_SUBSCRIPTION");
		}
	}

	private function handleSubscriptionFailed($mapData)
	{
		if ($mapData["subscriptionID"]){
			$this->updateSubscriptionStatus($mapData["subscriptionID"], "failed");
		} else {
			$this->logIPN("TRANS_UNMATCHED_SUBSCRIPTION");
		}
	}

	private function handleSubscriptionCanceled($mapData)
	{
		if ($mapData["subscriptionID"]){
			$this->updateSubscriptionStatus($mapData["subscriptionID"], "cancelled");
		} else {
			$this->logIPN("TRANS_UNMATCHED_SUBSCRIPTION");
		}
	}

	private function updateSubscriptionStatus($subscriptionID, $statusName)
	{
		dbg("updating subscription $subscriptionID to status $statusName");
		if ($subscriptionID){
			dbg("made it in");
			$h = new QueryMaster("update");
			$h->addTables("subscriptions");
			$h->addParam("subscription_status", getAPIStatusID("subscription", $statusName), "i");
			$h->addAndClause("subscription_id", "=", $subscriptionID, "i");
			dbg($h->getQuery());
			$h->execute();
		}
	}

	private function handleSubscriptionSignup($mapData, $data)
	{
		if (!$mapData["subscriptionID"]){
			if ($mapData["clientID"]){

				$h = new QueryMaster("insert");
				$h->addTables("subscriptions");
				$h->addParam("subscription_status", getAPIStatusID("subscription", "active"), "i");
				$h->addParam("client_id", $mapData["clientID"], "i");
				$h->addParam("subscription_date_created", time(), "i");
				$h->addParam("subscription_date_modified", time(), "i");
				if ( $mapData["paymentGatewayID"] ) {
					$h->addParam("payment_gateway_id", $mapData["paymentGatewayID"], "i");
				}
				$h->addParam("subscription_account_identifier", trim(rtrim($this->config["PayPal_ID"])), "s");
				$subresult = $h->execute();
				$subscriptionID = $h->getLastInsertID();

				$h = new QueryMaster("packages.package_id");
				$h->addAndClause("client_id", "=", $mapData["clientID"], "i");
				$packageIDs = $h->getCol();

				$h = new QueryMaster("package_addons.package_addon_id");
				$h->addTables("packages");
				$h->addJoin("packages", "package_id", "package_addons");
				$h->addAndClause("client_id", "=", $mapData["clientID"], "i");
				$packageAddonIDs = $h->getCol();

				foreach ($packageIDs as $pid){
					$h = new QueryMaster("insert");
					$h->addTables("subscription_package_rel");
					$h->addParam("subscription_id", $subscriptionID, "i");
					$h->addParam("package_id", $pid, "i");
					$h->execute();
				}

				foreach ($packageAddonIDs as $pid){
					$h = new QueryMaster("insert");
					$h->addTables("subscription_package_addon_rel");
					$h->addParam("subscription_id", $subscriptionID, "i");
					$h->addParam("package_addon_id", $pid, "i");
					$h->execute();
				}

			}
		} else {
			$h = new QueryMaster("update");
			$h->addTables("subscription");
			$h->addParam("subscription_status", getAPIStatusID("subscription", "active"), "i");
			$h->addAndClause("subscription_id", "=", $mapData["subscriptionID"], "i");
			$h->execute();
		}
	}


	/**
	 * function to handle a subscription payment
	 * BEGIN applying payment to packages and addons (for a subscription ipn)
	 * this will create a transaction to record the ipn, as well as creating the success ids for the
	 * line items that got paid, and changing the status of the invoice they are on accordingly as is necessary.
	 */
	private function handleSubscriptionPayment($mapData, $IPNData)
	{
		if ($IPNData["subscr_id"] && $this->subscriptionInfo["subscription_external_id"]==""){
			$h = new QueryMaster("update");
			$h->addTables("subscriptions");
			$h->addParam("subscription_status", getAPIStatusID("subscription", "active"), "i");
			$h->addParam("subscription_external_id", $IPNData["subscr_id"], "s");
			$h->addAndClause("subscription_id", "=", $this->subscriptionInfo["subscription_id"], "i");
			$h->execute();
			dbg("updated subscription status");
		}

		$transactionID = $this->makeTransaction($IPNData, false, $this->subscriptionInfo["payment_gateway_id"]);

		// lets set the client id in the main scope
		$clientID = $mapData["clientID"];
		$invoiceID = (int)$mapData["invoiceID"];

		// need to get all of the unpaid invoices and their line items that correspond to the package ids we have
		$allLineItems = $this->getLineItems($invoiceID, $this->getSubscriptionRelatedPackages());
		$lineItemIDs = $this->getLineItemIDsFromLineItems($allLineItems);
		$lineItemPayments = $this->getLineItemPayments($lineItemIDs);

		// the invoice id we will work with, will be the first invoice for a package we are paying for
		// that has one that isn't paid
		$invoiceID = (!$invoiceID)?$allLineItems[0]["invoice_id"]:$invoiceID;

		// now we need to apply the payment
		$this->payLineItemsAndDoAccounting(
			$mapData,
			$IPNData,
			$this->getUnpaidLineItems($allLineItems, $lineItemPayments, $invoiceID),
			$transactionID,
			$invoiceID,
			$clientID
		);

		// send subscription payment receipt email (LAST because people can mess up the template...)
		$this->sendSecondaryPaymentEmail($clientID, $transactionID, $invoiceID);
	}


	/**
	 * Handle a payment that is not for a subscription we have a record of
	 * this will create a transaction to record the ipn, as well as creating the success ids for the
	 * line items that got paid, and changing the status of the invoice as is necessary.
	 */
	private function handlePayment($mapData, $IPNData)
	{
		dbg("applying IPN paypal payment for invoice");

		$clientID = $mapData["clientID"];

		$invoiceID = $mapData["invoiceID"];
		if ( !$invoiceID ) {
			// if we dont have an invoice to pay lets get one
			$h = new QueryMaster("invoices.*");
			$h->addTables("invoice_line_items");
			$h->addTables("packages");
			$h->addJoin("packages", "package_id", "invoice_line_items");
			$h->addJoin("invoice_line_items", "invoice_id", "invoices");
			$h->addAndClause("packages.client_id", "=", $mapData["clientID"], "i");
			$h->addCustomAndClause("invoices.invoice_amount_paid <> invoices.invoice_amount");
			$h->addAndClause("invoices.invoice_status", "=", array(
				getAPIStatusID("invoice", "due"),
				getAPIStatusID("invoice", "declined"),
				getAPIStatusID("invoice", "errored"),
			), "i");
			$h->addOrderByColumns("invoice_date_due", "ASC");
			$invoiceInfo = $h->getRow();
			$invoiceID = $invoiceInfo["invoice_id"];
		} else {
			$h = new QueryMaster("invoices.*");
			$h->addAndClause("invoice_id", "=", $invoiceID, "i");
			$invoiceInfo = $h->getRow();
		}

		// lets go ahead and make the transaction
		$transactionID = $this->makeTransaction($IPNData, $invoiceInfo["invoice_number"], $mapData["paymentGatewayID"]);
		dbg("inserted transaction: ".$transactionID);

		if ( !$invoiceID ) {
			// if we couldn't find an invoice to pay, lets just give this client a credit and be done with it.
			$this->giveClientCredit($mapData, $IPNData);
		} else {
			// get the line items data...
			$allLineItems = $this->getLineItems($invoiceID);
			$lineItemIDs = $this->getLineItemIDsFromLineItems($allLineItems);
			$lineItemPayments = $this->getLineItemPayments($lineItemIDs);

			// now we need to apply the payment
			$this->payLineItemsAndDoAccounting(
				$mapData,
				$IPNData,
				$this->getUnpaidLineItems($allLineItems, $lineItemPayments),
				$transactionID,
				$invoiceID,
				$clientID
			);
		}

		// send the email last because people can break the template
		$this->sendSecondaryPaymentEmail($clientID, $transactionID, $invoiceID);
	}


	/**
	 * function to give a client a credit because we can't find anything to apply the payment to
	 */
	private function giveClientCredit($mapData, $IPNData)
	{
		dispatchMBAPI(toXML(array(
			"mbapi" => array(
				"command" => "SetCreditAccounting",
				"params" => array(
					"clientID" => $mapData["clientID"],
					"ledgerAccountName" => "cash",
					"amount" => $IPNData["mc_gross"],
					"currencyID" => $IPNData["mc_currency"],
					"description" => TRANS_PAYPAL_PAYMENT_WITH_NOTHING_TO_PAY,
				),
			),
		)));
	}

	/**
	 * function to get a transaction based on txn_id
	 */
	private function getParentTransaction($txn_id)
	{
		$h = new QueryMaster("transaction_log.*");
		$h->addAndClause("transaction_code", "=", "paypal".$txn_id, "s");
		dbg("got parent:");
		dbg($x = $h->getRow());
		return $x;
	}

	/**
	 * function to shortcut making a transaction
	 */
	private function makeTransaction($IPNData, $invoiceNumber, $paymentGatewayID)
	{
		$transactionID = $this->insertTransaction(array(
			"auth_return" => GATEWAY_SUCCESS,
			"transaction_payment_method" => getAPITypeID("payments", "paypal"),
			"transaction_code" => "paypal".$IPNData["txn_id"],
			"transaction_notes" => TRANS_PAYMENT_FOR_INVOICE. (($invoiceNumber)?": ".$invoiceNumber:''),
			"transaction_date_created" => time(),
			"transaction_amount" => doubleval($IPNData["mc_gross"]),
			"currency_id" => $IPNData["mc_currency"],
			"payment_gateway_id" => $paymentGatewayID,
		));
		if ((int)$this->callbackLogID){
			updateForKey("callbackLogs", $this->callbackLogID, array(
				"transactionID" => $transactionID,
			));
		}
		return $transactionID;
	}

	/**
	 * function to set the payments for line items and make the accounting entries
	 */
	private function payLineItemsAndDoAccounting($mapData, $IPNData, $unpaidLineItems, $transactionID, $invoiceID, $clientID)
	{
		// ok, got all the unpaid line items for the oldest invoice that has packages we are paying for
		// in an array called $unpaidLineItems, now we just need to loop through them and apply the money
		// towards them.
		$invoiceInfo = $unpaidLineItems[0]; // the invoice info was gotten in the line items query

		$amountAllowcated = 0;
		foreach ($unpaidLineItems as $k=>$li){
			$available = doubleval($IPNData["mc_gross"])-$amountAllowcated;
			$amountNeeded = $li["line_item_total_amount"]-$li["paid"];
			if ($amountNeeded>$available){
				$payAmount = $available;
			} else {
				$payAmount = $amountNeeded;
			}
			$unpaidLineItems[$k]["applyAmount"] = $payAmount;
			$amountAllowcated += $payAmount;
		}
		$unallowcated = doubleval($IPNData["mc_gross"])-$amountAllowcated;
		dbg("applied payments to line items, unallowcated: ".$unallowcated);


		// ok, so we made the amounts to pay, lets pay them
		// (that means inserting the transaction_success_rel entries and updating the invoice paid amt)
		foreach ($unpaidLineItems as $k=>$li){
			if ($li["applyAmount"]){
				$h = new QueryMaster("insert");
				$h->addTables("transaction_success_rel");
				$h->addParam("transaction_id", $transactionID, "i");
				$h->addParam("line_item_id", $li["line_item_id"], "i");
				$h->addParam("line_item_payment_amount", doubleval($li["applyAmount"]), "d");
				$h->execute();
				$invoiceInfo["amountPaidNow"] += doubleval($li["applyAmount"]);
				dbg("added payment for line item: ".$li["line_item_id"]." of ".doubleval($li["applyAmount"]));
			}
		}

		$h = new QueryMaster("update");
		$h->addTables("invoices");
		$h->addAndClause("invoice_id", "=", $invoiceID, "i");
		$h->addParam("invoice_amount_paid", round(doubleval($invoiceInfo["amountPaidNow"]+$invoiceInfo["invoice_amount_paid"]), 2), "d");
		$h->addParam("invoice_date_paid", time(), "i");
		if (round(($invoiceInfo["amountPaidNow"]+$invoiceInfo["invoice_amount_paid"]),2)==round($invoiceInfo["invoice_amount"], 2)){
			$h->addParam("invoice_status", 1, "i");
			dbg("invoice is closed");
		}
		$h->execute();
		dbg("updated invoice info");

		if ($unallowcated){
			$credit = $unallowcated;
			dbg("theres a credit amount");
		}

		// make the accounting entry
		require_once mb_path("/lib-billing/include/accounting/Payment.php");
		$receipt = new Payment();
		$receipt->setClientID($clientID);
		$receipt->setInvoiceID($invoiceID);
		$receipt->invoiceNumber = $invoiceInfo["invoice_number"];
		$receipt->setTransactionID($transactionID);
		$receipt->setDescription(TRANS_PAYPAL_PAYMENT_FOR_INVOICE.": ".$invoiceInfo["invoice_number"]);
		$receipt->setPaymentAmount(doubleval($IPNData["mc_gross"]));
		if ($credit>0){
			$receipt->setClientCreditAmount($credit);
		}
		$receipt->setCurrencyID($IPNData["mc_currency"]);
		$receipt->execute();
		dbg("made the acocunting entry");

		// associate the invoice with the transaction
		$h = new QueryMaster("insert");
		$h->addTables("invoice_transaction_rel");
		$h->addParam("invoice_id", $invoiceID, "i");
		$h->addParam("transaction_id", $transactionID, "i");
		$h->execute();
		dbg("set the invoice transaction rel record");

		$this->addPaypalFee($clientID, $invoiceID, $IPNData["mc_fee"], $IPNData["mc_currency"]);
		// handle currency conversion if necessary
		$this->makeAccountingCurrenyConversion($IPNData["mc_gross"], $IPNData["mc_currency"], $IPNData["settle_amount"], $IPNData["settle_currency"], $clientID, $invoiceID);
	}


	/**
	 * function to get the unpaid line items from a set of line items and their payment data
	 */
	private function getUnpaidLineItems($allLineItems, $lineItemPayments, $invoiceID=false)
	{
		$unpaidLineItems = array();
		if ($invoiceID){
			foreach ($allLineItems as $k=>$li){
				$allLineItems[$k]["paid"] = $lineItemPayments[$li["line_item_id"]]["paid"];
				if ($lineItemPayments[$li["line_item_id"]]["paid"]<$li["line_item_total_amount"] && $li["invoice_id"]==$invoiceID){
					$unpaidLineItems[] = $allLineItems[$k];
				}
			}
			dbg("figured out which ones weren't paid for: ".count($unpaidLineItems));
			return $unpaidLineItems;
		} else {
			foreach ($allLineItems as $k=>$li){
				$allLineItems[$k]["paid"] = $lineItemPayments[$li["line_item_id"]]["paid"];
				if ($lineItemPayments[$li["line_item_id"]]["paid"]<$li["line_item_total_amount"]){
					$unpaidLineItems[] = $allLineItems[$k];
				}
			}
			dbg("figured out which ones weren't paid for: ".count($unpaidLineItems));
			return $unpaidLineItems;
		}
	}

	/**
	 * function to get the line items ids out of the line items array
	 */
	private function getLineItemIDsFromLineItems($lineItems)
	{
		$lineItemIDs = array();
		foreach ($lineItems as $li){
			$lineItemIDs[] = $li["line_item_id"];
		}
		return $lineItemIDs;
	}

	/**
	 * function to get the line items for certain parameters
	 */
	private function getLineItems($invoiceID=false, $packageIDs=false)
	{
		if ($invoiceID){
			$h = new QueryMaster("invoices.*", "invoice_line_items.*");
			$h->addAndClause("invoices.invoice_id", "=", $invoiceID, "i");
			$allLineItems = $h->getResults(-1, 0);
			dbg("query: ".$h->getQuery());
			dbg("got all line items: ".count($allLineItems));
			return $allLineItems;
		} else {
			$h = new QueryMaster("invoices.*", "invoice_line_items.*");
			$h->addAndClause("package_id", "=", $packageIDs, "i");
			$h->addCustomAndClause("invoice_amount_paid < invoice_amount");
			$h->addOrderByColumns("invoice_date_due", "ASC");
			$allLineItems = $h->getResults(-1, 0);
			dbg("got all line items: ".count($allLineItems));
			return $allLineItems;
		}
	}

	/**
	 * function to get the success relations for certain line items
	 */
	private function getLineItemPayments($lineItemIDs)
	{
		$h = new QueryMaster("transaction_success_rel.*");
		$h->addAndClause("line_item_id", "=", $lineItemIDs, "i");
		$payments = $h->getResults(-1, 0);
		dbg("got transaction success rows: ".count($payments));

		// add up the payments according to line item id and organize them as well
		$lineItemPayments = array();
		foreach ($payments as $pmnt){
			$lineItemPayments[$pmnt["line_item_id"]]["paid"] += $pmnt["line_item_payment_amount"];
			$lineItemPayments[$pmnt["line_item_id"]]["payments"][] = $pmnt;
		}
		return $lineItemPayments;
	}

	/**
	 * function to get what packages we need to get for a subscription
	 */
	private function getSubscriptionRelatedPackages($subscriptionID=false, $clientID=false)
	{
		$subscriptionID = ($subscriptionID)?$subscriptionID:$this->subscriptionInfo["subscription_id"];
		$clientID = ($clientID)?$clientID:$this->subscriptionInfo["client_id"];
		// get packages related to this subscription
		$h = new QueryMaster("subscription_package_rel.*", "packages.*");
		$h->addJoin("subscription_package_rel", "package_id", "packages");
		$h->addAndClause("subscription_id", "=", $subscriptionID, "i");
		$relatedPackages = $h->getResults(-1, 0);
		dbg("Got related packages:");
		dbg($relatedPackages);
		// get package addons related to this subscription
		$h = new QueryMaster("subscription_package_addon_rel.*", "packages.*", "package_addons.*");
		$h->addJoin("subscription_package_addon_rel", "package_addon_id", "package_addons");
		$h->addJoin("package_addons", "package_id", "packages");
		$h->addAndClause("subscription_id", "=", $subscriptionID, "i");
		$relatedPackageAddons = $h->getResults(-1, 0);
		dbg("Got related package addons:");
		dbg($relatedPackageAddons);

		// if we don't have any packages related to the subscription, lets just
		// get them for the client
		if (!count($relatedPackages)){
			// get packages related to this subscription
			$h = new QueryMaster("packages.*");
			$h->addAndClause("packages.client_id", "=", $clientID, "i");
			$relatedPackages = $h->getResults(-1, 0);
			dbg("Got related packages:");
			dbg($relatedPackages);
			// get package addons related to this subscription
			$h = new QueryMaster("packages.*", "package_addons.*");
			$h->addAndClause("packages.client_id", "=", $clientID, "i");
			$relatedPackageAddons = $h->getResults(-1, 0);
			dbg("Got related package addons:");
			dbg($relatedPackageAddons);
		}

		$packageIDsToGet = array();
		foreach ((array)$relatedPackages as $rp){
			$packageIDsToGet[] = $rp["package_id"];
		}
		foreach ((array)$relatedPackageAddons as $rpa){
			$packageIDsToGet[] = $rpa["package_id"];
		}
		return $packageIDsToGet;
	}

	/**
	 * function to get subscription info using a few different parameters
	 */
	private function getSubscriptionInfo($subscriptionID=false, $externalSubscriptionID=false)
	{
		if ($subscriptionID){
			$h = new QueryMaster("subscriptions.*");
			$h->addAndClause("subscription_id", "=", $subscriptionID, "i");
			$subscriptionInfo = $h->getRow();
		} else {
			$h = new QueryMaster("subscriptions.*");
			$h->addAndClause("subscription_external_id", "=", $externalSubscriptionID, "s");
			$subscriptionInfo = $h->getRow();
		}
		return $subscriptionInfo;
	}

	/**
	 * function to set subscription info
	 */
	private function setSubscriptionInfo($data, $subscriptionID=false, $externalSubscriptionID=false)
	{
		if ($subscriptionID || $externalSubscriptionID){
			$h = new QueryMaster("update");
			$h->addTables("subscriptions");
			if ($subscriptionID){
				$h->addAndClause("subscription_id", "=", $subscriptionID, "i");
			} else {
				$h->addAndClause("subscription_external_id", "=", $externalSubscriptionID, "i");
			}
		} else {
			$h = new QueryMaster("insert");
			$h->addTables("subscriptions");
		}
		foreach ((array)$data as $field=>$row){
			$h->addParam($field, $row["data"], $row["type"]);
		}
		return $h->execute();
	}


	/**
	 * function to log an error'd ipn
	 */
	private function logIPN($message)
	{
		dbg("logging IPN: ".$message);
		insertFor("invalidIpnLogs", array(
			"ipnStatus" => getAPIStatusID("invalidipns", "new"),
			"ipnReason" => $message,
			"ipnModule" => getAPITypeID("ipnmodules", "paypal"),
			"ipnServer" => base64_encode(serialize($_SERVER)),
			"ipnGet" => base64_encode(serialize($_GET)),
			"ipnPost" => base64_encode(serialize($_POST)),
			"ipnDateCreated" => time(),
		));
	}

	/**
	 * do what needs to be done when a subscription expires
	 */
	private function runSubscriptionExpire()
	{
		// which is nothing at the moment.
	}

	/**
	*	get the number of times this package is going to renew
	*/
	private function getNumRenewals($packagerow)
	{
		if ((int)$packagerow["package_date_stop_renewal"]<(25*60*60)){
			return -1;
		} else {
			$num = 0;
			$runningdate = $packagerow["package_date_next_renewal"];
			if ($runningdate>(25*60*60)){
				while ($runningdate<(int)$packagerow["package_date_stop_renewal"]){
					$next_runningdate = getNextBillingTimestamp($runningdate, $packagerow["cycle_id"]);
					$num++;
					if ( $next_runningdate == $runningdate ) {
						dbg("... package #{$packagerow['package_id']} has zero cycle time (cycle_id={$packagerow['cycle_id']}), break");
						break;
					}
					$runningdate = $next_runningdate;
				}
				return $num;
			} else {
				return 1;
			}
		}
	}


	/**
	* get cycle unit and number
	*/
	private function translateCycleInfo($packagerow)
	{
		dbg($packagerow);
		require_once mb_path("/lib-billing/include/config/internal/cycles.php");
		$cycleData = getCycleData($packagerow["cycle_id"]);
		list($sec,$min,$hour,$day,$month,$year) = $cycleData;

		dbg($cycleData);

		// go through our cycle data and first check if they have more than one unit filled in
		// because paypal doesn't support multiple units
		$num = 0;
		foreach ((array)$cycleData as $unit){
			if ($unit>0)$num++;
		}
		// if true this isn't a cycle supported by paypal
		if ($num>1)return false;

		// init return
		$ret = array();

		// got past that check? ok, now to see what we have here...
		if ($day>0 && $day%7>0){// have a day value that isnt issued in whole weeks
			$ret["unit"] = "D";
			$ret["cycles"] = $day;
			return $ret;
		} elseif ($day>0){     // have a week based cycle expressed in days
			$ret["unit"] = "W";
			$ret["cycles"] = $day/7;
			return $ret;
		} elseif ($month>0){  // have a month based cycle
			$ret["unit"] = "M";
			$ret["cycles"] = $month;
			return $ret;
		} elseif ($year>0){  // have a year based cycle
			$ret["unit"] = "Y";
			$ret["cycles"] = $year;
			return $ret;
		}

		// this isn't a cycle that is supported by paypal
		return false;

	}

	/**
	* Calculates the taxes for this invoice amount
	*
	* @param amount    the amount to get taxes on
	* @param packageID  the package id we are getting tax zones for
	* @return the tax amount
	*/
	private function calculateTaxes($amount, $packageID, $packageAddonID=0)
	{
		$taxZones = $packageAddonID ? $this->currentAddonTaxZones[$packageAddonID] : $this->currentTaxZones[$packageID];
		$tax = 0;
		$taxableAmount = $amount;
		for ($i = 0; $i < count($taxZones); $i++){
			$taxZone = $taxZones[$i];
			if (!(int)$taxZone["tax_zone_stackable"]) continue;
			dbg("applying tax zone:");
			dbg($taxZone);
			if ($taxZone["line_item_tax_amount_type"]==0){ // if 0 we use as percentage
				$cTax = round($amount*($taxZone["line_item_tax_amount"]/100), 2);
				$taxableAmount = round($taxableAmount + $cTax, 2);
				$tax = round($tax + $cTax, 2);
			} else { // else  we use as set amount
				$cTax = $taxZone["line_item_tax_amount"];
				$tax = round($tax + $cTax, 2);
			}
		}
		for ($i = 0; $i < count($taxZones); $i++){
			$taxZone = $taxZones[$i];
			if ((int)$taxZone["tax_zone_stackable"]) continue;
			dbg("applying tax zone:");
			dbg($taxZone);
			if ($taxZone["line_item_tax_amount_type"]==0){ // if 0 we use as percentage
				$cTax = round($taxableAmount*($taxZone["line_item_tax_amount"]/100), 2);
				$tax = round($tax + $cTax, 2);
			} else { // else  we use as set amount
				$cTax = $taxZone["line_item_tax_amount"];
				$tax = round($tax + $cTax, 2);
			}
		}
		return $tax;
	}

	/**
	 * function to get the callback action and callback action status
	 */
	private function getCallbackLogAction($data)
	{
		$data["payment_status"] = str_replace("-", "_", $data["payment_status"]);
		$data["reason_code"] = str_replace("-", "_", $data["reason_code"]);
		$callbackLogAction = '';
		$callbackLogActionStatus = 'complete';
		if ($data["txn_type"]=="subscr_eot"){
			$callbackLogAction = 'subscriptionExpired';
		} elseif (strlen($data["payment_type"])){
			if ($data["payment_type"]=="echeck"){
				if (!$data["payment_status"] && $data["pending_reason"]=="echeck"){
					$callbackLogAction = 'paymentEcheckPending';
					$callbackLogActionStatus = 'pending';
				} elseif ($data["payment_status"]=="Denied" || $data["payment_status"]=="Expired"){
					$callbackLogAction = 'paymentEcheckDenied';
				} elseif ($data["payment_status"]=="Canceled_Reversal"){
					$callbackLogAction = 'paymentEcheckReversalCancelation';
				} elseif ($data["payment_status"]=="Reversed"){
					$callbackLogAction = 'paymentEcheckReversal';
				} elseif ($data["payment_status"]=="Processed" || $data["payment_status"]=="Completed"){
					$callbackLogAction = 'paymentEcheckFinished';
				}
			} else {
				if ($data["payment_status"]=="Completed" || $data["payment_status"]=="Processed"){
					if ($data["txn_type"]=="subscr_payment"){
						$callbackLogAction = 'subscriptionPayment';
					} else {
						$callbackLogAction = 'payment';
					}
				} elseif ($data["payment_status"]=="Reversed"){
					if ($data["reason_code"]=="chargeback"){
						$callbackLogAction = 'paymentChargeback';
					} elseif ($data["reason_code"]=="buyer_complaint"){
						$callbackLogAction = 'paymentChargeback';
					} else {
						$callbackLogAction = 'paymentReversal';
					}
				} elseif ($data["payment_status"]=="Canceled_Reversal"){
					if ($data["reason_code"]=="chargeback"){
						$callbackLogAction = 'paymentChargebackCancelation';
					} else {
						$callbackLogAction = 'paymentReversalCancelation';
					}
				} elseif ($data["payment_status"]=="Refunded"){
					$callbackLogAction = 'paymentRefundedGateway';
				}
			}
		} else {
			switch ($data["txn_type"]){
				case "subscr_signup":
					$callbackLogAction = 'subscriptionSignup';
					break;
				case "subscr_cancel":
					$callbackLogAction = 'subscriptionCancelation';
					break;
				case "subscr_failed":
					$callbackLogAction = 'subscriptionFailed';
					break;
				case "new_case":
					$callbackLogAction = 'newDisputeCase';
					$callbackLogActionStatus = 'pending';
					break;
			}
		}

		return array($callbackLogAction, $callbackLogActionStatus);
	}

	/**
	 * function to get the client, billing account, config group, and subscription that this ipn matches up to
	 */
	private function getIPNMapData($data)
	{
		$ret = array(
			"paymentGatewayID" => null,
			"subscriptionID" => 0,
			"clientID" => null,
			"billingAccountID" => 0,
			"invoiceID" => 0,
		);

		if ($this->manualClientID>0){
			dbg("returning a manual client id");
			$ret["clientID"] = $this->manualClientID;
			return $ret;
		}

		if (!(int)$ret["clientID"] && $data["subscr_id"]){
			$h = new QueryMaster("subscriptions.*");
			$h->addColumns("clients.*");
			$h->addJoin("clients", "client_id", "subscriptions");
			$h->addAndClause("subscriptions.subscription_external_id", "=", $data["subscr_id"], "s");
			$this->clientInfo = $this->subscriptionInfo = $h->getRow();
			$ret["subscriptionID"] = $this->subscriptionInfo["subscription_id"];
			$ret["clientID"] = $this->clientInfo["client_id"];
			$ret["paymentGatewayID"] = $this->subscriptionInfo["payment_gateway_id"];
			$ret["billingAccountID"] = $this->subscriptionInfo["billing_account_id"];
			dbg("had a subscription id and tried to get data, got: ");
			dbg($ret);
		}

		if (!(int)$ret["clientID"] && strtoupper(substr($data["item_number"], 0, 3))=="SCR" && $data["subscr_id"] && !(strtoupper($data["item_number"][0])=="I" && strpos(strtoupper($data["item_number"]), "P"))){
			$h = new QueryMaster("subscriptions.*");
			$h->addColumns("clients.*");
			$h->addJoin("clients", "client_id", "subscriptions");
			$h->addAndClause("subscriptions.subscription_id", "=", (int)substr($data["item_number"], 3), "i");
			$this->clientInfo = $this->subscriptionInfo = $h->getRow();
			$ret["subscriptionID"] = $this->subscriptionInfo["subscription_id"];
			$ret["clientID"] = $this->clientInfo["client_id"];
			$ret["paymentGatewayID"] = $this->subscriptionInfo["payment_gateway_id"];
			$ret["billingAccountID"] = $this->subscriptionInfo["billing_account_id"];
			dbg("it looks like a subscription, lets check into it: ");
			dbg($ret);
		}

		if (!(int)$ret["clientID"] && (int)$data["item_number"] && $data["subscr_id"] && !(strtoupper($data["item_number"][0])=="I" && strpos(strtoupper($data["item_number"]), "P"))){
			$h = new QueryMaster("subscriptions.*");
			$h->addColumns("clients.*");
			$h->addJoin("clients", "client_id", "subscriptions");
			$h->addAndClause("subscriptions.subscription_id", "=", (int)$data["item_number"], "i");
			$this->clientInfo = $this->subscriptionInfo = $h->getRow();
			$ret["subscriptionID"] = $this->subscriptionInfo["subscription_id"];
			$ret["clientID"] = $this->clientInfo["client_id"];
			$ret["paymentGatewayID"] = $this->subscriptionInfo["payment_gateway_id"];
			$ret["billingAccountID"] = $this->subscriptionInfo["billing_account_id"];
			dbg("it still looks like a subscription, so:");
			dbg($ret);
		}

		if (strtoupper($data["item_number"][0])=="I" && strpos(strtoupper($data["item_number"]), "P")){
			$split = explode("P", strtoupper($data["item_number"]));
			$ret["invoiceID"] = (int)substr($split[0], 1);
			$ret["paymentGatewayID"] = (int)$split[1];
			$h = new QueryMaster("clients.*");
			$h->addColumns("client_contacts.*");
			$h->addTables("packages");
			$h->addTables("invoice_line_items");
			$h->addJoin("packages", "package_id", "invoice_line_items");
			$h->addJoin("clients", "client_id", "packages");
			$h->addAndClause("invoice_line_items.invoice_id", "=", $ret["invoiceID"], "i");
			$h->addAndClause("client_contacts.contact_primary", "=", 1, "i");
			$h->addGroupByTables("clients", "client_contacts");
			$this->clientInfo = $h->getRow();
			$ret["clientID"] = $this->clientInfo["client_id"];
			dbg("looks like a v5 ipn with a formatted item_number:");
			dbg($ret);
		}

		if (!(int)$ret["clientID"] && $data["payer_email"]){
			$h = new QueryMaster("clients.*");
			$h->addColumns("client_contacts.*");
			$h->addJoin("clients", "client_id", "client_contacts");
			$h->addAndClause("client_contacts.contact_email", "=", $data["payer_email"], "s");
			$h->addOrderByColumns("client_contacts.contact_primary", "DESC");
			$this->clientInfo = $h->getRow();
			$ret["clientID"] = $this->clientInfo["client_id"];
			dbg("trying to track down via payer email:");
			dbg($ret);
		}

		if ( !(int)$ret["clientID"] && 1 == $this->params["enableNameBasedMapping"][0] && $data["last_name"] && $data["first_name"] ) {
			$h = new QueryMaster("clients.*");
			$h->addColumns("client_contacts.*");
			$h->addJoin("clients", "client_id", "client_contacts");
			$h->addAndClause("client_contacts.contact_first_name", "=", $data["first_name"], "s");
			$h->addAndClause("client_contacts.contact_last_name", "=", $data["last_name"], "s");
			$h->addOrderByColumns("client_contacts.contact_primary", "DESC");
			$this->clientInfo = $h->getRow();
			$ret["clientID"] = $this->clientInfo["client_id"];
			dbg("tried to look up based on last name:");
			dbg($ret);
		}

		if ( ((int)$this->createClientOnDemand || 1 == $this->params["enableClientAutoCreation"][0]) && !(int)$ret["clientID"] ) {
			dbg("creating client on demand");
			$ret["clientID"] = $this->doCreateOnDemandClient($data);
 		}

		dbg($data);
		return $ret;
	}

	private function doCreateOnDemandClient($data)
	{
		$clientID = insertFor("clients", array(
			"companyID" => 1,
			"clientCompany" => $data["payer_business_name"],
			"clientStatus" => getAPIStatusID("client", "pending"),
			"applyTax" => 1,
			"themeID" => 1,
			"countriesID" => $countriesID = $this->getCountriesID($data["residence_country"]),
			"clientActive" => 1,
			"clientDateCreated" => time(),
		));
		$contactID = insertFor("contacts", array(
			"clientID" => $clientID,
			"contactUsername" => $data["payer_email"],
			"contactPassword" => getPasswordHash(uniqid("")),
			"contactFirstName" => $data["first_name"],
			"contactLastName" => $data["last_name"],
			"contactEmail" => $data["payer_email"],
			"contactEmailFormat" => 1,
			"contactAddress1" => $data["address_street"],
			"contactCity" => $data["address_city"],
			"contactState" => $data["address_state"],
			"contactZip" => $data["address_zip"],
			"languageID" => 21,
			"countriesID" => $countriesID,
			"contactPrimary" => 1,
			"contactActive" => 1,
			"contactDateCreated" => time(),
		));

		dbg($contactID);
		$q = new MBQuery("GetEmailTemplateGroups", "");
		$r =& $q->getResult();
		$s =& $r->getSubjectFrom(array("mbapi", "results", "emailTplGroups", "emailTplGroup"));
		$t =& $s->getTable();
		$t->setKey("emailActionID");
		$emailActionIDs = $t->getKeys();

		foreach($emailActionIDs as $emailActionID) {
			insertFor("contactEmailActionRels", array(
				"clientContactID" => $contactID,
				"emailActionID" => $emailActionID
			));
		}

		return $clientID;
	}

	private function getCountriesID($iso2)
	{
		$h = new QueryMaster("countries.countries_id");
		$h->addAndClause("countries_iso_2", "=", $iso2, "s");
		return $h->getOne();
	}

	private function addPaypalFee($clientID, $invoiceID, $feeAmount, $feeCurrency)
	{
		if ((float)$feeAmount>0){
			$entries = array();
			$entries[] = array(
				"clientID" => $clientID,
				"invoiceID" => $invoiceID,
				"generalLedgerType" => 1,
				"ledgerAccountID" => getGeneralLedgerAccountID("paypal_fees"),
				"generalLedgerDescription" => TRANS_PAYPAL_FEES,
				"generalLedgerDebitAmount" => doubleval($feeAmount),
				"generalLedgerCreditAmount" => 0,
				"currencyID" => $feeCurrency,
			);
			$entries[] = array(
				"clientID" => $clientID,
				"invoiceID" => $invoiceID,
				"generalLedgerType" => 1,
				"ledgerAccountID" => getGeneralLedgerAccountID("cash"),
				"generalLedgerDescription" => TRANS_PAYPAL_FEES,
				"generalLedgerDebitAmount" => 0,
				"generalLedgerCreditAmount" => doubleval($feeAmount),
				"currencyID" => $feeCurrency,
			);
			require_once mb_path("/lib-billing/include/accounting/General.php");
			$gen = new General();
			$gen->setClientID($clientID);
			$gen->setInvoiceID($invoiceID);
			$gen->setDescription(TRANS_PAYPAL_FEE);
			$gen->setTotalDebits(doubleval($feeAmount));
			$gen->setTotalCredits(doubleval($feeAmount));
			$gen->setCurrencyID($feeCurrency);
			$gen->setLedgerEntries($entries);
			$gen->execute();
			dbg("made fee entries");
		}
	}

	private function authenticate($paymentGatewayID, $IPNData)
	{
		$paypalIDs = $this->getPayPalIDs($paymentGatewayID);
		if ( strtolower($IPNData["receiver_email"]) != strtolower($this->subscriptionInfo["subscription_account_identifier"]) &&
			strtolower($IPNData["business"]) != strtolower($this->subscriptionInfo["subscription_account_identifier"]) &&
			false === strpos($paypalIDs, strtolower($IPNData["receiver_email"])) &&
			false === strpos($paypalIDs, strtolower($IPNData["business"]))
		) {
			dbg("current paypal id: ".$paypalIDs);
			dbg("subscription stuff: ".$this->subscriptionInfo["subscription_account_identifier"]);
			dbg("trying to match these to it: ".$IPNData["receiver_email"]." -- ".$IPNData["business"]);
			return false;
		}
		return true;
	}

	private function getPayPalIDs($paymentGatewayID)
	{
		$paypalIDs = "";
		require_once("objects/PaymentGateway.php");
		$third_party_gateways_filter = $paymentGatewayID ? array("payment_gateway_id" => $paymentGatewayID) :
			array("payment_gateway_type" => PaymentGateway::TYPE_THIRD_PARTY, "payment_gateway_module" => $this->stringRep);
		foreach ( Factory()->PaymentGatewayList($third_party_gateways_filter) as $third_party_gateway ) {
			if ( isset($third_party_gateway->PaymentGatewayParams["PayPal_ID"]) ) {
				$paypalIDs .= " {$third_party_gateway->PaymentGatewayParams['PayPal_ID']->value}";
			}
			if ( isset($third_party_gateway->PaymentGatewayParams["Acceptable_PayPal_Payees"]) ) {
				$paypalIDs .= " {$third_party_gateway->PaymentGatewayParams['Acceptable_PayPal_Payees']->value}";
			}
		}
		return strtolower($paypalIDs);
	}

	private function setURLs($testing)
	{
		$this->IPNVerifyURL = $this->subscriptionURL = $this->paymentURL = (1 == $testing) ?
			"https://www.sandbox.paypal.com/cgi-bin/webscr" : "https://www.paypal.com/cgi-bin/webscr";
	}

	private $IPNVerifyURL;
	private $subscriptionURL;
	private $paymentURL;
	private $callbackLogID;
}
