<?php

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";

require_once mb_path("/lib-mbapi/include/modules/gateway/gateway.php");

/**
 * This class is an instantion of the Gateway class, that executes a command pertaining
 * to comunicating with a payment gateway, in the default case from the database. Pass in
 * only the array parameters that are set.
 *
 * <gateway>
 *  <planetpayment>
 *     <params>
 *       <configGroupName></configGroupName>						(Required: Configuration instance of PlanetPayment module)
 *       <subCommand>charge</subCommand>       						(Required: One of charge, auth, finalize, refund)
 *       <invoiceID></invoiceID>
 *       <authCode></authCode>										(Required for finalize)
 *       <transactionCode></transactionCode>						(Required for refund)
 *       <paymentDescription></paymentDescription>					(Required)
 *       <paymentAmount></paymentAmount>							(Required: 0.00)
 *       <paymentMethod></paymentMethod>							(Required: CC, echeck)
 *       <isRecurring></isRecurring>								(0,1)
 *       <billingAccountID></billingAccountID>			(Enter either this or billing info below)
 *     </params>
 *   </planetpayment>
 * </gateway>
 *
 * @package modernbill
 * @subpackage mbapi
 */

$className = "PlanetPayment";
$GLOBALS["moduleInfo"][$className] = array
(
    "type" => "gateway",
    "status" => PaymentGateway::STATUS_TESTING,
    "version" => "1.0.0",
    "gatewaytype" => "1",
    "displayName" => "PlanetPayment",
    "author" => "Parallels",
    "capabilities" => array(GATEWAY_CHARGE, GATEWAY_AUTHORIZE, GATEWAY_PRIOR_AUTH_CHARGE, GATEWAY_REFUND, GATEWAY_IPV6_COMPLIANCE),
    "countryCodes" => array("US", "CA"),
);

class PlanetPayment extends Gateway {

	/**
	 * Returns the module display for the PlanetPayment module. This includes version number and display name.
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the display info for the module.
	 */
    function getModuleInfo() {
        return $GLOBALS["moduleInfo"]["PlanetPayment"];
    }

	/**
	 * Execute the command based upon the given information, and return the result.
	 *
	 * @return  The result of the PlanetPayment gateway command based upon the given information.
	 */
	 function PlanetPayment($params){
		$this->stringRep = "planetpayment";
	 	parent::Gateway($params);

        $moduleInfo = $this->getModuleInfo();

        foreach ($moduleInfo["capabilities"] as $cap){
            $this->capabilities[$cap] = 1;
        }
		$this->version = "1.0.0";
		$this->liveURL = "https://secure.planetpayment.com/gateway/transact.dll";
		$this->testURL = "https://secure.planetpayment.com/gateway/transact.dll";

	 }

	/**
	 * Returns the default configuration parameters for the PlanetPayment module. The number of elements in each array must me identical to
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the parameters the module uses.
	 */
	function addDefaultConfigParams() {
		static $defaultConfigParams = array(
			array(NULL, NULL, "Login_ID", "testdrive", "", "TRANS_PLANETPAYMENT_LOGINID", "TRANS_PLANETPAYMENT_LOGINID_DESC", "t", 1, 1, 0, NULL),
			array(NULL, NULL, "Password", "testdrive", "", "TRANS_PLANETPAYMENT_PASSWORD", "TRANS_PLANETPAYMENT_PASSWORD_DESC", "t", 1, 2, 0, NULL),
			array(NULL, NULL, "Version", "3.1", "", "TRANS_PLANETPAYMENT_VERSION","TRANS_PLANETPAYMENT_VERSION_DESC", "t", 1, 3, 0, NULL),
			array(NULL, NULL, "Generic_Emails", "0", "", "TRANS_GATEWAY_GEN_EMAIL","TRANS_GATEWAY_GEN_EMAIL_DESC", "r", 0, 4, 0, NULL),
			array(NULL, NULL, "Batch_Recurring", "1", "", "TRANS_BATCH_RECURRING","TRANS_BATCH_RECURRING_DESC", "r", 1, 5, 0, NULL),
			array(NULL, NULL, "Testing", "1", "", "TRANS_GATEWAY_TESTING", "TRANS_GATEWAY_TESTING_DESC", "r", 1, 6, 0, NULL),
			array(NULL, NULL, "Invoice_Description", "", "", "TRANS_INVOICE_DESCRIPTION", "TRANS_INVOICE_DESCRIPTION_DESC", "a", 0, 7, 0, NULL),
            array(NULL, NULL, "card_accept_0", "1", "", "TRANS_ACCEPT_VISA", "TRANS_ACCEPT_VISA_DESC", "b", 1, 8, 0, NULL),
            array(NULL, NULL, "card_accept_1", "1", "", "TRANS_ACCEPT_MASTERCARD", "TRANS_ACCEPT_MASTERCARD_DESC", "b", 1, 9, 0, NULL),
			);
		return $defaultConfigParams;
	}

	 /**
	  *
	  * From a child module this function must do the following:
	  *   - Maintain $this->hasExecuted, $this->hasExecutedSuccessfully, $this->timestamp,
	  *     $this->status, $this->accountID
	  *   - Add any queries to the debug list with addDebug()
	  *   - Add any errors to the error list with addError()
	  *   - Return $this->returnData, or the appropriate error constant
	  *
	  * @return  Returns feedback from execution of this Gateway object, or an error constant defined
	  *          at the top of this file.
	  */
	function execute(){
		return $this->runAuthNETProtocol();
	} //execute

} // PlanetPayment

/* Debug
$debug = "
  <gateway>
    <planetpayment>
      <params>
        <configGroupName>0</configGroupName>
        <subCommand>charge</subCommand>
        <invoiceID>135</invoiceID>
        <authCode></authCode>
        <transactionCode></transactionCode>
        <paymentDescription>ModernBill Test Service</paymentDescription>
        <paymentAmount>1.54</paymentAmount>
        <paymentMethod>CC</paymentMethod>
        <isRecurring>1</isRecurring>
        <billingAccountID></billingAccountID>
      </params>
    </planetpayment>
  </gateway>
";
$payment = new PlanetPayment($debug);
$CONF["debug"]->add("MBAPI Result", htmlspecialchars($payment->getData()));
echo "<pre>".$CONF['debug']->show()."</pre>";
/* */
