<?php

require_once(dirname(__FILE__)."/../../../../lib-tk/shield.php");
require_once("lib-mbapi/include/modules/gateway/gateway.php");
require_once("lib-tk/include/creditcard/validate.php");

$className = "Protx";
$GLOBALS["moduleInfo"][$className] = array (
    "type" => "gateway",
    "status" => PaymentGateway::STATUS_STABLE,
    "version" => "1.2.0",
    "gatewaytype" => "1",
    "displayName" => "SagePay",
    "author" => "Parallels",
    "capabilities" => array(GATEWAY_CHARGE, GATEWAY_IPV6_COMPLIANCE), //, GATEWAY_AUTHORIZE, GATEWAY_PRIOR_AUTH_CHARGE) //, GATEWAY_REFUND) Does not seem to accept the DirectRefundTx service yet
	'countryCodes' => array("GB"),
);

/****************
 * base classes *
 ****************/

class ProtxBaseTransactionRequest extends TransactionRequest {
	/**
	 * Curl  for the request call
	 */
	public $call;
	/**
	 * TransactionResponse  for the gateway response
	 */
	public $response;

	/**
	 * string  protocol version
	 */
	public $version;

	/**
	 * string  transaction type
	 */

	/**
	 * string  account ID
	 */
	public $account;

	/**
	 * int  unique transaction ID generated by us
	 */
	public $id;

	/**
	 * array  map of URLs
	 */
	public $urls;

	function ProtxBaseTransactionRequest(&$that) {
		$this->TransactionRequest($that);
		$this->version = $this->config["Version"];
		$this->responseClass = "ProtxBaseTransactionResponse";
		$this->type = ""; // to be extended
		$this->account = $this->config["Login"];
		$this->id = sha1(
			  (string) mt_rand(0, 200000000)
			. (string) microtime()
			. (string) mt_rand(0, 200000000)
		);

		$urlMap = array(
			"simulation" => array(
				"direct" => "https://test.sagepay.com/simulator/VSPDirectGateway.asp",
				"server" => "https://test.sagepay.com/Simulator/VSPServerGateway.asp?Service=",
			),
			"test" => array(
				"direct" => "https://test.sagepay.com/gateway/service/vspdirect-register.vsp",
				"server" => "https://test.sagepay.com/gateway/service/",
			),
			"live" => array(
				"direct" => "https://live.sagepay.com/gateway/service/vspdirect-register.vsp",
				"server" => "https://live.sagepay.com/gateway/service/",
			),
		);
		$this->urls = $urlMap[$this->config["Mode"]];

		return true;
	}

	function buildCall() {
		$this->call->setConfigIdentity("Gateway_Protx");

		$this->call->addField("VPSProtocol", $this->version);
		$this->call->addField("TxType", $this->type);
		$this->call->addField("Vendor", $this->account);
		$this->call->addField("VendorTxCode", $this->id);
	}

}

class ProtxBaseTransactionResponse extends TransactionResponse {
	public $response;
	public $values;

	public $version;
	public $status;
	public $message;

	function ProtxBaseTransactionResponse(&$that, $response) {
		if(!$this->TransactionResponse($that, $response)) return false;

		$this->response = $response;
		$this->values = array();
		$assignments = explode("\r\n", $response);
		foreach($assignments as $assignment) {
			if(!$assignment) continue; // just in case of blank line
			list($var, $val) = explode("=", $assignment);
			$this->values[$var] = $val;
		}

		$statusMap = array(
			"OK" => GATEWAY_SUCCESS,
			"MALFORMED" => GATEWAY_FAILURE,
			"INVALID" => GATEWAY_FAILURE,
			"ERROR" => GATEWAY_FAILURE,
			"NOTAUTHED" => GATEWAY_DECLINED,
			"REJECTED" => GATEWAY_DECLINED,
		);
		$this->version = $this->values["VPSProtocol"];
		$this->status = $this->values["Status"];
		$this->gatewayStatus = $statusMap[$this->status];
		$this->message = $this->values["Status"] . ": " . $this->values["StatusDetail"];

		return true;
	}

	function getRejected() {
		return $this->status == "REJECTED";
	}

	function getStatus() {
		return $this->status;
	}

	function getGatewayStatus() {
		return $this->gatewayStatus;
	}

	function getMessage() {
		return $this->message;
	}

	function finish() {
		parent::finish();
	}
}

/****************
 * card classes *
 ****************/

class ProtxCardTransactionRequest extends ProtxBaseTransactionRequest {
	/**
	 * int  amount of transaction in minor currency units
	 */
	public $amount;

	/**
	 * string  ISO country code
	 */
	public $country;

	/**
	 * string  ISO currency code
	 */
	public $currency;

	public $customerFirstName;
	public $customerLastName;

	public $cardName;
	public $cardNum;
	public $terminal;
	public $expiryMonth;
	public $expiryYear;
	public $issueNum;
	public $startMonth;
	public $startYear;

	// cv2 verification
	public $cv2;

	// address verification
	public $address1;
	public $address2;
	public $city;
	public $postCode;
	public $state;

	// client info
	public $email;
	public $phone;

	function ProtxCardTransactionRequest(&$that) {
		if(!parent::ProtxBaseTransactionRequest($that)) return false;

		$this->responseClass = "ProtxAuthCardTransactionResponse";
		$this->amount = number_format($that->params["paymentAmount"][0], 2, ".", "");
		$this->currency = $that->params["currency"][0];
		$this->description = substr($that->params["paymentDescription"][0], 0, 100);
		if(!$this->description) {
			$this->description = "None.";
		}

		$this->customerFirstName = $that->params["contactFirstName"][0];
		$this->customerLastName = $that->params["contactLastName"][0];
		$this->cardName = $that->params["billingCardNameOnCard"][0];
		$type = new TypeOf("card", true);
		$typeMap = array(
			$type->id("visa") => "VISA",
			$type->id("mastercard") => "MC",
			$type->id("delta") => "DELTA",
			$type->id("solo") => "SOLO",
			$type->id("maestro") => "SWITCH",
			$type->id("electron") => "UKE",
			$type->id("amex") => "AMEX",
			$type->id("diners") => "DC",
			$type->id("jcb") => "JCB",
		);
		$this->cardType = $typeMap[$that->params["billingCardType"][0]];
		$this->cardNum = $that->params["billingCardNum"][0];
		$this->expiryYear  = date("y", mktime(0, 0, 0, $that->params['billingCardExpMonth'][0], 1, $that->params['billingCardExpYear'][0]));
		$this->expiryMonth = date("m", mktime(0, 0, 0, $that->params['billingCardExpMonth'][0], 1, $that->expiryYear));
				$this->issueNum = !empty($that->params["billingCardIssueNum"][0]) ? $that->params["billingCardIssueNum"][0] : NULL;

		if(!empty($that->params["billingCardStartDate"][0])) {
			list($startMonth, $startYear) = explode("/", $that->params["billingCardStartDate"][0]);
			$this->startYear  = date("y", mktime(0, 0, 0, $startMonth, 1, $startYear));
			$this->startMonth = date("m", mktime(0, 0, 0, $startMonth, 1, $this->startYear));
		} else {
			$this->startMonth = NULL;
			$this->startYear = NULL;
		}

		if(!empty($that->params['cardCVV2'][0]))
			$this->cv2 = $that->params['cardCVV2'][0];

		$this->address1 = $that->params["contactAddress1"][0];
		$this->address2 = $that->params["contactAddress2"][0];
		$this->city = $that->params["contactCity"][0];
		$this->postCode = $that->params["contactZip"][0];
		$this->country = $that->params["countriesISO2"][0];
		$this->state = $that->params["contactState"][0];

		$this->email = $that->params['contactEmail'][0];
		$this->phone = $that->params['contactPhone1'][0];
		$this->fax = $that->params['contactFax'][0];

		return true;
	}

	function buildCall() {
		parent::buildCall();

		$this->call->setURL($this->urls["direct"]);

		$this->call->addField("Amount", $this->amount);
		$this->call->addField("Currency", $this->currency);
		$this->call->addField("Description", $this->description);

		$this->call->addField("CardHolder", $this->cardName);
		$this->call->addField("CardNumber", $this->cardNum);
		if($this->startYear) $this->call->addField("StartDate", $this->startMonth . $this->startYear);
		$this->call->addField("ExpiryDate", $this->expiryMonth . $this->expiryYear);
		if($this->issueNum) $this->call->addField("IssueNumber", $this->issueNum);
		if($this->cv2) $this->call->addField("CV2", $this->cv2);
		$this->call->addField("CardType", $this->cardType);

		$this->call->addField("BillingSurname", $this->customerLastName);
		$this->call->addField("BillingFirstnames", $this->customerFirstName);
		$this->call->addField("BillingAddress1", $this->address1);
		if ( $this->address2 ) $this->call->addField("BillingAddress2", $this->address2);
		$this->call->addField("BillingCity", $this->city);
		$this->call->addField("BillingPostCode", $this->postCode);
		$this->call->addField("BillingCountry", $this->country);
		if ( "US" == $this->country && $this->state ) $this->call->addField("BillingState", $this->state);

		$this->call->addField("DeliverySurname", $this->customerLastName);
		$this->call->addField("DeliveryFirstnames", $this->customerFirstName);
		$this->call->addField("DeliveryAddress1", $this->address1);
		if ( $this->address2 ) $this->call->addField("DeliveryAddress2", $this->address2);
		$this->call->addField("DeliveryCity", $this->city);
		$this->call->addField("DeliveryPostCode", $this->postCode);
		$this->call->addField("DeliveryCountry", $this->country);
		if ( "US" == $this->country && $this->state ) $this->call->addField("DeliveryState", $this->state);

		if ( $this->email ) $this->call->addField("CustomerEMail", $this->email);
	}
}

class ProtxCardTransactionResponse extends ProtxBaseTransactionResponse {
	public $VPSTxId;
	public $SecurityKey;
	public $TxAuthNo;
	public $AVSCV2;

	function ProtxCardTransactionResponse(&$that, $response) {
		if(!$this->ProtxBaseTransactionResponse($that, $response)) return false;

		$this->VPSTxId = $this->values["VPSTxId"];
		$this->SecurityKey = $this->values["SecurityKey"];
		$this->TxAuthNo = $this->values["TxAuthNo"];

		return true;
	}

	function getTransactionID() {
		return $this->that->id;
	}

	function getAVSCode() {
		return $this->AVSCV2;
	}

	function getAuthorizationCode() {
		return $this->TxAuthNo;
	}

	function finish() {
		parent::finish();
		$this->t->addField("VPSTxID", $this->VPSTxId);
		$this->t->addField("SecurityKey", $this->SecurityKey);
		$this->t->addField("TxAuthNo", $this->TxAuthNo);
	}
}

class ProtxAuthCardTransactionResponse extends ProtxCardTransactionResponse {
	public $AddressResult;
	public $PostCodeResult;
	public $CV2Result;

	function ProtxAuthCardTransactionResponse(&$that, $response) {
		if(!$this->ProtxCardTransactionResponse($that, $response)) return false;

		$this->AVSCV2 = $this->values["AVSCV2"];
		$this->AddressResult = $this->values["AddressResult"];
		$this->PostCodeResult = $this->values["PostCodeResult"];
		$this->CV2Result = $this->values["CV2Result"];

		return true;
	}

	function finish() {
		parent::finish();
		$this->t->addField("AVSCV2", $this->AVSCV2);
		$this->t->addField("AddressResult", $this->AddressResult);
		$this->t->addField("PostCodeResult", $this->PostCodeResult);
		$this->t->addField("CV2Result", $this->CV2Result);
	}

}

/******************
 * Server classes *
 ******************/

class ProtxServerTransactionRequest extends ProtxBaseTransactionRequest {
	public $service;
	function buildCall() {
		parent::buildCall();
		$this->call->setURL($this->urls["server"] . $this->service);
	}
}

class ProtxServerCardTransactionRequest extends ProtxCardTransactionRequest {
	public $service;
	function buildCall() {
		parent::buildCall();
		$this->call->setURL($this->urls["server"] . $this->service);
	}
}

class ProtxServerReferenceTransactionRequest extends ProtxServerTransactionRequest {
	public $t;

	function ProtxServerReferenceTransactionRequest(&$that) {
		if(!parent::ProtxServerTransactionRequest($that)) return false;

		$this->responseClass = "ProtxCardTransactionResponse";
		$this->t = new TransactionRecord($that);
		$this->VPSTxId = $this->t->getField("VPSTxID");
		$this->TxAuthNo = $this->t->getField("TxAuthNo");
		$this->SecurityKey = $this->t->getField("SecurityKey");

		return true;
	}

	function buildCall() {
		parent::buildCall();
		$this->call->addField("VPSTxId", $this->VPSTxId);
		$this->call->addField("TxAuthNo", $this->TxAuthNo);
		$this->call->addField("SecurityKey", $this->SecurityKey);
	}
}

class ProtxServerDerivedTransactionRequest extends ProtxServerReferenceTransactionRequest {
	public $transactionCode;

	function ProtxServerDerivedTransactionRequest(&$that) {
		if(!parent::ProtxServerReferenceTransactionRequest($that)) return false;

		$this->amount = number_format($that->params["paymentAmount"][0], 2, ".", "");
		$this->currency = $that->params["currency"][0];
		$this->description = $that->params["paymentDescription"][0];
		if(!$this->description) {
			$this->description = "None.";
		}

		$this->transactionCode = $this->t->getTransactionCode();

		return true;
	}

	function buildCall() {
		// skip parent. We are overriding the parent
		ProtxServerTransactionRequest::buildCall();

		$this->call->addField("Amount", $this->amount);
		$this->call->addField("Currency", $this->currency);
		$this->call->addField("Description", $this->description);

		$this->call->addField("RelatedVPSTxId", $this->VPSTxId);
		$this->call->addField("RelatedTxAuthNo", $this->TxAuthNo);
		$this->call->addField("RelatedSecurityKey", $this->SecurityKey);

		$this->call->addField("RelatedVendorTxCode", $this->transactionCode);
	}
}

/********************
 * complete classes *
 ********************/

/**
 * Make a PAYMENT
 */
class ProtxPaymentTransactionRequest extends ProtxCardTransactionRequest {
	function ProtxPaymentTransactionRequest(&$that) {
		if(!parent::ProtxCardTransactionRequest($that)) return false;
		$this->type = "PAYMENT";
		return true;
	}
}

/**
 * Make a DEFERRED payment
 */
class ProtxDeferredTransactionRequest extends ProtxCardTransactionRequest {
	function ProtxDeferredTransactionRequest(&$that) {
		if(!parent::ProtxCardTransactionRequest($that)) return false;
		$this->type = "DEFERRED";
		return true;
	}
}

/**
 * PREAUTH a payment
 */
class ProtxPreauthTransactionRequest extends ProtxCardTransactionRequest {
	function ProtxPreauthTransactionRequest(&$that) {
		if(!parent::ProtxCardTransactionRequest($that)) return false;
		$this->type = "PREAUTH";
		return true;
	}
}

/**
 * PREAUTH a payment, but for CVV2 referencing.
 */
class ProtxReauthTransactionRequest extends ProtxPreauthTransactionRequest {
	function ProtxReauthTransactionRequest(&$that) {
		if(!parent::ProtxPreauthTransactionRequest($that)) return false;
		$this->reauth = true;
		return true;
	}
}

/**
 * RELEASE a DEFERRED or REPEATDEFERRED payment
 */
class ProtxReleaseTransactionRequest extends ProtxServerReferenceTransactionRequest {
	function ProtxReleaseTransactionRequest(&$that) {
		if(!parent::ProtxServerReferenceTransactionRequest($that)) return false;
		$this->type = "RELEASE";
		$this->service = "simulation" == $this->config["Mode"] ? "VendorReleaseTx" : "release.vsp";
		return true;
	}
}

/**
 * ABORT a DEFERRED payment
 */
class ProtxAbortTransactionRequest extends ProtxServerReferenceTransactionRequest {
	function ProtxAbortTransactionRequest(&$that) {
		if(!parent::ProtxServerReferenceTransactionRequest($that)) return false;
		$this->type = "ABORT";
		$this->service = "simulation" == $this->config["Mode"] ? "VendorAbortTx" : "abort.vsp";
		return true;
	}
}

/**
 * REFUND a payment
 */
class ProtxRefundTransactionRequest extends ProtxServerDerivedTransactionRequest {
	function ProtxRefundTransactionRequest(&$that) {
		if(!parent::ProtxServerDerivedTransactionRequest($that)) return false;
		$this->type = "REFUND";
		$this->service = "simulation" == $this->config["Mode"] ? "VendorRefundTx" : "refund.vsp";
		return true;
	}
}

/**
 * REPEAT a payment
 */
class ProtxRepeatTransactionRequest extends ProtxServerDerivedTransactionRequest {
	function ProtxRepeatTransactionRequest(&$that) {
		if(!parent::ProtxServerDerivedTransactionRequest($that)) return false;
		$this->type = "REPEAT";
		$this->service = "simulation" == $this->config["Mode"] ? "VendorRepeatTx" : "repeat.vsp";
		return true;
	}
}

/**
 * REPEATDEFERRED
 */
class ProtxRepeatDeferredTransactionRequest extends ProtxServerDerivedTransactionRequest {
	function ProtxRepeatDeferredTransactionRequest(&$that) {
		if(!parent::ProtxServerDerivedTransactionRequest($that)) return false;
		$this->type = "REPEATDEFERRED";
		$this->service = "simulation" == $this->config["Mode"] ? "VendorRepeatTx" : "repeat.vsp";
		return true;
	}
}

/**
 * VOID a transaction
 */
class ProtxVoidTransactionRequest extends ProtxServerReferenceTransactionRequest {
	function ProtxVoidTransactionRequest(&$that) {
		if(!parent::ProtxServerReferenceTransactionRequest($that)) return false;
		$this->type = "VOID";
		$this->service = "simulation" == $this->config["Mode"] ? "VendorVoidTx" : "void.vsp";
		return true;
	}
}

/**
 * MANUAL payment
 */
class ProtxManualTransactionRequest extends ProtxServerCardTransactionRequest {
	function ProtxManualTransactionRequest(&$that) {
		if(!parent::ProtxServerCardTransactionRequest($that)) return false;
		$this->type = "MANUAL";
		$this->service = "simulation" == $this->config["Mode"] ? "VendorManualTx" : "manualpayment.vsp";
		return true;
	}
}

/**
 * DIRECTREFUND
 */
class ProtxDirectRefundTransactionRequest extends ProtxServerCardTransactionRequest {
	function ProtxDirectRefundTransactionRequest(&$that) {
		if(!parent::ProtxServerCardTransactionRequest($that)) return false;
		$this->type = "DIRECTREFUND";
		$this->service = "simulation" == $this->config["Mode"] ? "DirectRefundTx" : "directrefund.vsp";
		return true;
	}
}

/**************
 * Main class *
 **************/
class Protx extends Gateway
{
	public function getModuleInfo()
	{
		return $GLOBALS["moduleInfo"]["Protx"];
	}

	public function addDefaultConfigParams()
	{
		static $defaultConfigParams = array(
			array(NULL, NULL, "Login", "", "", "TRANS_PROTX_LOGINID", "TRANS_PROTX_LOGINID_DESC", "t", 1, 1, 0, NULL),
			array(NULL, NULL, "Version", "2.23", "", "TRANS_PROTX_VERSION","TRANS_PROTX_VERSION_DESC", "t", 1, 1, 0, NULL),
            array(NULL, NULL, "Mode", "simulation", "", "TRANS_GATEWAY_MODE", "TRANS_GATEWAY_MODE_DESC", "s", 1, 2, 0, array
                    (
                        array(NULL, NULL, "simulation", "TRANS_SIMULATION", 1),
                        array(NULL, NULL, "test", "TRANS_TEST", 2),
                        array(NULL, NULL, "live", "TRANS_LIVE", 3),
                    )),
			array(NULL, NULL, "Refund_Description", "", "", "TRANS_REFUND_DESCRIPTION", "TRANS_REFUND_DESCRIPTION_DESC", "a", 0, 5, 0, NULL),
			array(NULL, NULL, "Invoice_Description", "", "", "TRANS_INVOICE_DESCRIPTION", "TRANS_INVOICE_DESCRIPTION_DESC", "a", 0, 6, 0, NULL),
            array(NULL, NULL, "card_accept_0", "1", "", "TRANS_ACCEPT_VISA", "TRANS_ACCEPT_VISA_DESC", "b", 1, 8, 0, NULL),
            array(NULL, NULL, "card_accept_1", "1", "", "TRANS_ACCEPT_MASTERCARD", "TRANS_ACCEPT_MASTERCARD_DESC", "b", 1, 9, 0, NULL),
            array(NULL, NULL, "card_accept_2", "1", "", "TRANS_ACCEPT_AMEX", "TRANS_ACCEPT_AMEX_DESC", "b", 1, 10, 0, NULL),
            array(NULL, NULL, "card_accept_3", "1", "", "TRANS_ACCEPT_DISCOVER", "TRANS_ACCEPT_DISCOVER_DESC", "b", 1, 11, 0, NULL),
            array(NULL, NULL, "card_accept_4", "1", "", "TRANS_ACCEPT_JCB", "TRANS_ACCEPT_JCB_DESC", "b", 1, 12, 0, NULL),
            array(NULL, NULL, "card_accept_6", "1", "", "TRANS_ACCEPT_DINERS", "TRANS_ACCEPT_DINERS_DESC", "b", 1, 14, 0, NULL),
            array(NULL, NULL, "card_accept_7", "1", "", "TRANS_ACCEPT_SOLO", "TRANS_ACCEPT_SOLO_DESC", "b", 1, 15, 0, NULL),
            array(NULL, NULL, "card_accept_8", "1", "", "TRANS_ACCEPT_MAESTRO", "TRANS_ACCEPT_MAESTRO_DESC", "b", 1, 16, 0, NULL),
            array(NULL, NULL, "card_accept_9", "1", "", "TRANS_ACCEPT_DELTA", "TRANS_ACCEPT_DELTA_DESC", "b", 1, 17, 0, NULL),
            array(NULL, NULL, "card_accept_11", "1", "", "TRANS_ACCEPT_ELECTRON", "TRANS_ACCEPT_ELECTRON_DESC", "b", 1, 19, 0, NULL),
			);
		return $defaultConfigParams;
	}

	public function execute()
	{
		$this->reauthRef = NULL;
		$subcommand = $this->params["subCommand"][0];
		switch ( $subcommand ) {
			case "charge":
				if($this->params["cardCVV2"][0]) {
					$this->requestClass = "ProtxPaymentTransactionRequest";
				} else {
					$t = new TransactionRecord($this);
					if($this->reauthRef = $t->getRefForReauth()) {
						$this->requestClass = "ProtxRepeatTransactionRequest";
					} else {
						$this->requestClass = "ProtxPaymentTransactionRequest";
					}
				}
				break;
			case "reauth":
				$this->requestClass = "ProtxReauthTransactionRequest";
				break;
			/*
			case "auth":
				if($this->params["cardCVV2"][0]) {
					$this->requestClass = "ProtxPreauthTransactionRequest";
				} else {
					$t = new TransactionRecord($this);
					if($this->reauthRef = $t->getRefForReauth()) {
						$this->requestClass = "ProtxRepeatTransactionRequest";
					} else {
						$this->requestClass = "ProtxPreauthTransactionRequest";
					}
				}
				break;
			case "finalize":
				$this->requestClass = "ProtxRepeatTransactionRequest";
				break;
			case "refund":
				$this->requestClass = "ProtxDirectRefundTransactionRequest";
				break;
			case "void":
				$this->requestClass = "ProtxVoidTransactionRequest";
				break;
				*/
			default:
				return $this->createOutputXML(array(
					"status" => GATEWAY_ERROR,
					"message" => "Invalid Gateway Action Requested: $subcommand",
				));
		}
		return parent::execute();
	 }
}
