<?php

require_once dirname(__FILE__).'/../../../../lib-tk/shield.php';
require_once mb_path('/lib-mbapi/include/modules/gateway/gateway.php');

$className = 'PSiGate';

$GLOBALS['moduleInfo'][$className] = array(
    'type' => 'gateway',
    'status' => PaymentGateway::STATUS_STABLE,
    'version' => '2.0.0',
    'gatewaytype' => '1',
    'displayName' => $className,
    'author' => 'Parallels',
    'capabilities' => array(
    	GATEWAY_CHARGE,
		GATEWAY_AUTHORIZE,
		GATEWAY_PRIOR_AUTH_CHARGE,
		GATEWAY_REFUND,
		GATEWAY_VOID,
		GATEWAY_IPV6_COMPLIANCE,
	),
	"countryCodes" => array("US", "CA"),
);

class PSiGate extends Gateway
{
	public function getModuleInfo()
	{
		return $GLOBALS['moduleInfo']['PSiGate'];
	}

	public function __construct($input)
	{
		parent::Gateway($input);
		$this->liveURL = 'https://secure.psigate.com:7934/Messenger/XMLMessenger';
		$this->testURL = 'https://dev.psigate.com:7989/Messenger/XMLMessenger';
	}

	public function addDefaultConfigParams()
	{
		/*
		 * [0] = option name
		 * [1] = option type (t = text, p = password, r = radio)
		 * [2] = default value
		 * [3] = option label
		 * [4] = option description
		 */
		$adminOptions = array(
			array('storeID',			't',		NULL,		'PSIGATE_STOREID',		'PSIGATE_STOREID_DESC'		),
			array('passphrase',			't',		NULL,		'PSIGATE_PASSPHRASE',	'PSIGATE_PASSPHRASE_DESC'	),
			array('testMode',			'b',		'0',		'PSIGATE_TEST_MODE',	'PSIGATE_TEST_MODE_DESC'	),
			array('card_accept_0',		'b', 		'0',		'ACCEPT_VISA',			'ACCEPT_VISA_DESC'			),
			array('card_accept_1',		'b',		'0',		'ACCEPT_MASTERCARD',	'ACCEPT_MASTERCARD_DESC'	),
			array('card_accept_2',		'b',		'0',		'ACCEPT_AMEX',			'ACCEPT_AMEX_DESC'			),
			array('card_accept_3',		'b',		'0'	,		'ACCEPT_DISCOVER',		'ACCEPT_DISCOVER_DESC'		),
			array('card_accept_4',		'b',		'0',		'ACCEPT_JCB',			'ACCEPT_JCB_DESC'			),
			array('card_accept_5',		'b',		'0'	,		'ACCEPT_ENROUTE',		'ACCEPT_ENROUTE_DESC'		),
			array('card_accept_6',		'b',		'0',		'ACCEPT_DINERS',		'ACCEPT_DINERS_DESC'		),
			array('card_accept_7',		'b',		'0',		'ACCEPT_SOLO',			'ACCEPT_SOLO_DESC'			),
			array('card_accept_8',		'b',		'0',		'ACCEPT_MAESTRO',		'ACCEPT_MAESTRO_DESC'		),
			array('card_accept_9',		'b',		'0',		'ACCEPT_DELTA',			'ACCEPT_DELTA_DESC'			),
			array('card_accept_10',		'b',		'0',		'ACCEPT_BANKCARD',		'ACCEPT_BANKCARD_DESC'		),
			array('card_accept_11',		'b',		'0',		'ACCEPT_ELECTRON',		'ACCEPT_ELECTRON_DESC'		)
		);

		// this is a a much easier way to generate the proper config options for display
		// you should never have to change the below, just modify the params in the array above
		$i = 1;
		foreach($adminOptions as $adminOption) {
			$defaultConfigParams[] = array(
				NULL,NULL,$adminOption[0],$adminOption[2],'','TRANS_'.$adminOption[3],'TRANS_'.$adminOption[4],$adminOption[1],0,$i++,0,NULL
			);
		}

		// Grab the configs set at the registrar level
		return $defaultConfigParams;
	}

	public function execute()
	{
		// populate $this->params with collapsed array
		// and set the proper URL to hit based on test mode
		$this->params = correctArray($this->params);
		$this->submitURL = ($this->config['testMode']) ? $this->testURL : $this->liveURL;

		if (isset($this->params['transactionCode']) && trim($this->params['transactionCode']) != '')
			$tempTransactionCodeArray = explode(':',$this->params['transactionCode']);

		$this->params['orderID'] = (isset($tempTransactionCodeArray[0]) && trim($tempTransactionCodeArray[0]) != '') ? $tempTransactionCodeArray[0] : '';
		$this->params['transRefNumber'] = (isset($tempTransactionCodeArray[1]) && trim($tempTransactionCodeArray[1]) != '') ? $tempTransactionCodeArray[1] : '';

		switch($this->params['subCommand']) {
			case 'charge':
				$cardAction = '0';
				$subTotal = $this->params['paymentAmount'];
			break;
			case 'auth':
				$cardAction = '1';
				$subTotal = $this->params['paymentAmount'];
			break;
			case 'finalize':
				$cardAction = '2';
				$orderID = $this->params['orderID'];
                $this->params['billingCardNum'] = '';
			break;
			case 'refund':
				$cardAction = '3';
				$subTotal = $this->params['paymentAmount'];
				$orderID =$this->params['orderID'];
                $this->params['billingCardNum'] = '';
			break;
			case 'void':
				$cardAction = '9';
				$transRefNumber = $this->params['transRefNumber'];
				$orderID = $this->params['orderID'];
                $this->params['billingCardNum'] = '';
			break;

			// just in case something funky is going on
			default:
				return $this->createOutputXML(array(
					"status" =>GATEWAY_ERROR,
					"message" => 'Invalid Gateway Action Requested',
					)
				);
		}

		$tempPostValues = array(
			// basic authentication that needs to be a part of any hit on PSiGate's API
			'StoreID'			=> 		$this->config['storeID'],
			'Passphrase'		=>		$this->config['passphrase'],

			// required postfields for all transactions
			'PaymentType'		=> 		'CC',
			'CardNumber'		=>		$this->params['billingCardNum'],
			'CardExpMonth'		=>		str_pad($this->params['billingCardExpMonth'],2,'0',STR_PAD_LEFT),
			'CardExpYear'		=>		substr($this->params['billingCardExpYear'],2,2),
			'CardAction'		=> 		$cardAction,

			// required postfields for some transaction types
			'OrderID'			=>		(isset($orderID) && trim($orderID) != '') ? $orderID : '',
			'TransRefNumber'	=>		(isset($transRefNumber) && trim($transRefNumber) != '') ? $transRefNumber : '',
			'Subtotal'			=>		(isset($subTotal) && trim($subTotal) != '') ? $subTotal : '',

			// optional postvalues that do can magic things like lower merchant rates
			// please note: magic things might also include appearance of unicorns and rainbows
			'Bname'				=>		billing_get_full_name($this->params['contactFirstName'], $this->params['contactLastName']),
			'Bcompany'			=>		(isset($this->params['clientCompany']) && trim($this->params['clientCompany']) != '') ? $this->params['clientCompany'] : '',
			'Baddress1'			=>		(isset($this->params['contactAddress1']) && trim($this->params['contactAddress1']) != '') ? $this->params['contactAddress1'] : '',
			'Baddress2'			=>		(isset($this->params['contactAddress2']) && trim($this->params['contactAddress2']) != '') ? $this->params['contactAddress2'] : '',
			'Bcity'				=>		(isset($this->params['contactCity']) && trim($this->params['contactCity']) != '') ? $this->params['contactCity'] : '',
			'Bprovince'			=>		(isset($this->params['contactState']) && trim($this->params['contactState']) != '') ? $this->params['contactState'] : '',
			'Bpostalcode'		=>		(isset($this->params['contactZip']) && trim($this->params['contactZip']) != '' ) ? $this->params['contactZip'] : '',
			'Bcountry'			=>		(isset($this->params["countriesISO2"][0]) && trim($this->params["countriesISO2"][0]) != '') ? $this->params["countriesISO2"][0] : '',
			'Phone'				=>		(isset($this->params['contactPhone1']) && trim($this->params['contactPhone1']) != '') ? $this->params['contactPhone1'] : '',
			'Fax'				=>		(isset($this->params['contactFax']) && trim($this->params['contactFax']) != '') ? $this->params['contactFax'] : '',
			'Email'				=>		(isset($this->params['contactEmail']) && trim($this->params['contactEmail']) != '') ? $this->params['contactEmail'] : '',
			'CardIDCode'		=>		(isset($this->params['cardCVV2']) && trim($this->params['cardCVV2']) != '') ? 1 : '',
			'CardIDNumber'		=>		(isset($this->params['cardCVV2']) && trim($this->params['cardCVV2']) != '') ? $this->params['cardCVV2'] : ''
		);

		// generate final list of postvalues to go through to PSiGate's API
		// PSiGate requires that no ampersands be present in the submitted data
		$postValues = array();
		foreach($tempPostValues as $postElement => $postValue) {
			if (isset($postValue) && trim($postValue) != '')
				$postValues[$postElement] = str_replace('&','and',$postValue);
		}

		// wrap the array with an "order" array for toXML processing
		$postValues = array(
			'Order' => $postValues
		);


		// free up memory, unset unneeded variables
		unset($tempTransactionCodeArray);
		unset($tempPostValues);
		unset($cardAction);
		unset($transRefNumber);
		unset($subTotal);


		// make the call...
		$curlCall = $this->getCurlObject($this->submitURL);
		$curlCall->addField('', toXML($postValues), 0);

		// store the API response into a string
		$curlResult = $curlCall->toString();

		if (!$curlResult) {
			$this->status = GATEWAY_CONNECTION_FAILURE;
		} else {

			// parse the response xml into an array
			$x = new MBXMLLite($curlResult);
			$curlResultArray = correctArray($x->toArray());

			// set more return data based on API response
			switch($curlResultArray['Result'][0]['Approved']) {
				case 'APPROVED':
					$this->status = GATEWAY_SUCCESS;
					$this->message = $curlResultArray['Result'][0]['ReturnCode'];
					$this->transactionID = $curlResultArray['Result'][0]['OrderID'].':'.$curlResultArray['Result'][0]['TransRefNumber'];
					$this->authorizationCode = $curlResultArray['Result'][0]['CardAuthNumber'];
				break;

				case 'DECLINED':
					$this->status = GATEWAY_DECLINED;
					$this->message = $curlResultArray['Result'][0]['ErrMsg'];
				break;

				case 'ERROR':
					$this->status = GATEWAY_FAILURE;
					$this->message = $curlResultArray['Result'][0]['ErrMsg'];
				break;

				default:
					$this->status = GATEWAY_FAILURE;
					$this->message = $curlResultArray['Result'][0]['ErrMsg'];
				break;
			}

		}

		// return necessary information back to ModernBill
		$this->hasExecuted = 1;
		return $this->createOutputXML(array(
			'status' => $this->status,
			'statusName' => getAPIStatusName('transactions', $this->status),
			'statusMessage' => getStatusName('transactions', $this->status),
			'authorizationCode' => $this->authorizationCode,
			'transactionID' => $this->transactionID,
			'message' => $this->message,
			)
		);
	}
}