<?php

require_once("lib-mbapi/include/modules/gateway/gateway.php");

$className = "SecurePay";
$GLOBALS["moduleInfo"][$className] = array(
	"type" => "gateway",
	"status" => PaymentGateway::STATUS_TESTING,
	"version" => "1.0.0",
	"gatewaytype" => "1",
	"displayName" => $className,
	"author" => "Parallels",
	"capabilities" => array(GATEWAY_CHARGE, GATEWAY_IPV6_COMPLIANCE),
	"countryCodes" => array("AU"),
);

class SecurePay extends Gateway
{
	public function getModuleInfo()
	{
		return $GLOBALS["moduleInfo"]["SecurePay"];
	}

	public function __construct($input)
	{
		parent::Gateway($input);
		$this->liveURL = "https://vault.safepay.com.au/cgi-bin/direct_process.pl";
		$this->testURL = "https://vault.safepay.com.au/cgi-bin/direct_test.pl";
	}

	public function addDefaultConfigParams()
	{
		static $defaultConfigParams = array(
			array(NULL, NULL, "Username", "", "", "TRANS_SECUREPAY_USERNAME", "TRANS_SECUREPAY_USERNAME_DESC", "t", 1, 1, 0, NULL),
			array(NULL, NULL, "Password", "", "", "TRANS_SECUREPAY_PASSWORD", "TRANS_SECUREPAY_PASSWORD_DESC", "t", 1, 2, 0, NULL),
			array(NULL, NULL, "Testing", "1", "", "TRANS_GATEWAY_TESTING", "TRANS_GATEWAY_TESTING_DESC", "b", 1, 3, 0, NULL),
			array(NULL, NULL, "InvoiceDescription", "", "", "TRANS_INVOICE_DESCRIPTION", "TRANS_INVOICE_DESCRIPTION_DESC", "a", 0, 4, 0, NULL),


			/** each of the sets below are for accepting different types of cards (or echeck), remove
			    any that are not an option with your gateway **/

			// VISA
			array(NULL, NULL, "card_accept_0", "1", "", "TRANS_ACCEPT_VISA", "TRANS_ACCEPT_VISA_DESC", "b", 1, 5, 0, NULL),

			// MASTERCARD
			array(NULL, NULL, "card_accept_1", "1", "", "TRANS_ACCEPT_MASTERCARD", "TRANS_ACCEPT_MASTERCARD_DESC", "b", 1, 6, 0, NULL),

			// AMERICAN EXPRESS
			array(NULL, NULL, "card_accept_2", "1", "", "TRANS_ACCEPT_AMEX", "TRANS_ACCEPT_AMEX_DESC", "b", 1, 7, 0, NULL),

			// JCB
			array(NULL, NULL, "card_accept_4", "1", "", "TRANS_ACCEPT_JCB", "TRANS_ACCEPT_JCB_DESC", "b", 1, 9, 0, NULL),

			// DINERS CLUB
			array(NULL, NULL, "card_accept_6", "1", "", "TRANS_ACCEPT_DINERS", "TRANS_ACCEPT_DINERS_DESC", "b", 1, 11, 0, NULL),

			// BANKCARD
			array(NULL, NULL, "card_accept_10", "1", "", "TRANS_ACCEPT_BANKCARD", "TRANS_ACCEPT_BANKCARD_DESC", "b", 1, 15, 0, NULL),

		);
		return $defaultConfigParams;
	}

	public function execute()
	{
		if ( GATEWAY_PARAM_FAILURE === $this->status ) {
			return $this->status;
		}

		$this->params = correctArray($this->params);

		/*
		$gatewayCodes = array(
			"00" => "Approved or completed successfully",
			"01" => "Refer to card issuer",
			"02" => "Refer to card issuers special conditions",
			"03" => "Invalid merchant",
			"04" => "Pick-up card",
			"05" => "Do not honour",
			"06" => "Error",
			"07" => "Pick-up card",
			"08" => "Honour with identification",
			"09" => "Request in progress",
			"10" => "Approved for partial amount",
			"11" => "Approved VIP",
			"12" => "Invalid transaction",
			"13" => "Invalid amount",
			"14" => "Invalid card number (no such number)",
			"15" => "No such issuer",
			"16" => "Approved",
			"17" => "Customer cancellation",
			"18" => "Customer dispute",
			"19" => "Re-enter transaction",
			"20" => "Invalid response",
			"21" => "No action taken",
			"22" => "Suspected malfunction",
			"23" => "Unacceptable transaction fee",
			"24" => "File update not supported by receiver",
			"25" => "Unable to locate record on file",
			"26" => "Duplicate file update record",
			"27" => "File update field edit error",
			"28" => "File update file locked out",
			"29" => "File update not successful",
			"30" => "Format error",
			"31" => "Bank not supported by switch",
			"32" => "Completed partially",
			"33" => "Expired card",
			"34" => "Suspected fraud",
			"35" => "Card acceptor contact acquirer",
			"36" => "Restricted card",
			"37" => "Card acceptor call acquirer security",
			"38" => "Allowable PIN tries exceeded",
			"39" => "No credit account",
			"40" => "Request function not supported",
			"41" => "Lost card",
			"42" => "No universal account",
			"43" => "Stolen card",
			"44" => "No investment account",
			"51" => "Not sufficient funds",
			"52" => "No cheque account",
			"53" => "No savings account",
			"54" => "Expired card",
			"55" => "Incorrect PIN",
			"56" => "No card record",
			"57" => "Transaction not permitted to cardholder",
			"58" => "Transaction not permitted to terminal",
			"59" => "Suspected fraud",
			"60" => "Card acceptor contact acquirer",
			"61" => "Exceeds withdrawal amount limits",
			"62" => "Restricted card",
			"63" => "Security violation",
			"64" => "Original amount incorrect",
			"65" => "Exceeds withdrawal frequency limit",
			"66" => "Card acceptor call acquirers security department",
			"67" => "Hard capture (requires that card be picked up at ATM)",
			"68" => "Response received too late",
			"75" => "Allowable number of PIN tries exceeded",
			"90" => "Cutoff is in process (Switch ending a days business and starting the next. The transaction can be sent again in a few minutes).",
			"91" => "Issuer or switch is inoperative",
			"92" => "Financial institution or intermediate network facility cannot be found for routing",
			"93" => "Transaction cannot be completed. Violation of law",
			"94" => "Duplicate transmission",
			"95" => "	Reconcile error",
			"96" => "System malfunction",
			"97" => "Advises that reconciliation totals have been reset",
			"98" => "MAC error",
			"99" => "Reserved for national use",
			"115" => "Internal Server Error/Exception",
		);
		*/

		/***********************************************************************
		  At this point the variable $this->params is an array of all data that
		  you will need to perform the requested action.  $this->gatewayInfo
		  contains all of the configuration variables you allow to be set in the
		  config arrays in the method above this one.

			Data in $this->params:
				subCommand
				transactionCode
				billingAccountType
				paymentMethod
				invoiceID
				paymentDescription
				paymentAmount
				paymentMethod
				billingCardNum (cc based)
				billingCardExpYear (cc based)
				billingCardExpMonth (cc based)
				billingCardNameOnCard (cc based)
				contactFirstName
				contactLastName
				cardCVV2 (ONLY AVAILABLE ON FIRST RUN, NOT PRESENT FOR RECURRING)
				billingBankName (echeck based)
				billingBankAbaCode (echeck based)
				billingBankAccountNum (echeck based)
				billingBankAccountType (echeck based)
				billingBankLicenseNum (echeck based)
				billingBankLicenseState (echeck based)
				billingBankDOB (echeck based, date of birth of client)
				clientContactID
				contactCompany
				contactAddress1
				contactAddress2
				contactCity
				contactState
				contactZip
				countriesID (country id corresponding to country record in MB database)
				contactPhone1
				contactFax
				contactEmail
				isRecurring

			Data in $this->gatewayInfo would be things like:
				Username
				Password
				accept_echeck
				Testing
				(values would be whatever the user set in the config screen)


			Follows is an example of what a gateway that used AuthorizeNet's
			protocol would/could look like:


		***********************************************************************/


		/**** START EDITING HERE ****/
		$url = (1 == $this->config["Testing"]) ? $this->testURL : $url = $this->liveURL;

		// these variables will be filled out by your program
		$vars = array();
		$vars["vendor_name"] = $this->config["Username"];
		$vars["vendor_password"] = $this->config["Password"];
		$vars["card_number"] = $this->params['billingCardNum'];
		$vars["card_expiry"] = $this->params['billingCardExpMonth'].$this->params['billingCardExpYear'];
		$vars["card_holder"] = $this->params['billingCardNameOnCard'];
		$vars["card_type"] = "AUTO";
		$vars["card_cvv"] = $this->params['cardCVV2'];
		$vars["payment_amount"] = $this->params['paymentAmount'];
		$vars["payment_reference"] = $this->params['paymentDescription'];

		$curlcall = $this->getCurlObject($url);
		$curlcall->addMultiFields($vars);
		dbg($vars);
		//$info = $this->getModuleInfo();
		//$product = "MB5" . $info["displayName"] . "Module/" . $info["version"];
		//$agent = $product . " (cURL " . $curlcall->getCurlVersion() . ")";
		$curlcall->addHeader("User-Agent: Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0; MB5)");
		$curlResult = $curlcall->toString();

		dbg($curlResult);
		$results = $this->parsetheoutput($curlResult);
		dbg($results);

		switch ($results["summary_code"]){
			case "0":
				$this->status = GATEWAY_SUCCESS;
				$this->authorizationCode = $results["payment_number"];
				$this->transactionID = $results["payment_number"];
			break;
			case "1": case "2":
				$this->status = GATEWAY_DECLINED;
				$this->authorizationCode = $results["payment_number"];
				$this->transactionID = $results["payment_number"];
			break;
			default:
			case "3":
				$this->status = GATEWAY_ERROR;
			break;
		}
		$this->message = $results["response_text"];


		$this->hasExecuted = 1;

		$this->returnData = array("gatewayReturn" => array());

		$this->returnData["gatewayReturn"]["status"] = $this->status;
		$this->returnData["gatewayReturn"]["authorizationCode"] = $this->authorizationCode;
		$this->returnData["gatewayReturn"]["AVSCode"] = '';
		$this->returnData["gatewayReturn"]["transactionID"] = $this->transactionID;
		$this->returnData["gatewayReturn"]["message"] = $this->message;

		$this->xmlData = toXML($this->returnData);
		return $retval;
	}

	private function parsetheoutput($output)
	{
		$output = str_replace(array("&", "\n"), array("&amp;", "&"), $output);
		$out = array();
		parse_str($output, $out);
		return $out;
	}
}