<?php

require_once("lib-mbapi/include/modules/gateway/gateway.php");

$className = "TwoCheckout";
$GLOBALS["moduleInfo"][$className] = array(
	"type" => "gateway",
	"status" => PaymentGateway::STATUS_STABLE,
	"version" => "1.0.1",
	"gatewaytype" => "2",
	"displayName" => "2CheckOut",
	"author" => "Parallels",
	"capabilities" => array(GATEWAY_CHARGE, GATEWAY_IPV6_COMPLIANCE),
	"countryCodes" => array("US"),
);

class TwoCheckout extends Gateway
{
	public function getModuleInfo()
	{
		return $GLOBALS["moduleInfo"]["TwoCheckout"];
	}

	public function __construct($input='')
	{
		parent::Gateway($input);
		$this->paymentURL = "https://www.2checkout.com/2co/buyer/purchase";
	}

	public function addDefaultConfigParams()
	{
		global $CONF;
		$defaultConfigParams = array(
			array(NULL, NULL, "Login_ID", "", "", "TRANS_2CHECKOUT_ID", "TRANS_2CHECKOUT_ID_DESC", "t", 1, 1, 0, NULL),
			array(NULL, NULL, "Testing", "1", "", "TRANS_GATEWAY_TESTING", "TRANS_GATEWAY_TESTING_DESC", "b", 1, 2, 0, NULL),
			array(NULL, NULL, "Language", "en", "", "TRANS_DISPLAY_LANGUAGE", "TRANS_DISPLAY_LANGUAGE_DESC", "r", 1, 3, 0, array
			(
				array(NULL, NULL, "en", "TRANS_ENGLISH", 0),
				array(NULL, NULL, "sp", "TRANS_SPANISH", 1)
			)),
			array(NULL, NULL, "Secret_Word", "", "", "TRANS_2CHECKOUT_SECRET_WORD", "TRANS_2CHECKOUT_SECRET_WORD_DESC", "t", 1, 4, 0, NULL),
			array(NULL, NULL, "IPN_URL", $CONF["secureURLProtocol"]."://".$CONF["secureURL"]."/gw/twocheckout.php", "", "TRANS_2CHECKOUT_IPNURL","TRANS_2CHECKOUT_IPNURL_DESC", "t", 1, 5, 0, NULL),
		);
		return $defaultConfigParams;
	}

	public function execute()
	{
		if ( GATEWAY_PARAM_FAILURE === $this->status ) {
			return $this->status;
		}

		// lets get all invoice info
		$h = new QueryMaster("invoices.*");
		$h->addAndClause("invoice_id", "=", $this->params["invoiceID"][0], "i");
		$invoiceInfo = $h->getRow();

		$h = new QueryMaster("invoice_line_items.*", "packages.*");
		$h->enableStrictMode();
		$h->addAndClause("invoice_id", "=", $this->params["invoiceID"][0], "i");
		$invoiceLineItems = $h->getResults(-1, 0);


		$returnHTML = array();

		// make the one time payment stuff here
		$oneTime = array();
		if ( 1 == $this->config["Testing"] ) {
			$oneTime["demo"] = "Y";
		}
		$oneTime["fixed"] = "Y";
		$oneTime["x_login"] = $this->config["Login_ID"];
		$oneTime["x_invoice_num"] = "MB".$invoiceInfo["invoice_number"];
		$oneTime["x_amount"] = $invoiceInfo["invoice_amount"]-$invoiceInfo["invoice_amount_paid"];
		$oneTime["MBR_Return_CID"] = $invoiceInfo["client_id"];
		$oneTime["MBR_Return_Invoice"] = $invoiceInfo["invoice_id"];
		$oneTime["MBR_Return_CGI"] = $this->configID;
		$oneTime["x_Receipt_Link_URL"] = $this->config["IPN_URL"];

		$oneTime["x_First_Name"] = $this->params["contactFirstName"][0];
		$oneTime["x_Last_Name"] = $this->params["contactLastName"][0];
		$oneTime["x_Phone"] = $this->params["contactPhone1"][0];
		$oneTime["x_Email"] = $this->params["contactEmail"][0];
		$oneTime["street_address"] = $this->params["contactAddress1"][0];
		$oneTime["street_address2"] = $this->params["contactAddress2"][0];
		$oneTime["x_City"] = $this->params["contactCity"][0];
		$oneTime["x_State"] = $this->params["contactState"][0];
		$oneTime["x_Zip"] = $this->params["contactZip"][0];
		$oneTime["x_Country"] = $this->params["countriesISO2"][0];

		$oneTime["id_type"] = 1;
		foreach ((array)$invoiceLineItems as $xx=>$li){
			$oneTime["c_prod_".$xx] = $li["package_id"].",1";
			$oneTime["c_name_".$xx] = $li["package_name"];
			$oneTime["c_description_".$xx] = $li["package_comments"];
			$oneTime["c_price_".$xx] = $li["line_item_total_amount"];
		}

		$returnHTML[] = $this->createOneTimeHTMLOutput($oneTime);


		$this->hasExecuted = 1;

		$retval .= "   <gatewayReturn>\n";
		$retval .= "     <status>1</status>\n";
		foreach ($returnHTML as $ret){
			$retval .= "     <paymentForm>".utf8_html_entity_encode($ret["paymentForm"])."</paymentForm>\n";
			$retval .= "     <paymentLink>".utf8_html_entity_encode($ret["paymentLink"])."</paymentLink>\n";
			$retval .= "     <paymentLinkURL>".utf8_html_entity_encode($ret["paymentLinkURL"])."</paymentLinkURL>\n";
		}
		$retval .= "   </gatewayReturn>\n";

		$this->xmlData = $retval;
		return $retval;
	}

	/**
	*	take a fields array and create the html output fields needed
	*
	*/
	private function createOneTimeHTMLOutput($fields)
	{
		// init return variable
		$ret = array();

		$ret["paymentForm"] = "<form action='".$this->paymentURL."' method='post' target='_blank'>
			<input type='submit' value='".TRANS_PAY_NOW."' />";

		$ret["paymentLinkURL"] = $this->paymentURL."?";

		foreach ($fields as $k=>$v){
			$ret["paymentLinkURL"] .= $k."=".urlencode($v)."&";
			$ret["paymentForm"] .= "<input type='hidden' name='".$k."' value='".$v."' />";
		}

		$ret["paymentForm"] .= "</form>";

		$ret["paymentLink"] = "<a href='".$ret["paymentLinkURL"]."' target='_blank'>".TRANS_PAY_WITH_2CHECKOUT."</a>";

		return $ret;

	}

	/**
	*	process an IPN
	*
	*/
	public function processReturn($post)
	{
		dbg("2Checkout: processing ipn return");

		foreach ($post as $k=>$v){
			$this->IPNData[$k] = secureInput(stripslashes($v));
		}


		foreach ($this->IPNData as $k=>$v) $hashString .= "<$k>$v";
		$this->IPNData["MBv5Signiture"] = sha1($hashString);

		// lets test for duplicate IPN's
		// we take the hash of our stuff and check for a callback log attribute
		// for a paypal callback log record that matches our hash.
		$h = new QueryMaster("callback_log_attributes.*");
		$h->addTables("callback_log");
		$h->addJoin("callback_log", "callback_log_id", "callback_log_attributes");
		$h->addAndClause("callback_log_attributes.callback_log_attribute_name", "=", "MBv5Signiture", "s");
		$h->addAndClause("callback_log.callback_log_type", "=", getAPITypeID("ipnmodules", "twocheckout"), "s");
		$h->addAndClause("callback_log_attributes.callback_log_attribute_value", "=", $this->IPNData["MBv5Signiture"], "s");
		$testRow = $h->getRow();
		if ($testRow["callback_log_attribute_value"]==$this->IPNData["MBv5Signiture"]){
			dbg($testRow);
			dbg($this->IPNData);
			$this->logIPN(blmsg("TRANS_DUPLICATE_IPN_POSTING"));
			return $this->create2coOutput(blmsg("TRANS_DUPLICATE_IPN_POSTING")); // no more processing, this was a dupe
		}

		dbg($this->IPNData);

		if ( isset($this->IPNData["mbr_return_cgi"]) ) {
			$payment_gateway = Factory()->PaymentGateway($this->IPNData["mbr_return_cgi"]);
		}

		$this->callbackLogID = $callbackLogID = $this->insertCallbackLog($callbackLogData = array(
			"callbackLogType" => getAPITypeID("ipnmodules", "twocheckout"),
			"callbackLogTransactionID" => $this->IPNData["order_number"],
			"callbackLogParentTransactionID" => '',
			"callbackLogAction" => getAPITypeID("callbackAction", "payment"),
			"callbackLogActionStatus" => 0,
			"callbackLogDateCreated" => time(),
			"clientID" => $this->IPNData["mbr_return_cid"],
			"paymentGatewayID" => (isset($payment_gateway) ? $payment_gateway->id : null),
		), $this->IPNData);

		$clientID = $this->IPNData["mbr_return_cid"];
		$invoiceID = $this->IPNData["mbr_return_invoice"];

		if ( !isset($payment_gateway) ) {
			dbg("invalid payment gateway id passed back");
			return $this->create2coOutput(blmsg("TRANS_THERE_WAS_A_PROBLEM_WITH_YOUR_ORDER"));
		}

		$this->config = PaymentGateway::getMapOf($payment_gateway->PaymentGatewayParams, "name", "value");

		if (strtolower($this->IPNData["demo"])=="y" && !(int)$this->config["Testing"]){
			return $this->create2coOutput(blmsg("TRANS_DEMO_TRANSACTIONS_NOT_PERMITTED_AT_THIS_TIME"));
		}

		dbg("gateway configs");
		dbg($this->config);
 // 2co happens to be retarded and not let you verify
		dbg("order verify hash");
		dbg($this->IPNData["x_MD5_Hash"]);

		$this->IPNData["x_amount"] = sprintf("%01.2f", $this->IPNData["x_amount"]);

		$hashCheck1 = strtoupper(md5($this->config["Secret_Word"].$this->config["Login_ID"]."1".$this->IPNData["x_amount"]));
		$hashCheck2 = strtoupper(md5($this->config["Secret_Word"].$this->config["Login_ID"].$this->IPNData["order_number"].$this->IPNData["x_amount"]));
		dbg($hashCheck1);
		dbg($hashCheck2);

		if ($hashCheck1!=$this->IPNData["x_MD5_Hash"] && $hashCheck2!=$this->IPNData["x_MD5_Hash"]){
			return $this->create2coOutput("There has been an error with your order.  Please do not attempt to process any payments again until the system administrator contacts you.");
		}

		dbg("applying IPN paypal payment for invoice");

		// if we dont have an invoice to pay lets get one
		if (!$invoiceID){
			$h = new QueryMaster("invoices.*");
			$h->addTables("invoice_line_items");
			$h->addTables("packages");
			$h->addJoin("packages", "package_id", "invoice_line_items");
			$h->addJoin("invoice_line_items", "invoice_id", "invoices");
			//$h->addAndClause("packages.client_id", "=", $clientID, "i");
			$h->addAndClause("invoices.invoice_id", "=", $this->IPNData["mbr_return_invoice"], "i");
			$h->addCustomAndClause("invoices.invoice_amount_paid <> invoices.invoice_amount");
			$h->addAndClause("invoices.invoice_status", "=", array(
				getAPIStatusID("invoice", "due"),
				getAPIStatusID("invoice", "declined"),
				getAPIStatusID("invoice", "errored"),
			), "i");
			$h->addOrderByColumns("invoice_date_due", "ASC");
			$invoiceInfo = $h->getRow();
			$invoiceID = $invoiceInfo["invoice_id"];
		} else {
			$h = new QueryMaster("invoices.*");
			$h->addAndClause("invoice_id", "=", $invoiceID, "i");
			$invoiceInfo = $h->getRow();
		}

		$clientID = $invoiceInfo["client_id"];

		$this->IPNData["currency_id"] = $invoiceInfo["currency_id"];

		// lets go ahead and make the transaction
		$transactionID = $this->makeTransaction($this->IPNData, $invoiceInfo["invoice_number"], $payment_gateway->id);
		dbg("inserted transaction: ".$transactionID);

		// if we couldn't find an invoice to pay, lets just give this client a credit and be done with it.
		if (!$invoiceID){
			$this->giveClientCredit($mapData, $this->IPNData);
		} else {
			// get the line items data...
			$allLineItems = $this->getLineItems($invoiceID);
			$lineItemIDs = $this->getLineItemIDsFromLineItems($allLineItems);
			$lineItemPayments = $this->getLineItemPayments($lineItemIDs);

			// now we need to apply the payment
			$this->payLineItemsAndDoAccounting(
				$this->getUnpaidLineItems($allLineItems, $lineItemPayments),
				$transactionID
			);
			$this->sendSecondaryPaymentEmail($clientID, $transactionID, $invoiceID);
		}

		return $transactionID;
	}

	private function create2coOutput($message)
	{
		throw new ProductException(ProductException::E_FAIL_OPERATION, $message);
	}

	/**
	 * function to give a client a credit because we can't find anything to apply the payment to
	 */
	private function giveClientCredit()
	{
		dispatchMBAPI(toXML(array(
			"mbapi" => array(
				"command" => "SetCreditAccounting",
				"params" => array(
					"clientID" => $this->IPNData["mbr_return_cid"],
					"ledgerAccountName" => "cash",
					"amount" => $this->IPNData["x_amount"],
					"currencyID" => $this->IPNData["currency_id"],
					"description" => TRANS_2CHECKOUT_PAYMENT_WITH_NOTHING_TO_PAY,
				),
			),
		)));
	}

	/**
	 * function to shortcut making a transaction
	 */
	private function makeTransaction($IPNData, $invoiceNumber, $paymentGatewayID)
	{
		$transactionID = $this->insertTransaction(array(
			"auth_return" => GATEWAY_SUCCESS,
			"transaction_payment_method" => getAPITypeID("payments", "twocheckout"),
			"transaction_code" => "2checkout".$this->IPNData["order_number"],
			"transaction_notes" => TRANS_PAYMENT_FOR_INVOICE.$this->IPNData["mbr_return_invoice"],
			"transaction_date_created" => time(),
			"transaction_amount" => doubleval($this->IPNData["x_amount"]),
			"currency_id" => $this->IPNData["currency_id"],
			"payment_gateway_id" => $paymentGatewayID,
		));
		if ((int)$this->callbackLogID){
			updateForKey("callbackLogs", $this->callbackLogID, array(
				"transactionID" => $transactionID,
			));
		}
		return $transactionID;
	}

	/**
	 * function to set the payments for line items and make the accounting entries
	 */
	private function payLineItemsAndDoAccounting($unpaidLineItems, $transactionID)
	{
		// ok, got all the unpaid line items for the oldest invoice that has packages we are paying for
		// in an array called $unpaidLineItems, now we just need to loop through them and apply the money
		// towards them.
		$invoiceInfo = $unpaidLineItems[0]; // the invoice info was gotten in the line items query

		$amountAllowcated = 0;
		foreach ($unpaidLineItems as $k=>$li){
			$available = doubleval($this->IPNData["x_amount"])-$amountAllowcated;
			$amountNeeded = $li["line_item_total_amount"]-$li["paid"];
			if ($amountNeeded>$available){
				$payAmount = $available;
			} else {
				$payAmount = $amountNeeded;
			}
			$unpaidLineItems[$k]["applyAmount"] = $payAmount;
			$amountAllowcated += $payAmount;
		}
		$unallowcated = doubleval($this->IPNData["x_amount"])-$amountAllowcated;
		dbg("applied payments to line items, unallowcated: ".$unallowcated);


		// ok, so we made the amounts to pay, lets pay them
		// (that means inserting the transaction_success_rel entries and updating the invoice paid amt)
		foreach ($unpaidLineItems as $k=>$li){
			if ($li["applyAmount"]){
				$h = new QueryMaster("insert");
				$h->addTables("transaction_success_rel");
				$h->addParam("transaction_id", $transactionID, "i");
				$h->addParam("line_item_id", $li["line_item_id"], "i");
				$h->addParam("line_item_payment_amount", doubleval($li["applyAmount"]), "d");
				$h->execute();
				$invoiceInfo["amountPaidNow"] += doubleval($li["applyAmount"]);
				dbg("added payment for line item: ".$li["line_item_id"]." of ".doubleval($li["applyAmount"]));
			}
		}

		$h = new QueryMaster("update");
		$h->addTables("invoices");
		$h->addAndClause("invoice_id", "=", $this->IPNData["mbr_return_invoice"], "i");
		$h->addParam("invoice_amount_paid", round(doubleval($invoiceInfo["amountPaidNow"]+$invoiceInfo["invoice_amount_paid"]), 2), "d");
		$h->addParam("invoice_date_paid", time(), "i");
		if (round(($invoiceInfo["amountPaidNow"]+$invoiceInfo["invoice_amount_paid"]),2)==round($invoiceInfo["invoice_amount"], 2)){
			$h->addParam("invoice_status", 1, "i");
			dbg("invoice is closed");
		}
		$h->execute();
		dbg("updated invoice info");

		if ($unallowcated){
			$credit = $unallowcated;
			dbg("theres a credit amount");
		}

		// make the accounting entry
		require_once mb_path("/lib-billing/include/accounting/Payment.php");
		$receipt = new Payment();
		$receipt->setClientID($this->IPNData["mbr_return_cid"]);
		$receipt->setInvoiceID($invoiceInfo["invoice_id"]);
		$receipt->invoiceNumber = $invoiceInfo["invoice_number"];
		$receipt->setTransactionID($transactionID);
		$receipt->setDescription(TRANS_2CHECKOUT_PAYMENT_FOR_INVOICE.": ".$invoiceInfo["invoice_number"]);
		$receipt->setPaymentAmount(doubleval($this->IPNData["x_amount"]));
		if ($credit>0){
			$receipt->setClientCreditAmount($credit);
		}
		$receipt->setCurrencyID($this->IPNData["currency_id"]);
		$receipt->execute();
		dbg("made the acocunting entry");

		// associate the invoice with the transaction
		$h = new QueryMaster("insert");
		$h->addTables("invoice_transaction_rel");
		$h->addParam("invoice_id", $invoiceInfo["invoice_id"], "i");
		$h->addParam("transaction_id", $transactionID, "i");
		$h->execute();
		dbg("set the invoice transaction rel record");

	}

	/**
	 * function to get the unpaid line items from a set of line items and their payment data
	 */
	private function getUnpaidLineItems($allLineItems, $lineItemPayments, $invoiceID=false)
	{
		$unpaidLineItems = array();
		if ($invoiceID){
			foreach ($allLineItems as $k=>$li){
				$allLineItems[$k]["paid"] = $lineItemPayments[$li["line_item_id"]]["paid"];
				if ($lineItemPayments[$li["line_item_id"]]["paid"]<$li["line_item_total_amount"] && $li["invoice_id"]==$invoiceID){
					$unpaidLineItems[] = $allLineItems[$k];
				}
			}
			dbg("figured out which ones weren't paid for: ".count($unpaidLineItems));
			return $unpaidLineItems;
		} else {
			foreach ($allLineItems as $k=>$li){
				$allLineItems[$k]["paid"] = $lineItemPayments[$li["line_item_id"]]["paid"];
				if ($lineItemPayments[$li["line_item_id"]]["paid"]<$li["line_item_total_amount"]){
					$unpaidLineItems[] = $allLineItems[$k];
				}
			}
			dbg("figured out which ones weren't paid for: ".count($unpaidLineItems));
			return $unpaidLineItems;
		}
	}

	/**
	 * function to get the line items ids out of the line items array
	 */
	private function getLineItemIDsFromLineItems($lineItems)
	{
		$lineItemIDs = array();
		foreach ($lineItems as $li){
			$lineItemIDs[] = $li["line_item_id"];
		}
		return $lineItemIDs;
	}

	/**
	 * function to get the line items for certain parameters
	 */
	private function getLineItems($invoiceID=false, $packageIDs=false)
	{
		if ($invoiceID){
			$h = new QueryMaster("invoices.*", "invoice_line_items.*");
			$h->addAndClause("invoices.invoice_id", "=", $invoiceID, "i");
			$allLineItems = $h->getResults(-1, 0);
			dbg("query: ".$h->getQuery());
			dbg("got all line items: ".count($allLineItems));
			return $allLineItems;
		} else {
			$h = new QueryMaster("invoices.*", "invoice_line_items.*");
			$h->addAndClause("package_id", "=", $packageIDs, "i");
			$h->addCustomAndClause("invoice_amount_paid < invoice_amount");
			$h->addOrderByColumns("invoice_date_due", "ASC");
			$allLineItems = $h->getResults(-1, 0);
			dbg("got all line items: ".count($allLineItems));
			return $allLineItems;
		}
	}

	/**
	 * function to get the success relations for certain line items
	 */
	private function getLineItemPayments($lineItemIDs)
	{
		$h = new QueryMaster("transaction_success_rel.*");
		$h->addAndClause("line_item_id", "=", $lineItemIDs, "i");
		$payments = $h->getResults(-1, 0);
		dbg("got transaction success rows: ".count($payments));

		// add up the payments according to line item id and organize them as well
		$lineItemPayments = array();
		foreach ($payments as $pmnt){
			$lineItemPayments[$pmnt["line_item_id"]]["paid"] += $pmnt["line_item_payment_amount"];
			$lineItemPayments[$pmnt["line_item_id"]]["payments"][] = $pmnt;
		}
		return $lineItemPayments;
	}

	/**
	 * function to log an error'd ipn
	 */
	private function logIPN($message)
	{
		dbg("logging IPN: ".$message);
		insertFor("invalidIpnLogs", array(
			"ipnStatus" => getAPIStatusID("invalidipns", "new"),
			"ipnReason" => $message,
			"ipnModule" => getAPITypeID("ipnmodules", "twocheckout"),
			"ipnServer" => base64_encode(serialize($_SERVER)),
			"ipnGet" => base64_encode(serialize($_GET)),
			"ipnPost" => base64_encode(serialize($_POST)),
			"ipnDateCreated" => time(),
		));
	}

	/**
	* Calculates the taxes for this invoice amount
	*
	* @param amount    the amount to get taxes on
	* @param packageID  the package id we are getting tax zones for
	* @return the tax amount
	*/
	private function calculateTaxes($amount, $packageID)
	{
		$taxZones = $this->currentTaxZones[$packageID];
		$tax = 0;
		$taxableAmount = $amount;
		for ($i = 0; $i < count($taxZones); $i++){
			$taxZone = $taxZones[$i];
			if ($taxZone["line_item_tax_amount_type"]==0){ // if 0 we use as percentage
				if ((int)$taxZone["tax_zone_stackable"]){
					$cTax = round($amount*($taxZone["line_item_tax_amount"]/100), 2);
					$taxableAmount = round($taxableAmount + $cTax, 2);
				} else {
					$cTax = round($taxableAmount*($taxZone["line_item_tax_amount"]/100), 2);
				}
				$tax = round($tax + $cTax, 2);
			} else { // else  we use as set amount
				$cTax = $taxZone["line_item_tax_amount"];
				if ((int)$taxZone["tax_zone_stackable"]){
					$taxableAmount = round($taxableAmount + $cTax, 2);
				}
				$tax = round($tax + $cTax, 2);
			}
		}
		return $tax;
	}
}
