<?php

require_once("lib-mbapi/include/modules/gateway/gateway.php");

$className = "WorldPay";
$GLOBALS["moduleInfo"][$className] = array(
	"type" => "gateway",
	"status" => PaymentGateway::STATUS_STABLE,
	"version" => "1.0.2",
	"gatewaytype" => "2",
	"displayName" => "WorldPay",
	"author" => "Parallels",
	"capabilities" => array(GATEWAY_CHARGE, GATEWAY_RECURRING_CHARGE, GATEWAY_IPV6_COMPLIANCE),
	"countryCodes" => array("GB"),
);

class WorldPay extends Gateway
{
	public function getModuleInfo()
	{
        return $GLOBALS["moduleInfo"]["WorldPay"];
    }

	public function __construct($input = "")
	{
		parent::__construct($input);
		$this->paymentURL = (0 == $this->config["Transaction_Mode"]) ? "https://secure.wp3.rbsworldpay.com/wcc/purchase" : "https://select-test.wp3.rbsworldpay.com/wcc/purchase";
	}

	public function addDefaultConfigParams()
	{
		global $CONF;
		$defaultConfigParams = array(
			array(NULL, NULL, "Installation_ID", "", "", "TRANS_WORLDPAY_ID", "TRANS_WORLDPAY_ID_DESC", "t", 1, 1, 0, NULL),
			array(NULL, NULL, "IPN_Password", rand(10000, 99999), "", "TRANS_WORLDPAY_CALLBACK_PASSWORD", "TRANS_WORLDPAY_CALLBACK_PASSWORD_DESC", "t", 1, 3, 0, NULL),
			array(NULL, NULL, "IPN_URL", $CONF["secureURLProtocol"]."://".$CONF["secureURL"]."/gw/worldpay.php", "", "TRANS_WORLDPAY_IPNURL","TRANS_WORLDPAY_IPNURL_DESC", "t", 1, 4, 0, NULL),
			array(NULL, NULL, "Submit_Type", "button", "", "TRANS_WORLDPAY_SUBMIT_TYPE","TRANS_WORLDPAY_SUBMIT_TYPE_DESC", "s", 0, 5, 0, array
                (
                    array(NULL, NULL, "button","TRANS_BUTTON",1),
                    array(NULL, NULL, "image","TRANS_IMAGE",2)
                )),
			array(NULL, NULL, "Pay_Image", "http://www.rbsworldpay.com/images/poweredByRBSWorldPay.gif", "", "TRANS_WORLDPAY_SUBMITIMAGE","TRANS_WORLDPAY_SUBMITIMAGE_DESC", "t", 1, 6, 0, NULL),
			array(NULL, NULL, "Subscribe_Image", "http://www.rbsworldpay.com/images/poweredByRBSWorldPay.gif", "", "TRANS_WORLDPAY_SUBIMAGE","TRANS_WORLDPAY_SUBIMAGE_DESC", "t", 1, 7, 0, NULL),
			array(NULL, NULL, "Enable_Subscriptions", "0", "", "TRANS_WORLDPAY_FURTUREPAY_ENABLE","TRANS_WORLDPAY_FURTUREPAY_ENABLE_DESC", "b", 0, 8, 0, NULL),
			array(NULL, NULL, "Auth_Mode", "0", "", "TRANS_WORLDPAY_TRANSACTION_OPERATION","TRANS_WORLDPAY_TRANSACTION_OPERATION_DESC", "r", 0, 9, 0, array
                (
                    array(NULL, NULL, "0","TRANS_CAPTURE_FUNDS",1),
                    array(NULL, NULL, "1","TRANS_AUTHORIZE_ONLY",2)
                )),
			array(NULL, NULL, "Transaction_Mode", "100", "", "TRANS_WORLDPAY_MODE", "TRANS_WORLDPAY_MODE_DESC", "s", 1, 10, 0, array
                (
                    array(NULL, NULL, "100","TRANS_TESTING_ALWAYS_APPROVE",1),
                    array(NULL, NULL, "101","TRANS_TESTING_ALWAYS_DECLINE",2),
                    array(NULL, NULL, "0","TRANS_LIVE_MODE",3)
                ))
		);
		return $defaultConfigParams;
	}

	public function execute()
	{
		if ( $this->status === GATEWAY_PARAM_FAILURE ) {
			return $this->status;
		}

		if ( isset($this->params["singlePayment"]) && 1 == $this->params["singlePayment"][0] ) {
			$this->config["Enable_Subscriptions"] = 0;
		}

		// lets get all invoice info
		$h = new QueryMaster("invoices.*");
		$h->addAndClause("invoice_id", "=", $this->params["invoiceID"][0], "i");
		$invoiceInfo = $h->getRow();

		// get the line items
		$h = new QueryMaster("invoice_line_items.*", "packages.*");
		$h->enableStrictMode();
		$h->addAndClause("invoice_id", "=", $this->params["invoiceID"][0], "i");
		$invoiceLineItems = $h->getResults(-1, 0);

		// get package and line item ids
		$lineItemIDs = array();
		$pids = array();
		$lineItemToPackage = array();
		$lineItemToAddon = array();
		foreach ($invoiceLineItems as $row){
			$clientID = $row["client_id"];
			$pids[] = $row["package_id"];
			$lineItemIDs[] = $row["line_item_id"];
			$lineItemToPackage[$row["line_item_id"]] = $row["package_id"];
			$lineItemToAddon[$row["line_item_id"]] = $row["package_addon_id"];
		}

		// getting all shares for those ids
		$hx = new QueryMaster("client_billing_account_shares.*");
		$hx->addAndClause("package_id", "=", $pids, "i");
		$bas = $hx->getResults(-1, 0);
		// making keyed array of billing accounts
		$billingAccounts = array();
		foreach ($bas as $row){
		 	$billingAccounts[$row["package_id"]] = $row["billing_account_id"];
		}

		$this->currentTaxZones = array();
		// if we dont have a tax exempt client:
		if ((int)$this->params["applyTax"][0]){

			// gotta get tax zones
			$h = new QueryMaster("line_items_tax_zone_rel.*");
			$h->addTables("tax_zones.*");
			$h->addJoin("tax_zones", "tax_zone_id", "line_items_tax_zone_rel");
			$h->addAndClause("line_item_id", "=", $lineItemIDs, "i");
			$h->addOrderByColumns("tax_zone_stackable", "DESC");
			$currentTaxZones = $h->getResults(-1, 0);
			dbg($currentTaxZones);

			foreach ($currentTaxZones as $row){
				if ((int)$lineItemToAddon[$row["line_item_id"]]){
					$this->currentAddonTaxZones[$lineItemToAddon[$row["line_item_id"]]][] = $row;
				} else {
					$this->currentTaxZones[$lineItemToPackage[$row["line_item_id"]]][] = $row;
				}
			}
		}
		dbg("this-taxzones:");
		dbg($this->currentTaxZones);

		// init addon ids we need to get
		$packageAddonIDs = array();

		// run through line items getting any addon ids we need
		foreach ($invoiceLineItems as $lineItem){
		 	if ($lineItem["package_addon_id"]){
				$packageAddonIDs[] = $lineItem["package_addon_id"];
			}
		}

		// query the db for addon info for the addon ids we need
		$h = new QueryMaster("package_addons.*");
		$h->addAndClause("package_addon_id", "=", $packageAddonIDs, "i");
		$addons = $h->getResults(-1, 0);

		// init keyed addon array
		$keyedAddons = array();

		// populate the keyed addon array (not enough data to worry about passing by reference)
		foreach ($addons as $addon){
		 	if ($addon["package_addon_id"]){
				$keyedAddons[$addon["package_addon_id"]] = $addon;
			}
		}

		// go through line items and add addon data into line items data where needed
		foreach ($invoiceLineItems as $k=>$lineItem){
		 	$invoiceLineItems[$k]["numRenewals"] = $this->getNumRenewals($lineItem); // get the number of times this needs to renew
		 	$invoiceLineItems[$k]["cycleInfo"] = $this->translateCycleInfo($lineItem);
		 	if ($lineItem["package_addon_id"]){
				$invoiceLineItems[$k]["addonInfo"] = $keyedAddons[$lineItem["package_addon_id"]];
			}
		}


		// initialize the array we will keep track of how many (and for what) subscriptions we will need
		$subscriptions = array();

		$unsupportedCycle = false;

		$description = "";

		// loop through the line items of this invoice and group them into subarrays based on their cycle and
		// coupon num cycles
		foreach ($invoiceLineItems as $lineItem){
		 	if ($lineItem["cycleInfo"]===false){
				$unsupportedCycle = true;
			}
			$billingAccountID = $billingAccounts[$lineItem["package_id"]];
		 	if ($lineItem["package_addon_id"]){
		 		if ($description==""){
		 			$description = $lineItem["addonInfo"]["package_addon_display_name"];
				} else {
		 			$description .= ", ".$lineItem["addonInfo"]["package_addon_display_name"];
				}
				$cycle_id = $lineItem["cycle_id"];
				$coupon_num_cycles = $lineItem["addonInfo"]["coupon_num_cycles"];
				$num_renewals = $lineItem["numRenewals"];
			} else {
		 		if ($description==""){
		 			$description = $lineItem["package_name"];
				} else {
		 			$description .= ", ".$lineItem["package_name"];
				}
				$cycle_id = $lineItem["cycle_id"];
				$coupon_num_cycles = $lineItem["coupon_num_cycles"];
				$num_renewals = $lineItem["numRenewals"];
			}
			$subscriptions[$cycle_id."-".$num_renewals."-".$coupon_num_cycles."-".$billingAccountID][] = $lineItem;
		}

		dbg($subscriptions);

		$returnHTML = array();

		// make the one time payment stuff here
		$oneTime = array();
		$oneTime["instId"] = $this->config["Installation_ID"];
		$oneTime["cartId"] = $this->params["invoiceID"][0]."P".$this->configID;
		$oneTime["amount"] = $invoiceInfo["invoice_amount"];
		$oneTime["currency"] = $this->params["currency"][0];
		$oneTime["M_invoice"] = $this->params["invoiceID"][0];
		$oneTime["M_secondary"] = $this->configID;
		$oneTime["M_callback"] = $this->config["IPN_URL"];
		$oneTime["testMode"] = $this->config["Transaction_Mode"];

		$oneTime["authMode"] = (int)$this->config["Auth_Mode"] ? "E" : "A";

		$oneTime["desc"] = $description;
		$oneTime["name"] = billing_get_full_name($this->params["contactFirstName"][0], $this->params["contactLastName"][0]);
		$oneTime["address"] = $this->params["contactAddress1"][0]." &#10;".$this->params["contactAddress2"][0]." &#10;".$this->params["contactState"][0].", ".$this->params["contactCity"][0];
		$oneTime["postcode"] = $this->params["contactZip"][0];
		$oneTime["tel"] = $this->params["contactPhone1"][0];
		$oneTime["email"] = $this->params["contactEmail"][0];
		$oneTime["country"] = $this->params["countriesISO2"][0];

		$returnHTML[] = $this->createOneTimeHTMLOutput($oneTime);

		// ok, added first one time payment, now doing subscriptions

		if ( 1 == $this->config["Enable_Subscriptions"] ) {

			// for each subscription group we have, loop through the items and get the prices and the cycle unit
			// and the num promotional period cycles.
			foreach ($subscriptions as $items){

				$description = "";
				$firstprice = 0;
				$regprice = 0;
				$packages = array();
				$addons = array();
				foreach ($items as $item){
					if ($item["package_addon_id"]){
						$addons[] = $item["package_addon_id"];
						$couponcycles = $item["addonInfo"]["coupon_num_cycles"];
						$tp = $item["addonInfo"]["package_addon_price"]*$item["line_item_quantity"];
						$tp += $this->calculateTaxes($tp, $item["package_id"], $item["package_addon_id"]);
						$regprice = round($regprice+$tp, 2);
		 				if ($description==""){
		 					$description = $lineItem["package_name"];
						} else {
		 					$description .= ", ".$lineItem["package_name"];
						}
					} else {
						$packages[] = $item["package_id"];
						$couponcycles = $item["coupon_num_cycles"];
						$tp = $item["package_price"]*$item["line_item_quantity"];
						$tp += $this->calculateTaxes($tp, $item["package_id"]);
						$regprice = round($regprice+$tp, 2);
		 				if ($description==""){
		 					$description = $lineItem["addonInfo"]["package_addon_display_name"];
						} else {
		 					$description .= ", ".$lineItem["addonInfo"]["package_addon_display_name"];
						}
					}
				}

				$h = new QueryMaster("insert");
				$h->addTables("subscriptions");
				$h->addParam("subscription_status", getStatusID("subscription", "inactive"), "i");
				$h->addParam("client_id", $clientID, "i");
				$h->addParam("subscription_date_created", mktime(), "i");
				$h->addParam("subscription_date_modified", mktime(), "i");
				$h->addParam("payment_gateway_id", $this->configID, "i");
				$h->addParam("billing_account_id", $billingAccounts[$item[0]["package_id"]], "i");
				$subresult = $h->execute();
				$subscriptionID = $h->getLastInsertID();

				dbg($packages);
				dbg($addons);

				foreach ($packages as $pid){
					$h = new QueryMaster("insert");
					$h->addTables("subscription_package_rel");
					$h->addParam("subscription_id", $subscriptionID, "i");
					$h->addParam("package_id", $pid, "i");
					$h->execute();
				}

				foreach ($addons as $pid){
					$h = new QueryMaster("insert");
					$h->addTables("subscription_package_addon_rel");
					$h->addParam("subscription_id", $subscriptionID, "i");
					$h->addParam("package_addon_id", $pid, "i");
					$h->execute();
				}


				$initialPeriodPrice = 0;

				$periodLength = $items[0]["cycleInfo"]["cycles"];
				$periodUnit = $items[0]["cycleInfo"]["unit"];

				$subscription = array();
				$subscription["futurePayType"] = "regular";
				$subscription["authMode"] = (int)$this->config["Auth_Mode"] ? "E" : "A";
				$subscription["startDate"] = date("Y-m-d", $items[0]["package_date_next_renewal"]);
				//$subscription["startDelayUnit"] = $periodUnit;
				//$subscription["startDelayMult"] = $periodLength;
				if ($items[0]["numRenewals"]>-1){
					$subscription["noOfPayments"] = $items[0]["numRenewals"]; // number of times this payment will happen
				}
				$subscription["intervalUnit"] = $periodUnit;
				$subscription["intervalMult"] = $periodLength;
				$subscription["normalAmount"] = $regprice;
				$subscription["option"] = "1";
				$subscription["instId"] = $this->config["Installation_ID"];
				$subscription["cartId"] = $subscriptionID;
				$subscription["amount"] = $regprice;
				$subscription["currency"] = $this->params["currency"][0];
				$subscription["testMode"] = $this->config["Transaction_Mode"];
				$subscription["desc"] = $description;

				$returnHTML[] = $this->createSubscriptionHTMLOutput($subscription);

			}

			////

		}

		$this->hasExecuted = 1;

		if ($unsupportedCycle){
			$retval .= "   <gatewayReturn>\n";
			$retval .= "     <status>3</status>\n";
			$retval .= "     <message>Unsupported cycle</message>\n";
			$retval .= "	 <unsupportedCycle>1</unsupportedCycle>\n";
			$retval .= "   </gatewayReturn>\n";
		} else {
			$retval .= "   <gatewayReturn>\n";
			$retval .= "     <status>1</status>\n";
			foreach ($returnHTML as $ret){
				$retval .= "     <paymentForm>".utf8_html_entity_encode($ret["paymentForm"])."</paymentForm>\n";
				$retval .= "     <paymentLink>".utf8_html_entity_encode($ret["paymentLink"])."</paymentLink>\n";
				$retval .= "     <paymentLinkURL>".utf8_html_entity_encode($ret["paymentLinkURL"])."</paymentLinkURL>\n";
			}
			$retval .= "   </gatewayReturn>\n";
		}

		$this->xmlData = $retval;
		return $retval;
	} //execute

	/**
	 *	process an IPN
	 *
	 */
	public function processReturn($post)
	{
		global $CONF;

	 	dbg("WorldPay: processing ipn return");

	 	$this->IPN = (isset($post['V4IPN']) && $post['V4IPN']) ? unserialize(base64_decode($post['V4IPN'])) + array('V4IPN' => 'TRUE') : $post;

		foreach ($this->IPN as $k=>$v){
	 	 	$this->IPNData[$k] = secureInput(stripslashes($v));
		}

		dbg($this->IPNData);

 		if ($this->IPNData["futurePayStatusChange"]=="Merchant Cancelled" || $this->IPNData["futurePayStatusChange"]=="Customer Cancelled"){

			dbg("subscription cancellation IPN.");
			$h = new QueryMaster("update");
			$h->addTables("subscription");
			$h->addParam("subscription_status", getStatusID("subscription", "cancelled"), "i");
			$h->addAndClause("subscription_external_id", "=", $this->IPNData["futurePayID"], "s");
			$h->execute();
			$this->runSubscriptionCancellation();
			dbg("updated subscription status");
			return false;

		}

		if ( isset($this->IPNData["M_secondary"]) ) {
			$payment_gateway = Factory()->PaymentGateway($this->IPNData["M_secondary"]);
		} else if ( isset($this->IPNData["cartId"]) ) {
			$pieces = explode("P", $this->IPNData["cartId"]);
			if ( isset($this->IPNData["M_invoice"]) ) {
				$payment_gateway = Factory()->PaymentGateway($pieces[1]);
			} else {
				$payment_gateway = Factory()->Subscription($pieces[0])->PaymentGateway;
			}
		}
		if ( !isset($payment_gateway) ) {
			dbg("Failed to find payment gateway");
			return false;
		}

		if ( $post["callbackPW"] != $payment_gateway->PaymentGatewayParams["IPN_Password"]->value ) {
			dbg("IPN is NOT valid");
			return false;
		}

		// is a valid ipn call
		dbg("IPN is valid");

		// we need to check the database to see if we have already recorded and processed this transaction
		$h = new QueryMaster("transaction_log.*");
		$h->addAndClause("transaction_code", "=", "worldpay".$this->IPNData["transId"], "s");
		$checkTransaction = $h->getRow();

		if ((int)$checkTransaction["transaction_id"]>0){
			// we already recorded this
			dbg("we have already recorded this transaction");
			return false;
		} else {

			// if this is a subscription
			if (!isset($this->IPNData["M_invoice"])){
/**
* BEGIN applying payment to packages and addons (for a subscription ipn)
* this will create a transaction to record the ipn, as well as creating the success ids for the
* line items that got paid, and changing the status of the invoice they are on accordingly as is necessary.
*/
				// get the subscription data
				$pieces = explode("P", $this->IPNData["cartId"]);
				$subscription_id = $pieces[0];
				$h = new QueryMaster("subscriptions.*");
				$h->addAndClause("subscription_id", "=", $subscription_id, "i");
				$subscriptionInfo = $h->getRow();
				dbg("Got subscription info:");
				dbg($subscriptionInfo);
				if (!(int)$subscriptionInfo["subscription_id"]){
					dbg("invalid subscription id");
					return false;
				}

				if ($this->IPNData["transStatus"]=="N"){

					$h = new QueryMaster("insert");
					$h->addTables("transaction_log");
					$h->addParam("auth_return", GATEWAY_DECLINED, "i");
					$h->addParam("transaction_payment_method", 0, "i");
					$h->addParam("transaction_code", "worldpay".$this->IPNData["transId"], "s");
					$h->addParam("transaction_notes", $this->IPNData["rawAuthMessage"], "s");
					$h->addParam("transaction_date_created", mktime(), "i");
					$h->addParam("transaction_amount", doubleval($this->IPNData["amount"]), "d");
					$h->addParam("currency_id", $this->IPNData["currency"], "s");
					$h->addParam("payment_gateway_id", $subscriptionInfo["payment_gateway_id"], "s");
					$h->execute();

				} else {

					if ($subscriptionInfo["subscription_status"]!=getStatusID("subscription", "active")){
						$h = new QueryMaster("update");
						$h->addTables("subscription");
						$h->addParam("subscription_status", getStatusID("subscription", "active"), "i");
						$h->addAndClause("subscription_id", "=", $subscriptionInfo["subscription_id"], "i");
						$h->execute();
						dbg("updated subscription status");
					}

					// lets go ahead and make the transaction
					$h = new QueryMaster("insert");
					$h->addTables("transaction_log");
					$h->addParam("auth_return", GATEWAY_SUCCESS, "i");
					$h->addParam("transaction_payment_method", 0, "i");
					$h->addParam("transaction_code", "worldpay".$this->IPNData["transId"], "s");
					$h->addParam("transaction_notes", "", "s");
					$h->addParam("transaction_date_created", mktime(), "i");
					$h->addParam("transaction_amount", doubleval($this->IPNData["amount"]), "d");
					$h->addParam("currency_id", $this->IPNData["currency"], "s");
					$h->addParam("payment_gateway_id", $subscriptionInfo["payment_gateway_id"], "s");
					$h->execute();
					$transactionID = $h->getLastInsertID();
					dbg("inserted transaction: ".$transactionID);

					// get packages related to this subscription
					$h = new QueryMaster("subscription_package_rel.*", "packages.*");
					$h->addAndClause("subscription_id", "=", $subscriptionInfo["subscription_id"], "i");
					$relatedPackages = $h->getResults(-1, 0);
					dbg("Got related packages:");
					dbg($relatedPackages);
					// get package addons related to this subscription
					$h = new QueryMaster("subscription_package_addon_rel.*", "packages.*", "package_addons.*");
					$h->addAndClause("subscription_id", "=", $subscriptionInfo["subscription_id"], "i");
					$relatedPackageAddons = $h->getResults(-1, 0);
					dbg("Got related package addons:");
					dbg($relatedPackageAddons);

					$packageIDsToGet = array();
					foreach ($relatedPackages as $rp){
						$packageIDsToGet[] = $rp["package_id"];
					}
					foreach ($relatedPackageAddons as $rpa){
						$packageIDsToGet[] = $rpa["package_id"];
					}

					if (count($relatedPackages)){
						$clientID = $relatedPackages[0]["client_id"];
					} else {
						$clientID = $relatedPackageAddons[0]["client_id"];
					}
					dbg("got client id: ".$clientID);

					// need to get all of the unpaid invoices and their line items that correspond to the package ids we have
					$h = new QueryMaster("invoices.*", "invoice_line_items.*");
					$h->addAndClause("package_id", "=", $packageIDsToGet, "i");
					$h->addCustomAndClause("invoice_amount_paid < invoice_amount");
					$h->addOrderByColumns("invoice_date_due", "ASC");
					$allLineItems = $h->getResults(-1, 0);
					dbg("got all line items: ".count($allLineItems));

					// create an array of line item id's to check to see if they need to be paid.
					$lineItemIDs = array();
					foreach ($allLineItems as $li){
						$lineItemIDs[] = $li["line_item_id"];
					}

					// get all the payments made on these line items
					$h = new QueryMaster("transaction_success_rel.*");
					$h->addAndClause("line_item_id", "=", $lineItemIDs, "i");
					$payments = $h->getResults(-1, 0);
					dbg("got transaction success rows: ".count($payments));

					// add up the payments according to line item id and organize them as well
					$lineItemPayments = array();
					foreach ($payments as $pmnt){
						$lineItemPayments[$pmnt["line_item_id"]]["paid"] += $pmnt["line_item_payment_amount"];
						$lineItemPayments[$pmnt["line_item_id"]]["payments"][] = $pmnt;
					}

					// the invoice id we will work with, will be the first invoice for a package we are paying for
					// that has one that isn't paid
					$invoiceID = $allLineItems[0]["invoice_id"];
					$stop = false;

					// map the unpaid line items
					$unpaidLineItems = array();
					foreach ($allLineItems as $k=>$li){
						$allLineItems[$k]["paid"] = $lineItemPayments[$li["line_item_id"]]["paid"];
						if ($lineItemPayments[$li["line_item_id"]]["paid"]<$li["line_item_total_amount"] && $li["invoice_id"]==$invoiceID){
							$unpaidLineItems[] = $allLineItems[$k];
						}
					}
					dbg("figured out which ones weren't paid for: ".count($unpaidLineItems));

					// ok, got all the unpaid line items for the oldest invoice that has packages we are paying for
					// in an array called $unpaidLineItems, now we just need to loop through them and apply the money
					// towards them.
					$invoiceInfo = $unpaidLineItems[0]; // the invoice info was gotten in the line items query

					$amountAllowcated = 0;
					foreach ($unpaidLineItems as $k=>$li){
						$available = doubleval($this->IPNData["amount"])-$amountAllowcated;
						$amountNeeded = $li["line_item_total_amount"]-$li["paid"];
						if ($amountNeeded>$available){
							$payAmount = $available;
						} else {
							$payAmount = $amountNeeded;
						}
						$unpaidLineItems[$k]["applyAmount"] = $payAmount;
						$amountAllowcated += $payAmount;
					}
					$unallowcated = doubleval($this->IPNData["amount"])-$amountAllowcated;
					dbg("applied payments to line items, unallowcated: ".$unallowcated);


					// ok, so we made the amounts to pay, lets pay them
					foreach ($unpaidLineItems as $k=>$li){
						if ($li["applyAmount"]){
							$h = new QueryMaster("insert");
							$h->addTables("transaction_success_rel");
							$h->addParam("transaction_id", $transactionID, "i");
							$h->addParam("line_item_id", $li["line_item_id"], "i");
							$h->addParam("line_item_payment_amount", doubleval($li["applyAmount"]), "d");
							$h->execute();
							$invoiceInfo["amountPaidNow"] += doubleval($li["applyAmount"]);
							dbg("added payment for line item: ".$li["line_item_id"]." of ".doubleval($li["applyAmount"]));
						}
					}

					$h = new QueryMaster("update");
					$h->addTables("invoices");
					$h->addAndClause("invoice_id", "=", $invoiceID, "i");
					$h->addParam("invoice_amount_paid", round(doubleval($invoiceInfo["amountPaidNow"]+$invoiceInfo["invoice_amount_paid"]), 2), "d");
					if (round(($invoiceInfo["amountPaidNow"]+$invoiceInfo["invoice_amount_paid"]),2)==$invoiceInfo["invoice_amount"]){
						$h->addParam("invoice_status", 1, "i");
						dbg("invoice is closed");
					}
					$h->execute();
					dbg("updated invoice info");

					if ($unallowcated){
						$credit = $unallowcated;
						dbg("theres a credit amount");
					}

					// make the accounting entry
					require_once("lib-billing/include/accounting/Payment.php");
					$receipt = new Payment();
					$receipt->setClientID($clientID);
					$receipt->setInvoiceID($invoiceID);
					$receipt->setTransactionID($transactionID);
					$receipt->setDescription(TRANS_PAYPAL_PAYMENT_FOR_INVOICE);
					$receipt->setPaymentAmount(doubleval($this->IPNData["amount"]));
					if ($credit>0){
					$receipt->setClientCreditAmount($credit);
					}
					$receipt->setCurrencyID($this->IPNData["currency"]);
					$receipt->execute();
					dbg("made the acocunting entry");

					// associate the invoice with the transaction
					$h = new QueryMaster("insert");
					$h->addTables("invoice_transaction_rel");
					$h->addParam("invoice_id", $invoiceID, "i");
					$h->addParam("transaction_id", $transactionID, "i");
					$h->execute();
					dbg("set the invoice transaction rel record");

					// handle currency conversion if neccessary.
					$this->makeAccountingCurrenyConversion($this->IPNData["amount"], $this->IPNData["currency"], $this->IPNData["authAmount"], $this->IPNData["authCurrency"], $clientID, $invoiceID);

					$this->sendSecondaryPaymentEmail($clientID, $transactionID, $invoiceID);

				}

/**
* END applying to packages and addons
*/


			} else {

/**
* BEGIN applying payment to an invoice
* this will create a transaction to record the ipn, as well as creating the success ids for the
* line items that got paid, and changing the status of the invoice as is necessary.
*/

				$invoiceID = $this->IPNData["M_invoice"];
				if ( !(int)$invoiceID ) {
					dbg("invalid ipn return, bad invoice or config group id sent");
					return false;
				}

				if ($this->IPNData["transStatus"]=="N"){

					// lets go ahead and make the transaction
					$h = new QueryMaster("insert");
					$h->addTables("transaction_log");
					$h->addParam("auth_return", GATEWAY_DECLINED, "i");
					$h->addParam("transaction_payment_method", 0, "i");
					$h->addParam("transaction_code", "worldpay".$this->IPNData["transId"], "s");
					$h->addParam("transaction_notes", TRANS_PAYMENT_FOR_INVOICE.": ".$invoiceID, "s");
					$h->addParam("transaction_date_created", mktime(), "i");
					$h->addParam("transaction_amount", doubleval($this->IPNData["amount"]), "d");
					$h->addParam("currency_id", $this->IPNData["currency"], "s");
					$h->addParam("payment_gateway_id", $payment_gateway->id, "s");
					$h->execute();

				} else {

					dbg("applying IPN worldpay payment for invoice");

					// need to get all of the unpaid invoices and their line items that correspond to the package ids we have
					$h = new QueryMaster("invoices.*", "invoice_line_items.*");
					$h->addAndClause("invoices.invoice_id", "=", $invoiceID, "i");
					$allLineItems = $h->getResults(-1, 0);
					dbg("query: ".$h->getQuery());
					dbg("got all line items: ".count($allLineItems));

					// lets go ahead and make the transaction
					$h = new QueryMaster("insert");
					$h->addTables("transaction_log");
					$h->addParam("auth_return", GATEWAY_SUCCESS, "i");
					$h->addParam("transaction_payment_method", 0, "i");
					$h->addParam("transaction_code", "worldpay".$this->IPNData["transId"], "s");
					$h->addParam("transaction_notes", TRANS_PAYMENT_FOR_INVOICE.": ".$invoiceID, "s");
					$h->addParam("transaction_date_created", mktime(), "i");
					$h->addParam("transaction_amount", doubleval($this->IPNData["amount"]), "d");
					$h->addParam("currency_id", $this->IPNData["currency"], "s");
					$h->addParam("payment_gateway_id", $payment_gateway->id, "s");
					$h->execute();
					$transactionID = $h->getLastInsertID();
					dbg("inserted transaction: ".$transactionID);

					// create an array of line item id's to check to see if they need to be paid.
					$lineItemIDs = array();
					foreach ($allLineItems as $li){
						$packageID = $li["package_id"];
						$lineItemIDs[] = $li["line_item_id"];
					}

					$h = new QueryMaster("packages.client_id");
					$h->addAndClause("package_id", "=", $packageID, "i");
					$clientID = $h->getOne();

					// get all the payments made on these line items
					$h = new QueryMaster("transaction_success_rel.*");
					$h->addAndClause("line_item_id", "=", $lineItemIDs, "i");
					$payments = $h->getResults(-1, 0);
					dbg("got transaction success rows: ".count($payments));

					// add up the payments according to line item id and organize them as well
					$lineItemPayments = array();
					foreach ($payments as $pmnt){
						$lineItemPayments[$pmnt["line_item_id"]]["paid"] += $pmnt["line_item_payment_amount"];
						$lineItemPayments[$pmnt["line_item_id"]]["payments"][] = $pmnt;
					}

					// the invoice id we will work with, will be the first invoice for a package we are paying for
					// that has one that isn't paid

					// map the unpaid line items
					$unpaidLineItems = array();
					foreach ($allLineItems as $k=>$li){
						$allLineItems[$k]["paid"] = $lineItemPayments[$li["line_item_id"]]["paid"];
						if ($lineItemPayments[$li["line_item_id"]]["paid"]<$li["line_item_total_amount"]){
							$unpaidLineItems[] = $allLineItems[$k];
						}
					}
					dbg("figured out which ones weren't paid for: ".count($unpaidLineItems));

					// ok, got all the unpaid line items for the oldest invoice that has packages we are paying for
					// in an array called $unpaidLineItems, now we just need to loop through them and apply the money
					// towards them.
					$invoiceInfo = $unpaidLineItems[0]; // the invoice info was gotten in the line items query

					$amountAllowcated = 0;
					foreach ($unpaidLineItems as $k=>$li){
						$available = doubleval($this->IPNData["amount"])-$amountAllowcated;
						$amountNeeded = $li["line_item_total_amount"]-$li["paid"];
						if ($amountNeeded>$available){
							$payAmount = $available;
						} else {
							$payAmount = $amountNeeded;
						}
						$unpaidLineItems[$k]["applyAmount"] = $payAmount;
						$amountAllowcated += $payAmount;
					}
					$unallowcated = doubleval($this->IPNData["amount"])-$amountAllowcated;
					dbg("applied payments to line items, unallowcated: ".$unallowcated);


					// ok, so we made the amounts to pay, lets pay them
					foreach ($unpaidLineItems as $k=>$li){
						if ($li["applyAmount"]){
							$h = new QueryMaster("insert");
							$h->addTables("transaction_success_rel");
							$h->addParam("transaction_id", $transactionID, "i");
							$h->addParam("line_item_id", $li["line_item_id"], "i");
							$h->addParam("line_item_payment_amount", doubleval($li["applyAmount"]), "d");
							$h->execute();
							$invoiceInfo["amountPaidNow"] += doubleval($li["applyAmount"]);
							dbg("added payment for line item: ".$li["line_item_id"]." of ".doubleval($li["applyAmount"]));
						}
					}

					$h = new QueryMaster("update");
					$h->addTables("invoices");
					$h->addAndClause("invoice_id", "=", $invoiceID, "i");
					$h->addParam("invoice_amount_paid", round(doubleval($invoiceInfo["amountPaidNow"]+$invoiceInfo["invoice_amount_paid"]), 2), "d");
					if (round(($invoiceInfo["amountPaidNow"]+$invoiceInfo["invoice_amount_paid"]),2)==$invoiceInfo["invoice_amount"]){
						$h->addParam("invoice_status", 1, "i");
						dbg("invoice is closed");
					}
					$h->execute();
					dbg("updated invoice info");

					if ($unallowcated){
						$credit = $unallowcated;
						dbg("theres a credit amount");
					}

					// make the accounting entry
					require_once("lib-billing/include/accounting/Payment.php");
					$receipt = new Payment();
					$receipt->setClientID($clientID);
					$receipt->setInvoiceID($invoiceID);
					$receipt->setTransactionID($transactionID);
					$receipt->setDescription(TRANS_WORLDPAY_PAYMENT_FOR_INVOICE);
					$receipt->setPaymentAmount(doubleval($this->IPNData["amount"]));
					if ($credit>0){
						$receipt->setClientCreditAmount($credit);
					}
					$receipt->setCurrencyID($this->IPNData["currency"]);
					$receipt->execute();
					dbg("made the acocunting entry");

					// associate the invoice with the transaction
					$h = new QueryMaster("insert");
					$h->addTables("invoice_transaction_rel");
					$h->addParam("invoice_id", $invoiceID, "i");
					$h->addParam("transaction_id", $transactionID, "i");
					$h->execute();
					dbg("set the invoice transaction rel record");

					// handle currency conversion if neccessary.
					$this->makeAccountingCurrenyConversion($this->IPNData["amount"], $this->IPNData["currency"], $this->IPNData["authAmount"], $this->IPNData["authCurrency"], $clientID, $invoiceID);

					$this->sendSecondaryPaymentEmail($clientID, $transactionID, $invoiceID);
				}
/**
* END applying for invoice
*/


			}

		}
		// at this point the code should have handled everything neccessary already.... done...
		return true;
	}

	/**
	 *	take a fields array and create the html output fields needed
	 *
	 */
	private function createSubscriptionHTMLOutput($fields)
	{
	 	// init return variable
	 	$ret = array();

	 	if ( "button" == $this->config["Submit_Type"] || "" == $this->config["Subscribe_Image"] ) {
			$ret["paymentForm"] = "<form action='".$this->paymentURL."' method='post' target='_blank'>
				<input type='submit' value='".TRANS_SUBSCRIBE_NOW."' />";
		} else {
			$ret["paymentForm"] = "<form action='".$this->paymentURL."' method='post' target='_blank'>
				<input type='image' src='".$this->config["Subscribe_Image"]."' border='0' />";
		}

		$ret["paymentLinkURL"] = $this->paymentURL."?";

		foreach ($fields as $k=>$v){
		 	$ret["paymentLinkURL"] .= $k."=".urlencode($v)."&";
		 	$ret["paymentForm"] .= "<input type='hidden' name='".$k."' value='".$v."' />";
		}

		$ret["paymentForm"] .= "</form>";

		$ret["paymentLink"] = "<a href='".$ret["paymentLinkURL"]."' target='_blank'>".TRANS_PAY_WITH_WORLDPAY."</a>";

		return $ret;

	}

	/**
	 *	take a fields array and create the html output fields needed
	 *
	 */
	private function createOneTimeHTMLOutput($fields)
	{
	 	// init return variable
	 	$ret = array();

	 	if ( "button" == $this->config["Submit_Type"] || "" == $this->config["Pay_Image"] ) {
			$ret["paymentForm"] = "<form action='".$this->paymentURL."' method='post' target='_blank'>
				<input type='submit' value='".TRANS_PAY_NOW."' />";
		} else {
			$ret["paymentForm"] = "<form action='".$this->paymentURL."' method='post' target='_blank'>
				<input type='image' src='".$this->config["Pay_Image"]."' border='0' />";
		}

		$ret["paymentLinkURL"] = $this->paymentURL."?";

		foreach ($fields as $k=>$v){
		 	$ret["paymentLinkURL"] .= $k."=".urlencode($v)."&";
		 	$ret["paymentForm"] .= "<input type='hidden' name='".$k."' value='".$v."' />";
		}

		$ret["paymentForm"] .= "</form>";

		$ret["paymentLink"] = "<a href='".$ret["paymentLinkURL"]."' target='_blank'>".TRANS_PAY_WITH_WORLDPAY."</a>";

		return $ret;

	}

	/**
	 *	do what needs to be done when a subscription expires
	 */
	private function runSubscriptionCancellation()
	{
		// which is nothing at the moment.
	}

	/**
	 *	get the number of times this package is going to renew
	 */
	private function getNumRenewals($packagerow)
	{
		if ((int)$packagerow["package_date_stop_renewal"]<1){
			return -1;
		} else {
			$num = 0;
			$runningdate = $packagerow["package_date_next_renewal"];
			while ($runningdate<(int)$packagerow["package_date_stop_renewal"]){
				$next_runningdate = getNextBillingTimestamp($runningdate, $packagerow["cycle_id"]);
				$num++;
				if ( $next_runningdate == $runningdate ) {
					dbg("... package #{$packagerow['package_id']} has zero cycle time (cycle_id={$packagerow['cycle_id']}), break");
					break;
				}
				$runningdate = $next_runningdate;
			}
			return $num;
		}
	}

	/**
	 * get cycle unit and number
	 */
	private function translateCycleInfo($packagerow)
	{
		require_once mb_path("/lib-billing/include/config/internal/cycles.php");
	 	$cycleData = getCycleData($packagerow["cycle_id"]);
	 	list($sec,$min,$hour,$day,$month,$year) = $cycleData;

	 	// go through our cycle data and first check if they have more than one unit filled in
	 	// because paypal doesn't support multiple units
	 	$num = 0;
	 	foreach ($cycleData as $unit){
			if ($unit>0)$num++;
		}
		// if true this isn't a cycle supported by worldpay
		if ($num>1)return false;

		// init return
		$ret = array();

		// got past that check? ok, now to see what we have here...
		if ($day>0 && $day%7>0){// have a day value that isnt issued in whole weeks
			$ret["unit"] = "1";
			$ret["cycles"] = $day;
			return $ret;
		} elseif ($day>0){     // have a week based cycle expressed in days
			$ret["unit"] = "2";
			$ret["cycles"] = $day/7;
			return $ret;
		} elseif ($month>0){  // have a month based cycle
			$ret["unit"] = "3";
			$ret["cycles"] = $month;
			return $ret;
		} elseif ($year>0){  // have a year based cycle
			$ret["unit"] = "4";
			$ret["cycles"] = $year;
			return $ret;
		}

		// this isn't a cycle that is supported by worldpay
		return false;
	}

    /**
     * Calculates the taxes for this invoice amount
     *
     * @param amount    the amount to get taxes on
     * @param packageID  the package id we are getting tax zones for
     * @return the tax amount
     */
    private function calculateTaxes($amount, $packageID, $packageAddonID=0)
	{
		$taxZones = $packageAddonID ? $this->currentAddonTaxZones[$packageAddonID] : $this->currentTaxZones[$packageID];
		$tax = 0;
		$taxableAmount = $amount;
		for ($i = 0; $i < count($taxZones); $i++){
			$taxZone = $taxZones[$i];
			if (!(int)$taxZone["tax_zone_stackable"]) continue;
			dbg("applying tax zone:");
			dbg($taxZone);
			if ($taxZone["line_item_tax_amount_type"]==0){ // if 0 we use as percentage
				$cTax = round($amount*($taxZone["line_item_tax_amount"]/100), 2);
				$taxableAmount = round($taxableAmount + $cTax, 2);
				$tax = round($tax + $cTax, 2);
			} else { // else  we use as set amount
				$cTax = $taxZone["line_item_tax_amount"];
				$tax = round($tax + $cTax, 2);
			}
		}
		for ($i = 0; $i < count($taxZones); $i++){
			$taxZone = $taxZones[$i];
			if ((int)$taxZone["tax_zone_stackable"]) continue;
			dbg("applying tax zone:");
			dbg($taxZone);
			if ($taxZone["line_item_tax_amount_type"]==0){ // if 0 we use as percentage
				$cTax = round($taxableAmount*($taxZone["line_item_tax_amount"]/100), 2);
				$tax = round($tax + $cTax, 2);
			} else { // else  we use as set amount
				$cTax = $taxZone["line_item_tax_amount"];
				$tax = round($tax + $cTax, 2);
			}
		}
		return $tax;
    }
}
