<?php

class WWDAPIError extends Exception {}

/**
 * This is a class to make dealing with the Wild West Domains API easier.
 */
class WWDAPI
{
	const certify_wsdl	= 'https://api.ote.wildwestdomains.com/wswwdapi/wapi.asmx?wsdl';
	const production_wsdl	= 'https://api.wildwestdomains.com/wswwdapi/wapi.asmx?wsdl';

	const RESULT_CODE_FAILURE = 1001;

	public function __construct($account, $password)
	{
		$this->account	= $account;
		$this->password	= $password;
	}

	public function testMode()
	{
		$this->wsdl = self::certify_wsdl;
	}

	public function liveMode()
	{
		$this->wsdl = self::production_wsdl;
	}

	public function getErrors()
	{
		return $this->errors;
	}

	public function certifyAccount()
	{
		$this->testMode();
		// Do this every time to make sure we don't get stuck if it failed halfway through
		// last time.  We wouldn't have the ID's we need complete the later steps.
		if (false === $this->resetCertificationStatus()) {
			$this->errors[] = "Failed to reset certification status.";
			return false;
		}

		$sROID = '111';
		$exampleusRIID = '123';
		$examplebizRIID = '124';
		$pwd = 'abcde';
		$dbppwd = 'defgh';
		$joepwd = 'ghijk';
		$exampleusPrivacyRIID = 223;
		$examplebizPrivacyRIID = 224;

		/** TASK #1 */
		$domainCheck		= $this->checkAvailability(array('example.us', 'example.biz'));
		if (!$domainCheck) {
			$this->errors[] = 'Failed certification task #1: domain check';
			return false;
		}

		/** TASK #2 */
		$domainRegistration	= $this->registerDomain(array(
			'sROID' => $sROID,
			'user' => 'createNew',
			'pwd' => $pwd,
			'pwdhint' => 'certification',
			'email' => 'agordon@wildwestdomains.com',
			'firstname' => 'Artemus',
			'lastname' => 'Gordon',
			'phone' => '+1.8885551212',
			'nameservers' => array(
				'ns1.example.com',
				'ns2.example.com',
			),
			'domains' => array(
				array(
					'sld' => 'example',
					'tld' => 'us',
					'riid' => $exampleusRIID,
					'years' => 2,
					'nexus' => array(
						'category' => 'citizen of US',
						'use' => 'personal',
						'country' => 'us',
					),
				),
				array(
					'sld' => 'example',
					'tld' => 'biz',
					'riid' => $examplebizRIID,
					'years' => 2,
				),
			),
			'registrant' => array(
				'fname' => 'Artemus',
				'lname' => 'Gordon',
				'org' => '',
				'sa1' => '2 N. Main St.',
				'sa2' => '',
				'city' => 'Valdosta',
				'sp' => 'Georgia',
				'cc' => 'United States',
				'pc' => '17123',
				'email' => 'agordon@wildwestdomains.com',
				'phone' => '+1.8885551212',
				'fax' => '',
			),
			'admin' => array(
				'fname' => 'Artemus',
				'lname' => 'Gordon',
				'org' => '',
				'sa1' => '2 N. Main St.',
				'sa2' => '',
				'city' => 'Valdosta',
				'sp' => 'Georgia',
				'cc' => 'United States',
				'pc' => '17123',
				'email' => 'agordon@wildwestdomains.com',
				'phone' => '+1.8885551212',
				'fax' => '',
			),
			'billing' => array(
				'fname' => 'Artemus',
				'lname' => 'Gordon',
				'org' => '',
				'sa1' => '2 N. Main St.',
				'sa2' => '',
				'city' => 'Valdosta',
				'sp' => 'Georgia',
				'cc' => 'United States',
				'pc' => '17123',
				'email' => 'agordon@wildwestdomains.com',
				'phone' => '+1.8885551212',
				'fax' => '',
			),
			'tech' => array(
				'fname' => 'Artemus',
				'lname' => 'Gordon',
				'org' => '',
				'sa1' => '2 N. Main St.',
				'sa2' => '',
				'city' => 'Valdosta',
				'sp' => 'Georgia',
				'cc' => 'United States',
				'pc' => '17123',
				'email' => 'agordon@wildwestdomains.com',
				'phone' => '+1.8885551212',
				'fax' => '',
			),
		));

		// there was an error of some sort
		// we would have better error handling here but this is just certification
		if ($domainRegistration===false) {
			$this->errors[] = 'Failed certification task #2: domain registration';
			return false;
		}

		// grab the needed information
		$userID = $domainRegistration['user'];
		$orderID = $domainRegistration['orderid'];

		/** TASK #3 */
		$pollCheck		= $this->poll();
		if (!$pollCheck) {
			$this->errors[] = 'Failed certification task #3: account polling';
			return false;
		}
		$exampleusDomainID = $pollCheck[$exampleusRIID];
		$examplebizDomainID = $pollCheck[$examplebizRIID];

		$privacyCheck		= $this->orderDomainPrivacy(array(
			'user' => $userID,
			'dbppwd' => $dbppwd,
			'dbpemail' => 'info@example.biz',
			'sROID' => '112',
			'domains' => array(
				array(
					'sld' => 'example',
					'tld' => 'biz',
					'resourceid' => $examplebizDomainID,
					'riid' => $examplebizPrivacyRIID,
				),
			),
		));
		if (!$privacyCheck) {
			$this->errors[] = 'Failed certification task #3: purchase domain privacy';
			return false;
		}
		$dbpuserID = $privacyCheck['dbpuser'];
		$dbporderID = $privactCheck['orderid'];

		/** TASK #4 */
		$domainCheck		= $this->checkAvailability(array('example.us', 'example.biz'));
		if (!$domainCheck) {
			$this->errors[] = 'Failed certification task #4: second domain check';
			return false;
		}

		/** TASK #5 */
		$domainInfoCheck	= $this->domainInfo($examplebizDomainID);
		if (!$domainInfoCheck) {
			$this->errors[] = 'Failed certification task #5: fetch domain information';
			return false;
		}

		/** TASK #6 */
		// poll for the privacy resourceid
		$pollCheck		= $this->poll();
		$privacyResourceID = $pollCheck[$examplebizPrivacyRIID];
		$renewProductsCheck	= $this->renewDomain(array(
			'user' => $userID,
			'dbpuser' => $dbpuserID,
			'dbppwd' => $dbppwd,
			'sROID' => '112',
			'domains' => array(
				array(
					'sld' => 'example',
					'tld' => 'us',
					'resourceid' => $exampleusDomainID,
					'riid' => $exampleusRIID,
				),
				array(
					'sld' => 'example',
					'tld' => 'biz',
					'resourceid' => $examplebizDomainID,
					'riid' => $examplebizRIID,
				),
				array(
					'sld' => 'example',
					'tld' => 'biz',
					'resourceid' => $privacyResourceID,
					'riid' => $examplebizPrivacyRIID,
				),
			),
		));
		if (!$renewProductsCheck) {
			$this->errors[] = 'Failed certification task #6: domain renewal';
			return false;
		}

		/** TASK #7 */
		$transferCheck		= $this->transferDomain(array(
			'sROID' => $sROID,
			'user' => 'createNew',
			'pwd' => $joepwd,
			'pwdhint' => 'certification',
			'email' => 'joe@smith.us',
			'firstname' => 'Joe',
			'lastname' => 'Smith',
			'phone' => '+1.7775551212',
			'nameservers' => array(
				'ns1.example.com',
				'ns2.example.com',
			),
			'domains' => array(
				array(
					'sld' => 'example',
					'tld' => 'com',
					'riid' => '125',
				),
			),
			'registrant' => array(
				'fname' => 'Joe',
				'lname' => 'Smith',
				'org' => '',
				'sa1' => '1 S. Main St.',
				'sa2' => '',
				'city' => 'Oakland',
				'sp' => 'California',
				'cc' => 'United States',
				'pc' => '97123',
				'email' => 'joe@smith.us',
				'phone' => '+1.7775551212',
				'fax' => '',
			),
			'admin' => array(
				'fname' => 'Joe',
				'lname' => 'Smith',
				'org' => '',
				'sa1' => '1 S. Main St.',
				'sa2' => '',
				'city' => 'Oakland',
				'sp' => 'California',
				'cc' => 'United States',
				'pc' => '97123',
				'email' => 'joe@smith.us',
				'phone' => '+1.7775551212',
				'fax' => '',
			),
			'billing' => array(
				'fname' => 'Joe',
				'lname' => 'Smith',
				'org' => '',
				'sa1' => '1 S. Main St.',
				'sa2' => '',
				'city' => 'Oakland',
				'sp' => 'California',
				'cc' => 'United States',
				'pc' => '97123',
				'email' => 'joe@smith.us',
				'phone' => '+1.7775551212',
				'fax' => '',
			),
			'tech' => array(
				'fname' => 'Joe',
				'lname' => 'Smith',
				'org' => '',
				'sa1' => '1 S. Main St.',
				'sa2' => '',
				'city' => 'Oakland',
				'sp' => 'California',
				'cc' => 'United States',
				'pc' => '97123',
				'email' => 'joe@smith.us',
				'phone' => '+1.7775551212',
				'fax' => '',
			),
		));

		if ($transferCheck) {
			unset($this->client);
			$this->liveMode();
			return true;
		} else {
			$this->errors[] = 'Failed certification task #7: domain transfer';
			return false;
		}
	}

	/**
	 * The following functions build the data to pass to the WWD API Soap calls.
	 */

	/**
	 * checks for the availability and backorder-ability of any number of domains
	 */
	public function checkAvailability($domains)
	{
		$domains = is_array($domains)?$domains:array($domains);
		$params = array(
			'sCLTRID' => 'PB'.microtime(1),
			'credential' => array(
				'Account' => $this->account,
				'Password' => $this->password,
			),
			'sDomainArray' => $domains,
		);
		try {
			$availability = $this->getSoapClient()->CheckAvailability($params);
		} catch (Exception $e) {
			$this->errors[] = $e->getMessage();
			return false;
		}
		if (is_array($availability) && is_array($availability['check'][0]) && is_array($availability['check'][0]['domain'][0])) {
			$ret = array();
			foreach ($availability['check'][0]['domain'] as $d) {
				$result = array(
					'name' => $d['_attributes']['name'],
					'available' => $d['_attributes']['avail'],
					'canBackorder' => $d['_attributes']['canBackorder'],
				);
				if (isset($d['error'])) {
					$result['error']['code'] = $d['error'][0]["_attributes"]["code"];
					$result['error']['desc'] = $d['error'][0]["_attributes"]["desc"];
				}
				$ret[] = $result;
			}
			return $ret;
		} else {
			if (is_array($availability['response']) && is_array($availability['response'][0]['result']) && is_array($availability['response'][0]['result'][0]['msg'])) {
				$this->errors[] = $availability['response'][0]['result'][0]['msg'][0];
			}
			return false;
		}
	}

	/**
	 * registers any number of domains
	 */
	public function registerDomain($input=array())
	{
		$params = array(
			'sCLTRID' => 'PB'.microtime(1),
			'credential' => array(
				'Account' => $this->account,
				'Password' => $this->password,
			),
			'shopper' => array(
				'user' => $input['user'],
				'pwd' => $input['pwd'],
				'pwdhint' => $input['pwdhint'],
				'email' => $input['email'],
				'firstname' => $input['firstname'],
				'lastname' => $input['lastname'],
				'phone' => $input['phone'],
			),
			'items' => array(
				'DomainRegistration' => array(),
			),
			'sROID' => $input['sROID'],
		);

		try {
			foreach ($input['domains'] as $domain) {
				$domain['years'] = $domain['years']?$domain['years']:($input['years']?$input['years']:1);
				$item = array(
					'sld' => $domain['sld'],
					'tld' => $domain['tld'],
					'period' => $domain['years'],
					'autorenewflag' => (int)$domain['autorenew'],
					'order' => array(
						'productid' => $this->getProductID('new', strtolower($domain['tld']), $domain['years']),
						'quantity' => $domain['quantity']?$domain['quantity']:1,
						'riid' => $domain['riid'],
						'duration' => $domain['years'],
					),
					'registrant' => array(),
					'nsArray' => array(
						'NS' => array(),
					),
				);
				// extra info for certain tlds
				if ($domain['nexus']) {
					$item['nexus'] = $domain['nexus'];
				}

				if ($domain['nameservers'] = isset($domain['nameservers'])?$domain['nameservers']:$input['nameservers']) {
					foreach ($domain['nameservers'] as $ns) {
						$item['nsArray']['NS'][] = array(
							'name' => $ns,
						);
					}
				}

				$domain['registrant']	= $this->cleanContactData($domain['registrant']	?$domain['registrant']	:$input['registrant']);
				$domain['admin']	= $this->cleanContactData($domain['admin']	?$domain['admin']	:$input['admin']);
				$domain['billing']	= $this->cleanContactData($domain['billing']	?$domain['billing']	:$input['billing']);
				$domain['tech']		= $this->cleanContactData($domain['tech']	?$domain['tech']	:$input['tech']);
				// we do this to fill even fields that were not given...
				foreach (array('registrant', 'admin', 'billing', 'tech') as $ctype) {
					$item[$ctype] = array();
					foreach (array('fname', 'lname', 'org', 'email', 'sa1', 'sa2', 'city', 'sp', 'pc', 'cc', 'phone', 'fax') as $field) {
						$item[$ctype][$field] = $domain[$ctype][$field];
					}
				}
				$params['items']['DomainRegistration'][] = $item;
			}

			$register = $this->getSoapClient()->OrderDomains($params);
			if (!is_array($register['response'][0]) || !is_array($register['response'][0]['_attributes']) || $register['response'][0]['result'][0]['_attributes']['code']!=1000) {
				$this->errors[] = $register['response'][0]['result'][0]['msg'][0];
				$this->errors[] = $register['response'][0]['resdata'][0]['domainRegistration'][0]['error'][0]['_attributes']['code'].' - '.$register['response'][0]['resdata'][0]['domainRegistration'][0]['error'][0]['_attributes']['desc'];
				dbg($register);
				return false;
			}
			return array(
				'user' => $register['response'][0]['_attributes']['user'],
				'orderid' => $register['response'][0]['resdata'][0]['orderid'][0],
			);
		} catch (Exception $e) {
			$this->errors[] = $e->getMessage();
			return false;
		}
	}

	/**
	 * fetches notifications from WWD so that you can pair domains/other resources
	 * to their respective resource ids
	 */
	public function poll($sOp='req')
	{
		$params = array(
			'sCLTRID' => 'PB'.microtime(1),
			'credential' => array(
				'Account' => $this->account,
				'Password' => $this->password,
			),
			'sOp' => $sOp,
		);
		try {
			$poll = $this->getSoapClient()->Poll($params);
			$ret = array();
			if (!is_array($poll['response'][0]) ||
			!is_array($poll['response'][0]['resdata'][0]) ||
			!is_array($poll['response'][0]['resdata'][0]['REPORT'][0]) ||
			!is_array($poll['response'][0]['resdata'][0]['REPORT'][0]['ITEM'][0])) {
				// We didn't find any notifications
				dbg('no notifications:');
				dbg($poll);
				return array();
			} else {
				foreach ($poll['response'][0]['resdata'][0]['REPORT'][0]['ITEM'] as $item) {
					if ($item['_attributes']['riid']) {
						$ret[$item['_attributes']['riid']] = $item['_attributes']['resourceid'];
					}
				}
			}
			return $ret;
		} catch (Exception $e) {
			$this->errors[] = $e->getMessage();
			return false;
		}
	}

	/**
	 * adds idprotect to any domain controlled by this reseller
	 */
	public function orderDomainPrivacy($input)
	{
		$params = array(
			'sCLTRID' => 'PB'.microtime(1),
			'credential' => array(
				'Account' => $this->account,
				'Password' => $this->password,
			),
			'shopper' => array(
				'user' => $input['user'],
				'dbpuser' => $input['dbpuser']?$input['dbpuser']:'createNew',
				'dbppwd' => $input['dbppwd'],
				'dbpemail' => $input['dbpemail'],
			),
			'items' => array(
				'DomainByProxy' => array(),
			),
			'sROID' => $input['sROID'],
		);

		foreach ($input['domains'] as $domain) {
			$item = array(
				'order' => array(
					'productid' => $this->productIDs['new']['idprotect'],
					'quantity' => $domain['quantity']?$domain['quantity']:1,
					'riid' => $domain['riid'],
					'duration' => $domain['years']?$domain['years']:1,
				),
				'sld' => $domain['sld'],
				'tld' => $domain['tld'],
			);
			if (!empty($domain['resourceid']))
				$item['resourceid'] = $domain['resourceid'];

			$params['items']['DomainByProxy'][] = $item;
		}

		try {
			$privacy = $this->getSoapClient()->OrderDomainPrivacy($params);
			if (!is_array($privacy['response'][0]) || !is_array($privacy['response'][0]['_attributes']) || $privacy['response'][0]['result'][0]['_attributes']['code']!=1000) {
				// there was an error of some sort
				// add better error handling here
				dbg($privacy);
				return false;
			}
			return array(
				'user' => $privacy['response'][0]['_attributes']['user'],
				'dbpuser' => $privacy['response'][0]['_attributes']['dbpuser'],
				'orderid' => $privacy['response'][0]['resdata'][0]['orderid'][0],
			);
		} catch (Exception $e) {
			$this->errors[] = $e->getMessage();
			return false;
		}
	}

	/**
	 * gets information about any domain controlled by this reseller
	 */
	public function domainInfo($domain, $type=false)
	{
		$params = array(
			'sCLTRID' => 'PB'.microtime(1),
			'credential' => array(
				'Account' => $this->account,
				'Password' => $this->password,
			),
			'sResourceID' => $resourceid,
		);
		if (strpos($domain, ':')) {
			$params['sResourceID'] = $domain;
		} else {
			$params['sDomain'] = $domain;
		}
		if ($type) {
			$params['sType'] = $type;
		}
		try {
			$info = $this->getSoapClient()->Info($params);
			return $info;
		} catch (Exception $e) {
			$this->errors[] = $e->getMessage();
			return false;
		}
	}

	/**
	 * a function to get the domain contact data
	 */
	public function domainContactInfo($domain)
	{
		$data = $this->domainInfo($domain, 'contacts');
		if (is_array($data['info']) && is_array($data['info'][0]['_attributes'])) {
			$contactDataXML = utf8_html_entity_decode($data['info'][0]['_attributes']['contacts']);
			$x = new MBXMLLite($contactDataXML);
			$contactData = $x->toArray();
			if (is_array($contactData['contacts']) && is_array($contactData['contacts'][0]['contact'])) {
				$returnData = array(
					'registrant' => array(),
					'admin' => array(),
					'billing' => array(),
					'tech' => array(),
				);
				$countryNameToCode = array_flip($this->countries);
				foreach ((array)$contactData['contacts'][0]['contact'] as $contact) {
					if (isset($contact['_attributes']['contactType'])) {
						$type = $this->contactTypes[(int)$contact['_attributes']['contactType']];
						$returnData[$type] = $contact['_attributes'];
						// set the codes here so it matches back up to our stuff
						if ("" != $returnData[$type]['spcode']) {
							$returnData[$type]['sp'] = strtoupper(trim($returnData[$type]['spcode']));
						}
						if (isset($countryNameToCode[$returnData[$type]['cc']])) {
							$returnData[$type]['cc'] = $countryNameToCode[$returnData[$type]['cc']];
						} else if ("" != trim($returnData[$type]['cccode'])) {
							$returnData[$type]['cc'] = strtoupper(trim($returnData[$type]['cccode']));
						}
						$returnData[$type]['phone'] = str_replace('+', '', $returnData[$type]['phone']);
						$returnData[$type]['fax'] = str_replace('+', '', $returnData[$type]['fax']);
						unset($returnData[$type]['spcode']);
						unset($returnData[$type]['cccode']);
						unset($returnData[$type]['contactType']);
					}
				}
				return $returnData;
			}
			$this->errors[] = 'Unable to parse domain contact data for domain: '.$domain;
			return false;
		} else {
			$this->errors[] = 'Unable to fetch domain contact data for domain: '.$domain;
			return false;
		}
	}

	/**
	 * set the whois contact info for a domain
	 */
	public function setDomainContactInfo($input)
	{
		$params = array(
			'sCLTRID' => 'PB'.microtime(1),
			'credential' => array(
				'Account' => $this->account,
				'Password' => $this->password,
			),
			'domainArray' => array(),
			'sROID' => $input['sROID'],
		);

		if (!$input['resourceid'] && $input['sld'] && $input['tld']) {
			try {
				$input['resourceid'] = $this->getDomainResourceID($input['sld'], $input['tld']);
				if ($input['resourceid'] === false)
					return false;
			} catch (Exception $e) {
				dbg('Error looking up resourceid for domain: '.$input['sld'].'.'.$input['tld']);
				$this->errors[] = $e->getMessage();
				return false;
			}
		}

		$params['domainArray']['Domain'][] = array(
			'resourceid' => $input['resourceid'],
			'mngTRID' => $input['resourceid'],
		);

		$params['registrant']	= $this->cleanContactData($input['registrant']);
		$params['admin']	= $this->cleanContactData($input['admin']);
		$params['billing']	= $this->cleanContactData($input['billing']);
		$params['tech']		= $this->cleanContactData($input['tech']);

		try {
			$setContactData = $this->getSoapClient()->UpdateDomainContact($params);
			if (!is_array($setContactData['response'][0]) || $setContactData['response'][0]['result'][0]['_attributes']['code']!=1000) {
				$this->errors[] = $setContactData['response'][0]['result'][0]['msg'][0];
				dbg($setContactData);
				return false;
			}
			return $setContactData;
		} catch (Exception $e) {
			$this->errors[] = $e->getMessage();
			return false;
		}
	}

	/**
	 * set the domain lock parameter for a domain
	 */
	public function setDomainLock($input)
	{
		$params = array(
			'sCLTRID' => 'PB'.microtime(1),
			'credential' => array(
				'Account' => $this->account,
				'Password' => $this->password,
			),
			'domainArray' => array(),
			'sLock' => 'no',
			'sROID' => $input['sROID'],
		);

		if (!$input['resourceid'] && $input['sld'] && $input['tld']) {
			try {
				$input['resourceid'] = $this->getDomainResourceID($input['sld'], $input['tld']);
				if ($input['resourceid'] === false)
					return false;
			} catch (Exception $e) {
				dbg('Error looking up resourceid for domain: '.$input['sld'].'.'.$input['tld']);
				$this->errors[] = $e->getMessage();
				return false;
			}
		}

		$params['domainArray']['Domain'][] = array(
			'resourceid' => $input['resourceid'],
			'mngTRID' => $input['resourceid'],
		);

		if (in_array(strtolower($input['lock']), array(
			true,
			'true',
			'yes',
			1,
			'1',
		))) {
			$params['sLock'] = 'yes';
		}

		try {
			$lock = $this->getSoapClient()->SetDomainLocking($params);
			if (!is_array($lock['response'][0]) || $lock['response'][0]['result'][0]['_attributes']['code']!=1000) {
				$this->errors[] = $lock['response'][0]['result'][0]['msg'][0];
				dbg($lock);
				return false;
			}
			return $lock;
		} catch (Exception $e) {
			$this->errors[] = $e->getMessage();
			return false;
		}
	}

	/**
	 * processes renewals for any domain controlled by this reseller, and also for
	 * other resources like idprotect (must be renewed at same time as domain it is for)
	 */
	public function renewDomain($input)
	{
		$params = array(
			'sCLTRID' => 'PB'.microtime(1),
			'credential' => array(
				'Account' => $this->account,
				'Password' => $this->password,
			),
			'shopper' => array(
				'user' => $input['user'],
			),
			'items' => array(
				'DomainRenewal' => array(),
			),
			'sROID' => $input['sROID'],
		);

		$addDbp = false;

		try {
			foreach ((array)$input['domains'] as $i=>$domain) {
				dbg($domain);
				if (!$domain['resourceid'] && $domain['sld'] && $domain['tld']) {
					try {
						$domain['resourceid'] = $this->getDomainResourceID($domain['sld'], $domain['tld']);
						if ($domain['resourceid'] === false)
							return false;
					} catch (Exception $e) {
						dbg('Error looking up resourceid for domain: '.$domain['sld'].'.'.$domain['tld']);
						$this->errors[] = $e->getMessage();
						return false;
					}
				} else {
					// need to figure out a way to look up privacy resource id's if needed
				}
				$domain['years'] = $domain['years']?$domain['years']:($input['years']?$input['years']:1);
				if (substr($domain['resourceid'], 0, 3)=='dbp') {
					$addDbp = true;
					$prodID = $this->productIDs['renew']['idprotect'];
					$item = array(
						'order' => array(
							'productid' => $prodID,
							'quantity' => $domain['quantity']?$domain['quantity']:1,
							'riid' => $domain['riid'],
							'duration' => $domain['years']?$domain['years']:1,
						),
						'resourceid' => $domain['resourceid'],
					);
					if (!$params['dbpItems']) {
						$params['dbpItems'] = array(
							'ResourceRenewal' => array(
								$item,
							),
						);
					} else {
						$params['items']['ResourceRenewal'][] = $item;
					}
				} else {
					$prodID = $this->getProductID('renew', strtolower($domain['tld']), $domain['years']);
					$item = array(
						'order' => array(
							'productid' => $prodID,
							'quantity' => $domain['quantity']?$domain['quantity']:1,
							'riid' => $domain['riid'],
							'duration' => $domain['years']?$domain['years']:1,
						),
						'sld' => $domain['sld'],
						'tld' => $domain['tld'],
						'resourceid' => $domain['resourceid'],
						'period' => $domain['years']?$domain['years']:1,
					);
					$params['items']['DomainRenewal'][] = $item;
				}
			}

			if ($addDbp) {
				$params['shopper']['dbpuser'] = $input['dbpuser'];
				$params['shopper']['dbppwd'] = $input['dbppwd'];
			}

			if ($addDbp) {
				$renewals = $this->getSoapClient()->OrderPrivateDomainRenewals($params);
			} else {
				$renewals = $this->getSoapClient()->OrderDomainRenewals($params);
			}
			if (!is_array($renewals['response'][0]) || !is_array($renewals['response'][0]['_attributes']) || $renewals['response'][0]['result'][0]['_attributes']['code']!=1000) {
				// there was an error of some sort
				// add better error handling here
				dbg($renewals);
				return false;
			}
			return array(
				'user' => $renewals['response'][0]['_attributes']['user'],
				'orderid' => $renewals['response'][0]['resdata'][0]['orderid'][0],
			);
		} catch (Exception $e) {
			$this->errors[] = $e->getMessage();
			return false;
		}
		return $renewals;
	}

	/**
	 * transfers domains to a user under this WWD reseller account
	 */
	public function transferDomain($input=array())
	{
		$params = array(
			'sCLTRID' => 'PB'.microtime(1),
			'credential' => array(
				'Account' => $this->account,
				'Password' => $this->password,
			),
			'shopper' => array(
				'user' => $input['user'],
				'pwd' => $input['pwd'],
				'pwdhint' => $input['pwdhint'],
				'email' => $input['email'],
				'firstname' => $input['firstname'],
				'lastname' => $input['lastname'],
				'phone' => $input['phone'],
			),
			'items' => array(
				'DomainTransfer' => array(),
			),
			'sROID' => $input['sROID'],
		);

		try {
			foreach ($input['domains'] as $domain) {
				$domain['years'] = $domain['years']?$domain['years']:($input['years']?$input['years']:1);
				$item = array(
					'sld' => $domain['sld'],
					'tld' => $domain['tld'],
					'authInfo' => $domain['authInfo'],
					'autorenewflag' => (int)$domain['autorenew'],
					'order' => array(
						'productid' => $this->getTransferProductID(strtolower($domain['tld'])),
						'quantity' => $domain['quantity']?$domain['quantity']:1,
						'riid' => $domain['riid'],
						'duration' => 1,
					),
					'registrant' => array(),
					'nsArray' => array(
						'NS' => array(),
					),
				);
				// extra info for certain tlds
				if ($domain['nexus']) {
					$item['nexus'] = $domain['nexus'];
				}

				if ($domain['nameservers'] = isset($domain['nameservers'])?$domain['nameservers']:$input['nameservers']) {
					foreach ($domain['nameservers'] as $ns) {
						$item['nsArray']['NS'][] = array(
							'name' => $ns,
						);
					}
				}

				$domain['registrant']	= $this->cleanContactData($domain['registrant']	?$domain['registrant']	:$input['registrant']);
				$domain['admin']	= $this->cleanContactData($domain['admin']	?$domain['admin']	:$input['admin']);
				$domain['billing']	= $this->cleanContactData($domain['billing']	?$domain['billing']	:$input['billing']);
				$domain['tech']		= $this->cleanContactData($domain['tech']	?$domain['tech']	:$input['tech']);
				// we do this to fill even fields that were not given...
				foreach (array('registrant', 'admin', 'billing', 'tech') as $ctype) {
					$item[$ctype] = array();
					foreach (array('fname', 'lname', 'org', 'email', 'sa1', 'sa2', 'city', 'sp', 'pc', 'cc', 'phone', 'fax') as $field) {
						$item[$ctype][$field] = $domain[$ctype][$field];
					}
				}
				$params['items']['DomainTransfer'][] = $item;
			}

			$transfer = $this->getSoapClient()->OrderDomainTransfers($params);
			if (!is_array($transfer['response'][0]) || !is_array($transfer['response'][0]['_attributes']) || $transfer['response'][0]['result'][0]['_attributes']['code']!=1000) {
				// there was an error of some sort
				// add better error handling here
				dbg($transfer);
				return false;
			}
			return array(
				'user' => $transfer['response'][0]['_attributes']['user'],
				'orderid' => $transfer['response'][0]['resdata'][0]['orderid'][0],
			);
		} catch (Exception $e) {
			$this->errors[] = $e->getMessage();
			return false;
		}
	}

	function setDNS($input=array(), $doNotRemoveExisting=false)
	{
		if (isset($input['nameservers'])) {
			$params = array(
				'sCLTRID' => 'PB'.microtime(1),
				'credential' => array(
					'Account' => $this->account,
					'Password' => $this->password,
				),
				'domainArray' => array(
					'Domain' => array(),
				),
				'nsArray' => array(
					'NS' => array(),
				),
			);
			foreach ((array)$input['nameservers'] as $ns) {
				$params['nsArray']['NS'][] = array(
					'name' => $ns,
				);
			}
			try {
				if (is_array($input['domains'])) {
					foreach ((array)$input['domains'] as $i=>$domain) {
						if (!$domain['resourceid'] && $domain['sld'] && $domain['tld']) {
							try {
								$domain['resourceid'] = $this->getDomainResourceID($domain['sld'], $domain['tld']);
								if ($domain['resourceid'] === false)
									return false;
							} catch (Exception $e) {
								dbg('Error looking up resourceid for domain: '.$domain['sld'].'.'.$domain['tld']);
								$this->errors[] = $e->getMessage();
								return false;
							}
						} else {
							$this->errors[] = 'No resourceid was provided and the sld and tld were also missing.  The resourceid or the sld AND tld must be present.';
							return false;
						}
						$prodID = $this->getProductID('renew', strtolower($domain['tld']), $domain['years']);
						$item = array(
							'resourceid' => $domain['resourceid'],
							'mngTRID' => $domain['resourceid'],
						);
						$params['domainArray']['Domain'][] = $item;
					}
				} elseif ((isset($input['sld']) && isset($input['tld'])) || isset($input['resourceid'])){
					if (!isset($input['resourceid'])) {
						try {
							$input['resourceid'] = $this->getDomainResourceID($input['sld'], $input['tld']);
							if ($input['resourceid'] === false)
								return false;
						} catch (Exception $e) {
							dbg('Error looking up resourceid for domain: '.$input['sld'].'.'.$input['tld']);
							$this->errors[] = $e->getMessage();
							return false;
						}
					}
					$params['domainArray']['Domain'][] = array(
						'resourceid' => $input['resourceid'],
						'mngTRID' => $input['resourceid'],
					);
				}
				$update = $this->getSoapClient()->UpdateNameServer($params);
				return $update;
			} catch (Exception $e) {
				$this->errors[] = $e->getMessage();
				return false;
			}

		} else if (isset($input['records'])) {
			$params = array(
				'sCLTRID' => 'PB'.microtime(1),
				'credential' => array(
					'Account' => $this->account,
					'Password' => $this->password,
				),
				'dnsRequestArray' => array(
					'DNSRequest' => array(),
				),
				'sDomain' => $input['sld'].'.'.$input['tld'],
			);
			if ($doNotRemoveExisting==false) {
				// set up an array with keys that uniquely identify a resource record
				$toBeSet = array();
				foreach ($input['records'] as $x=>$r) {
					$resourceRecordKey = strtolower($r['recType'].'|'.trim($r['key']).'|'.trim($r['recValue']));
					// key = the resource record's unique key
					// value = the array index of the resource record row
					//		in the $input['records'] array
					//		this is used to 'unset' it later if needed
					$toBeSet[$resourceRecordKey] = $x;
				}

				// go through each of the current dns resource records to see if we need to remove them
				$currentDNSRecords = $this->domainInfo($input['sld'].'.'.$input['tld'], 'dns');
				if (false === $currentDNSRecords) {
					return false;
				}
				if (is_array($currentDNSRecords['info']) && is_array($currentDNSRecords['info'][0]['dns']) && is_array($currentDNSRecords['info'][0]['dns'][0]['rec'])) {
					$records = $currentDNSRecords['info'][0]['dns'][0]['rec'];
					foreach ((array)$records as $rec) {
						// if this is a valid resource record return from the api....
						if (is_array($rec['_attributes']) && isset($rec['_attributes']['type'])) {
							$resourceRecordKey = strtolower($rec['_attributes']['type'].'|'.trim($rec['_attributes']['key']).'|'.trim($rec['_attributes']['val']));
							// if this resource record's unique key (of form: type|hostname|value) is NOT
							// in our list of keys we are going to set, then add it to the remove list
							if (!isset($toBeSet[$resourceRecordKey])) {
								$params['dnsRequestArray']['DNSRequest'][] = array(
									'action' => 'remove',
									'ttl' => 3600,
									'key' => $rec['_attributes']['key'],
									'recType' => $rec['_attributes']['type'],
									'recValue' => $rec['_attributes']['val'],
								);
							} else {
								// if it IS in the list we are going to set, then remove it because
								// it is already on this domain and we do not need to add it again
								unset($input['records'][$toBeSet[$resourceRecordKey]]);
							}
						}
					}
				}
			}
			foreach ($input['records'] as $r) {
				$record = array(
					'action' => 'set', // default
					'ttl' => 3600, // default
				);
				foreach (array('recType', 'action', 'key', 'ttl', 'recValue') as $f) {
					if ($r[$f]) $record[$f] = $r[$f];
				}
				$params['dnsRequestArray']['DNSRequest'][] = $record;
			}
			try {
				$update = $this->getSoapClient()->ModifyDNS($params);
				return $update;
			} catch (Exception $e) {
				$this->errors[] = $e->getMessage();
				return false;
			}
		} else {
			$this->errors[] = 'You must either set nameservers or dns records.';
			return false;
		}
	}


	/**
	 * this resets the certification status for a WWD account
	 */
	private function resetCertificationStatus()
	{
		try {
			$reset = $this->getSoapClient()->ProcessRequest(array(
				'sRequestXML' => '<wapi clTRID=\'PLESKBILLING'.time().'\' account=\''.$this->account.'\' pwd=\''.$this->password.'\'><manage><script cmd=\'reset\' /></manage></wapi>',
			));
		} catch (Exception $e) {
			$this->errors[] = $e->getMessage();
			return false;
		}
		return $reset;
	}

	/**
	 */
	function getDomainResourceID($sld, $tld)
	{
		$info = $this->domainInfo($sld.'.'.$tld);
		if (is_array($info['info']) && is_array($info['info'][0]['_attributes']) && isset($info['info'][0]['_attributes']['resourceid'])) {
			return $input['domains'][$i]['resourceid'] = $info['info'][0]['_attributes']['resourceid'];
		} else {
			// we were unable to find the resourceid for some reason.
			dbg('Error looking up resourceid for domain: '.$sld.'.'.$tld);
			dbg($info);
			if (is_array($info['response']) && is_array($info['response'][0]['resdata']) && is_array($info['response'][0]['resdata'][0]['error']) && is_array($info['response'][0]['resdata'][0]['error'][0]['_attributes'])) {
				$this->errors[] = $info['response'][0]['resdata'][0]['error'][0]['_attributes']['msg'];
			}
			return false;
		}
	}

	/**
	 * GoDaddy has some strict rules about the state and country names so this
	 * function helps make sure our calls work rather than fail.
	 */
	private function filterCountryAndState($country, $state)
	{
		// if this country is in the list of ones that the state is verified in...
		if (isset($this->stateVerifiedCountries[$country])) {
			$states = $this->stateVerifiedCountries[$country];
			if (!in_array($state=ucwords($state), array_values($states))) {
				if ($states[strtoupper($state)]) {
					// is it an abbreviation?
					$state = $states[strtoupper($state)];
				} else {
					// if the state isn't in the list of allowed states for this country
					// then we need to calculate which of the states in this country
					// are the closest match to what they entered.  We do this by
					// calculating a levenshtein difference and then using the one with
					// the lowest result.
					$state = $this->getClosestMatch($state, array_values($states));
				}
			}
		}

		// convert country code to name
		if (isset($this->countries[$country])) {
			$country = $this->countries[$country];
		}

		return array($country, $state);
	}

	/**
	 * returns the closest match to the needle from the haystack using a levenshtein difference
	 */
	private function getClosestMatch($needle, $haystack)
	{
		$change = 1000;
		$current = $needle;
		foreach ($haystack as $test) {
			if (($newchange=levenshtein(strtolower($needle), strtolower($test)))<$change) {
				$change = $newchange;
				$current = $test;
			}
		}
		return $current;
	}

	/**
	 * cleans up the contact data, making sure things fit into the api right
	 */
	private function cleanContactData($contact)
	{
		// trim and clean up input a bit and truncate to proper lengths
		// specified by the GoDaddy documentation
		foreach (array(
			'fname'	=> 30,
			'lname'	=> 30,
			'org'	=> 100,
			'sa1'	=> 41,
			'sa2'	=> 41,
			'city'	=> 30,
			'sp'	=> 30,
			'pc'	=> 10,
			'email'	=> 80,
			'phone'	=> 17,
			'fax'	=> 17,
		) as $f=>$l) {
			$contact[$f] = preg_replace('/[[:space:]]+/', ' ', trim($contact[$f]));
			if (strlen($contact[$f]) > $l) {
				$contact[$f] = substr($contact[$f], 0, $l);
			}
		}

		// if they did not enter an organization name, populate with first and last name
		$contact['org'] = (isset($contact['org']) && trim($contact['org']) != '')
			? $contact['org']
			: $contact['fname'] . ' ' . $contact['lname'];

		$contact['pc'] = preg_replace('/[[:space:]]+/', ' ', trim($contact['pc']));

		$phoneComponents = billing_get_phone_src($contact['phone']);
		$contact['phone'] = "+{$phoneComponents['country_code']}.{$phoneComponents['area_code']}{$phoneComponents['number']}{$phoneComponents['extension']}";
		if ("" != $contact['fax']) {
			$faxComponents = billing_get_phone_src($contact['fax']);
			$contact['fax'] = "+{$faxComponents['country_code']}.{$faxComponents['area_code']}{$faxComponents['number']}{$faxComponents['extension']}";
		}

		if (!preg_match('/^[[:alnum:]_\+\.]+@[[:alnum:]\-\.]+$/', $contact['email'])) {
			$contact['email'] = '';
		}

		list($contact['cc'], $contact['sp']) =
			$this->filterCountryAndState($contact['cc'], $contact['sp']);

		return $contact;
	}

	/**
	 * utility method to create/cache/fetch a soap client object
	 */
	private function getSoapClient()
	{
		$this->client = isset($this->client)?$this->client:new WWDSOAPClientWrapper($this->wsdl, array(
			'trace' => true,
		));
		return $this->client;
	}

	private function getProductID($type, $tld, $years)
	{
		if (isset($this->productIDs[$type]) && isset($this->productIDs[$type][$tld]) && isset($this->productIDs[$type][$tld][$years])) {
			return $this->productIDs[$type][$tld][$years];
		}
		throw new WWDAPIError("Product is not available: type='{$type}', tld='{$tld}', years='{$years}'", self::RESULT_CODE_FAILURE);
	}

	private function getTransferProductID($tld)
	{
		if (isset($this->productIDs["transfer"][$tld])) {
			return $this->productIDs["transfer"][$tld];
		}
		throw new WWDAPIError("Product is not available for transfer: tld='{$tld}'", self::RESULT_CODE_FAILURE);
	}

	private $wsdl		= self::production_wsdl;

	private $account	= '';
	private $password	= '';

	private $errors		= array();

	private $contactTypes	= array(
		0 => 'registrant',
		1 => 'tech',
		2 => 'admin',
		3 => 'billing',
	);

	private $productIDs	= array(
		'new' => array(
			'me' => array(
				'1'  => 9140,
				'2'  => 9141,
				'3'  => 9142,
				'4'  => 9143,
				'5'  => 9144,
				'6'  => 9145,
				'7'  => 9146,
				'8'  => 9147,
				'9'  => 9148,
				'10' => 9149,
			),
			'in' => array(
				'1'  => 9850,
				'2'  => 9851,
				'3'  => 9852,
				'4'  => 9853,
				'5'  => 9854,
				'6'  => 9855,
				'7'  => 9856,
				'8'  => 9857,
				'9'  => 9858,
				'10' => 9859,
			),
			'bz' => array(
				'1'  => 15201,
				'2'  => 15202,
				'3'  => 15203,
				'4'  => 15204,
				'5'  => 15205,
				'6'  => 15206,
				'7'  => 15207,
				'8'  => 15208,
				'9'  => 15209,
				'10' => 152010,
			),
			'mobi' => array(
				'1'  => 40801,
				'2'  => 40802,
				'3'  => 40803,
				'4'  => 40804,
				'5'  => 40805,
				'6'  => 40806,
				'7'  => 40807,
				'8'  => 40808,
				'9'  => 40809,
				'10' => 40810,
			),
			'co.in' => array(
				'1'  => 41000,
				'2'  => 41001,
				'3'  => 41002,
				'4'  => 41003,
				'5'  => 41004,
				'6'  => 41005,
				'7'  => 41006,
				'8'  => 41007,
				'9'  => 41008,
				'10' => 41009,
			),
			'firm.in' => array(
				'1'  => 41100,
				'2'  => 41101,
				'3'  => 41102,
				'4'  => 41103,
				'5'  => 41104,
				'6'  => 41105,
				'7'  => 41106,
				'8'  => 41107,
				'9'  => 41108,
				'10' => 41109,
			),
			'gen.in' => array(
				'1'  => 41200,
				'2'  => 41201,
				'3'  => 41202,
				'4'  => 41203,
				'5'  => 41204,
				'6'  => 41205,
				'7'  => 41206,
				'8'  => 41207,
				'9'  => 41208,
				'10' => 41209,
			),
			'ind.in' => array(
				'1'  => 41300,
				'2'  => 41301,
				'3'  => 41302,
				'4'  => 41303,
				'5'  => 41304,
				'6'  => 41305,
				'7'  => 41306,
				'8'  => 41307,
				'9'  => 41308,
				'10' => 41309,
			),
			'net.in' => array(
				'1'  => 41400,
				'2'  => 41401,
				'3'  => 41402,
				'4'  => 41403,
				'5'  => 41404,
				'6'  => 41405,
				'7'  => 41406,
				'8'  => 41407,
				'9'  => 41408,
				'10' => 41409,
			),
			'org.in' => array(
				'1'  => 41500,
				'2'  => 41501,
				'3'  => 41502,
				'4'  => 41503,
				'5'  => 41504,
				'6'  => 41505,
				'7'  => 41506,
				'8'  => 41507,
				'9'  => 41508,
				'10' => 41509,
			),
			'com.bz' => array(
				'1'  => 41600,
				'2'  => 41601,
				'3'  => 41602,
				'4'  => 41603,
				'5'  => 41604,
				'6'  => 41605,
				'7'  => 41606,
				'8'  => 41607,
				'9'  => 41608,
				'10' => 41609,
			),
			'net.bz' => array(
				'1'  => 41700,
				'2'  => 41701,
				'3'  => 41702,
				'4'  => 41703,
				'5'  => 41704,
				'6'  => 41705,
				'7'  => 41706,
				'8'  => 41707,
				'9'  => 41708,
				'10' => 41709,
			),
			'es' => array(
				'1'  => 54001,
				'2'  => 54002,
				'3'  => 54003,
				'4'  => 54004,
				'5'  => 54005,
				'6'  => 54006,
				'7'  => 54007,
				'8'  => 54008,
				'9'  => 54009,
				'10' => 54010,
			),
			'com.es' => array(
				'1'  => 55001,
				'2'  => 55002,
				'3'  => 55003,
				'4'  => 55004,
				'5'  => 55005,
				'6'  => 55006,
				'7'  => 55007,
				'8'  => 55008,
				'9'  => 55009,
				'10' => 55010,
			),
			'org.es' => array(
				'1'  => 56001,
				'2'  => 56002,
				'3'  => 56003,
				'4'  => 56004,
				'5'  => 56005,
				'6'  => 56006,
				'7'  => 56007,
				'8'  => 56008,
				'9'  => 56009,
				'10' => 56010,
			),
			'nom.es' => array(
				'1'  => 57001,
				'2'  => 57002,
				'3'  => 57003,
				'4'  => 57004,
				'5'  => 57005,
				'6'  => 57006,
				'7'  => 57007,
				'8'  => 57008,
				'9'  => 57009,
				'10' => 57010,
			),
			'com' => array(
				'1'  => 350001,
				'2'  => 350002,
				'3'  => 350003,
				'4'  => 350004,
				'5'  => 350005,
				'6'  => 350006,
				'7'  => 350007,
				'8'  => 350008,
				'9'  => 350009,
				'10' => 350001,
			),
			'net' => array(
				'1'  => 350030,
				'2'  => 350031,
				'3'  => 350032,
				'4'  => 350033,
				'5'  => 350034,
				'6'  => 350035,
				'7'  => 350036,
				'8'  => 350037,
				'9'  => 350038,
				'10' => 350039,
			),
			'info' => array(
				'1'  => 350051,
				'2'  => 350052,
				'3'  => 350053,
				'4'  => 350054,
				'5'  => 350055,
				'6'  => 350056,
				'7'  => 350057,
				'8'  => 350058,
				'9'  => 350059,
				'10' => 350060,
			),
			'biz' => array(
				'1'  => 350076,
				'2'  => 350077,
				'3'  => 350078,
				'4'  => 350079,
				'5'  => 350080,
				'6'  => 350081,
				'7'  => 350082,
				'8'  => 350083,
				'9'  => 350084,
				'10' => 350085,
			),
			'us' => array(
				'1'  => 350126,
				'2'  => 350127,
				'3'  => 350128,
				'4'  => 350129,
				'5'  => 350130,
				'6'  => 350131,
				'7'  => 350132,
				'8'  => 350133,
				'9'  => 350134,
				'10' => 350135,
			),
			'ws' => array(
				'1'  => 350101,
				'2'  => 350102,
				'3'  => 350103,
				'4'  => 350104,
				'5'  => 350105,
				'6'  => 350106,
				'7'  => 350107,
				'8'  => 350108,
				'9'  => 350109,
				'10' => 350110,
			),
			'org' => array(
				'1'  => 350150,
				'2'  => 350151,
				'3'  => 350152,
				'4'  => 350153,
				'5'  => 350154,
				'6'  => 350155,
				'7'  => 350156,
				'8'  => 350157,
				'9'  => 350158,
				'10' =>  350159,
			),
			'com.mx' => array(
				'1'  => 350301,
				'2'  => 350302,
				'3'  => 350303,
				'4'  => 350304,
				'5'  => 350305,
			),
			'tv' => array(
				'1'  => 350401,
				'2'  => 350402,
				'3'  => 350403,
				'4'  => 350404,
				'5'  => 350405,
				'6'  => 350406,
				'7'  => 350407,
				'8'  => 350408,
				'9'  => 350409,
				'10' => 350410,
			),
			'idprotect' => 377001,
		),
		'renew' => array(
			'me' => array(
				'1'  => 19140,
				'2'  => 19141,
				'3'  => 19142,
				'4'  => 19143,
				'5'  => 19144,
				'6'  => 19145,
				'7'  => 19146,
				'8'  => 19147,
				'9'  => 19148,
				'10' => 19149,
			),
			'in' => array(
				'1'  => 19850,
				'2'  => 19851,
				'3'  => 19852,
				'4'  => 19853,
				'5'  => 19854,
				'6'  => 19855,
				'7'  => 19856,
				'8'  => 19857,
				'9'  => 19858,
				'10' => 19859,
			),
			'bz' => array(
				'1'  => 15212,
				'2'  => 15213,
				'3'  => 15214,
				'4'  => 15215,
				'5'  => 15216,
				'6'  => 15217,
				'7'  => 15218,
				'8'  => 15219,
				'9'  => 15220,
				'10' => 15221,
			),
			'mobi' => array(
				'1'  => 50801,
				'2'  => 50802,
				'3'  => 50803,
				'4'  => 50804,
				'5'  => 50805,
				'6'  => 50806,
				'7'  => 50807,
				'8'  => 50808,
				'9'  => 50809,
				'10' => 50810,
			),
			'co.in' => array(
				'1'  => 51000,
				'2'  => 51001,
				'3'  => 51002,
				'4'  => 51003,
				'5'  => 51004,
				'6'  => 51005,
				'7'  => 51006,
				'8'  => 51007,
				'9'  => 51008,
				'10' => 51009,
			),
			'firm.in' => array(
				'1'  => 51100,
				'2'  => 51101,
				'3'  => 51102,
				'4'  => 51103,
				'5'  => 51104,
				'6'  => 51105,
				'7'  => 51106,
				'8'  => 51107,
				'9'  => 51108,
				'10' => 51109,
			),
			'gen.in' => array(
				'1'  => 51200,
				'2'  => 51201,
				'3'  => 51202,
				'4'  => 51203,
				'5'  => 51204,
				'6'  => 51205,
				'7'  => 51206,
				'8'  => 51207,
				'9'  => 51208,
				'10' => 51209,
			),
			'ind.in' => array(
				'1'  => 51300,
				'2'  => 51301,
				'3'  => 51302,
				'4'  => 51303,
				'5'  => 51304,
				'6'  => 51305,
				'7'  => 51306,
				'8'  => 51307,
				'9'  => 51308,
				'10' => 51309,
			),
			'net.in' => array(
				'1'  => 51400,
				'2'  => 51401,
				'3'  => 51402,
				'4'  => 51403,
				'5'  => 51404,
				'6'  => 51405,
				'7'  => 51406,
				'8'  => 51407,
				'9'  => 51408,
				'10' => 51409,
			),
			'org.in' => array(
				'1'  => 51500,
				'2'  => 51501,
				'3'  => 51502,
				'4'  => 51503,
				'5'  => 51504,
				'6'  => 51505,
				'7'  => 51506,
				'8'  => 51507,
				'9'  => 51508,
				'10' => 51509,
			),
			'com.bz' => array(
				'1'  => 51600,
				'2'  => 51601,
				'3'  => 51602,
				'4'  => 51603,
				'5'  => 51604,
				'6'  => 51605,
				'7'  => 51606,
				'8'  => 51607,
				'9'  => 51608,
				'10' => 51609,
			),
			'net.bz' => array(
				'1'  => 51700,
				'2'  => 51701,
				'3'  => 51702,
				'4'  => 51703,
				'5'  => 51704,
				'6'  => 51705,
				'7'  => 51706,
				'8'  => 51707,
				'9'  => 51708,
				'10' => 51709,
			),
			'es' => array(
				'1'  => 64001,
				'2'  => 64002,
				'3'  => 64003,
				'4'  => 64004,
				'5'  => 64005,
				'6'  => 64006,
				'7'  => 64007,
				'8'  => 64008,
				'9'  => 64009,
				'10' => 64010,
			),
			'com.es' => array(
				'1'  => 65001,
				'2'  => 65002,
				'3'  => 65003,
				'4'  => 65004,
				'5'  => 65005,
				'6'  => 65006,
				'7'  => 65007,
				'8'  => 65008,
				'9'  => 65009,
				'10' => 65010,
			),
			'org.es' => array(
				'1'  => 66001,
				'2'  => 66002,
				'3'  => 66003,
				'4'  => 66004,
				'5'  => 66005,
				'6'  => 66006,
				'7'  => 66007,
				'8'  => 66008,
				'9'  => 66009,
				'10' => 66010,
			),
			'nom.es' => array(
				'1'  => 67001,
				'2'  => 67002,
				'3'  => 67003,
				'4'  => 67004,
				'5'  => 67005,
				'6'  => 67006,
				'7'  => 67007,
				'8'  => 67008,
				'9'  => 67009,
				'10' => 67010,
			),
			'com' => array(
				'1'  => 350012,
				'2'  => 350013,
				'3'  => 350014,
				'4'  => 350015,
				'5'  => 350016,
				'6'  => 350017,
				'7'  => 350018,
				'8'  => 350019,
				'9'  => 350020,
				'10' => 350021,
			),
			'net' => array(
				'1'  => 350041,
				'2'  => 350042,
				'3'  => 350043,
				'4'  => 350044,
				'5'  => 350045,
				'6'  => 350046,
				'7'  => 350047,
				'8'  => 350048,
				'9'  => 350049,
				'10' => 350050,
			),
			'info' => array(
				'1'  => 350062,
				'2'  => 350063,
				'3'  => 350064,
				'4'  => 350065,
				'5'  => 350066,
				'6'  => 350067,
				'7'  => 350068,
				'8'  => 350069,
				'9'  => 350070,
				'10' => 350071,
			),
			'biz' => array(
				'1'  => 350087,
				'2'  => 350088,
				'3'  => 350089,
				'4'  => 350090,
				'5'  => 350091,
				'6'  => 350092,
				'7'  => 350093,
				'8'  => 350094,
				'9'  => 350095,
				'10' => 350096,
			),
			'ws' => array(
				'1'  => 350112,
				'2'  => 350113,
				'3'  => 350114,
				'4'  => 350115,
				'5'  => 350116,
				'6'  => 350117,
				'7'  => 350118,
				'8'  => 350119,
				'9'  => 350120,
				'10' => 350121,
			),
			'us' => array(
				'1'  => 350137,
				'2'  => 350138,
				'3'  => 350139,
				'4'  => 350140,
				'5'  => 350141,
				'6'  => 350142,
				'7'  => 350143,
				'8'  => 350144,
				'9'  => 350145,
				'10' => 350146,
			),
			'org' => array(
				'1'  => 350161,
				'2'  => 350162,
				'3'  => 350163,
				'4'  => 350164,
				'5'  => 350165,
				'6'  => 350166,
				'7'  => 350167,
				'8'  => 350168,
				'9'  => 350169,
				'10' => 350170,
			),
			'tv' => array(
				'1'  => 350412,
				'2'  => 350413,
				'3'  => 350414,
				'4'  => 350415,
				'5'  => 350416,
				'6'  => 350417,
				'7'  => 350418,
				'8'  => 350419,
				'9'  => 350420,
				'10' => 350421,
			),
			'idprotect' => 387001,
		),
		'transfer' => array(
			'me'      => 9150,
			'in'      => 9860,
			'bz'      => 15211,
			'mobi'    => 40811,
			'co.in'   => 41010,
			'firm.in' => 41110,
			'gen.in'  => 41210,
			'ind.in'  => 41310,
			'net.in'  => 41410,
			'org.in'  => 41510,
			'com.bz'  => 41610,
			'net.bz'  => 41710,
			'es'      => 54011,
			'com.es'  => 55011,
			'org.es'  => 56011,
			'nom.es'  => 57011,
			'com'     => 350011,
			'net'     => 350040,
			'info'    => 350061,
			'biz'     => 350086,
			'us'      => 350136,
			'org'     => 350160,
			'com.mx'  => 350311,
			'tv'      => 350411,
		),
	);

	private $stateVerifiedCountries = array(
		'AU' => array(
			'ACT' => 'Capital Territory',
			'NSW' => 'New South Wales',
			'NT'  => 'Northern Territory',
			'QLD' => 'Queensland',
			'SA'  => 'South Australia',
			'TAS' => 'Tasmania',
			'VIC' => 'Victoria',
			'WA'  => 'Western Australia',
		),
		'BE' => array(
			'AN'  => 'Antwerpen',
			'BW'  => 'Brabant Wallon',
			'BRU' => 'Brussels',
			'HT'  => 'Hainaut',
			'LG'  => 'Liege',
			'LI'  => 'Limburg',
			'LX'  => 'Luxembourg',
			'NA'  => 'Namur',
			'OV'  => 'Oost­Vlaanderen',
			'VB'  => 'Vlaams Brabant',
			'WV'  => 'West­Vlaanderen',
		),
		'BR' => array(
			'AC' => 'Acre',
			'AL' => 'Alagoas',
			'AP' => 'Amapa',
			'AM' => 'Amazonas',
			'BA' => 'Bahia',
			'CE' => 'Ceara',
			'DF' => 'Distrito Federal',
			'ES' => 'Espirito Santo',
			'GO' => 'Goias',
			'MA' => 'Maranhao',
			'MT' => 'Mato Grosso',
			'MS' => 'Mato Grosso do Sul',
			'MG' => 'Minas Gerais',
			'PA' => 'Para',
			'PB' => 'Paraiba',
			'PR' => 'Parana',
			'PE' => 'Pernambuco',
			'PI' => 'Piaui',
			'RJ' => 'Rio de Janeiro',
			'RN' => 'Rio Grande do Norte',
			'RS' => 'Rio Grande do Sul',
			'RO' => 'Rondonia',
			'RR' => 'Roraima',
			'SC' => 'Santa Catarina',
			'SP' => 'Sao Paulo',
			'SE' => 'Sergipe',
			'TO' => 'Tocantins',
		),
		'CA' => array(
			'AB' => 'Alberta',
			'BC' => 'British Columbia',
			'MB' => 'Manitoba',
			'NB' => 'New Brunswick',
			'NF' => 'Newfoundland',
			'NT' => 'Northwest Territories',
			'NS' => 'Nova Scotia',
			'NU' => 'Nunavut',
			'ON' => 'Ontario',
			'PE' => 'Prince Edward Island',
			'QC' => 'Quebec',
			'SK' => 'Saskatchewan',
			'YT' => 'Yukon Territory',
		),
		'IN' => array(
			'AN' => 'Andaman and Nicobar Islands',
			'AP' => 'Andhra Pradesh',
			'AR' => 'Arunachal Pradesh',
			'AS' => 'Assam',
			'BR' => 'Bihar',
			'CH' => 'Chandigarh',
			'CT' => 'Chhattisgarh',
			'DN' => 'Dadra and Nagar Haveli',
			'DD' => 'Daman and Diu',
			'DL' => 'Delhi',
			'GA' => 'Goa',
			'GJ' => 'Gujarat',
			'HR' => 'Haryana',
			'HP' => 'Himachal Pradesh',
			'JK' => 'Jammu and Kashmir',
			'JH' => 'Jharkhand',
			'KA' => 'Karnataka',
			'KL' => 'Kerala',
			'LD' => 'Lakshadweep',
			'MP' => 'Madhya Pradesh',
			'MH' => 'Maharashtra',
			'MN' => 'Manipur',
			'ML' => 'Meghalaya',
			'MZ' => 'Mizoram',
			'NL' => 'Nagaland',
			'OR' => 'Orissa',
			'PY' => 'Puducherry',
			'PB' => 'Punjab',
			'RJ' => 'Rajasthan',
			'SK' => 'Sikkim',
			'TN' => 'Tamil Nadu',
			'TR' => 'Tripura',
			'UT' => 'Uttar Pradesh',
			'UP' => 'Uttarakhand',
			'WB' => 'West Bengal',
		),
		'JP' => array(
			'Aichi'		=> 'Aichi',
			'Akita'		=> 'Akita',
			'Aomori'	=> 'Aomori',
			'Chiba'		=> 'Chiba',
			'Ehime'		=> 'Ehime',
			'Fukui'		=> 'Fukui',
			'Fukuoka'	=> 'Fukuoka',
			'Fukushima'	=> 'Fukushima',
			'Gifu'		=> 'Gifu',
			'Gumma'		=> 'Gumma',
			'Hiroshima'	=> 'Hiroshima',
			'Hokkaido'	=> 'Hokkaido',
			'Hyogo'		=> 'Hyogo',
			'Ibaraki'	=> 'Ibaraki',
			'Ishikawa'	=> 'Ishikawa',
			'Iwate'		=> 'Iwate',
			'Kagawa'	=> 'Kagawa',
			'Kagoshima'	=> 'Kagoshima',
			'Kanagawa'	=> 'Kanagawa',
			'Kouchi'	=> 'Kouchi',
			'Kumamoto'	=> 'Kumamoto',
			'Kyoto'		=> 'Kyoto',
			'Mie'		=> 'Mie',
			'Miyagi'	=> 'Miyagi',
			'Miyazaki'	=> 'Miyazaki',
			'Nagano'	=> 'Nagano',
			'Nagasaki'	=> 'Nagasaki',
			'Nara'		=> 'Nara',
			'Niigata'	=> 'Niigata',
			'Oita'		=> 'Oita',
			'Okayama'	=> 'Okayama',
			'Okinawa'	=> 'Okinawa',
			'Osaka'		=> 'Osaka',
			'Saga'		=> 'Saga',
			'Saitama'	=> 'Saitama',
			'Shiga'		=> 'Shiga',
			'Shimane'	=> 'Shimane',
			'Shizuoka'	=> 'Shizuoka',
			'Tochigi'	=> 'Tochigi',
			'Tokushima'	=> 'Tokushima',
			'Tokyo'		=> 'Tokyo',
			'Tottori'	=> 'Tottori',
			'Toyama'	=> 'Toyama',
			'Wakayama'	=> 'Wakayama',
			'Yamagata'	=> 'Yamagata',
			'Yamaguchi'	=> 'Yamaguchi',
			'Yamanashi'	=> 'Yamanashi',
		),
		'MX' => array(
			'AGS'   => 'Aguascalientes',
			'BC'    => 'Baja California',
			'BCS'   => 'Baja California Sur',
			'CAMP'  => 'Campeche',
			'CHIS'  => 'Chiapas',
			'CHIH'  => 'Chihuahua',
			'COAH'  => 'Coahuila',
			'COL'   => 'Colima',
			'DF'    => 'Distrito Federal',
			'DGO'   => 'Durango',
			'GTO'   => 'Guanajuato',
			'GRO'   => 'Guerrero',
			'HGO'   => 'Hidalgo',
			'JAL'   => 'Jalisco',
			'EDM'   => 'Mexico',
			'MICH'  => 'Michoacan',
			'MOR'   => 'Morelos',
			'NAY'   => 'Nayarit',
			'NL'    => 'Nuevo Leon',
			'OAX'   => 'Oaxaca',
			'PUE'   => 'Puebla',
			'QRO'   => 'Queretaro',
			'QROO'  => 'Quintana Roo',
			'SLP'   => 'San Luis Potosi',
			'SIN'   => 'Sinaloa',
			'SON'   => 'Sonora',
			'TAB'   => 'Tabasco',
			'TAMPS' => 'Tamaulipas',
			'TLAX'  => 'Tlaxcala',
			'VER'   => 'Veracruz',
			'YUC'   => 'Yucatan',
			'ZAC'   => 'Zacatecas',
		),
		'US' => array(
			'AL' => 'Alabama',
			'AK' => 'Alaska',
			'AS' => 'American Samoa',
			'AZ' => 'Arizona',
			'AR' => 'Arkansas',
			'AA' => 'Armed Forces America',
			'AE' => 'Armed Forces Other Areas',
			'AP' => 'Armed Forces Pacific',
			'CA' => 'California',
			'CO' => 'Colorado',
			'CT' => 'Connecticut',
			'DE' => 'Delaware',
			'DC' => 'District of Columbia',
			'FM' => 'Federated States of Micronesia',
			'FL' => 'Florida',
			'GA' => 'Georgia',
			'GU' => 'Guam',
			'HI' => 'Hawaii',
			'ID' => 'Idaho',
			'IL' => 'Illinois',
			'IN' => 'Indiana',
			'IA' => 'Iowa',
			'KS' => 'Kansas',
			'KY' => 'Kentucky',
			'LA' => 'Louisiana',
			'ME' => 'Maine',
			'MH' => 'Marshall Islands',
			'MD' => 'Maryland',
			'MA' => 'Massachusetts',
			'MI' => 'Michigan',
			'MN' => 'Minnesota',
			'MS' => 'Mississippi',
			'MO' => 'Missouri',
			'MT' => 'Montana',
			'NE' => 'Nebraska',
			'NV' => 'Nevada',
			'NH' => 'New Hampshire',
			'NJ' => 'New Jersey',
			'NM' => 'New Mexico',
			'NY' => 'New York',
			'NC' => 'North Carolina',
			'ND' => 'North Dakota',
			'MP' => 'Northern Mariana Islands',
			'OH' => 'Ohio',
			'OK' => 'Oklahoma',
			'OR' => 'Oregon',
			'PW' => 'Palau',
			'PA' => 'Pennsylvania',
			'PR' => 'Puerto Rico',
			'RI' => 'Rhode Island',
			'SC' => 'South Carolina',
			'SD' => 'South Dakota',
			'TN' => 'Tennessee',
			'TX' => 'Texas',
			'UT' => 'Utah',
			'VT' => 'Vermont',
			'VI' => 'Virgin Islands',
			'VA' => 'Virginia',
			'WA' => 'Washington',
			'WV' => 'West Virginia',
			'WI' => 'Wisconsin',
			'WY' => 'Wyoming',
		),
	);

	private $countries	= array(
		'AF' => 'Afghanistan',
		// 'AX' => 'Åland Islands',
		'AL' => 'Albania',
		'DZ' => 'Algeria',
		'AS' => 'American Samoa',
		'AD' => 'Andorra',
		'AO' => 'Angola',
		'AI' => 'Anguilla',
		'AQ' => 'Antarctica',
		'AG' => 'Antigua and Barbuda',
		'AR' => 'Argentina',
		'AM' => 'Armenia',
		'AW' => 'Aruba',
		'AC' => 'Ascension Island',
		'AU' => 'Australia',
		'AT' => 'Austria',
		'AZ' => 'Azerbaijan',
		'BS' => 'Bahamas',
		'BH' => 'Bahrain',
		'BD' => 'Bangladesh',
		'BB' => 'Barbados',
		'BY' => 'Belarus',
		'BE' => 'Belgium',
		'BZ' => 'Belize',
		'BJ' => 'Benin',
		'BM' => 'Bermuda',
		'BT' => 'Bhutan',
		'BO' => 'Bolivia',
		'BQ' => 'Bonaire',
		'BA' => 'Bosnia and Herzegovina',
		'BW' => 'Botswana',
		'BV' => 'Bouvet Island',
		'BR' => 'Brazil',
		'IO' => 'British Indian Ocean Territory',
		'BN' => 'Brunei Darussalam',
		'BG' => 'Bulgaria',
		'BF' => 'Burkina Faso',
		'BI' => 'Burundi',
		'KH' => 'Cambodia',
		'CM' => 'Cameroon',
		'CA' => 'Canada',
		'CV' => 'Cape Verde Islands',
		'KY' => 'Cayman Islands',
		'CF' => 'Central African Republic',
		'TD' => 'Chad',
		'CL' => 'Chile',
		'CN' => 'China',
		'CX' => 'Christmas Island',
		'CC' => 'Cocos (Keeling) Islands',
		'CO' => 'Colombia',
		'KM' => 'Comoros',
		'CG' => 'Republic of the Congo',
		'CD' => 'Congo, Democratic Republic of',
		'CG' => 'Congo, Republic of',
		'CK' => 'Cook Islands',
		'CR' => 'Costa Rica',
		'CI' => 'Cote d\'Ivoire',
		'HR' => 'Croatia/Hrvatska',
		// 'CU' => 'Cuba',
		// 'CW' => 'Curaçao',
		'CY' => 'Cyprus',
		'CZ' => 'Czech Republic',
		'DK' => 'Denmark',
		'DJ' => 'Djibouti',
		'DM' => 'Dominica',
		'DO' => 'Dominican Republic',
		'TP' => 'East Timor',
		'EC' => 'Ecuador',
		'EG' => 'Egypt',
		'SV' => 'El Salvador',
		'GQ' => 'Equatorial Guinea',
		'ER' => 'Eritrea',
		'EE' => 'Estonia',
		'ET' => 'Ethiopia',
		'FK' => 'Falkland Islands',
		'FO' => 'Faroe Islands',
		'FJ' => 'Fiji',
		'FI' => 'Finland',
		'FR' => 'France',
		'GF' => 'French Guiana',
		'PF' => 'French Polynesia',
		'TF' => 'French Southern Territories',
		'GA' => 'Gabon',
		'GM' => 'Gambia',
		'GE' => 'Georgia',
		'DE' => 'Germany',
		'GH' => 'Ghana',
		'GI' => 'Gibraltar',
		'GR' => 'Greece',
		'GL' => 'Greenland',
		'GD' => 'Grenada',
		'GP' => 'Guadeloupe',
		'GU' => 'Guam',
		'GT' => 'Guatemala',
		'GG' => 'Guernsey',
		'GN' => 'Guinea',
		'GW' => 'Guinea-Bissau',
		'GY' => 'Guyana',
		'HT' => 'Haiti',
		'HM' => 'Heard Island and McDonald Islands',
		'HN' => 'Honduras',
		'HK' => 'Hong Kong',
		'HU' => 'Hungary',
		'IS' => 'Iceland',
		'IN' => 'India',
		'ID' => 'Indonesia',
		// 'IR' => 'Iran',
		'IQ' => 'Iraq',
		'IE' => 'Ireland',
		'IM' => 'Isle of Man',
		'IL' => 'Israel',
		'IT' => 'Italy',
		'JM' => 'Jamaica',
		'JP' => 'Japan',
		'JE' => 'Jersey',
		'JO' => 'Jordan',
		'KZ' => 'Kazakhstan',
		'KE' => 'Kenya',
		'KI' => 'Kiribati',
		// 'KP' => 'Korea, Democratic People\'s Republic of (North Korea)',
		'KR' => 'Korea, Republic of (South Korea)',
		'XK' => 'Kosovo',
		'KW' => 'Kuwait',
		'KG' => 'Kyrgyzstan',
		'LA' => 'Laos',
		'LV' => 'Latvia',
		'LB' => 'Lebanon',
		'LS' => 'Lesotho',
		'LR' => 'Liberia',
		'LY' => 'Libya',
		'LI' => 'Liechtenstein',
		'LT' => 'Lithuania',
		'LU' => 'Luxembourg',
		'MO' => 'Macau',
		'MK' => 'Macedonia',
		'MG' => 'Madagascar',
		'MW' => 'Malawi',
		'MY' => 'Malaysia',
		'MV' => 'Maldives',
		'ML' => 'Mali',
		'MT' => 'Malta',
		'MH' => 'Marshall Islands',
		'MQ' => 'Martinique',
		'MR' => 'Mauritania',
		'MU' => 'Mauritius',
		'YT' => 'Mayotte Island',
		'MX' => 'Mexico',
		'FM' => 'Micronesia',
		'MD' => 'Moldova',
		'MC' => 'Monaco',
		'MN' => 'Mongolia',
		'ME' => 'Montenegro',
		'MS' => 'Montserrat',
		'MA' => 'Morocco',
		'MZ' => 'Mozambique',
		'MM' => 'Myanmar',
		'NA' => 'Namibia',
		'NR' => 'Nauru',
		'NP' => 'Nepal',
		'NL' => 'Netherlands',
		'AN' => 'Netherlands Antilles',
		'NC' => 'New Caledonia',
		'NZ' => 'New Zealand',
		'NI' => 'Nicaragua',
		'NE' => 'Niger',
		'NG' => 'Nigeria',
		'NU' => 'Niue',
		'NF' => 'Norfolk Island',
		'MP' => 'Northern Mariana Islands',
		'NO' => 'Norway',
		'OM' => 'Oman',
		'PK' => 'Pakistan',
		'PW' => 'Palau',
		'PS' => 'Palestinian Territories',
		'PA' => 'Panama',
		'PG' => 'Papua New Guinea',
		'PY' => 'Paraguay',
		'PE' => 'Peru',
		'PH' => 'Philippines',
		'PN' => 'Pitcairn Island',
		'PL' => 'Poland',
		'PT' => 'Portugal',
		'PR' => 'Puerto Rico',
		'QA' => 'Qatar',
		'RE' => 'Reunion Island',
		'RO' => 'Romania',
		'RU' => 'Russian Federation',
		'RW' => 'Rwanda',
		// 'BL' => 'Saint Barthélemy',
		'SH' => 'Saint Helena',
		'KN' => 'Saint Kitts and Nevis',
		'LC' => 'Saint Lucia',
		// 'MF' => 'Saint Martin',
		'PM' => 'Saint Pierre and Miquelon',
		'VC' => 'Saint Vincent and the Grenadines',
		'WS' => 'Western Samoa',
		'SM' => 'San Marino',
		'ST' => 'Sao Tome and Principe',
		'SA' => 'Saudi Arabia',
		'SN' => 'Senegal',
		'RS' => 'Serbia',
		'SC' => 'Seychelles',
		'SL' => 'Sierra Leone',
		'SG' => 'Singapore',
		// 'SX' => 'Sint Maarten',
		'SK' => 'Slovak Republic',
		'SI' => 'Slovenia',
		'SB' => 'Solomon Islands',
		'SO' => 'Somalia',
		'ZA' => 'South Africa',
		'GS' => 'South Georgia and the South Sandwich Islands',
		'ES' => 'Spain',
		'LK' => 'Sri Lanka',
		// 'SD' => 'Sudan',
		'SR' => 'Suriname',
		'SJ' => 'Svalbard and Jan Mayen Islands',
		'SZ' => 'Swaziland',
		'SE' => 'Sweden',
		'CH' => 'Switzerland',
		// 'SY' => 'Syrian Arab Republic',
		'TW' => 'Taiwan',
		'TJ' => 'Tajikistan',
		'TZ' => 'Tanzania',
		'TH' => 'Thailand',
		'TL' => 'Timor-Leste',
		'TG' => 'Togo',
		'TK' => 'Tokelau',
		'TO' => 'Tonga Islands',
		'TT' => 'Trinidad and Tobago',
		'TN' => 'Tunisia',
		'TR' => 'Turkey',
		'TM' => 'Turkmenistan',
		'TC' => 'Turks and Caicos Islands',
		'TV' => 'Tuvalu',
		'UG' => 'Uganda',
		'UA' => 'Ukraine',
		'AE' => 'United Arab Emirates',
		'GB' => 'United Kingdom',
		'US' => 'United States',
		'UM' => 'US Minor Outlying Islands',
		'UY' => 'Uruguay',
		'UZ' => 'Uzbekistan',
		'VU' => 'Vanuatu',
		'VA' => 'Vatican City',
		'VE' => 'Venezuela',
		'VN' => 'Vietnam',
		'VG' => 'Virgin Islands (British)',
		'VI' => 'Virgin Islands (USA)',
		'WF' => 'Wallis and Futuna Islands',
		'EH' => 'Western Sahara',
		'YE' => 'Yemen',
		'ZM' => 'Zambia',
		'ZW' => 'Zimbabwe',
	);
}

class WWDSoapClient extends SoapClient
{
	public function __doRequest($request, $location, $action, $version, $one_way = 0) {
		dbg($request);
		$response = parent::__doRequest($request, $location, $action, $version, $one_way);
		dbg($response);
		return $response;
	}
}


/**
 * A class to wrap around soap calls so I don't have to parse the XML every time.
 * (Yes, GoDaddy's SOAP XML API returns encoded XML inside of XML...)
 */
class WWDSOAPClientWrapper
{
	public function __construct($wsdl, $ops=array())
	{
		$this->client = new WWDSoapClient($this->wsdl = $wsdl, $ops);
	}

	/**
	 * A function to make this object emulate the soap client but automatically parse
	 * the returned xml and return that, while also logging debug info for each call
	 */
	public function __call($name, $params=array())
	{
		dbg('======== WWD API CALL DATA ========');
		dbg('WSDL LOCATION: '.$this->wsdl);
		$soapResult = $this->client->$name($params[0]);
		$resultName = $name.'Result';
		$x = new MBXMLLite($soapResult->$resultName);
		$result = $x->toArray();
		dbg('returning from soap call:');
		dbg($result);

		if (isset($result['response']) &&
			isset($result['response'][0]['result']) &&
			isset($result['response'][0]['result'][0]['_attributes']) &&
			isset($result['response'][0]['result'][0]['_attributes']['code']) &&
			WWDAPI::RESULT_CODE_FAILURE == $result['response'][0]['result'][0]['_attributes']['code']
		) {
			throw new WWDAPIError(
				blmsg("TRANS_REGISTRAR_ERROR_MSG", array("MESSAGE" => $soapResult->$resultName)),
				blmsg($result['response'][0]['result'][0]['_attributes']['code'])
			);
		}

		if (isset($result['error']) &&
			isset($result['error'][0]['_attributes']) &&
			isset($result['error'][0]['_attributes']['code']) &&
			isset($result['error'][0]['_attributes']['msg'])
		) {
			throw new WWDAPIError(
				blmsg("TRANS_REGISTRAR_ERROR_MSG", array("MESSAGE" => $result['error'][0]['_attributes']['msg'])),
				blmsg($result['error'][0]['_attributes']['code'])
			);
		}

		return $result;
	}

	private $client;
	private $wsdl;
}

/**
 * This is the wsdl methods and types printout for quick reference

functions:

[0] = ProcessRequestResponse ProcessRequest(ProcessRequest $parameters)
[1] = DescribeResponse Describe(Describe $parameters)
[2] = CheckAvailabilityResponse CheckAvailability(CheckAvailability $parameters)
[3] = InfoResponse Info(Info $parameters)
[4] = NameGenResponse NameGen(NameGen $parameters)
[5] = NameGenDBResponse NameGenDB(NameGenDB $parameters)
[6] = NameGenDBWithTimeLimitResponse NameGenDBWithTimeLimit(NameGenDBWithTimeLimit $parameters)
[7] = PollResponse Poll(Poll $parameters)
[8] = OrderCreditsResponse OrderCredits(OrderCredits $parameters)
[9] = OrderDomainsResponse OrderDomains(OrderDomains $parameters)
[10] = OrderDomainRenewalsResponse OrderDomainRenewals(OrderDomainRenewals $parameters)
[11] = OrderPrivateDomainRenewalsResponse OrderPrivateDomainRenewals(OrderPrivateDomainRenewals $parameters)
[12] = OrderDomainTransfersResponse OrderDomainTransfers(OrderDomainTransfers $parameters)
[13] = OrderDomainPrivacyResponse OrderDomainPrivacy(OrderDomainPrivacy $parameters)
[14] = OrderResourceRenewalsResponse OrderResourceRenewals(OrderResourceRenewals $parameters)
[15] = OrderDomainBackOrdersResponse OrderDomainBackOrders(OrderDomainBackOrders $parameters)
[16] = CancelResponse Cancel(Cancel $parameters)
[17] = UpdateDomainOwnershipResponse UpdateDomainOwnership(UpdateDomainOwnership $parameters)
[18] = ResetPasswordResponse ResetPassword(ResetPassword $parameters)
[19] = SetShopperInfoResponse SetShopperInfo(SetShopperInfo $parameters)
[20] = CreateNewShopperResponse CreateNewShopper(CreateNewShopper $parameters)
[21] = CheckUserResponse CheckUser(CheckUser $parameters)
[22] = SetupDomainAlertResponse SetupDomainAlert(SetupDomainAlert $parameters)
[23] = UpdateDomainAlertResponse UpdateDomainAlert(UpdateDomainAlert $parameters)
[24] = RemoveDomainAlertResponse RemoveDomainAlert(RemoveDomainAlert $parameters)
[25] = GetDomainAlertCreditsResponse GetDomainAlertCredits(GetDomainAlertCredits $parameters)
[26] = GetMonitoredDomainListResponse GetMonitoredDomainList(GetMonitoredDomainList $parameters)
[27] = GetExpiringNameListResponse GetExpiringNameList(GetExpiringNameList $parameters)
[28] = DomainForwardingResponse DomainForwarding(DomainForwarding $parameters)
[29] = UpdateNameServerResponse UpdateNameServer(UpdateNameServer $parameters)
[30] = UpdateDomainContactResponse UpdateDomainContact(UpdateDomainContact $parameters)
[31] = SetDomainLockingResponse SetDomainLocking(SetDomainLocking $parameters)
[32] = ManageTransferResponse ManageTransfer(ManageTransfer $parameters)
[33] = UpdateDomainForwardingResponse UpdateDomainForwarding(UpdateDomainForwarding $parameters)
[34] = UpdateDomainMaskingResponse UpdateDomainMasking(UpdateDomainMasking $parameters)
[35] = ModifyDNSResponse ModifyDNS(ModifyDNS $parameters)
[36] = ValidateRegistrationResponse ValidateRegistration(ValidateRegistration $parameters)

*//**
types:

[0] = struct ProcessRequest {
 string sRequestXML;
}
[1] = struct ProcessRequestResponse {
 string ProcessRequestResult;
}
[2] = struct Describe {
 string sCLTRID;
 Credential credential;
}
[3] = struct Credential {
 string Account;
 string Password;
}
[4] = struct DescribeResponse {
 string DescribeResult;
}
[5] = struct CheckAvailability {
 string sCLTRID;
 Credential credential;
 ArrayOfString sDomainArray;
 ArrayOfString sHostArray;
 ArrayOfString sNSArray;
}
[6] = struct ArrayOfString {
 string string;
}
[7] = struct CheckAvailabilityResponse {
 string CheckAvailabilityResult;
}
[8] = struct Info {
 string sCLTRID;
 Credential credential;
 string sResourceID;
 string sType;
 string sDomain;
 string sOrderID;
}
[9] = struct InfoResponse {
 string InfoResult;
}
[10] = struct NameGen {
 string sCLTRID;
 Credential credential;
 string sSLD;
 string sTLD;
 string sTLDs;
 unsignedShort nRows;
 boolean bHypens;
 boolean bNumbers;
 unsignedShort nMaxLen;
 boolean bIntelliSearch;
 unsignedShort nPage;
 string sTime;
}
[11] = struct NameGenResponse {
 string NameGenResult;
}
*//**
[12] = struct NameGenDB {
 string sCLTRID;
 Credential credential;
 string sKey;
 int nMaxResults;
 boolean bExcludeTaken;
 string sTLDs;
 boolean bAddPrefix;
 boolean bAddSuffix;
 boolean bAddDashes;
 boolean bAddRelated;
 boolean bAdvancedSplit;
 boolean bBaseOnTop;
}
[13] = struct NameGenDBResponse {
 string NameGenDBResult;
}
[14] = struct NameGenDBWithTimeLimit {
 string sCLTRID;
 Credential credential;
 string sKey;
 int nMaxResults;
 boolean bExcludeTaken;
 string sTLDs;
 boolean bAddPrefix;
 boolean bAddSuffix;
 boolean bAddDashes;
 boolean bAddRelated;
 boolean bAdvancedSplit;
 boolean bBaseOnTop;
 int nMaxWaitSeconds;
}
[15] = struct NameGenDBWithTimeLimitResponse {
 string NameGenDBWithTimeLimitResult;
}
[16] = struct Poll {
 string sCLTRID;
 Credential credential;
 string sOp;
}
[17] = struct PollResponse {
 string PollResult;
}
[18] = struct OrderCredits {
 string sCLTRID;
 Credential credential;
 Shopper shopper;
 ArrayOfOrderItem items;
 string sROID;
}
[19] = struct Shopper {
 string user;
 string pwd;
 string pwdhint;
 string email;
 string firstname;
 string lastname;
 string phone;
 string pin;
 string dbpuser;
 string dbppwd;
 string dbppwdhint;
 string dbpemail;
 string dbppin;
}
[20] = struct ArrayOfOrderItem {
 OrderItem OrderItem;
}
[21] = struct OrderItem {
 int productid;
 unsignedInt quantity;
 string riid;
 double duration;
}
*//**
[22] = struct Order {
}
[23] = struct OrderCreditsResponse {
 string OrderCreditsResult;
}
[24] = struct OrderDomains {
 string sCLTRID;
 Credential credential;
 Shopper shopper;
 ArrayOfDomainRegistration items;
 ArrayOfDomainByProxy dbpItems;
 string sROID;
}
[25] = struct ArrayOfDomainRegistration {
 DomainRegistration DomainRegistration;
}
[26] = struct DomainRegistration {
 OrderItem order;
 string sld;
 string tld;
 int period;
 ContactInfo registrant;
 Nexus nexus;
 ArrayOfNS nsArray;
 ContactInfo admin;
 ContactInfo billing;
 ContactInfo tech;
 int autorenewflag;
}
[27] = struct ContactInfo {
 string fname;
 string lname;
 string org;
 string email;
 string sa1;
 string sa2;
 string city;
 string sp;
 string pc;
 string cc;
 string phone;
 string fax;
}
[28] = struct Nexus {
 string category;
 string use;
 string country;
}
[29] = struct ArrayOfNS {
 NS NS;
}
[30] = struct NS {
 string name;
}
[31] = struct ArrayOfDomainByProxy {
 DomainByProxy DomainByProxy;
}
[32] = struct DomainByProxy {
 OrderItem order;
 string sld;
 string tld;
 string resourceid;
}
[33] = struct OrderDomainsResponse {
 string OrderDomainsResult;
}
[34] = struct OrderDomainRenewals {
 string sCLTRID;
 Credential credential;
 Shopper shopper;
 ArrayOfDomainRenewal items;
 string sROID;
}
*//**
[35] = struct ArrayOfDomainRenewal {
 DomainRenewal DomainRenewal;
}
[36] = struct DomainRenewal {
 OrderItem order;
 string resourceid;
 string sld;
 string tld;
 int period;
}
[37] = struct OrderDomainRenewalsResponse {
 string OrderDomainRenewalsResult;
}
[38] = struct OrderPrivateDomainRenewals {
 string sCLTRID;
 Credential credential;
 Shopper shopper;
 ArrayOfDomainRenewal items;
 ArrayOfResourceRenewal dbpItems;
 string sROID;
}
[39] = struct ArrayOfResourceRenewal {
 ResourceRenewal ResourceRenewal;
}
[40] = struct ResourceRenewal {
 OrderItem order;
 string resourceid;
}
[41] = struct OrderPrivateDomainRenewalsResponse {
 string OrderPrivateDomainRenewalsResult;
}
[42] = struct OrderDomainTransfers {
 string sCLTRID;
 Credential credential;
 Shopper shopper;
 ArrayOfDomainTransfer items;
 string sROID;
}
[43] = struct ArrayOfDomainTransfer {
 DomainTransfer DomainTransfer;
}
[44] = struct DomainTransfer {
 OrderItem order;
 string sld;
 string tld;
 string authInfo;
}
[45] = struct OrderDomainTransfersResponse {
 string OrderDomainTransfersResult;
}
*//**
[46] = struct OrderDomainPrivacy {
 string sCLTRID;
 Credential credential;
 Shopper shopper;
 ArrayOfDomainByProxy items;
 string sROID;
}
[47] = struct OrderDomainPrivacyResponse {
 string OrderDomainPrivacyResult;
}
[48] = struct OrderResourceRenewals {
 string sCLTRID;
 Credential credential;
 Shopper shopper;
 ArrayOfResourceRenewal items;
 string sROID;
}
[49] = struct OrderResourceRenewalsResponse {
 string OrderResourceRenewalsResult;
}
[50] = struct OrderDomainBackOrders {
 string sCLTRID;
 Credential credential;
 Shopper shopper;
 ArrayOfDomainBackOrder items;
 string sROID;
}
[51] = struct ArrayOfDomainBackOrder {
 DomainBackOrder DomainBackOrder;
}
[52] = struct DomainBackOrder {
 OrderItem order;
 boolean privateBackOrder;
 string sld;
 string tld;
 ContactInfo registrant;
 Nexus nexus;
 ContactInfo admin;
 ContactInfo billing;
 ContactInfo tech;
}
[53] = struct OrderDomainBackOrdersResponse {
 string OrderDomainBackOrdersResult;
}
[54] = struct Cancel {
 string sCLTRID;
 Credential credential;
 string sType;
 ArrayOfString sIDArray;
}
*//**
[55] = struct CancelResponse {
 string CancelResult;
}
[56] = struct UpdateDomainOwnership {
 string sCLTRID;
 Credential credential;
 string sResourceID;
 string sLosingOwner;
 GainingOwner gaining;
}
[57] = struct GainingOwner {
 ShopperNoDBP shopperNoDBP;
 ContactInfo registrant;
 ContactInfo admin;
 ContactInfo billing;
 ContactInfo tech;
}
[58] = struct ShopperNoDBP {
 string user;
 string pwd;
 string pwdhint;
 string email;
 string firstname;
 string lastname;
 string phone;
 string pin;
}
[59] = struct UpdateDomainOwnershipResponse {
 string UpdateDomainOwnershipResult;
}
[60] = struct ResetPassword {
 string sCLTRID;
 Credential credential;
 string sUser;
 string sPwd;
 string sEmail;
 string sPwdHint;
}
[61] = struct ResetPasswordResponse {
 string ResetPasswordResult;
}
[62] = struct SetShopperInfo {
 string sCLTRID;
 Credential credential;
 string sUser;
 string sEmail;
 string sFirstName;
 string sLastName;
 string sPhone;
 string sFax;
 string sCompany;
 string sAddress;
 string sCity;
 string sState;
 string sZip;
 string sCountry;
 string sShopperPin;
}
[63] = struct SetShopperInfoResponse {
 string SetShopperInfoResult;
}
[64] = struct CreateNewShopper {
 string sCLTRID;
 Credential credential;
 string sPwd;
 string sEmail;
 string sFirstName;
 string sLastName;
 string sPhone;
 string sPwdHint;
 string sFax;
 string sCompany;
 string sAddress;
 string sCity;
 string sState;
 string sZip;
 string sCountry;
 string sShopperPin;
}
*//**
[65] = struct CreateNewShopperResponse {
 string CreateNewShopperResult;
}
[66] = struct CheckUser {
 string sCLTRID;
 Credential credential;
 string sType;
 string sUser;
 string sPwd;
}
[67] = struct CheckUserResponse {
 string CheckUserResult;
}
[68] = struct SetupDomainAlert {
 string sCLTRID;
 Credential credential;
 DomainAlert domainAlert;
 ContactInfo registrant;
}
[69] = struct DomainAlert {
 string user;
 string domain;
 string email;
 string tld;
 int privateReg;
 int backorder;
}
[70] = struct SetupDomainAlertResponse {
 string SetupDomainAlertResult;
}
[71] = struct UpdateDomainAlert {
 string sCLTRID;
 Credential credential;
 DomainAlert domainAlert;
 string sDomainMonitorID;
 ContactInfo registrant;
}
[72] = struct UpdateDomainAlertResponse {
 string UpdateDomainAlertResult;
}
[73] = struct RemoveDomainAlert {
 string sCLTRID;
 Credential credential;
 string sDomainMonitorID;
 string sUser;
}
[74] = struct RemoveDomainAlertResponse {
 string RemoveDomainAlertResult;
}
[75] = struct GetDomainAlertCredits {
 string sCLTRID;
 Credential credential;
 string sUser;
}
[76] = struct GetDomainAlertCreditsResponse {
 string GetDomainAlertCreditsResult;
}
[77] = struct GetMonitoredDomainList {
 string sCLTRID;
 Credential credential;
 string sUser;
 int nPage;
 int nRowsPerPage;
}
[78] = struct GetMonitoredDomainListResponse {
 string GetMonitoredDomainListResult;
}
[79] = struct GetExpiringNameList {
 string sCLTRID;
 Credential credential;
 string sUser;
 int nLength;
 string sDashes;
 string sNumbers;
 string sWildCardSearch;
 string sSearchCriteria;
 string sTLDs;
 int nPage;
 int nRowsPerPage;
}
[80] = struct GetExpiringNameListResponse {
 string GetExpiringNameListResult;
}
*//**
[81] = struct DomainForwarding {
 string sCLTRID;
 Credential credential;
 string sUser;
 int nDomainID;
 string sForwardingURL;
 string sMngTRID;
}
[82] = struct DomainForwardingResponse {
 string DomainForwardingResult;
}
[83] = struct UpdateNameServer {
 string sCLTRID;
 Credential credential;
 ArrayOfDomain domainArray;
 ArrayOfNS nsArray;
}
[84] = struct ArrayOfDomain {
 Domain Domain;
}
[85] = struct Domain {
 string resourceid;
 string mngTRID;
 string transferKey;
 string newAuthInfo;
}
[86] = struct UpdateNameServerResponse {
 string UpdateNameServerResult;
}
[87] = struct UpdateDomainContact {
 string sCLTRID;
 Credential credential;
 ArrayOfDomain domainArray;
 ContactInfo registrant;
 ContactInfo admin;
 ContactInfo billing;
 ContactInfo tech;
}
[88] = struct UpdateDomainContactResponse {
 string UpdateDomainContactResult;
}
[89] = struct SetDomainLocking {
 string sCLTRID;
 Credential credential;
 ArrayOfDomain domainArray;
 string sLock;
}
[90] = struct SetDomainLockingResponse {
 string SetDomainLockingResult;
}
[91] = struct ManageTransfer {
 string sCLTRID;
 Credential credential;
 ArrayOfDomain domainArray;
 string sAction;
}
[92] = struct ManageTransferResponse {
 string ManageTransferResult;
}
[93] = struct UpdateDomainForwarding {
 string sCLTRID;
 Credential credential;
 ArrayOfDomain domainArray;
 string sAction;
 string sForwardingURL;
}
[94] = struct UpdateDomainForwardingResponse {
 string UpdateDomainForwardingResult;
}
*//**
[95] = struct UpdateDomainMasking {
 string sCLTRID;
 Credential credential;
 ArrayOfDomain domainArray;
 string sAction;
 string sForwardingURL;
 string sMaskingMetatagTitle;
 string sMaskingMetatagDesc;
 string sMaskingMetatagKeyword;
}
[96] = struct UpdateDomainMaskingResponse {
 string UpdateDomainMaskingResult;
}
[97] = struct ModifyDNS {
 string sCLTRID;
 Credential credential;
 ArrayOfDNSRequest dnsRequestArray;
 string sDomain;
}
[98] = struct ArrayOfDNSRequest {
 DNSRequest DNSRequest;
}
[99] = struct DNSRequest {
 string recType;
 string action;
 string key;
 int ttl;
 string recValue;
}
[100] = struct ModifyDNSResponse {
 string ModifyDNSResult;
}
[101] = struct ValidateRegistration {
 string sCLTRID;
 Credential credential;
 DomainContacts domainContacts;
 ArrayOfDomainData domainDataArray;
}
[102] = struct DomainContacts {
 ContactInfo registrant;
 ContactInfo admin;
 ContactInfo billing;
 ContactInfo tech;
}
[103] = struct ArrayOfDomainData {
 DomainData DomainData;
}
[104] = struct DomainData {
 string sld;
 string tld;
 int period;
}
[105] = struct ValidateRegistrationResponse {
 string ValidateRegistrationResult;
}


 */