<?php

abstract class RuCenterAction {
	
	const TEMPLATE_PATH = "templates";
	const TEMPLATE_EXTENSION = ".php";
	
	public function __construct() 
	{
		$className = get_class($this);
		$templateName = trim(str_replace(array('RuCenter', 'Action'), "", $className));
		$this->template = ($templateName && file_exists(self::getTemplatePath($templateName))) ? $templateName : false;
		$this->properties = $_REQUEST;
	}
	
	public function run() 
	{
		$this->execute();
		return $this->processTemplate();
	}
	
	public function setTemplate($name) 
	{
		$this->template = $name;
	}
	
	public function setProperty($name, $value) 
	{
		$this->properties[$name] = $value;
	}
	
	public function getProperty($name, $default = null) 
	{
		return isset($this->properties[$name]) ? $this->properties[$name] : $default;
	}
	
	public function __get($name)
	{
		return isset($this->properties[$name]) ? $this->properties[$name] : null;
	}
		
	public function __set($name, $value) 
	{
		$this->properties[$name] = $value;
	}
	
	protected function redirect($url) 
	{
		header("Location: " . $url);
		exit;
	}
	
	abstract protected function execute();
	
	private function processTemplate() 
	{
		$content = false;
		if ($this->template) {
			ob_start();
			extract($this->properties);
			include(self::getTemplatePath($this->template));
			$content = ob_get_clean();
		}
		return $content;
	}
	
	private static function getTemplatePath($name) 
	{
		return realpath(dirname(__FILE__)) . "/" . self::TEMPLATE_PATH . "/" . $name . self::TEMPLATE_EXTENSION;
	}
	
	private $template;
	private $properties = array();
	
}