<?php

class RuCenterMessageBody
{

	private $parameters;

	private $sections;

	public function __construct()
	{
		$this->parameters = array();
		$this->sections = array();
		$this->errors = array();
	}

	public function addParameter(RuCenterMessageParameter $parameter)
	{
		return $this->parameters[$parameter->getName()] = $parameter;
	}

	public function hasParameter($name)
	{
		return isset($this->parameters[$name]);
	}

	public function hasParameters()
	{
		return $this->parameters ? true : false;
	}

	public function getParameter($name)
	{
		return isset($this->parameters[$name]) ? $this->parameters[$name] : null;
	}

	public function getParameters()
	{
		return $this->parameters;
	}

	public function addSection(RuCenterMessageSection $section)
	{
		return $this->sections[] = $section;
	}

	public function hasSection($name)
	{
		$result = false;
		foreach ($this->getSections() as $section) {
			if ($section->getName() == $name) {
				$result = true;
				break;
			}
		}
		return $result;
	}

	public function hasSections()
	{
		return $this->sections ? true : false;
	}

	public function getSection($name)
	{
		$result = null;
		foreach ($this->getSections() as $section) {
			if ($section->getName() == $name) {
				$result = $section;
				break;
			}
		}
		return $result;
	}

	public function getSectionsByName($name)
	{
		$sections = array();
		foreach ($this->getSections() as $section) {
			if ($section->getName() == $name) {
				$sections[] = $section;
			}
		}
		return $sections;
	}

	public function getSections()
	{
		return $this->sections;
	}

	public function __get($name)
	{
		$methodName = "get" . ucfirst($name);
		if (method_exists($this, $methodName)) {
			return $this->$methodName();
		} else {
			$itemName = str_replace("_", "-", $name);
			if ($this->hasParameter($itemName)) {
				return $this->getParameter($itemName);
			} else if ($this->hasSection($itemName)) {
				return $this->getSection($itemName);
			} else {
				return null;
			}
		}
	}

	public function __set($name, $value)
	{
		$itemName = str_replace("_", "-", $name);
		if (!$this->hasParameter($itemName)) {
			$parameter = $this->addParameter(new RuCenterMessageParameter($itemName));
		} else {
			$parameter = $this->getParameter($itemName);
			$parameter->clean;
		}
		$values = is_array($value) ? $value : array($value);
		foreach ($values as $value) {
			$parameter->addValue($value);
		}
	}

}