<?php

class RuCenterMessageSection
{

	private $name;

	private $parameters;

	public function __construct($name)
	{
		$this->name = $name;
		$this->parameters = array();
		for ($i = 1; $i < func_num_args(); $i++) {
			$this->addParameter(func_get_arg($i));
		}
	}

	public function addParameter(RuCenterMessageParameter $parameter)
	{
		return $this->parameters[$parameter->getName()] = $parameter;
	}

	public function getParameter($name)
	{
		return isset($this->parameters[$name]) ? $this->parameters[$name] : null;
	}

	public function hasParameter($name)
	{
		return isset($this->parameters[$name]);
	}

	public function getName()
	{
		return $this->name;
	}

	public function getParameters()
	{
		return $this->parameters;
	}

	public function __get($name)
	{
		$methodName = "get" . ucfirst($name);
		if (method_exists($this, $methodName)) {
			return $this->$methodName();
		} else {
			$itemName = str_replace("_", "-", $name);
			if ($this->hasParameter($itemName)) {
				return $this->getParameter($itemName);
			} else {
				return null;
			}
		}
	}

	public function __set($name, $value)
	{
		$itemName = str_replace("_", "-", $name);
		if (!$this->hasParameter($itemName)) {
			$parameter = $this->addParameter(new RuCenterMessageParameter($itemName));
		} else {
			$parameter = $this->getParameter($itemName);
			$parameter->clean;
		}
		$values = is_array($value) ? $value : array($value);
		foreach ($values as $value) {
			$parameter->addValue($value);
		}
	}

}