<?php

class RuCenterEditDNSAction extends RuCenterAction {
	
	const EMTPY_ROWS_COUNT = 3;
	const MODE_DISPLAY = "display";
	const MODE_PROCESS = "process";
	
	protected function execute() {
		switch ($this->_mode) {
			case self::MODE_DISPLAY:
				$this->display();
				break;
			case self::MODE_PROCESS:
				$this->process();
				break;
			default:
				throw new Exception("Unknown mode: " . $this->_mode);
		}
	}
	
	private function process() 
	{
		$header = new RuCenterMessageHeader();
		$header->lang = RuCenterModule::getLang();
		$header->login = $this->_login;
		$header->password = $this->_password;
		$header->request = "order";
		$header->operation = "create";
		$header->subject_contract = $this->_subjectContract;
		$header->request_id = RuCenterModule::generateRequestId();
		
		$body = new RuCenterMessageBody();
		$orderItemSection = $body->addSection(new RuCenterMessageSection("order-item"));
		$orderItemSection->service = RuCenterModule::getServiceByTLD($this->_tld);
		$orderItemSection->template = RuCenterModule::getTemplateByTLD($this->_tld);
		$orderItemSection->action = "update";
		$orderItemSection->domain = $this->_domain;
		$nameServersParameter = $orderItemSection->addParameter(new RuCenterMessageParameter("nserver"));
		
		foreach ($this->getProperty("nameServers", array()) as $nameServer) {
			if ($nameServer = trim($nameServer)) {
				$nameServersParameter->addValue($nameServer);
			}
		}
		
		$request = new RuCenterRequest($header, $body);
		$responce = $request->send();
		if ($responce->hasErrors()) {			
			$this->errors = $responce->getErrors();
		} else if ($responce->getState() != 200) {
			$this->errors = array($responce->header->State->value);
		}
	}
	
	private function display() 
	{
		$header = new RuCenterMessageHeader();
		$header->lang = RuCenterModule::getLang();
		$header->login = $this->_login;
		$header->password = $this->_password;
		$header->request = "service-object";
		$header->operation = "search";
		$header->subject_contract = $this->_subjectContract;
		
		$body = new RuCenterMessageBody();
		$serviceObjectSection = $body->addSection(new RuCenterMessageSection("service-object"));
		$serviceObjectSection->service = RuCenterModule::getServiceByTLD($this->_tld);
		$serviceObjectSection->domain = $this->_domain;
		$serviceObjectSection->service_objects_limit = 1;
		$serviceObjectSection->object_only = true;
		
		$request = new RuCenterRequest($header, $body);
		
		try {
			$responce = $request->send();
			$nameServers = $responce->body->service_object->nameservers->values;
		} catch (Exception $e) {
			$this->errors = array($e->getMessage());
		}
		for ($i = 0; $i < self::EMTPY_ROWS_COUNT; $i++) {
			$nameServers[] = "";
		}
		$this->nameServers = $nameServers;
	}
	
}