<?php

require_once("lib-mbapi/include/modules/registrar/registrar.php");

$className = "distributeit";
$GLOBALS["moduleInfo"][$className] = array(
    "type" => "registrar",
    "status" => Registrar::STATUS_STABLE,
    "version" => "1.0.1",
    "displayName" => "distribute.IT",
    "author" => "Distribute IT Pty Ltd",
	"file" => __FILE__,
	"countryCodes" => array("AU", "NZ"),
);

class distributeit extends Registrar
{
	/**
	 * Returns the module display for the DistributeIT module. This includes version number and display name.
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the display info for the module.
	 */
    function getModuleInfo() {
        return $GLOBALS["moduleInfo"]["distributeit"];
    }

	/**
	 * Construct a new DistributeIT object. This function merely sets up capabilities after calling the
	 * constructor for Registrar.
	 */
	function distributeit($input) {

		$this->capabilities[REGISTRAR_REGISTERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_TRANSFERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_RENEWDOMAIN] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINEXPDATE] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINEXTENDEDINFO] = 1;
		$this->capabilities[REGISTRAR_CHECKDOMAINAVAILABILITY] = 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINTRANSFERAVAILABILITY] = 1;

		//Domain Management options
		$this->capabilities[REGISTRAR_GETDOMAINCONTACTDATA] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINCONTACTDATA] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWTYPEOPTIONS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINRENEWTYPEOPTIONS] = 0;
		$this->capabilities[REGISTRAR_GETDOMAINREGISTRARLOCKOPTIONS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINREGISTRARLOCKOPTIONS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINACCESSPASSWORD] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINACCESSPASSWORD] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWDATE] = 1;
		$this->capabilities[REGISTRAR_LIVERENEW] = 1;

		$this->capabilities[REGISTRAR_CHECKACCOUNTBALANCE] = 1;
		$this->capabilities[REGISTRAR_GETTLDPRICING] = 0;
		$this->capabilities[REGISTRAR_GETTRANSFERSTATUS] = 1;
		$this->capabilities[REGISTRAR_GETTLDLIST] = 1;
		$this->capabilities[REGISTRAR_TLDS] = array("ac.nz"   , "asn.au", "biz"   , "ca"     , "cc"    , "cn"   , "co.nz" ,
		                                            "co.uk"   , "com"   , "com.au", "geek.nz", "gen.nz", "id.au", "info"  ,
		                                            "maori.nz", "name"  , "net"   , "net.au" , "net.nz", "org"  , "org.au",
		                                            "org.nz"  , "org.uk", "school.nz",         "tv"    , "us"   , "mobi");

		parent::Registrar($input);
	}

	/**
	 * This function merely sets the DistributeIT connection URL.
	 */
  function getURL($type = '') {
      global $CONF;

    	//$DistributeITTestMode = $CONF["domain"]["registrar"][strtolower($this->stringRep)][$this->getConfigID()]["moduleTestMode"];

			$url = "https://www.distributeit.com.au/api/";
      switch ($type) {
        case 'auth':
          $url .= "auth.pl";
          break;
        case 'query':
				  $url .= "query.pl";
          break;
        default:
				  $url .= "order.pl";
      }
      return $url;
  }

    /**
     * Returns null or sessionid
     */
	  function getSessionId() {
			global $CONF;
			$url = $this->getURL('auth');
	    $c = $this->getCurlObject();
	    $c->setURL($url);
	    $c->addField("AccountNo", $this->config["moduleAccountNo"]);
	    $c->addField("UserId"   , $this->UID);
	    $c->addField("Password" , $this->PW);
	    $c->addField("Requester", 'ModernBill');

      $result = $c->toString();
      $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::inside::getSessionId() ~ $c->toString', htmlentities ($result), __FILE__, 130, 'MBAPI_ProcessRegistrarCommand');

			if ((strpos($result, "ERR") !== false) || !$result) {
				return null;
			} else {
	  		$res = explode(": ", $result);
				//$res[0]; // contains 'OK'
				//$res[1]; // contains the sessionid
				return $res[1];
			}
	  }


    /**
     * Checks one or more domains
     */
    function checkDomainAvailability ()
    {
      global $CONF;
			if ($sessionid = $this->getSessionId()) {
	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('query'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"Availability");
  	    $c->addField("Requester",'ModernBill');

        $domains = $this->input['domainSLD'] . '.' . $this->input['domainTLD'];
        if ((int)$this->input['checkAllTLDs']) {
  	      foreach ($this->capabilities[REGISTRAR_TLDS] as $tld)	{
  	      	if (strpos($tld, $this->input['domainTLD']) !== false) {
 	            $domains .= ',' . $this->input['domainSLD'] . '.' . $tld;
 	          }
        	}
        }
        // (int)$this->input['checkNameSpinner']
        $c->addField("Domain"   ,$domains);

        $resultString = $c->toString();

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

		    $result = preg_split('/\n/', $resultString);
		    reset($result);

        $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::checkDomainAvailability() ~ $c->toString', $result, __FILE__, 177, 'MBAPI_ProcessRegistrarCommand');

      	if ($result) {
	        if (strpos(current($result), "ERR: ") === false || preg_match('/304/i', current($result)))
	        {
	          $this->arrayData['OriginalDomain'] = $this->input['domainSLD'] . '.' . $this->input['domainTLD'];
	          $this->arrayData['SuggestedDomainCount'] = 0;
	          $this->arrayData['OriginalDomain'] = $data['interface-response'][0]['Domain1'][0];
	          if (strpos(current($result), "OK: ") !== false)
	          {
	            $this->arrayData['OriginalDomainIsAvailable'] = 1;
	          }
	          else
	          {
	            if (strpos(current($result), "ERR: 304") !== false)
	            {
	              $this->arrayData['OriginalDomainIsAvailable'] = 0;
	            }
	            else
	            {
	              $this->arrayData['OriginalDomainIsAvailable'] = 0;
	              $this->arrayData['OriginalDomainAvailabilityUnknown'] = 1;
	            }
	          }

	          if ($this->input['checkAllTLDs'])
	          {
	            $arrayElement = 0;
	            while (next($result))
	            {
	            	$dn = explode(":",current($result),2);
	              $data['SuggestedName' . $arrayElement] = $dn[0];
	              if (strpos(current($result), "OK: ") !== false) {
	                $data['DomainIsAvailable' . $arrayElement] = 1;
	              } else {
	                if (strpos(current($result), "ERR: 304") !== false) {
	                  $data['DomainIsAvailable' . $arrayElement] = 0;
	                } else {
	                  $data['DomainIsAvailable' . $arrayElement] = 0;
	                  $data['DomainAvailabilityUnknown' . $arrayElement] = 1;
	                }
	              }
	              ++$arrayElement;
	            }
	            $this->arrayData['SuggestedDomainCount'] = $arrayElement;
	            $this->arrayData['SuggestedDomains'] = $data;
	          }
	          $this->hasExecutedSuccessfully = 1;
	        } else {
	          $this->errors[] = array ('DIT Error:', current($result));
	          ++$this->numErrors;
	          $this->hasExecutedSuccessfully = 0;
	        }
	      } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
          $this->hasExecutedSuccessfully = 0;
	      }
	    } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
        $this->hasExecutedSuccessfully = 0;
	    }

	      $this->hasExecuted = 1;
	      $this->xmlData = $this->getRegistrarReturnXML ();
	      return $this->xmlData;
    }



	/**
	 * Call the registrar to check a domain's availability for transfer.
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function checkDomainTransferAvailability() {
		global $CONF;

		if ($sessionid = $this->getSessionId()) {
      $c = $this->getCurlObject();
      $c->setURL($this->getURL('query'));
      $c->addField("SessionID",$sessionid);
      $c->addField("Type"     ,"Domains");
      $c->addField("Object"   ,"Domain");
      $c->addField("Action"   ,"Transfer");
 	    $c->addField("Requester",'ModernBill');
      $c->addField("Domain"   ,$this->input['domainSLD'] . '.' . $this->input['domainTLD']);

      $result = $c->toString();

      $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::checkDomainTransferAvailability() ~ $c->toString', $result, __FILE__, 267, 'MBAPI_ProcessRegistrarCommand');

      $this->connectionInfo[] = &$c->connectionInfo;
	    $this->getCurlErrors ($c);

      $data = array();

     	if ($result) {
        if (strpos($result, "ERR: ") === false)
        {
		  		$res = explode(": ", $result);
					//$res[0]; // contains domain
					//$res[1]; // contains 'OK'
					//$res[2]; // contains min, max years, owner, email, expiry date

	        $this->arrayData['DomainName']        = $res[0];
  	      $this->arrayData['DomainRegistrar']   = 'Known';
    	    $this->arrayData['DomainInAccount']   = 0;
      	  $this->arrayData['DomainLastOrderID'] = 0;
        	$this->arrayData['OtherData']         = $res[2];
        	$this->hasExecutedSuccessfully = 1;

        } else {
          $this->errors[] = array ('DIT Error:', $result);
          ++$this->numErrors;
          $this->hasExecutedSuccessfully = 0;
        }
      } else {
        $this->errors[] = array ('DIT Error: No answer');
	      ++$this->numErrors;
        $this->hasExecutedSuccessfully = 0;
      }
    } else {
      $this->errors[] = array ('DIT Error: Auth Error');
      ++$this->numErrors;
      $this->hasExecutedSuccessfully = 0;
    }

    $this->hasExecuted = 1;
    $this->xmlData = $this->getRegistrarReturnXML ();
    return $this->xmlData;
  }




/********************************************************
*
* splits a phone or fax number in its components
* @return number components : seperated
********************************************************/
  function _splitPhone($number) {
			$countyprefix = array (
						'1'  , '20' , '212', '213', '216', '218', '220', '221', '222', '225', '226', '227', '228',
						'229', '230', '231', '232', '233', '234', '235', '236', '237', '238', '239', '240', '241',
						'242', '244', '245', '248', '249', '250', '251', '252', '253', '254', '255', '256', '257',
						'258', '260', '261', '263', '264', '265', '266', '267', '268', '269', '27' , '290', '291',
						'297', '299', '30', '31', '32', '33', '34', '350', '351', '352', '353', '354', '355', '356',
						'357', '358', '359', '36' , '370', '371', '372', '374', '375', '376', '377', '378', '379',
						'380', '385', '386', '387', '389', '39', '40', '41', '420', '421', '423', '43', '44', '45',
						'46', '47', '48', '49', '500', '501', '502', '503', '504', '505', '506', '507', '508', '509',
						'51', '52', '53', '54', '55', '56', '58', '590', '591', '592', '593', '594', '595', '596',
						'597', '598', '599', '60', '61', '62', '63', '64', '65', '66', '670', '672', '673', '674',
						'675', '676', '677', '678', '679', '680', '681', '682', '683', '685', '686', '687', '688',
						'689', '690', '691', '692', '7', '81', '82', '84', '850', '852', '853', '855', '856', '86',
						'880', '886', '90', '91', '92', '93', '94', '95', '960', '961', '962', '963', '964', '965',
						'966', '967', '968', '971', '972', '973', '974', '975', '976', '977', '98', '992', '993',
						'994', '995', '996', '998');

			// todo: maybe use countries_phone_code from countries;

			// country code
			if (substr($number, 0, 1) === '+' || substr($number, 0, 2) === '00') {
				// there is a country code

				// remove all nondigits
				$number = preg_replace('/\D/', '', $number);

				// remove double 0 at start
				if (substr($number, 0, 2) === '00') {
					$number = substr($number, 2);
				}
				if (in_array (substr($number, 0, 1), $countyprefix)) {
					$countrycode = substr($number, 0, 1);
					$number = substr($number, 1);
				} elseif (in_array (substr($number, 0, 2), $countyprefix)) {
					$countrycode = substr($number, 0, 2);
				  $number = substr($number, 2);
				} elseif (in_array (substr($number, 0, 3), $countyprefix)) {
					$countrycode = substr($number, 0, 3);
					$number = substr($number, 3);
				} else {
					// no valid country
					$countrycode = '';
				}
			} else {
  			// remove all nondigits
				$number = preg_replace('/\D/', '', $number);
			}

			// area code
			if (substr($number, 0, 1) === '0') {
				$areacode = substr($number, 0, 2);
				$number = substr($number, 2);
			} else {
				$areacode = '0' . substr($number, 0, 1);
				$number = substr($number, 1);
			}

			return $countrycode . ":" . $areacode . ":" . $number;
  }


/********************************************************
* Create Contacts
* @return contactids : seperated
********************************************************/

  function _createContacts($all = 1) {
  	  global $CONF;
  		$map = $this->getAttributesMap();

      if (is_array($this->input['domainContactData'])) {
	      $validContactTypes = array ('registrant', 'tech', 'admin', 'auxBilling');

  	    $newContactData = collapsearrayelements ($this->input['domainContactData']);
        foreach ($newContactData as $contactType => $contactData)
        {
          //$CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::inside::_createContacts() ~ UpdateDomain ' . $contactType, $newContactData[$contactType], __FILE__, 395, 'MBAPI_ProcessRegistrarCommand');
          if (in_array ($contactType, $validContactTypes))
          {
            foreach ($map as $k => $v)
            {
              if ($v)
              {
                if (0 < strlen ($newContactData[$contactType][$k]))
                {
	                $postValues[$contactType][$v] = $newContactData[$contactType][$k];
                }
              }
            }
						/* handle phone and fax details....*/
						$res = explode (':', $this->_splitPhone($postValues[$contactType]['Phone']));
		        $postValues[$contactType]['PhoneCountryCode'] = $res[0];
		        $postValues[$contactType]['PhoneAreaCode']    = $res[1];
		        $postValues[$contactType]['PhoneNumber']      = $res[2];
		        if ($postValues[$contactType]['PhoneExt']) {
		          $postValues[$contactType]['PhoneNumber'] .= $postValues[$contactType]['PhoneExt'];
		        }
		        if ($postValues[$contactType]['Fax']) {
		        	$res = explode (':', $this->_splitPhone($postValues[$contactType]['Fax']));
		          $postValues[$contactType]['FaxCountryCode'] = $res[0];
		          $postValues[$contactType]['FaxAreaCode']    = $res[1];
		 	        $postValues[$contactType]['FaxNumber']      = $res[2];
		 	      }
		 	      if (! $postValues[$contactType]['Region']) {
		 	      	$postValues[$contactType]['Region'] = 'n/a';
		 	      }
	          //$CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::inside::_createContacts() ~ UpdateDomain ' . $contactType, $postValues[$contactType], __FILE__, 425, 'MBAPI_ProcessRegistrarCommand');
					}
				}
      } // end domain update handling
      else {
	      // $validContactTypes = array ('registrant', 'tech', 'admin', 'auxbilling');
	      // build up an array for the contacts we need to create
	      // registrant has always to be created
	      $validContactTypes[] = 'registrant';
	      if ($CONF["domain"]["registrar"][strtolower($this->stringRep)][$this->getConfigID()]["useDefaultAdmin"]) {
	      	$validContactTypes[] = 'admin';
	      }
	      if ($CONF["domain"]["registrar"][strtolower($this->stringRep)][$this->getConfigID()]["useDefaultTech"]) {
	      	$validContactTypes[] = 'tech';
	      }
	      if ($CONF["domain"]["registrar"][strtolower($this->stringRep)][$this->getConfigID()]["useDefaultBilling"]) {
	      	$validContactTypes[] = 'auxBilling';
	      }

				// Loop the map to get the registrant details
				foreach($validContactTypes as $contactType) {
					foreach($map as $k => $v) {
						if(eregi($contactType, $k)) {
							// if this element has a value it's part of this panel's API, but only add it if it's set
							if($contactType === 'registrant') {
								if($v && strlen($this->input[$k])>0) {
								  $postValues[$contactType][$v] = $this->input[$k];
							  }
							} else {
								if($v && strlen($this->config[$k])>0) {
								  $postValues[$contactType][$v] = $this->config[$k];
								}
							}
						}
					}
					/* handle phone and fax details....*/
					$res = explode (':', $this->_splitPhone($postValues[$contactType]['Phone']));
	        $postValues[$contactType]['PhoneCountryCode'] = $res[0];
	        $postValues[$contactType]['PhoneAreaCode']    = $res[1];
	        $postValues[$contactType]['PhoneNumber']      = $res[2];
	        if ($postValues[$contactType]['PhoneExt']) {
	          $postValues[$contactType]['PhoneNumber'] .= $postValues[$contactType]['PhoneExt'];
	        }
	        if ($postValues[$contactType]['Fax']) {
	        	$res = explode (':', $this->_splitPhone($postValues[$contactType]['Fax']));
	          $postValues[$contactType]['FaxCountryCode'] = $res[0];
	          $postValues[$contactType]['FaxAreaCode']    = $res[1];
	 	        $postValues[$contactType]['FaxNumber']      = $res[2];
	 	      }
				}
			} // end domain reg handling


      /* create contacts.... */
			if ($sessionid = $this->getSessionId()) {
   			foreach($validContactTypes as $contactType) {

		      $c = $this->getCurlObject();
		      $c->setURL($this->getURL('order'));
		      $c->addField("SessionID",$sessionid);
		      $c->addField("Type"     ,"Domains");
		      $c->addField("Object"   ,"Contact");
		      $c->addField("Action"   ,"Create");
		      $c->addField("Requester",'ModernBill');
          $c->addMultiFields ($postValues[$contactType]);
		      $result = $c->toString();

          $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::inside::_createContacts() ~ $postValues[$contactType]' . $contactType, $postValues[$contactType], __FILE__, 492, 'MBAPI_ProcessRegistrarCommand');
          $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::inside::_createContacts() ~ $c->toString ', htmlentities ($result), __FILE__, 493, 'MBAPI_ProcessRegistrarCommand');

	        $this->connectionInfo[] = &$c->connectionInfo;
		      $this->getCurlErrors ($c);

	     	  if ($result) {
	          if (strpos($result, "ERR: ") === false)
	          {
			  		  $res = explode(": ", $result);
						  //$res[0]; // contains 'OK'
						  //$res[1]; // contains contact id
              switch ($contactType) {
                case 'registrant':
                  $registrantid = $res[1];
                break;
                case 'tech':
                  $techid = $res[1];
                break;
                case 'admin':
                  $adminid = $res[1];
                break;
                case 'auxBilling':
                  $billingid = $res[1];
                break;
              }
            }
          }
        }

        if (!($registrantid || $techid || $adminid || $billingid)) {
        	$error = 'no contacts created';
	      } elseif ($all) {
	        if (!$registrantid) {
	        	if ($adminid) {
	        		$registrantid = $adminid;
	        	} elseif ($billingid) {
	        		$registrantid = $billingid;
	        	} elseif ($techid) {
	        		$registrantid = $techid;
	        	}
	        }

	        if (!$techid) {
	        	if ($registrantid) {
	        		$techid = $registrantid;
	        	} elseif ($adminid) {
	        		$techid = $adminid;
	        	} elseif ($billingid) {
	        		$techid = $billingid;
	        	}
	        }

	        if (!$adminid) {
	        	if ($registrantid) {
	        		$adminid = $registrantid;
	        	} elseif ($billingid) {
	        		$adminid = $billingid;
	        	} elseif ($techid) {
	        		$adminid = $techid;
	        	}
	        }

	        if (!$billingid) {
	        	if ($registrantid) {
	        		$billingid = $registrantid;
	        	} elseif ($adminid) {
	        		$billingid = $adminid;
	        	} elseif ($techid) {
	        		$billingid = $techid;
	        	}
	        }
	      }

	    } else {
	    	$error = 'Auth Error';
      }

      return "$error:$registrantid:$techid:$adminid:$billingid";
  }





	/**
	 * Call the registrar to register a new domain
	 * @return  string  XML containing registration results for this domain registration
	 */
	function registerDomain() {
	 	global $CONF;
      $validDomainParams = array ('SLD',
                                  'TLD',
                                  'Period',
                                  'UnLockRegistrar',
                                  'DomainPassword',
                                  'UseDNS',
                                  'NS1',
                                  'NS2',
                                  'NS3',
                                  'NS4',
                                  'NS5',
                                  'NS6',
                                  'NS7',
                                  'NS8',
                                  'NS9',
                                 );


      $postValues = $this->getPostValues ($validDomainParams);
      if ($postValues['UseDNS'] == 'default')
      {
        $postValues[NS1] = $this->config[domainNameserver1];
        $postValues[NS2] = $this->config[domainNameserver1];
        $postValues[NS3] = $this->config[domainNameserver1];
        $postValues[NS4] = $this->config[domainNameserver1];
        unset ($postValues[NS5]);
        unset ($postValues[NS6]);
        unset ($postValues[NS7]);
        unset ($postValues[NS8]);
        unset ($postValues[NS9]);
      }


      if (eregi ('au', $this->input['domainTLD']))
      {
        $RegistrantID      = preg_replace('/\D/', '', $this->input['AU_reg_id']);
        $EligibilityID     = preg_replace('/\D/', '', $this->input['AU_elig_id']);

        $RegistrantName    = $this->input['AU_reg_name'];
        $RegistrantIDType  = substr($this->input['AU_reg_id_type']     , 1);
        $EligibilityIDType = substr($this->input['AU_reg_elig_id_type'], 1);
        $EligibilityName   = $this->input['AU_elig_name'];
        $EligibilityType   = substr($this->input['AU_reg_elig_type']   , 1);
        $EligibilityReason = substr($this->input['AU_reg_elig_reasion'], 1);

        if (! $this->input['AU_reg_name']) {
          $this->addError (TRANS_REGISTRARERROR, 'No Registrant Name provided for .au domain.');
	        ++$this->numErrors;
        }
        // check .id.au specific fields
	      if (eregi ('id.au', $this->input['domainTLD'])) {
	        if ($RegistrantIDType != 3) {
	          $this->addError (TRANS_REGISTRARERROR, 'Woring Registrant ID Type for .id.au domain.');
		        ++$this->numErrors;
	        }
	        if ($EligibilityType != 2) {
	          $this->addError (TRANS_REGISTRARERROR, 'Woring Eligibility Type for .id.au domain.');
		        ++$this->numErrors;
	        }
	        if ($EligibilityReason != 1) {
	          $this->addError (TRANS_REGISTRARERROR, 'Woring Eligibility Reason for .id.au domain.');
		        ++$this->numErrors;
	        }
        } else {
        	// all other .au extenstions than .id.au
          if      ($RegistrantIDType == 1) { # ABN
		        if (strlen($RegistrantID) != 11) {
		          $this->addError (TRANS_REGISTRARERROR, 'Woring Registrand ID for .au domain.');
			        ++$this->numErrors;
		        }
          } elseif ($RegistrantIDType == 2) { # ACN
		        if (strlen($RegistrantID) != 9) {
		          $this->addError (TRANS_REGISTRARERROR, 'Woring Registrand ID for .au domain.');
			        ++$this->numErrors;
		        }
          } elseif ($RegistrantIDType == 3) { # Other
          	if (eregi ('com.au', $this->input['domainTLD']) || eregi ('net.au', $this->input['domainTLD']))
		        if (!$RegistrantID || strlen($RegistrantID) < 5) {
		          $this->addError (TRANS_REGISTRARERROR, 'Woring / missing Registrand ID for .com.au / .net.au domain.');
			        ++$this->numErrors;
		        }
          } else {
	          $this->addError (TRANS_REGISTRARERROR, 'Woring / missing Registrand ID Type for .au domain.');
		        ++$this->numErrors;
          }
          if ($EligibilityID || $EligibilityName || $EligibilityIDType) {
             if      ($EligibilityIDType == 1) { # ACN
				        if (strlen($EligibilityID) != 9) {
				          $this->addError (TRANS_REGISTRARERROR, 'Woring Eligibility ID for .au domain.');
					        ++$this->numErrors;
				        }
             } elseif ($EligibilityIDType == 12) {
				        if (strlen($EligibilityID) != 11) {
				          $this->addError (TRANS_REGISTRARERROR, 'Woring Eligibility ID for .au domain.');
					        ++$this->numErrors;
				        }
             } elseif ($EligibilityIDType > 1 && $EligibilityIDType < 12) {
				        if (!$EligibilityID || strlen($EligibilityID) < 5) {
				          $this->addError (TRANS_REGISTRARERROR, 'Woring / missing Eligibility ID for .au domain.');
					        ++$this->numErrors;
				        }
             } else {
			          $this->addError (TRANS_REGISTRARERROR, 'Woring / missing Eligibility ID Type for .au domain.');
				        ++$this->numErrors;
             }
             if (!$EligibilityName) {
			          $this->addError (TRANS_REGISTRARERROR, 'Woring / missing Eligibility Name for .au domain.');
				        ++$this->numErrors;
             }
          }
          if (!$EligibilityType || $EligibilityType > 26 || $EligibilityType < 1) {
	          $this->addError (TRANS_REGISTRARERROR, 'Woring / missing Eligibility Type for .au domain.');
		        ++$this->numErrors;
          }
          if (!$EligibilityReason || $EligibilityReason >  2 || $EligibilityReason < 1) {
	          $this->addError (TRANS_REGISTRARERROR, 'Woring / missing Eligibility Reason for .au domain.');
		        ++$this->numErrors;
          }
        }
      }

      /* when an error happend, return */
      if ($this->numErrors) {
        $this->hasExecutedSuccessfully = 0;
        $this->hasExecuted = 0;
        $this->xmlData = $this->getRegistrarReturnXML ();
        return $this->xmlData;
      }

      $contacts = explode(":", $this->_createContacts());
      if ($contacts[0]) {
        $this->errors[] = array ('DIT Error: ' . $contacts[0]);
	      ++$this->numErrors;
        $this->hasExecutedSuccessfully = 0;
        $this->hasExecuted = 0;
        $this->xmlData = $this->getRegistrarReturnXML ();
        return $this->xmlData;
      } else {
        $registrantid = $contacts[1];
        $techid       = $contacts[2];
        $adminid      = $contacts[3];
        $billingid    = $contacts[4];
      }

      $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::inside::registerDomain() ~ contacts created ', $contacts, __FILE__, 727, 'MBAPI_ProcessRegistrarCommand');

			if ($sessionid = $this->getSessionId()) {
	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('order'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"Create");
	      $c->addField("Requester",'ModernBill');
	      $domain = $this->input['domainSLD'] . '.' . $this->input['domainTLD'];
	      $c->addField("Domain"   ,$domain);
	      $c->addField("Period"   ,$this->input['domainYears']);
        $accessUserId = substr(preg_replace('/\W/', '', $domain), 0, 10);
	      $c->addField("UserID"   ,$accessUserId);
	      if ($this->input[domainAccessPassword]) {
	      	$pw = $this->input[domainAccessPassword];
	      } else {
	        $pw = md5(uniqid(rand(), true));
	      }
	      $pw = substr(preg_replace('/\W/', '', $pw), 0, 10);
	      $c->addField("Password" ,$pw);

	      $c->addField("OwnerContactID"          ,$registrantid);
	      $c->addField("AdministrationContactID" ,$adminid);
	      $c->addField("TechnicalContactID"      ,$techid);
	      $c->addField("BillingContactID"        ,$billingid);

        if (eregi ('.au', $this->input['domainTLD'])) {
	        $c->addField('RegistrantName'    ,$RegistrantName);
	        $c->addField('RegistrantID'      ,$RegistrantID);
	        $c->addField('RegistrantIDType'  ,$RegistrantIDType);
	        $c->addField('EligibilityName'   ,$EligibilityName);
	        $c->addField('EligibilityID'     ,$EligibilityID);
	        $c->addField('EligibilityIDType' ,$EligibilityIDType);
	        $c->addField('EligibilityType'   ,$EligibilityType);
	        $c->addField('EligibilityReason' ,$EligibilityReason);
	      }

        for ($i = 1; $i < 10; $i++) {
          if (strlen($postValues['NS'.$i]) > 0) {
          	$hosts .= $postValues['NS'.$i] . ',';
          }
        }
        if (strlen($hosts) > 0) {
  	      $c->addField("Host"   ,$hosts);
  	    }

	      if ($this->config[ditCCNumber] && $this->config[ditCCExpiryDate]) {
		      $c->addField("CreditCardSupplierCode" ,$this->config[ditCCSupplierID]);
		      $c->addField("CreditCardNumber"       ,$this->config[ditCCNumber]);
		      $c->addField("CreditCardExpiry"       ,$this->config[ditCCExpiryDate]);
		    }

        $result = $c->toString();

        $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::registerDomain() ~ $c->toString xyz', htmlentities ($result), __FILE__, 783, 'MBAPI_ProcessRegistrarCommand');

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

	     	if ($result) {
  	      if (eregi ('OK: ', $result))
    	    {
		  			$res = explode(": ", $result);
						//$res[0]; // contains 'OK'
						//$res[1]; // contains order id

            $this->arrayData['domainRemotePurchaseID'] = $res[1];
            $this->arrayData['PurchaseIsSuccess'] = 1;
            $this->arrayData['OtherData'] = $result;
            $this->hasExecutedSuccessfully = 1;
          } else {
            $this->arrayData['PurchaseIsSuccess'] = 0;
	          $this->errors[] = array ('DIT Error:', $result);
            ++$this->numErrors;
            $this->hasExecutedSuccessfully = 0;
          }
        } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
	        $this->hasExecutedSuccessfully = 0;
        }
      } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
        $this->hasExecutedSuccessfully = 0;
      }
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;
    }


	/**
	 * Call the registrar to transfer an existing domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function transferDomain() {
			global $CONF;

			//Username is in $this->UID
			//Password is in $this->PW

		  $validDomainParams = array("SLD","TLD","Period","DomainPassword");
      $postValues = $this->getPostValues($validDomainParams);

      $contacts = explode(":", $this->_createContacts());
      if ($contacts[0]) {
        $this->errors[] = array ('DIT Error: ' . $contacts[0]);
	      ++$this->numErrors;
        $this->hasExecutedSuccessfully = 0;
        $this->hasExecuted = 0;
        $this->xmlData = $this->getRegistrarReturnXML ();
        return $this->xmlData;
      } else {
        $registrantid = $contacts[1];
        $techid       = $contacts[2];
        $adminid      = $contacts[3];
        $billingid    = $contacts[4];
      }


      /* create contacts.... */
			if ($sessionid = $this->getSessionId()) {

	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('order'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"TransferRequest");
	      $c->addField("Requester",'ModernBill');
	      $domain = $this->input['domainSLD'] . '.' . $this->input['domainTLD'];
	      $c->addField("Domain"   ,$domain);
	      $c->addField("Period"   ,$this->input['domainYears']);
	      if (strlen($this->input['domainTransferKey']) > 0) {
	        $c->addField("DomainPassword" ,$this->input['domainTransferKey']);
	      }

        $accessUserId = substr(preg_replace('/\W/', '', $domain), 0, 10);
	      $c->addField("UserID"   ,$accessUserId);
	      if ($this->input[domainAccessPassword]) {
	      	$pw = $this->input[domainAccessPassword];
	      } else {
	        $pw = md5(uniqid(rand(), true));
	      }
	      $pw = substr(preg_replace('/\W/', '', $pw), 0, 10);
	      $c->addField("Password" ,$pw);

	      $c->addField("OwnerContactID"          ,$registrantid);
	      $c->addField("AdministrationContactID" ,$adminid);
	      $c->addField("TechnicalContactID"      ,$techid);
	      $c->addField("BillingContactID"        ,$billingid);

        $result = $c->toString();

        $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::transferDomain() ~ $c->toString', $result, __FILE__, 825, 'MBAPI_ProcessRegistrarCommand');

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

	     	if ($result) {
  	      if (strpos($result, "ERR: ") === false)
    	    {
		  			$res = explode(": ", $result);
						//$res[0]; // contains 'OK'
						//$res[1]; // contains order id
	          $this->arrayData['TransferOrderID'] = $res[1];
  	        $this->arrayData['OtherData'] = $result;
    	      $this->hasExecutedSuccessfully = 1;
      	  }
        	else
	        {
  	        $this->errors[] = array ('DIT Error:', $result);
    	      ++$this->numErrors;
      	    $this->hasExecutedSuccessfully = 0;
        	}
	      } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
  	      $this->hasExecutedSuccessfully = 0;
    	  }
      } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
 	      $this->hasExecutedSuccessfully = 0;
   	  }
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;
    }


	/**
	 * This method contains the registrar specific code for renewing a domain.
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function renewDomain() {
      global $CONF;
      $validParams = array ('SLD', 'TLD', 'Period');

			if ($sessionid = $this->getSessionId()) {

	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('order'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"Renewal");
	      $c->addField("Requester",'ModernBill');
	      $c->addField("Domain"   ,$this->input['domainSLD'] . '.' . $this->input['domainTLD']);
	      $c->addField("Period"   ,$this->input['domainYears']);
//	      $c->addField("UserID"   ,$this->input[domainAccessUsername]);
//	      $c->addField("Password" ,$this->input[domainAccessPassword]);

        $result = $c->toString();

        $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::renewDomain() ~ $c->toString', $result, __FILE__, 946, 'MBAPI_ProcessRegistrarCommand');

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

	     	if ($result) {
  	      if (strpos($result, "ERR: ") === false)
    	    {
		  			$res = explode(": ", $result);
						//$res[0]; // contains 'OK'
						//$res[1]; // contains order id
            $this->arrayData['DomainRenewSuccess'] = 1;
  	        $this->arrayData['OtherData'] = $result;
    	      $this->hasExecutedSuccessfully = 1;
      	  }
        	else
	        {
            $this->arrayData['DomainRenewSuccess'] = 0;
  	        $this->errors[] = array ('DIT Error:', $result);
    	      ++$this->numErrors;
      	    $this->hasExecutedSuccessfully = 0;
        	}
	      } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
  	      $this->hasExecutedSuccessfully = 0;
    	  }
      } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
 	      $this->hasExecutedSuccessfully = 0;
   	  }
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;
    }



	/**
	 * Call the registrar to fetch a domain's expiry date
	 *
	 * @return  string  XML containing contact data for this domain
	 */
    function getDomainExpDate()
    {
      global $CONF;
			if ($sessionid = $this->getSessionId()) {

	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('query'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"Details");
	      $c->addField("Requester",'ModernBill');
	      $c->addField("Domain"   ,$this->input['domainSLD'] . '.' . $this->input['domainTLD']);

        $resultString = $c->toString();

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

		    $result = preg_split('/\n/', $resultString);
		    reset($result);

        $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::getDomainExpDate() ~ $c->toString', $result, __FILE__, 1012, 'MBAPI_ProcessRegistrarCommand');

	     	if ($result) {
  	      if (strpos(current($result), "ERR: ") === false)
    	    {
    	    	$this->arrayData['DomainData']['domainname'] = $this->input['domainSLD'] . '.' . $this->input['domainTLD'];
            while (next($result))
            {
              if (strpos(current($result), "ExpiryDate") !== false) {
              	$res = explode("=", current($result));
              	$this->arrayData[$res[0]] = $res[1];
			          $this->arrayData['DomainExpDate'] = $res[1];
			          $this->arrayData['DomainExpTimestamp'] = strtotime ($this->arrayData['DomainExpDate']);
			          $this->arrayData['OtherData'] = $result;
			          $this->hasExecutedSuccessfully = 1;
                break;
              } else {
              	next;
              }
            }
    	      $this->hasExecutedSuccessfully = 1;
      	  }
        	else
	        {
            $this->arrayData['DomainRenewSuccess'] = 0;
  	        $this->errors[] = array ('DIT Error:', current($result));
    	      ++$this->numErrors;
      	    $this->hasExecutedSuccessfully = 0;
        	}
	      } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
  	      $this->hasExecutedSuccessfully = 0;
    	  }
      } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
 	      $this->hasExecutedSuccessfully = 0;
   	  }
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;

    }



	/**
	 * Call the registrar to fetch a domain's contact data
	 * @return  string  XML containing contact data for this domain
	 */
	function getDomainContactData() {
	 	global $CONF;

			if ($sessionid = $this->getSessionId()) {

	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('query'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"Details");
	      $c->addField("Requester",'ModernBill');
	      $c->addField("Domain"   ,$this->input['domainSLD'] . '.' . $this->input['domainTLD']);

        $resultString = $c->toString();

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

		    $result = preg_split('/\n/', $resultString);
		    reset($result);

        $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::getDomainContactData() ~ $c->toString', $result, __FILE__, 1085, 'MBAPI_ProcessRegistrarCommand');

	     	if ($result) {
  	      if (strpos(current($result), "ERR: ") === false)
    	    {
    	    	$this->arrayData['DomainData']['domainname'] = $this->input['domainSLD'] . '.' . $this->input['domainTLD'];
            while (next($result))
            {
            	$prefix = '';
              if (strpos(current($result), "ExpiryDate") !== false) {
              	$res = explode("=", current($result));
              	$this->arrayData[$res[0]] = $res[1];
              } elseif (strpos(current($result), "Owner-") !== false) {
              	$prefix = 'Registrant';
              } elseif (strpos(current($result), "Administration-") !== false) {
              	$prefix = 'Admin';
              } elseif (strpos(current($result), "Technical-") !== false) {
              	$prefix = 'Tech';
              } elseif (strpos(current($result), "Billing-") !== false) {
              	$prefix = 'Billing';
              	$res  = explode("=", current($result));
              	$res2 = explode("-", $res[0]);
              	$this->arrayData['AuxBillingContactData']["AuxBilling".$res2[1]] = $res[1];
              	$this->arrayData['BillingContactData']["Billing".$res2[1]] = $res[1];
              }

              if (strlen($prefix) > 0) {
              	$res  = explode("=", current($result));
              	$res2 = explode("-", $res[0]);
              	if ($res2[1] === 'Email') {
              		$res2[1] = 'EmailAddress';
              	}
              	if ($res2[1] === 'PhoneNumber') {
              		$res2[1] = 'Phone';
              		$res[1] = $this->arrayData[$prefix.'ContactData'][$prefix."PhoneAreaCode"] . $res[1];
              	}
              	if ($res2[1] === 'Country') {
              		$res2[1] = 'CountryFullName';
              	}
              	if ($res2[1] === 'CountryCode') {
              		$res2[1] = 'Country';
              	}
              	if ($res2[1] === 'OrganisationName') {
              		$res2[1] = 'OrganizationName';
              	}

              	if ($res2[1] === 'FaxNumber') {
              		$res2[1] = 'Fax';
              		$res[1] = $this->arrayData[$prefix.'ContactData'][$prefix."FaxAreaCode"] . $res[1];
              	}
              	if ($res2[1] === 'Region') {
              		$res2[1] = 'StateProvince';
              		$this->arrayData[$prefix.'ContactData'][$prefix."StateProvinceChoice"] = 'P';
                	if ($prefix === 'Billing') {
                		$this->arrayData['Aux'.$prefix.'ContactData']['Aux'.$prefix."StateProvinceChoice"] = 'P';
                	}
              	}
              	$this->arrayData[$prefix.'ContactData'][$prefix.$res2[1]] = $res[1];
              	if ($prefix === 'Billing') {
              		$this->arrayData['Aux'.$prefix.'ContactData']['Aux'.$prefix.$res2[1]] = $res[1];
              		if ($res2[1] === 'Phone') {
              			$this->arrayData[$prefix.'ContactData'][$prefix.$res2[1]] = '+' . $this->arrayData[$prefix.'ContactData'][$prefix.'PhoneCountryCode'] . $this->arrayData[$prefix.'ContactData'][$prefix.$res2[1]];
              		} elseif ($res2[1] === 'Fax' && strlen($this->arrayData[$prefix.'ContactData'][$prefix.$res2[1]]) > 0) {
              			$this->arrayData[$prefix.'ContactData'][$prefix.$res2[1]] = '+' . $this->arrayData[$prefix.'ContactData'][$prefix.'FaxCountryCode'] . $this->arrayData[$prefix.'ContactData'][$prefix.$res2[1]];
              		} elseif ($res2[1] === 'Country') {
              			$this->arrayData[$prefix.'ContactData'][$prefix.$res2[1]] = $this->arrayData[$prefix.'ContactData'][$prefix.'CountryFullName'];
              		}
              	}
              }

            }


    	      $this->hasExecutedSuccessfully = 1;
      	  }
        	else
	        {
            $this->arrayData['DomainRenewSuccess'] = 0;
  	        $this->errors[] = array ('DIT Error:', current($result));
    	      ++$this->numErrors;
      	    $this->hasExecutedSuccessfully = 0;
        	}
	      } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
  	      $this->hasExecutedSuccessfully = 0;
    	  }
      } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
 	      $this->hasExecutedSuccessfully = 0;
   	  }
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;

	} // END getDomainContactData()


	/**
	 * Send the updated domain contact data to the registrar
	 * @return  string  XML result
	 */
	function setDomainContactData() {
	 	Global $CONF;


      $contacts = explode(":", $this->_createContacts(0));
      if ($contacts[0]) {
        $this->errors[] = array ('DIT Error: ' . $contacts[0]);
	      ++$this->numErrors;
        $this->hasExecutedSuccessfully = 0;
        $this->hasExecuted = 0;
        $this->xmlData = $this->getRegistrarReturnXML ();
        return $this->xmlData;
      } else {
        $registrantid = $contacts[1];
        $techid       = $contacts[2];
        $adminid      = $contacts[3];
        $billingid    = $contacts[4];
      }

			if ($sessionid = $this->getSessionId()) {
	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('order'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"UpdateContacts");
	      $c->addField("Requester",'ModernBill');
	      $c->addField("Domain"   ,$this->input['domainSLD'] . '.' . $this->input['domainTLD']);

	      if ($registrantid) {
	        $c->addField("OwnerContactID"          ,$registrantid);
	      }
	      if ($adminid) {
	        $c->addField("AdministrationContactID" ,$adminid);
	      }
	      if ($techid) {
	        $c->addField("TechnicalContactID"      ,$techid);
	      }
	      if ($billingid) {
	        $c->addField("BillingContactID"        ,$billingid);
	      }

        $result = $c->toString();

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

	     	if ($result) {
  	      if (strpos($result, "ERR: ") === false)
    	    {
            $this->hasExecutedSuccessfully = 1;
			      $this->arrayData['OtherData'] = $result;
          } else {
            $this->arrayData['PurchaseIsSuccess'] = 0;
	          $this->errors[] = array ('DIT Error:', $result);
            ++$this->numErrors;
            $this->hasExecutedSuccessfully = 0;
          }
        } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
	        $this->hasExecutedSuccessfully = 0;
        }
      } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
        $this->hasExecutedSuccessfully = 0;
      }
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;
	}


	/**
	 * This method contains the registrar specific code for setting a domain's auto-renewal options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setDomainRenewTypeOptions() {
		global $CONF;

    $this->errors[] = array ('DIT Error: Function not supported');
    ++$this->numErrors;
    $this->hasExecutedSuccessfully = 0;
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}


	/**
	 * This method contains the registrar specific code for getting a domain's auto-renewal options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
    function getDomainRenewTypeOptions()
    {
      global $CONF;
      $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::getdomainRenewTypeOptions() ~ $c->toString', '', __FILE__, 1288, 'MBAPI_ProcessRegistrarCommand');
      $this->arrayData['domainRenewType'] = 0;
      $this->arrayData['EmailForwardAutoRenew'] = 0;
      $this->arrayData['IDProtectAutoRenew'] = 0;
      $this->arrayData['URLForwardAutoRenew'] = 0;
      $this->arrayData['OtherData'] = '';
      $this->hasExecutedSuccessfully = 1;

      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;
    }



	/**
	 * This method contains the registrar specific code for fetching the domain registrar-lock options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainRegistrarLockOptions() {
      global $CONF;
			if ($sessionid = $this->getSessionId()) {

	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('query'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"Details");
	      $c->addField("Requester",'ModernBill');
	      $c->addField("Domain"   ,$this->input['domainSLD'] . '.' . $this->input['domainTLD']);

        $resultString = $c->toString();

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

		    $result = preg_split('/\n/', $resultString);
		    reset($result);

        $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::getDomainRegistrarLockOptions() ~ $c->toString', $result, __FILE__, 1329, 'MBAPI_ProcessRegistrarCommand');

	     	if ($result) {
  	      if (strpos(current($result), "ERR: ") === false)
    	    {
    	    	$this->arrayData['DomainData']['domainname'] = $this->input['domainSLD'] . '.' . $this->input['domainTLD'];
            while (next($result))
            {
              if (strpos(current($result), "DomainStatus") !== false) {
              	$res = explode("=", current($result));
								if (preg_match('/hold/i', $res[1])) {
									$this->arrayData["DomainRegistrarHold"] = 1;
									$this->arrayData["OtherData"]["RegistrarHold"] = 1;
								} else {
									$this->arrayData["DomainRegistrarHold"] = 0;
									$this->arrayData["OtherData"]["RegistrarHold"] = 0;
								}
              } else {
              	next;
              }
              if (strpos(current($result), "LockStatus") !== false) {
              	$res = explode("=", current($result));
              	$this->arrayData[$res[0]] = $res[1];
			          $this->arrayData['DomainRegistrarLock'] = ($res[1] == 2 ? 1 : 0);
			          if ($res[1] == 0 || $res[1] == 3) {
			          	 $this->arrayData['DomainIsLockable']    = 0;
			          } else {
			          	 $this->arrayData['DomainIsLockable']    = 1;
			          }
			          $this->arrayData['OtherData'] = $result;
			          $this->hasExecutedSuccessfully = 1;
                break;
              } else {
              	next;
              }
            }
    	      $this->hasExecutedSuccessfully = 1;
      	  }
        	else
	        {
            $this->arrayData['DomainRenewSuccess'] = 0;
  	        $this->errors[] = array ('DIT Error:', current($result));
    	      ++$this->numErrors;
      	    $this->hasExecutedSuccessfully = 0;
        	}
	      } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
  	      $this->hasExecutedSuccessfully = 0;
    	  }
      } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
 	      $this->hasExecutedSuccessfully = 0;
   	  }
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for setting the domain registrar-lock options.
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setDomainRegistrarLockOptions() {
		global $CONF;

			if ($sessionid = $this->getSessionId()) {

	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('order'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"UpdateDomainLock");
	      $c->addField("Requester",'ModernBill');
	      $domain = $this->input['domainSLD'] . '.' . $this->input['domainTLD'];
	      $c->addField("Domain"   ,$domain);

        $lock = $this->input[domainRegistrarLock] ? 'Lock' : 'Unlock';

	      $c->addField("DomainLock"   ,$lock);

        $resultString = $c->toString();
        $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::setDomainRegistrarLockOptions() ~ $c->toString', $resultString, __FILE__, 1414, 'MBAPI_ProcessRegistrarCommand');

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

		    $result = preg_split('/\n/', $resultString);
		    reset($result);

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

 	      $this->hasExecutedSuccessfully = 0;
	     	if ($result) {
  	      if (strpos(current($result), "ERR: ") === false)
    	    {
	          $this->arrayData['DomainRegistrarLock'] = $this->input[domainRegistrarLock];
	          if (preg_match('/.au/i', $this->input['domainTLD']) || preg_match('/.nz/i', $this->input['domainTLD'])) {
	          	$this->arrayData['DomainIsLockable'] = 0;
	          } else {
	          	$this->arrayData['DomainIsLockable'] = 1;
	          }
           	$this->arrayData["OtherData"]  = $result;
     	      $this->hasExecutedSuccessfully = 1;
      	  }
        	else
	        {
  	        $this->errors[] = array ('DIT Error:', current($result));
    	      ++$this->numErrors;
      	    $this->hasExecutedSuccessfully = 0;
        	}
	      } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
  	      $this->hasExecutedSuccessfully = 0;
    	  }
      } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
 	      $this->hasExecutedSuccessfully = 0;
   	  }
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;

	}


	/**
	 * This method contains the registrar specific code for fetching the domain access password.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainAccessPassword() {
		global $CONF;
		//Username is in $this->UID
		//Password is in $this->PW

			if ($sessionid = $this->getSessionId()) {

	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('query'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"Details");
	      $c->addField("Requester",'ModernBill');
	      $c->addField("Domain"   ,$this->input['domainSLD'] . '.' . $this->input['domainTLD']);

        $resultString = $c->toString();
        $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::getDomainAccessPassword() ~ $c->toString', $resultString, __FILE__, 1483, 'MBAPI_ProcessRegistrarCommand');

		    $result = preg_split('/\n/', $resultString);
		    reset($result);

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

 	      $this->hasExecutedSuccessfully = 0;
	     	if ($result) {
  	      if (strpos(current($result), "ERR: ") === false)
    	    {
    	    	$this->arrayData['DomainData']['domainname'] = $this->input['domainSLD'] . '.' . $this->input['domainTLD'];
            while (next($result))
            {
              if (strpos(current($result), "DomainAccessPassword") !== false) {
              	$res = explode("=", current($result));
              	$this->arrayData[$res[0]] = $res[1];

                // valid at point of time
                $this->arrayData["OtherData"]["RegistrarHold"] = 0;
                if ($res[1]) {
                	$this->arrayData["OtherData"]["PasswordSet"]    = 1;
                	$this->arrayData["OtherData"]["DomainPassword"] = $res[1];
       	          $this->arrayData['DomainAccessPasswordSet'] = 1;
                	$this->arrayData['DomainAccessPassword'] = $res[1];
                } else {
       	          $this->arrayData['DomainAccessPasswordSet']  = 0;
                	$this->arrayData["OtherData"]["PasswordSet"] = 0;
                }
         	      $this->hasExecutedSuccessfully = 1;
		            break;
              }
            }
      	  }
        	else
	        {
            $this->arrayData['DomainRenewSuccess'] = 0;
  	        $this->errors[] = array ('DIT Error:', current($result));
    	      ++$this->numErrors;
      	    $this->hasExecutedSuccessfully = 0;
        	}
	      } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
  	      $this->hasExecutedSuccessfully = 0;
    	  }
      } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
 	      $this->hasExecutedSuccessfully = 0;
   	  }
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;
	}

	/**
	 * This method contains the registrar specific code for setting the domain access password.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setDomainAccessPassword() {
		global $CONF;

			if ($sessionid = $this->getSessionId()) {

	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('order'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"CreateNewProfile");
	      $c->addField("Requester",'ModernBill');
	      $domain = $this->input['domainSLD'] . '.' . $this->input['domainTLD'];
	      $c->addField("Domain"   ,$domain);

        $accessUserId = substr(preg_replace('/\W/', '', $domain), 0, 10);

	      $c->addField("UserID_new"   ,$accessUserId);
	      $c->addField("Password_new" ,$this->input[domainAccessPassword]);

        $resultString = $c->toString();
        $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::setDomainAccessPassword() ~ $c->toString', $resultString, __FILE__, 1566, 'MBAPI_ProcessRegistrarCommand');

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

		    $result = preg_split('/\n/', $resultString);
		    reset($result);

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

 	      $this->hasExecutedSuccessfully = 0;
	     	if ($result) {
  	      if (strpos(current($result), "ERR: ") === false)
    	    {
           	$this->arrayData["OtherData"]  = $result;
     	      $this->hasExecutedSuccessfully = 1;
      	  }
        	else
	        {
  	        $this->errors[] = array ('DIT Error:', current($result));
    	      ++$this->numErrors;
      	    $this->hasExecutedSuccessfully = 0;
        	}
	      } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
  	      $this->hasExecutedSuccessfully = 0;
    	  }
      } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
 	      $this->hasExecutedSuccessfully = 0;
   	  }
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;

	}


	/**
	 * This method contains the registrar specific code to get expiry date, max and min renew period
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
    function getDomainExtendedInfo()
    {
      global $CONF;


			if ($sessionid = $this->getSessionId()) {

	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('query'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"Renewal");
	      $c->addField("Requester",'ModernBill');
	      $c->addField("Domain"   ,$this->input['domainSLD'] . '.' . $this->input['domainTLD']);

        $result = $c->toString();

        $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::getDomainExtendedInfo() ~ $c->toString', $result, __FILE__, 1630, 'MBAPI_ProcessRegistrarCommand');

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

 	      $this->hasExecutedSuccessfully = 0;
	     	if ($result) {
  	      if (strpos($result, "ERR: ") === false)
    	    {
    	    	$this->arrayData['DomainData']['domainname'] = $this->input['domainSLD'] . '.' . $this->input['domainTLD'];

    	    	$res = explode(": ", $result);
    	    	// res[0]: 'domain'
    	    	// res[1]: 'OK'
    	    	// res[2]: 'Minimum=1&Maximum=9&ExpiryDate=2007-05-28'

    	    	$res2 = explode("&", $res[2]);
    	    	// res2[0]: 'Minimum=1'
    	    	// res2[1]: 'Maximum=9'
    	    	// res2[2]: 'ExpiryDate=2007-05-28'

    	    	$min = explode("=", $res2[0]);
    	    	$max = explode("=", $res2[1]);
    	    	$exp = explode("=", $res2[2]);

            $this->arrayData['DomainExpDate'] = $exp[1];
            $this->arrayData['DomainExpTimestamp'] = strtotime ($this->arrayData['DomainExpDate']);

	          // max
	          $this->arrayData['DomainRenewMaxYears'] = $max[1];
	          // min
	          $this->arrayData['DomainRenewDefaultYears'] = $min[1];
	          // always set 0
	          $this->arrayData['DomainRenewPrice'] = 0;

           	$this->arrayData["OtherData"] = $result;
     	      $this->hasExecutedSuccessfully = 1;
      	  } elseif (preg_match('/Not available outside/i', $result)) {
	          // max
	          $this->arrayData['DomainRenewMaxYears'] = 0;
	          // min
	          $this->arrayData['DomainRenewDefaultYears'] = '0';
	          // always set 0
	          $this->arrayData['DomainRenewPrice'] = 0;
           	$this->arrayData["OtherData"] = $result;
     	      $this->hasExecutedSuccessfully = 1;
      	  }
        	else
	        {
  	        $this->errors[] = array ('DIT Error:', $result);
    	      ++$this->numErrors;
      	    $this->hasExecutedSuccessfully = 0;
        	}
	      } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
  	      $this->hasExecutedSuccessfully = 0;
    	  }
      } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
 	      $this->hasExecutedSuccessfully = 0;
   	  }
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;
    }



	/**
	 * Call the registrar to check TLD pricing cost (cogs).
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getTLDPricing() {
		global $CONF;

		//Username is in $this->UID
		//Password is in $this->PW

    $this->errors[] = array ('DIT Error: Function not supported');
    ++$this->numErrors;
    $this->hasExecutedSuccessfully = 0;
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * Call the registrar to check the status of a transfer
	 *
	 * @return  string status of the transfer
	 */
	function getTransferStatus() {
		global $CONF;

		if ($sessionid = $this->getSessionId()) {
      $c = $this->getCurlObject();
      $c->setURL($this->getURL('query'));
      $c->addField("SessionID",$sessionid);
      $c->addField("Type"     ,"Domains");
      $c->addField("Object"   ,"Domain");
      $c->addField("Action"   ,"Transfer");
      $c->addField("Requester",'ModernBill');
      $c->addField("Domain"   ,$this->input['domainSLD'] . '.' . $this->input['domainTLD']);

      $result = $c->toString();


      $this->connectionInfo[] = &$c->connectionInfo;
	    $this->getCurlErrors ($c);

      $data = array();

     	if ($result) {
        if (strpos(current($result), "ERR: 306") !== false)
        {
		  		$res = explode(": ", current($result));
					//$res[0]; // contains domain
					//$res[1]; // contains 'ERR'
					//$res[2]; // contains 306,ErrorDescription

	        $res2 = explode(",",$res[2]);
					//$res2[0]; // contains '306'
					//$res2[1]; // contains ErrorDescription

					if (strpos($res2[1], "Transfer NOT Required") !== false) {
					  $this->arrayData = "Transfered successfully";
					} else {
					  $this->arrayData = $res2[1];
					}
        	$this->hasExecutedSuccessfully = 1;
				} elseif (strpos(current($result), "ERR: ") !== false) {
          $this->errors[] = array ('DIT Error:', current($result));
          ++$this->numErrors;
          $this->hasExecutedSuccessfully = 0;
				} elseif (strpos(current($result), "OK: ") !== false) {
				  $this->arrayData = "No transfer request received";
        	$this->hasExecutedSuccessfully = 1;
				} else {
          $this->errors[] = array ('DIT Error:', current($result));
          ++$this->numErrors;
          $this->hasExecutedSuccessfully = 0;
        }
      } else {
        $this->errors[] = array ('DIT Error: No answer');
	      ++$this->numErrors;
        $this->hasExecutedSuccessfully = 0;
      }
    } else {
      $this->errors[] = array ('DIT Error: Auth Error');
      ++$this->numErrors;
      $this->hasExecutedSuccessfully = 0;
    }

		$this->hasExecutedSuccessfully = 1;
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}



	/**
	 * Call the registrar to get the account balance
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
    function checkAccountBalance ()
    {
      global $CONF;
			if ($sessionid = $this->getSessionId()) {
	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('query'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Action"   ,"AccountBalance");
	      $c->addField("Requester",'ModernBill');

	      $result = $c->toString();

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

	     	if ($result) {
	        if (strpos($result, "ERR: 108") !== false)
	        {
	        	// account is a invoice account
	        	$this->arrayData['AvailableBalance'] = 0;
	        	$this->hasExecutedSuccessfully = 1;
					} elseif (strpos($result, "ERR: ") !== false) {
	          $this->errors[] = array ('DIT Error:', $result);
	          ++$this->numErrors;
	          $this->hasExecutedSuccessfully = 0;
					} elseif (strpos($result, "OK: ") !== false) {
		  			$res = explode(": ", $result);
						//$res[0]; // contains 'OK'
						//$res[1]; // contains available account balance

	        	$this->arrayData['AvailableBalance'] = $res[1];
	        	$this->hasExecutedSuccessfully = 1;
					} else {
	          $this->errors[] = array ('DIT Error:', $result);
	          ++$this->numErrors;
	          $this->hasExecutedSuccessfully = 0;
	        }
	      } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
	        $this->hasExecutedSuccessfully = 0;
	      }
	    } else {
	      $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
	      $this->hasExecutedSuccessfully = 0;
	    }

			$this->hasExecutedSuccessfully = 1;
			$this->hasExecuted = 1;
			$this->xmlData = $this->getRegistrarReturnXML();
			return $this->xmlData;
		}


	/**
	 * Call the registrar to get a list of TLDs
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	/*function getTLDList() {

      global $CONF;

      foreach ($this->capabilities[REGISTRAR_TLDS] as $tld)
      {
        $tlds[$tld] = 1;
      }

      $this->arrayData = $tlds;
      $this->hasExecutedSuccessfully = 1;
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;

	}*/


	/**
	 * Call the registrar to fetch a domain's dns details
	 * @return  string  XML containing contact data for this domain
	 */
	function getDomainDNS() {
	 	global $CONF;

			if ($sessionid = $this->getSessionId()) {

	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('query'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"Details");
	      $c->addField("Requester",'ModernBill');
	      $c->addField("Domain"   ,$this->input['domainSLD'] . '.' . $this->input['domainTLD']);

        $resultString = $c->toString();

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

		    $result = preg_split('/\n/', $resultString);
		    reset($result);

        $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::getDomainDNS() ~ $c->toString', $result, __FILE__, 1907, 'MBAPI_ProcessRegistrarCommand');

		    $this->hasExecutedSuccessfully = 0;

	     	if ($result) {
  	      if (strpos(current($result), "ERR: ") === false)
    	    {
    	    	$this->arrayData['DomainData']['domainname'] = $this->input['domainSLD'] . '.' . $this->input['domainTLD'];
    	    	$cnt = 0;
            while (next($result))
            {
              if (strpos(current($result), "Nameserver=") !== false) {
              	$res = explode("=", current($result));
              	$this->arrayData['listDNS'][$cnt] = $res[1];
              	++$cnt;
              	$this->arrayData['UseDNS'] = 0;
        	      $this->hasExecutedSuccessfully = 1;
              }
            }
      	  }
        	else
	        {
  	        $this->errors[] = array ('DIT Error:', current($result));
    	      ++$this->numErrors;
      	    $this->hasExecutedSuccessfully = 0;
        	}
	      } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
  	      $this->hasExecutedSuccessfully = 0;
    	  }
      } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
 	      $this->hasExecutedSuccessfully = 0;
   	  }
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;

	} // END getDomainDNS()


	/**
	 * This method contains the registrar specific code for fetching a domain's extended information.
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setDomainDNS() {
		global $CONF;

			if ($sessionid = $this->getSessionId()) {

	      $c = $this->getCurlObject();
	      $c->setURL($this->getURL('order'));
	      $c->addField("SessionID",$sessionid);
	      $c->addField("Type"     ,"Domains");
	      $c->addField("Object"   ,"Domain");
	      $c->addField("Action"   ,"UpdateHosts");
	      $c->addField("Requester",'ModernBill');
	      $c->addField("Domain"   ,$this->input['domainSLD'] . '.' . $this->input['domainTLD']);

	      $validParams = array ('domainNameserver1',
	                            'domainNameserver2',
	                            'domainNameserver3',
	                            'domainNameserver4',
	                            'domainNameserver5',
	                            'domainNameserver6',
	                            'domainNameserver7',
	                            'domainNameserver8',
	                            'domainNameserver9'
	                           );

        foreach ($validParams as $key)
	      {
	        if (strlen($this->input[$key]) > 0)
	        {
	          $addHost .= $this->input[$key] . ',';
	        }
	      }
        $c->addField("AddHost"    ,$addHost);
        $c->addField("RemoveHost" ,"ALL");

        $result = $c->toString();
        $CONF['debug']->add ('MBAPI_ProcessRegistrarCommand::setDomainDNS() ~ $c->toString', $result, __FILE__, 1990, 'MBAPI_ProcessRegistrarCommand');

	      $this->connectionInfo[] = &$c->connectionInfo;
		    $this->getCurlErrors ($c);

 	      $this->hasExecutedSuccessfully = 0;
	     	if ($result) {
  	      if (strpos($result, "ERR: ") === false)
    	    {
            $this->arrayData['result'] = 'TRANS_REGISTRAR_MANAGE_DNS_UPDATED';
            $this->arrayData["OtherData"]  = $result;
            $this->hasExecutedSuccessfully = 1;
      	  }
        	else
	        {
  	        $this->errors[] = array ('DIT Error:', $result);
            $this->arrayData['result'] = 'TRANS_REGISTRAR_MANAGE_DNS_FAILED';
            ++$this->numErrors;
            $this->hasExecutedSuccessfully = 0;
        	}
	      } else {
	        $this->errors[] = array ('DIT Error: No answer');
		      ++$this->numErrors;
  	      $this->hasExecutedSuccessfully = 0;
    	  }
      } else {
        $this->errors[] = array ('DIT Error: Auth Error');
	      ++$this->numErrors;
 	      $this->hasExecutedSuccessfully = 0;
   	  }
      $this->hasExecuted = 1;
      $this->xmlData = $this->getRegistrarReturnXML ();
      return $this->xmlData;
	}


	 /**
	  *	This fucntion will return the map of all possible values with the names needed to pass to the registrar
	  *
	  * @return Returns the Registrar map array for this registrar
	 */
	 function getAttributesMap() {

		return array
		(
		// Primary Domain Vars
		"domainSLD" => "SLD",
		"domainTLD" => "TLD",
		"domainIDN" => "",
		"domainYears" => "Period",
		"domainDNS" => "UseDNS",
		"domainNameserver1" => "NS1",
		"domainNameserver2" => "NS2",
		"domainNameserver3" => "NS3",
		"domainNameserver4" => "NS4",
		"domainNameserver5" => "NS5",
		"domainNameserver6" => "NS6",
		"domainNameserver7" => "NS7",
		"domainNameserver8" => "NS8",
		"domainNameserver9" => "NS9",

		// check availability
		"checkNameSpinner" => "",
		"checkAllTLDs" => "checkAllTLDs",
		"checkWithDashes" => "",
		"checkKeyword1" => "",
		"checkKeyword2" => "",
		"checkKeyword3" => "",

		// Extended Domain Vars
		// .us domains
		"domainNexusCategory" => "",
		"domainNexusPurpose" => "",
		"domainNexusValidator" => "",
		// .jp domains
		"domainPrefecture" => "",
		// .eu
		"eu_agreedelete" => "",
		"eu_adr_lang" => "",

		// RegistrarLock Options Vars
		"domainRegistrarLock" => "",

		// AutoRenew Options Vars
		"domainRenewType" => "", // "RenewName"
		"emailForwardAutoRenew" => "",
		"IDProtectAutoRenew" => "",
		"URLForwardAutoRenew" => "",

		// Access Password Vars for Registrar Domain Management
		"domainAccessPassword" => "DomainPassword",
		"domainAccessUsername" => "DomainUsername",

		// ContactType=REGISTRANT Vars
		"registrantFirstName"           => "FirstName",
		"registrantLastName"            => "LastName",
		"registrantJobTitle"            => "",
		"registrantCompany"             => "OrganisationName",
		"registrantOrganizationName"    => "OrganisationName",
		"registrantAddress1"            => "Address1",
		"registrantAddress2"            => "Address2",
		"registrantAddress3"            => "Address3",
		"registrantCity"                => "City",
		"registrantStateProvinceChoice" => "",
		"registrantStateProvince"       => "Region",
		"registrantState"               => "Region",
		"registrantPostalCode"          => "PostalCode",
		"registrantCountry"             => "CountryCode",
		"registrantPhone"               => "Phone",
		"registrantPhoneExt"            => "PhoneExt",
		"registrantFax"                 => "Fax",
		"registrantEmailAddress"        => "Email",
		"registrantEmail"               => "Email",

		// ContactType=TECH Vars
		"techFirstName"           => "FirstName",
		"techLastName"            => "LastName",
		"techJobTitle"            => "",
		"techCompany"             => "OrganisationName",
		"techOrganizationName"    => "OrganisationName",
		"techAddress1"            => "Address1",
		"techAddress2"            => "Address2",
		"techAddress3"            => "Address3",
		"techCity"                => "City",
		"techStateProvinceChoice" => "",
		"techStateProvince"       => "Region",
		"techState"               => "Region",
		"techPostalCode"          => "PostalCode",
		"techCountry"             => "CountryCode",
		"techPhone"               => "Phone",
		"techPhoneExt"            => "PhoneExt",
		"techFax"                 => "Fax",
		"techEmailAddress"        => "Email",
		"techEmail"               => "Email",

		// ContactType=ADMIN Vars
		"adminFirstName"           => "FirstName",
		"adminLastName"            => "LastName",
		"adminJobTitle"            => "",
		"adminCompany"             => "OrganisationName",
		"adminOrganizationName"    => "OrganisationName",
		"adminAddress1"            => "Address1",
		"adminAddress2"            => "Address2",
		"adminAddress3"            => "Address3",
		"adminCity"                => "City",
		"adminStateProvinceChoice" => "",
		"adminStateProvince"       => "Region",
		"adminState"               => "Region",
		"adminPostalCode"          => "PostalCode",
		"adminCountry"             => "CountryCode",
		"adminPhone"               => "Phone",
		"adminPhoneExt"            => "PhoneExt",
		"adminFax"                 => "Fax",
		"adminEmailAddress"        => "Email",
		"adminEmail"               => "Email",

		// ContactType=AUXBILLING Vars
		"auxBillingFirstName"           => "FirstName",
		"auxBillingLastName"            => "LastName",
		"auxBillingJobTitle"            => "",
		"auxBillingCompany"             => "OrganisationName",
		"auxBillingOrganizationName"    => "OrganisationName",
		"auxBillingAddress1"            => "Address1",
		"auxBillingAddress2"            => "Address2",
		"auxBillingAddress3"            => "Address3",
		"auxBillingCity"                => "City",
		"auxBillingStateProvinceChoice" => "",
		"auxBillingStateProvince"       => "Region",
		"auxBillingState"               => "Region",
		"auxBillingPostalCode"          => "PostalCode",
		"auxBillingCountry"             => "CountryCode",
		"auxBillingPhone"               => "Phone",
		"auxBillingPhoneExt"            => "PhoneExt",
		"auxBillingFax"                 => "Fax",
		"auxBillingEmailAddress"        => "Email",
		"auxBillingEmail"               => "Email",

		);

	 } // END getAttributesMap()


	/**
	 * Returns the default configuration parameters for the DistributeIT module. The number of elements in each array must be identical to
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the parameters the module uses.
	 */

/**
         The addDefaultConfigParams function builds an array of configuration
         items that the panel configuration screen will use to setup the module
         in the admin interface.  Each item is commented with weather or not it
         can be removed.  You can add as many more as you need.  It will
         automatically be added to $CONF at runtime and can be accessed in the
         same way that the url, username, and password are pulled out in the top
         of each function.

         $newItem = $CONF["server"]["serverGroups"]["panel"][strtolower($this->stringRep)][$this->getConfigID()]["newItem"];
*/



	function addDefaultConfigParams() {
		//This needs to be tested with SetConfigGroups once a "add DistributeIT" link is added to the GUI
		$defaultConfigParams = array(
//                p_id  g_id  config_param_name,              ,value, config_param_group_trans_key        , config_param_trans_key                           , config_param_description,          config_param_input_type,config_param_required,config_param_order,
            array(NULL, NULL, 'moduleAccountNo'                , '' , 'TRANS_REGISTRAR_MODULE_SETTINGS'           , 'TRANS_REGISTRAR_ACCOUNT_NO'                     , ''                                                   , 't', 1,3 , 0, NULL),
            array(NULL, NULL, 'moduleUsername'                 , '' , 'TRANS_REGISTRAR_MODULE_SETTINGS'           , 'TRANS_REGISTRAR_USERNAME'                       , ''                                                   , 't', 1,4 , 0, NULL),
            array(NULL, NULL, 'modulePassword'                 , '' , 'TRANS_REGISTRAR_MODULE_SETTINGS'           , 'TRANS_REGISTRAR_PASSWORD'                       , ''                                                   , 'p', 1,5 , 0, NULL),
		array(NULL, NULL, 'moduleAllowAutorenew'           , '1', 'TRANS_REGISTRAR_MODULE_SETTINGS'           , 'TRANS_REGISTRAR_DIT_ALLOW_AUTORENEW'            , 'TRANS_REGISTRAR_DIT_ALLOW_AUTORENEW_DESC'           , 'r', 0,8 , 0, array(array(NULL,NULL,'0','TRANS_DISABLED',1), array(NULL,NULL,'2','TRANS_MODERNBILL_RENEW_OPTION',3))),
		array(NULL, NULL, 'moduleAllowChangeDNS'           , '1', 'TRANS_REGISTRAR_MODULE_SETTINGS'           , 'TRANS_REGISTRAR_DIT_ALLOW_CHANGE_DNS'           , 'TRANS_REGISTRAR_DIT_ALLOW_CHANGE_DNS_DESC'          , 'b', 0,13, 0, NULL),
		array(NULL, NULL, 'moduleAllowChangeContactInfo'   , '1', 'TRANS_REGISTRAR_MODULE_SETTINGS'           , 'TRANS_REGISTRAR_DIT_ALLOW_CHANGE_CONTACT_INFO'  , 'TRANS_REGISTRAR_DIT_ALLOW_CHANGE_CONTACT_INFO_DESC' , 'b', 0,14, 0, NULL),
		array(NULL, NULL, 'moduleAllowChangeRegistrarLock' , '1', 'TRANS_REGISTRAR_MODULE_SETTINGS'           , 'TRANS_REGISTRAR_DIT_ALLOW_CHANGE_REGISTRAR_LOCK' , 'TRANS_REGISTRAR_DIT_ALLOW_CHANGE_REGISTRAR_LOCK_DESC', 'b', 0,15, 0, NULL),
		array(NULL, NULL, 'moduleAllowChangeUpdateDomainPassword','1','TRANS_REGISTRAR_MODULE_SETTINGS'       , 'TRANS_REGISTRAR_DIT_ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD', 'TRANS_REGISTRAR_DIT_ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD_DESC', 'b', 0,17, 0, NULL),
            // prepaid cc data
            array(NULL, NULL, 'ditCCSupplierID'            	   , '' , 'TRANS_DIT_PREPAID_ACCOUNT'         , 'TRANS_REGISTRAR_DIT_CC_SUPPLIER_ID'             , 'TRANS_REGISTRAR_DIT_CC_SUPPLIER_ID_DES'             , 'r', 0,19, 0, array(array(NULL,NULL,'1','TRANS_DIT_AMEX',1), array(NULL,NULL,'2','TRANS_DIT_BANKCARD',2), array(NULL,NULL,'3','TRANS_DIT_DINERS',3), array(NULL,NULL,'4','TRANS_DIT_MC',4), array(NULL,NULL,'5','TRANS_DIT_VISA',5))),
            array(NULL, NULL, 'ditCCNumber'                	   , '' , 'TRANS_DIT_PREPAID_ACCOUNT'         , 'TRANS_REGISTRAR_DIT_CC_NUMBER'                  , 'TRANS_REGISTRAR_DIT_CC_NUMBER_DES'                  , 't', 0,20, 0, NULL),
            array(NULL, NULL, 'ditCCExpiryDate'            	   , '' , 'TRANS_DIT_PREPAID_ACCOUNT'         , 'TRANS_REGISTRAR_DIT_CC_EXPIRY'                  , 'TRANS_REGISTRAR_DIT_CC_EXPIRY_DES'                  , 't', 0,21, 0, NULL),

            // admin contact data
            array(NULL, NULL, 'useDefaultAdmin'           	   , '0', 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_DIT_DEFAULT_ADMIN'              , 'TRANS_REGISTRAR_DIT_DEFAULT_ADMIN_DESC'             , 'b', 1,22, 0, NULL),
            array(NULL, NULL, 'adminFirstname'            	   , '' , 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_FIRST_NAME'                     , ''                                                   , 't', 0,23, 0, NULL),
            array(NULL, NULL, 'adminLastName'             	   , '' , 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_LAST_NAME'                      , ''                                                   , 't', 0,24, 0, NULL),
            array(NULL, NULL, 'adminCompany'              	   , '' , 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_COMPANY'                        , ''                                                   , 't', 0,25, 0, NULL),
            array(NULL, NULL, 'adminEmail'                	   , '' , 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_EMAIL'                          , ''                                                   , 't', 0,26, 0, NULL),
            array(NULL, NULL, 'adminAddress1'             	   , '' , 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_ADDRESS1'                       , ''                                                   , 't', 0,27, 0, NULL),
            array(NULL, NULL, 'adminAddress2'             	   , '' , 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_ADDRESS2'                       , ''                                                   , 't', 0,28, 0, NULL),
            array(NULL, NULL, 'adminAddress3'             	   , '' , 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_ADDRESS2'                       , ''                                                   , 't', 0,29, 0, NULL),
            array(NULL, NULL, 'adminCity'                 	   , '' , 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_CITY'                           , ''                                                   , 't', 0,30, 0, NULL),
            array(NULL, NULL, 'adminState'                	   , '' , 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_STATE'                          , ''                                                   , 't', 0,31, 0, NULL),
            array(NULL, NULL, 'adminZip'                  	   , '' , 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_ZIP'                            , ''                                                   , 't', 0,32, 0, NULL),
            array(NULL, NULL, 'adminCountry'              	   , '' , 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_COUNTRY'                        , ''                                                   , 't', 0,33, 0, NULL),
            array(NULL, NULL, 'adminPhone'                	   , '' , 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_PHONE'                          , ''                                                   , 't', 0,34, 0, NULL),
            array(NULL, NULL, 'adminFax'                  	   , '' , 'TRANS_ADMINISTRATIVE_CONTACT'     , 'TRANS_REGISTRAR_FAX'                            , ''                                                   , 't', 0,35, 0, NULL),
            array(NULL, NULL, 'useDefaultTech'            	   , '0', 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_DIT_DEFAULT_TECH'               , 'TRANS_REGISTRAR_DIT_DEFAULT_TECH_DESC'              , 'b', 1,36, 0, NULL),
            array(NULL, NULL, 'techFirstname'             	   , '' , 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_FIRST_NAME'                     , ''                                                   , 't', 0,37, 0, NULL),
            array(NULL, NULL, 'techLastName'              	   , '' , 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_LAST_NAME'                      , ''                                                   , 't', 0,38, 0, NULL),
            array(NULL, NULL, 'techCompany'               	   , '' , 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_COMPANY'                        , ''                                                   , 't', 0,39, 0, NULL),
            array(NULL, NULL, 'techEmail'                 	   , '' , 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_EMAIL'                          , ''                                                   , 't', 0,40, 0, NULL),
            array(NULL, NULL, 'techAddress1'              	   , '' , 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_ADDRESS1'                       , ''                                                   , 't', 0,41, 0, NULL),
            array(NULL, NULL, 'techAddress2'              	   , '' , 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_ADDRESS2'                       , ''                                                   , 't', 0,42, 0, NULL),
            array(NULL, NULL, 'techAddress3'              	   , '' , 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_ADDRESS2'                       , ''                                                   , 't', 0,43, 0, NULL),
            array(NULL, NULL, 'techCity'                  	   , '' , 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_CITY'                           , ''                                                   , 't', 0,44, 0, NULL),
            array(NULL, NULL, 'techState'                 	   , '' , 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_STATE'                          , ''                                                   , 't', 0,45, 0, NULL),
            array(NULL, NULL, 'techZip'                   	   , '' , 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_ZIP'                            , ''                                                   , 't', 0,46, 0, NULL),
            array(NULL, NULL, 'techCountry'               	   , '' , 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_COUNTRY'                        , ''                                                   , 't', 0,47, 0, NULL),
            array(NULL, NULL, 'techPhone'                 	   , '' , 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_PHONE'                          , ''                                                   , 't', 0,48, 0, NULL),
            array(NULL, NULL, 'techFax'                   	   , '' , 'TRANS_TECHNICAL_CONTACT'      , 'TRANS_REGISTRAR_FAX'                            , ''                                                   , 't', 0,49, 0, NULL),
            array(NULL, NULL, 'useDefaultBilling'         	   , '0', 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_DIT_DEFAULT_AUXBILLING'         , 'TRANS_REGISTRAR_DIT_DEFAULT_AUXBILLING_DESC'        , 'b', 1,50, 0, NULL),
            array(NULL, NULL, 'auxBillingFirstname'       	   , '' , 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_FIRST_NAME'                     , ''                                                   , 't', 0,51, 0, NULL),
            array(NULL, NULL, 'auxBillingLastName'        	   , '' , 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_LAST_NAME'                      , ''                                                   , 't', 0,52, 0, NULL),
            array(NULL, NULL, 'auxBillingCompany'         	   , '' , 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_COMPANY'                        , ''                                                   , 't', 0,53, 0, NULL),
            array(NULL, NULL, 'auxBillingEmail'           	   , '' , 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_EMAIL'                          , ''                                                   , 't', 0,54, 0, NULL),
            array(NULL, NULL, 'auxBillingAddress1'        	   , '' , 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_ADDRESS1'                       , ''                                                   , 't', 0,55, 0, NULL),
            array(NULL, NULL, 'auxBillingAddress2'        	   , '' , 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_ADDRESS2'                       , ''                                                   , 't', 0,56, 0, NULL),
            array(NULL, NULL, 'auxBillingAddress3'        	   , '' , 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_ADDRESS2'                       , ''                                                   , 't', 0,57, 0, NULL),
            array(NULL, NULL, 'auxBillingCity'            	   , '' , 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_CITY'                           , ''                                                   , 't', 0,58, 0, NULL),
            array(NULL, NULL, 'auxBillingState'           	   , '' , 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_STATE'                          , ''                                                   , 't', 0,59, 0, NULL),
            array(NULL, NULL, 'auxBillingZip'             	   , '' , 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_ZIP'                            , ''                                                   , 't', 0,60, 0, NULL),
            array(NULL, NULL, 'auxBillingCountry'         	   , '' , 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_COUNTRY'                        , ''                                                   , 't', 0,61, 0, NULL),
            array(NULL, NULL, 'auxBillingPhone'           	   , '' , 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_PHONE'                          , ''                                                   , 't', 0,62, 0, NULL),
            array(NULL, NULL, 'auxBillingFax'             	   , '' , 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_FAX'                            , ''                                                   , 't', 0,63, 0, NULL),

            array(NULL, NULL, 'domainNameserver1'         	   , '' , 'TRANS_REGISTRAR_NS'                , 'TRANS_REGISTRAR_NS1'                            , ''                                                   , 't', 0,64, 0, NULL),
            array(NULL, NULL, 'domainNameserver2'         	   , '' , 'TRANS_REGISTRAR_NS'                , 'TRANS_REGISTRAR_NS2'                            , ''                                                   , 't', 0,65, 0, NULL),
            array(NULL, NULL, 'domainNameserver3'         	   , '' , 'TRANS_REGISTRAR_NS'                , 'TRANS_REGISTRAR_NS3'                            , ''                                                   , 't', 0,66, 0, NULL),
            array(NULL, NULL, 'domainNameserver4'         	   , '' , 'TRANS_REGISTRAR_NS'                , 'TRANS_REGISTRAR_NS4'                            , ''                                                   , 't', 0,67, 0, NULL),

        ); // $defaultConfigParams = array(

		// Grab the configs set at the registrar level
		return array_merge((array)$this->configParams,$defaultConfigParams);
		//return $defaultConfigParams;

	} // END addDefaultConfigParams()
}
