<?php

require_once("lib-mbapi/include/modules/registrar/registrar.php");

$className = "Dotster";
$GLOBALS["moduleInfo"][$className] = array(
	"type" => "registrar",
	"status" => Registrar::STATUS_TESTING,
	"version" => "1.0.0",
	"displayName" => "Dotster",
	"author" => "Parallels",
	"file" => __FILE__,
	"countryCodes" => array("US"),
);

class Dotster extends Registrar
{
	public $_parentID = 0;
	public $_customerID = 0;
	public $_contactIDs = array();
	public $_domainDetails = array();

	/**
	 * Returns the module display for the Dotster module. This includes version number and display name.
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the display info for the module.
	 */
	function getModuleInfo() {
		return $GLOBALS["moduleInfo"]["Dotster"];
	}

	/**
	 * Construct a new Dotster object. This function merely sets up capabilities after calling the
	 * constructor for Registrar.
	 */
	function Dotster($input) {
		$this->capabilities[REGISTRAR_REGISTERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_TRANSFERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_RENEWDOMAIN] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINEXTENDEDINFO] = 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINAVAILABILITY] = 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINTRANSFERAVAILABILITY] = 0;

		//Domain Management options
		$this->capabilities[REGISTRAR_GETDOMAINCONTACTDATA] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINCONTACTDATA] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWTYPEOPTIONS] = 0;
		$this->capabilities[REGISTRAR_SETDOMAINRENEWTYPEOPTIONS] = 0;
		$this->capabilities[REGISTRAR_GETDOMAINREGISTRARLOCKOPTIONS] = 0;
		$this->capabilities[REGISTRAR_SETDOMAINREGISTRARLOCKOPTIONS] = 0;
		$this->capabilities[REGISTRAR_GETDOMAINACCESSPASSWORD] = 0;
		$this->capabilities[REGISTRAR_SETDOMAINACCESSPASSWORD] = 0;
		$this->capabilities[REGISTRAR_GETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWDATE] = 0;
		$this->capabilities[REGISTRAR_LIVERENEW] = 0;

		$this->capabilities[REGISTRAR_CHECKACCOUNTBALANCE] = 0;
		$this->capabilities[REGISTRAR_GETTLDPRICING] = 0;
		$this->capabilities[REGISTRAR_GETTRANSFERSTATUS] = 0;
		$this->capabilities[REGISTRAR_GETTLDLIST] = 0;
		$this->capabilities[REGISTRAR_TLDS] = array();

		$this->apiSSL = "ssl://";
		$this->apiHost = "api.dotster.com";
		$this->apiPort = 786;

		//Setup the array of all items to send for new registration
		$contactTemplate = array(
			"usetrustee" => "",
			"firstname" => "",
			"lastname" => "",
			"company" => "",
			"address1" => "",
			"address2" => "",
			"address3" => "",
			"city" => "",
			"country" => "",
			"zipcode" => "",
			"email" => "",
			"phone" => "",	// +CCC.ZZZZZZZZZZZZxYYYY max 20 including the +
			"fax" => "",
			);

		$this->registrationArray = array(
			"action" => "create",
			"contact" => array(
				"admin" => $contactTemplate,
				"billing" => $contactTemplate,
				"org" => $contactTemplate,
				"tech" => $contactTemplate
				),
			"domain" => "",
			"year" => "",
			"lang_pref" => 0,		//Required for dot.us (probably really for .ca, documentation error)
			"legal_type" => "",		//required for .ca same list as eNom
			);

		if (isset($input['moduleParentID'])) {
			$this->_parentID = $input['moduleParentID'];
		}

		parent::Registrar($input);
	}

// ---------------------------------------------------------------------- INTERNAL HELPER METHODS
// ---------------------------------------------------------------------- INTERNAL HELPER METHODS
	/**
	 * This function merely sets the Dotster connection URL based on TEST mode status.
	 */
	function getURL() {
		global $CONF;
		// there is no test mode for dotster...
		return "api.dotster.com";
	}


	/**
	 * this function initiates the async connection to dotster
	 */
	function dotsterConnect($user=false, $pass=false){
		if ($this->dotsterConnection) return;
		$user = ($user) ? $user : $this->UID;
		$pass = ($pass) ? $pass : $this->PW;
		$err = "";
		$errn = "";
		$this->dotsterConnection = fsockopen($this->apiSSL.$this->apiHost, $this->apiPort, $err, $errn, 30);
		if (!$this->dotsterConnection){
			$this->addError(TRANS_MBAPIERROR, "Failed connection to Registrar Server. Error: $errn > $err");
			return false;
		}
		$l = false;
		while (trim($l)!="."){
			dbg("line from dotster:");
			dbg($l = fgets($this->dotsterConnection));
		}
		$this->initiateSession($user, $pass);
	}

	/**
	 * function to start the session on the connection
	 */
	function initiateSession($user=false, $pass=false){
		$user = ($user) ? $user : $this->UID;
		$pass = ($pass) ? $pass : $this->PW;
		if (!$this->dotsterConnection) $this->dotsterConnect($user, $pass);
		$result = $this->runDotsterCommand(array(
			"command" => "session",
			"partner" => $user,
			"password" => $pass,
		));
		if ($result["data"]["authstatus"]!="yes"){
			$this->addError(TRANS_MBAPIERROR, "Failed authenticate with the Registrar Server.");
			return false;
		} else {
			return true;
		}
	}

	/**
	 * function that runs a command and returns the parsed result
	 */
	function runDotsterCommand($params=false){
		if (!$params){
			$this->addError(TRANS_MBAPIERROR, "No command params provided.");
			return false;
		}
		$cmd = "command:".$params["command"]."\n";
		foreach ($params as $k=>$v){
			if ($k!="command"){
				$cmd .= "$k:$v\n";
			}
		}
		$cmd .= ".\n";
		fwrite($this->dotsterConnection, $cmd);

		$buffer = array();
		$l = false;
		while (trim($l)!="."){
			if ($l) $buffer[] = $l;
			dbg("line from dotster:");
			dbg($l = fgets($this->dotsterConnection));
		}
		dbg("full buffer:");
		dbg($buffer);
		if (!$buffer){
			$this->addError(TRANS_MBAPIERROR, "No response from registrar api.");
			return false;
		}
		$statusNo = substr($buffer[0], 0, 3);
		$statusMessage = substr($buffer[0], 4);
		$dataReturn = array();
		for ($i=1; $i<count($buffer); $i++){
			$t = $buffer[$i];
			$dataReturn[substr($t, 0, strpos($t, ":"))] = trim(substr($t, strpos($t, ":")+1));
		}
		return dbg(array("statusNumber" => $statusNo, "statusMessage" => trim($statusMessage), "data" => $dataReturn));
	}


	/**
	 * Put the contact info in place
	 *
	 */
	function _getRegistrationContactsData() {
		//Check the module setttings and place the proper contact info in place

		//Set the owner info
		$this->registrationArray["contact_set"]["owner"]["first_name"] = $this->input["registrantFirstName"];
		$this->registrationArray["contact_set"]["owner"]["last_name"] = $this->input["registrantLastName"];
		$this->registrationArray["contact_set"]["owner"]["org_name"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];
		$this->registrationArray["contact_set"]["owner"]["address1"] = $this->input["registrantAddress1"];
		$this->registrationArray["contact_set"]["owner"]["address2"] = $this->input["registrantAddress2"];
		$this->registrationArray["contact_set"]["owner"]["address3"] = "";
		$this->registrationArray["contact_set"]["owner"]["city"] = $this->input["registrantCity"];
		$this->registrationArray["contact_set"]["owner"]["state"] = $this->input["registrantStateProvince"];
		$this->registrationArray["contact_set"]["owner"]["country"] = $this->input["registrantCountry"];
		$this->registrationArray["contact_set"]["owner"]["postal_code"] = $this->input["registrantPostalCode"];
		$this->registrationArray["contact_set"]["owner"]["email"] = $this->input["registrantEmail"];
		$this->registrationArray["contact_set"]["owner"]["phone"] = $this->input["registrantPhone"];
		$this->registrationArray["contact_set"]["owner"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		$this->registrationArray["contact_set"]["owner"]["url"] = "";
		$this->registrationArray["contact_set"]["owner"]["lang_pref"] = $this->input["domain_idn_language"];

		//If the module has contact fields set, override the client info
		if($this->input["config"]["adminFirstname"] || $this->input["config"]["adminLastName"] || $this->input["config"]["adminCompany"]) {
			$this->registrationArray["contact_set"]["admin"]["first_name"] = $this->input["config"]["adminFirstname"];
			$this->registrationArray["contact_set"]["admin"]["last_name"] = $this->input["config"]["adminLastName"];
			$this->registrationArray["contact_set"]["admin"]["org_name"] = $this->input["config"]["adminCompany"];
			$this->registrationArray["contact_set"]["admin"]["address1"] = $this->input["config"]["adminAddress1"];
			$this->registrationArray["contact_set"]["admin"]["address2"] = $this->input["config"]["adminAddress2"];
			$this->registrationArray["contact_set"]["admin"]["address3"] = "";
			$this->registrationArray["contact_set"]["admin"]["city"] = $this->input["config"]["adminCity"];
			$this->registrationArray["contact_set"]["admin"]["state"] = $this->input["config"]["adminState"];
			$this->registrationArray["contact_set"]["admin"]["country"] = $this->input["config"]["adminCountry"];
			$this->registrationArray["contact_set"]["admin"]["postal_code"] = $this->input["config"]["adminPostalCode"];
			$this->registrationArray["contact_set"]["admin"]["email"] = $this->input["config"]["adminEmail"];
			$this->registrationArray["contact_set"]["admin"]["phone"] = $this->input["config"]["adminPhone"];
			$this->registrationArray["contact_set"]["admin"]["fax"] = (strlen($this->input["config"]["adminFax"])>4)? $this->input["registrantFax"] : "";
			$this->registrationArray["contact_set"]["admin"]["url"] = "";
			//$this->registrationArray["contact_set"]["admin"]["lang_pref"] = $this->input["config"]["domain_idn_language"];
		} else {
			$this->registrationArray["contact_set"]["admin"]["first_name"] = $this->input["registrantFirstName"];
			$this->registrationArray["contact_set"]["admin"]["last_name"] = $this->input["registrantLastName"];
			$this->registrationArray["contact_set"]["admin"]["org_name"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];
			$this->registrationArray["contact_set"]["admin"]["address1"] = $this->input["registrantAddress1"];
			$this->registrationArray["contact_set"]["admin"]["address2"] = $this->input["registrantAddress2"];
			$this->registrationArray["contact_set"]["admin"]["address3"] = "";
			$this->registrationArray["contact_set"]["admin"]["city"] = $this->input["registrantCity"];
			$this->registrationArray["contact_set"]["admin"]["state"] = $this->input["registrantState"];
			$this->registrationArray["contact_set"]["admin"]["country"] = $this->input["registrantCountry"];
			$this->registrationArray["contact_set"]["admin"]["postal_code"] = $this->input["registrantPostalCode"];
			$this->registrationArray["contact_set"]["admin"]["email"] = $this->input["registrantEmail"];
			$this->registrationArray["contact_set"]["admin"]["phone"] = $this->input["registrantPhone"];
			$this->registrationArray["contact_set"]["admin"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		}
		if($this->input["config"]["auxBillingFirstname"] || $this->input["config"]["auxBillingLastName"] || $this->input["config"]["auxBillingCompany"]) {
			$this->registrationArray["contact_set"]["billing"]["first_name"] = $this->input["config"]["auxBillingFirstname"];
			$this->registrationArray["contact_set"]["billing"]["last_name"] = $this->input["config"]["auxBillingLastName"];
			$this->registrationArray["contact_set"]["billing"]["org_name"] = $this->input["config"]["auxBillingCompany"];
			$this->registrationArray["contact_set"]["billing"]["address1"] = $this->input["config"]["auxBillingAddress1"];
			$this->registrationArray["contact_set"]["billing"]["address2"] = $this->input["config"]["auxBillingAddress2"];
			$this->registrationArray["contact_set"]["billing"]["address3"] = "";
			$this->registrationArray["contact_set"]["billing"]["city"] = $this->input["config"]["auxBillingCity"];
			$this->registrationArray["contact_set"]["billing"]["state"] = $this->input["config"]["auxBillingState"];
			$this->registrationArray["contact_set"]["billing"]["country"] = $this->input["config"]["auxBillingCountry"];
			$this->registrationArray["contact_set"]["billing"]["postal_code"] = $this->input["config"]["auxBillingPostalCode"];
			$this->registrationArray["contact_set"]["billing"]["email"] = $this->input["config"]["auxBillingEmail"];
			$this->registrationArray["contact_set"]["billing"]["phone"] = $this->input["config"]["auxBillingPhone"];
			$this->registrationArray["contact_set"]["billing"]["fax"] = (strlen($this->input["config"]["auxBillingFax"])>4)? $this->input["registrantFax"] : "";
		} else {
			$this->registrationArray["contact_set"]["billing"]["first_name"] = $this->input["registrantFirstName"];
			$this->registrationArray["contact_set"]["billing"]["last_name"] = $this->input["registrantLastName"];
			$this->registrationArray["contact_set"]["billing"]["org_name"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];;
			$this->registrationArray["contact_set"]["billing"]["address1"] = $this->input["registrantAddress1"];
			$this->registrationArray["contact_set"]["billing"]["address2"] = $this->input["registrantAddress2"];
			$this->registrationArray["contact_set"]["billing"]["address3"] = "";
			$this->registrationArray["contact_set"]["billing"]["city"] = $this->input["registrantCity"];
			$this->registrationArray["contact_set"]["billing"]["state"] = $this->input["registrantState"];
			$this->registrationArray["contact_set"]["billing"]["country"] = $this->input["registrantCountry"];
			$this->registrationArray["contact_set"]["billing"]["postal_code"] = $this->input["registrantPostalCode"];
			$this->registrationArray["contact_set"]["billing"]["email"] = $this->input["registrantEmail"];
			$this->registrationArray["contact_set"]["billing"]["phone"] = $this->input["registrantPhone"];
			$this->registrationArray["contact_set"]["billing"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		}
		if($this->input["config"]["techFirstname"] || $this->input["config"]["techLastName"] || $this->input["config"]["techCompany"]) {
			$this->registrationArray["contact_set"]["tech"]["first_name"] = $this->input["config"]["techFirstname"];
			$this->registrationArray["contact_set"]["tech"]["last_name"] = $this->input["config"]["techLastName"];
			$this->registrationArray["contact_set"]["tech"]["org_name"] = $this->input["config"]["techCompany"];
			$this->registrationArray["contact_set"]["tech"]["address1"] = $this->input["config"]["techAddress1"];
			$this->registrationArray["contact_set"]["tech"]["address2"] = $this->input["config"]["techAddress2"];
			$this->registrationArray["contact_set"]["tech"]["address3"] = "";
			$this->registrationArray["contact_set"]["tech"]["city"] = $this->input["config"]["techCity"];
			$this->registrationArray["contact_set"]["tech"]["state"] = $this->input["config"]["techState"];
			$this->registrationArray["contact_set"]["tech"]["country"] = $this->input["config"]["techCountry"];
			$this->registrationArray["contact_set"]["tech"]["postal_code"] = $this->input["config"]["techPostalCode"];
			$this->registrationArray["contact_set"]["tech"]["email"] = $this->input["config"]["techEmail"];
			$this->registrationArray["contact_set"]["tech"]["phone"] = $this->input["config"]["techPhone"];
			$this->registrationArray["contact_set"]["tech"]["fax"] = (strlen($this->input["config"]["techFax"])>4)? $this->input["registrantFax"] : "";
		} else {
			$this->registrationArray["contact_set"]["tech"]["first_name"] = $this->input["registrantFirstName"];
			$this->registrationArray["contact_set"]["tech"]["last_name"] = $this->input["registrantLastName"];
			$this->registrationArray["contact_set"]["tech"]["org_name"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];
			$this->registrationArray["contact_set"]["tech"]["address1"] = $this->input["registrantAddress1"];
			$this->registrationArray["contact_set"]["tech"]["address2"] = $this->input["registrantAddress2"];
			$this->registrationArray["contact_set"]["tech"]["address3"] = "";
			$this->registrationArray["contact_set"]["tech"]["city"] = $this->input["registrantCity"];
			$this->registrationArray["contact_set"]["tech"]["state"] = $this->input["registrantState"];
			$this->registrationArray["contact_set"]["tech"]["country"] = $this->input["registrantCountry"];
			$this->registrationArray["contact_set"]["tech"]["postal_code"] = $this->input["registrantPostalCode"];
			$this->registrationArray["contact_set"]["tech"]["email"] = $this->input["registrantEmail"];
			$this->registrationArray["contact_set"]["tech"]["phone"] = $this->input["registrantPhone"];
			$this->registrationArray["contact_set"]["tech"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		}

	}



	function _getCCData() {

	}




// ---------------------------------------------------------------------- MODULE ACTIONS
// ---------------------------------------------------------------------- MODULE ACTIONS
	/**
	 * Call the registrar to register a new domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function registerDomain() {
		//Popluate contact fields
		$this->_getRegistrationContactsData();
		$this->_getCustomerID();
		$this->_associateContacts();

		if (notNull($this->getErrors())) {
			return $this->_exit();
		}

		$domainName = $this->input["domainSLD"].".".$this->input["domainTLD"];
		$years = $this->input["domainYears"];

		$ns = array();
		for( $i=1; $i<=4; $i++ ) {
			if (notNull($this->input['domainNameserver'.$i])) {
// 				$ns[] = $this->input['domainNameserver'.$i];
			}
		}

		$ccTLDOpts = $this->_getCCData();

		//Do the TLD specific hoop jumping
		$args = array(
			'userName' => $this->UID,
			'password' => $this->PW,
			'role' => 'reseller',
			'langpref' => 'en',
			'parentid' => $this->_parentID,
			'addParamList' => array(array(
				'domainhash' => array($domainName => $years),
				'contacthash' => array(
					'registrantcontactid' => $this->_contactIDs[ $this->registrationArray['contact_set']['owner']['email'] ],
					'admincontactid' => $this->_contactIDs[ $this->registrationArray['contact_set']['admin']['email'] ],
					'technicalcontactid' => $this->_contactIDs[ $this->registrationArray['contact_set']['tech']['email'] ],
					'billingcontactid' => $this->_contactIDs[ $this->registrationArray['contact_set']['billing']['email'] ],
				)
			)),
			'nameServersList' => $ns,
			'customerId' => $this->_customerID,
			'invoiceOption' => 'PayInvoice',
			'enablePrivacyProtection' => false,
			'validate' => true,
			(notNull($ccTLDOpts) ? array('extraInfo' => $ccTLDOpts) : '<extraInfo xsi:type="apachesoap:Map">'."\n".'</extraInfo>')
			);

		$result = $this->_query('order.DomOrder', 'registerDomain', $args );

		if (isset($result['error'])) {
			$this->addError(TRANS_MBAPIERROR, 'Error occured while registering domain ['.$result['error']['message'].'].');
		} elseif (isset($result['Map'][ $domainName ])) {
			$response = $result['Map'][ $domainName ];

			if (eregi('success', $response['status']) == false) {
				$this->addError(TRANS_MBAPIERROR, "Domain [${domainName}] failed registration [${response[error]}].");
			} else {
				$this->arrayData += $response;
			}
		}

		$this->hasExecuted =  1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}  // registerDomain


	/**
	 * Call the registrar to transfer a domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function transferDomain() {
		//Popluate contact fields
		$this->_getRegistrationContactsData();

		$this->_getCustomerID();
		$this->_associateContacts();

		$eppCode = $this->input['domainTransferKey'];
		$domainName = $this->input["domainSLD"].".".$this->input["domainTLD"];

		$result = $this->_query('order.DomOrder', 'transferDomain', array(
			'userName' => $this->UID,
			'password' => $this->PW,
			'role' => 'reseller',
			'langpref' => 'en',
			'parentid' => $this->_parentID,
			'domainHash' => array($domainName => $eppCode),

			'registrantContactId' => $this->_contactIDs[ $this->registrationArray['contact_set']['owner']['email'] ],
			'adminContactId' => $this->_contactIDs[ $this->registrationArray['contact_set']['admin']['email'] ],
			'techContactId' => $this->_contactIDs[ $this->registrationArray['contact_set']['tech']['email'] ],
			'billingContactId' => $this->_contactIDs[ $this->registrationArray['contact_set']['billing']['email'] ],

			'customerId' => $this->_customerID,
			'invoiceOption' => 'PayInvoice'
			) );
		if (isset($result['error'])) {
			$this->addError(TRANS_MBAPIERROR, "Error occured during transfer [".$result['error']['message']."]");
		} else {
			$response = $result['Map'][ $domainName ];

			if (isset($response['status']) && eregi('error', $response['status'])) {
				$this->addError(TRANS_MBAPIERROR, "There was an error during registrar call [".$response['error']."].");
			} else {
				$this->arrayData += $response;
			}
		}

		$this->hasExecuted =  1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/**
	 * Call the registrar to renew a domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function renewDomain() {

		$domainName = $this->input["domainSLD"].".".$this->input["domainTLD"];
		$years = $this->input["domainYears"];

		$details = $this->_getDomainDetails( $domainName );

		$domainHash = array($domainName => array(
			'entityid' => $details['Map']['entityid'],
			'noofyears' => $years,
			'expirydate' => $details['Map']['endtime']));

		$result = $this->_query('order.DomOrder', 'renewDomain', array(
			'userName' => $this->UID,
			'password' => $this->PW,
			'role' => 'reseller',
			'langpref' => 'en',
			'parentid' => $this->_parentID,
			'domainHash' => $domainHash,
			'invoiceOption' => 'PayInvoice'
			) );

		if (isset($result['error'])) {
			$this->addError(TRANS_MBAPIERROR, "Domain Renewal Failed [".$result['error']['message']."]");
		} else {
			$this->arrayData += $result['Map'][ $domainName ];
		}

		$this->hasExecuted =  1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/**
	 *	@short this method is to fetch the nameservers of a domain
	 *	@name getDomainDNS
	 *	@return string  XML containing registration results for this domain name servers
	 */
	function getDomainDNS() {
		if ( ($details = $this->_getDomainDetails()) == false ) {
			$this->hasExecuted =  1;
			$this->xmlData = $this->getRegistrarReturnXML();
			return $this->xmlData;
		}

		for( $i=0; $i<=8; $i++ ) {
			if (isset($details['Map']["ns${i}"])) {
				$this->arrayData['listDNS'][] = $details['Map']["ns${i}"];
			}
		}


		$this->hasExecuted =  1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/**
	 *	@short this method is to set the name servers and dns records of a domain
	 *	@name setDomainDNS
	 *	@return string  XML containing registration results for this domain name servers
	 */
	function setDomainDNS() {
		// need to get the domain orderID
		if ( ($details = $this->_getDomainDetails()) == false ) {
			$this->hasExecuted =  1;
			$this->xmlData = $this->getRegistrarReturnXML();
			return $this->xmlData;
		}

		$ns2Create = false;
		$domainName = $this->input['domainSLD'] .'.'. $this->input['domainTLD'];

		$ns = array();
		for( $i=1; $i<=4; $i++ ) {
			if (isset($this->input['domainNameserver'.$i])) {
				$ns['ns'.$i] = $this->input['domainNameserver'.$i];

				if (eregi("\.$domainName$", $ns['ns'.$i])) {
					$ns2Create = true;
				}
			}
		}


		if ($ns2Create) {
			foreach( (array)$ns as $n ) {

				$result = $this->_query('order.DomOrder', 'addChildNameServer', array(
					'userName' => $this->UID,
					'password' => $this->PW,
					'role' => 'reseller',
					'langpref' => 'en',
					'parentid' => $this->_parentID,
					'orderId' => (int)$details['Map']['orderid'],
					'cns' => $n,
					'ipAddress' => array(gethostbyname($n))
					));

				if (isset($result['error'])) {
					$this->addError(TRANS_MBAPIERROR, "Error occured while registering name servers [".$result['error']['message']."]");
				} else {

				}

			} unset($n, $result);
		}

		$result = $this->_query('order.DomOrder', 'modifyNameServer', array(
			'userName' => $this->UID,
			'password' => $this->PW,
			'role' => 'reseller',
			'langpref' => 'en',
			'parentid' => $this->_parentID,
			'orderId' => (int)$details['Map']['orderid'],
			'nsHash' => $ns) );

		if (isset($result['error'])) {
			$this->addError(TRANS_MBAPIERROR, "Directi Error: ".$result['error']['message']);
		} else {

			$this->arrayData += $result['Map'];

			$this->arrayData['result'] = "TRANS_REGISTRAR_MANAGE_DNS_UPDATED";
			$this->hasExecutedSuccessfully = 1;
		}

		$this->hasExecuted =  1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}


	function getDomains() {
		$result = $this->_query('order.DomOrder', 'list', array(
			'userName' => $this->UID,
			'password' => $this->PW,
			'role' => 'reseller',
			'langpref' => 'en',
			'parentid' => $this->_parentID,
			'<orderId xsi:nil="true" xsi:type="SOAP-ENC:Array" SOAP-ENC:arrayType="xsd:string[0]"/>',
			'<resllerId xsi:nil="true" xsi:type="SOAP-ENC:Array" SOAP-ENC:arrayType="xsd:string[0]"/>',
			'<customerId xsi:nil="true" xsi:type="SOAP-ENC:Array" SOAP-ENC:arrayType="xsd:string[0]"/>',
			'showChildOrders' => true, // showChildOrders
			'<classKey xsi:nil="true" xsi:type="SOAP-ENC:Array" SOAP-ENC:arrayType="xsd:string[0]"/>',
			'<currentStatus xsi:nil="true" xsi:type="SOAP-ENC:Array" SOAP-ENC:arrayType="xsd:string[0]"/>',
			'description' => "",
			'<ns xsi:nil="true" xsi:type="SOAP-ENC:Array" SOAP-ENC:arrayType="xsd:string[0]"/>',
			'contactName' => "",
			'contactCompanyName' => "",
			'creationDTRangStart' => '',
			'creationDTRandEnd' => '',
			'endTimeRangStart' => '',
			'endTimeRangEnd' => '',
			'numOfRecordPerPage' => 25, // num records per page
			'pageNum' => $this->input['offset'], // pageNum
			'<orderBy xsi:nil="true" xsi:type="SOAP-ENC:Array" SOAP-ENC:arrayType="xsd:string[0]"/>'
			) );

		if (isset($result['error'])) {
			$this->addError(TRANS_MBAPIERROR, "Directi Error: ".$result['error']['message']);
		} else {

			foreach( (array)$result['Map'] as $k => $v ) {
				if (is_numeric($k)) {
					if (notNull($v['entity.description'])) {
						$res = explode('.', $v['entity.description']);
						$domain = array(
							'sld' => $res[0],
							'tld' => $res[1] . (isset($res[2]) ? '.'. $res[2] : null) );

						$this->arrayData["domains"][] = array(
							"domainName" => $v['entity.description'],
							"domainSLD" => $domain['sld'],
							"domainTLD" => $domain["tld"],
	// 						"domainAutoRenew" => (strtolower($domains[$i]["auto-renew"][0]) == "yes")? 1 : 0,
	// 						"domainRenewal" => (strtolower($domains[$i]["auto-renew"][0]) == "yes")? 1 : 0, // Required duplicate param
	// 						"domainPrivacy" => (strtolower($domains[$i]["WPPSStatus"][0]) == "enabled")? 1 : 0,
	// 						"domainNameserverStatus" => (strtolower($domains[$i]["NS-Status"][0]) == "yes")? 1 : 0,
							"domainDateCreated" => is_numeric($v['orders.creationdt']) ? $v['orders.creationdt'] : strtotime($v['orders.creationdt']),
							"domainDateExpires" => is_numeric($v['orders.endtime']) ? $v['orders.endtime'] : strtotime($v['orders.endtime'])
							);
					}
				}
			}
		}

		$this->hasExecuted =  1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}


	function checkDomainAvailability() {

		$this->dotsterConnect();

		$result = $this->runDotsterCommand(array(
			"command" => "check",
			"domainname" => $this->input['domainSLD'] .'.'. $this->input['domainTLD'],
		));

		if ($result['statusNumber']!="200") {
			$this->addError(TRANS_MBAPIERROR, "Dotster Error: ".$result["statusNumber"]." > ".$result["statusMessage"]);
		} else {
			if ($result["data"]["available"]=="no") {
				$this->arrayData['OriginalDomainIsAvailable'] = 0;
			} else {
				$this->arrayData['OriginalDomainIsAvailable'] = 1;
			}
		}

		$this->hasExecuted =  1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}


	function checkDomainTransferAvailability() {
		$domainName = $this->input['domainSLD'] .'.'. $this->input['domainTLD'];

		$this->dotsterConnect();

		$result = $this->runDotsterCommand(array(
			"command" => "check",
			"domainname" => $domainName,
		));

		if (isset($result['error'])) {
			$this->addError(TRANS_MBAPIERROR, "Directi Error: ".$result['error']['message']);
		} else {
			$this->arrayData['DomainName'] = $domainName;
			$this->arrayData['DomainRegistrar'] = (notNull($details['Map']) ? 'Known' : 'Unknown' );
			$this->arrayData['DomainInAccount'] = (notNull($details['Map']) ? 1 : 0);
			$this->arrayData['DomainLastOrderID'] = $details['Map']['orderid'];
			$this->arrayData['DomainAvailable'] = ($result['boolean'] === true) ? 'true' : 'false';
		}

		$this->hasExecuted =  1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}



	function getDomainContactData() {
		if ( ($details = $this->_getDomainDetails()) == false ) {
			$this->hasExecuted =  1;
			$this->xmlData = $this->getRegistrarReturnXML();
			return $this->xmlData;
		}

		$details = $details['Map'];


		$trans =  array(
			'techcontact' => 'Tech',
			'registrantcontact' => 'Registrant',
			'admincontact' => 'Admin',
			'billingcontact' => 'AuxBilling');

		$contacts = array();
		foreach( $trans as $d => $mb ) {

			$thisContact =& $details[ $d ]; // pointer
			$p =& $contacts[ "${mb}ContactData" ];

			foreach( (array)$thisContact as $k => $v ) {
				switch( strtolower($k) ) {
				case 'address1':
				case 'address2':
				case 'address3':
				case 'country':
				case 'city':

					$p[ $mb . ucwords($k) ] = $v;
					break;

				case 'name':
					if ( ($t = explode(' ', $v)) != false ) {
						$p["${mb}FirstName"] = $t[0];
						array_shift($t);
						$p["${mb}LastName"] = implode(' ', $t);
					} unset($t);
					break;

				case 'emailaddr':
					$p[ "${mb}EmailAddress" ] = $v;
					break;

				case 'company':
					$p[ "${mb}OrganizationName" ] = $v;
					break;

				case 'zip':
					$p[ "${mb}PostalCode" ] = $v;
					break;

				case 'telnocc': // append to beginning
					$p[ "${mb}Phone" ] = $v .'.'. $p[ "${mb}Phone" ];
					break;

				case 'telno': // append to end
					$p[ "${mb}Phone" ] .= $v;
					break;

				case 'state':
					$p[ "${mb}StateProvince" ] = $v;
					break;
				}
			}
		} unset($t);

		$this->arrayData += array(
			'DomainData' => array(
				'sld' => $this->input['domainSLD'],
				'tld' => $this->input['domainTLD'],
				'domainnameid' => $details['orderid'],
				'domainname' => $details['domainname'],
				'entityid' => $details['entityid']
			)) + $contacts;

		return $this->_exit();
	}


	function setDomainContactData() {

		foreach ((array)$this->input['domainContactData'] as $type => $contact) {
			$this->input += $contact;
		}

		$this->_getRegistrationContactsData();

		$this->_getCustomerID();

		if ( ($details = $this->_getDomainDetails()) == false ) {
			$this->hasExecuted =  1;
			$this->xmlData = $this->getRegistrarReturnXML();
			return $this->xmlData;
		}

		$details = $details['Map'];
		$contactIDs = array();

		foreach ((array)$this->input['domainContactData'] as $type => $contact) {
			if (!notNull($contact)) {
				if (isset($contactIDs['registrant'])) {
					$contactIDs[ $type ] = $contactIDs['registrant'];
					continue;
				}

				$contact =& $this->input['domainContactData']['registrant'];
			}

			$result = $this->_query('order.DomContact', "listByType", array(
				'SERVICE_USERNAME' => (string)$this->UID,
				'SERVICE_PASSWORD' => (string)$this->PW,
				'SERVICE_ROLE' => 'reseller',
				'SERVICE_LANGPREF' => 'en',
				'SERVICE_PARENTID' => (int)$this->_parentID,
				'customerId' => (int)$this->_customerID,
				'<contactId xsi:nil="true" xsi:type="SOAP-ENC:Array" SOAP-ENC:arrayType="xsd:int[0]"/>',
				'<currentStatus xsi:nil="true" xsi:type="SOAP-ENC:Array" SOAP-ENC:arrayType="xsd:string[0]"/>',
				'creationDTRangStart' => "",
				'creationDTRandEnd' => "",
				'contactName' => $contact["${type}FirstName"] .' '. $contact["${type}LastName"],
				'companyName' => $contact["${type}OrganizationName"],
				'emailAddr' => str_replace('@', '\@', $contact["${type}EmailAddress"]),
				'numOfRecordPerPage' => 50,
				'pageNum' => 1,
				'orderBy' => array(),
				'type' => 'Contact',
				'requirements' => array()
				) );

			// this is a matched contact, probably need to update the address
			foreach( (array)$result['Map']['result'] as $remoteContact ) {
				if (
					(trim($contact["${type}Phone"]) == trim($remoteContact['contact.telnocc']) .'.'. trim($remoteContact['contact.telno'])) &&
					(strtolower(trim($contact["${type}Address1"])) == strtolower(trim($remoteContact['contact.address1']))) &&
					(strtolower(trim($contact["${type}Address2"])) == strtolower(trim($remoteContact['contact.address2']))) &&
					(strtolower(trim($contact["${type}City"])) == strtolower(trim($remoteContact['contact.city']))) &&
					(strtolower(trim($contact["${type}StateProvince"])) == strtolower(trim($remoteContact['contact.state']))) &&
					(strtolower(trim($contact["${type}PostalCode"])) == strtolower(trim($remoteContact['contact.zip'])))
					) {
					$contactIDs[ $type ] = $remoteContact['contact.contactid'];
				}
			}

			if (!notNull($contactIDs[ $type ])) {
				// need to create the new contact
				$phone = explode('.', $contact["${type}Phone"]);
				$phone[0] = str_replace('+', null, $phone[0]);

				$fax = explode('.', $contact["${type}Fax"]);
				$fax[0] = str_replace('+', null, $fax[0]);

				$result = $this->_query('order.DomContact', 'addContact', array(
					'SERVICE_USERNAME' => $this->UID,
					'SERVICE_PASSWORD' => $this->PW,
					'SERVICE_ROLE' => 'reseller',
					'SERVICE_LANGPREF' => 'en',
					'SERVICE_PARENTID' => $this->_parentID,
					'name' => $contact["${type}FirstName"] .' '. $contact["${type}LastName"],
					'company' => $contact["${type}OrganizationName"],
					'emailAddr' => $contact["${type}EmailAddress"],
					'address1' => (string)$contact["${type}Address1"],
					'address2' => (string)$contact["${type}Address2"],
					'address3' => (string)$contact["${type}Address3"],
					'city' => $contact["${type}City"],
					'state' => $contact["${type}StateProvince"],
					'country' => $contact["${type}Country"],
					'zip' => $contact["${type}PostalCode"],

					// phone1
					'telNoCc' => (int)$phone[0],
					'telNo' => (int)$phone[1],

					// fax
					'faxNoCc' => (int)$fax[0],
					'faxNo' => (int)$fax[1],

					'customerId' => $this->_customerID,
					'type' => 'Contact',
					'extraInfo' => array(array())
				) );

				if (isset($result['error'])) {
					$this->addError(TRANS_MBAPIERROR, "Unable to create contact '".$contact["{$type}EmailAddress"]."' [".$result['error']['message']."].");
					return $this->_exit();
				} else {
					$contactIDs[ $type ] = (int)$result['int'];
				}
			}
		}

		$result = $this->_query('order.DomOrder', 'modifyContact', array(
			'userName' => $this->UID,
			'password' => $this->PW,
			'role' => 'reseller',
			'langpref' => 'en',
			'parentid' => $this->_parentID,
			'orderid' => (int)$details['orderid'],
			'registrantContactId' => (int)$contactIDs['registrant'],
			'adminContactId' => (int)$contactIDs['admin'],
			'techContactId' => (int)$contactIDs['tech'],
			'billingContactId' => (int)$contactIDs['auxBilling']
			) );

		if (isset($result['error'])) {
			$this->addError(TRANS_MBAPIERROR, "Unable to modify Domain [".$result['error']['message']."]");
		} else {
			$this->arrayData += $result['Map'];
		}

		return $this->_exit();
	}





// ---------------------------------------------------------------------- MODULE CONFIG PARAMS
// ---------------------------------------------------------------------- MODULE CONFIG PARAMS
	/**
	 * Returns the default configuration parameters for the Directi module. The number of elements in each array must be identical to
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the parameters the module uses.
	 */
	function addDefaultConfigParams() {
		//This needs to be tested with SetConfigGroups once a "add directi" link is added to the GUI
		$defaultConfigParams = array(
			array(NULL, NULL, "moduleUsername", "", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_USERNAME", "", "t", 0,2,0, NULL),
			array(NULL, NULL, "modulePassword", "", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_PASSWORD", "", "p", 0,3,0, NULL),
			array(NULL, NULL, "moduleParentID", "", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_PARENT_ID", "", "t", 0,4,0, NULL),
			array(NULL, NULL, "moduleTestMode", "1", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_ENABLE_DIRECTI_TEST_MODE", "", "b", 0,5,0, NULL),
			array(NULL, NULL, "moduleAllowAutorenew", "0","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_DIRECTI_ALLOW_AUTORENEW", "TRANS_REGISTRAR_DIRECTI_ALLOW_AUTORENEW_DESC", "r", 0,8, 0,array
				(array(NULL, NULL, "0","TRANS_DISABLED",1),
				array(NULL, NULL, "1","TRANS_REGISTRAR_AUTORENEW_OPTION",2),
				array(NULL, NULL, "2","TRANS_MODERNBILL_RENEW_OPTION",3))),
			array(NULL, NULL, "moduleAllowChangeDNS", "0","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_DIRECTI_ALLOW_CHANGE_DNS", "TRANS_REGISTRAR_DIRECTI_ALLOW_CHANGE_DNS_DESC", "b", 0,13, 0, NULL),
			array(NULL, NULL, "moduleAllowChangeContactInfo", "1", "TRANS_REGISTRAR_MODULE_SETTINGS","TRANS_REGISTRAR_DIRECTI_ALLOW_CHANGE_CONTACT_INFO", "TRANS_REGISTRAR_DIRECTI_ALLOW_CHANGE_CONTACT_INFO_DESC", "b", 0,14, 0, NULL),
			array(NULL, NULL, "moduleAllowChangeRegistrarLock", "0","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_DIRECTI_ALLOW_CHANGE_REGISTRAR_LOCK", "TRANS_REGISTRAR_DIRECTI_ALLOW_CHANGE_REGISTRAR_LOCK_DESC", "b", 0,15, 0, NULL),
			array(NULL, NULL, "moduleAllowChangeAutorenew", "0","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_DIRECTI_ALLOW_CHANGE_AUTORENEW", "TRANS_REGISTRAR_DIRECTI_ALLOW_CHANGE_AUTORENEW_DESC", "b", 0,16, 0, NULL),
			array(NULL, NULL, "moduleAllowChangeUpdateDomainPassword", "0","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_DIRECTI_ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD", "TRANS_REGISTRAR_DIRECTI_ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD_DESC", "b", 0,17, 0, NULL),
			// admin contact data
			array(NULL, NULL, "adminFirstname", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,19,0, NULL),
			array(NULL, NULL, "adminLastName", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,20,0, NULL),
			array(NULL, NULL, "adminCompany", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,21,0, NULL),
			array(NULL, NULL, "adminEmail", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,22,0, NULL),
			array(NULL, NULL, "adminAddress1", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,23,0, NULL),
			array(NULL, NULL, "adminAddress2", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,24,0, NULL),
			array(NULL, NULL, "adminCity", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,25,0, NULL),
			array(NULL, NULL, "adminState", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,26,0, NULL),
			array(NULL, NULL, "adminPostalCode", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,27,0, NULL),
			array(NULL, NULL, "adminCountry", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,28,0, NULL),
			array(NULL, NULL, "adminPhone", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,29,0, NULL),
			array(NULL, NULL, "adminFax", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,30,0, NULL),
			// tech contact data
			array(NULL, NULL, "techFirstname", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,31,0, NULL),
			array(NULL, NULL, "techLastName", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,32,0, NULL),
			array(NULL, NULL, "techCompany", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,33,0, NULL),
			array(NULL, NULL, "techEmail", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,34,0, NULL),
			array(NULL, NULL, "techAddress1", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,35,0, NULL),
			array(NULL, NULL, "techAddress2", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,36,0, NULL),
			array(NULL, NULL, "techCity", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,37,0, NULL),
			array(NULL, NULL, "techState", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,38,0, NULL),
			array(NULL, NULL, "techPostalCode", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,39,0, NULL),
			array(NULL, NULL, "techCountry", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,40,0, NULL),
			array(NULL, NULL, "techPhone", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,41,0, NULL),
			array(NULL, NULL, "techFax", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,42,0, NULL),
			// aux billing contact data
			array(NULL, NULL, "auxBillingFirstname", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,43,0, NULL),
			array(NULL, NULL, "auxBillingLastName", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,44,0, NULL),
			array(NULL, NULL, "auxBillingCompany", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,45,0, NULL),
			array(NULL, NULL, "auxBillingEmail", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,46,0, NULL),
			array(NULL, NULL, "auxBillingAddress1", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,47,0, NULL),
			array(NULL, NULL, "auxBillingAddress2", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,48,0, NULL),
			array(NULL, NULL, "auxBillingCity", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,49,0, NULL),
			array(NULL, NULL, "auxBillingState", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,50,0, NULL),
			array(NULL, NULL, "auxBillingPostalCode", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,51,0, NULL),
			array(NULL, NULL, "auxBillingCountry", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,52,0, NULL),
			array(NULL, NULL, "auxBillingPhone", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,53,0, NULL),
			array(NULL, NULL, "auxBillingFax", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,54,0, NULL),
			// name server data
			array(NULL, NULL, "domainNameserver1", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS1", "", "t", 0,55,0, array()),
			array(NULL, NULL, "domainNameserver2", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS2", "", "t", 0,56,0, array()),
			array(NULL, NULL, "domainNameserver3", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS3", "", "t", 0,57,0, array()),
			array(NULL, NULL, "domainNameserver4", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS4", "", "t", 0,58,0, array())

        		); // $defaultConfigParams = array(


		// Grab the configs set at the registrar level
		return array_merge($this->configParams,$defaultConfigParams);

	} // END addDefaultConfigParams()

} // Directi

if (!function_exists('array_combine')) {
	function array_combine($keys, $values) {
		// make sure matches system function behavior

		if (count($keys) != count($values)) {
			return false;
		}

		if (!count($keys)) {
			return false;
		}

		$params = array();
		foreach( $keys as $k => $v ) {
			$params[ $v ] = $args[ $k ];
		}
		return $params;
	}
}
