<?php

require_once('lib-mbapi/include/modules/registrar/registrar.php');

$className = 'eNom';
$GLOBALS['moduleInfo'][$className] = array(
    'type'          =>      'registrar',
    'status'        =>      Registrar::STATUS_STABLE,
    'version'       =>      '2.0.0',
    'displayName'   =>      'eNom',
    'author'        =>      'Parallels',
	'file' 			=> 		__FILE__,
	'countryCodes' => array('US'),
);

class eNom extends Registrar
{
    /**
     * Returns the module display for the eNom module. This includes version number and display name.
     * the columns in config_params. This array will be looped by SetConfigGroups.
     *
     * @return    An array of arrays containing all the display info for the module.
     */
    function getModuleInfo() {
        return $GLOBALS['moduleInfo']['eNom'];
    }

    /**
     * Construct a new eNom object. This function merely sets up capabilities after calling the
     * constructor for Registrar.
     */
    function eNom($input) {

        // quick fix to enable IDN support across all functions
    if (isset($this->input['domainSLD']) && trim($this->input['domainSLD']) != '') {
        $this->input['domainSLD'] = $this->getPUNYCode($this->input['domainSLD']);
    }

         // the following are types of contacts allowed by eNom
         $this->contactTypes = array(
             'Registrant',
             'Tech',
             'Admin',
             'AuxBilling'
         );

         // the following contact information is allowed on most actions
         $this->contactParams = array(
             'FirstName',
             'LastName',
             'OrganizationName',
             'JobTitle',
             'Address1',
             'Address2',
             'City',
             'StateProvince',
             'PostalCode',
             'Country',
             'EmailAddress',
             'Phone',
             'Fax'
         );

        parent::Registrar($input);

         // Create an array of boolean values as to what capabilities the eNom module has
         $eNomCapabilities = array(

             // Basic Options
             REGISTRAR_TRANSFERDOMAIN                   => 1,
             REGISTRAR_RENEWDOMAIN                      => 1,
             REGISTRAR_GETDOMAINEXTENDEDINFO            => 1,
             REGISTRAR_CHECKDOMAINAVAILABILITY          => 0,
             REGISTRAR_CHECKDOMAINTRANSFERAVAILABILITY  => 1,

             // Domain Management Options
             REGISTRAR_GETDOMAINCONTACTDATA             => 1,
             REGISTRAR_SETDOMAINCONTACTDATA             => 1,
             REGISTRAR_GETDOMAINRENEWTYPEOPTIONS        => 1,
             REGISTRAR_SETDOMAINRENEWTYPEOPTIONS        => 1,
             REGISTRAR_GETDOMAINREGISTRARLOCKOPTIONS    => 1,
             REGISTRAR_SETDOMAINREGISTRARLOCKOPTIONS    => 1,
             REGISTRAR_GETDOMAINACCESSPASSWORD          => 1,
             REGISTRAR_SETDOMAINACCESSPASSWORD          => 1,
             REGISTRAR_GETDOMAINDNS                     => 1,
             REGISTRAR_SETDOMAINDNS                     => 1,
             REGISTRAR_GETDOMAINRENEWDATE               => 1,
             REGISTRAR_LIVERENEW                        => 1,

             // Account Management Options
             REGISTRAR_CHECKACCOUNTBALANCE              => 0,
             REGISTRAR_GETTLDPRICING                    => 0,
             REGISTRAR_GETTRANSFERSTATUS                => 0,
             REGISTRAR_GETTLDLIST                       => 0,
			 REGISTRAR_TLDS                             => array(),
         );

		$this->UID = ! empty($this->UID) ? trim($this->UID) : '';
		$this->PW = ! empty($this->PW) ? trim($this->PW) : '';

         // merge any passed down capabilities and the eNom specific capabilities
         $this->capabilities = (isset($this->capabilities) && is_array($this->capabilities))
             ? $this->capabilities + $eNomCapabilities
             : $eNomCapabilities;
     }

    // this function sends the call to register a new domain
    function registerDomain() {

        // initialize the $postValues array
        $postValuesArray = array();

    // initialize the arrayData variable
        $this->arrayData = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'Purchase';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];
        $postValuesArray['NumYears'] = $this->input['domainYears'];
        $postValuesArray['UnLockRegistrar'] = 0;
        $postValuesArray['DomainPassword'] = '';
        $postValuesArray['EmailNotify'] = 0;
        $postValuesArray['QueueOrder'] = 0;
        $postValuesArray['IgnoreNSFail'] = 'Yes';
        $postValuesArray['UseCreditCard'] = 'no';

        if (1 == $this->input['domainRenewType']) { // 1 = registrar, 2 = ppb renew, 0 = don't renew
            $postValuesArray['Renewname'] = 1;
        } else {
            $postValuesArray['Renewname'] = 0;
        }

        if ($this->config['moduleUseCustomNameserver'] && $this->config['domainNameserver1']) {
            $configNameservers = array(
                'domainNameserver1',
                'domainNameserver2',
                'domainNameserver3',
                'domainnameserver4'
            );

            $i = 1;
            foreach ($configNameservers as $configNameserver) {
                if ($this->config[$configNameserver]) {
                    $postValuesArray['NS'.$i] = $this->config[$configNameserver];
                }
                $i++;
            }

        } else {
            $postValuesArray['UseDNS'] = 'default';
        }

        // the information coming through $this->input does not have the proper
        // element names for everything, the below code maps everything to the
        // proper element names for eNom's API... it also applies the same rules
        // that are applied above to $this->config
        $this->input['RegistrantFirstName'] = $this->input['registrantFirstName'];
        $this->input['RegistrantLastName'] = $this->input['registrantLastName'];
        $this->input['RegistrantOrganizationName'] = $this->input['registrantOrganizationName'];
        $this->input['RegistrantJobTitle'] = $this->input['registrantJobTitle'];
        $this->input['RegistrantAddress1'] = $this->input['registrantAddress1'];
        $this->input['RegistrantAddress2'] = $this->input['registrantAddress2'];
        $this->input['RegistrantCity'] = $this->input['registrantCity'];
        $this->input['RegistrantStateProvince'] = $this->input['registrantState'];
        $this->input['RegistrantPostalCode'] = $this->input['registrantPostalCode'];
        $this->input['RegistrantCountry'] = $this->input['registrantCountry'];
        $this->input['RegistrantEmailAddress'] = $this->input['registrantEmail'];
        $this->input['RegistrantPhone'] = $this->input['registrantPhone'];
        $this->input['RegistrantFax'] = $this->input['registrantFax'];


        // place all contact input/config into one array for analysis
        $impureContactArray = array();
        foreach ($this->contactTypes as $contactType) {
            if ($contactType == 'Registrant') {
                foreach ($this->contactParams as $contactParam) {
                    $impureContactArray[$contactType][$contactParam] = $this->input[$contactType.$contactParam];
                }
            } else {
                foreach ($this->contactParams as $contactParam) {
                    $impureContactArray[$contactType][$contactParam] = $this->config[$contactType.$contactParam];
                }
            }
        }


        // populate $this->registrationArray with validated/massaged data
        $this->getRegistrationContactsData($impureContactArray);

        // if the registrant contact is invalid, kill the function immediately, return an informative message
        if (!$this->hasValidContactParams('Registrant')) {
            $this->addError(TRANS_REGISTRARERROR, 'Missing or invalid required Registrant contact information.');
            foreach($this->invalidParams['Registrant'] as $invalidParam) {
                $this->addError(TRANS_REGISTRARERROR, 'Missing or invalid parameter: '.$invalidParam);
            }
            $this->setFlagsAndXML();
            return $this->xmlData;
        }

        // populate $postValuesArray with valid contact data
        foreach($this->contactTypes as $contactType) {
            if ($this->hasValidContactParams($contactType)) {
                foreach($this->contactParams as $contactParam) {
                    $postValuesArray[$contactType.$contactParam] = $this->registrationArray['Contacts'][$contactType][$contactParam];
                }
            }
        }

        // fetch extended attributes for this TLD and make sure that we have all required parameters
        $TLDExtendedAttributes = $this->getExtendedAttributes($this->input['domainTLD']);
        if (isset($TLDExtendedAttributes['Attributes'])
            && is_array($TLDExtendedAttributes['Attributes'])
            && count($TLDExtendedAttributes['Attributes']) > 0) {
			reset($TLDExtendedAttributes['Attributes']);
            foreach (is_int(key($TLDExtendedAttributes['Attributes'])) ? $TLDExtendedAttributes['Attributes'] : array($TLDExtendedAttributes['Attributes'])  as $TLDExtendedAttribute) {
                if ($TLDExtendedAttribute['Required'] == 1) {
                    if (isset($TLDExtendedAttribute['Options']) && is_array($TLDExtendedAttribute['Options']) && count($TLDExtendedAttribute['Options']) > 0) {
                        $allowedOptions = array();
						reset($TLDExtendedAttribute['Options']);
                        foreach (is_int(key($TLDExtendedAttribute['Options'])) ? $TLDExtendedAttribute['Options'] : array($TLDExtendedAttribute['Options']) as $TLDExtendedAttributeOption) {
                            $allowedOptions[] = $TLDExtendedAttributeOption['Value'];
                        }
                        if (isset($this->input[$TLDExtendedAttribute['Name']])
                            && trim($this->input[$TLDExtendedAttribute['Name']]) != ''
                            && array_search($this->input[$TLDExtendedAttribute['Name']], $allowedOptions) !== FALSE) {
                            $postValuesArray[$TLDExtendedAttribute['Name']] = $this->input[$TLDExtendedAttribute['Name']];
                        } else {
                            $this->addError(TRANS_REGISTRARERROR, 'Missing or invalid required parameter: '.$TLDExtendedAttribute['Name']);
                            $this->hasExecutedSuccessfully = 0;
                            $this->hasExecuted = 1;
                            $this->xmlData = $this->getRegistrarReturnXML();
                            return $this->xmlData;
                        }
                    } else {
                        if (isset($this->input[$TLDExtendedAttribute['Name']]) && trim($this->input[$TLDExtendedAttribute['Name']]) != '') {
                            $postValuesArray[$TLDExtendedAttribute['Name']] = $this->input[$TLDExtendedAttribute['Name']];
                        } else {
                            $this->addError(TRANS_REGISTRARERROR, 'Missing or invalid required parameter: '.$TLDExtendedAttribute['Name']);
                            $this->setFlagsAndXML();
                            return $this->xmlData;
                        }
                    }
                } else {
                    if (isset($this->input[$TLDExtendedAttribute['Name']]) && trim($this->input[$TLDExtendedAttribute['Name']]) != '') {
                        $postValuesArray[$TLDExtendedAttribute['Name']] = $this->input[$TLDExtendedAttribute['Name']];
                    }
                }
            }

        }

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        // return some arrayData
        $this->arrayData = $eNomAPIReturn;
        $this->arrayData['domainRemotePurchaseID'] = $eNomAPIReturn['OrderID'];

    	switch ($eNomAPIReturn['RRPCode']) {
    		//successful
    		case 200:
    		case 1300:
	            // if necessary, purchase ID protect too
	            if ( isset($this->input['idProtect']) && $this->input['idProtect'] ) {
	            	dbg("============= IDPROTECT WAS ATTEMPTED ==============");
	                $this->purchaseIDProtect();
	            } else {
	            	dbg("============= NO IDPROTECT WAS ATTEMPTED ==============");
	            }
	            $this->setFlagsAndXML(1);
	            break;
	        // error
    		case NULL:
    			$this->addEnomErrors($eNomAPIReturn["errors"]);
				break;
			// error
			default:
	            $this->addError(blmsg("TRANS_REGISTRARERROR"), $eNomAPIReturn['RRPText']);
	            $this->setFlagsAndXML();
	            break;
    	}
    	return $this->xmlData;
    }


    // this function executes a domain transfer
    function transferDomain() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'TP_CreateOrder';
        $postValuesArray['DomainCount'] = 1;
        $postValuesArray['OrderType'] = 'Autoverification';
        $postValuesArray['SLD1'] = $this->input['domainSLD'];
        $postValuesArray['TLD1'] = $this->input['domainTLD'];
        $postValuesArray['AuthInfo1'] = $this->input['domainTransferKey'];
        $postValuesArray['UseContacts'] = 1;
        $postValuesArray['UseCreditCard'] = 'no';

        if (1 == $this->input['domain_renew_type']) { // 1 = registrar, 2 = ppb renew, 0 = don't renew
            $postValuesArray['Renewname'] = 1;
        } else {
            $postValuesArray['Renewname'] = 0;
        }

        if (!isset($this->input['domainTransferKey']) || trim($this->input['domainTransferKey']) == '') {
            $this->addError(TRANS_REGISTRARERROR, 'Error: No EPP Code Present!');
            $this->setFlagsAndXML();
            return $this->xmlData;
        }

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        // return some arrayData
        $this->arrayData = $eNomAPIReturn;
        $this->arrayData['domainRemotePurchaseID'] = $eNomAPIReturn['TransferOrderID'];

        if ($eNomAPIReturn['ErrCount'] == 0) {
        	$this->setFlagsAndXML(1);
        } else {
            $this->addEnomErrors($eNomAPIReturn["errors"]);
        }
        return $this->xmlData;

    }

    // this function executes a domain renewal
    function renewDomain() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'Extend';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];
        $postValuesArray['NumYears'] = $this->input['domainYears'];
        $postValuesArray['OverrideOrder'] = 0;
        $postValuesArray['UseCreditCard'] = 'no';

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        // return some array data
        $this->arrayData['domainRemotePurchaseID'] = $eNomAPIReturn['TransferOrderID'];
        $this->arrayData['DomainRenewOrderID'] = $eNomAPIReturn['TransferOrderID'];
        $this->arrayData['DomainRenewMessage'] = $eNomAPIReturn['RRPText'];
        $this->arrayData['OtherData'] = $eNomAPIReturn;

        // check to see if successful
        switch ($eNomAPIReturn['RRPCode']) {
    		//successful
    		case 200:
    		case 1300:
	            $this->arrayData["DomainRenewSuccess"] = 1;
				$this->setFlagsAndXML(1);
	            break;
	        // error
    		case NULL:
    			$this->arrayData["DomainRenewSuccess"] = 0;
    			$this->addEnomErrors($eNomAPIReturn["errors"]);
				break;
			// error
			default:
				$this->arrayData["DomainRenewSuccess"] = 0;
	            $this->addError(blmsg("TRANS_REGISTRARERROR"), $eNomAPIReturn['RRPText']);
	            $this->setFlagsAndXML();
	            break;
    	}
        return $this->xmlData;

    }


    // this function fetches a domain's contact data
    function getDomainContactData() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'GetContacts';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        if ($eNomAPIReturn['ErrCount'] == 0) {

            $this->arrayData['DomainData'] = $eNomAPIReturn['GetContacts']['domainname']['_attributes'];
            $this->arrayData['DomainData']['domainname'] = $eNomAPIReturn['GetContacts']['domainname'][0];
            $this->arrayData['RegistrantContactData'] = $eNomAPIReturn['GetContacts']['Registrant'];
            $this->arrayData['AuxBillingContactData'] = $eNomAPIReturn['GetContacts']['AuxBilling'];
            $this->arrayData['TechContactData'] = $eNomAPIReturn['GetContacts']['Tech'];
            $this->arrayData['AdminContactData'] = $eNomAPIReturn['GetContacts']['Admin'];
            $this->arrayData['BillingContactData'] = $eNomAPIReturn['GetContacts']['Billing'];
            $this->arrayData['OtherData'] = $eNomAPIReturn['GetContacts'];

			$this->setFlagsAndXML(1);

        } else {
            $this->addEnomErrors($eNomAPIReturn["errors"]);
        }
        return $this->xmlData;
    }


    // this function sets a domain's contact data
    function setDomainContactData() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'Contacts';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];

        $newDomainContactData = $this->input['domainContactData'];

        // we don't have the right names coming into this module for passing
        // straight to eNom, so we're going to remap them below! :D
        // and place all contact input/config into one array for analysis
        $impureContactArray = array();

        foreach(array('registrant','admin','tech','auxBilling') as $impureContactType) {
            foreach($this->contactParams as $contactParam) {
                $impureContactArray[ucfirst($impureContactType)][$contactParam] =
                    $newDomainContactData[$impureContactType][$impureContactType.$contactParam];
            }
        }

        // populate $this->registrationArray
        $this->getRegistrationContactsData($impureContactArray);

        // if the registrant contact is invalid, kill the function immediately, return an informative message
        if (!$this->hasValidContactParams('Registrant')) {
            $this->addError(TRANS_REGISTRARERROR, 'Missing or invalid required Registrant contact information.');
            foreach($this->invalidParams['Registrant'] as $invalidParam) {
                $this->addError(TRANS_REGISTRARERROR, 'Missing or invalid parameter: '.$invalidParam);
            }
			$this->setFlagsAndXML();
            return $this->xmlData;
        }

        // populate $postValuesArray with valid contact data
        foreach($this->contactTypes as $contactType) {
            if ($this->hasValidContactParams($contactType)) {
                foreach($this->contactParams as $contactParam) {
                    $postValuesArray[$contactType.$contactParam] = $this->registrationArray['Contacts'][$contactType][$contactParam];
                }
            }
        }

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        // return some arrayData
        $this->arrayData = $eNomAPIReturn;

        if ($eNomAPIReturn['ErrCount'] == 0) {
			$this->setFlagsAndXML(1);
        } else {
            $this->addEnomErrors($eNomAPIReturn["errors"]);
        }
		return $this->xmlData;
    }


    // this function fetches a domain's renew type
    function getdomainRenewTypeOptions() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'GetRenew';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        if ($eNomAPIReturn['ErrCount'] == 0) {

            $this->arrayData['domainRenewType'] = $eNomAPIReturn['auto-renew'];
            $this->arrayData['EmailForwardAutoRenew'] = $eNomAPIReturn['EmailForwardRenew'];
            $this->arrayData['IDProtectAutoRenew'] = $eNomAPIReturn['IDProtectRenew'];
            $this->arrayData['URLForwardAutoRenew'] = $eNomAPIReturn['URLForwardRenew'];
            $this->arrayData['OtherData'] = $eNomAPIReturn;

			$this->setFlagsAndXML(1);

        } else {
            $this->addEnomErrors($eNomAPIReturn["errors"]);
        }
        return $this->xmlData;
    }


    // this function sets a domain's renew type
    function setdomainRenewTypeOptions() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'SetRenew';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];
        $postValuesArray['RenewFlag'] = $this->input['domainRenewType'];

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        if ($eNomAPIReturn['ErrCount'] == 0) {

            $this->arrayData['domainRenewType'] = $eNomAPIReturn['RenewName'];
            $this->arrayData["OtherData"] = $eNomAPIReturn;

			$this->setFlagsAndXML(1);
            return $this->xmlData;
        } else {
            $this->addEnomErrors($eNomAPIReturn["errors"]);
        }

        return $this->xmlData;
    }


    // this function fetches a domain's registrar lock status
    function getDomainRegistrarLockOptions() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'GetRegLock';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        // check to see if successful
        switch ($eNomAPIReturn['RRPCode']) {
    		//successful
    		case 200:
    		case 1300:
	            $this->arrayData['DomainRegistrarLock'] = $eNomAPIReturn['reg-lock'];
            	$this->arrayData['DomainIsLockable'] = ($eNomAPIReturn['IsLockable'] == 'True') ? 1 : 0;
            	$this->arrayData['OtherData'] = $eNomAPIReturn;
				$this->setFlagsAndXML(1);
	            break;
	        // error
    		case NULL:
    			$this->addEnomErrors($eNomAPIReturn["errors"]);
				break;
			// error
			default:
	            $this->addError(blmsg("TRANS_REGISTRARERROR"), $eNomAPIReturn['RRPText']);
	            $this->setFlagsAndXML();
	            break;
    	}

        return $this->xmlData;
    }


    // this function sets a domain's registrar lock status
    function setDomainRegistrarLockOptions() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'SetRegLock';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];
        $postValuesArray['UnlockRegistrar'] = ($this->input['domainRegistrarLock'] == 1) ? 0 : 1;

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        if ($eNomAPIReturn['ErrCount'] == 0) {

            $this->arrayData["DomainRegistrarLock"] = ($eNomAPIReturn['RegistrarLock'] != 'Failed') ? 1 : 0;
            $this->arrayData["OtherData"] = $eNomAPIReturn;
			$this->setFlagsAndXML(1);

        } else {
            $this->addEnomErrors($eNomAPIReturn["errors"]);
        }

        return $this->xmlData;
    }


    // this function fetches a domain's access password
    function getDomainAccessPassword() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'GetPasswordBit';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        if ($eNomAPIReturn['ErrCount'] == 0) {

           $this->arrayData["DomainAccessPasswordSet"] = $eNomAPIReturn['password-set'];
           $this->arrayData["DomainAccessPassword"] = ($eNomAPIReturn['password-set']) ? $eNomAPIReturn['DomainPassword'] : '';
           $this->arrayData["OtherData"] = $eNomAPIReturn;

            $this->setFlagsAndXML(1);
        } else {
        	$this->addEnomErrors($eNomAPIReturn["errors"]);
        }
        return $this->xmlData;
    }


    // this function sets a domain's access password
    function setDomainAccessPassword() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'SetPassword';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];
        $postValuesArray['DomainPassword'] = $this->input['domainAccessPassword'];

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        // return from ArrayData
        $this->arrayData["OtherData"] = $eNomAPIReturn;

        if ($eNomAPIReturn['ErrCount'] == 0) {
            $this->setFlagsAndXML(1);
        } else {
            $this->addEnomErrors($eNomAPIReturn["errors"]);
        }
		return $this->xmlData;
    }


    // this function fetches a domain's expiration date
    function getDomainExpDate() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'GetDomainExp';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        // return from ArrayData
        $this->arrayData['OtherData'] = $eNomAPIReturn;

        if ($eNomAPIReturn['ErrCount'] == 0) {
            $this->arrayData['DomainExpDate'] = $eNomAPIReturn['ExpirationDate'];
            $this->arrayData['DomainExpTimestamp'] = strtotime($this->arrayData['DomainExpDate']);
            $this->setFlagsAndXML(1);
        } else {
            $this->addEnomErrors($eNomAPIReturn["errors"]);
        }
        return $this->xmlData;
    }


    // this function fetches a domain's renewal information
     function getDomainExtendedInfo() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'GetExtendInfo';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        if ($eNomAPIReturn['ErrCount'] == 0) {

            $this->arrayData["DomainExpDate"] = $eNomAPIReturn['Expiration'];
            $this->arrayData["DomainExpTimestamp"] = strtotime($this->arrayData["DomainExpDate"]);
            $this->arrayData["DomainRenewMaxYears"] = $eNomAPIReturn['MaxExtension'];
            $this->arrayData["DomainRenewDefaultYears"] =
                ((int)$eNomAPIReturn['DefPeriod'])
                    ? $eNomAPIReturn['DefPeriod']
                    : 1;
            $this->arrayData["DomainRenewPrice"] =
                ($eNomAPIReturn['Price'])
                    ? $eNomAPIReturn['Price']
                    : 0;
            $this->arrayData["OtherData"] = $eNomAPIReturn;

            $this->setFlagsAndXML(1);
        } else {
            $this->addEnomErrors($eNomAPIReturn["errors"]);
        }
        return $this->xmlData;
    }


    // this function fetches a domain's dns info
    function getDomainDNS() {

        $dnsOptions = array(
            'A' => 'A',
            'URL' => 'URL',
            'MX' => 'MX',
            'MXE' => 'MXE',
            'CNAME' => 'CNAME',
            'FRAME' => 'FRAME',
            'TXT' => 'TXT'
        );

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'GetDNS';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        // return some ArrayData
        $this->arrayData["OtherData"] = $eNomAPIReturn;

        if ($eNomAPIReturn['ErrCount'] == 0) {

            $this->arrayData["listDNS"] = $eNomAPIReturn['dns'];
            $this->arrayData["UseDNS"] = ($eNomAPIReturn['UseDNS'] == 'default') ? 1 : 0 ;

        } else {

            $this->addEnomErrors($eNomAPIReturn["errors"]);
            return $this->xmlData;

        }

        // ...making the second call
        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'GetHosts';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        if ($eNomAPIReturn['ErrCount'] == 0) {

            $this->arrayData['host'] = $eNomAPIReturn['host'];
            $this->arrayData['DomainServices'] = $eNomAPIReturn['DomainServices'];
            $this->arrayData['dnsOptions'] = $dnsOptions;

            $this->setFlagsAndXML(1);
        } else {
            $this->addEnomErrors($eNomAPIReturn["errors"]);
        }
        return $this->xmlData;

    }


    // this function sets a domains dns info
    function setDomainDNS() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'ModifyNS';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];

        if ($this->input['domainDNS'] != 0) {
            $postValuesArray['UseDNS'] = 'Default';
        } else {
            if (!is_array($this->input['domainNameserver'])) {
            	$this->input['domainNameserver'] = array($this->input['domainNameserver']);
            }
            for($i=1;$i<=8;$i++) {
                $postValuesArray['NS'.$i] = $this->input['domainNameserver'][$i-1];
            }
        }

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        if ($eNomAPIReturn['ErrCount'] == 0) {
            $this->arrayData["result"] = "TRANS_REGISTRAR_MANAGE_DNS_UPDATED";
        } else {
            $this->arrayData["result"] = "TRANS_REGISTRAR_MANAGE_DNS_FAILED";
            $this->addEnomErrors($eNomAPIReturn["errors"]);
            return $this->xmlData;
        }

        // ... and making the second call
        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'SetHosts';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];

	if (isset($this->input['host']['RecordType'])) {
		$this->input['host'] = array($this->input['host']);
	}
        // Loop the map and pull the needed fields
        foreach($this->input['host'] as $x => $value) {
            $fieldNum = $x + 1;
            $postValuesArray['HostName'.$fieldNum] = (substr($value['HostName'], -1)==".") ? substr($value['HostName'], 0, -1) : $value['HostName'];
            $postValuesArray['RecordType'.$fieldNum] = $value['RecordType'];
            $postValuesArray['Address'.$fieldNum] = $value['Address'];
        }

        // make the curl call and capture the return array to be evaluated
        $eNomAPIReturn = $this->makeTheCall($postValuesArray);

        if ($eNomAPIReturn['ErrCount'] == 0) {
            $this->arrayData["result"] = "TRANS_REGISTRAR_MANAGE_DNS_UPDATED";
            $this->setFlagsAndXML(1);
        } else {

            $this->arrayData["result"] = "TRANS_REGISTRAR_MANAGE_DNS_FAILED";
            $this->addEnomErrors($eNomAPIReturn["errors"]);
        }
        return $this->xmlData;

    }


    // if the client is purchasing ID Protect as well, make the call to buy it for them
    function purchaseIDProtect() {

        // initialize the $postValues array
        $postValuesArray = array();

        // set the main parameters we'll be sending to eNom
        $postValuesArray['Command'] = 'PurchaseServices';
        $postValuesArray['Service'] = 'WPPS';
        $postValuesArray['SLD'] = $this->input['domainSLD'];
        $postValuesArray['TLD'] = $this->input['domainTLD'];
        $postValuesArray['NumYears'] = $this->input['domainYears'];
        $postValuesArray['EmailNotify'] = 0;
        $postValuesArray['UseCreditCard'] = 'no';

        return $eNomAPIReturn = $this->makeTheCall($postValuesArray);
    }

    // from any function that needs to make a cURL call, we will pass the
    // post values to this function, which will actually make the call
    function makeTheCall($postValuesArray) {

        // turn the post values array into a url string which can be appended to the url in the curl call
        $postValues = '';
        foreach($postValuesArray as $parameter => $value) {
            $postValues .= '&'.urlencode($parameter).'='.urlencode($value);
        }

        // make the curl call, parse the xml return into an array
        $eNomAPICall = $this->getCurlObject($this->getURL().'/interface.asp?UID='.$this->UID.'&PW='.$this->PW.'&ResponseType=XML'.$postValues);
        $eNomAPICall = new MBXMLLite($eNomAPICall->toString());
        $eNomAPICall = correctArray($eNomAPICall->toArray());

        $eNomAPICall = collapseArrayElements($eNomAPICall['interface-response'][0]);

        // return the array
        return $eNomAPICall;

    }

    // this function sets the API URL based on whether we're in LIVE mode or TEST mode
    function getURL() {
        global $CONF;

        if ($this->config['moduleTestMode']) {
            // TEST Mode URL
            $url = 'https://resellertest.enom.com';
        } else {
            // LIVE Mode URL
            $url = 'https://reseller.enom.com';
        }

        return $url;
     }

    // this function fetches any special parameters required by a given TLD
    function getExtendedAttributes($tld) {
        $postValuesArray['Command'] = 'GetExtAttributes';
        $postValuesArray['TLD'] = $tld;

        return $this->makeTheCall($postValuesArray);
    }

    // this function checks a given contact type to see whether is has all the
    // data necessary in order to be placed successfully through the eNom API
    function hasValidContactParams($contactType) {

        $this->invalidParams[$contactType] = array();
        $missingRequiredParam = FALSE;

        // loop through each of the contact params for this contact type
        // (except for Address2) and make sure they are populated
        foreach ($this->contactParams as $contactParam) {
            if ($contactParam != 'Address2') {
                if (trim($this->registrationArray['Contacts'][$contactType][$contactParam]) == '') {
                    $missingRequiredParam = TRUE;
                    $this->invalidParams[$contactType][] = $contactParam;
                }

            }
        }

        // return a status based on the validity of the data for this contact type
        if ($missingRequiredParam)
            return FALSE;
        else
            return TRUE;
    }

    // this function creates an array of contact information that comes through $this->input
    // and $this->config that can be used in any of the standard module functions
    // this function also massages the data as much as it can prior to inserting it into the
    // array in order to help ensure success through eNom's API
     function getRegistrationContactsData($impureContactArray) {

        $this->registrationArray = array();

        foreach($this->contactTypes as $contactType) {

            // if they did not enter an organization name, populate with first and last name
            $impureContactArray[$contactType]['OrganizationName'] =
                (isset($impureContactArray[$contactType]['OrganizationName']) && trim($impureContactArray[$contactType]['OrganizationName']) != '')
                    ? $impureContactArray[$contactType]['OrganizationName']
                    : $impureContactArray[$contactType]['FirstName'].' '.$impureContactArray[$contactType]['LastName'];

            // if they did not enter a job title, populate with "Owner"
            $impureContactArray[$contactType]['JobTitle'] =
                (isset($impureContactArray[$contactType]['JobTitle']) && trim($impureContactArray[$contactType]['JobTitle']) != '')
                    ? $impureContactArray[$contactType]['JobTitle']
                    : 'Owner';

            // if they did not enter a fax number, populate with the phone number
            $impureContactArray[$contactType]['Fax'] =
                (isset($impureContactArray[$contactType]['Fax']) && strlen($impureContactArray[$contactType]['Fax']) > 6)
                    ? $impureContactArray[$contactType]['Fax']
                    : $impureContactArray[$contactType]['Phone'];

            // make sure the phone number follows the correct format for eNom's API
            // if it does not, massage the data so that it does
            if (!preg_match("/\+[[:digit:]]+\.[[:digit:]]+/", $impureContactArray[$contactType]['Phone'])) {
                $impureContactArray[$contactType]['Phone'] = preg_replace("/[^[:digit:]]/", "", $impureContactArray[$contactType]['Phone']);
                $impureContactArray[$contactType]['Phone'] = '+1'.$impureContactArray[$contactType]['Phone'];
            }

            // make sure the fax number follows the correct format for eNom's API
            // if it does not, massage the data so that it does
            if (!preg_match("/\+[[:digit:]]+\.[[:digit:]]+/", $impureContactArray[$contactType]['Fax'])) {
                $impureContactArray[$contactType]['Fax'] = preg_replace("/[^[:digit:]]/", "", $impureContactArray[$contactType]['Fax']);
                $impureContactArray[$contactType]['Fax'] = '+1'.$impureContactArray[$contactType]['Fax'];
            }

            // make sure the email address is entered properly, if not blank
            // it so that the contact type will not be submitted to eNom's API
            if (!preg_match(" /^[[:alnum:]_\+\.]+@[[:alnum:]\-\.]+$/", $impureContactArray[$contactType]['EmailAddress'])) {
                $impureContactArray[$contactType]['EmailAddress'] = '';
            }

            // make sure the country is in ISO2 format, if not force it through as "US"
            $impureContactArray[$contactType]['Country'] = strtoupper($impureContactArray[$contactType]['Country']);
            if (strlen($impureContactArray[$contactType]['Country']) != 2) {
                $impureContactArray[$contactType]['Country'] = 'US';
            }

            // now that all the contact data is proper formatted, populate the array which
            // we'll use for hitting eNom's API
            foreach($this->contactParams as $contactParam) {
                $this->registrationArray['Contacts'][$contactType][$contactParam] =
                    (isset($impureContactArray[$contactType][$contactParam]) && trim($impureContactArray[$contactType][$contactParam]) != '')
                        ? $impureContactArray[$contactType][$contactParam]
                        : '';
            }

        }

    }

    // this function populates the module configuration screen with options the user can set
    function addDefaultConfigParams() {

        // the below are the administrative options that can be configured for the eNom module
        /*
         * [0] = option name
         * [1] = option type (t = text, p = password, r = radio)
         * [2] = default value
         * [3] = option label
         * [4] = option description
         */
        $adminOptions = array(
            array('moduleUsername',                         't',    NULL,   'USERNAME',                             NULL,                                   NULL, 1),
            array('modulePassword',                         'p',    NULL,   'PASSWORD',                             NULL,                                   NULL, 1),
            array('moduleTestMode',                         'b',    '1',    'ENABLE_TEST_MODE',                     NULL,                                   NULL, 0),
            array('moduleAllowAutoRenew',                   'r',    '0',    'ALLOW_AUTORENEW',                      'ALLOW_AUTORENEW_DESC',
                array(
                    array(
                        NULL,
                        NULL,
                        '0',
                        'TRANS_OBEY_TLD',
                        1
                    ),
                    array(
                        NULL,
                        NULL,
                        '1',
                        'TRANS_REGISTRAR_AUTORENEW_OPTION',
                        2
                    ),
                    array(
                        NULL,
                        NULL,
                        '2',
                        'TRANS_MODERNBILL_RENEW_OPTION',
                        3
                    ),
                ), 0
            ),
            array('moduleUseCustomNameserver',              'b',    '0',    'USE_CUSTOM_NAMESERVER',                'USE_CUSTOM_NAMESERVER_DESC',           NULL, 0),
            array('moduleAllowChangeDNS',                   'b',    '0',    'ALLOW_CHANGE_DNS',                     'ALLOW_CHANGE_DNS_DESC',                NULL, 0),
            array('moduleAllowChangeContactInfo',           'b',    '1',    'ALLOW_CHANGE_CONTACT_INFO',            'ALLOW_CHANGE_CONTACT_INFO_DESC',       NULL, 0),
            array('moduleAllowChangeRegistrarLock',         'b',    '0',    'ALLOW_CHANGE_REGISTRAR_LOCK',          'ALLOW_CHANGE_REGISTRAR_LOCK_DESC',     NULL, 0),
            array('moduleAllowChangeAutorenew',             'b',    '0',    'ALLOW_CHANGE_AUTORENEW',               'ALLOW_CHANGE_AUTORENEW_DESC',          NULL, 0),
            array('moduleAllowChangeUpdateDomainPassword',  'b',    '0',    'ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD',  'ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD',  NULL, 0),
        );

        // the below should not need to be changed,
        // edit config params in the temp array above
        $i = 1;
        foreach($adminOptions as $adminOption) {
            $moduleConfig[] = array(
                NULL,
                NULL,
                $adminOption[0],
                $adminOption[2],
                'TRANS_REGISTRAR_MODULE_SETTINGS',
                'TRANS_REGISTRAR_ENOM_'.$adminOption[3],
                (is_null($adminOption[4]) ? '' : 'TRANS_REGISTRAR_ENOM_'.$adminOption[4]),
                $adminOption[1],
                $adminOption[6],
                $i++,
                0,
                ($adminOption[1] == 'r')
                    ? (($adminOption[5] != NULL) ? $adminOption[5] : array(
                        array(NULL,NULL,'1','TRANS_YES',2),
                        array(NULL,NULL,'0','TRANS_NO',1)
                    ))
                    : NULL
            );
        }

        // generate the necessary fields so that Admin/Tech/AuxBilling contacts can be filled
        // out in the eNom module configuration screen
        foreach($this->contactTypes as $contactType) {
            if ($contactType != 'Registrant') {
                foreach($this->contactParams as $contactParam) {
					switch($contactType) {
						case 'Admin' : $groupTransKey = 'TRANS_ADMINISTRATIVE_CONTACT'; break;
						case 'Tech' : $groupTransKey = 'TRANS_TECHNICAL_CONTACT'; break;
						case 'AuxBilling' : $groupTransKey = 'TRANS_BILLING_CONTACT'; break;
						default: $groupTransKey = 'TRANS_REGISTRAR_'.strtoupper($contactType).'_CONTACT';
					}
					$contactConfig[] = array(
					NULL,
					NULL,
					$contactType.$contactParam,
					'',
					$groupTransKey,
					'TRANS_REGISTRAR_'.strtoupper($contactType).'_'.strtoupper(str_replace(' ','_',variableToHuman($contactParam))),
					'',
					't',
					0,
					$i++,
					0,
					NULL
					);
                }
            }
        }

        // generate the nameserver fields that can be filled out in the
        // eNom module configuration screen
        $nameServerConfig = array(
            array(NULL,NULL,'domainNameserver1','','TRANS_REGISTRAR_NS','TRANS_REGISTRAR_NS1','','t',0,$i++,0,array()),
            array(NULL,NULL,'domainNameserver2','','TRANS_REGISTRAR_NS','TRANS_REGISTRAR_NS2','','t',0,$i++,0,array()),
            array(NULL,NULL,'domainNameserver3','','TRANS_REGISTRAR_NS','TRANS_REGISTRAR_NS3','','t',0,$i++,0,array()),
            array(NULL,NULL,'domainNameserver4','','TRANS_REGISTRAR_NS','TRANS_REGISTRAR_NS4','','t',0,$i++,0,array())
        );


        // create the master array which we'll return containing all configuration parameters
        $defaultConfigParams = array_merge($moduleConfig, $contactConfig);
        $defaultConfigParams = array_merge($defaultConfigParams, $nameServerConfig);

        // return the master array containing all configuration parameters
        return $defaultConfigParams;

    }

    public function getTLDList()
    {
        $tlds = array();
        if ( empty($this->UID) || empty($this->PW)) {
            return $tlds;        
        }
        $getTLDList = $this->makeTheCall(array('Command' => 'GetTLDList'));
        if (0 != $getTLDList['ErrCount']) {
            $this->addEnomErrors($getTLDList["errors"]);
        }
        foreach ((array)$getTLDList['tldlist']['tld'] as $tld) {
            if (trim($tld['tld']) != '') {
                 $tlds[] = $tld['tld'];
            }
        }
        $this->capabilities[REGISTRAR_TLDS] = $tlds;
        return $this->capabilities[REGISTRAR_TLDS];
    }

    private function setFlagsAndXML($hasExecutedSuccessfully = 0)
    {
	    $this->hasExecutedSuccessfully = $hasExecutedSuccessfully;
        $this->hasExecuted = 1;
        $this->xmlData = $this->getRegistrarReturnXML();
    }

    private function addEnomErrors($enomErrors)
    {
		foreach ($enomErrors as $error) {
			$this->addError(blmsg("TRANS_REGISTRARERROR"), $error);
		}
		$this->setFlagsAndXML();
    }
}
