<?php

require_once("lib-mbapi/include/modules/registrar/registrar.php");

$className = "Eurodns";
$GLOBALS["moduleInfo"][$className] = array(
    "type" => "registrar",
    "status" => Registrar::STATUS_TESTING,
    "version" => "1.0.0",
    "displayName" => "EuroDNS",
    "author" => "Parallels",
	"file" => __FILE__,
	"countryCodes" => array("EU"),
);

class Eurodns extends Registrar
{
	/**
	 * Returns the module display for the Eurodns module. This includes version number and display name.
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the display info for the module.
	 */
	function getModuleInfo() {
		return $GLOBALS["moduleInfo"]["Eurodns"];
	}

	/**
	 * Construct a new Eurodns object. This function merely sets up capabilities after calling the
	 * constructor for Registrar.
	 */
	function Eurodns($input) {

		global $CONF;
		if($CONF["eurodns"]["partnerid"]) {
			$this->UID = $this->partnerid = $CONF["eurodns"]["partnerid"];
			$this->PW = $this->signkey = $CONF["eurodns"]["signkey"];
		}

		$this->capabilities[REGISTRAR_REGISTERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_TRANSFERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_RENEWDOMAIN] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINEXTENDEDINFO] = 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINAVAILABILITY] = 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINTRANSFERAVAILABILITY] = 0;

		//Domain Management options
		$this->capabilities[REGISTRAR_GETDOMAINCONTACTDATA] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINCONTACTDATA] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWTYPEOPTIONS] = 0;
		$this->capabilities[REGISTRAR_SETDOMAINRENEWTYPEOPTIONS] = 0;
		$this->capabilities[REGISTRAR_GETDOMAINREGISTRARLOCKOPTIONS] = 0;
		$this->capabilities[REGISTRAR_SETDOMAINREGISTRARLOCKOPTIONS] = 0;
		$this->capabilities[REGISTRAR_GETDOMAINACCESSPASSWORD] = 0;
		$this->capabilities[REGISTRAR_SETDOMAINACCESSPASSWORD] = 0;
		$this->capabilities[REGISTRAR_GETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWDATE] = 0;
		$this->capabilities[REGISTRAR_LIVERENEW] = 0;

		$this->capabilities[REGISTRAR_CHECKACCOUNTBALANCE] = 1;
		$this->capabilities[REGISTRAR_GETTLDPRICING] = 0;
		$this->capabilities[REGISTRAR_GETTRANSFERSTATUS] = 0;
		$this->capabilities[REGISTRAR_GETTLDLIST] = 0;
		$this->capabilities[REGISTRAR_TLDS] = array("com", "net", "org", "biz", "fr", "se",
					"vn", "mobi", "hu", "eu", "at", "be", "cz",
					"dk", "co.ee", "de", "gr", "lv", "li", "lt",
					"lu", "nl", "pl", "com.pl", "ro", "ru", "sk",
					"es", "com.es", "se", "ch", "co.uk");
		//Per http://www.eurodns.com/info/price.php
		//for .fr
		//whois whois.nic.fr
		//response No entries found

		//Setup the array of all items to send for new registration
		$gc = array(
			"usetrustee" => "",
			"firstname" => "",
			"lastname" => "",
			"company" => "",
			"address1" => "",
			"address2" => "",
			"address3" => "",
			"city" => "",
			"country_code" => "",
			"zipcode" => "",
			"email" => "",
			"phone" => "",	// +CCC.ZZZZZZZZZZZZxYYYY max 20 including the +
			"fax" => "",
			);
		$this->registrationArray = array(
			"action" => "create",
			"contact" => array(
				"admin" => $gc,
				"billing" => $gc,
				"org" => $gc,
				"tech" => $gc,
				),
			"domain" => "",
			"year" => "",
			"lang_pref" => 0,		//Required for dot.us (probably really for .ca, documentation error)
			"legal_type" => "",		//required for .ca same list as eNom
			);

		parent::Registrar($input);
	}


// ------------------------------------------------------------------------------------------------------------- CONNECT METHODS
// ------------------------------------------------------------------------------------------------------------- CONNECT METHODS

	/**
	 * This function merely sets the Eurodns connection URL based on TEST mode status.
	 */
	function getURL() {
		global $CONF;

		//See if this is a DNZ run
		//This is guessing at what DNZ needs, the details need to be worked out and adjusted
		$this->domainExternalID = NULL;

		if($this->partnerid && $this->signkey) {
			//Not sure how this is going to get set, but...
			//$this->domainExternalID = "<subuser:ID>".$this->input["domainExternalID"]."</subuser:ID>";
			$this->domainExternalID = $this->input["domainExternalID"];

			$this->UID = $this->partnerid;
			$this->PW = $this->signkey;
		}

	        $testMode = $CONF["domain"]["registrar"][strtolower($this->stringRep)][$this->getConfigID()]["moduleTestMode"];
		$url = "https://secure.api-eurodns.com:20015/v2/";
		$this->cert = "api_certificate_prod.crt";

		if ($testMode) {
			$url = "https://secure.tryout-eurodns.com:20015/v2/";
			$this->cert = "api_certificate_test.crt";
		}
		return $url;
	}

	/**
	 * This function performs a query given params
	 */
	function _query( $xml ) {
		// get the hostname
		$host = $this->getURL();

		$c = $this->getCurlObject();

		$c->constantsSetting["CURLOPT_SSL_VERIFYPEER"] = 0;
		$c->constantsSetting["CURLOPT_SSL_VERIFYHOST"] = 1;

		$c->setUsernameAndPassword($this->UID, "MD5".md5($this->PW));

		if($this->domainExternalID) {
			$c->addField("subUserID",$this->domainExternalID,1);
		}

		$c->setURL($host);


		//has to be double url encoded
		$c->addField("xml", urlencode($xml), 1);

		// setup the SSL Cert up
		$c->setSSLCertPath(dirname(__file__)."/core/eurodns");
		$c->caInfo = $this->cert;
		$tlds = $this->capabilities[REGISTRAR_TLDS];


		$rawResult = $c->toString();

		$x = new MBXML($rawResult);
		$result = $x->toArray();

		$msg = $return = array();

		if (!is_array($result) || !is_array($result['response'])) {
			$this->addError(TRANS_MBAPIERROR, "Invalid or Empty response from Registrar.");
		} else {
			$responseCode = (int)$result["response"][0]["result"][0]["_attributes"]["code"];
			$msg = array(
				'responseCode' => $responseCode,
				'responseMsg' => $result["response"][0]["result"][0]['msg'][0]
				);

			if (isset($msg['respomseMsg']['responseText']))
				$msg['responseMsg'] = $msg['respomseMsg']['responseText'];

			if ($responseCode < 1000 || $responseCode > 1999) {
				// an error occured
				$msg['isError'] = 1;

			} elseif (isset($result['response'][0]['resData'])) {
				$return = $result['response'][0]['resData'][0];
			}

			if (is_array($result['response'][0]['extension'])) {
				$return['extension'] = $result['response'][0]['extension'][0];
			}
		}


		$return = array_merge($msg, $return);
		return $return;
	}

	/**
	 * Helper function to exit quickly from module
	 */
	function &_exit() {
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

// ------------------------------------------------------------------------------------------------------------- HELPER METHODS
// ------------------------------------------------------------------------------------------------------------- HELPER METHODS
	/**
	 * Put the contact info in place
	 *
	 */
	function getRegistrationContactsData() {

		//Check the module setttings and place the proper contact info in place
		if($this->input["registrantZip"] && ! $this->input["registrantPostalCode"]) {
			$this->input["registrantPostalCode"] = $this->input["registrantZip"];
		}
		if($this->input["registrantPostalCode"] && ! $this->input["registrantZip"]) {
			$this->input["registrantZip"] = $this->input["registrantPostalCode"];
		}
		if($this->input["registrantStateProvince"] && ! $this->input["registrantState"]) {
			$this->input["registrantState"] = $this->input["registrantStateProvince"];
		}
		if($this->input["registrantState"] && ! $this->input["registrantStateProvince"]) {
			$this->input["registrantStateProvince"] = $this->input["registrantState"];
		}

		//Set the owner info
		$this->registrationArray["contact_set"]["org"]["first_name"] = $this->input["registrantFirstName"];
		$this->registrationArray["contact_set"]["org"]["last_name"] = $this->input["registrantLastName"];
		$this->registrationArray["contact_set"]["org"]["org_name"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];
		$this->registrationArray["contact_set"]["org"]["address1"] = $this->input["registrantAddress1"];
		$this->registrationArray["contact_set"]["org"]["address2"] = $this->input["registrantAddress2"];
		$this->registrationArray["contact_set"]["org"]["address3"] = "";
		$this->registrationArray["contact_set"]["org"]["city"] = $this->input["registrantCity"];
		$this->registrationArray["contact_set"]["org"]["state"] = $this->input["registrantStateProvince"];
		$this->registrationArray["contact_set"]["org"]["country_code"] = $this->input["registrantCountry"];
		$this->registrationArray["contact_set"]["org"]["postal_code"] = $this->input["registrantZip"];
		$this->registrationArray["contact_set"]["org"]["email"] = $this->input["registrantEmail"];
		$this->registrationArray["contact_set"]["org"]["phone"] = $this->input["registrantPhone"];
		$this->registrationArray["contact_set"]["org"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		$this->registrationArray["contact_set"]["org"]["url"] = "";
		$this->registrationArray["contact_set"]["org"]["lang_pref"] = $this->input["domain_idn_language"];

		//If the module has contact fields set, override the client info
		if($this->input["config"]["adminFirstname"] || $this->input["config"]["adminLastName"] || $this->input["config"]["adminCompany"]) {
			$this->registrationArray["contact_set"]["admin"]["first_name"] = $this->input["config"]["adminFirstname"];
			$this->registrationArray["contact_set"]["admin"]["last_name"] = $this->input["config"]["adminLastName"];
			$this->registrationArray["contact_set"]["admin"]["org_name"] = $this->input["config"]["adminCompany"];
			$this->registrationArray["contact_set"]["admin"]["address1"] = $this->input["config"]["adminAddress1"];
			$this->registrationArray["contact_set"]["admin"]["address2"] = $this->input["config"]["adminAddress2"];
			$this->registrationArray["contact_set"]["admin"]["address3"] = "";
			$this->registrationArray["contact_set"]["admin"]["city"] = $this->input["config"]["adminCity"];
			$this->registrationArray["contact_set"]["admin"]["state"] = $this->input["config"]["adminState"];
			$this->registrationArray["contact_set"]["admin"]["country_code"] = $this->input["config"]["adminCountry"];
			$this->registrationArray["contact_set"]["admin"]["postal_code"] = $this->input["config"]["adminZip"];
			$this->registrationArray["contact_set"]["admin"]["email"] = $this->input["config"]["adminEmail"];
			$this->registrationArray["contact_set"]["admin"]["phone"] = $this->input["config"]["adminPhone"];
			$this->registrationArray["contact_set"]["admin"]["fax"] = (strlen($this->input["config"]["adminFax"])>4)? $this->input["registrantFax"] : "";
			$this->registrationArray["contact_set"]["admin"]["url"] = "";
			//$this->registrationArray["contact_set"]["admin"]["lang_pref"] = $this->input["config"]["domain_idn_language"];
		} else {
			$this->registrationArray["contact_set"]["admin"]["first_name"] = $this->input["registrantFirstName"];
			$this->registrationArray["contact_set"]["admin"]["last_name"] = $this->input["registrantLastName"];
			$this->registrationArray["contact_set"]["admin"]["org_name"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];
			$this->registrationArray["contact_set"]["admin"]["address1"] = $this->input["registrantAddress1"];
			$this->registrationArray["contact_set"]["admin"]["address2"] = $this->input["registrantAddress2"];
			$this->registrationArray["contact_set"]["admin"]["address3"] = "";
			$this->registrationArray["contact_set"]["admin"]["city"] = $this->input["registrantCity"];
			$this->registrationArray["contact_set"]["admin"]["state"] = $this->input["registrantState"];
			$this->registrationArray["contact_set"]["admin"]["country_code"] = $this->input["registrantCountry"];
			$this->registrationArray["contact_set"]["admin"]["postal_code"] = $this->input["registrantZip"];
			$this->registrationArray["contact_set"]["admin"]["email"] = $this->input["registrantEmail"];
			$this->registrationArray["contact_set"]["admin"]["phone"] = $this->input["registrantPhone"];
			$this->registrationArray["contact_set"]["admin"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		}
		if($this->input["config"]["auxBillingFirstname"] || $this->input["config"]["auxBillingLastName"] || $this->input["config"]["auxBillingCompany"]) {
			$this->registrationArray["contact_set"]["billing"]["first_name"] = $this->input["config"]["auxBillingFirstname"];
			$this->registrationArray["contact_set"]["billing"]["last_name"] = $this->input["config"]["auxBillingLastName"];
			$this->registrationArray["contact_set"]["billing"]["org_name"] = $this->input["config"]["auxBillingCompany"];
			$this->registrationArray["contact_set"]["billing"]["address1"] = $this->input["config"]["auxBillingAddress1"];
			$this->registrationArray["contact_set"]["billing"]["address2"] = $this->input["config"]["auxBillingAddress2"];
			$this->registrationArray["contact_set"]["billing"]["address3"] = "";
			$this->registrationArray["contact_set"]["billing"]["city"] = $this->input["config"]["auxBillingCity"];
			$this->registrationArray["contact_set"]["billing"]["state"] = $this->input["config"]["auxBillingState"];
			$this->registrationArray["contact_set"]["billing"]["country_code"] = $this->input["config"]["auxBillingCountry"];
			$this->registrationArray["contact_set"]["billing"]["postal_code"] = $this->input["config"]["auxBillingZip"];
			$this->registrationArray["contact_set"]["billing"]["email"] = $this->input["config"]["auxBillingEmail"];
			$this->registrationArray["contact_set"]["billing"]["phone"] = $this->input["config"]["auxBillingPhone"];
			$this->registrationArray["contact_set"]["billing"]["fax"] = (strlen($this->input["config"]["auxBillingFax"])>4)? $this->input["registrantFax"] : "";
		} else {
			$this->registrationArray["contact_set"]["billing"]["first_name"] = $this->input["registrantFirstName"];
			$this->registrationArray["contact_set"]["billing"]["last_name"] = $this->input["registrantLastName"];
			$this->registrationArray["contact_set"]["billing"]["org_name"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];;
			$this->registrationArray["contact_set"]["billing"]["address1"] = $this->input["registrantAddress1"];
			$this->registrationArray["contact_set"]["billing"]["address2"] = $this->input["registrantAddress2"];
			$this->registrationArray["contact_set"]["billing"]["address3"] = "";
			$this->registrationArray["contact_set"]["billing"]["city"] = $this->input["registrantCity"];
			$this->registrationArray["contact_set"]["billing"]["state"] = $this->input["registrantState"];
			$this->registrationArray["contact_set"]["billing"]["country_code"] = $this->input["registrantCountry"];
			$this->registrationArray["contact_set"]["billing"]["postal_code"] = $this->input["registrantZip"];
			$this->registrationArray["contact_set"]["billing"]["email"] = $this->input["registrantEmail"];
			$this->registrationArray["contact_set"]["billing"]["phone"] = $this->input["registrantPhone"];
			$this->registrationArray["contact_set"]["billing"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		}
		if($this->input["config"]["techFirstname"] || $this->input["config"]["techLastName"] || $this->input["config"]["techCompany"]) {
			$this->registrationArray["contact_set"]["tech"]["first_name"] = $this->input["config"]["techFirstname"];
			$this->registrationArray["contact_set"]["tech"]["last_name"] = $this->input["config"]["techLastName"];
			$this->registrationArray["contact_set"]["tech"]["org_name"] = $this->input["config"]["techCompany"];
			$this->registrationArray["contact_set"]["tech"]["address1"] = $this->input["config"]["techAddress1"];
			$this->registrationArray["contact_set"]["tech"]["address2"] = $this->input["config"]["techAddress2"];
			$this->registrationArray["contact_set"]["tech"]["address3"] = "";
			$this->registrationArray["contact_set"]["tech"]["city"] = $this->input["config"]["techCity"];
			$this->registrationArray["contact_set"]["tech"]["state"] = $this->input["config"]["techState"];
			$this->registrationArray["contact_set"]["tech"]["country_code"] = $this->input["config"]["techCountry"];
			$this->registrationArray["contact_set"]["tech"]["postal_code"] = $this->input["config"]["techZip"];
			$this->registrationArray["contact_set"]["tech"]["email"] = $this->input["config"]["techEmail"];
			$this->registrationArray["contact_set"]["tech"]["phone"] = $this->input["config"]["techPhone"];
			$this->registrationArray["contact_set"]["tech"]["fax"] = (strlen($this->input["config"]["techFax"])>4)? $this->input["registrantFax"] : "";
		} else {
			$this->registrationArray["contact_set"]["tech"]["first_name"] = $this->input["registrantFirstName"];
			$this->registrationArray["contact_set"]["tech"]["last_name"] = $this->input["registrantLastName"];
			$this->registrationArray["contact_set"]["tech"]["org_name"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];
			$this->registrationArray["contact_set"]["tech"]["address1"] = $this->input["registrantAddress1"];
			$this->registrationArray["contact_set"]["tech"]["address2"] = $this->input["registrantAddress2"];
			$this->registrationArray["contact_set"]["tech"]["address3"] = "";
			$this->registrationArray["contact_set"]["tech"]["city"] = $this->input["registrantCity"];
			$this->registrationArray["contact_set"]["tech"]["state"] = $this->input["registrantState"];
			$this->registrationArray["contact_set"]["tech"]["country_code"] = $this->input["registrantCountry"];
			$this->registrationArray["contact_set"]["tech"]["postal_code"] = $this->input["registrantZip"];
			$this->registrationArray["contact_set"]["tech"]["email"] = $this->input["registrantEmail"];
			$this->registrationArray["contact_set"]["tech"]["phone"] = $this->input["registrantPhone"];
			$this->registrationArray["contact_set"]["tech"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		}
//show($this->registrationArray["contact_set"]);
//exit;
	}


	/**
	* create XML from an array with attributes specific to resellone
	*
	* @author  Kris Bailey
	* @author  Andy Christ
	* @param  array  array to convert to XML
	* @param  string  starting indentation whitespace. Defaults to emtpy string.
	* @return  string  XML
	*/
	function toXML($array, $nameserverXML) {

		$output = "";

		$output .= "<domain:".$array["action"].">\n";
		$output .= "\t<domain:name>".$array["domain"]."</domain:name>\n";
		$output .= "\t<domain:year>".$array["year"]."</domain:year>\n";
		$output .= "</domain:".$array["action"].">\n";

		$output .= $nameserverXML."\n";

		foreach($array["contact_set"] as $type => $value) {
			$output .= "<contact:".$array["action"].">\n";
			$output .= "\t<contact:type>".$type."</contact:type>\n";
			$output .= "\t<contact:usetrustee>".$value["usetrustee"]."</contact:usetrustee>\n";
			$output .= "\t<contact:firstname>".$value["first_name"]."</contact:firstname>\n";
			$output .= "\t<contact:lastname>".$value["last_name"]."</contact:lastname>\n";
			$output .= "\t<contact:company>".$value["org_name"]."</contact:company>\n";
			$output .= "\t<contact:address1>".$value["address1"]."</contact:address1>\n";
			$output .= "\t<contact:address2>".$value["address2"]."</contact:address2>\n";
			$output .= "\t<contact:address3>".$value["address3"]."</contact:address3>\n";
			$output .= "\t<contact:city>".$value["city"]."</contact:city>\n";
			$output .= "\t<contact:zipcode>".$value["postal_code"]."</contact:zipcode>\n";
			$output .= "\t<contact:country_code>".$value["country_code"]."</contact:country_code>\n";
			$output .= "\t<contact:email>".$value["email"]."</contact:email>\n";
			$output .= "\t<contact:phone>".$value["phone"]."</contact:phone>\n";
			$output .= "\t<contact:fax>".$value["fax"]."</contact:fax>\n";
			$output .= "</contact:".$array["action"].">\n";

		}

		return $output;
	}


// ------------------------------------------------------------------------------------------------------------- MODULE METHODS
// ------------------------------------------------------------------------------------------------------------- MODULE METHODS

	/**
	 * Call the registrar to register a new domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function registerDomain() {
		if(!in_array($this->input["domainTLD"], (array)$this->capabilities[REGISTRAR_TLDS])) {
			$this->addError(TRANS_MBAPIERROR, "TLD not supported");
			return $this->_exit();
		}

		//Popluate contact fields
		$this->getRegistrationContactsData();

		//What nameservers do we need to pass
		if($this->input["domainNameserver1"]) {
			$dnsServers[] = $this->input["domainNameserver1"];
			$dnsServers[] = $this->input["domainNameserver2"];
			$dnsServers[] = $this->input["domainNameserver3"];
			$dnsServers[] = $this->input["domainNameserver4"];
		}

		$this->registrationArray["domain"] = $this->input["domainSLD"].".".$this->input["domainTLD"];
		$this->registrationArray["year"] = $this->input["domainYears"];

		//Do the TLD specific hoop jumping
		//FR: postalcode, registrant_dob, city, country, siren
		//VN: fullcompanyname, provincename
		//SE: sereg_type, sereg_value
		$extraInfo = NULL;
		if($this->input["domainTLD"] == "fr") {
			$extraInfo = "<extension:create>\n";
			if($this->input["postalcode"]) {
				$extraInfo .= "\t<extension:postalcode>".$this->input["postalcode"]."</extension:postalcode>\n";
			}
			if($this->input["registrant_dob"]) {
				$extraInfo .= "\t<extension:date>".$this->input["registrant_dob"]."</extension:date>\n";
			}
			if($this->input["city"]) {
				$extraInfo .= "\t<extension:city>".$this->input["city"]."</extension:city>\n";
			}
			if($this->input["country"]) {
				$extraInfo .= "\t<extension:country>".$this->input["country"]."</extension:country>\n";
			}
			if($this->input["siren"]) {
				$extraInfo .= "\t<extension:siren>".$this->input["siren"]."</extension:siren>\n";
			}
			$extraInfo .= "</extension:create>\n";
		}
		if($this->input["domainTLD"] == "vn") {
			$extraInfo = "<extension:create>\n";
			if($this->input["fullcompanyname"]) {
				$extraInfo .= "\t<extension:fullcompanyname>".$this->input["fullcompanyname"]."</extension:fullcompanyname>\n";
			}
			if($this->input["provincename"]) {
				$extraInfo .= "\t<extension:provincename>".$this->input["provincename"]."</extension:provincename>\n";
			}
			$extraInfo .= "</extension:create>\n";
		}
		if($this->input["domainTLD"] == "se") {
			$extraInfo = "<extension:create>\n";
			if($this->input["sereg_type"]) {
				$extraInfo .= "\t<extension:sereg_type>".$this->input["sereg_type"]."</extension:sereg_type>\n";
			}
			if($this->input["sereg_value"]) {
				$extraInfo .= "\t<extension:sereg_value>".$this->input["sereg_value"]."</extension:sereg_value>\n";
			}
			$extraInfo .= "</extension:create>\n";
		}

		//Nameserver needs special XML
		$nameserverXML = NULL;
		foreach($dnsServers as $key => $value) {
			if($value) {
				$nameserverXML .= "<nameserver:create>\n";
				$ip = gethostbyname($this->input["domainNameserver".($key+1)].".");
				$nameserverXML .= "\t<nameserver:priority>$key</nameserver:priority>\n";
				$nameserverXML .= "\t<nameserver:fqdn>$value</nameserver:fqdn>\n";
				$nameserverXML .= "\t<nameserver:ipaddr>$ip</nameserver:ipaddr>\n";
				$nameserverXML .= "</nameserver:create>\n";
			}
		}

		$registrationXML = $this->toXML($this->registrationArray, $nameserverXML);

		$registrationXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n
					<request\n
					\txmlns:domain=\"http://www.eurodns.com/\"\n
					\txmlns:nameserver=\"http://www.eurodns.com/\"\n
					\txmlns:contact=\"http://www.eurodns.com\"\n
					\txmlns:extension=\"http://www.eurodns.com\">\n
					$registrationXML\n
					$extraInfo
					</request>\n";

		$result = $this->_query($registrationXML);

		if ($result['isError']) {
			$this->addError(TRANS_MBAPIERROR, "EuroDNS Error: $result[responseMsg]");
			return $this->_exit();
		}

        $this->commandStatus = "1001" == $result['responseCode']
					? self::ACTION_STATUS_COMPLETED
					: self::ACTION_STATUS_WAITING;

		$this->arrayData = array("OrderCode" => $result['responseCode'], "domainOrderMessage" => $result['responseMsg']);

		return $this->_exit();
	}

	/**
	 * Call the registrar to transfer an existing domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function transferDomain() {
		if(!in_array($this->input["domainTLD"], (array)$this->capabilities[REGISTRAR_TLDS])) {
			$this->addError(TRANS_MBAPIERROR, "TLD not supported");
			return $this->_exit();
		}

		//Popluate contact fields
		$this->getRegistrationContactsData();

		$this->registrationArray["action"] = "transfer";

		//What nameservers do we need to pass
		if($this->input["domainNameserver1"]) {
			$dnsServers[] = $this->input["domainNameserver1"];
			$dnsServers[] = $this->input["domainNameserver2"];
			$dnsServers[] = $this->input["domainNameserver3"];
			$dnsServers[] = $this->input["domainNameserver4"];
		}

		$this->registrationArray["domain"] = $this->input["domainSLD"].".".$this->input["domainTLD"];
		$this->registrationArray["year"] = $this->input["domainYears"];

		//Do the TLD specific hoop jumping

		//Nameserver needs special XML
		$nameserverXML = NULL;
		foreach($dnsServers as $key => $value) {
			if($value) {
				$nameserverXML .= "<nameserver:create>\n";
				$ip = gethostbyname($this->input["domainNameserver".($key+1)].".");
				$nameserverXML .= "\t<nameserver:priority>$key</nameserver:priority>\n";
				$nameserverXML .= "\t<nameserver:fqdn>$value</nameserver:fqdn>\n";
				$nameserverXML .= "\t<nameserver:ipaddr>$ip</nameserver:ipaddr>\n";
				$nameserverXML .= "</nameserver:create>\n";
			}
		}

		$registrationXML = $this->toXML($this->registrationArray, $nameserverXML);

		$registrationXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n
					<request\n
					\txmlns:domain=\"http://www.eurodns.com/\"\n
					\txmlns:nameserver=\"http://www.eurodns.com/\"\n
					\txmlns:contact=\"http://www.eurodns.com\"\n
					\txmlns:extension=\"http://www.eurodns.com\">\n
					$registrationXML\n
					</request>\n";
		$result = $this->_query($registrationXML);

		if ($result['isError']) {
			$this->addError(TRANS_MBAPIERROR, "EuroDNS Error: $result[responseMsg]");
			return $this->_exit();
		}

		$this->hasExecutedSuccessfully = 1;
		$this->arrayData = array("OrderID" => "xyz", "domainRemotePurchaseID" => $result['responseMsg']);

		return $this->_exit();
	}

	/**
	 * This method contains the registrar specific code for renewing a domain.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function renewDomain() {
		if(!in_array($this->input["domainTLD"], (array)$this->capabilities[REGISTRAR_TLDS])) {
			$this->addError(TRANS_MBAPIERROR, "TLD not supported");
			return $this->_exit();
		}

		$this->registrationArray["domain"] = $this->input["domainSLD"].".".$this->input["domainTLD"];
		$this->registrationArray["year"] = $this->input["domainYears"];

		$registrationXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n
					<request xmlns:domain=\"http://www.eurodns.com/\">\n
						<domain:renew>
							<domain:name>".$this->registrationArray["domain"]."</domain:name>
							<domain:year>".$this->registrationArray["year"]."</domain:year>
						</domain:renew>
					</request>\n";
		$result = $this->_query($registrationXML);

		if ($result['isError']) {
			$this->addError(TRANS_MBAPIERROR, "EuroDNS Error: $result[responseMsg]");
			return $this->_exit();
		}

		$this->hasExecutedSuccessfully = 1;
		$this->arrayData = array("OrderID" => "xyz", "domainRemotePurchaseID" => $result['responseMsg']);
		return $this->_exit();
	}

	/**
	 * Call the registrar to fetch a domain's contact data
	 *
	 * @return  string  XML containing contact data for this domain
	 */
	function getDomainContactData() {

	 	global $CONF;

		$this->registrationArray["domain"] = $this->input["domainSLD"].".".$this->input["domainTLD"];
		$this->registrationArray["year"] = $this->input["domainYears"];

		$infoXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n
					<request xmlns:domain=\"http://www.eurodns.com/\">\n
						<domain:info>
							<domain:name>".$this->registrationArray["domain"]."</domain:name>
						</domain:info>
					</request>
";
		$result = $this->_query($infoXML);

		if ($result['isError']) {
			$this->addError(TRANS_MBAPIERROR, "Error occured [Code: $result[responseCode] Message: $result[responseMsg]");
			return $this->_exit();
		}

		//Grab the contact ID and call back for the info
		$contactIDs = array();
		foreach((array)$result['domain:contact'] as $value) {
			$contactIDs[$value["_attributes"]["type"]] = $value[0];
		}

		foreach($contactIDs as $type => $value) {

			$infoXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n
						<request xmlns:domain=\"http://www.eurodns.com/\">\n
							<contact:info>
								<contact:ID>".$value."</contact:ID>
							</contact:info>
						</request>
						";

			$result = $this->_query($infoXML);

			if ($result['isError']) {
				$this->addError(TRANS_MBAPIERROR, "EuroDNS Error: $result[responseMsg]");
				return $this->_exit();
			}
			$response = $result["contact:postalInfo"][0];

			//I couldn't make this up if I wanted to.  EuroDNS has a firstname and lastname field for
			// a registration call.  But when you pull the conatact info later, it returns:
			// <contact:name>Christ Andy</contact:name>
			// Backwards and in one field!  Wow!  A new low in API functionality... AC

			//Split the name field
			$nameFields = NULL;
			$nameFields = explode(" ",$response["contact:name"][0]);
			$firstNameIndex = (count($nameFields) - 1);

			$this->registrationArray["contact"][$type]["firstname"] = $nameFields[$firstNameIndex];
			unset($nameFields[$firstNameIndex]);

			foreach($nameFields as $value) {
				$this->registrationArray["contact"][$type]["lastname"] .= $value." ";
			}

			$this->registrationArray["contact"][$type]["company"] = $response["contact:org"][0];
			$this->registrationArray["contact"][$type]["address1"] = $response["contact:addr"][0]["contact:street"][0];
			$this->registrationArray["contact"][$type]["address2"] = NULL;
			$this->registrationArray["contact"][$type]["address3"] = NULL;
			$this->registrationArray["contact"][$type]["city"] = $response["contact:addr"][0]["contact:city"][0];
			$this->registrationArray["contact"][$type]["country_code"] = $response["contact:addr"][0]["contact:cc"][0];
			$this->registrationArray["contact"][$type]["zipcode"] = $response["contact:addr"][0]["contact:pc"][0];
			$this->registrationArray["contact"][$type]["city"] = $response["contact:addr"][0]["contact:city"][0];
			$this->registrationArray["contact"][$type]["email"] = $response["contact:email"][0];
			$this->registrationArray["contact"][$type]["phone"] = $response["contact:voice"][0];
			$this->registrationArray["contact"][$type]["fax"] = $response["contact:fax"][0];
		} // foreach $contactIDs

		//Format the data for MBAPI
		$this->arrayData["DomainData"] = NULL;
		$this->arrayData["DomainData"]["domainname"] = $this->registrationArray["domain"];

/*
		$this->registrationArray["contact"][$type]["firstname"] = $response["contact:name"][0];
		$this->registrationArray["contact"][$type]["lastname"] = NULL;
		$this->registrationArray["contact"][$type]["company"] = $response["contact:org"][0];
		$this->registrationArray["contact"][$type]["address1"] = $response["contact:addr"][0]["contact:street"][0];
		$this->registrationArray["contact"][$type]["address2"] = NULL;
		$this->registrationArray["contact"][$type]["address3"] = NULL;
		$this->registrationArray["contact"][$type]["city"] = $response["contact:addr"][0]["contact:city"][0];
		$this->registrationArray["contact"][$type]["country_code"] = $response["contact:addr"][0]["contact:cc"][0];
		$this->registrationArray["contact"][$type]["zipcode"] = $response["contact:addr"][0]["contact:pc"][0];
		$this->registrationArray["contact"][$type]["email"] = $response["contact:email"][0];
		$this->registrationArray["contact"][$type]["phone"] = $response["contact:voice"][0];
		$this->registrationArray["contact"][$type]["fax"] = $response["contact:fax"][0];
*/
		$this->arrayData["DomainData"]["domainname"] = $this->registrationArray["domain"];
		$this->arrayData["RegistrantContactData"] = array(
			"RegistrantAddress1" => $this->registrationArray["contact"]["org"]["address1"],
			"RegistrantAddress2" => $this->registrationArray["contact"]["org"]["address2"],
			"RegistrantCity" => $this->registrationArray["contact"]["org"]["city"],
			"RegistrantCountry" => $this->registrationArray["contact"]["org"]["country_code"],
			"RegistrantEmailAddress" => $this->registrationArray["contact"]["org"]["email"],
			"RegistrantFax" => $this->registrationArray["contact"]["org"]["fax"],
			"RegistrantFirstName" => $this->registrationArray["contact"]["org"]["firstname"],
			"RegistrantLastName" => $this->registrationArray["contact"]["org"]["lastname"],
			"RegistrantOrganizationName" => $this->registrationArray["contact"]["org"]["company"],
			"RegistrantPhone" => $this->registrationArray["contact"]["org"]["phone"],
			"RegistrantPostalCode" => $this->registrationArray["contact"]["org"]["zipcode"],
			"RegistrantStateProvince" => NULL,
			"RegistrantFullCountry" => $this->registrationArray["contact"]["org"]["country_code"],
			);
		$this->arrayData["AuxBillingContactData"] = array(
			"AuxBillingAddress1" => $this->registrationArray["contact"]["billing"]["address1"],
			"AuxBillingAddress2" => $this->registrationArray["contact"]["billing"]["address2"],
			"AuxBillingCity" => $this->registrationArray["contact"]["billing"]["city"],
			"AuxBillingCountry" => $this->registrationArray["contact"]["billing"]["country_code"],
			"AuxBillingEmailAddress" => $this->registrationArray["contact"]["billing"]["email"],
			"AuxBillingFax" => $this->registrationArray["contact"]["billing"]["fax"],
			"AuxBillingFirstName" => $this->registrationArray["contact"]["billing"]["firstname"],
			"AuxBillingLastName" => $this->registrationArray["contact"]["billing"]["lastname"],
			"AuxBillingOrganizationName" => $this->registrationArray["contact"]["billing"]["company"],
			"AuxBillingPhone" => $this->registrationArray["contact"]["billing"]["phone"],
			"AuxBillingPostalCode" => $this->registrationArray["contact"]["billing"]["zipcode"],
			"AuxBillingStateProvince" => NULL,
			"AuxBillingFullCountry" => $this->registrationArray["contact"]["billing"]["country_code"],
			);
		$this->arrayData["TechContactData"] = array(
			"TechAddress1" => $this->registrationArray["contact"]["tech"]["address1"],
			"TechAddress2" => $this->registrationArray["contact"]["tech"]["address2"],
			"TechCity" => $this->registrationArray["contact"]["tech"]["city"],
			"TechCountry" => $this->registrationArray["contact"]["tech"]["country_code"],
			"TechEmailAddress" => $this->registrationArray["contact"]["tech"]["email"],
			"TechFax" => $this->registrationArray["contact"]["tech"]["fax"],
			"TechFirstName" => $this->registrationArray["contact"]["tech"]["firstname"],
			"TechLastName" => $this->registrationArray["contact"]["tech"]["lastname"],
			"TechOrganizationName" => $this->registrationArray["contact"]["tech"]["company"],
			"TechPhone" => $this->registrationArray["contact"]["tech"]["phone"],
			"TechPostalCode" => $this->registrationArray["contact"]["tech"]["zipcode"],
			"TechStateProvince" => NULL,
			"TechFullCountry" => $this->registrationArray["contact"]["tech"]["country_code"],
			);
		$this->arrayData["AdminContactData"] = array(
			"AdminAddress1" => $this->registrationArray["contact"]["admin"]["address1"],
			"AdminAddress2" => $this->registrationArray["contact"]["admin"]["address2"],
			"AdminCity" => $this->registrationArray["contact"]["admin"]["city"],
			"AdminCountry" => $this->registrationArray["contact"]["admin"]["country_code"],
			"AdminEmailAddress" => $this->registrationArray["contact"]["admin"]["email"],
			"AdminFax" => $this->registrationArray["contact"]["admin"]["fax"],
			"AdminFirstName" => $this->registrationArray["contact"]["admin"]["firstname"],
			"AdminLastName" => $this->registrationArray["contact"]["admin"]["lastname"],
			"AdminOrganizationName" => $this->registrationArray["contact"]["admin"]["company"],
			"AdminPhone" => $this->registrationArray["contact"]["admin"]["phone"],
			"AdminPostalCode" => $this->registrationArray["contact"]["admin"]["zipcode"],
			"AdminStateProvince" => NULL,
			"AdminFullCountry" => $this->registrationArray["contact"]["admin"]["country_code"],
			);
		$this->arrayData["BillingContactData"] = array(
			"BillingAddress1" => $this->registrationArray["contact"]["billing"]["address1"],
			"BillingAddress2" => $this->registrationArray["contact"]["billing"]["address2"],
			"BillingCity" => $this->registrationArray["contact"]["billing"]["city"],
			"BillingCountry" => $this->registrationArray["contact"]["billing"]["country_code"],
			"BillingEmailAddress" => $this->registrationArray["contact"]["billing"]["email"],
			"BillingFax" => $this->registrationArray["contact"]["billing"]["fax"],
			"BillingFirstName" => $this->registrationArray["contact"]["billing"]["firstname"],
			"BillingLastName" => $this->registrationArray["contact"]["billing"]["lastname"],
			"BillingOrganizationName" => $this->registrationArray["contact"]["billing"]["company"],
			"BillingPhone" => $this->registrationArray["contact"]["billing"]["phone"],
			"BillingPostalCode" => $this->registrationArray["contact"]["billing"]["zipcode"],
			"BillingStateProvince" => NULL,
			"BillingFullCountry" => $this->registrationArray["contact"]["billing"]["country_code"],
			);
		return $this->_exit();
	} // END getDomainContactData()

	/**
	 * Send the updated domain contact data to the registrar
	 *
	 * @return  string  XML result
	 */
	function setDomainContactData() {
		//Popluate contact fields
		//Don't do this because it might override some of the data
		//$this->getRegistrationContactsData();

		$this->registrationArray["domain"] = $this->input["domainSLD"].".".$this->input["domainTLD"];
		$this->registrationArray["year"] = $this->input["domainYears"];

		$contactData["admin"]["firstname"] = $this->input["domainContactData"]["admin"]["adminFirstName"];
		$contactData["admin"]["lastname"] = $this->input["domainContactData"]["admin"]["adminLastName"];
		$contactData["admin"]["company"] = $this->input["domainContactData"]["admin"]["adminOrganizationName"];
		$contactData["admin"]["address1"] = $this->input["domainContactData"]["admin"]["adminAddress1"];
		$contactData["admin"]["address2"] = $this->input["domainContactData"]["admin"]["adminAddress2"];
		$contactData["admin"]["city"] = $this->input["domainContactData"]["admin"]["adminCity"];
		$contactData["admin"]["zipcode"] = $this->input["domainContactData"]["admin"]["adminPostalCode"];
		$contactData["admin"]["country_code"] = $this->input["domainContactData"]["admin"]["adminCountry"];
		$contactData["admin"]["email"] = $this->input["domainContactData"]["admin"]["adminEmailAddress"]; $contactData["admin"]["phone"] = $this->input["domainContactData"]["admin"]["adminPhone"];
		$contactData["admin"]["fax"] = $this->input["domainContactData"]["admin"]["adminFax"];

		$contactData["org"]["firstname"] = $this->input["domainContactData"]["registrant"]["registrantFirstName"];
		$contactData["org"]["lastname"] = $this->input["domainContactData"]["admin"]["adminLastName"];
		$contactData["org"]["company"] = $this->input["domainContactData"]["registrant"]["registrantOrganizationName"];
		$contactData["org"]["address1"] = $this->input["domainContactData"]["registrant"]["registrantAddress1"];
		$contactData["org"]["address2"] = $this->input["domainContactData"]["registrant"]["registrantAddress2"];
		$contactData["org"]["city"] = $this->input["domainContactData"]["registrant"]["registrantCity"];
		$contactData["org"]["zipcode"] = $this->input["domainContactData"]["registrant"]["registrantPostalCode"];
		$contactData["org"]["country_code"] = $this->input["domainContactData"]["registrant"]["registrantCountry"];
		$contactData["org"]["email"] = $this->input["domainContactData"]["registrant"]["registrantEmailAddress"]; $contactData["org"]["phone"] = $this->input["domainContactData"]["registrant"]["registrantPhone"];
		$contactData["org"]["fax"] = $this->input["domainContactData"]["registrant"]["registrantFax"];

		$contactData["billing"]["firstname"] = $this->input["domainContactData"]["auxBilling"]["auxBillingFirstName"];
		$contactData["billing"]["lastname"] = $this->input["domainContactData"]["admin"]["adminLastName"];
		$contactData["billing"]["company"] = $this->input["domainContactData"]["auxBilling"]["auxBillingOrganizationName"];
		$contactData["billing"]["address1"] = $this->input["domainContactData"]["auxBilling"]["auxBillingAddress1"];
		$contactData["billing"]["address2"] = $this->input["domainContactData"]["auxBilling"]["auxBillingAddress2"];
		$contactData["billing"]["city"] = $this->input["domainContactData"]["auxBilling"]["auxBillingCity"];
		$contactData["billing"]["zipcode"] = $this->input["domainContactData"]["auxBilling"]["auxBillingPostalCode"];
		$contactData["billing"]["country_code"] = $this->input["domainContactData"]["auxBilling"]["auxBillingCountry"];
		$contactData["billing"]["email"] = $this->input["domainContactData"]["auxBilling"]["auxBillingEmailAddress"]; $contactData["billing"]["phone"] = $this->input["domainContactData"]["auxBilling"]["auxBillingPhone"];
		$contactData["billing"]["fax"] = $this->input["domainContactData"]["auxBilling"]["auxBillingFax"];

		$contactData["tech"]["firstname"] = $this->input["domainContactData"]["tech"]["techFirstName"];
		$contactData["tech"]["lastname"] = $this->input["domainContactData"]["admin"]["adminLastName"];
		$contactData["tech"]["company"] = $this->input["domainContactData"]["tech"]["techOrganizationName"];
		$contactData["tech"]["address1"] = $this->input["domainContactData"]["tech"]["techAddress1"];
		$contactData["tech"]["address2"] = $this->input["domainContactData"]["tech"]["techAddress2"];
		$contactData["tech"]["city"] = $this->input["domainContactData"]["tech"]["techCity"];
		$contactData["tech"]["zipcode"] = $this->input["domainContactData"]["tech"]["techPostalCode"];
		$contactData["tech"]["country_code"] = $this->input["domainContactData"]["tech"]["techCountry"];
		$contactData["tech"]["email"] = $this->input["domainContactData"]["tech"]["techEmailAddress"]; $contactData["tech"]["phone"] = $this->input["domainContactData"]["tech"]["techPhone"];
		$contactData["tech"]["fax"] = $this->input["domainContactData"]["tech"]["techFax"];

		$updateXML = NULL;
		foreach($contactData as $key => $value) {
			$updateXML .= "<contact:update>";
			$updateXML .= "\t<contact:type>$key</contact:type>\n";
			foreach($value as $name => $item) {
				$updateXML .= "\t\t<contact:$name>$item</contact:$name>\n";
			}
			$updateXML .= "</contact:update>\n";
		}

		$registrationXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n
					<request xmlns:domain=\"http://www.eurodns.com/domain\"> xmlns:contact=\"http://www.eurodns.com/contact\"\n
					<domain:update>
						<domain:name>".$this->registrationArray["domain"]."</domain:name>
					</domain:update>
					$updateXML\n
					</request>\n";
		$result = $this->_query($registrationXML);

		if ($result['isError']) {
			$this->addError(TRANS_MBAPIERROR, "EuroDNS Error: $result[responseMsg]");
			return $this->_exit();
		}

		$this->hasExecutedSuccessfully = 1;
		$this->arrayData = array("UpdateCode" => $result['responseCode'], "domainUpdateMessage" => $result['responseMsg']);

		return $this->_exit();
	}


	/*
	 * Call the registrar to check an account's balance.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function checkAccountBalance() {
		$infoXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n
					<request xmlns:domain=\"http://www.eurodns.com/agent\">\n
						<agent:balance/>
					</request>\n";

		$result = $this->_query($infoXML);

		if ($result['isError']) {
			$this->addError(TRANS_MBAPIERROR, "EuroDNS Error: $result[responseMsg]");
			return $this->_exit();
		}

		$response = $result["agent:balance"][0];

		// Remove commas from the pre-formatted numbers &/or amounts
		$this->arrayData["AvailableBalance"] = str_replace(",","",$response[0]);
		$this->arrayData["Balance"] = str_replace(",","",$response[0]);
		$this->arrayData["DomainCount"] = "TRANS_NOT_SUPPORTED";
		$this->arrayData["currency"] = $response["_attributes"]["currency"];

		return $this->_exit();
	}

	/**
	 * This method contains the registrar specific code for setting a domain's auto-renewal options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setdomainRenewTypeOptions() {
		global $CONF;

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}


	/**
	 * This method contains the registrar specific code for fetching the domain expiration date.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainExpDate() {
		global $CONF;


		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for fetching a domain's extended information.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainExtendedInfo() {
		global $CONF;

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * Call the registrar to check TLD pricing cost (cogs).
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getTLDPricing() {
		global $CONF;


		$this->arrayData = $tld;

		$this->hasExecutedSuccessfully = 1;
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/*
	 * Call the registrar to check the status of a transfer
	 *
	 * @return  string status of the transfer
	 */
	function getTransferStatus() {
		global $CONF;


		$this->hasExecutedSuccessfully = 1;
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/*
	 * Call the registrar to get a list of domains
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomains() {
		$listXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n
					<request xmlns:domain=\"http://www.eurodns.com/domain\">\n
					<domain:list>\n
					</domain:list>\n
					</request>\n";

		$result = $this->_query($listXML);

		if ($result['isError']) {
			$this->addError(TRANS_MBAPIERROR, "EuroDNS Error: $result[responseMsg]");
			return $this->_exit();
		}

		$rawList = $result["domain:list"][0]["domain:name"];


		$domainList = array();
		if($rawList && count($rawList)>0) {
			foreach($rawList as $value) {
				$sld = $tld = NULL;

				list($sld, $tld[0], $tld[1]) = explode(".",$value);
				ksort($tld);

				if (notNull($tld[1])) {
					$tld = implode('.', $tld);
				} else {
					$tld = $tld[0];
				}

				$this->arrayData["domains"][] = array(
						"domainName" => $sld.".".$tld,
						"domainSLD" => $sld,
						"domainTLD" => $tld,
						"domainDateExpires" => "",
					);
			}
		}

		return $this->_exit();
	}

	/**
	 * This method contains the registrar specific code for fetching a domain's DNS information.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainDNS() {
		$this->registrationArray["domain"] = $this->input["domainSLD"].".".$this->input["domainTLD"];
		$this->registrationArray["year"] = $this->input["domainYears"];

		$infoXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n
					<request xmlns:domain=\"http://www.eurodns.com/\">\n
						<domain:info>
							<domain:name>".$this->registrationArray["domain"]."</domain:name>
						</domain:info>
					</request>\n";

		$result = $this->_query($infoXML);

		if ($result['isError']) {
			$this->addError(TRANS_MBAPIERROR, "EuroDNS Error: $result[responseMsg]");
			return $this->_exit();
		}

		$dnsServers = array();
		foreach((array)$result["extension"]["eurodns:domain"][0]["domain:ns"] as $value) {
			$dnsServers[] = $value[0];
		}

		$this->arrayData["listDNS"] = $dnsServers;
		$this->arrayData["UseDNS"] = 0;

		return $this->_exit();
	}

	/**
	 * Call EuroDNS and drop a new account under our master so that
	 * we can convert a non-api account to an api account
	 *
	 * @return  string xmlData of containting results of the remote parking connection
	 */
	function validateAccount() {
		//Andy: I'm assuming that the UID and the PW will be that of the user
		// and we will have our master account user and pass in the config file
		//Will make that switch up top.

		$registrationXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n
					<request xmlns:subuser=\"http://www.eurodns.com/subuser\">\n
						<subuser:add>
							<subuser:login>".$this->input["username"]."</subuser:login>
							<subuser:password>".$this->input["password"]."</subuser:password>
						</subuser:add>
					</request>\n";

		$result = $this->_query($registrationXML);

		if ($result['responseCode'] == 5401) {
			// user already associated, but we don't know about it, so lets find them, disassociate user and then run this function again.
			$xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
			$xml .= "<request xmlns:subuser=\"http://www.eurodns.com/subuser\">\n";
			$xml .= "\t<subuser:info>\n";
			$xml .= "\t\t<eurodns:login>".$this->input['username']."</eurodns:login>\n";
			$xml .= "\t</subuser:info>\n";
			$xml .= "</request>";

			$result = $this->_query($xml);

			if (isset($result['subuser:id'])) {
				$response = $result['subuser:id'][0];
				$responseCode = $result['responseCode'];
			} else {
				$this->addError(TRANS_MBAPIERROR, "Registrar Error: [$result[responseCode]] $result[responseMsg].");
			}
		} else {
			$response = $result["subuser:id"][0];
			$responseCode = $result['responseCode'];
		}

		if($responseCode == 1000) {
			$this->arrayData["accountIsValid"] = 1;
			$this->arrayData["domainExternalID"] = $response;
		}

		return $this->_exit();
	}

	/**
	 * This method contains the registrar specific code for setting a domain's DNS information.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setDomainDNS() {

		$this->registrationArray["domain"] = $this->input["domainSLD"].".".$this->input["domainTLD"];
		$this->registrationArray["year"] = $this->input["domainYears"];

		if($this->input["domainNameserver1"]) {
			$dnsServers[] = $this->input["domainNameserver1"];
			$dnsServers[] = $this->input["domainNameserver2"];
			$dnsServers[] = $this->input["domainNameserver3"];
			$dnsServers[] = $this->input["domainNameserver4"];
			$dnsServers[] = $this->input["domainNameserver5"];
			$dnsServers[] = $this->input["domainNameserver6"];
			$dnsServers[] = $this->input["domainNameserver7"];
			$dnsServers[] = $this->input["domainNameserver8"];
		}

		//Nameserver needs special XML
		$nameserverXML = NULL;
		foreach($this->input["domainNameserver"] as $key =>  $nameServer) {
			if("" != $nameServer) {
				$nameserverXML .= "<nameserver:update>\n";
				$ip = gethostbyname($nameServer);
				$nameserverXML .= "\t<nameserver:priority>".($key+1)."</nameserver:priority>\n";
				$nameserverXML .= "\t<nameserver:fqdn>$nameServer</nameserver:fqdn>\n";
				$nameserverXML .= "\t<nameserver:ipaddr>$ip</nameserver:ipaddr>\n";
				$nameserverXML .= "</nameserver:update>\n";
			}
		}
		$updateXML = NULL;
		$registrationXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n
					<request xmlns:domain=\"http://www.eurodns.com/domain\"> xmlns:nameserver=\"http://www.eurodns.com/nameserver\"\n
					<domain:update>
						<domain:name>".$this->registrationArray["domain"]."</domain:name>
					</domain:update>
					$nameserverXML\n
					</request>\n";

		$result = $this->_query($registrationXML);

		if ($result['isError']) {
			$this->addError(TRANS_MBAPIERROR, "EuroDNS Error: $result[responseMsg]");
			return $this->_exit();
		}

		$this->hasExecutedSuccessfully = 1;
		$this->arrayData = array("UpdateDNSCode" => $result['responseCode'], "domainUpdateDNSMessage" => $result['responseMsg']);

		return $this->_exit();
	}

	/**
	 * Returns the default configuration parameters for the Eurodns module. The number of elements in each array must be identical to
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the parameters the module uses.
	 */
	function addDefaultConfigParams() {
		//This needs to be tested with SetConfigGroups once a "add eurodns" link is added to the GUI
		$defaultConfigParams = array(
			array(NULL, NULL, "moduleUsername", "", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_USERNAME", "", "t", 1,2,0, NULL),
			array(NULL, NULL, "modulePassword", "", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_PASSWORD", "", "p", 1,3,0, NULL),
			array(NULL, NULL, "moduleTestMode", "1", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_ENABLE_EURODNS_TEST_MODE", "", "b", 0,4,0, NULL),
			array(NULL, NULL, "moduleAllowChangeDNS", "0","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_EURODNS_ALLOW_CHANGE_DNS", "TRANS_REGISTRAR_EURODNS_ALLOW_CHANGE_DNS_DESC", "b", 0,12, 0, NULL),
			array(NULL, NULL, "moduleAllowChangeContactInfo", "1", "TRANS_REGISTRAR_MODULE_SETTINGS","TRANS_REGISTRAR_EURODNS_ALLOW_CHANGE_CONTACT_INFO", "TRANS_REGISTRAR_EURODNS_ALLOW_CHANGE_CONTACT_INFO_DESC", "b", 0,13, 0, NULL),
			// admin contact data
			array(NULL, NULL, "adminFirstname", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,18,0, NULL),
			array(NULL, NULL, "adminLastName", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,19,0, NULL),
			array(NULL, NULL, "adminCompany", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,20,0, NULL),
			array(NULL, NULL, "adminEmail", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,21,0, NULL),
			array(NULL, NULL, "adminAddress1", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,22,0, NULL),
			array(NULL, NULL, "adminAddress2", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,23,0, NULL),
			array(NULL, NULL, "adminCity", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,24,0, NULL),
			array(NULL, NULL, "adminState", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,25,0, NULL),
			array(NULL, NULL, "adminZip", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,26,0, NULL),
			array(NULL, NULL, "adminCountry", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,27,0, NULL),
			array(NULL, NULL, "adminPhone", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,28,0, NULL),
			array(NULL, NULL, "adminFax", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,29,0, NULL),
			// tech contact data
			array(NULL, NULL, "techFirstname", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,30,0, NULL),
			array(NULL, NULL, "techLastName", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,31,0, NULL),
			array(NULL, NULL, "techCompany", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,32,0, NULL),
			array(NULL, NULL, "techEmail", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,33,0, NULL),
			array(NULL, NULL, "techAddress1", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,34,0, NULL),
			array(NULL, NULL, "techAddress2", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,35,0, NULL),
			array(NULL, NULL, "techCity", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,36,0, NULL),
			array(NULL, NULL, "techState", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,37,0, NULL),
			array(NULL, NULL, "techZip", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,38,0, NULL),
			array(NULL, NULL, "techCountry", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,39,0, NULL),
			array(NULL, NULL, "techPhone", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,40,0, NULL),
			array(NULL, NULL, "techFax", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,41,0, NULL),
			// aux billing contact data
			array(NULL, NULL, "auxBillingFirstname", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,42,0, NULL),
			array(NULL, NULL, "auxBillingLastName", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,43,0, NULL),
			array(NULL, NULL, "auxBillingCompany", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,44,0, NULL),
			array(NULL, NULL, "auxBillingEmail", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,45,0, NULL),
			array(NULL, NULL, "auxBillingAddress1", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,46,0, NULL),
			array(NULL, NULL, "auxBillingAddress2", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,47,0, NULL),
			array(NULL, NULL, "auxBillingCity", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,48,0, NULL),
			array(NULL, NULL, "auxBillingState", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,49,0, NULL),
			array(NULL, NULL, "auxBillingZip", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,50,0, NULL),
			array(NULL, NULL, "auxBillingCountry", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,51,0, NULL),
			array(NULL, NULL, "auxBillingPhone", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,52,0, NULL),
			array(NULL, NULL, "auxBillingFax", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,53,0, NULL),
			array(NULL, NULL, "domainNameserver1", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS1", "", "t", 0,54,0, array()),
			array(NULL, NULL, "domainNameserver2", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS2", "", "t", 0,55,0, array()),
			array(NULL, NULL, "domainNameserver3", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS3", "", "t", 0,56,0, array()),
			array(NULL, NULL, "domainNameserver4", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS4", "", "t", 0,57,0, array())
		); // $defaultConfigParams = array(

		// Grab the configs set at the registrar level
		return array_merge($this->configParams,$defaultConfigParams);

	} // END addDefaultConfigParams()
}
