<?php

require_once("lib-mbapi/include/modules/registrar/registrar.php");

$className = "ISPproxy";
$GLOBALS["moduleInfo"][$className] = array(
	"type" => "registrar",
	"status" => Registrar::STATUS_TESTING,
	"version" => "1.0.0",
	"displayName" => "ISPproxy",
	"author" => "Parallels",
	"file" => __FILE__,
	"countryCodes" => array("US"),
);

class ISPproxy extends Registrar
{
// ---------------------------------------------------------------------------------------------------- MODULE VARS
// ---------------------------------------------------------------------------------------------------- MODULE VARS
	public $_debugMode = false;

	public $_contactIDs = array();


// ---------------------------------------------------------------------------------------------------- MODULE METHODS
// ---------------------------------------------------------------------------------------------------- MODULE METHODS

	/**
	 * Returns the module display for the ISPproxy module. This includes version number and display name.
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the display info for the module.
	 */
	function getModuleInfo() {
		return $GLOBALS["moduleInfo"]["ISPproxy"];
	}

	/**
	 * Construct a new ISPproxy object. This function merely sets up capabilities after calling the
	 * constructor for Registrar.
	 */
	function ISPproxy($input) {

		$this->capabilities[REGISTRAR_REGISTERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_TRANSFERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_RENEWDOMAIN] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINEXTENDEDINFO] = 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINAVAILABILITY] = 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINTRANSFERAVAILABILITY] = 1;

		//Domain Management options
		$this->capabilities[REGISTRAR_GETDOMAINCONTACTDATA] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINCONTACTDATA] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWTYPEOPTIONS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINRENEWTYPEOPTIONS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINREGISTRARLOCKOPTIONS] = 0;
		$this->capabilities[REGISTRAR_SETDOMAINREGISTRARLOCKOPTIONS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINACCESSPASSWORD] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINACCESSPASSWORD] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWDATE] = 1;
		$this->capabilities[REGISTRAR_LIVERENEW] = 1;

		$this->capabilities[REGISTRAR_CHECKACCOUNTBALANCE] = 0;
		$this->capabilities[REGISTRAR_GETTLDPRICING] = 0;
		$this->capabilities[REGISTRAR_GETTRANSFERSTATUS] = 1;
		$this->capabilities[REGISTRAR_GETTLDLIST] = 0;
		$this->capabilities[REGISTRAR_TLDS] = array(
			"ag", "at", "co.at", "or.at", "be",
			"biz", "bz", "cc", "cn", "com.cn",
			"net.cn", "org.cn", "com", "de",
			"eu", "fm", "gs", "info", "it",
			"ms", "net", "nl", "nu", "org",
			"se", "tc", "tk", "tv", "uk",
			"co.uk", "me.uk", "org.uk", "us", "vg",
			"ws"
			);

		//Setup the array of all items to send for new registration
		$genericProfile = array(
			"usetrustee" => "",
			"firstname" => "",
			"lastname" => "",
			"company" => "",
			"address1" => "",
			"address2" => "",
			"address3" => "",
			"city" => "",
			"country" => "",
			"zipcode" => "",
			"email" => "",
			"phone" => "",	// +CCC.ZZZZZZZZZZZZxYYYY max 20 including the +
			"fax" => "",
			);
		$this->registrationArray = array(
			"action" => "create",
			"contact" => array(
				"admin" => &$genericProfile,
				"auxBilling" => &$genericProfile,
				"registrant" => &$genericProfile,
				"tech" => &$genericProfile
			),

			"domain" => "",
			"year" => "",
			"lang_pref" => 0,		//Required for dot.us (probably really for .ca, documentation error)
			"legal_type" => "",		//required for .ca same list as eNom
		);

		parent::Registrar($input);
	}


// ---------------------------------------------------------------------------------------------------- INTERNAL METHODS
// ---------------------------------------------------------------------------------------------------- INTERNAL METHODS

	/**
	 * This function merely sets the ISPproxy connection URL based on TEST mode status.
	 */
	function getURL() {
		global $CONF;

		$testMode = $CONF["domain"]["registrar"][strtolower($this->stringRep)][$this->getConfigID()]["moduleTestMode"];

		$url = "https://api.ispproxy.net/api/call.cgi";
		if ($testMode) {
			$this->_debugMode = true;
			$url = "https://api.ispproxy.net/api/call.cgi";
		}
		return $url;
	}

	/**
	 * Put the contact info in place
	 *
	 */
	function getRegistrationContactsData() {

		//Check the module setttings and place the proper contact info in place
		if($this->input["registrantZip"] && ! $this->input["registrantPostalCode"]) {
			$this->input["registrantPostalCode"] = $this->input["registrantZip"];
		}
		if($this->input["registrantPostalCode"] && ! $this->input["registrantZip"]) {
			$this->input["registrantZip"] = $this->input["registrantPostalCode"];
		}
		if($this->input["registrantState"] && ! $this->input["registrantState"]) {
			$this->input["registrantState"] = $this->input["registrantState"];
		}
		if($this->input["registrantState"] && ! $this->input["registrantState"]) {
			$this->input["registrantState"] = $this->input["registrantState"];
		}

		//Set the owner info
		$this->registrationArray["contact_set"]["registrant"]["first_name"] = $this->input["registrantFirstName"];
		$this->registrationArray["contact_set"]["registrant"]["last_name"] = $this->input["registrantLastName"];
		$this->registrationArray["contact_set"]["registrant"]["organization"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];
		$this->registrationArray["contact_set"]["registrant"]["address1"] = $this->input["registrantAddress1"];
		$this->registrationArray["contact_set"]["registrant"]["address2"] = $this->input["registrantAddress2"];
		$this->registrationArray["contact_set"]["registrant"]["city"] = $this->input["registrantCity"];
		$this->registrationArray["contact_set"]["registrant"]["state_province"] = $this->input["registrantState"];
		$this->registrationArray["contact_set"]["registrant"]["state_province_choice"] = $this->input["registrantStateProvinceChoice"];
		$this->registrationArray["contact_set"]["registrant"]["country"] = $this->input["registrantCountry"];
		$this->registrationArray["contact_set"]["registrant"]["postal_code"] = $this->input["registrantZip"];
		$this->registrationArray["contact_set"]["registrant"]["email"] = $this->input["registrantEmail"];
		$this->registrationArray["contact_set"]["registrant"]["phone"] = $this->input["registrantPhone"];
		$this->registrationArray["contact_set"]["registrant"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";

		//If the module has contact fields set, override the client info
		if($this->input["config"]["adminFirstname"] || $this->input["config"]["adminLastName"] || $this->input["config"]["adminCompany"]) {
			$this->registrationArray["contact_set"]["admin"]["first_name"] = $this->input["config"]["adminFirstname"];
			$this->registrationArray["contact_set"]["admin"]["last_name"] = $this->input["config"]["adminLastName"];
			$this->registrationArray["contact_set"]["admin"]["organization"] = $this->input["config"]["adminCompany"];
			$this->registrationArray["contact_set"]["admin"]["address1"] = $this->input["config"]["adminAddress1"];
			$this->registrationArray["contact_set"]["admin"]["address2"] = $this->input["config"]["adminAddress2"];
			$this->registrationArray["contact_set"]["admin"]["city"] = $this->input["config"]["adminCity"];
			$this->registrationArray["contact_set"]["admin"]["state_province"] = $this->input["config"]["adminState"];
			$this->registrationArray["contact_set"]["admin"]["state_province_choice"] = $this->input["adminStateProvinceChoice"];
			$this->registrationArray["contact_set"]["admin"]["country"] = $this->input["config"]["adminCountry"];
			$this->registrationArray["contact_set"]["admin"]["postal_code"] = $this->input["config"]["adminZip"];
			$this->registrationArray["contact_set"]["admin"]["email"] = $this->input["config"]["adminEmail"];
			$this->registrationArray["contact_set"]["admin"]["phone"] = $this->input["config"]["adminPhone"];
			$this->registrationArray["contact_set"]["admin"]["fax"] = (strlen($this->input["config"]["adminFax"])>4)? $this->input["registrantFax"] : "";
		} else {
			$this->registrationArray["contact_set"]["admin"]["first_name"] = $this->input["registrantFirstName"];
			$this->registrationArray["contact_set"]["admin"]["last_name"] = $this->input["registrantLastName"];
			$this->registrationArray["contact_set"]["admin"]["organization"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];
			$this->registrationArray["contact_set"]["admin"]["address1"] = $this->input["registrantAddress1"];
			$this->registrationArray["contact_set"]["admin"]["address2"] = $this->input["registrantAddress2"];
			$this->registrationArray["contact_set"]["admin"]["city"] = $this->input["registrantCity"];
			$this->registrationArray["contact_set"]["admin"]["state_province"] = $this->input["registrantState"];
			$this->registrationArray["contact_set"]["admin"]["state_province_choice"] = $this->input["registrantStateProvinceChoice"];
			$this->registrationArray["contact_set"]["admin"]["country"] = $this->input["registrantCountry"];
			$this->registrationArray["contact_set"]["admin"]["postal_code"] = $this->input["registrantZip"];
			$this->registrationArray["contact_set"]["admin"]["email"] = $this->input["registrantEmail"];
			$this->registrationArray["contact_set"]["admin"]["phone"] = $this->input["registrantPhone"];
			$this->registrationArray["contact_set"]["admin"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		}
		if($this->input["config"]["auxBillingFirstname"] || $this->input["config"]["auxBillingLastName"] || $this->input["config"]["auxBillingCompany"]) {
			$this->registrationArray["contact_set"]["auxBilling"]["first_name"] = $this->input["config"]["auxBillingFirstname"];
			$this->registrationArray["contact_set"]["auxBilling"]["last_name"] = $this->input["config"]["auxBillingLastName"];
			$this->registrationArray["contact_set"]["auxBilling"]["organization"] = $this->input["config"]["auxBillingCompany"];
			$this->registrationArray["contact_set"]["auxBilling"]["address1"] = $this->input["config"]["auxBillingAddress1"];
			$this->registrationArray["contact_set"]["auxBilling"]["address2"] = $this->input["config"]["auxBillingAddress2"];
			$this->registrationArray["contact_set"]["auxBilling"]["city"] = $this->input["config"]["auxBillingCity"];
			$this->registrationArray["contact_set"]["auxBilling"]["state_province"] = $this->input["config"]["auxBillingState"];
			$this->registrationArray["contact_set"]["auxBilling"]["state_province_choice"] = $this->input["config"]["auxBillingStateProvinceChoice"];
			$this->registrationArray["contact_set"]["auxBilling"]["country"] = $this->input["config"]["auxBillingCountry"];
			$this->registrationArray["contact_set"]["auxBilling"]["postal_code"] = $this->input["config"]["auxBillingZip"];
			$this->registrationArray["contact_set"]["auxBilling"]["email"] = $this->input["config"]["auxBillingEmail"];
			$this->registrationArray["contact_set"]["auxBilling"]["phone"] = $this->input["config"]["auxBillingPhone"];
			$this->registrationArray["contact_set"]["auxBilling"]["fax"] = (strlen($this->input["config"]["auxBillingFax"])>4)? $this->input["registrantFax"] : "";
		} else {
			$this->registrationArray["contact_set"]["auxBilling"]["first_name"] = $this->input["registrantFirstName"];
			$this->registrationArray["contact_set"]["auxBilling"]["last_name"] = $this->input["registrantLastName"];
			$this->registrationArray["contact_set"]["auxBilling"]["organization"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];;
			$this->registrationArray["contact_set"]["auxBilling"]["address1"] = $this->input["registrantAddress1"];
			$this->registrationArray["contact_set"]["auxBilling"]["address2"] = $this->input["registrantAddress2"];
			$this->registrationArray["contact_set"]["auxBilling"]["city"] = $this->input["registrantCity"];
			$this->registrationArray["contact_set"]["auxBilling"]["state_province"] = $this->input["registrantState"];
			$this->registrationArray["contact_set"]["auxBilling"]["state_province_choice"] = $this->input["registrantStateProvinceChoice"];
			$this->registrationArray["contact_set"]["auxBilling"]["country"] = $this->input["registrantCountry"];
			$this->registrationArray["contact_set"]["auxBilling"]["postal_code"] = $this->input["registrantZip"];
			$this->registrationArray["contact_set"]["auxBilling"]["email"] = $this->input["registrantEmail"];
			$this->registrationArray["contact_set"]["auxBilling"]["phone"] = $this->input["registrantPhone"];
			$this->registrationArray["contact_set"]["auxBilling"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		}
		if($this->input["config"]["techFirstname"] || $this->input["config"]["techLastName"] || $this->input["config"]["techCompany"]) {
			$this->registrationArray["contact_set"]["tech"]["first_name"] = $this->input["config"]["techFirstname"];
			$this->registrationArray["contact_set"]["tech"]["last_name"] = $this->input["config"]["techLastName"];
			$this->registrationArray["contact_set"]["tech"]["organization"] = $this->input["config"]["techCompany"];
			$this->registrationArray["contact_set"]["tech"]["address1"] = $this->input["config"]["techAddress1"];
			$this->registrationArray["contact_set"]["tech"]["address2"] = $this->input["config"]["techAddress2"];
			$this->registrationArray["contact_set"]["tech"]["city"] = $this->input["config"]["techCity"];
			$this->registrationArray["contact_set"]["tech"]["state_province"] = $this->input["config"]["techState"];
			$this->registrationArray["contact_set"]["tech"]["state_province_choice"] = $this->input["config"]["techStateProvinceChoice"];
			$this->registrationArray["contact_set"]["tech"]["country"] = $this->input["config"]["techCountry"];
			$this->registrationArray["contact_set"]["tech"]["postal_code"] = $this->input["config"]["techZip"];
			$this->registrationArray["contact_set"]["tech"]["email"] = $this->input["config"]["techEmail"];
			$this->registrationArray["contact_set"]["tech"]["phone"] = $this->input["config"]["techPhone"];
			$this->registrationArray["contact_set"]["tech"]["fax"] = (strlen($this->input["config"]["techFax"])>4)? $this->input["registrantFax"] : "";
		} else {
			$this->registrationArray["contact_set"]["tech"]["first_name"] = $this->input["registrantFirstName"];
			$this->registrationArray["contact_set"]["tech"]["last_name"] = $this->input["registrantLastName"];
			$this->registrationArray["contact_set"]["tech"]["organization"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];
			$this->registrationArray["contact_set"]["tech"]["address1"] = $this->input["registrantAddress1"];
			$this->registrationArray["contact_set"]["tech"]["address2"] = $this->input["registrantAddress2"];
			$this->registrationArray["contact_set"]["tech"]["city"] = $this->input["registrantCity"];
			$this->registrationArray["contact_set"]["tech"]["state_province"] = $this->input["registrantState"];
			$this->registrationArray["contact_set"]["tech"]["state_province_choice"] = $this->input["registrantStateProvinceChoice"];
			$this->registrationArray["contact_set"]["tech"]["country"] = $this->input["registrantCountry"];
			$this->registrationArray["contact_set"]["tech"]["postal_code"] = $this->input["registrantZip"];
			$this->registrationArray["contact_set"]["tech"]["email"] = $this->input["registrantEmail"];
			$this->registrationArray["contact_set"]["tech"]["phone"] = $this->input["registrantPhone"];
			$this->registrationArray["contact_set"]["tech"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		}
	}

	function _getContactIDs() {
		$this->getRegistrationContactsData();

		foreach( $this->registrationArray['contact_set'] as $type => $contact ) {

			if (isset($this->_contactIDs[ $type ])) {
				continue;
			}

			$response = $this->_query("QueryContactList", array(
				'firstname' => $contact['first_name'],
				'lastname' => $contact['last_name'],
				'organization' => $contact['organization'],
				'zip' => $contact['postal_code'],
				'country' => $contact['country'],
				'email' => $contact['email'],
				'phone' => str_replace('+', null,$contact['phone'])
				) );

			if ($response['PROPERTY']['TOTAL'][0] > 0) {
				echo "<h1>Contact's found!</h1>";
				show($response,1);
			} else {
				// must create user

				$response = $this->_query("AddContact", array(
					'firstname' => $contact['first_name'],
					'lastname' => $contact['last_name'],
					'organization' => $contact['organization'],
					'street' => $contact['address1'] .' '. $contact['address2'],
					'city' => $contact['city'],
					'state' => $contact['state_province'],
					'zip' => $contact['postal_code'],
					'country' => $contact['country'],
					'phone' => $contact['phone'],
					'fax' => $contact['fax'],
					'email' => $contact['email']
					) );

				if (!$response || isset($response['error'])) {
					$this->addError(TRANS_MBAPIERROR, 'Error creating contact []');
					return false;
				} else {
					$this->_contactIDs[ $type ] = $response['PROPERTY']['CONTACT'][0];
				}
			}
		}
		return true;
	}

	function _domainDetails() {
		$response = $this->_query("StatusDomain", array(
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']
			) );

		if ($response['CODE'] != 200) {
			return false;
		}

		return $response['PROPERTY'];
	}


	function _getNexusData() {

		$ret = array();

		$nexusData = $this->getTLDs($this->input["domainTLD"], true);
		$nexusData = $nexusData[ $this->input["domainTLD"] ];

		foreach( (array)$nexusData as $extAttrib ) {
			if (isset($extAttrib['name']) && isset($this->input[ $extAttrib['name'] ])) {
				$inputName = $extAttrib['name'];
			}

			$skip = false;
			switch( $inputName ) {
			/*
			 *	US
			 */
			case "us_nexus":
				$name = "x-us-nexus-category";
				break;
			case "global_cc_us":
				$name = "x-us-nexus-validator";
				break;
			case "us_purpose":
				$name = "x-us-nexus-apppurpose";
				break;
			/*
			 *	CA
			 */
			case "circa_legal_type":
				$name = "CIRCAType";
				break;
			case "circa-registrant":
				$name = 'CircaRegistrant';
				break;
			case "cira-registrant-desc":
				$name = 'CiraRegistrantDesc';
				break;
			case "cira-trademark-no":
				$name = 'CiraTrademarkNo';
				break;
			case "cira-org-registered-in":
				$name = 'CiraOrgRegisteredIn';
				break;
			/*
			 *	CO.UK
			 */
			case 'uk_legal_type':
				$name = 'COUKLegalType';
				break;
			case 'uk_reg_co_no':
				$name = 'COUKCompanyID';
				break;
			case 'registered_for':
				$name = 'COUKRegisteredfor';
				break;
			case 'uk_reg_opt_out':
				$name = 'COUKRegOptOut';
				break;
			/*
			 *	ORG.UK
			 */
			case 'uk_legal_type':
				$name = 'ORGUKLegalType';
				break;
			case 'uk_reg_co_no':
				$name = 'ORGUKCompanyID';
				break;
			case 'registered_for':
				$name = 'ORGUKRegisteredfor';
				break;
			case 'uk_reg_opt_out':
				$name = 'ORGUKRegOptOut';
				break;
			/*
			 *	ME.UK
			 */
			case 'uk_legal_type':
				$name = 'MEUKLegalType';
				break;
			case 'uk_reg_co_no':
				$name = 'MEUKCompanyID';
				break;
			case 'registered_for':
				$name = 'MEUKRegisteredfor';
				break;
			case 'uk_reg_opt_out':
				$name = 'MEUKRegOptOut';
				break;
			default:
				$skip = true;
				break;
			}
			if (!$skip) {
				$ret[ $name ] = $this->input[ $inputName ];
			}
		}
		return $ret;
	}

	function _query( $command, $params ) {

		dbg("ISPproxy query '$command'");
		dbg($params);

		$c = $this->getCurlObject();
		$c->setURL($this->getURL());
		$c->setFieldMethod(1); // get method

		$c->addField('s_entity', ($this->_debugMode ? 1234 : 'dasd'));

		$c->addField('s_login', $this->UID);
		$c->addField("s_pw",$this->PW);
		$c->addField("command", $command);

		$c->addMultiFields( $this->_getNexusData() );
		$c->addMultiFields( (array)$params );

		$rawResult = $c->toString();

		dbg("cURL object for ISPproxy call $command");
		dbg($c);

		if (!notNull($rawResult)) {
			$this->addError(TRANS_MBAPIERROR, "Registrar returned an empty result.");
		} else {
			$result = array();

			foreach( (array)split("\r?\n", $rawResult) as $line ) {
				if (notNull($line) && ($line != 'EOF') && ($line != '[RESPONSE]')) {
					$x = explode('=', $line);

					if (mb_strpos($x[0], '[')) {

						// pointer
						$p =& $result;

						// clean off the trailing bracket
						foreach( (array)explode('[', $x[0]) as $k ) {
							$k = strtoupper( trim(str_replace(']', null, $k)) );

							if (!is_array($p[ $k ]))
								$p[ $k ] = array();

							$p =& $p[ $k ];
						}

						$p = $x[1];
						unset($p);

						continue;
					}
					$result[ strtoupper( trim($x[0]) ) ] = $x[1];
				}
			}

			return $result;
		}

		return false;
	}

	function _exit() {
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}



// ---------------------------------------------------------------------------------------------------- REGISTRAR METHODS
// ---------------------------------------------------------------------------------------------------- REGISTRAR METHODS


	/**
	 * Call the registrar to register a new domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function registerDomain() {
		//Popluate contact fields
		$this->_getContactIDs();

		if ($this->getErrors()) {
			return $this->_exit();
		}

		// What nameservers do we need to pass
		$dnsServers = array();
		if($this->input["domainNameserver1"]) {
			for ($i=1, $n=4; $i<=4; $i++ ) {
				if (notNull( $this->input["domainNameserver$i"] )) {
					// registrar uses 0 indexes
					$dnsServers['nameserver'.($i-1)] = $this->input["domainNameserver$i"];
				}
			}
		}

		$params = array(
			"domain" => $this->input["domainSLD"].".".$this->input["domainTLD"],
			"period" => $this->input["domainYears"],
			 );

		$params += $dnsServers;
		$params += $this->_getNexusData();

		$params += array(
			'ownercontact0' => $this->_contactIDs['registrant'],
			'admincontact0' => $this->_contactIDs['admin'],
			'techcontact0' => $this->_contactIDs['tech'],
			'billingcontact0' => $this->_contactIDs['auxBilling']
			);

		if ($this->input['domainRegistrarLock']) {
			$params['transferlock'] = 1;
		}

		$response = $this->_query("AddDomain", $params);

		if (isset($response['CODE']) && $response['CODE'] == 200) {
			$this->arrayData += array(
				'domainDateCreated' => strtotime($response['PROPERTY']['CREATEDDATE'][0]),
				'domainDateExpires' => strtotime($response['PROPERTY']['REGISTRATIONEXPIRATIONDATE'][0]),
				'registrantContactID' => $response['PROPERTY']['OWNERCONTACT'][0],
				'adminContactID' => $response['PROPERTY']['ADMINCONTACT'][0],
				'auxBillingContactID' => $response['PROPERTY']['BILLINGCONTACT'][0],
				'techContactID' => $response['PROPERTY']['TECHCONTACT'][0],
				'nameservers' => $response['PROPERTY']['NAMESERVER'],
				'domainStatus' => ( $response['PROPERTY']['STATUS'][0] == 'ACTIVE' ? 0 : 1 ),
				'domainName' => $response['PROPERTY']['DOMAIN'][0],
				'domainAuthCode' => $response['PROPERTY']['AUTH'][0],
				'zone' => $response['PROPERTY']['ZONE'][0]
				);
			$this->hasExecutedSuccessfully = 1;
		} else {
			$this->addError(TRANS_MBAPIERROR, "Failed to Register Domain, Registrar Returned Error [".$response['DESCRIPTION']."]");
		}

		return $this->_exit();
	}

	/**
	 * Call the registrar to transfer an existing domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function transferDomain() {
		$response = $this->_query("TransferDomain", array(
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD'],
			'auth' => $this->input['DomainPassword']
			) );

		switch( $response['CODE'] ) {
		case 200:
			$this->arrayData['result'] = "TRANS_SUCCESS";
			$this->hasExecutedSuccessfully = 1;
			break;
		case 531:
			$this->addError(TRANS_MBAPIERROR, "Authorization failed (wrong or missing auth-info code)");
			break;
		default:
			$this->addError(TRANS_MBAPIERROR, "Domain Transfer Failed [$response[DESCRIPTION]]");
			break;
		}

		return $this->_exit();
	}

	/**
	 * This method contains the registrar specific code for renewing a domain.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function renewDomain() {
		$response = $this->_query("PayDomainRenewal", array(
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD'],
			'period' => $this->input['NumYears']
			) );

		if ($response['CODE'] == 200) {
			$this->arrayData['result'] = "TRANS_SUCCESS";
			$this->hasExecutedSuccessfully = 1;
		} else {
			$this->arrayData['result'] = "TRANS_FAILURE";
		}

		return $this->_exit();
	}



	/**
	 *	Call the registrar and fetch domain list
	 *
	 *	@return string XML containing domain list
	 */
	function getDomains() {
		global $CONF;

		$params["first"] = (int)$this->input["offset"];

		$response = $this->_query("QueryDomainList", $params);

		if (isset($response['PROPERTY']) && $response['PROPERTY']['TOTAL'] > 0) {
			$domainList = $response['PROPERTY']['DOMAIN'];

			$this->arrayData["domains"] = array();

			foreach( (array)$domainList as $domain ) {
				if (notNull($domain)) {

					list($sld, $tld, $tld2) = explode('.', $domain);
					if (notNull($tld2))
						$tld .= ".$tld2";
					unset($tld2);

					$this->arrayData["domains"][] = array(
						"domainName" => $domain,
						"domainSLD" => $sld,
						"domainTLD" => $tld,
						);
				}
			}
		}

		if (isset($response['PROPERTY'])) {
			$this->arrayData["totalDomains"] = $response['PROPERTY']['TOTAL'][0];
			$this->arrayData["domainStartPosition"] = (int)$response['PROPERTY']['FIRST'][0];
			$this->arrayData["domainEndPosition"] = $response['PROPERTY']['LAST'][0];
		}

		return $this->_exit();
	}

	function getDomainDNS() {
		if ( ($details = $this->_domainDetails()) == false ) {
			$this->addError(TRANS_MBAPIERROR, "Registrar Failed to find Domain");
			return $this->_exit();
		}

		$this->arrayData['listDNS'] = $details['NAMESERVER'];

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/**
	 * This method contains the registrar specific code for fetching a domain's extended information.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setDomainDNS() {
		$params = array(
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']
			);


		// check if any ns exist
		for( $i=1; $i<=8; $i++ ) {
			if (isset($this->input["domainNameserver${i}"])) {
				// ispproxy is 0 indexed
				$params['nameserver'.($i-1)] = $this->input["domainNameserver${i}"];

				if (stripos($this->input["domainNameserver${i}"], $params['domain'])) {
// 					$r = $this->_query("
				}
			}
		}

		$response = $this->_query("ModifyDomain", $params);

		if ($response['CODE'] == 200) {
			$this->arrayData["result"] = "TRANS_REGISTRAR_MANAGE_DNS_UPDATED";
			$this->hasExecutedSuccessfully = 1;
		} else {
			$this->arrayData["result"] = "TRANS_REGISTRAR_MANAGE_DNS_FAILED";
		}

		return $this->_exit();
	}


	/**
	 * Call the registrar to fetch a domain's contact data
	 *
	 * @return  string  XML containing contact data for this domain
	 */
	function getDomainContactData() {
		if ( ($details = $this->_domainDetails()) == false ) {
			$this->addError(TRANS_MBAPIERROR, "Registrar Failed to find Domain");
			return $this->_exit();
		}

		if (notNull($this->input['domainSLD'])) {
			$this->arrayData += array(
				'DomainData' => array(
					'sld' => $this->input['domainSLD'],
					'tld' => $this->input['domainTLD'],
					'domainName' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']
					) );
		}

		$contactTrans = array(
			'ADMINCONTACT' => 'Admin',
			'TECHCONTACT' => 'Tech',
			'BILLINGCONTACT' => 'AuxBilling',
			'OWNERCONTACT' => 'Registrant');

		foreach( $contactTrans as $rType => $mb ) {
			$response = $this->_query("StatusContact", array('contact' => $details[ $rType ][0]));

			$this->arrayData += array(
				"${mb}ContactData" => array(
					"${mb}FirstName" => $response['PROPERTY']['FIRSTNAME'][0],
					"${mb}LastName" => $response['PROPERTY']['LASTNAME'][0],
					"${mb}Company" => $response['PROPERTY']['ORGANIZATION'][0],
					"${mb}EmailAddress" => $response['PROPERTY']['EMAIL'][0],
					"${mb}Address1" => $response['PROPERTY']['STREET'][0],
					"${mb}City" => $response['PROPERTY']['CITY'][0],
					"${mb}State" => $response['PROPERTY']['STATE'][0],
					"${mb}PostalCode" => $response['PROPERTY']['ZIP'][0],
					"${mb}Country" => $response['PROPERTY']['COUNTRY'][0],
					"${mb}Phone" => $response['PROPERTY']['PHONE'][0],
					"${mb}Fax" => $response['PROPERTY']['FAX'][0]
					) );

		}
		$this->hasExecutedSuccessfully = 1;

		return $this->_exit();
	} // END getDomainContactData()

	/**
	 * Send the updated domain contact data to the registrar
	 *
	 * @return  string  XML result
	 */
	function setDomainContactData() {
		$modifyDomainParams = array(
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']
			);

		foreach( (array)$this->input['domainContactData'] as $type => $contact ) {
			$rType = $type;

			switch( $type ) {
			case 'registrant':
				$rType = 'owner';
				break;
			case 'auxBilling':
				$rType = 'billing';
				break;
			}

			$modifyDomainParams += array(
				"${rType}contact0title" => $contact["${type}JobTitle"],
				"${rType}contact0firstname" => $contact["${type}FirstName"],
				"${rType}contact0lastname" => $contact["${type}LastName"],
				"${rType}contact0organization" => $contact["${type}OrganizationName"],
				"${rType}contact0street" => $contact["${type}Address1"] .' '. $contact["${type}Address2"],
				"${rType}contact0city" => $contact["${type}City"],
				"${rType}contact0state" => $contact["${type}StateProvince"],
				"${rType}contact0zip" => $contact["${type}PostalCode"],
				"${rType}contact0country" => $contact["${type}Country"],
				"${rType}contact0phone" => $contact["${type}Phone"] .($contact["${type}PhoneExt"] ? ' x'.$contact["${type}PhoneExt"] : null),
				"${rType}contact0fax" => $contact["${type}Fax"],
				"${rType}contact0email" => $contact["${type}EmailAddress"],
				);

		}

		$response = $this->_query("ModifyDomain", $modifyDomainParams);

		if ($response['CODE'] == 200) {
			$this->arrayData['result'] = "TRANS_SUCCESS";
			$this->hasExecutedSuccessfully = 1;
		} else {
			$this->addError(TRANS_MBAPIERROR, "Contact Data Update Failed [$response[DESCRIPTION]]");
			$this->arrayData['result'] = "TRANS_FAILURE";
		}

		return $this->_exit();
	}

	/**
	 * This method contains the registrar specific code for setting a domain's auto-renewal options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setdomainRenewTypeOptions() {
		$response = $this->_query("SetDomainRenewalMode", array(
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD'],
			'renewalMode' => $this->input["RenewFlag"]
			) );

		if ($response['CODE']== 200) {
			$this->arrayData['result'] = "TRANS_SUCCESS";
		} else {
			$this->arrayData['result'] = "TRANS_FAILURE";
		}

		$this->hasExecutedSuccessfully = 1;

		return $this->_exit();
	}

	/**
	 * This method contains the registrar specific code for fetching the domain domain autorenew options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainRenewTypeOptions() {
		if ( ($details = $this->_domainDetails()) == false ) {
			$this->addError(TRANS_MBAPIERROR, "Registrar Failed to find Domain");
			return $this->_exit();
		}

		$this->arrayData['domainRenewType'] = 0;
		if (mb_eregi("autorenew", $details['RENEWALMODE'][0])) {
			$this->arrayData['domainRenewType'] = 1;
		}
		$this->hasExecutedSuccessfully = 1;


		return $this->_exit();

	}

	/**
	 * This method contains the registrar specific code for fetching the domain expiration date.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainExpDate() {
		if ( ($details = $this->_domainDetails()) == false ) {
			$this->addError(TRANS_MBAPIERROR, "Registrar Failed to find Domain");
			return $this->_exit();
		}

		$this->arrayData += array(
			'DomainExpDate' => $details['REGISTRATIONEXPIRATIONDATE'][0],
			'DomainExpTimestamp' => strtotime($details['REGISTRATIONEXPIRATIONDATE'][0])
			);

		return $this->_exit();

	}


	/*
	 * Call the registrar to check the status of a transfer
	 *
	 * @return  string status of the transfer
	 */
	function getTransferStatus() {
		$response = $this->_query("StatusDomainTransfer", array(
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']
			) );

		if ($response['CODE']== 200) {
			$this->arrayData = $response['TRANSFERSTATUS'][0];
			$this->hasExecutedSuccessfully = 1;
		}

		return $this->_exit();
	}

	function checkDomainAvailability() {
		$response = $this->_query("CheckDomain", array(
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']
			) );

		switch ($response['CODE'][0]) {
		/// is available
		case 210:$this->arrayData['OriginalDomainIsAvailable'] = 1;break;

		/// not available
		case 211:$this->arrayData['OriginalDomainIsAvailable'] = 0;break;

		/// unknown
		default:
			$this->arrayData += array(
				'OriginalDomainIsAvailable' => 0,
				'OriginalDomainAvailabilityUnknown' => 1
				);
			break;
		}

		return $this->_exit();
	}

	function checkDomainTransferAvailability() {
		$response = $this->_query("CheckDomainTransfer", array(
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']
			) );

		switch ($response['CODE'][0]) {
		/// is available
		case 218:
			$this->arrayData += array(
				'result' => "TRANS_AVAILABLE",
				'DomainName' => $this->input['domainSLD'] .'.'. $this->input['domainTLD'],
				'DomainRegistrar' => $response['PROPERTY']['REGISTRAR'][0],
				'OtherData' => array(
					'whoisServer' => $response['PROPERTY']['WHOISSERVER'][0],
					'domainDateCreated' => strtotime($response['PROPERTY']['CREATEDDATE'][0]),
					'domainAgeInDays' => $response['PROPERTY']['AGEINDAYS'][0],
					'transferLock' => $response['PROPERTY']['TRANSFERLOCK'][0],
					'adminContactName' => $response['PROPERTY']['ADMINCONTACTNAME'][0],
					'adminContactEmail' => $response['PROPERTY']['ADMINCONTACTEMAIL'][0],
					'adminContactLang' => $response['PROPERTY']['ADMINCONTACTLANGUAGE'][0]
					)
				);
			break;

		/// not available
		case 219:
			$this->arrayData['result'] = "TRANS_NOT_AVAILABLE";
			break;

		/// unknown
		default:
			$this->arrayData['result'] = "TRANS_FAILURE";
			break;
		}

		return $this->_exit();
	}



	function getDomainAccessPassword() {
		if ( ($details = $this->_domainDetails()) == false ) {
			$this->addError(TRANS_MBAPIERROR, "Registrar Failed to find Domain");
			return $this->_exit();
		}

		$this->arrayData += array(
			'DomainAccessPasswordSet' => 1,
			'DomainAccessPassword' => $details['AUTH'][0]
			);
		$this->hasExecutedSuccessfully = 1;

		return $this->_exit();
	}

	function setDomainAccessPassword() {
		$response = $this->_query("ModifyDomain", array(
			'domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD'],
			'auth' => $this->input['DomainPassword']
			) );

		if ($response['CODE']= 200) {
			$this->arrayData['result'] = "TRANS_SUCCESS";
			$this->hasExecutedSuccessfully = 1;
		} else {
			$this->arrayData['result'] = "TRANS_FAILURE";
		}

		return $this->_exit();
	}


	function getDomainRegistrarLockOptions() {
		if ( ($details = $this->_domainDetails()) == false ) {
			$this->addError(TRANS_MBAPIERROR, "Registrar Failed to find Domain");
			return $this->_exit();
		}

		show($details,1);
		return $this->_exit();
	}

	function setDomainRegistrarLockOptions() {
		$actionValue = '';

		switch( strtoupper($this->input['domainTLD']) ) {
		case 'COM':
		case 'NET':
			$actionValue = 'registrar-lock';
			break;
		default:
			$actionValue = 'ClientTransferProhibited';
			break;
		}

		$params = array('domain' => $this->input['domainSLD'] .'.'. $this->input['domainTLD']);

		if ($this->input['UnlockRegistrar']) {
			$params['delstatus0'] = $actionValue;
		} else {
			$params['addstatus0'] = $actionValue;
		}

		$response = $this->_query("ModifyDomain", $params);

		if ($response['CODE'] == 200) {
			$this->arrayData['result'] = $response['DESCRIPTION'];
			$this->hasExecutedSuccessfully = 1;
		} else {
			$this->addError(TRANS_MBAPIERROR, "Updating Registrar Lock Option Failed [$response[DESCRIPTION]]");
		}

		return $this->_exit();
	}


// ---------------------------------------------------------------------------------------------------- CONFIG PARAMS
// ---------------------------------------------------------------------------------------------------- CONFIG PARAMS

	/**
	 * Returns the default configuration parameters for the ISPproxy module. The number of elements in each array must be identical to
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the parameters the module uses.
	 */
	function addDefaultConfigParams() {
		//This needs to be tested with SetConfigGroups once a "add ispproxy" link is added to the GUI
		$defaultConfigParams = array(
			array(NULL, NULL, "moduleUsername", "", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_USERNAME", "", "t", 1,2,0, NULL),
			array(NULL, NULL, "modulePassword", "", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_API_KEY", "", "p", 1,3,0, NULL),
			array(NULL, NULL, "moduleTestMode", "1", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_ENABLE_ISPPROXY_TEST_MODE", "", "b", 0,4,0, NULL),
			array(NULL, NULL, "moduleAllowAutorenew", "0","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_ISPPROXY_ALLOW_AUTORENEW", "TRANS_REGISTRAR_ISPPROXY_ALLOW_AUTORENEW_DESC", "r", 0,7, 0,array(
				array(NULL, NULL, "0","TRANS_OBEY_TLD",1),
				array(NULL, NULL, "1","TRANS_REGISTRAR_AUTORENEW_OPTION",2),
				array(NULL, NULL, "2","TRANS_MODERNBILL_RENEW_OPTION",3)
				)),
			array(NULL, NULL, "moduleAllowChangeDNS", "0","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_ISPPROXY_ALLOW_CHANGE_DNS", "TRANS_REGISTRAR_ISPPROXY_ALLOW_CHANGE_DNS_DESC", "b", 0,12, 0, NULL),
			array(NULL, NULL, "moduleAllowChangeContactInfo", "1", "TRANS_REGISTRAR_MODULE_SETTINGS","TRANS_REGISTRAR_ISPPROXY_ALLOW_CHANGE_CONTACT_INFO", "TRANS_REGISTRAR_ISPPROXY_ALLOW_CHANGE_CONTACT_INFO_DESC", "b", 0,13, 0, NULL),
			array(NULL, NULL, "moduleAllowChangeRegistrarLock", "0","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_ISPPROXY_ALLOW_CHANGE_REGISTRAR_LOCK", "TRANS_REGISTRAR_ISPPROXY_ALLOW_CHANGE_REGISTRAR_LOCK_DESC", "b", 0,14, 0, NULL),
			array(NULL, NULL, "moduleAllowChangeAutorenew", "0","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_ISPPROXY_ALLOW_CHANGE_AUTORENEW", "TRANS_REGISTRAR_ISPPROXY_ALLOW_CHANGE_AUTORENEW_DESC", "b", 0,15, 0, NULL),
			array(NULL, NULL, "moduleAllowChangeUpdateDomainPassword", "0","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_ISPPROXY_ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD", "TRANS_REGISTRAR_ISPPROXY_ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD_DESC", "b", 0,16, 0, NULL),
			// admin contact data
			array(NULL, NULL, "adminFirstname", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,18,0, NULL),
			array(NULL, NULL, "adminLastName", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,19,0, NULL),
			array(NULL, NULL, "adminCompany", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,20,0, NULL),
			array(NULL, NULL, "adminEmail", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,21,0, NULL),
			array(NULL, NULL, "adminAddress1", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,22,0, NULL),
			array(NULL, NULL, "adminAddress2", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,23,0, NULL),
			array(NULL, NULL, "adminCity", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,24,0, NULL),
			array(NULL, NULL, "adminState", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,25,0, NULL),
			array(NULL, NULL, "adminZip", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,26,0, NULL),
			array(NULL, NULL, "adminCountry", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,27,0, NULL),
			array(NULL, NULL, "adminPhone", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,28,0, NULL),
			array(NULL, NULL, "adminFax", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,29,0, NULL),
			// tech contact data
			array(NULL, NULL, "techFirstname", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,30,0, NULL),
			array(NULL, NULL, "techLastName", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,31,0, NULL),
			array(NULL, NULL, "techCompany", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,32,0, NULL),
			array(NULL, NULL, "techEmail", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,33,0, NULL),
			array(NULL, NULL, "techAddress1", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,34,0, NULL),
			array(NULL, NULL, "techAddress2", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,35,0, NULL),
			array(NULL, NULL, "techCity", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,36,0, NULL),
			array(NULL, NULL, "techState", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,37,0, NULL),
			array(NULL, NULL, "techZip", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,38,0, NULL),
			array(NULL, NULL, "techCountry", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,39,0, NULL),
			array(NULL, NULL, "techPhone", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,40,0, NULL),
			array(NULL, NULL, "techFax", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,41,0, NULL),
			// aux billing contact data
			array(NULL, NULL, "auxBillingFirstname", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,42,0, NULL),
			array(NULL, NULL, "auxBillingLastName", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,43,0, NULL),
			array(NULL, NULL, "auxBillingCompany", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,44,0, NULL),
			array(NULL, NULL, "auxBillingEmail", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,45,0, NULL),
			array(NULL, NULL, "auxBillingAddress1", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,46,0, NULL),
			array(NULL, NULL, "auxBillingAddress2", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,47,0, NULL),
			array(NULL, NULL, "auxBillingCity", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,48,0, NULL),
			array(NULL, NULL, "auxBillingState", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,49,0, NULL),
			array(NULL, NULL, "auxBillingZip", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,50,0, NULL),
			array(NULL, NULL, "auxBillingCountry", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,51,0, NULL),
			array(NULL, NULL, "auxBillingPhone", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,52,0, NULL),
			array(NULL, NULL, "auxBillingFax", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,53,0, NULL),
			array(NULL, NULL, "domainNameserver1", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS1", "", "t", 0,54,0, array()),
			array(NULL, NULL, "domainNameserver2", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS2", "", "t", 0,55,0, array()),
			array(NULL, NULL, "domainNameserver3", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS3", "", "t", 0,56,0, array()),
			array(NULL, NULL, "domainNameserver4", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS4", "", "t", 0,57,0, array())

		); // $defaultConfigParams = array(


		// Grab the configs set at the registrar level
		return array_merge($this->configParams,$defaultConfigParams);

	} // END addDefaultConfigParams()
}
