<?php

require_once("lib-mbapi/include/modules/registrar/registrar.php");

$className = "Namecheap";
$GLOBALS["moduleInfo"][$className] = array(
	"type" => "registrar",
	"status" => Registrar::STATUS_TESTING,
	"version" => "1.0.0",
	"displayName" => "Namecheap",
	"author" => "Parallels",
	"file" => __FILE__,
	"countryCodes" => array("US"),
);

class Namecheap extends Registrar
{
	public $dnsOptions = array();


	/**
	 * Returns the module display for the Namecheap module. This includes version number and display name.
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the display info for the module.
	 */
	function getModuleInfo() {
		return $GLOBALS["moduleInfo"]["Namecheap"];
	}

	/**
	 * Construct a new Namecheap object. This function merely sets up capabilities after calling the
	 * constructor for Registrar.
	 */
	function Namecheap($input) {

		$this->capabilities[REGISTRAR_REGISTERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_TRANSFERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_RENEWDOMAIN] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINEXTENDEDINFO] = 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINAVAILABILITY] = 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINTRANSFERAVAILABILITY] = 0;

		//Domain Management options
		$this->capabilities[REGISTRAR_GETDOMAINCONTACTDATA] = 0;
		$this->capabilities[REGISTRAR_SETDOMAINCONTACTDATA] = 0;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWTYPEOPTIONS] = 0;
		$this->capabilities[REGISTRAR_SETDOMAINRENEWTYPEOPTIONS] = 0;
		$this->capabilities[REGISTRAR_GETDOMAINREGISTRARLOCKOPTIONS] = 0;
		$this->capabilities[REGISTRAR_SETDOMAINREGISTRARLOCKOPTIONS] = 0;
		$this->capabilities[REGISTRAR_GETDOMAINACCESSPASSWORD] = 0;
		$this->capabilities[REGISTRAR_SETDOMAINACCESSPASSWORD] = 0;
		$this->capabilities[REGISTRAR_GETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWDATE] = 0;
		$this->capabilities[REGISTRAR_LIVERENEW] = 0;

		$this->capabilities[REGISTRAR_CHECKACCOUNTBALANCE] = 1;
		$this->capabilities[REGISTRAR_GETTLDPRICING] = 0;
		$this->capabilities[REGISTRAR_GETTRANSFERSTATUS] = 0;
		$this->capabilities[REGISTRAR_GETTLDLIST] = 0;
		$this->capabilities[REGISTRAR_TLDS] = array("com", "net", "org", "biz");

		//Setup the array of all items to send for new registration
		$genericProfile = array(
			"usetrustee" => "",
			"firstname" => "",
			"lastname" => "",
			"company" => "",
			"address1" => "",
			"address2" => "",
			"address3" => "",
			"city" => "",
			"country" => "",
			"zipcode" => "",
			"email" => "",
			"phone" => "",	// +CCC.ZZZZZZZZZZZZxYYYY max 20 including the +
			"fax" => "",
			);
		$this->registrationArray = array(
			"action" => "create",
			"contact" => array(
				"admin" => &$genericProfile,
				"auxBilling" => &$genericProfile,
				"registrant" => &$genericProfile,
				"tech" => &$genericProfile
			),

			"domain" => "",
			"year" => "",
			"lang_pref" => 0,		//Required for dot.us (probably really for .ca, documentation error)
			"legal_type" => "",		//required for .ca same list as eNom
		);

		$this->dnsOptions = array('A','CNAME','MX','MXE','TXT','URL','FRAME');

		parent::Registrar($input);

	}

	/**
	 * This function merely sets the Namecheap connection URL based on TEST mode status.
	 */
	function getURL() {
		global $CONF;

		$testMode = $CONF["domain"]["registrar"][strtolower($this->stringRep)][$this->getConfigID()]["moduleTestMode"];
		$url = "https://api.namecheap.com/xml.response";
// $testMode=1;
		if ($testMode) {
			$url = "http://api.sandbox.namecheap.com/xml.response";
		}
		return $url;
	}

	/**
	 * Call the registrar to register a new domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function registerDomain() {
		//$this->config will have all of the module info

	 	global $CONF;

		//Popluate contact fields
		$this->getRegistrationContactsData();

		//What nameservers do we need to pass
		$dnsServers = array();
		if($this->input["domainNameserver1"]) {
			for ($i=1, $n=4; $i<=4; $i++ ) {
				if (notNull( $this->input["domainNameserver$i"] )) {
					$dnsServers[] = $this->input["domainNameserver$i"];
				}
			}
		}

		$params = array(
			"DomainName" => $this->input["domainSLD"].".".$this->input["domainTLD"],
			"Years" => $this->input["domainYears"],
			"Nameservers" => implode(",", $dnsServers) );

		$params += $this->getProperContactInfo();

		$response = $this->_query("namecheap.domains.create", $params);

		if (isset($response['CreateResult'])) {
			$this->arrayData = array(
				"OrderID" => $response['CreateResult'][0]['_attributes']['OrderID'],
				'TransactionID' => $response['CreateResult'][0]['_attributes']['TransactionID']
				);
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/**
	 * Call the registrar to transfer an existing domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function transferDomain() {
		global $CONF;

		$params = array(
			"DomainName" => $this->input["domainSLD"].".".$this->input["domainTLD"],
			"Years" => (int)$this->input["domainYears"],
			"EPPCode" => $this->input["domainTransferKey"]);

		$response = $this->_query("namecheap.domains.transfer.create", $params);

		if (isset($response["DomainTransferCreateResult"])) {
			$response = $response['DomainTransferCreateResult'][0]['_attributes'];
			if ($response["Transfer"] != "true") {
				$this->addError("Namecheap Error:", "Transfer failed");
			} else {
				$this->arrayData = array(
					"OrderID" => $response['OrderID'],
					'TransactionID' => $response['TransactionID'],
					"TransferID" => $response["TransferID"]
					);
			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/**
	 * This method contains the registrar specific code for renewing a domain.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function renewDomain() {
		//$this->config will have all of the module info
		$params = array(
			"DomainName" => $this->input["domainSLD"].".".$this->input["domainTLD"],
			"Years" => (int)$this->input["domainYears"]);

		$response = $this->_query("namecheap.domains.renew", $params);

		if (isset($response["RenewResult"])) {
			$this->arrayData = array(
				"domainName" => $response["RenewResult"]["_attributes"]["Domain"],
				"OrderID" => $response["RenewResult"]["_attributes"]["OrderId"],
				"TransactionID" => $response["RenewResult"]["_attributes"]["TransactionId"]);
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}


	/**
	 * Put the contact info in place
	 *
	 */
	function getRegistrationContactsData() {

		//Check the module setttings and place the proper contact info in place
		if($this->input["registrantZip"] && ! $this->input["registrantPostalCode"]) {
			$this->input["registrantPostalCode"] = $this->input["registrantZip"];
		}
		if($this->input["registrantPostalCode"] && ! $this->input["registrantZip"]) {
			$this->input["registrantZip"] = $this->input["registrantPostalCode"];
		}
		if($this->input["registrantState"] && ! $this->input["registrantState"]) {
			$this->input["registrantState"] = $this->input["registrantState"];
		}
		if($this->input["registrantState"] && ! $this->input["registrantState"]) {
			$this->input["registrantState"] = $this->input["registrantState"];
		}

		//Set the owner info
		$this->registrationArray["contact_set"]["registrant"]["first_name"] = $this->input["registrantFirstName"];
		$this->registrationArray["contact_set"]["registrant"]["last_name"] = $this->input["registrantLastName"];
		$this->registrationArray["contact_set"]["registrant"]["organization_name"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];
		$this->registrationArray["contact_set"]["registrant"]["address1"] = $this->input["registrantAddress1"];
		$this->registrationArray["contact_set"]["registrant"]["address2"] = $this->input["registrantAddress2"];
// 		$this->registrationArray["contact_set"]["registrant"]["address3"] = "";
		$this->registrationArray["contact_set"]["registrant"]["city"] = $this->input["registrantCity"];
		$this->registrationArray["contact_set"]["registrant"]["state_province"] = $this->input["registrantState"];
		$this->registrationArray["contact_set"]["registrant"]["state_province_choice"] = $this->input["registrantStateProvinceChoice"];
		$this->registrationArray["contact_set"]["registrant"]["country"] = $this->input["registrantCountry"];
		$this->registrationArray["contact_set"]["registrant"]["postal_code"] = $this->input["registrantZip"];
		$this->registrationArray["contact_set"]["registrant"]["email_address"] = $this->input["registrantEmail"];
		$this->registrationArray["contact_set"]["registrant"]["phone"] = $this->input["registrantPhone"];
		$this->registrationArray["contact_set"]["registrant"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
// 		$this->registrationArray["contact_set"]["registrant"]["url"] = "";
// 		$this->registrationArray["contact_set"]["registrant"]["lang_pref"] = $this->input["domain_idn_language"];

		//If the module has contact fields set, override the client info
		if($this->input["config"]["adminFirstname"] || $this->input["config"]["adminLastName"] || $this->input["config"]["adminCompany"]) {
			$this->registrationArray["contact_set"]["admin"]["first_name"] = $this->input["config"]["adminFirstname"];
			$this->registrationArray["contact_set"]["admin"]["last_name"] = $this->input["config"]["adminLastName"];
			$this->registrationArray["contact_set"]["admin"]["organization_name"] = $this->input["config"]["adminCompany"];
			$this->registrationArray["contact_set"]["admin"]["address1"] = $this->input["config"]["adminAddress1"];
			$this->registrationArray["contact_set"]["admin"]["address2"] = $this->input["config"]["adminAddress2"];
// 			$this->registrationArray["contact_set"]["admin"]["address3"] = "";
			$this->registrationArray["contact_set"]["admin"]["city"] = $this->input["config"]["adminCity"];
			$this->registrationArray["contact_set"]["admin"]["state_province"] = $this->input["config"]["adminState"];
			$this->registrationArray["contact_set"]["admin"]["state_province_choice"] = $this->input["adminStateProvinceChoice"];
			$this->registrationArray["contact_set"]["admin"]["country"] = $this->input["config"]["adminCountry"];
			$this->registrationArray["contact_set"]["admin"]["postal_code"] = $this->input["config"]["adminZip"];
			$this->registrationArray["contact_set"]["admin"]["email_address"] = $this->input["config"]["adminEmail"];
			$this->registrationArray["contact_set"]["admin"]["phone"] = $this->input["config"]["adminPhone"];
			$this->registrationArray["contact_set"]["admin"]["fax"] = (strlen($this->input["config"]["adminFax"])>4)? $this->input["registrantFax"] : "";
// 			$this->registrationArray["contact_set"]["admin"]["url"] = "";
			//$this->registrationArray["contact_set"]["admin"]["lang_pref"] = $this->input["config"]["domain_idn_language"];
		} else {
			$this->registrationArray["contact_set"]["admin"]["first_name"] = $this->input["registrantFirstName"];
			$this->registrationArray["contact_set"]["admin"]["last_name"] = $this->input["registrantLastName"];
			$this->registrationArray["contact_set"]["admin"]["organization_name"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];
			$this->registrationArray["contact_set"]["admin"]["address1"] = $this->input["registrantAddress1"];
			$this->registrationArray["contact_set"]["admin"]["address2"] = $this->input["registrantAddress2"];
// 			$this->registrationArray["contact_set"]["admin"]["address3"] = "";
			$this->registrationArray["contact_set"]["admin"]["city"] = $this->input["registrantCity"];
			$this->registrationArray["contact_set"]["admin"]["state_province"] = $this->input["registrantState"];
			$this->registrationArray["contact_set"]["admin"]["state_province_choice"] = $this->input["registrantStateProvinceChoice"];
			$this->registrationArray["contact_set"]["admin"]["country"] = $this->input["registrantCountry"];
			$this->registrationArray["contact_set"]["admin"]["postal_code"] = $this->input["registrantZip"];
			$this->registrationArray["contact_set"]["admin"]["email_address"] = $this->input["registrantEmail"];
			$this->registrationArray["contact_set"]["admin"]["phone"] = $this->input["registrantPhone"];
			$this->registrationArray["contact_set"]["admin"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		}
		if($this->input["config"]["auxBillingFirstname"] || $this->input["config"]["auxBillingLastName"] || $this->input["config"]["auxBillingCompany"]) {
			$this->registrationArray["contact_set"]["auxBilling"]["first_name"] = $this->input["config"]["auxBillingFirstname"];
			$this->registrationArray["contact_set"]["auxBilling"]["last_name"] = $this->input["config"]["auxBillingLastName"];
			$this->registrationArray["contact_set"]["auxBilling"]["organization_name"] = $this->input["config"]["auxBillingCompany"];
			$this->registrationArray["contact_set"]["auxBilling"]["address1"] = $this->input["config"]["auxBillingAddress1"];
			$this->registrationArray["contact_set"]["auxBilling"]["address2"] = $this->input["config"]["auxBillingAddress2"];
// 			$this->registrationArray["contact_set"]["auxBilling"]["address3"] = "";
			$this->registrationArray["contact_set"]["auxBilling"]["city"] = $this->input["config"]["auxBillingCity"];
			$this->registrationArray["contact_set"]["auxBilling"]["state_province"] = $this->input["config"]["auxBillingState"];
			$this->registrationArray["contact_set"]["auxBilling"]["state_province_choice"] = $this->input["config"]["auxBillingStateProvinceChoice"];
			$this->registrationArray["contact_set"]["auxBilling"]["country"] = $this->input["config"]["auxBillingCountry"];
			$this->registrationArray["contact_set"]["auxBilling"]["postal_code"] = $this->input["config"]["auxBillingZip"];
			$this->registrationArray["contact_set"]["auxBilling"]["email_address"] = $this->input["config"]["auxBillingEmail"];
			$this->registrationArray["contact_set"]["auxBilling"]["phone"] = $this->input["config"]["auxBillingPhone"];
			$this->registrationArray["contact_set"]["auxBilling"]["fax"] = (strlen($this->input["config"]["auxBillingFax"])>4)? $this->input["registrantFax"] : "";
		} else {
			$this->registrationArray["contact_set"]["auxBilling"]["first_name"] = $this->input["registrantFirstName"];
			$this->registrationArray["contact_set"]["auxBilling"]["last_name"] = $this->input["registrantLastName"];
			$this->registrationArray["contact_set"]["auxBilling"]["organization_name"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];;
			$this->registrationArray["contact_set"]["auxBilling"]["address1"] = $this->input["registrantAddress1"];
			$this->registrationArray["contact_set"]["auxBilling"]["address2"] = $this->input["registrantAddress2"];
// 			$this->registrationArray["contact_set"]["auxBilling"]["address3"] = "";
			$this->registrationArray["contact_set"]["auxBilling"]["city"] = $this->input["registrantCity"];
			$this->registrationArray["contact_set"]["auxBilling"]["state_province"] = $this->input["registrantState"];
			$this->registrationArray["contact_set"]["auxBilling"]["state_province_choice"] = $this->input["registrantStateProvinceChoice"];
			$this->registrationArray["contact_set"]["auxBilling"]["country"] = $this->input["registrantCountry"];
			$this->registrationArray["contact_set"]["auxBilling"]["postal_code"] = $this->input["registrantZip"];
			$this->registrationArray["contact_set"]["auxBilling"]["email_address"] = $this->input["registrantEmail"];
			$this->registrationArray["contact_set"]["auxBilling"]["phone"] = $this->input["registrantPhone"];
			$this->registrationArray["contact_set"]["auxBilling"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		}
		if($this->input["config"]["techFirstname"] || $this->input["config"]["techLastName"] || $this->input["config"]["techCompany"]) {
			$this->registrationArray["contact_set"]["tech"]["first_name"] = $this->input["config"]["techFirstname"];
			$this->registrationArray["contact_set"]["tech"]["last_name"] = $this->input["config"]["techLastName"];
			$this->registrationArray["contact_set"]["tech"]["organization_name"] = $this->input["config"]["techCompany"];
			$this->registrationArray["contact_set"]["tech"]["address1"] = $this->input["config"]["techAddress1"];
			$this->registrationArray["contact_set"]["tech"]["address2"] = $this->input["config"]["techAddress2"];
// 			$this->registrationArray["contact_set"]["tech"]["address3"] = "";
			$this->registrationArray["contact_set"]["tech"]["city"] = $this->input["config"]["techCity"];
			$this->registrationArray["contact_set"]["tech"]["state_province"] = $this->input["config"]["techState"];
			$this->registrationArray["contact_set"]["tech"]["state_province_choice"] = $this->input["config"]["techStateProvinceChoice"];
			$this->registrationArray["contact_set"]["tech"]["country"] = $this->input["config"]["techCountry"];
			$this->registrationArray["contact_set"]["tech"]["postal_code"] = $this->input["config"]["techZip"];
			$this->registrationArray["contact_set"]["tech"]["email_address"] = $this->input["config"]["techEmail"];
			$this->registrationArray["contact_set"]["tech"]["phone"] = $this->input["config"]["techPhone"];
			$this->registrationArray["contact_set"]["tech"]["fax"] = (strlen($this->input["config"]["techFax"])>4)? $this->input["registrantFax"] : "";
		} else {
			$this->registrationArray["contact_set"]["tech"]["first_name"] = $this->input["registrantFirstName"];
			$this->registrationArray["contact_set"]["tech"]["last_name"] = $this->input["registrantLastName"];
			$this->registrationArray["contact_set"]["tech"]["organization_name"] = ($this->input["registrantOrganizationName"])? $this->input["registrantOrganizationName"] : $this->input["domainSLD"];
			$this->registrationArray["contact_set"]["tech"]["address1"] = $this->input["registrantAddress1"];
			$this->registrationArray["contact_set"]["tech"]["address2"] = $this->input["registrantAddress2"];
// 			$this->registrationArray["contact_set"]["tech"]["address3"] = "";
			$this->registrationArray["contact_set"]["tech"]["city"] = $this->input["registrantCity"];
			$this->registrationArray["contact_set"]["tech"]["state_province"] = $this->input["registrantState"];
			$this->registrationArray["contact_set"]["tech"]["state_province_choice"] = $this->input["registrantStateProvinceChoice"];
			$this->registrationArray["contact_set"]["tech"]["country"] = $this->input["registrantCountry"];
			$this->registrationArray["contact_set"]["tech"]["postal_code"] = $this->input["registrantZip"];
			$this->registrationArray["contact_set"]["tech"]["email_address"] = $this->input["registrantEmail"];
			$this->registrationArray["contact_set"]["tech"]["phone"] = $this->input["registrantPhone"];
			$this->registrationArray["contact_set"]["tech"]["fax"] = (strlen($this->input["registrantFax"])>4)? $this->input["registrantFax"] : "";
		}


//show($this->registrationArray["contact_set"]);
//exit;
	}

	/**
	 * reprocesses the registration array into attributes specific to namecheap
	 *
	 * @author Brian Smith
	 * @return array
	 */
	function getProperContactInfo() {
		$output = array();
		foreach( (array)$this->registrationArray['contact_set'] as $type => $details ) {
			foreach( (array)$details as $key => $info ) {

				// basically [tech][email_address] becomes 'TechEmailAddress'
				$key = ucwords($type).str_replace(' ', null, ucwords(str_replace('_', ' ', $key)));

				if (stristr($key,'choice') && !notNull($info))	$info = 's';

				$output[ $key ] = $info;

			}	unset($key, $info);
		}	unset($type, $details);

		return $output;
	}

	/**
	 *	Call the registrar and fetch domain list
	 *
	 *	@return string XML containing domain list
	 */
	function getDomains() {
		global $CONF;

		$params = array("ListType" => "ALL");

		if (isset($this->input["offset"])) {
			$params['Page'] = floor((int)$this->input["offset"] / 20)+1;
		}

		$response = $this->_query("namecheap.domains.getList", $params);

		if (isset($response["DomainGetListResult"])) {
			$domainList = $response["DomainGetListResult"][0]["Domain"];
			$pagingData = $response["Paging"][0];

			$this->arrayData["domains"] = array();

			foreach( (array)$domainList as $domain ) {
				if (isset($domain["_attributes"])) {

					list($sld, $tld, $tld2) = explode('.', $domain["_attributes"]["Name"]);
					if (notNull($tld2))
						$tld .= ".$tld2";
					unset($tld2);

					$this->arrayData["domains"][] = array(
						"domainName" => $domain["_attributes"]['Name'],
						"domainSLD" => $sld,
						"domainTLD" => $tld,
						"domainDateCreated" => strtotime($domain["_attributes"]['Created']),
						"domainDateExpires" => strtotime($domain["_attributes"]["Expires"]),
						"domainPrivacy" => (strtolower($domain["_attributes"]["WhoisGuard"]) == "present") ? 1 : 0,
						"domainIsExpired" => (strtolower($domain["_attributes"]["IsExpired"]) == "true") ? 1 : 0,
						"domainIsLocked" => (strtolower($domain["_attributes"]["IsLocked"]) == "true") ? 1 : 0,
						"domainAutoRenew" => (strtolower($domain["_attributes"]["AutoRenew"]) == "true") ? 1 : 0
						);
				}
			}

			$this->arrayData["totalDomains"] = $pagingData["TotalItems"][0];
			$this->arrayData["domainStartPosition"] = (int)$this->input["offset"];
			$this->arrayData["domainEndPosition"] = floor($pagingData["CurrentPage"][0] * $pagingData["PageSize"][0]);
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	function getDomainDNS() {
		global $CONF;

		$params = array(
			"SLD" => $this->input["domainSLD"],
			"TLD" => $this->input["domainTLD"]);

		$result = $this->_query("namecheap.domains.dns.getList", $params);

		if (isset($result["DomainDNSGetListResult"])) {
			$response = $result["DomainDNSGetListResult"][0];

			foreach( (array)$response["Nameserver"] as $ns ) {
				if (notNull( $ns ))
					$this->arrayData["listDNS"][] = $ns;
			}	unset($ns);

			$this->arrayData["UseDNS"] = ($response["_attributes"]["IsUsingOurDNS"] == "true") ? 1 : 0;
		}

		$result = $this->_query("namecheap.domains.dns.getHosts", $params);

		if (isset($result['DomainDNSGetHostsResult'])) {
			$response = $result['DomainDNSGetHostsResult'][0];

			$this->arrayData['host'] = array();

			foreach( (array)$response['Host'] as $h ) {
				if (!is_array($h))	continue;

				$this->arrayData['host'][] = array(
					'name' => $h['_attributes']['Name'],
					'type' => $h['_attributes']['Type'],
					'address' => $h['_attributes']['Address'],
					'mxpref' => $h['_attributes']['MxPref']
					);
			}	unset($h);
		}

		$this->arrayData['dnsOptions'] =& $this->dnsOptions;

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/**
	 * This method contains the registrar specific code for fetching a domain's extended information.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setDomainDNS() {

		$params = array(
			"SLD" => $this->input["domainSLD"],
			"TLD" => $this->input["domainTLD"]);

		// check if any ns exist
		$updateNS = false;
		$addNS = array();
		for( $i=1; $i<=8; $i++ ) {
			if (notNull($this->input["domainNameserver$i"])) {
				$updateNS = true;

				if (eregi("$params[SLD]\.$params[TLD]$", $this->input["domainNameserver${i}"])) {
					$addNS[] = $i;
				}
				break;
			}
		}


		if ($updateNS == true) {
			/*
			 *	Set to default
			 */
			if ($this->input["domainDNS"]) {
				// this means set default and exit
				$result = $this->_query("namecheap.domains.dns.setDefault", $params);

				if (isset($result["DomainDNSSetDefaultResult"])) {

				}

			} else {

				/*
				 * Create NS if domain name matches the domain
				 */
				foreach ((array)$addNS as $nsID ) {
					$args = $params + array(
						"Nameserver" => $this->input["domainNameserver${nsID}"],
						"IP" => gethostbyname($this->input["domainNameserver${nsID}"])
						);
					if (!$this->_query("namecheap.domains.ns.create", $args)) {
						$this->addError(TRANS_MBAPIERROR, "Failed to add nameserver ".$this->input["domainNameserver${nsID}"]);
					}
				}

				/*
				 *	And then insert new ones
				 */
				$nsCSV = array();
				for( $i=1; $i<=8; $i++ ) {
					if (notNull($this->input["domainNameserver$i"])) {
						$nsCSV[] = $this->input["domainNameserver$i"];
					}
				}

				if (notNull($nsCSV)) {
					$nsCSV = implode(',', $nsCSV);

					$args = $params + array(
						'Nameservers' => $nsCSV
						);
					if (!$this->_query("namecheap.domains.dns.setCustom", $args)) {
						$this->addError(TRANS_MBAPIERROR, "Failed to add nameserver ".$this->input["domainNameserver${nsID}"]);
					}
				}
			}	// end if ($this->input["domainDNS"])
		}

		// check if need to update host records
		if (is_array($this->input['host'])) {
			for( $i=0, $n=count($this->input['host']); $i<$n; $i++ ) {
				$it = $i+1;
				$args = $params + array(
					"HostName$it" => $this->input['host'][ $i ]["HostName$it"],
					"RecordType$it" => $this->input['host'][ $i ]["RecordType$it"],
					"Address$it" => $this->input['host'][ $i ]["Address$it"],
					"MXPref$it" => $this->input['host'][ $i ]["MXPref$it"]);

				if (!$this->_query("namecheap.domains.dns.setHosts", $args)) {
					$this->addError(TRANS_MBAPIERROR, "Failed to add host record ".$this->input["host"][ $i ]["HostName$it"]);
				}
			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	function _getNexusData() {

		$ret = array();

		$nexusData = $this->getTLDs($this->input["domainTLD"], true);
		$nexusData = $nexusData[ $this->input["domainTLD"] ];

		foreach( (array)$nexusData as $extAttrib ) {
			if (isset($extAttrib['name']) && isset($this->input[ $extAttrib['name'] ])) {
				$inputName = $extAttrib['name'];
			}

			$skip = false;
			switch( $inputName ) {
			/*
			 *	US
			 */
			case "us_nexus":
				$name = "RegistrantNexus";
				break;
			case "global_cc_us":
				$name = "RegistrantNexusCountry";
				break;
			case "us_purpose":
				$name = "RegistrantPurpose";
				break;
			/*
			 *	CA
			 */
			case "circa_legal_type":
				$name = "CIRCAType";
				break;
			case "circa-registrant":
				$name = 'CircaRegistrant';
				break;
			case "cira-registrant-desc":
				$name = 'CiraRegistrantDesc';
				break;
			case "cira-trademark-no":
				$name = 'CiraTrademarkNo';
				break;
			case "cira-org-registered-in":
				$name = 'CiraOrgRegisteredIn';
				break;
			/*
			 *	CO.UK
			 */
			case 'uk_legal_type':
				$name = 'COUKLegalType';
				break;
			case 'uk_reg_co_no':
				$name = 'COUKCompanyID';
				break;
			case 'registered_for':
				$name = 'COUKRegisteredfor';
				break;
			case 'uk_reg_opt_out':
				$name = 'COUKRegOptOut';
				break;
			/*
			 *	ORG.UK
			 */
			case 'uk_legal_type':
				$name = 'ORGUKLegalType';
				break;
			case 'uk_reg_co_no':
				$name = 'ORGUKCompanyID';
				break;
			case 'registered_for':
				$name = 'ORGUKRegisteredfor';
				break;
			case 'uk_reg_opt_out':
				$name = 'ORGUKRegOptOut';
				break;
			/*
			 *	ME.UK
			 */
			case 'uk_legal_type':
				$name = 'MEUKLegalType';
				break;
			case 'uk_reg_co_no':
				$name = 'MEUKCompanyID';
				break;
			case 'registered_for':
				$name = 'MEUKRegisteredfor';
				break;
			case 'uk_reg_opt_out':
				$name = 'MEUKRegOptOut';
				break;
			default:
				$skip = true;
				break;
			}
			if (!$skip) {
				$ret[ $name ] = $this->input[ $inputName ];
			}
		}
		return $ret;
	}

	function _query( $command, $params ) {
		$c = $this->getCurlObject();
		$c->setURL($this->getURL());
		$c->setFieldMethod(1); // get method

		$c->addField("ApiUser",$this->UID);
		$c->addField("ApiKey",$this->PW);
		$c->addField("UserName", $this->UID);
		$c->addField("Command", $command);

		$c->addField("ClientIp", "127.0.0.1");

		$c->addMultiFields( $this->_getNexusData() );
		$c->addMultiFields( (array)$params );

		$rawResult = $c->toString();

		dbg("cURL object for Namecheap call $command");
		dbg($c);

		if (!notNull($rawResult)) {
			$this->addError(TRANS_MBAPIERROR, "Registrar returned an empty result.");
		} else {
			$x = new MBXML( $rawResult );
			$result = $x->toArray();

			if (!isset($result["ApiResponse"])) {
				$this->addError(TRANS_MBAPIERROR, "Invalid Response from Registrar");
			} else {
				// alright! good response
				switch( strtoupper($result["ApiResponse"][0]["_attributes"]["Status"]) ) {
				case "ERROR":
					$this->addError("Namecheap Error: ", (string)$result["ApiResponse"][0]["Errors"][0]["Error"][0][0]);
					break;

				case "OK":
					return $result["ApiResponse"][0]["CommandResponse"][0];

				default:
					$this->addError("Namecheap Error: ", "Unknown response. $rawResult");
					break;
				}
			}

		}

		return false;
	}

	/**
	 * Call the registrar to fetch a domain's contact data
	 *
	 * @return  string  XML containing contact data for this domain
	 */
	function getDomainContactData() {
	 	global $CONF;

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	} // END getDomainContactData()

	/**
	 * Send the updated domain contact data to the registrar
	 *
	 * @return  string  XML result
	 */
	function setDomainContactData() {
	 	Global $CONF;


		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for setting a domain's auto-renewal options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setdomainRenewTypeOptions() {
		global $CONF;

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for fetching the domain registrar-lock options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainRegistrarLockOptions() {
		global $CONF;


		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for setting the domain registrar-lock options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setDomainRegistrarLockOptions() {
		global $CONF;


		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for fetching the domain access password.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainAccessPassword() {
		global $CONF;

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for setting the domain access password.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setDomainAccessPassword() {
		global $CONF;

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for fetching the domain expiration date.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainExpDate() {
		global $CONF;


		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for fetching a domain's extended information.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainExtendedInfo() {
		global $CONF;

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/*
	 * Call the registrar to check an account's balance.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function checkAccountBalance() {
		global $CONF;

		$response = $this->_query("namecheap.users.getBalances", array());

		if (isset($response["UserGetBalancesResult"])) {
			$response = $response["UserGetBalancesResult"][0]["_attributes"];

			$this->arrayData = array(
				"Currency" => $response["Currency"],
				"AvailableBalance" => $response["AvailableBalance"],
				"Balance" => $response["AccountBalance"],
				"EarnedAmount" => $response["EarnedAmount"],
				"WithdrawableAmount" => $response["WithdrawableAmount"],
				"FundsRequiredForAutoRenew" => $response["FundsRequiredForAutoRenew"]);
		}

		$this->hasExecutedSuccessfully = 1;
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/*
	 * Call the registrar to check TLD pricing cost (cogs).
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getTLDPricing() {
		global $CONF;


		$this->arrayData = $tld;

		$this->hasExecutedSuccessfully = 1;
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/*
	 * Call the registrar to check the status of a transfer
	 *
	 * @return  string status of the transfer
	 */
	function getTransferStatus() {
		global $CONF;


		$this->hasExecutedSuccessfully = 1;
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/*
	 * Call the registrar to get a list of TLDs
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	/*function getTLDList() {
	 	global $CONF;

		$listXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n
					<request xmlns:domain=\"http://www.namecheap.com/domain\">\n
					<domain:list>\n
					</domain:list>\n
					</request>\n
";

		$host = $this->getURL();
		$c = $this->getCurlObject();
		$c->constantsSetting["CURLOPT_SSL_VERIFYPEER"] = 0;
		$c->constantsSetting["CURLOPT_SSL_VERIFYHOST"] = 1;
		$c->setUsernameAndPassword($this->UID, "MD5".md5($this->PW));

		$c->setURL($host);
		//has to be double url encoded
		$c->addField("xml",urlencode($listXML),1);
		$c->setSSLCertPath("/home/achrist/web/cvs/lib-mbapi/include/modules/registrar/");
		$c->caInfo = $this->cert;

		$rawResult = $c->toString();
		$x = new MBXML($rawResult);
		$result = $x->toArray();
		$response = $result["response"][0]["result"]["msg"][0];

		$message = $response["responseText"];
		$responseCode = $result["response"][0]["result"]["_attributes"]["code"];

		$rawList = $result["response"][0]["resData"][0]["domain:list"][0]["domain:name"];
		$domainList = array();
		if($rawList && count($rawList)>0) {
			foreach($rawList as $value) {
				$domainList[$value] = $value;
			}
		}

		if($responseCode == 1000) {
			$this->hasExecutedSuccessfully = 1;
			$this->arrayData = $domainList;
		} else {
			$this->errors[] = array("Namecheap Error:", $message);
			$this->numErrors ++;
			$this->hasExecutedSuccessfully = 0;
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}*/


	/**
	 * Returns the default configuration parameters for the Namecheap module. The number of elements in each array must be identical to
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the parameters the module uses.
	 */
	function addDefaultConfigParams() {
		//This needs to be tested with SetConfigGroups once a "add namecheap" link is added to the GUI
		$defaultConfigParams = array(
			array(NULL, NULL, "moduleUsername", "", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_USERNAME", "", "t", 1,2,0, NULL),
			array(NULL, NULL, "modulePassword", "", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_API_KEY", "", "p", 1,3,0, NULL),
			array(NULL, NULL, "moduleTestMode", "1", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_ENABLE_NAMECHEAP_TEST_MODE", "", "b", 0,4,0, NULL),
			array(NULL, NULL, "moduleAllowAutorenew", "0","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_NAMECHEAP_ALLOW_AUTORENEW", "TRANS_REGISTRAR_NAMECHEAP_ALLOW_AUTORENEW_DESC", "r", 0,7, 0,array(
				array(NULL, NULL, "0","TRANS_OBEY_TLD",1),
				array(NULL, NULL, "1","TRANS_REGISTRAR_AUTORENEW_OPTION",2),
				array(NULL, NULL, "2","TRANS_MODERNBILL_RENEW_OPTION",3)
				)),
			array(NULL, NULL, "moduleAllowChangeDNS", "0","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_NAMECHEAP_ALLOW_CHANGE_DNS", "TRANS_REGISTRAR_NAMECHEAP_ALLOW_CHANGE_DNS_DESC", "b", 0,12, 0, NULL),
			// admin contact data
			array(NULL, NULL, "adminFirstname", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,18,0, NULL),
			array(NULL, NULL, "adminLastName", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,19,0, NULL),
			array(NULL, NULL, "adminCompany", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,20,0, NULL),
			array(NULL, NULL, "adminEmail", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,21,0, NULL),
			array(NULL, NULL, "adminAddress1", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,22,0, NULL),
			array(NULL, NULL, "adminAddress2", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,23,0, NULL),
			array(NULL, NULL, "adminCity", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,24,0, NULL),
			array(NULL, NULL, "adminState", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,25,0, NULL),
			array(NULL, NULL, "adminZip", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,26,0, NULL),
			array(NULL, NULL, "adminCountry", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,27,0, NULL),
			array(NULL, NULL, "adminPhone", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,28,0, NULL),
			array(NULL, NULL, "adminFax", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,29,0, NULL),
			// tech contact data
			array(NULL, NULL, "techFirstname", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,30,0, NULL),
			array(NULL, NULL, "techLastName", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,31,0, NULL),
			array(NULL, NULL, "techCompany", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,32,0, NULL),
			array(NULL, NULL, "techEmail", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,33,0, NULL),
			array(NULL, NULL, "techAddress1", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,34,0, NULL),
			array(NULL, NULL, "techAddress2", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,35,0, NULL),
			array(NULL, NULL, "techCity", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,36,0, NULL),
			array(NULL, NULL, "techState", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,37,0, NULL),
			array(NULL, NULL, "techZip", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,38,0, NULL),
			array(NULL, NULL, "techCountry", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,39,0, NULL),
			array(NULL, NULL, "techPhone", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,40,0, NULL),
			array(NULL, NULL, "techFax", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,41,0, NULL),
			// aux billing contact data
			array(NULL, NULL, "auxBillingFirstname", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,42,0, NULL),
			array(NULL, NULL, "auxBillingLastName", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,43,0, NULL),
			array(NULL, NULL, "auxBillingCompany", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,44,0, NULL),
			array(NULL, NULL, "auxBillingEmail", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,45,0, NULL),
			array(NULL, NULL, "auxBillingAddress1", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,46,0, NULL),
			array(NULL, NULL, "auxBillingAddress2", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,47,0, NULL),
			array(NULL, NULL, "auxBillingCity", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,48,0, NULL),
			array(NULL, NULL, "auxBillingState", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,49,0, NULL),
			array(NULL, NULL, "auxBillingZip", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,50,0, NULL),
			array(NULL, NULL, "auxBillingCountry", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,51,0, NULL),
			array(NULL, NULL, "auxBillingPhone", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,52,0, NULL),
			array(NULL, NULL, "auxBillingFax", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,53,0, NULL),
			array(NULL, NULL, "domainNameserver1", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS1", "", "t", 0,54,0, array()),
			array(NULL, NULL, "domainNameserver2", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS2", "", "t", 0,55,0, array()),
			array(NULL, NULL, "domainNameserver3", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS3", "", "t", 0,56,0, array()),
			array(NULL, NULL, "domainNameserver4", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS4", "", "t", 0,57,0, array())

		); // $defaultConfigParams = array(


		// Grab the configs set at the registrar level
		return array_merge($this->configParams,$defaultConfigParams);

	} // END addDefaultConfigParams()
}
