<?php

require_once("lib-mbapi/include/modules/registrar/registrar.php");

$className = "Nominet";
$GLOBALS["moduleInfo"][$className] = array(
    "type" => "registrar",
    "status" => Registrar::STATUS_TESTING,
    "version" => "1.0.1",
    "displayName" => "Nominet",
    "author" => "Parallels",
	"file" => __FILE__,
	"countryCodes" => array("GB"),
);

class Nominet extends Registrar
{
	/**
	 * Returns the module display for the Nominet module. This includes version number and display name.
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the display info for the module.
	 */
	function getModuleInfo() {
		return $GLOBALS["moduleInfo"]["Nominet"];
	}

	/**
	 * Construct a new Nominet object. This function merely sets up capabilities after calling the
	 * constructor for Registrar.
	 */
	function Nominet($input) {
		parent::Registrar($input);
		$this->capabilities[REGISTRAR_REGISTERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_TRANSFERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_RENEWDOMAIN] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINEXPDATE] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINEXTENDEDINFO] = 1;
 		$this->capabilities[REGISTRAR_CHECKDOMAINAVAILABILITY] = 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINTRANSFERAVAILABILITY] = 1;

		$this->capabilities[REGISTRAR_TLDS] = array("co.uk", "ltd.uk", "me.uk", "net.uk", "org.uk", "plc.uk", "sch.uk");
	}


	/**
	 * Return the appropriate email address to send to based on the subCommand of what's going on.
	 */
	function getDestEmailAddress() {
		if (($registrarInfo = $this->grabThisConf()) === false) {
			$this->hasExecutedSuccessfully = 0;
			return false;
		}

		switch( $this->input['subCommand'] ) {
		case 'registerDomain':
			return (notZero($registrarInfo['moduleTestMode']) ? 'automaton-' : null )."applications@nic.uk";
		break;

		case 'renewDomain':
			if (!in_array( $this->input['domainTLD'], $this->capabilities[REGISTRAR_TLDS] )) {
				break;
			}

			$tld = explode('.', $this->input['domainTLD']);

			if (notNull( $tld[0] )) {
				return (notZero($registrarInfo['moduleTestMode']) ? 'automaton-' : null )."auto-$tld[0]@nic.uk";
			}
		break;
		default:

		break;
		}

		return false;
	}

	/**
	 *  Register a new domain
	 */
	function registerDomain() {
		if (($registrarInfo = $this->grabThisConf()) === false) {
			$this->hasExecutedSuccessfully = 0;
			return false;
		}


		$msg =array();
		$msg[] = NULL;
		$msg[] = "operation: request";
		$msg[] = "key:".(notZero($registrarInfo['moduleTestMode']) ? 'automaton-' : null ).$this->input['domainSLD'].'.'.$this->input['domainTLD'];
		$msg[] = "for: ".$this->input["registrantFirstName"].' '.$this->input['registrantLastName'];
		$msg[] = "dns0: ".$registrarInfo['domainNameserver1'];
		$msg[] = "dns1: ".$registrarInfo['domainNameserver2'];
		$msg[] = "dns2: ".$registrarInfo['domainNameserver3'];
		$msg[] = "dns3: ".$registrarInfo['domainNameserver4'];

		$msg[] = "reg-contact: ".$this->input["registrantFirstName"].' '.$this->input['registrantLastName'];
#		$msg[] = "reg-trad-name:";
		$msg[] = "reg-type: ".$this->input['uk_legal_type'];
		$msg[] = "reg-co-no: ".$this->input['uk_reg_co_no'];
		$msg[] = "reg-opt-out: ".$this->input['uk_reg_opt_out'];
		$msg[] = "reg-addr: ".$this->input['registrantAddress1'];
		$msg[] = "reg-addr: ".$this->input['registrantAddress2'];
#		$msg[] = "reg-locality: ";
		$msg[] = "reg-city: ".$this->input['registrantCity'];
#		$msg[] = "reg-county: ".;
		$msg[] = "reg-country: ".( isset($this->input['registrantCountry']) ? $this->input['registrantCountry'] : $registrarInfo['auxBillingCountry']);
		$msg[] = "reg-postcode: ".$this->input['registrantPostalCode'];
		$msg[] = "reg-phone: ".$this->input['registrantPhone'];
		$msg[] = "reg-fax: ".$this->input['registrantFax'];
		$msg[] = "reg-email: ".$this->input['registrantEmail'];
#		$msg[] = "reg-ref: ".$this;

		$msg[] = "admin-c: ".$registrarInfo['adminFirstname'].' '.$registrarInfo['adminLastName'];
		$msg[] = "a-phone: ".$registrarInfo['adminPhone'];
		$msg[] = "a-fax: ".$registrarInfo['adminFax'];
		$msg[] = "a-email: ".$registrarInfo['adminEmail'];
		$msg[] = "a-addr: ".$registrarInfo['adminAddress1'];
		$msg[] = "a-addr: ".$registrarInfo['adminAddress2'];

#		$msg[] = "b-locality: ";
		$msg[] = "b-city: ".$registrarInfo['auxBillingCity'];
#		$msg[] = "b-county: ";
		$msg[] = "b-postcode: ".$registrarInfo['auxBillingZip'];
		$msg[] = "b-country: ".$registrarInfo['auxBillingCountry'];
		$msg[] = "billing-c: ".$registrarInfo['auxBillingFirstname'].' '.$registrarInfo['auxBillingLastName'];
		$msg[] = "b-phone: ".$registrarInfo['auxBillingPhone'];
		$msg[] = "b-fax: ".$registrarInfo['auxBillingFax'];
		$msg[] = "b-email: ".$registrarInfo['auxBillingEmail'];
		$msg[] = "b-addr: ".$registrarInfo['auxBillingAddress1'];
		$msg[] = "b-addr: ".$registrarInfo['auxBillingAddress2'];

		$msg[] = "first-bill:";
		$msg[] = "recur-bill:";
		$msg[] = "auto-bill:";
		$msg[] = "next-bill:";
		$msg[] = "notes:";

		$msg[] = NULL;


		if (($message = $this->signThis( implode("\n", $msg) )) === false) {
			$this->addError(TRANS_MBAPIERROR, "Error while signing message");
			$this->hasExecutedSuccessfully = 0;
			return false;
		}

		// construct the message to send
		$params = array(
			'emailSubject' => strtoupper($registrarInfo['nominetTagName'])." Domain Request",
			'emailBody' => $message,
			'emailTextBody' => $message,
			'emailFromAddress' => $registrarInfo['nominetRetEmail'],
			'emailFromName' => $this->input['registrantFirstName'].' '.$this->input['registrantLastName'],
			'emailToAddresses' => array(
				'emailAddress' => array(
					'address' => $this->getDestEmailAddress(),
					'name' => 'Nominet Automation'
					),
				)
			);
		$sendEmail = new MBQuery("ProcessEmail", "", $params);
		$sendEmail = $sendEmail->getResult();

		if ($sendEmail->getErrorCount()) {
			foreach( (array)$sendEmail->getErrors() as $err ) {
				$this->addError(TRANS_MBAPIERROR, $err);
			}
			$this->hasExecutedSuccessfully = 0;
			return false;
		}

		$this->arrayData['requestSent'] = mktime();
		$this->arrayData['domain'] = $this->input['domainSLD'].'.'.$this->input['domainTLD'];

		$this->hasExecuted = $this->hasExecutedSuccessfully = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/**
	 *  Return what CONF holds for THIS registrar's configuration.
	 */
	function grabThisConf() {
		global $CONF;

		$reg =& $CONF["domain"]["registrar"][strtolower($this->stringRep)][$this->getConfigID()];

		if (!notNull( $reg ) || !notZero($reg['configGroupID'])) {
			$this->addError(TRANS_MBAPIERROR, "Please check registrar system configuration.");
			return false;
		}

		return $reg;
	}

	/**
	 *  Sign the message passed to it based on the configuration.
	 */
	function signThis( $xx ) {
		if (($registrarInfo = $this->grabThisConf()) === false) {
			return false;
		}

		if (!is_executable( $registrarInfo['nominetLocationPGPBin'] )) {
			$this->addError(TRANS_MBAPIERROR, "PGP Binary is not executable.");
			return false;
		}

		$cmd = "echo '".addslashes($xx)."' | $registrarInfo[nominetLocationPGPBin] ".str_replace('${MY_TAG}', $registrarInfo['nominetTagName'], unsecureInput(rawurldecode($registrarInfo['nominetLocationPGPBinArgs'])));

		dbg("The command to execute");
		dbg($cmd);

		$results = array();
		$code = 0;
		exec( $cmd, $results, $code );


		dbg("Return code:\t $code");

		dbg("Results of result from command.");
		dbg($results);

		if (is_array($results) && notNull($results)) {
			return implode("\n", $results);
		}

		$this->addError(TRANS_MBAPIERROR, "Results from signing didn't return properly");
		return false;
	}

	/**
	 *  Check if the  domain is available.
	 *
	function checkDomainAvailability() {
		if (($registrarInfo = $this->grabThisConf()) === false) {
			$this->hasExecutedSuccessfully = 0;
			return false;
		}


		echo "HEYHEYHEY";
		dbg("###############################\nJDKJLFSDJKLJKLFDSJLKDFSJLKJLKFDJL\n####################################");
		exit;
	}
	/**/

	/**
	 *  Attempt to renew the domain.
	 */
	function renewDomain() {
		if (($registrarInfo = $this->grabThisConf()) === false) {
			$this->hasExecutedSuccessfully = 0;
			return false;
		}

		$msg =array();
		$msg[] = "operation: renew";
		$msg[] = "key: ".(notZero($registrarInfo['moduleTestMode']) ? 'automaton-' : null ).$this->input['domainSLD'].'.'.$this->input['domainTLD'];

// 		$msg[] = NULL;


		if (($message = $this->signThis( implode("\n", $msg) )) === false) {
			$this->addError(TRANS_MBAPIERROR, "Error while signing message");
			$this->hasExecutedSuccessfully = 0;
			return false;
		}

		// construct the message to send
		$params = array(
			'emailSubject' => strtoupper($registrarInfo['nominetTagName'])." Domain Request",
			'emailBody' => $message,
			'emailTextBody' => $message,
			'emailFromAddress' => $registrarInfo['nominetRetEmail'],
			'emailFromName' => $this->input['registrantFirstName'].' '.$this->input['registrantLastName'],
			'emailToAddresses' => array(
				'emailAddress' => array(
					'address' => $this->getDestEmailAddress(),
					'name' => 'Nominet Automation'
					),
				)
			);
		$sendEmail = new MBQuery("ProcessEmail", "", $params);
		$sendEmail = $sendEmail->getResult();

		if ($sendEmail->getErrorCount()) {
			foreach( (array)$sendEmail->getErrors() as $err ) {
				$this->addError(TRANS_MBAPIERROR, $err);
			}
			$this->hasExecutedSuccessfully = 0;
			return false;
		}

		$this->arrayData['requestSent'] = mktime();
		$this->arrayData['domain'] = $this->input['domainSLD'].'.'.$this->input['domainTLD'];

		$this->hasExecuted = $this->hasExecutedSuccessfully = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}


	/**
	 * Returns the default configuration parameters for the Nominet module. The number of elements in each array must be identical to
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the parameters the module uses.
	 */
	function addDefaultConfigParams() {
		$i=1;
		$defaultConfigParams = array(
                	// PGP Binary location
			array(NULL,NULL,"nominetLocationPGPBin", "/usr/bin/gpg", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_NOMINET_LOCATION_PGP_BIN", "", "t", 1,$i++,0,NULL),
			array(NULL,NULL,"nominetLocationPGPBinArgs", "-u \${MY_TAG} -a --always-trust --no-secmem-warning --clearsign", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_NOMINET_LOCATION_PGP_BIN_ARGS", "", "t", 1,$i++,0,NULL),
			array(NULL,NULL,"nominetTagName", "MY_TAG", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_NOMINET_TAG_NAME", "", "t", 0,$i++,0,NULL),

			// email info
			array(NULL,NULL,"nominetRetEmail","","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_NOMINET_RETURN_EMAIL","","t",0,$i++,0,NULL),
			array(NULL,NULL,"nominetRetEmailUsername","","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_NOMINET_EMAIL_USERNAME","","t",0,$i++,0,NULL),
			array(NULL,NULL,"nominetRetEmailPassword","","TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_NOMINET_EMAIL_PASSWORD","","p",0,$i++,0,NULL),
			array(NULL,NULL,"nominetRetPOPServer", "", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_NOMINET_EMAIL_POP_SERVER", "", "t",0,$i++,0,NULL),
            		array(NULL, NULL, "moduleTestMode", "1", "TRANS_REGISTRAR_MODULE_SETTINGS", "TRANS_REGISTRAR_ENABLE_NOMINET_TEST_MODE", "", "b", 0,$i++,0, NULL),
			// admin contact data
            		array(NULL, NULL, "adminFirstname", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,$i++,0,NULL),
            		array(NULL, NULL, "adminLastName", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "adminCompany", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "adminEmail", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "adminAddress1", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "adminAddress2", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "adminCity", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "adminState", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "adminZip", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "adminCountry", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "adminPhone", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "adminFax", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,$i++,0,NULL),
			// tech contact data
			array(NULL, NULL, "techFirstname", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "techLastName", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "techCompany", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "techEmail", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "techAddress1", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "techAddress2", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "techCity", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "techState", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "techZip", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "techCountry", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "techPhone", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "techFax", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,$i++,0,NULL),
			// aux billing contact data
			array(NULL, NULL, "auxBillingFirstname", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "auxBillingLastName", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "auxBillingCompany", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "auxBillingEmail", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "auxBillingAddress1", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "auxBillingAddress2", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "auxBillingCity", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "auxBillingState", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "auxBillingZip", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "auxBillingCountry", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "auxBillingPhone", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "auxBillingFax", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,$i++,0,NULL),
			array(NULL, NULL, "domainNameserver1", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS1", "", "t", 0,$i++,0, array()),
			array(NULL, NULL, "domainNameserver2", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS2", "", "t", 0,$i++,0, array()),
			array(NULL, NULL, "domainNameserver3", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS3", "", "t", 0,$i++,0, array()),
			array(NULL, NULL, "domainNameserver4", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS4", "", "t", 0,$i++,0, array())
        		); // $defaultConfigParams = array(

		// Grab the configs set at the registrar level
		return array_merge($this->configParams,$defaultConfigParams);

	} // END addDefaultConfigParams()
}