<?php

require_once ("lib-mbapi/include/modules/registrar/registrar.php");
require_once ("objects/BillingRegistrar.php");

class PBARegistrarError extends Exception {}

$className = "PBARegistrar";
$GLOBALS["moduleInfo"][$className] = array(
	"type"			=> "registrar",
	"status"		=> Registrar::STATUS_STABLE,
	"version"		=> "1.0.0",
	"displayName"	=> "PBARegistrar",
	"author"		=> "Parallels",
	"file"			=> __FILE__,
);

class PBARegistrar extends Registrar
{
	public function getModuleInfo($sub_module_name = NULL)
	{
		$moduleInfo = $GLOBALS["moduleInfo"]["PBARegistrar"];
		if (! is_null($sub_module_name)) {
			$moduleInfo = array_merge($moduleInfo, array(
				"displayName" => $sub_module_name,
			));
		} else if (isset($this) && ! is_null($this->subModuleName)) {
			$moduleInfo = array_merge($moduleInfo, array(
				"displayName" =>  $this->subModuleName,
			));
		}
		switch ($moduleInfo["displayName"]) {
		case "OpenProvider":
			$moduleInfo["countryCodes"] = array("NL");
			break;
		}
		return $moduleInfo;
	}

	public function __construct(array $input)
	{
		if (! isset($input["subModuleName"])) {
			throw new ProductException(ProductException::E_OBJECT_HAS_INVALID_ATTRS, "The 'subModuleName' parameter is absent!");
		}
		if (isset($input["configGroupID"])) {
			$this->registrar = Factory()->BillingRegistrar($input["configGroupID"]);
		}

		// quick fix to enable IDN support across all functions
		if (isset($input["domainSLD"]) && "" != trim($input["domainSLD"])) {
			$input["domainSLD"] = $this->getPUNYCode($input["domainSLD"]);
		}
		$this->initCapability($input);
		parent::__construct($input);

		$this->configID = $input["configGroupID"];
		$this->arrayData = array(NULL);
	}

	public function initCapability(array $input)
	{
		$capabilities_func_map = array(
			REGISTRAR_CHECKDOMAINAVAILABILITY			=> "check_register",
			REGISTRAR_CHECKDOMAINTRANSFERAVAILABILITY	=> "check_transfer",
			REGISTRAR_REGISTERDOMAIN					=> "register_domain",
			REGISTRAR_TRANSFERDOMAIN					=> "transfer_domain",
			REGISTRAR_RENEWDOMAIN						=> "renew_domain",
			REGISTRAR_GETDOMAINEXTENDEDINFO				=> NULL,
			REGISTRAR_GETDOMAINCONTACTDATA				=> NULL,
			REGISTRAR_SETDOMAINCONTACTDATA				=> NULL,
			REGISTRAR_GETDOMAINRENEWTYPEOPTIONS			=> NULL,
			REGISTRAR_SETDOMAINRENEWTYPEOPTIONS			=> NULL,
			REGISTRAR_GETDOMAINREGISTRARLOCKOPTIONS		=> NULL,
			REGISTRAR_SETDOMAINREGISTRARLOCKOPTIONS		=> NULL,
			REGISTRAR_GETDOMAINACCESSPASSWORD			=> NULL,
			REGISTRAR_SETDOMAINACCESSPASSWORD			=> NULL,
			REGISTRAR_GETDOMAINDNS						=> true,
			REGISTRAR_SETDOMAINDNS						=> "synchronize_domain_ns",
			REGISTRAR_GETDOMAINRENEWDATE				=> NULL,
			REGISTRAR_LIVERENEW							=> NULL,
			REGISTRAR_CHECKACCOUNTBALANCE				=> NULL,
			REGISTRAR_GETTLDPRICING						=> NULL,
			REGISTRAR_GETTRANSFERSTATUS					=> NULL,
			REGISTRAR_GETTLDLIST						=> NULL,
		);
		$this->capabilities[REGISTRAR_TLDS] = array();

		require_once("HSPC/DomainManagerProcess.php");
		foreach ($capabilities_func_map as $bill_capability => $pba_capability) {
			if (true === $pba_capability || false === $pba_capability) {
				$this->capabilities[$bill_capability] = $pba_capability ? 1 : 0;
				continue;
			}
			if (is_null($pba_capability)) {
				$this->capabilities[$bill_capability] = 0;
				continue;
			}
			$hspc_result = HSPCDomainManagerProcess::checkCapability($input["subModuleName"], $pba_capability);
			$this->capabilities[$bill_capability] = $hspc_result->data->supported ? 1 : 0;
		}
	}

	public function addDefaultConfigParams()
	{
		require_once("HSPC/DomainManagerProcess.php");
		$hspc_result = HSPCDomainManagerProcess::getEditConfigForm($this->subModuleName, $this->getPBAConfig());

		if ("" != $hspc_result->data->user_error) {
			billing_put_error("{$this->subModuleName}: {$hspc_result->data->user_error}");
		}

		require_once("HSPC/Tools.php");
		return array(
			array(NULL, NULL, "moduleConfig", HSPCTools::stylizeDomainManagerForm($hspc_result->data->form_content), NULL, NULL, NULL, "x", 0, 1, 0, NULL),
            array(NULL, NULL, "domainNameserver1", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS1", "", "t", ("OpenProvider" == $this->subModuleName ? 1 : 0), 2, 0, NULL),
            array(NULL, NULL, "domainNameserver2", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS2", "", "t", ("OpenProvider" == $this->subModuleName ? 1 : 0), 3, 0, NULL),
            array(NULL, NULL, "domainNameserver3", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS3", "", "t", 0, 4, 0, NULL),
            array(NULL, NULL, "domainNameserver4", "", "TRANS_REGISTRAR_NS", "TRANS_REGISTRAR_NS4", "", "t", 0, 5, 0, NULL),
		);
	}

	public function collectConfigData(array $collect_input = array())
	{
		$result = $this->collectPBAConfig();
		$collect_input["moduleConfig"] = $result["data"];
		return array(
			"data" => $collect_input,
			"error" => $result["error"],
		);
	}

	public function getRegistrantParams($action, $domain_sld, $domain_tld, ClientContact $contact = NULL)
	{
		require_once("HSPC/Tools.php");
		require_once("HSPC/DomainManagerProcess.php");

		if (is_null($contact)) {
			$contact = Factory()->ClientContact();
		}
		$prior_subscription = is_null($contact->client_id) ? NULL : $this->getPriorSubscription($this->registrar->id, $contact->client_id);
		$domain = "{$domain_sld}.{$domain_tld}";
		$contacts_panels = array();

		// Contacts
		foreach ($this->getHSPCContactTypes() as $hspc_contact_type) {
			list($type, $title) = array($hspc_contact_type->type, $hspc_contact_type->title);
			$hspc_cnt = isset($this->hspc_contacts[$type]) ? $this->hspc_contacts[$type] : NULL;
			$hspc_cnt_ext = isset($this->hspc_contacts_extdata[$type]) ? $this->hspc_contacts_extdata[$type] : NULL;
			$hspc_cnt = FALSE === ($hspc_cnt = unserialize($this->decryptData($hspc_cnt))) ? NULL : $hspc_cnt;
			$hspc_cnt_ext = FALSE === ($hspc_cnt_ext = unserialize($this->decryptData($hspc_cnt_ext))) ? NULL : $hspc_cnt_ext;

			if (is_null($hspc_cnt) && ! is_null($prior_subscription)) {
				$hspc_cnt = FALSE === ($hspc_cnt = unserialize($this->decryptData($prior_subscription->PackageAttributes["hspc_{$type}_cnt"]->value))) ? NULL : $hspc_cnt;
				$hspc_cnt_ext = FALSE === ($hspc_cnt_ext = unserialize($this->decryptData($prior_subscription->PackageAttributes["hspc_{$type}_cnt_ext"]->value))) ? NULL : $hspc_cnt_ext;
			}
			$hspc_result = HSPCDomainManagerProcess::getEditContactForm(
				$this->subModuleName,
				$this->getPBAConfig(),
				$domain,
				$this->hspc_actions_map[$action],
				is_null($hspc_cnt) ? HSPCTools::getHSPCClientContact($contact->id, true) : $hspc_cnt,
				$type,
				is_null($hspc_cnt_ext) ? array() : $hspc_cnt_ext
			);
			$panel = array(
				"title" => $title,
				"description" => "",
			);
			$inputs = array();
			if ("owner" != $type) {
				$chooser_id = "chooser_{$type}_{$domain_sld}_{$domain_tld}";
				$chooser_value = isset($prior_subscription->PackageAttributes["hspc_{$type}_as_owner"])
					? $prior_subscription->PackageAttributes["hspc_{$type}_as_owner"]->value
					: (isset($_REQUEST[$chooser_id]) ? bgpc($chooser_id) : 1);
				$inputs[] = array(
					"id" => $chooser_id,
					"title" => "TRANS_PBA_DOMAIN_CONTACT_AS_OWNER_OPTION",
					"type" => "b",
					"default" => $chooser_value,
					"before_title" => true,
				);
			}
			$panel_id = "panel_contact_{$type}_{$domain_sld}_{$domain_tld}";
			$inputs[] = array(
				"id" => $panel_id,
				"type" => "x",
				"default" => HSPCTools::stylizeDomainManagerForm($hspc_result->data->form_content) . ("owner" == $type ? "" : "
					<script language='javascript'>
					//<!--
						if ('1' == '{$chooser_value}') {
							document.getElementById('{$panel_id}').style.display = 'none';
							document.getElementById('{$panel_id}').style.visibility = 'hidden';
						} else {
							document.getElementById('{$panel_id}').style.display = '';
							document.getElementById('{$panel_id}').style.visibility = 'visible';
						}
						// 'domainDetails_' prefix - for Store
						// 'fid-' prefix - for administrative CP
						chooser_el = document.getElementById('domainDetails_{$chooser_id}')
							? document.getElementById('domainDetails_{$chooser_id}')
							: document.getElementById('fid-{$chooser_id}');
						chooser_el.onclick = function (x) {
							if (this.checked) {
								document.getElementById('{$panel_id}').style.display = 'none';
								document.getElementById('{$panel_id}').style.visibility = 'hidden';
							} else {
								document.getElementById('{$panel_id}').style.display = '';
								document.getElementById('{$panel_id}').style.visibility = 'visible';
							}
							return true;
						};
					//-->
					</script>")
			);
			$panel["inputs"] = $inputs;
			$contacts_panels[] = $panel;
		}

		// Domain's extra data
		$hspc_dom_ext = $this->hspc_domain_extdata;

		if (is_null($hspc_dom_ext) && ! is_null($prior_subscription)) {
			$hspc_dom_ext = $prior_subscription->PackageAttributes["hspc_dom_ext"]->value;
		}
		$hspc_result = HSPCDomainManagerProcess::getEditDomainExtdataForm(
			$this->subModuleName,
			$this->getPBAConfig(),
			$domain,
			$this->hspc_actions_map[$action],
			FALSE === ($hspc_dom_ext = unserialize($this->decryptData($hspc_dom_ext))) ? array() : $hspc_dom_ext
		);
		if ("" != $hspc_result->data->form_content) {
			$panel = array(
				"title" => blmsg("TRANS_PBA_DOMAIN_EXT_DATA_TITLE", array("DOMAIN" => $domain)),
				"description" => "",
				"inputs" => array(array(
					"id" => "panel_domain_extdata_{$domain_sld}_{$domain_tld}",
					"type" => "x",
					"default" => HSPCTools::stylizeDomainManagerForm($hspc_result->data->form_content),
				)),
			);
			$contacts_panels[] = $panel;
		}
		return $contacts_panels;
	}

	public function collectRegistrantParams($action, $domain_sld, $domain_tld, ClientContact $contact = NULL)
	{
		require_once("HSPC/DomainManagerProcess.php");
		$domain = "{$domain_sld}.{$domain_tld}";

		// Collect contacts data
		$old_contacts = array();
		$old_contacts_extdata = array();
		foreach ($this->getHSPCContactTypes() as $hspc_contact_type) {
			list($type, $title) = array($hspc_contact_type->type, $hspc_contact_type->title);
			$hspc_cnt = FALSE === ($hspc_cnt = unserialize($this->decryptData($prior_subscription->PackageAttributes["hspc_{$type}_cnt"]))) ? NULL : $hspc_cnt;
			$hspc_cnt_ext = FALSE === ($hspc_cnt_ext = unserialize($this->decryptData($prior_subscription->PackageAttributes["hspc_{$type}_cnt_ext"]))) ? NULL : $hspc_cnt_ext;
			$old_contacts[$type] = $hspc_cnt;
			$old_contacts_extdata[$type] = $hspc_cnt_ext;
		}
		$contacts_hspc_result = HSPCDomainManagerProcess::getCollectedContactsData(
			$this->subModuleName,
			$this->getPBAConfig(),
			$domain,
			$this->hspc_actions_map[$action],
			$old_contacts,
			$old_contacts_extdata
		);

		$collected_data = array();
		$owner_contact = $contacts_hspc_result->data->contacts->owner;
		$owner_contact_extdata = $contacts_hspc_result->data->contacts_extdata->owner;

		$hspc_contacts = array();
		$hspc_contacts_extdata = array();
		foreach (array_keys((array)$contacts_hspc_result->data->contacts) as $type) {
			$chooser_id = "chooser_{$type}_{$domain_sld}_{$domain_tld}";
			if ("owner" != $type && 1 == bgpc($chooser_id)) {
				$hspc_contact = $owner_contact;
				$hspc_contact_extdata = $owner_contact_extdata;
				$collected_data["hspc_{$type}_as_owner"] = 1;
			} else {
				$hspc_contact = $contacts_hspc_result->data->contacts->$type;
				$hspc_contact_extdata = $contacts_hspc_result->data->contacts_extdata->$type;
				$collected_data["hspc_{$type}_as_owner"] = 0;
			}

			$this->hspc_contacts[$type] = $collected_data["hspc_{$type}_cnt"] = $this->encryptData(serialize($hspc_contacts[$type] = $hspc_contact));
			$this->hspc_contacts_extdata[$type] = $collected_data["hspc_{$type}_cnt_ext"] = $this->encryptData(serialize($hspc_contacts_extdata[$type] = $hspc_contact_extdata));
		}

		// Collect domain's extdata
		$hspc_dom_ext = FALSE === ($hspc_dom_ext = unserialize($this->decryptData($prior_subscription->PackageAttributes["hspc_dom_ext"]))) ? NULL : $hspc_dom_ext;
		$extdata_hspc_result = HSPCDomainManagerProcess::getCollectedDomainExtdata(
			$this->subModuleName,
			$this->getPBAConfig(),
			$domain,
			$this->hspc_actions_map[$action],
			$hspc_dom_extdata
		);
		$this->hspc_domain_extdata = $collected_data["hspc_dom_ext"] = $this->encryptData(serialize($hspc_domain_extdata = $extdata_hspc_result->data->domain_extdata));

		// Validate all data
		$validate_hspc_result = HSPCDomainManagerProcess::validateData(
			$this->subModuleName,
			$this->getPBAConfig(),
			$domain,
			$this->hspc_actions_map[$action],
			$hspc_contacts,
			$hspc_contacts_extdata,
			$hspc_domain_extdata
		);
		$errors_list = array();
		if (1 != $validate_hspc_result->data->is_valid) {
			foreach ($validate_hspc_result->data->error_list as $error_desc) {
				$panel_type = ("contact" == $error_desc->form ? "contact_{$error_desc->contact_type}" : $error_desc->form);
				$panel_id = "panel_{$panel_type}_{$domain_sld}_{$domain_tld}";
				if (! isset($errors_list[$panel_id])) {
					$errors_list[$panel_id] = array();
				}
				$errors_list[$panel_id][] = $error_desc->message;
			}
			foreach ($errors_list as $panel_id => $errors) {
				$errors_list[$panel_id] = join("\n", $errors);
			}
		}

		$result = array(
			"data" => $collected_data,
			"form_descriptor" => $this->getRegistrantParams($action, $domain_sld, $domain_tld, $contact),
		);
		if (0 < count($errors_list)) {
			$result["errors_list"] = $errors_list;
		}
		return $result;
	}

	public function checkDomainAvailability()
	{
		$domains = array();
		foreach ($this->input["domains"] as $domain) {
			$domains[] = "{$domain['sld']}.{$domain['tld']}";
		}
		$hspc_result = HSPCDomainManagerProcess::checkRegister($this->subModuleName, $this->getPBAConfig(), $domains);
		if ("" != $hspc_result->data->user_error) {
			return $this->createReturn($hspc_result->data->user_error, false);
		}
		$result = array();
		foreach ($hspc_result->data->domains as $domain => $pba_dom_status) {
			$result[$domain]["status"] = $this->avail_status_map[$pba_dom_status];
		}
		return $this->createReturn($result);
	}

	// this function sends the call to register a new domain
	public function registerDomain()
	{
		require_once("HSPC/DomainManagerProcess.php");
		$this->attributes = array();

		try {
			if (1 == $this->input["hspc_need_check_status_only"]) {
				$domain_status = $this->hspcGetDomainStatus($domain, "register");
				$this->commandStatus = "registering" == $domain_status
					? self::ACTION_STATUS_WAITING
					: self::ACTION_STATUS_COMPLETED;
				$this->attributes["hspc_need_check_status_only"] = self::ACTION_STATUS_WAITING == $this->commandStatus ? 1 : 0;
				$this->setFlagsAndXML(1);

				return;
			}
			$domain = $this->input["domainSLD"].".".$this->input["domainTLD"];
			$period = $this->input["domainYears"];
			$nses = array();
			for ($i = 1; $i <= 4; $i++) {
				if (! isset($this->input["domainNameserver{$i}"]) || "" == $this->input["domainNameserver{$i}"]) {
					continue;
				}
				$ip = gethostbyname($this->input["domainNameserver{$i}"]);
				if (0 < preg_match("/\d+\.\d+\.\d+\.\d+/", $ip)) {
					$nses[] = array(
						"hostname" => $this->input["domainNameserver{$i}"],
						"ip" => $ip,
					);
				}
				$this->attributes["domainNameserver{$i}"] = $this->input["domainNameserver{$i}"];
			}
			$contacts = array();
			$contacts_extdata = array();
			$hspc_contact_types = $this->getHSPCContactTypes();
			foreach ($hspc_contact_types as $hspc_contact_type) {
				$contacts[$hspc_contact_type->type] = unserialize($this->decryptData($this->input["hspc_{$hspc_contact_type->type}_cnt"]));
				$contacts_extdata[$hspc_contact_type->type] = unserialize($this->decryptData($this->input["hspc_{$hspc_contact_type->type}_cnt_ext"]));
			}
			$domain_extdata = unserialize($this->decryptData($this->input["hspc_dom_ext"]));

			$hspc_result = HSPCDomainManagerProcess::registerDomain(
				$this->subModuleName,
				$this->getPBAConfig(),
				$domain,
				$period,
				$nses,
				$contacts,
				$contacts_extdata,
				$domain_extdata
			);
			if (1 != $hspc_result->data->plugin_result->is_success) {
				throw new PBARegistrarError($hspc_result->data->plugin_result->message);
			}
			foreach ($hspc_contact_types as $hspc_contact_type) {
				$type = $hspc_contact_type->type;
				$this->attributes["hspc_{$type}_cnt"] = $this->encryptData(serialize($hspc_result->data->contacts->$type));
				$this->attributes["hspc_{$type}_cnt_ext"] = $this->encryptData(serialize($hspc_result->data->contacts_extdata->$type));
			}
			$this->attributes["hspc_dom_ext"] = $this->encryptData(serialize($hspc->result->data->domain_extdata));
			$this->attributes["hspc_domain_action"] = "register";

			if ("registering" == $domain_status = $hspc_result->data->plugin_result->domain_status) {
				try {
					$domain_status = $this->hspcGetDomainStatus($domain, "register");
				} catch (Exception $e) {
					$this->addError(blmsg("TRANS_REGISTRARERROR"), $e->getMessage());
				}
			}
			$this->commandStatus = "registering" == $domain_status
				? self::ACTION_STATUS_WAITING
				: self::ACTION_STATUS_COMPLETED;
			$this->attributes["hspc_need_check_status_only"] = self::ACTION_STATUS_WAITING == $this->commandStatus ? 1 : 0;
			$this->setFlagsAndXML(1);
		} catch (PBARegistrarError $e) {
			$this->addError(blmsg("TRANS_REGISTRARERROR"), $e->getMessage());
			$this->commandStatus = self::ACTION_STATUS_ERROR;
			$this->setFlagsAndXML(0);
		}
	}

	public function isDone()
	{
		if (self::ACTION_STATUS_WAITING == $this->commandStatus) {
			return false;
		}
		return true;
	}

	public function getDomainDNS() {
		$this->arrayData = array("listDNS" => array());
		for ($i = 1; $i <= 4; $i++) {
			$this->arrayData["listDNS"][] = $this->input["domainNameserver{$i}"];
		}
		$this->commandStatus = self::ACTION_STATUS_COMPLETED;
		$this->setFlagsAndXML(1);
	}

	public function setDomainDNS()
	{
		require_once("HSPC/DomainManagerProcess.php");
		$this->attributes = array();

		try {
			$domain = $this->input["domainSLD"].".".$this->input["domainTLD"];
			$nses = array();
			$ns_number = 1;
			foreach ($this->input["domainNameserver"] as $ns) {
				if (4 < $ns_number) {
					break;
				}
				if ("" == $ns) {
					continue;
				}
				$ip = gethostbyname($ns);
				if (0 < preg_match("/\d+\.\d+\.\d+\.\d+/", $ip)) {
					$nses[] = array("hostname" => $ns, "ip" => $ip);
				}
				$this->attributes["domainNameserver{$ns_number}"] = $ns;
				++$ns_number;
			}
			$hspc_result = HSPCDomainManagerProcess::syncDomainNS(
				$this->subModuleName,
				$this->getPBAConfig(),
				$domain,
				$nses
			);
			if (1 != $hspc_result->data->plugin_result->is_success) {
				throw new PBARegistrarError($hspc_result->data->plugin_result->message);
			}
			$this->arrayData = array("success" => 1);
			$this->commandStatus = self::ACTION_STATUS_COMPLETED;
			$this->setFlagsAndXML(1);
		} catch (PBARegistrarError $e) {
			$this->arrayData = array("success" => 0);
			$this->addError(blmsg("TRANS_REGISTRARERROR"), $e->getMessage());
			$this->commandStatus = self::ACTION_STATUS_ERROR;
			$this->setFlagsAndXML(0);
		}
	}

	private function hspcGetDomainStatus($domain, $action) {
		require_once("HSPC/DomainManagerProcess.php");
		$hspc_result = HSPCDomainManagerProcess::getDomainStatus(
			$this->subModuleName,
			$this->getPBAConfig(),
			$domain,
			$action
		);
		if (1 != $hspc_result->data->plugin_result->is_success) {
			throw new PBARegistrarError($hspc_result->data->plugin_result->message);
		}
		return $hspc_result->data->plugin_result->domain_status;
	}

	private function getPBAConfig()
	{
		return FALSE === ($config = unserialize($this->decryptData($this->registrar->ConfigParams["moduleConfig"]->value))) ? NULL : $config;
	}

	private function collectPBAConfig()
	{
		require_once("HSPC/DomainManagerProcess.php");
		$hspc_result = HSPCDomainManagerProcess::getCollectedConfigData($this->subModuleName, $this->getPBAConfig());
		return array(
			"data" => $this->encryptData(serialize($hspc_result->data->plugin_data)),
			"error" => $hspc_result->data->user_error,
		);
	}

	private function getHSPCContactTypes()
	{
		require_once("HSPC/DomainManagerProcess.php");
		return HSPCDomainManagerProcess::getContactTypes($this->subModuleName)->data;
	}

	private function encryptData($value)
	{
		return $this->getCryptor()->encrypt($value);
	}

	private function decryptData($value)
	{
		return $this->getCryptor()->decrypt($value);
	}

	private function getCryptor()
	{
		if (! is_null($this->cryptor))
		{
			$this->cryptor->errors = array();
			return $this->cryptor;
		}

		require_once("lib-tk/include/crypt/crypt.php");
		return $this->cryptor = new Crypt($this->cryptor_secret_key);
	}

	private function setFlagsAndXML($hasExecutedSuccessfully = 0)
	{
		$this->hasExecutedSuccessfully = $hasExecutedSuccessfully;
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
	}

	private $registrar;
	private $cryptor;
	private $avail_status_map = array(
		"avail"			=> self::DOMAIN_AVAIL_STATUS_AVAIL,
		"not-avail"		=> self::DOMAIN_AVAIL_STATUS_NOT_AVAIL,
		"cant-check"	=> self::DOMAIN_AVAIL_STATUS_CANT_CHECK,
	);
	private $hspc_actions_map = array(
		BillingRegistrar::DOMAIN_ACTION_REGISTER => "register",
		BillingRegistrar::DOMAIN_ACTION_TRANSFER => "transfer",
	);
	private $hspc_contacts = array();
	private $hspc_contacts_extdata = array();
	private $hspc_domain_extdata;
}
