<?php

require_once("lib-mbapi/include/modules/registrar/registrar.php");

$className = "Registercom";
$GLOBALS["moduleInfo"][$className] = array(
    "type" => "registrar",
    "status" => Registrar::STATUS_STABLE,
    "version" => "1.0.0",
    "displayName" => "Register.com",
    "author" => "Parallels",
	"file" => __FILE__,
	"countryCodes" => array("US"),
);

class Registercom extends Registrar
{
	/**
	 * Returns the module display for the Registercom module. This includes version number and display name.
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the display info for the module.
	 */
    function getModuleInfo() {
        return $GLOBALS["moduleInfo"]["Registercom"];
    }

	/**
	 * Construct a new Registercom object. This function merely sets up capabilities after calling the
	 * constructor for Registrar.
	 */
	function RegisterCom($input) {
		parent::Registrar($input);
		$this->capabilities[REGISTRAR_REGISTERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_TRANSFERDOMAIN] = 1;
		$this->capabilities[REGISTRAR_RENEWDOMAIN] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINEXPDATE] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINEXTENDEDINFO] = 1;
		$this->capabilities[REGISTRAR_CHECKDOMAINAVAILABILITY] = 0;
		$this->capabilities[REGISTRAR_CHECKDOMAINTRANSFERAVAILABILITY] = 1;

		//Domain Management options
		$this->capabilities[REGISTRAR_GETDOMAINCONTACTDATA] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINCONTACTDATA] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWTYPEOPTIONS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINRENEWTYPEOPTIONS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINREGISTRARLOCKOPTIONS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINREGISTRARLOCKOPTIONS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINACCESSPASSWORD] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINACCESSPASSWORD] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_SETDOMAINDNS] = 1;
		$this->capabilities[REGISTRAR_GETDOMAINRENEWDATE] = 1;
		$this->capabilities[REGISTRAR_LIVERENEW] = 1;

		$this->capabilities[REGISTRAR_CHECKACCOUNTBALANCE] = 1;
		$this->capabilities[REGISTRAR_GETTLDPRICING] = 1;
		$this->capabilities[REGISTRAR_GETTRANSFERSTATUS] = 1;
		$this->capabilities[REGISTRAR_GETTLDLIST] = 1;
		$this->capabilities[REGISTRAR_TLDS] = array(
			"be", "biz", "ca", "cc", "cn", "com.cn", "net.cn", "org.cn", "com", "br.com", "cn.com", "de.com", "eu.com",
			"hu.com", "jpn.com", "no.com", "qc.com", "ru.com", "sa.com", "se.com", "uk.com", "us.com", "uy.com",
			"web.com", "za.com", "de", "eu", "gs", "info", "jp", "ms", "net", "se.net", "co.nz", "org", "tc", "tv",
			"co.uk", "me.uk", "org.uk", "us", "vg", "ws");
		$this->fixXMLTags = array(
			"<0" => "<X0",
			"<1" => "<X1",
			"<2" => "<X2",
			"<3" => "<X3",
			"<4" => "<X4",
			"<5" => "<X5",
			"<6" => "<X6",
			"<7" => "<X7",
			"<8" => "<X8",
			"<9" => "<X9",
			"</0" => "</X0",
			"</1" => "</X1",
			"</2" => "</X2",
			"</3" => "</X3",
			"</4" => "</X4",
			"</5" => "</X5",
			"</6" => "</X6",
			"</7" => "</X7",
			"</8" => "</X8",
			"</9" => "</X9",
		);
	}

	/**
	 * This function merely sets the Registercom connection URL based on TEST mode status.
	 */
	function getURL() {
		global $CONF;
		$registercomTestMode = $CONF["domain"]["registrar"][strtolower($this->stringRep)][$this->getConfigID()]["moduleTestMode"];
		return ($registercomTestMode)
			? "https://partnertest.rcomexpress.com/interface.asp"
			: "https://partner.rcomexpress.com/interface.asp";
	}

	/*
	 * Call the registrar to check a domain's availability for registration.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function checkDomainAvailability() {
		global $CONF;

        // Create the Registercom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","Check");
	 	$c->addField("SLD",$this->getPUNYCode($this->input["domainSLD"]));

	 	if ((int) $this->input["checkAllTLDs"]) {
	 		// Registercom wildcard TLD
            $c->addField("TLD","*");
		} else if ((int) $this->input["checkNameSpinner"]) {
            // Registercom name spinning TLD
	 		$c->addField("TLD",$this->input["domainTLD"]);
			$c->addField("DomainSpinner",1);
			$c->addField("AllowDash",(int) $this->input["checkWithDashes"]);
            // registercom discontinued these options for now
            /*
			if ($this->input["checkKeyword1"]) {
				$c->addField("Word1",$this->input["checkKeyword1"]);
			}
			if ($this->input["checkKeyword2"]) {
				$c->addField("Word2",$this->input["checkKeyword2"]);
			}
			if ($this->input["checkKeyword3"]) {
				$c->addField("Word3",$this->input["checkKeyword3"]);
			}
	 		$c->addField("StartPosition",0);
            */
		} else {
            // Registercom single TLD
	 		$c->addField("TLD",$this->input["domainTLD"]);
		}

	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::checkDomainAvailability() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();

		$CONF["debug"]->add
		(
			"Registercom::checkDomainAvailability() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
Registercom Returned parameters and values
***********************************************************************************************
DomainX 		If return includes multiple domains, the name of domain X
RRPCodeX 		Success codes
					210 indicates that the name is available at the Registry.
					211 indicates the domain is not available at the Registry.
RRPTextX 		Text which accompanies and describes the RRPCode.
					210 = available
					211 = not available.
AuctionDateX 	Scheduled closing date for this auction
AuctionIDX 		Auction ID number for this domain name, assigned by us
DomainCount 	The number of names checked.
SuggestedNameY 	Currently available suggested names.
					Y is 1 through Spincount passed to this command.
					Returned only if DomainSpinner=1.
com-available 	Returns whether .com is available. Value will be yes or no.
				Additional params Returned only if DomainSpinner=1.
					net-available,
					org-available,
					cc-available,
					tv-available.

originalSLD 	The original domain name being checked. Returned only if DomainSpinner=1.
originalTLD 	The original top-level name (extension) being checked. Returned only if DomainSpinner=1.
Command 		Name of command executed
ErrCount 		The number of errors if any occurred. If greater than 0 check the Err(1 to ErrCount) values.
Err(x) 			Error messages explaining the failure. These can be presented as is back to the client.
Done 			True indicates this entire response has reached you successfully.
**********************************************************************************************/

                // ---------------------------------------------------------------
			 	// set original domain values (default)
                // ---------------------------------------------------------------
			 	$this->arrayData["OriginalDomain"] = $this->input["domainSLD"].".".$this->input["domainTLD"];
 				// set original domain availability
				if ($data["interface-response"][0]["RRPCode"][0]==210) {
				 	$this->arrayData["OriginalDomainIsAvailable"] = 1;
				} else if ($data["interface-response"][0]["RRPCode"][0]==211) {
			 		$this->arrayData["OriginalDomainIsAvailable"] = 0;
				} else {
				 	$this->arrayData["OriginalDomainIsAvailable"] = 0;
				 	$this->arrayData["OriginalDomainAvailabilityUnknown"] = 1;
				}
				$this->arrayData["SuggestedDomainCount"] = 0;

                // ---------------------------------------------------------------
				// are we checking for all TLDs ?
                // ---------------------------------------------------------------
				if($this->input["checkAllTLDs"]) {
				 	// set original domain
				 	$this->arrayData["OriginalDomain"] =  $data["interface-response"][0]["Domain1"][0];
	 				// set original domain availability
 					if ($data["interface-response"][0]["RRPCode1"][0]==210) {
					 	$this->arrayData["OriginalDomainIsAvailable"] = 1;
					} else if ($data["interface-response"][0]["RRPCode1"][0]==211) {
				 		$this->arrayData["OriginalDomainIsAvailable"] = 0;
					} else {
					 	$this->arrayData["OriginalDomainIsAvailable"] = 0;
					 	$this->arrayData["OriginalDomainAvailabilityUnknown"] = 1;
					}
					$c = $data["interface-response"][0]["DomainCount"][0];
                    for($i=2;$i<=$c;$i++){ // STARTS AT 2, NOT 0 !!
                    	$arrayElement = $i - 2;
						$data["interface-response"][0]["SuggestedNames"][0]["SuggestedName$arrayElement"] =
							$data["interface-response"][0]["Domain$i"][0];
	 					if ($data["interface-response"][0]["RRPCode$i"][0]==210) {
						 	$data["interface-response"][0]["SuggestedNames"][0]["DomainIsAvailable$arrayElement"] = 1;
						} else if ($data["interface-response"][0]["RRPCode$i"][0]==211) {
						 	$data["interface-response"][0]["SuggestedNames"][0]["DomainIsAvailable$arrayElement"] = 0;
						} else {
						 	$data["interface-response"][0]["SuggestedNames"][0]["DomainIsAvailable$arrayElement"] = 0;
						 	$data["interface-response"][0]["SuggestedNames"][0]["DomainAvailabilityUnknown$arrayElement"] = 0;
						}
					}
					// Set the number of domains spun
					$this->arrayData["SuggestedDomainCount"] = $c - 1;
					// Send back the domains spun
					$this->arrayData["SuggestedDomains"] = $data["interface-response"][0]["SuggestedNames"][0];
				}

                // ---------------------------------------------------------------
				// are we domain spinning ?
                // ---------------------------------------------------------------
				if($this->input["checkNameSpinner"]) {
				 	// set original domain
				 	$this->arrayData["OriginalDomain"] = $data["interface-response"][0]["originalSLD"][0].".".$data["interface-response"][0]["originalTLD"][0];
	 				// set original domain availability
 					if ($data["interface-response"][0]["RRPCode"][0]==210) {
					 	$this->arrayData["OriginalDomainIsAvailable"] = 1;
					} else if ($data["interface-response"][0]["RRPCode"][0]==211) {
				 		$this->arrayData["OriginalDomainIsAvailable"] = 0;
					} else {
					 	$this->arrayData["OriginalDomainIsAvailable"] = 0;
					 	$this->arrayData["OriginalDomainAvailabilityUnknown"] = 1;
					}
					// hack return XML so that the MBAPI can parse it correctly
					$c = count($data["interface-response"][0]["SuggestedNames"][0]["SuggestedName"]);
                    for($i=0;$i<$c;$i++){
						$data["interface-response"][0]["SuggestedNames"][0]["SuggestedName$i"] =
							$data["interface-response"][0]["SuggestedNames"][0]["SuggestedName"][$i];
						$data["interface-response"][0]["SuggestedNames"][0]["DomainIsAvailable$i"] = 1;
					}
					unset($data["interface-response"][0]["SuggestedNames"][0]["SuggestedName"]);
					// Set the number of domains spun
					$this->arrayData["SuggestedDomainCount"] = $c;
					// Send back the domains spun
					$this->arrayData["SuggestedDomains"] = $data["interface-response"][0]["SuggestedNames"][0];
				}

			 	$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]);
				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {
                // Log errors
				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("Registercom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/**
	 * Call the registrar to check a domain's availability for transfer.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function checkDomainTransferAvailability() {
		global $CONF;

	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD",
		);
		$postValues = $this->getPostValues($validParams);
		$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);

        // Create the Registercom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","StatusDomain");
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::checkDomainTransferAvailability() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$CONF["debug"]->add
		(
			"Registercom::checkDomainTransferAvailability() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			#if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
Registercom Returned parameters and values
***********************************************************************************************
DomainName 	Name being statused
Registrar 	Returns Known, Unknown, or None.
InAccount 	Returns 0 if not in an Registercom account, 1 if in your Registercom account or 2 if in another Registercom account.
OrderID 	ID number of the most recent order that included this domain.
Command 	Name of command executed
ErrCount 	The number of errors if any occurred. If greater than 0 check the Err(1 to ErrCount) values.
Err(x) 		Error messages explaining the failure. These can be presented as is back to the client.
Done 		True indicates this entire response has reached you successfully.
**********************************************************************************************/

			 	$this->arrayData["DomainName"] = $data["interface-response"][0]["DomainName"][0];
				$this->arrayData["DomainRegistrar"] = $data["interface-response"][0]["Registrar"][0];
				$this->arrayData["DomainInAccount"] = $data["interface-response"][0]["InAccount"][0];
				$this->arrayData["DomainLastOrderID"] = $data["interface-response"][0]["OrderID"][0];

			 	$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]);

				$this->hasExecutedSuccessfully = 1;

			#} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

			#	foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
            #    	$this->errors[] = array("Registercom Error (checkDomainTransferAvailability):", $v[0]);
			#		$this->numErrors ++;
			#	}

			#	$this->hasExecutedSuccessfully = 0;

			#}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * Call the registrar to register a new domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function registerDomain() {
	 	global $CONF;
	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD","NumYears","IDNCode","UnLockRegistrar","DomainPassword",
			"UseDNS","NS1","NS2","NS3","NS4",
		);

        $postValues = $this->getPostValues($validParams);
		// Registercom rule: Either UseDNS or NSX is required, not both
		if ($postValues["UseDNS"]=="default") {
			unset($postValues["NS1"],$postValues["NS2"],$postValues["NS3"],$postValues["NS4"]);
		}elseif(!$postValues["NS1"]&&!$postValues["NS2"]&&!$postValues["NS3"]&&!$postValues["NS4"]){
			$postValues["UseDNS"]="default";
		}

		if($postValues["IDNCode"] && $postValues["IDNCode"] != "eng"){
			$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);
		}

	 	// define the contact types we are allowed to add for a new domain
		$validContactTypes = array("registrant", "tech", "admin", "auxbilling");

		// remove the excess [0] elements to make it easier to manage the data below
		$newContactData = collapseArrayElements($this->input["domainContactData"]);

		// If it's the order form, all we pass is the registrant info
		if(!count($newContactData)){
			$newContactData["registrant"] = collapseArrayElements($this->input);
		}

		$map = $this->getAttributesMap();

		// each contact param is prepended with the contactType
		if(is_array($newContactData)) {
			foreach($newContactData as $contactType => $contactData) {
				// we can expect up to 4 contact types ONLY: registrant, tech, admin, auxbilling
				if (in_array($contactType,$validContactTypes)) {
					// this command only accepts the following user input
					$validContactParams = array
					(
						$contactType."FirstName", $contactType."LastName", $contactType."JobTitle", $contactType."OrganizationName",
						$contactType."Address1", $contactType."Address2", $contactType."City", $contactType."StateProvinceChoice",
						$contactType."StateProvince", $contactType."PostalCode", $contactType."Country",
						$contactType."Phone", $contactType."PhoneExt", $contactType."Fax", $contactType."EmailAddress",
					);
					// Loop the map and pull the needed fields for THIS contact type only
					foreach($map as $k => $v) {
						// if this element has a value it's part of this registrar's API, but only add it if it's set
						if($v && strlen($newContactData[$contactType][$k])>0 && in_array($k,$validContactParams)) {
							$postValues[$v] = $newContactData[$contactType][$k];
						}
					}

				// Registercom rule: $contactType.JobTitle Required if param $contactType.OrganizationName is used
					if ($postValues[$contactType."OrganizationName"] && !$postValues[$contactType."JobTitle"]) {
						$postValues[$contactType."JobTitle"] = "None";
					}

				// Registercom rule: $contactType.Fax Required if param $contactType.OrganizationName is used
					if ($postValues[$contactType."OrganizationName"] && !$postValues[$contactType."Fax"]) {
						$postValues[$contactType."Fax"] = $postValues[$contactType."Phone"];
					}

				}
			} // NEXT contactTYpe
		}

		//Moved up because these need to be set before throwing errors in their absence.
		//Extra items to pass if set
		$extraSettings = array(
			"us_nexus" => "us_nexus",
			"global_cc_us" => "global_cc_us",
			"us_purpose" => "us_purpose",
			"AU_reg_name" => "AU_reg_name",
			"AU_reg_id" => "AU_reg_id",
			"AU_reg_id_type" => "AU_reg_id_type",
			"AU_elig_name" => "AU_elig_name",
			"AU_elig_id" => "AU_elig_id",
			"AU_reg_elig_id_type" => "AU_reg_elig_id_type",
			"AU_reg_elig_type" => "AU_reg_elig_type",
			"AU_reg_elig_reasion" => "AU_reg_elig_reasion",
			"uk_legal_type" => "uk_legal_type",
			"uk_reg_co_no" => "uk_reg_co_no",
			"registered_for" => "registered_for",
			"uk_reg_opt_out" => "uk_reg_opt_out",
			"eu_agreedelete" => "eu_agreedelete",
			"eu_adr_lang" => "eu_adr_lang",
			"cira_legal_type" => "cira_legal_type",
			"cira-registrant" => "cira-registrant",
			"cira-registrant-desc" => "cira-registrant-desc",
			"cira-trademark-no" => "cira-trademark-no",
			"cira-org-registered-in" => "cira-org-registered-in");

		foreach($extraSettings as $key => $value) {
			if($this->input[$value]) {
				$postValues[$value] = $this->input[$value];
			}
		}

		// Apply rules for ccTLD
		// I got the data needed to apply the following rules by running
		// include/system/scripts/getTLDEnomExtAttribs.php

		// -------------------------------------------------------------------
		// .us rules
		// -------------------------------------------------------------------
		// Make sure Nexus information is set for .us domains (REQUIRED)
		// Valid nexus categories
		$nexusCats = array
		(
			"C11", // a US Citizen - A natural person who is a US Citizen
			"C12", // a Permanent Resident - A natural person who is a Permanent Resident
			"C21", // a Business Entity
			"C31", // a Foreign Entity
		);
		// Valid nexus purposes
		$nexusPurpose = array
		(
			"P1", // for profit - Business use for profit
			"P2", // for non-profit - Non-profit business, club, association, religious organization, etc.
		);
		if(eregi("us|kids\.us", $this->input["domainTLD"])) {
			if(!$this->input["us_nexus"][0]) {
				$this->addError(TRANS_REGISTRARERROR, "Missing required Nexus category for .us domain.");
			} else if(array_search($this->input["us_nexus"][0], $nexusCats) === false) {
				$this->addError(TRANS_REGISTRARERROR, "Invalid Nexus category provided for .us domain.");
			} else {
				$postValues[$map["us_nexus"]] = $this->input["us_nexus"][0];
			}
			if(!$this->input["us_purpose"][0]) {
				$this->addError(TRANS_REGISTRARERROR, "Missing required Nexus purpose for .us domain.");
			} else if(array_search($this->input["domainNexusPurpose"][0], $nexusCats) === false) {
				$this->addError(TRANS_REGISTRARERROR, "Invalid Nexus purpose provided for .us domain.");
			} else {
				$postValues[$map["us_purpose"]] = $this->input["us_purpose"][0];
			}

		}

		// ------------------------------------------------------------------
		// .eu rules
		// pass the eu_agreedelete and eu_adr_lan
		// ------------------------------------------------------------------
		if(eregi("eu", $this->input["domainTLD"])) {
		 	if (!$postValues["eu_agreedelete"]) {
				$this->addError(TRANS_REGISTRARERROR, "Missing required EU Agree to delete rules.");
			}
		 	if (!$postValues["eu_adr_lang"]) {
				$this->addError(TRANS_REGISTRARERROR, "Missing required EU Language.");
			}
		}
		// -------------------------------------------------------------------
		// .ca rules
		// -------------------------------------------------------------------
		// Make sure CIRA information is set for .ca domains (REQUIRED)
		// All we need here is to make sure CCO is set
		if(eregi("ca", $this->input["domainTLD"])) {
		 	if (!$postValues["cira-registrant"]) {
				$this->addError(TRANS_REGISTRARERROR, "Missing required CIRA corporation name.");
			} else {
				$postValues["cira_legal_type"] = $postValues["cira-org-registered-in"] = $postValues["registrantOrganizationName"];
				$postValues["registrantStateProvince"] = $this->input["registrantState"];
			}
		}

		// -------------------------------------------------------------------
		// .x.uk rules
		// -------------------------------------------------------------------
		// Check who this domain is being registered for if it's a .co.uk or .org.uk domain
		// (person or company) (REQUIRED)
		// Also check the type of individual registering this domain (once again required for
		// .co.uk and .org.uk domains) (REQUIRED)
		$validIndividualTypes = array("individual","non-uk-individual","uk-limited-company");
		if(eregi("co\.uk|org\.uk", $this->input["domainTLD"])) {
			if(!$postValues["registrantOrganizationName"] && !$postValues["registrantFirstName"] && !$postValues["registrantLastName"]) {
				$this->addError(TRANS_REGISTRARERROR, "Missing specification of who this domain is being registered for (required for .co.uk and .org.uk domains).");
			} else if($domain["registrantOrganizationName"]) {
				$postValues["registered_for"] = $postValues["registrantOrganizationName"];
			} else if($postValues["registrantFirstName"] && $postValues["registrantLastName"]) {
				$postValues["registered_for"] = $postValues["registrantFirstName"]." ".$domain["registrantLastName"];
			}
			if(!$postValues["registrantType"]) {
				$this->addError(TRANS_REGISTRARERROR, "Missing registrant type (individual, non-uk, or uk-limited-company).");
			} else if(array_search($postValues["registrantType"], $validIndividualTypes) === false) {
				$this->addError(TRANS_REGISTRARERROR, "Invalid registrant type (individual, non-uk-individual, or uk-limited-company only).");
			} else if($domain["registrantType"] == "individual") {
				$postValues["uk_legal_type"] = $postValues["uk_reg_co_no"] = "IND";
			} else if($domain["registrantType"] == "non-uk-individual") {
				$postValues["uk_legal_type"] = $postValues["uk_reg_co_no"] = "Non-UK";
			} else if($domain["registrantType"] == "uk-limited-company") {
				$postValues["uk_legal_type"] = $postValues["uk_reg_co_no"] = "LTD";
			}
		}

		// -------------------------------------------------------------------
		// .de rules
		// -------------------------------------------------------------------
		// Make sure the owner put in an address in Germany (REQUIRED)
		if(eregi("de",$this->input["domainTLD"]) && !eregi("de",$postValues["registrantCountry"])) {
			$this->addError(TRANS_REGISTRARERROR, "The registrant does not have a German address, and is trying to register a domain with the \"de\" TLD.");
		} else {
			$postValues["confirmaddress"] = "DE";
		}

		$validPrefectures = $this->getJapanesePrefectures();

		if(eregi("jp", $this->input["domainTLD"]) && $validPrefectures[$postValues["jp_prefecture"]]) {
			$postValues[$map["domainPrefecture"]] = $validPrefectures[$postValues["jp_prefecture"]];
		}

        // Create the Registercom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","Purchase");
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::registerDomain() ~ \$c->toString xyz",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);
##echo "<pre>".print_r($c,1);
		// is this needed here?
		$this->connectionInfo =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$this->arrayData["domainRemotePurchaseID"] = $data["interface-response"]["OrderID"];
		$CONF["debug"]->add
		(
			"Registercom::getDomainContactData() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
Registercom Returned parameters and values
***********************************************************************************************
OrderID 	Order number if successful. We recommend that you store this number for future use.
RRPCode 	Success code. Only a 200 indicates success.
RRPText 	Text which accompanies and describes the RRPCode value.
Command 	Name of command executed
ErrCount 	The number of errors if any occurred. If greater than 0 check the Err(1 to ErrCount) values.
Err(x) 		Error messages explaining the failure. These can be presented as is back to the client.
Done 		True indicates this entire response has reached you successfully.
**********************************************************************************************/

			 	$this->arrayData["PurchaseOrderID"] = $data["interface-response"][0]["OrderID"][0];
			 	if ($data["interface-response"][0]["RRPCode"][0]==200) {
			 		$this->arrayData["PurchaseIsSuccess"] = 1;
				} else {
			 		$this->arrayData["PurchaseIsSuccess"] = 0;
				}
				//$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]);
				$this->arrayData = collapseArrayElements($data["interface-response"][0]);

				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("Registercom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;


	}

	/**
	 * Call the registrar to transfer an existing domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function transferDomain() {
	 	global $CONF;

	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD","NumYears","IDNCode","UnLockRegistrar","DomainPassword",
			"UseDNS","NS1","NS2","NS3","NS4",
		);
         $postValues = $this->getPostValues($validParams);
		// Registercom rule: Either UseDNS or NSX is required, not both
		if ($postValues["UseDNS"]=="default") {
			unset($postValues["NS1"],$postValues["NS2"],$postValues["NS3"],$postValues["NS4"]);
		}elseif(!$postValues["NS1"]&&!$postValues["NS2"]&&!$postValues["NS3"]&&!$postValues["NS4"]){
			$postValues["UseDNS"]="default";
		}

		if($postValues["IDNCode"] && $postValues["IDNCode"] != "eng"){
			$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);
		}

	 	// define the contact types we are allowed to add for a new domain
		$validContactTypes = array("registrant", "tech", "admin", "auxbilling");

		// remove the excess [0] elements to make it easier to manage the data below
		$newContactData = collapseArrayElements($this->input["domainContactData"]);

		// If it's the order form, all we pass is the registrant info
		if(!count($newContactData)){
			$newContactData["registrant"] = collapseArrayElements($this->input);
		}

		$map = $this->getAttributesMap();

		// each contact param is prepended with the contactType
		foreach($newContactData as $contactType => $contactData) {
		 	// we can expect up to 4 contact types ONLY: registrant, tech, admin, auxbilling
			if (in_array($contactType,$validContactTypes)) {
			 	// this command only accepts the following user input
				$validContactParams = array
				(
			  		$contactType."FirstName", $contactType."LastName", $contactType."JobTitle", $contactType."OrganizationName",
			  		$contactType."Address1", $contactType."Address2", $contactType."City", $contactType."StateProvinceChoice",
		            $contactType."StateProvince", $contactType."PostalCode", $contactType."Country",
		            $contactType."Phone", $contactType."PhoneExt", $contactType."Fax", $contactType."EmailAddress",
				);
				// Loop the map and pull the needed fields for THIS contact type only
				foreach($map as $k => $v) {
				 	// if this element has a value it's part of this registrar's API, but only add it if it's set
					 if($v && strlen($newContactData[$contactType][$k])>0 && in_array($k,$validContactParams)) {
						$postValues[$v] = $newContactData[$contactType][$k];
					}
				}

				// Registercom rule: $contactType.JobTitle Required if param $contactType.OrganizationName is used
				if ($postValues[$contactType."OrganizationName"] && !$postValues[$contactType."JobTitle"]) {
					$postValues[$contactType."JobTitle"] = "None";
				}

				// Registercom rule: $contactType.Fax Required if param $contactType.OrganizationName is used
				if ($postValues[$contactType."OrganizationName"] && !$postValues[$contactType."Fax"]) {
					$postValues[$contactType."Fax"] = $postValues[$contactType."Phone"];
				}

			}
		} // NEXT contactTYpe

		// Apply rules for ccTLD
		// I got the data needed to apply the following rules by running
		// include/system/scripts/getTLDEnomExtAttribs.php

		// -------------------------------------------------------------------
		// .us rules
		// -------------------------------------------------------------------
		// Make sure Nexus information is set for .us domains (REQUIRED)
		// Valid nexus categories
		$nexusCats = array
		(
			"C11", // a US Citizen - A natural person who is a US Citizen
			"C12", // a Permanent Resident - A natural person who is a Permanent Resident
			"C21", // a Business Entity
			"C31", // a Foreign Entity
		);
		// Valid nexus purposes
		$nexusPurpose = array
		(
			"P1", // for profit - Business use for profit
			"P2", // for non-profit - Non-profit business, club, association, religious organization, etc.
		);
		if(eregi("us|kids\.us", $this->input["domainTLD"])) {
			if(!$this->input["domainNexusCategory"][0]) {
				$this->addError(TRANS_REGISTRARERROR, "Missing required Nexus category for .us domain.");
			} else if(array_search($this->input["domainNexusCategory"][0], $nexusCats) === false) {
				$this->addError(TRANS_REGISTRARERROR, "Invalid Nexus category provided for .us domain.");
			} else {
				$postValues[$map["domainNexusCategory"]] = $this->input["domainNexusCategory"][0];
			}
			if(!$this->input["domainNexusPurpose"][0]) {
				$this->addError(TRANS_REGISTRARERROR, "Missing required Nexus purpose for .us domain.");
			} else if(array_search($this->input["domainNexusPurpose"][0], $nexusCats) === false) {
				$this->addError(TRANS_REGISTRARERROR, "Invalid Nexus purpose provided for .us domain.");
			} else {
				$postValues[$map["domainNexusPurpose"]] = $this->input["domainNexusPurpose"][0];
			}

		}

		// -------------------------------------------------------------------
		// .ca rules
		// -------------------------------------------------------------------
		// Make sure CIRA information is set for .ca domains (REQUIRED)
		// All we need here is to make sure CCO is set
		if(eregi("ca", $this->input["domainTLD"])) {
		 	if (!$postValues["registrantOrganizationName"]) {
				$this->addError(TRANS_REGISTRARERROR, "Missing required CIRA corporation name.");
			} else {
				$postValues["cira_legal_type"] = $postValues["cira-org-registered-in"] = $postValues["registrantOrganizationName"];
			}
		}

		// -------------------------------------------------------------------
		// .x.uk rules
		// -------------------------------------------------------------------
		// Check who this domain is being registered for if it's a .co.uk or .org.uk domain
		// (person or company) (REQUIRED)
		// Also check the type of individual registering this domain (once again required for
		// .co.uk and .org.uk domains) (REQUIRED)
		$validIndividualTypes = array("individual","non-uk-individual","uk-limited-company");
		if(eregi("co\.uk|org\.uk", $this->input["domainTLD"])) {
			if(!$postValues["registrantOrganizationName"] && !$postValues["registrantFirstName"] && !$postValues["registrantLastName"]) {
				$this->addError(TRANS_REGISTRARERROR, "Missing specification of who this domain is being registered for (required for .co.uk and .org.uk domains).");
			} else if($domain["registrantOrganizationName"]) {
				$postValues["registered_for"] = $postValues["registrantOrganizationName"];
			} else if($postValues["registrantFirstName"] && $postValues["registrantLastName"]) {
				$postValues["registered_for"] = $postValues["registrantFirstName"]." ".$domain["registrantLastName"];
			}
			if(!$postValues["registrantType"]) {
				$this->addError(TRANS_REGISTRARERROR, "Missing registrant type (individual, non-uk, or uk-limited-company).");
			} else if(array_search($postValues["registrantType"], $validIndividualTypes) === false) {
				$this->addError(TRANS_REGISTRARERROR, "Invalid registrant type (individual, non-uk-individual, or uk-limited-company only).");
			} else if($domain["registrantType"] == "individual") {
				$postValues["uk_legal_type"] = $postValues["uk_reg_co_no"] = "IND";
			} else if($domain["registrantType"] == "non-uk-individual") {
				$postValues["uk_legal_type"] = $postValues["uk_reg_co_no"] = "Non-UK";
			} else if($domain["registrantType"] == "uk-limited-company") {
				$postValues["uk_legal_type"] = $postValues["uk_reg_co_no"] = "LTD";
			}
		}

		// -------------------------------------------------------------------
		// .de rules
		// -------------------------------------------------------------------
		// Make sure the owner put in an address in Germany (REQUIRED)
		if(eregi("de",$this->input["domainTLD"]) && !eregi("de",$postValues["registrantCountry"])) {
			$this->addError(TRANS_REGISTRARERROR, "The registrant does not have a German address, and is trying to register a domain with the \"de\" TLD.");
		} else {
			$postValues["confirmaddress"] = "DE";
		}

		// -------------------------------------------------------------------
		// .jp rules
		// -------------------------------------------------------------------
		// Check that a prefecture is set if a .jp domain is being registered (NOT REQUIRED)
		// Change these to Japanese characters or translation constants
		$validPrefectures = $this->getJapanesePrefectures();

		if(eregi("jp", $this->input["domainTLD"]) && $validPrefectures[$postValues["domainPrefecture"]]) {
			$postValues[$map["domainPrefecture"]] = $validPrefectures[$postValues["domainPrefecture"]];
		}

        // Create the Registercom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","TP_CreateOrder");
	 	$c->addField("DomainCount",1);
	 	$c->addField("OrderType","Autoverification"); // Fax
		// Required for EPP TLDs Current (?losing?) registrar?s authorization key

	 	$c->addField("AuthInfo",$this->input['domainTransferKey']);
        // UseContacts Optional Set =1 to transfer existing WhoIs contacts when the transfer is complete
	 	// $c->addField("UseContacts",1);
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::registerDomain() ~ \$c->toString xyz",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);
##echo "<pre>".print_r($c,1);
		// is this needed here?
		$this->connectionInfo =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$this->arrayData["domainRemotePurchaseID"] = $data["interface-response"]["TransferOrderID"];
		$CONF["debug"]->add
		(
			"Registercom::getDomainContactData() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
Registercom Returned parameters and values
***********************************************************************************************
TransferOrderID 	Transfer order number.
Command 			Name of command executed
ErrCount 			The number of errors if any occurred. If greater than 0 check the Err(1 to ErrCount) values.
Err(x) 				Error messages explaining the failure. These can be presented as is back to the client.
Done 				True indicates this entire response has reached you successfully.
**********************************************************************************************/

			 	$this->arrayData["TransferOrderID"] = $data["interface-response"][0]["TransferOrderID"];
				$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]);

				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("Registercom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for renewing a domain.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function renewDomain() {
		global $CONF;

	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD","NumYears"
		);
		$postValues = $this->getPostValues($validParams);
		$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);

        // Create the Registercom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","Extend");
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::renewDomain() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$CONF["debug"]->add
		(
			"Registercom::renewDomain() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
Registercom Returned parameters and values
***********************************************************************************************
RegistrarHold 	The enabled status of a domain. If RegistrarHold is True, the domain is disabled.
OrderID 		Order number if successful. We recommend that you store this value for future use.
Extension 		Returns Successful, otherwise this parameter isn't returned.
RRPCode 		Success code. Only a 200 indicates success.
RRPText 		Text which accompanies and describes the RRPCode.
Command 		Name of command executed
ErrCount 		The number of errors if any occurred. If greater than 0 check the Err(1 to ErrCount) values.
Err(x) 			Error messages explaining the failure. These can be presented as is back to the client.
Done 			True indicates this entire response has reached you successfully.
**********************************************************************************************/

			 	$this->arrayData["DomainRenewOrderID"] = $data["interface-response"][0]["OrderID"][0];
				$this->arrayData["DomainRenewMessage"] = $data["interface-response"][0]["RRPText"][0];

			 	if ($data["interface-response"][0]["RRPCode"][0]==200) {
			 		$this->arrayData["DomainRenewSuccess"] = 1;
				} else {
					$this->arrayData["DomainRenewSuccess"] = 0;
				}

			 	$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]);

				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("Registercom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * Call the registrar to fetch a domain's contact data
	 *
	 * @return  string  XML containing contact data for this domain
	 */
	function getDomainContactData() {
	 	global $CONF;

	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD"
		);
		$postValues = $this->getPostValues($validParams);
		$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);

        // Create the RCom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","GetContacts");
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::getDomainContactData() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();

		$CONF["debug"]->add
		(
			"Registercom::getDomainContactData() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

			 	$this->arrayData["DomainData"] = $data["interface-response"][0]["GetContacts"][0]["domainname"][0]["_attributes"];
			 	$this->arrayData["DomainData"]["domainname"] = $data["interface-response"][0]["GetContacts"][0]["domainname"][0][0];
				$this->arrayData["RegistrantContactData"] = collapseArrayElements($data["interface-response"][0]["GetContacts"][0]["Registrant"][0]);
				$this->arrayData["AuxBillingContactData"] = collapseArrayElements($data["interface-response"][0]["GetContacts"][0]["AuxBilling"][0]);
				$this->arrayData["TechContactData"] = collapseArrayElements($data["interface-response"][0]["GetContacts"][0]["Tech"][0]);
				$this->arrayData["AdminContactData"] = collapseArrayElements($data["interface-response"][0]["GetContacts"][0]["Admin"][0]);
				$this->arrayData["BillingContactData"] = collapseArrayElements($data["interface-response"][0]["GetContacts"][0]["Billing"][0]);

			 	unset
			 	(
			 		$data["interface-response"][0]["GetContacts"][0]["domainname"],
					$data["interface-response"][0]["GetContacts"][0]["Registrant"],
					$data["interface-response"][0]["GetContacts"][0]["AuxBilling"],
					$data["interface-response"][0]["GetContacts"][0]["Tech"],
					$data["interface-response"][0]["GetContacts"][0]["Admin"],
					$data["interface-response"][0]["GetContacts"][0]["Billing"]
			 	);

			 	$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]["GetContacts"][0]);

				/*
				foreach($this->arrayData as $key => $value) {
					if(is_array($value)) {
						// Drop the first letter of each index to lower case
						foreach($value as $k => $v) {
							$this->arrayData[$key][lcfirst($k)] = $v;
							unset($this->arrayData[$key][$k]);
						}
					} else {
						unset($this->arrayData["$key"]);
					}
				}
				*/

				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("RCom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	} // END getDomainContactData()

	/**
	 * Send the updated domain contact data to the registrar
	 *
	 * @return  string  XML result
	 */
	function setDomainContactData() {
	 	Global $CONF;

	 	// reset array so we can fill it in each loop
	 	$this->connectionInfo = array();

	 	// define the contact types we are allowed to change
		$validContactTypes = array("registrant", "tech", "admin", "auxBilling");

		// load the attributes first, we need to use them many times below
	  	$map = $this->getAttributesMap();

		// RCom will only let you change one type at a time so we need to run this command once for
		// each type specified in the input params
		$newContactData = collapseArrayElements($this->input["domainContactData"]);

		$numConnections = 0;
		foreach($newContactData as $contactType => $contactData) {
		 	// we can expect up to 4 contact types ONLY: registrant, tech, admin, auxbilling
			if (in_array($contactType,$validContactTypes)) {

			 	// this command only accepts the following user input
				$validContactParams = array
				(
			  		$contactType."FirstName", $contactType."LastName", $contactType."JobTitle", $contactType."OrganizationName",
			  		$contactType."Address1", $contactType."Address2", $contactType."City", $contactType."StateProvinceChoice",
		            $contactType."StateProvince", $contactType."PostalCode", $contactType."Country",
		            $contactType."Phone", $contactType."PhoneExt", $contactType."Fax", $contactType."EmailAddress",
				);

			    // Create the RCom curl object for THIS contact type only
			    $numConnections ++;
				$c = $this->getCurlObject();
			 	$c->addField("UID",$this->UID);
			 	$c->addField("PW",$this->PW);
			 	$c->addField("Command","Contacts");
			 	$c->addField("ContactType",$contactType);
			 	$c->addField("SLD",$this->getPUNYCode($this->input["domainSLD"]));
			 	$c->addField("TLD",$this->input["domainTLD"]);
///////////This line does not work
//change this to pull [registrant], [admin], ..etc items out of $this->input
			 	$c->addMultiFields($this->getPostValues($validContactParams, $contactType));
			 	$c->addField("ResponseType","XML");
				$c->setURL($this->getURL());
				$c->setFieldMethod(1);

				// run THIS contact command and return the results as a string
//				$rawResult = $c->toString();
				$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

				$CONF["debug"]->add
				(
					"MBAPI_ProcessRegistrarCommand::setDomainContactData($contactType) ~ \$c->toString",
					htmlentities($rawResult),
					__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
				);

				// is this needed here?
				$this->connectionInfo[] = $c->connectionInfo; // PUSH INTO AN ARRAY!

				// check for a cURL error
				## CHECK FOR ERRORS ????
				$this->getCurlErrors($c);

				// fetch the XML from the results
				$matches = array();
				preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

				if (!is_array($matches[0])) {
					// Need to die
				}

				// parse the raw XML to a usable array of data
				$x = new MBXML($matches[0]);
				$data = $x->toArray();
				$CONF["debug"]->add
				(
					"Registercom::setDomainContactData($contactType) ~ Cleaned Array",
					$data,
					__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
				);

				// Process the return value
				// Check the resultArray first
				if(is_array($data)){
					if($data["interface-response"][0]["ErrCount"][0]=="0") {
					 	$arrayData["response-".$numConnections] = collapseArrayElements($data["interface-response"][0]);
						$this->hasExecutedSuccessfully = 1;
					} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {
				 		foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
	                		$this->errors[] = array("RCom Error:", $v[0]);
							$this->numErrors ++;
						}
						$this->hasExecutedSuccessfully = 0;
					}
				}

			} else { // if (in_array($contactType,$validContactTypes)) {
        		$this->errors[] = array("RCom Error:", "Invalid ContactType \"$contactType\"");
				$this->numErrors ++;
				$this->hasExecutedSuccessfully = 0;
			}

		} // NEXT contactType

		$this->arrayData = $arrayData;

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	} // END setDomainContactData()

	/**
	 * This method contains the registrar specific code for fetching the domain auto-renewal options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getdomainRenewTypeOptions() {
		global $CONF;

	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD"
		);

		$postValues = $this->getPostValues($validParams);
		$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);

        // Create the Registercom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","GetRenew");
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::getdomainRenewTypeOptions() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$CONF["debug"]->add
		(
			"Registercom::getdomainRenewTypeOptions() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
RCom Returned parameters and values
***********************************************************************************************
RegistrarHold 		The enabled status of a domain. If RegistrarHold is True, the domain is disabled.
RenewName 			Returns 1 if this domain is set to auto-renew, 0 otherwise.
PakExist 			Returns 1 if there are POP paks associated with this domain, 0 otherwise.
AutoPakRenew 		Returns 1 if the POP paks for this domain are set to auto-renew, 0 otherwise.
EMailFwdExists 		Returns 1 if this domain subscribes to email forwarding, 0 otherwise.
EMailForwardRenew 	Returns 1 if the email forwarding for this domain is set to auto-renew, 0 otherwise.
URLFwdExists 		Returns 1 if this domain subscribes to URL forwarding, 0 otherwise.
URLForwardRenew 	Returns 1 if the URL forwarding for this domain is set to auto-renew, 0 otherwise.
Command 			Name of command executed
ErrCount 			The number of errors if any occurred. If greater than 0 check the Err(1 to ErrCount) values.
Err(x) 				Error messages explaining the failure. These can be presented as is back to the client.
Done 				True indicates this entire response has reached you successfully.
**********************************************************************************************/

				if ($data["interface-response"][0]["auto-renew"][0] || $data["interface-response"][0]["RenewName"][0]) {
			 		$this->arrayData["domainRenewType"] = 1;
				} else {
					$this->arrayData["domainRenewType"] = 0;
				}
			 	$this->arrayData["EmailForwardAutoRenew"] = $data["interface-response"][0]["EmailForwardRenew"][0];
			 	$this->arrayData["IDProtectAutoRenew"] = $data["interface-response"][0]["IDProtectRenew"][0];
			 	$this->arrayData["URLForwardAutoRenew"] = $data["interface-response"][0]["URLForwardRenew"][0];
			 	$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]);

				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("RCom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for setting a domain's auto-renewal options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setdomainRenewTypeOptions() {
		global $CONF;

	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD","RenewFlag"
		);
		$postValues = $this->getPostValues($validParams);
		$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);

        // Create the RCom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","SetRenew");
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::setdomainRenewTypeOptions() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$CONF["debug"]->add
		(
			"Registercom::setdomainRenewTypeOptions() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
RCom Returned parameters and values
***********************************************************************************************
RegistrarHold 	The enabled status of a domain. If RegistrarHold is True, the domain is disabled.
RenewName 		Current auto-renew setting (1 = on, 0 = off)
AutoPakRenew 	Current auto-renew setting for POP paks (True = on, False = off).
				This return parameter only appears if the AutoPakRenew is an input parameter.
Command 		Name of command executed
ErrCount 		The number of errors if any occurred. If greater than 0 check the Err(1 to ErrCount) values.
Err(x) 			Error messages explaining the failure. These can be presented as is back to the client.
Done 			True indicates this entire response has reached you successfully.
**********************************************************************************************/

			 	$this->arrayData["domainRenewType"] = $data["interface-response"][0]["RenewName"][0];
			 	$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]);

				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("RCom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for fetching the domain registrar-lock options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainRegistrarLockOptions() {
		global $CONF;

	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD"
		);

		$postValues = $this->getPostValues($validParams);
		$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);

        // Create the Registercom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","GetRegLock");
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::getDomainRegistrarLockOptions() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$CONF["debug"]->add
		(
			"Registercom::getDomainRegistrarLockOptions() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
RCom Returned parameters and values
***********************************************************************************************
RegistrarHold 	The enabled status of a domain. If RegistrarHold is True, the domain is disabled.
RegLock 		Returns 1 if locked, 0 otherwise if the responsetype is Text.
RRPCode 		Response code from NSI.
RRPText 		Response text from NSI.
Command 		Name of command executed
ErrCount 		The number of errors if any occurred. If greater than 0 check the Err(1 to ErrCount) values.
Err(x) 			Error messages explaining the failure. These can be presented as is back to the client.
IsLockable 		Indicates whether this TLD can use the Registrar Lock feature. Return values are True or False.
Done 			True indicates this entire response has reached you successfully.
**********************************************************************************************/

				$this->arrayData["DomainRegistrarLock"] =
					($data["interface-response"][0]["RegLock"][0] || $data["interface-response"][0]["reg-lock"][0]) ? 1 : 0 ;

			 	$this->arrayData["DomainIsLockable"] =
			 		($data["interface-response"][0]["IsLockable"][0] == "True") ? 1 : 0 ;

			 	$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]);

				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("RCom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for setting the domain registrar-lock options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setDomainRegistrarLockOptions() {
		global $CONF;

	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD","UnlockRegistrar"
		);

	  	$map = $this->getAttributesMap();
		// Loop the map and pull the needed fields
		$postValues = array();
		foreach($map as $k => $v) {
		 	// if this element has a value it's part of this registrar's API, but only add it if it's set
			if($v && strlen($this->input[$k])>0 && in_array($v,$validParams)) {
				if ($v == "UnlockRegistrar") {
				 	// RCom uses the inverse ...
					// Set the registrar unlock value to 0 to prevent unauthorized transfer of a domain name to another registrar.
					// Set the registrar unlock value to 1 to allow unrestricted transfer of a domain name.
					$this->input[$k] = ($this->input[$k]) ? 0 : 1 ;
				}
				$postValues[$v] = $this->input[$k];
			}
		}

		$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);

        // Create the Registercom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","SetRegLock");
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::setDomainRegistrarLockOptions() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$CONF["debug"]->add
		(
			"Registercom::setDomainRegistrarLockOptions() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
RCom Returned parameters and values
***********************************************************************************************
RegistrarHold 	The enabled status of a domain. If RegistrarHold is True, the domain is disabled.
RegistrarLock 	Current lock status if successful, (=Failed otherwise).
Command 		Name of command executed
ErrCount 		The number of errors if any occurred. If greater than 0 check the Err(1 to ErrCount) values.
Err(x) 			Error messages explaining the failure. These can be presented as is back to the client.
Done 			True indicates this entire response has reached you successfully.
**********************************************************************************************/

				$this->arrayData["DomainRegistrarLock"] =
					 (eregi("LOCK",$data["interface-response"][0]["RegistrarLock"][0])) ? 1 : 0 ;

			 	$this->arrayData["DomainIsLockable"] =
			 		($data["interface-response"][0]["IsLockable"][0] == "True") ? 1 : 0 ;

			 	$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]);

				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("RCom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for fetching the domain access password.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainAccessPassword() {
		global $CONF;

	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD"
		);
		$postValues = $this->getPostValues($validParams);
		$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);

        // Create the RCom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","GetPasswordBit");
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::getDomainAccessPassword() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$CONF["debug"]->add
		(
			"Registercom::getDomainAccessPassword() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
RCom Returned parameters and values
***********************************************************************************************
RegistrarHold 	The enabled status of a domain. If RegistrarHold is True, the domain is disabled.
PasswordSet 	Returns 1 if a password is set, 0 otherwise.
DomainPassword 	If PasswordSet=1, then a password is returned in this parameter.
Command 		Name of command executed
ErrCount 		The number of errors if any occurred. If greater than 0 check the Err(1 to ErrCount) values.
Err(x) 			Error messages explaining the failure. These can be presented as is back to the client.
IsLockable 		Indicates whether this TLD can use the Registrar Lock feature. Return values are True or False.
Done 			True indicates this entire response has reached you successfully.
**********************************************************************************************/

				$this->arrayData["DomainAccessPasswordSet"] =
					($data["interface-response"][0]["PasswordSet"][0] || $data["interface-response"][0]["password-set"][0]) ? 1 : 0 ;

			 	$this->arrayData["DomainAccessPassword"] =
			 		($this->arrayData["DomainAccessPasswordSet"]) ? $data["interface-response"][0]["DomainPassword"][0] : "" ;

			 	$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]);

				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("RCom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for setting the domain access password.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setDomainAccessPassword() {
		global $CONF;

	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD","DomainPassword"
		);

		$postValues = $this->getPostValues($validParams);
		$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);

        // Create the Registercom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","SetPassword");
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::setDomainAccessPassword() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$CONF["debug"]->add
		(
			"Registercom::setDomainAccessPassword() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
RCom Returned parameters and values
***********************************************************************************************
RegistrarHold 	The enabled status of a domain. If RegistrarHold is True, the domain is disabled.
Command 		Name of command executed
ErrCount 		The number of errors if any occurred. If greater than 0 check the Err(1 to ErrCount) values.
Err(x) 			Error messages explaining the failure. These can be presented as is back to the client.
Done 			True indicates this entire response has reached you successfully.
**********************************************************************************************/

			 	$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]);

				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("RCom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for fetching the domain expiration date.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainExpDate() {
		global $CONF;

	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD"
		);

		$postValues = $this->getPostValues($validParams);
		$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);

        // Create the Registercom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","GetDomainExp");
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::getDomainExpDate() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$CONF["debug"]->add
		(
			"Registercom::getDomainExpDate() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
RCom Returned parameters and values
***********************************************************************************************
RegistrarHold 	The enabled status of a domain. If RegistrarHold is True, the domain is disabled.
ExpirationDate 	Expiration date for the domain registration. (11/11/2007 3:40:47 PM)
Command 		Name of command executed
ErrCount 		The number of errors if any occurred. If greater than 0 check the Err(1 to ErrCount) values.
Err(x) 			Error messages explaining the failure. These can be presented as is back to the client.
IsLockable 		Indicates whether this TLD can use the Registrar Lock feature. Return values are True or False.
Done 			True indicates this entire response has reached you successfully.
**********************************************************************************************/

			 	$this->arrayData["DomainExpDate"] = $data["interface-response"][0]["ExpirationDate"][0];
			 	$this->arrayData["DomainExpTimestamp"] = strtotime($this->arrayData["DomainExpDate"]);
			 	$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]);

				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("RCom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/**
	 * This method contains the registrar specific code for fetching a domain's extended information.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainExtendedInfo() {
		global $CONF;

	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD"
		);

		$postValues = $this->getPostValues($validParams);
		$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);

        // Create the Registercom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","GetExtendInfo");
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::getDomainExtendedInfo() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$CONF["debug"]->add
		(
			"Registercom::getDomainExtendedInfo() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
RCom Returned parameters and values
***********************************************************************************************
RegistrarHold 		The enabled status of a domain. If RegistrarHold is True, the domain is disabled.
Expiration 			Current expiration date.
MaxExtension 		Maximum # of years that can be added.
CCAuthorized 		Credit card transaction successfully authorized.
Price 				Price for one-year extension for this type of TLD.
Balance 			Current balance in this account.
AvailableBalance 	Available balance in this account.
DefPeriod 			Default renewal period for domain name registrations
AllowDNS 			True allows domain name servers other than RCom?s
ShowPopups 			True shows popup menus
AutoRenew 			True automatically renews the domain registration 30 days before it expires
RegLock 			True requires the account holder?s permissiont to transfer the domain to another registrar
AutoPakRenew 		True automatically renews POP paks 30 days before they expire
UseDNS 				True uses RCom?s name servers
ResellerStatus 		Reseller status of this account
RenewalSetting 		0 Send no renewal reminder when domain expiration approaches
					1 Send renewal reminder by email
					2 Contact customer, renew registration, and charge account
RenewalBCC 			1 Send blind carbon copy of renewal email to reseller
					0 Do not send blind carbon copy to reseller
RenewalURLForward 	True automatically renews URL forwarding 30 days before the subscription expires
RenewalEmailForward True automatically renews email forwarding 30 days before the subscription expires
HostNameX 			Name of default host record
AddressX 			Address of default host record
RecordTypeX 		Default host record type
DefaultHostRecordOwn 	Indicates whether this domain is associated with a Web hosting account
AcceptTerms 		True has signed a credit card processing agreement with RCom
URL 				URL for reseller site, used in email notices
Command 			Name of command executed
ErrCount 			The number of errors if any occurred. If greater than 0 check the Err(1 to ErrCount) values.
Err(x) 				Error messages explaining the failure. These can be presented as is back to the client.
Done 				True indicates this entire response has reached you successfully.
**********************************************************************************************/

				// fix host records because the default return data makes for bad "XML2Array" parsing in MB
				/*
                <defaulthostrecords>
                	<hostrecord hostname="www" address="69.25.142.5" recordtype="A" />
                	<hostrecord hostname="@" address="69.25.142.5" recordtype="A" />
                	<hostrecord hostname="*" address="69.25.142.5" recordtype="A" />
                	<hostrecord hostname="" address="" recordtype="A" />
                	<hostrecord hostname="" address="" recordtype="A" />
                </defaulthostrecords>
				*/
				if (is_array($data["interface-response"][0]["CustomerPrefs"][0]["defaulthostrecords"][0]["hostrecord"])) {
					$c = count($data["interface-response"][0]["CustomerPrefs"][0]["defaulthostrecords"][0]["hostrecord"]);
                    for($i=0;$i<$c;$i++){
                     	$data["interface-response"][0]["CustomerPrefs"][0]["defaulthostrecords"][0]["hostrecord$i"] =
							$data["interface-response"][0]["CustomerPrefs"][0]["defaulthostrecords"][0]["hostrecord"][$i];
					}
					unset($data["interface-response"][0]["CustomerPrefs"][0]["defaulthostrecords"][0]["hostrecord"]);
				}

			 	$this->arrayData["DomainExpDate"] = $data["interface-response"][0]["Expiration"][0];
			 	$this->arrayData["DomainExpTimestamp"] = strtotime($this->arrayData["DomainExpDate"]);

			 	$this->arrayData["DomainRenewMaxYears"] = $data["interface-response"][0]["MaxExtension"][0];

			 	if ((int) $data["interface-response"][0]["CustomerPrefs"][0]["DefPeriod"][0]) {
					$this->arrayData["DomainRenewDefaultYears"] = $data["interface-response"][0]["CustomerPrefs"][0]["DefPeriod"][0];
			 	} else {
			 		$this->arrayData["DomainRenewDefaultYears"] = 1;
				}

			 	if ($data["interface-response"][0]["Price"][0]) {
					$this->arrayData["DomainRenewPrice"] = $data["interface-response"][0]["Price"][0];
			 	} else {
			 		$this->arrayData["DomainRenewPrice"] = 0;
				}

			 	$this->arrayData["OtherData"] = collapseArrayElements($data["interface-response"][0]);

				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("RCom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;

	}

	/*
	 * Call the registrar to check an account's balance.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function checkAccountBalance() {
		global $CONF;

        // Create the RCom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","GetBalance");
	 	$c->addField("SLD",$this->getPUNYCode($this->input["domainSLD"]));

	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$x = new MBXML($rawResult);
		$data = $x->toArray();
		$this->arrayData = collapseArrayElements($data["interface-response"]);

		$this->hasExecutedSuccessfully = 1;
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/*
	 * Call the registrar to check TLD pricing cost (cogs).
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getTLDPricing() {
		global $CONF;

        // Create the RCom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","GetBalance");
	 	$c->addField("SLD",$this->getPUNYCode($this->input["domainSLD"]));

	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$x = new MBXML($rawResult);
		$data = $x->toArray();
		$data = collapseArrayElements($data["interface-response"]);

		//Pull the tld names out
		foreach($data as $key => $value) {
			if(strpos($key,"Price")) {
				$tld[strtolower(str_replace("Price","",$key))] = $value;
			}
		}
		$this->arrayData = $tld;

		$this->hasExecutedSuccessfully = 1;
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/*
	 * Call the registrar to check the status of a transfer
	 *
	 * @return  string status of the transfer
	 */
	function getTransferStatus() {
		global $CONF;

		$transferorderid = $transferorderdetailid = $transferStatus = NULL;

        // //Get the Detail ID
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","TP_GetOrdersByDomain");
	 	$c->addField("SLD",$this->getPUNYCode($this->input["domainSLD"]));
	 	$c->addField("TLD",$this->input["domainTLD"]);

	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$x = new MBXML($rawResult);
		$data = $x->toArray();
		$data = collapseArrayElements($data["interface-response"]);

		if(is_array($data) && is_array($data["TransferOrder"][0])) {
			$transferorderid = $data["TransferOrder"][0]["transferorderid"];
		}

		if($transferorderid) {
			// Get the Status
			$c = $this->getCurlObject();
			$c->addField("UID",$this->UID);
			$c->addField("PW",$this->PW);
			$c->addField("Command","TP_GetOrder");
			$c->addField("transferorderid",$transferorderid);

			$c->addField("ResponseType","XML");
			$c->setURL($this->getURL());
			$c->setFieldMethod(1);

			// run the command and return the results as a string
//			$rawResult = $c->toString();
			$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

			$x = new MBXML($rawResult);
			$data = $x->toArray();
			$data = collapseArrayElements($data["interface-response"]);

			if(is_array($data) && is_array($data["transferorder"]["transferorderdetail"])) {
				$transferorderdetailid = $data["transferorder"]["transferorderdetail"]["transferorderdetailid"];
				$transferStatus = $data["transferorder"]["transferorderdetail"]["statusdesc"];
			}
		}

		$transferorderdetailid = NULL;
		if($transferorderdetailid) {
			// Get Everything this pulls all of the contact info, not needed right now
			$c = $this->getCurlObject();
			$c->addField("UID",$this->UID);
			$c->addField("PW",$this->PW);
			$c->addField("Command","TP_GetOrderDetail");
			$c->addField("transferorderdetailid",$transferorderdetailid);

			$c->addField("ResponseType","XML");
			$c->setURL($this->getURL());
			$c->setFieldMethod(1);

			// run the command and return the results as a string
//			$rawResult = $c->toString();
			$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

			$x = new MBXML($rawResult);
			$data = $x->toArray();
			$data = collapseArrayElements($data["interface-response"]);
		}

		$this->arrayData = $transferStatus;

		$this->hasExecutedSuccessfully = 1;
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}


	/*
	 * Call the registrar to get a list of TLDs
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	/*function getTLDList() {
		global $CONF;

        // Create the RCom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","GetTLDList");

	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$x = new MBXML($rawResult);
		$data = $x->toArray();
		$data = collapseArrayElements($data["interface-response"]);

		//Pull the tld names out
		foreach($data as $key => $value) {
			if(strpos($key,"Price")) {
				$tld[strtolower(str_replace("Price","",$key))] = $value;
			}
		}
		$this->arrayData = $tld;

		$this->hasExecutedSuccessfully = 1;
		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}*/

	/**
	 * This method contains the registrar specific code for fetching the domain registrar-lock options.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function getDomainDNS() {
		global $CONF;

		//Not sure if this list will vary by registrar, so this is an attempt at flexibility -AC
		$dnsOptions = array("A" => "A","URL" => "URL","MX" => "MX",
							"MXE" => "MXE","CNAME" => "CNAME",
							"FRAME" => "FRAME", "TXT" => "TXT");

	 	// this command only accepts the following user input
		$validParams = array
		(
			// domain information
			"SLD","TLD"
		);

		$postValues = $this->getPostValues($validParams);
		$postValues["SLD"] = $this->getPUNYCode($postValues["SLD"]);

        // Create the Registercom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","GetDNS");
	 	$c->addMultiFields($postValues);
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::getDomainRegistrarLockOptions() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$CONF["debug"]->add
		(
			"Registercom::getDomainRegistrarLockOptions() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
RCom Returned parameters and values: Their Documentation does not match the real return values
***********************************************************************************************
dns				listned n times with a value in each
UseDNS			default = using RCom, empty is custom
NSStatus		Yes = using registrar's name servers, empty is use custom
IsLockable 		Indicates whether this TLD can use the Registrar Lock feature. Return values are True or False.
Done 			True indicates this entire response has reached you successfully.
**********************************************************************************************/

				$this->arrayData["listDNS"] = $data["interface-response"][0]["dns"];

			 	$this->arrayData["UseDNS"] =
			 		($data["interface-response"][0]["UseDNS"][0] == "default") ? 1 : 0 ;

				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("RCom Error:", $v[0]);
					$this->numErrors ++;
				}

				$this->hasExecutedSuccessfully = 0;

			}
		}

		//Make a second call to get the Host info
        // Create the RCom curl object
		$c = $this->getCurlObject();
	 	$c->addField("UID",$this->UID);
	 	$c->addField("PW",$this->PW);
	 	$c->addField("Command","GetHosts");
	 	$c->addMultiFields($this->getPostValues($validParams));
	 	$c->addField("ResponseType","XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1);

		// run the command and return the results as a string
//		$rawResult = $c->toString();
		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::getDomainRegistrarLockOptions() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		// fetch the XML from the results
		$matches = array();
		preg_match("/<interface-response>.*<\/interface-response>/is",trim($rawResult),$matches);

		if (!is_array($matches[0])) {
			// Need to die
		}

		// parse the raw XML to a usable array of data
		$x = new MBXML($matches[0]);
		$data = $x->toArray();
		$CONF["debug"]->add
		(
			"Registercom::getDomainRegistrarLockOptions() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["interface-response"][0]["ErrCount"][0]=="0") {

/**********************************************************************************************
RCom Returned parameters and values: Their Documentation does not match the real return values
***********************************************************************************************
host			n number of possible host records. Ex: www, @, *
DomainServices	.EmailForwarding and HostRecords
IsLockable 		Indicates whether this TLD can use the Registrar Lock feature. Return values are True or False.
Done 			True indicates this entire response has reached you successfully.
**********************************************************************************************/
				$data = correctArray($data["interface-response"][0]);

				$this->arrayData["host"] = $data["host"];
			 	$this->arrayData["DomainServices"] = $data["DomainServices"][0];
				$this->arrayData["dnsOptions"] = $dnsOptions;
				$this->hasExecutedSuccessfully = 1;

			} elseif($data["interface-response"][0]["ErrCount"][0]>"0") {

				foreach($data["interface-response"][0]["errors"][0] as $k => $v) {
                	$this->errors[] = array("RCom Error:", $v[0]);
					$this->numErrors ++;
				}
				$this->hasExecutedSuccessfully = 0;
			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}



	/**
	 * This method contains the registrar specific code for fetching a domain's extended information.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function setDomainDNS() {
		global $CONF;

		$c = $this->getCurlObject();
		$c->addField("UID", $this->UID);
		$c->addField("PW", $this->PW);
		$c->addField("TLD", $this->input['domainTLD']);
		$c->addField("SLD", $this->input['domainSLD']);
		$c->addField("Command", "ModifyNS");
		$c->addField("ResponseType", "XML");
		$c->setURL($this->getURL());
		$c->setFieldMethod(1); // set to POST

		$allowSetHosts = false;
		if (0 < $this->input['domainDNS']) {
			$allowSetHosts = true;
			$c->addField('UseDNS', 'default');
		}

		if (!is_array($this->input['domainNameserver']))
			$this->input['domainNameserver'] = array($this->input['domainNameserver']);

		for ($i=1; $i<=8; $i++) {
			$c->addField('NS'.$i, $this->input['domainNameserver'][$i-1]);
		}

		$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

		$CONF["debug"]->add
		(
			"MBAPI_ProcessRegistrarCommand::getDomainExtendedInfo() ~ \$c->toString",
			htmlentities($rawResult),
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// is this needed here?
		$this->connectionInfo[] =& $c->connectionInfo;

		// check for a cURL error
		## CHECK FOR ERRORS ????
		$this->getCurlErrors($c);

		$x = new MBXML($rawResult);
		$data = $x->toArray();
		$data = $data["interface-response"][0];
		$CONF["debug"]->add
		(
			"Registercom::getDomainExtendedInfo() ~ Cleaned Array",
			$data,
			__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
		);

		// assign the raw data to usable values
		if(is_array($data)){
			if($data["ErrCount"][0]) {
				foreach($data["errors"][0] as $k => $v) {
					$this->errors[] = array("RCom Error:", $v[0]);
					$this->arrayData["result"] = "TRANS_REGISTRAR_MANAGE_DNS_FAILED";
					$this->numErrors ++;
					$this->hasExecutedSuccessfully = 0;
				}
			} else {
				$this->arrayData["result"] = "TRANS_REGISTRAR_MANAGE_DNS_UPDATED";
				$this->hasExecutedSuccessfully = 1;
			}
		}

		if (true == $allowSetHosts) {
			$postArray = array();

			$iterator = 1;
			foreach ((array)$this->input["host"] as $value){
				$postArray['HostName'.$iterator] = $value['HostName'];
				$postArray['RecordType'.$iterator] = $value['RecordType'];
				$postArray['Address'.$iterator] = $value['Address'];
				$iterator++;
			}

			// Create the RCom curl object
			$c = $this->getCurlObject();
			$c->addField("UID",$this->UID);
			$c->addField("PW",$this->PW);
			$c->addField("TLD",$this->input['domainTLD']);
			$c->addField("SLD",$this->input['domainSLD']);
			$c->addField("Command","SetHosts");
			$c->addMultiFields($postArray);
			$c->addField("ResponseType","XML");
			$c->setURL($this->getURL());
			$c->setFieldMethod(1);

			$rawResult = onlyPrintableASCIIChars(str_replace(array_keys($this->fixXMLTags), array_values($this->fixXMLTags), $c->toString()));

			$CONF["debug"]->add
			(
				"MBAPI_ProcessRegistrarCommand::getDomainExtendedInfo() ~ \$c->toString",
				htmlentities($rawResult),
				__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
			);

			// is this needed here?
			$this->connectionInfo[] =& $c->connectionInfo;

			// check for a cURL error
			## CHECK FOR ERRORS ????
			$this->getCurlErrors($c);

			$x = new MBXML($rawResult);
			$data = $x->toArray();
			$data = $data["interface-response"][0];
			$CONF["debug"]->add
			(
				"Registercom::getDomainExtendedInfo() ~ Cleaned Array",
				$data,
				__FILE__,__LINE__,"MBAPI_ProcessRegistrarCommand"
			);

			// assign the raw data to usable values
			if(is_array($data)){
				if($data["ErrCount"][0]) {
					foreach($data["errors"][0] as $k => $v) {
				$this->errors[] = array("RCom Error:", $v[0]);
						$this->arrayData["result"] = "TRANS_REGISTRAR_MANAGE_DNS_FAILED";
						$this->numErrors ++;
						$this->hasExecutedSuccessfully = 0;
					}
				} else {
					$this->arrayData["result"] = "TRANS_REGISTRAR_MANAGE_DNS_UPDATED";
					$this->hasExecutedSuccessfully = 1;
				}
			}
		}

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}







	 /**
	  *	This fucntion will return the map of all possible values with the names needed to pass to the registrar
	  *
	  * @return Returns the Registrar map array for this registrar
	 */
	 function getAttributesMap() {

	 	return array
	  	(
			// Primary Domain Vars
	  		"domainSLD" => "SLD",
            "domainTLD" => "TLD",
            "domainIDNLanguage" => "IDNCode",
            "domainYears" => "NumYears",
            "domainDNS" => "UseDNS",
            "domainNameserver1" => "NS1",
            "domainNameserver2" => "NS2",
            "domainNameserver3" => "NS3",
            "domainNameserver4" => "NS4",
            "domainNameserver5" => "NS5",
            "domainNameserver6" => "NS6",
            "domainNameserver7" => "NS7",
            "domainNameserver8" => "NS8",

            // check availability
            "checkNameSpinner" => "DomainSpinner",
            "checkAllTLDs" => "checkAllTLDs",
			"checkWithDashes" => "AllowDash",
			"checkKeyword1" => "Word1",
			"checkKeyword2" => "Word2",
			"checkKeyword3" => "Word3",

			// Extended Domain Vars
			// .us domains
			"domainNexusCategory" => "us_nexus",
			"domainNexusPurpose" => "us_purpose",
			// .jp domains
			"domainPrefecture" => "jp_prefecture",
			// .eu
			"eu_agreedelete" => "eu_agreedelete",
			"eu_adr_lang" => "eu_adr_lang",

			// RegistrarLock Options Vars
            "domainRegistrarLock" => "UnlockRegistrar",

			// AutoRenew Options Vars
			"domainRenewType" => "RenewFlag", // "RenewName"
			"emailForwardAutoRenew" => "EmailForwardRenew",
			"IDProtectAutoRenew" => "IDProtectRenew",
			"URLForwardAutoRenew" => "URLForwardRenew",

			// Access Password Vars for Registrar Domain Management
            "domainAccessPassword" => "DomainPassword",

			// ContactType=REGISTRANT Vars
	  		"registrantFirstName" => "registrantFirstName",
            "registrantLastName" => "registrantLastName",
            "registrantJobTitle" => "registrantJobTitle",
            "registrantOrganizationName" => "registrantOrganizationName",
            "registrantAddress1" => "registrantAddress1",
            "registrantAddress2" => "registrantAddress2",
            "registrantCity" => "registrantCity",
            "registrantStateProvinceChoice" => "registrantStateProvinceChoice",
            "registrantStateProvince" => "registrantStateProvince",
            "registrantPostalCode" => "registrantPostalCode",
            "registrantCountry" => "registrantCountry",
            "registrantPhone" => "registrantPhone",
            "registrantPhoneExt" => "registrantPhoneExt",
            "registrantFax" => "registrantFax",
            "registrantEmail" => "registrantEmailAddress",

			// ContactType=TECH Vars
	  		"techFirstName" => "techFirstName",
            "techLastName" => "techLastName",
            "techJobTitle" => "techJobTitle",
            "techOrganizationName" => "techOrganizationName",
            "techAddress1" => "techAddress1",
            "techAddress2" => "techAddress2",
            "techCity" => "techCity",
            "techStateProvinceChoice" => "techStateProvinceChoice",
            "techStateProvince" => "techStateProvince",
            "techPostalCode" => "techPostalCode",
            "techCountry" => "techCountry",
            "techPhone" => "techPhone",
            "techPhoneExt" => "techPhoneExt",
            "techFax" => "techFax",
            "techEmail" => "techEmailAddress",

			// ContactType=ADMIN Vars
	  		"adminFirstName" => "adminFirstName",
            "adminLastName" => "adminLastName",
            "adminJobTitle" => "adminJobTitle",
            "adminOrganizationName" => "adminOrganizationName",
            "adminAddress1" => "adminAddress1",
            "adminAddress2" => "adminAddress2",
            "adminCity" => "adminCity",
            "adminStateProvinceChoice" => "adminStateProvinceChoice",
            "adminStateProvince" => "adminStateProvince",
            "adminPostalCode" => "adminPostalCode",
            "adminCountry" => "adminCountry",
            "adminPhone" => "adminPhone",
            "adminPhoneExt" => "adminPhoneExt",
            "adminFax" => "adminFax",
            "adminEmail" => "adminEmailAddress",

			// ContactType=AUXBILLING Vars
	  		"auxBillingFirstName" => "auxbillingFirstName",
            "auxBillingLastName" => "auxbillingLastName",
            "auxBillingJobTitle" => "auxbillingJobTitle",
            "auxBillingOrganizationName" => "auxbillingOrganizationName",
            "auxBillingAddress1" => "auxbillingAddress1",
            "auxBillingAddress2" => "auxbillingAddress2",
            "auxBillingCity" => "auxbillingCity",
            "auxBillingStateProvinceChoice" => "auxbillingStateProvinceChoice",
            "auxBillingStateProvince" => "auxbillingStateProvince",
            "auxBillingPostalCode" => "auxbillingPostalCode",
            "auxBillingCountry" => "auxbillingCountry",
            "auxBillingPhone" => "auxbillingPhone",
            "auxBillingPhoneExt" => "auxbillingPhoneExt",
            "auxBillingFax" => "auxbillingFax",
            "auxBillingEmail" => "auxbillingEmailAddress",

		);

	 } // END getAttributesMap()

	/**
	 * Returns the default configuration parameters for the Registercom module. The number of elements in each array must be identical to
	 * the columns in config_params. This array will be looped by SetConfigGroups.
	 *
	 * @return	An array of arrays containing all the parameters the module uses.
	 */
	function addDefaultConfigParams() {


        // the below are the administrative options that can be configured for the eNom module
        /*
         * [0] = option name
         * [1] = option type (t = text, p = password, r = radio)
         * [2] = default value
         * [3] = option label
         * [4] = option description
         */
        $adminOptions = array(
            array('moduleUsername',                 't',    NULL,   'USERNAME',                             NULL,                                   NULL, 1),
            array('modulePassword',                 'p',    NULL,   'PASSWORD',                             NULL,                                   NULL, 1),
            array('moduleTestMode',                 'b',    '1',    'ENABLE_TEST_MODE',                     NULL,                                   NULL, 0),
            array('moduleAllowAutorenew',          'r',    '0',    'ALLOW_AUTORENEW',                      'ALLOW_AUTORENEW_DESC',                 array(
                array(
                    NULL,
                    NULL,
                    '0',
                    'TRANS_OBEY_TLD',
                    1
                ),
                array(
                    NULL,
                    NULL,
                    '1',
                    'TRANS_REGISTRAR_AUTORENEW_OPTION',
                    2
                ),
                array(
                    NULL,
                    NULL,
                    '2',
                    'TRANS_MODERNBILL_RENEW_OPTION',
                    3
                )
            ), 0
            ),
            array('moduleAllowChangeDNS',          'b',    '0',    'ALLOW_CHANGE_DNS',                     'ALLOW_CHANGE_DNS_DESC',                NULL, 0),
            array('moduleAllowChangeContactInfo',      'b',    '1',    'ALLOW_CHANGE_CONTACT_INFO',            'ALLOW_CHANGE_CONTACT_INFO_DESC',       NULL, 0),
            array('moduleAllowChangeRegistrarLock',        'b',    '0',    'ALLOW_CHANGE_REGISTRAR_LOCK',          'ALLOW_CHANGE_REGISTRAR_LOCK_DESC',     NULL, 0),
            array('moduleAllowChangeAutorenew',        'b',    '0',    'ALLOW_CHANGE_AUTORENEW',               'ALLOW_CHANGE_AUTORENEW_DESC',          NULL, 0),
            array('moduleAllowChangeUpdateDomainPassword', 'b',    '0',    'ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD',  'ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD',  NULL, 0),
        );

        // the below should not need to be changed,
        // edit config params in the temp array above
        $i = 1;
        foreach($adminOptions as $adminOption) {
            $moduleConfig[] = array(
                NULL,
                NULL,
                $adminOption[0],
                $adminOption[2],
                'TRANS_REGISTRAR_MODULE_SETTINGS',
                'TRANS_REGISTRAR_REGISTERCOM_'.$adminOption[3],
                "" != $adminOption[4] ? 'TRANS_REGISTRAR_REGISTERCOM_'.$adminOption[4] : NULL,
                $adminOption[1],
                $adminOption[6],
                $i++,
                0,
                ($adminOption[1] == 'r')
                    ? (($adminOption[5] != NULL) ? $adminOption[5] : array(
                        array(NULL,NULL,'1','TRANS_YES',2),
                        array(NULL,NULL,'0','TRANS_NO',1)
                    ))
                    : NULL
            );
        }

        $contactConfig = array(

            array(NULL, NULL, "registercomAdminFirstnameAdmin", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,18,0,array()),
            array(NULL, NULL, "registercomAdminLastNameAdmin", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,19,0,array()),
            array(NULL, NULL, "adminCompany", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,20,0,array()),
            array(NULL, NULL, "adminEmail", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,21,0,array()),
            array(NULL, NULL, "adminAddress1", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,22,0,array()),
            array(NULL, NULL, "adminAddress2", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,23,0,array()),
            array(NULL, NULL, "adminCity", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,24,0,array()),
            array(NULL, NULL, "adminState", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,25,0,array()),
            array(NULL, NULL, "adminZip", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,26,0,array()),
            array(NULL, NULL, "adminCountry", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,27,0,array()),
            array(NULL, NULL, "adminPhone", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,28,0,array()),
            array(NULL, NULL, "adminFax", "", "TRANS_ADMINISTRATIVE_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,29,0,array()),
            // tech contact data
            array(NULL, NULL, "techFirstname", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,30,0,array()),
            array(NULL, NULL, "techLastName", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,31,0,array()),
            array(NULL, NULL, "techCompany", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,32,0,array()),
            array(NULL, NULL, "techEmail", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,33,0,array()),
            array(NULL, NULL, "techAddress1", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,34,0,array()),
            array(NULL, NULL, "techAddress2", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,35,0,array()),
            array(NULL, NULL, "techCity", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,36,0,array()),
            array(NULL, NULL, "techState", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,37,0,array()),
            array(NULL, NULL, "techZip", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,38,0,array()),
            array(NULL, NULL, "techCountry", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,39,0,array()),
            array(NULL, NULL, "techPhone", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,40,0,array()),
            array(NULL, NULL, "techFax", "", "TRANS_TECHNICAL_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,41,0,array()),
            // aux billing contact data
            array(NULL, NULL, "auxBillingFirstname", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_FIRST_NAME", "", "t", 0,42,0,array()),
            array(NULL, NULL, "auxBillingLastName", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_LAST_NAME", "", "t", 0,43,0,array()),
            array(NULL, NULL, "auxBillingCompany", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_COMPANY", "", "t", 0,44,0,array()),
            array(NULL, NULL, "auxBillingEmail", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_EMAIL", "", "t", 0,45,0,array()),
            array(NULL, NULL, "auxBillingAddress1", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS1", "", "t", 0,46,0,array()),
            array(NULL, NULL, "auxBillingAddress2", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ADDRESS2", "", "t", 0,47,0,array()),
            array(NULL, NULL, "auxBillingCity", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_CITY", "", "t", 0,48,0,array()),
            array(NULL, NULL, "auxBillingState", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_STATE", "", "t", 0,49,0,array()),
            array(NULL, NULL, "auxBillingZip", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_ZIP", "", "t", 0,50,0,array()),
            array(NULL, NULL, "auxBillingCountry", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_COUNTRY", "", "t", 0,51,0,array()),
            array(NULL, NULL, "auxBillingPhone", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_PHONE", "", "t", 0,52,0,array()),
            array(NULL, NULL, "auxBillingFax", "", "TRANS_BILLING_CONTACT", "TRANS_REGISTRAR_FAX", "", "t", 0,53,0,array()),

        );

        // create the master array which we'll return containing all configuration parameters
        $defaultConfigParams = array_merge($moduleConfig, $contactConfig);

        // return the master array containing all configuration parameters
        return $defaultConfigParams;
    }
}
