<?php

require_once("lib-mbapi/include/modules/registrar/registrar.php");
require_once("lib-mbapi/include/modules/registrar/core/godaddy/WWDAPI.php");

class WildWestDomains extends Registrar
{
	const status		= Registrar::STATUS_STABLE;
	const version		= '1.0.0';
	const displayName	= 'Wild West Domains/GoDaddy';
	const author		= 'Parallels';
	const file			= __FILE__;

	const POLL_RETRY_TIME = 15; // length of time to wait to poll again if first poll didn't return anything
	const GET_DOMAIN_INFO_ATTEMTS_NUMBER = 10;

	private static $moduleCapabilities = array(
		REGISTRAR_REGISTERDOMAIN => 1,
		REGISTRAR_TRANSFERDOMAIN => 1,
		REGISTRAR_RENEWDOMAIN => 1,
		REGISTRAR_GETDOMAINEXTENDEDINFO => 0,
		REGISTRAR_CHECKDOMAINAVAILABILITY => 1,
		REGISTRAR_CHECKDOMAINTRANSFERAVAILABILITY => 0,
		REGISTRAR_GETDOMAINCONTACTDATA => 1,
		REGISTRAR_SETDOMAINCONTACTDATA => 1,
		REGISTRAR_GETDOMAINRENEWTYPEOPTIONS => 0,
		REGISTRAR_SETDOMAINRENEWTYPEOPTIONS => 0,
		REGISTRAR_GETDOMAINREGISTRARLOCKOPTIONS => 0,
		REGISTRAR_SETDOMAINREGISTRARLOCKOPTIONS => 0,
		REGISTRAR_GETDOMAINACCESSPASSWORD => 0,
		REGISTRAR_SETDOMAINACCESSPASSWORD => 0,
		REGISTRAR_GETDOMAINDNS => 1,
		REGISTRAR_SETDOMAINDNS => 1,
		REGISTRAR_GETDOMAINRENEWDATE => 0,
		REGISTRAR_LIVERENEW => 0,
		REGISTRAR_CHECKACCOUNTBALANCE => 0,
		REGISTRAR_GETTLDPRICING => 0,
		REGISTRAR_GETTRANSFERSTATUS => 0,
		REGISTRAR_GETTLDLIST => 0,
		REGISTRAR_TLDS => array(
			'com',
			'net',
			'org',
			'biz',
			'in',
			'co.in',
			'firm.in',
			'gen.in',
			'ind.in',
			'net.in',
			'org.in',
			'info',
			'me',
			'mobi',
			'us',
			'ws',
			'es',
			'com.es',
			'org.es',
			'nom.es',
			'com.mx',
			'tv',
			'bz',
			'com.bz',
			'net.bz',
		),
	);

	private $contactTypes = array(
		'Registrant',
		'Tech',
		'Admin',
		'AuxBilling'
	);

	private $nameTranslations = array(
		'FirstName' => 'fname',
		'LastName' => 'lname',
		'OrganizationName' => 'org',
		'Address1' => 'sa1',
		'Address2' => 'sa2',
		'City' => 'city',
		'StateProvince' => 'sp',
		'Country' => 'cc',
		'PostalCode' => 'pc',
		'EmailAddress' => 'email',
		'Phone' => 'phone',
		'Fax' => 'fax',
	);


	private $wwd = false;

	public static function staticGetModuleInfo()
	{
		return array(
			'type' => 'registrar',
			'status' => self::status,
			'version' => self::version,
			'displayName' => self::displayName,
			'author' => self::author,
			'file' => self::file,
			'countryCodes' => array('US'),
		);
	}

	// not static because this is overriding a non-static method in the Gateway class
	public function getModuleInfo()
	{
		return self::staticGetModuleInfo();
	}

	/**
	 * Construct a new GoDaddy object. This function merely sets up capabilities after calling the
	 * constructor for Registrar.
	 */
	public function __construct($input)
	{
		// set up the capabilities array
		$this->capabilities = (isset($this->capabilities) && is_array($this->capabilities))
			? array_merge($this->capabilities, self::$moduleCapabilities)
			: self::$moduleCapabilities;

		// decrypts the password if needed...
		// the getStuff method returns blank if decryption fails
		// and can be used as a true/false test
		if ($plainText = getStuff($input['packagePassword'])) {
			$input['packagePassword'] = substr($plainText, 0, 20);
		}

		// The packageID might not be here for all instantiations because we instantiate
		// the module to do things like get the config_params and stuff.  We only need
		// to do this code when the packageID IS here.
		if ((int)$input['packageID']) {
			// we need to check for previously created GoDaddy user's for this customer
			$package = Factory()->Package($input['packageID']);
			$clientID = $package->client_id;

			$existingGodaddyUser = DatabaseManager::getConnection()->getOne("SELECT
				package_attribute_value
			FROM
				packages, package_attributes
			WHERE
				packages.package_id = package_attributes.package_id &&
				package_attributes.package_attribute_name = 'domain_godaddyUser' &&
				packages.client_id = ".$clientID);
			if ($existingGodaddyUser){
				$input['godaddyUser'] = $existingGodaddyUser;
			}

			// domains by proxy user
			$existingGodaddyDBPUser = DatabaseManager::getConnection()->getOne("SELECT
				package_attribute_value
			FROM
				packages, package_attributes
			WHERE
				packages.package_id = package_attributes.package_id &&
				package_attributes.package_attribute_name = 'domain_godaddyDBPUser' &&
				packages.client_id = ".$clientID);
			if ($existingGodaddyDBPUser){
				$input['godaddyDBPUser'] = $existingGodaddyDBPUser;
			}
		}

		// the registrar system is stupid and prepends every attribute we pass back with
		// 'domain_' instead of just using what we pass.  This code causes the godaddyUser
		// and godaddyDBPUser for this specific domain to overwrite the master ones found
		// possibly on other domains if there were any.  It also takes the domain_ off of
		// other attributes we stored.
		foreach ((array)$input as $k=>$v) {
			if (substr($k, 0, 7)=='domain_') {
				$input[substr($k, 7)] = $v;
			}
		}

		parent::__construct($input);
	}

	/**
	 * helper method to create or get the instance of the wwdapi class
	 */
	function getWWD()
	{
		if (false === $this->wwd) {
			$this->wwd = new WWDAPI($this->UID, $this->PW);
			if ((int)$this->config['moduleTestMode'])
				$this->wwd->testMode();
		}
		return $this->wwd;
	}

	/**
	 * Call the registrar to register a new domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function registerDomain()
	{
		// create the contact data array
		$contactArray = $this->constructContactsData();

		$dnsServers = $this->getDNSServers();

		if (1 == $this->input['domain_renew_type']) { // 1 = registrar, 2 = ppb renew
			$autoRenew = 1;
		} else {
			$autoRenew = 0;
		}

		$input = array(
			'sROID' => $this->input['packageID'],
			'nameservers' => $dnsServers,
			'domains' => array(
				array(
					'sld' => strtolower($this->input['domainSLD']),
					'tld' => strtolower($this->input['domainTLD']),
					'riid' => $this->input['packageID'],
					'years' => $this->input['domainYears'],
					'autorenew' => $autoRenew,
				),
			),
			'registrant' => $contactArray['Registrant'],
			'admin' => $contactArray['Admin'],
			'billing' => $contactArray['AuxBilling'],
			'tech' => $contactArray['Tech'],
		);

		if ($this->input['godaddyUser'] > 0) {
			$input['user'] = $this->input['godaddyUser'];
		} else {
			$input['user'] = 'createNew';
			if ("" == $this->input['packagePassword']) {
				$this->input['packagePassword'] = getRandomPassword(12);
			}
			$input['pwd'] = $this->input['packagePassword'];
			$input['pwdhint'] = 'Billing Password';
			$input['email'] = $contactArray['Registrant']['email'];
			$input['firstname'] = $contactArray['Registrant']['fname'];
			$input['lastname'] = $contactArray['Registrant']['lname'];
			$input['phone'] = $contactArray['Registrant']['phone'];
		}

		$this->addAdditionalRegistrantFields($input);

		$results = $this->getWWD()->registerDomain($input);

		if ($results!==false && $results['user']) {
			$ret = array(
				'godaddyUser' => $results['user'],
				'godaddyOrder' => $results['orderid'],
			);
			if (0 == (int)$this->input['godaddyUser']){
				$ret['godaddyPass'] = $this->input['packagePassword'];
			}
			$ret['dnsServers'] = serialize($dnsServers);
			$ret['useDNS'] = empty($dnsServers) ? 1 : 0;

			if (!($poll = $this->getWWD()->poll()) || count($poll) == 0) {
				sleep(self::POLL_RETRY_TIME); // try again in a second
				$poll = $this->getWWD()->poll();
			}
			if ($poll[$this->input['packageID']]) { // good, the domain made it apparently
				$ret['godaddyResourceID'] = $poll[$this->input['packageID']];
			}

			$this->waitDomainCreation($this->input['domainSLD'] . '.' . $this->input['domainTLD'], $ret['useDNS']);

			// add idprotect if they wanted it
			if (in_array(strtolower(trim($this->input['idProtect'])), array(1, '1', 'yes', 'true'))) {
				$input = array(
					'user' => $results['user'],
					'sROID' => $this->input['packageID'].'p',
					'domains' => array(
						array(
							'sld' => strtolower($this->input['domainSLD']),
							'tld' => strtolower($this->input['domainTLD']),
							'riid' => $this->input['packageID'].'p',
							'years' => $this->input['domainYears'],
						),
					),
				);
				if ($this->input['godaddyDBPUser'] > 0) {
					$input['dbpuser'] = $this->input['godaddyDBPUser'];
				} else {
					$input['dbppwd'] = $this->input['packagePassword'];
					$input['dbpemail'] = $contactArray['Registrant']['email'];
				}
				$idprotect = $this->getWWD()->orderDomainPrivacy($input);
				if (!$idprotect) {
					// idprotect failed for some reason but registration worked
					$ret['idProtectFailure'] = implode('. \n', $this->getWWD()->getErrors());
				} else {
					if (!($poll = $this->getWWD()->poll()) || count($poll) == 0) {
						sleep(self::POLL_RETRY_TIME); // try again in a second
						$poll = $this->getWWD()->poll();
					}
					if ($poll[$this->input['packageID'] . 'p']) { // good, the idprotect made it apparently
						$ret['godaddyPrivacyResourceID'] = $poll[$this->input['packageID'] . 'p'];
					}
					$ret['godaddyDBPUser'] = $idprotect['dbpuser'];
					$ret['godaddyDBPOrder'] = $idprotect['orderid'];
				}
			}

			return $this->createReturn($ret);
		} else {
			return $this->createReturn($this->getWWD()->getErrors(), false);
		}
	}

	/**
	 * Call the registrar to transfer an existing domain
	 *
	 * @return  string  XML containing registration results for this domain registration
	 */
	function transferDomain()
	{
		// create the contact data array
		$contactArray = $this->constructContactsData();

		$dnsServers = $this->getDNSServers();

		if (1 == $this->input['domain_renew_type']) { // 1 = registrar, 2 = ppb renew
			$autoRenew = 1;
		} else {
			$autoRenew = 0;
		}

		$input = array(
			'sROID' => $this->input['packageID'],
			'nameservers' => $dnsServers,
			'domains' => array(
				array(
					'sld' => strtolower($this->input['domainSLD']),
					'tld' => strtolower($this->input['domainTLD']),
					'riid' => $this->input['packageID'],
					'authInfo' => $this->input['domainTransferKey'],
					'autorenew' => $autoRenew,
				),
			),
			'registrant' => $contactArray['Registrant'],
			'admin' => $contactArray['Admin'],
			'billing' => $contactArray['AuxBilling'],
			'tech' => $contactArray['Tech'],
		);

		if ($this->input['godaddyUser'] > 0) {
			$input['user'] = $this->input['godaddyUser'];
		} else {
			$input['user'] = 'createNew';
			if ("" == $this->input['packagePassword']) {
				$this->input['packagePassword'] = getRandomPassword(12);
			}
			$input['pwd'] = $this->input['packagePassword'];
			$input['pwdhint'] = 'Billing Password';
			$input['email'] = $contactArray['Registrant']['email'];
			$input['firstname'] = $contactArray['Registrant']['fname'];
			$input['lastname'] = $contactArray['Registrant']['lname'];
			$input['phone'] = $contactArray['Registrant']['phone'];
		}

		$results = $this->getWWD()->transferDomain($input);

		if ($results!==false && $results['user']) {
			$ret = array(
				'godaddyUser' => $results['user'],
				'godaddyOrder' => $results['orderid'],
			);
			if (0 == (int)$this->input['godaddyUser']){
				$ret['godaddyPass'] = $this->input['packagePassword'];
			}
			$ret['dnsServers'] = serialize($dnsServers);
			$ret['useDNS'] = empty($dnsServers) ? 1 : 0;

			if (!($poll = $this->getWWD()->poll()) || count($poll) == 0) {
				sleep(self::POLL_RETRY_TIME); // try again in a second
				$poll = $this->getWWD()->poll();
			}
			if ($poll[$this->input['packageID']]) { // good, the domain made it apparently
				$ret['godaddyResourceID'] = $poll[$this->input['packageID']];
			}

			// add idprotect if they wanted it
			if (in_array(strtolower(trim($this->input['idProtect'])), array(1, '1', 'yes', 'true'))) {
				$input = array(
					'user' => $results['user'],
					'sROID' => $this->input['packageID'].'p',
					'domains' => array(
						array(
							'sld' => strtolower($this->input['domainSLD']),
							'tld' => strtolower($this->input['domainTLD']),
							'riid' => $this->input['packageID'].'p',
							'years' => $this->input['domainYears'],
						),
					),
				);

				if ($this->input['godaddyDBPUser'] > 0) {
					$input['dbpuser'] = $this->input['godaddyDBPUser'];
				} else {
					$input['dbppwd'] = $this->input['packagePassword'];
					$input['dbpemail'] = $contactArray['Registrant']['email'];
				}

				$idprotect = $this->getWWD()->orderDomainPrivacy($input);
				if (!$idprotect) {
					// idprotect failed for some reason but registration worked
					$ret['idProtectFailure'] = implode('. \n', $this->getWWD()->getErrors());
				} else {
					if (!($poll = $this->getWWD()->poll()) || count($poll) == 0) {
						sleep(self::POLL_RETRY_TIME); // try again in a second
						$poll = $this->getWWD()->poll();
					}
					if ($poll[$this->input['packageID'] . 'p']) { // good, the idprotect made it apparently
						$ret['godaddyPrivacyResourceID'] = $poll[$this->input['packageID'] . 'p'];
					}
					$ret['godaddyDBPUser'] = $idprotect['dbpuser'];
					$ret['godaddyDBPOrder'] = $idprotect['orderid'];
				}
			}

			return $this->createReturn($ret);
		} else {
			return $this->createReturn($this->getWWD()->getErrors(), false);
		}
	}

	/**
	 * This method contains the registrar specific code for renewing a domain.
	 *
	 * @return  string xmlData of containting results of the remote registrar connection
	 */
	function renewDomain()
	{
		$input = array(
			'sROID' => $this->input['packageID'],
			'user' => $this->input['godaddyUser'],
			'dbpuser' => $this->input['godaddyDBPUser'], // if it's here
			'domains' => array(
				array(
					'sld' => strtolower($this->input['domainSLD']),
					'tld' => strtolower($this->input['domainTLD']),
					'riid' => $this->input['packageID'],
					'years' => $this->input['domainYears'],
				),
			),
		);

		if ($this->input['godaddyResourceID'] > 0) {
			$input['domains'][0]['resourceid'] = $this->input['godaddyResourceID'];
		}

		if ($this->input['godaddyPrivacyResourceID']) {
			$input['domains'][] = array(
				'riid' => $this->input['packageID'].'p',
				'years' => $this->input['domainYears'],
				'resourceid' => $this->input['godaddyPrivacyResourceID'],
			);
		}

		$results = $this->getWWD()->renewDomain($input);

		if ($results!==false && $results['orderid']) {

			if (!($poll = $this->getWWD()->poll()) || count($poll) == 0) {
				sleep(self::POLL_RETRY_TIME); // try again in a second
				$poll = $this->getWWD()->poll();
			} // at this point the poll calls are really just to get things off the stack

			return $this->createReturn(array(
				'renewalOrderID' => $results['orderid'],
			));
		} else {
			return $this->createReturn($this->getWWD()->getErrors(), false);
		}
	}

	/**
	 * This method creates the dns servers array from $this->input
	 *
	 * @return	array	$dnsServers
	 */
	function getDNSServers()
	{
		$dnsServers = array();
		if ($this->input['domainNameserver1']) {
			$dnsServers[] = $this->input['domainNameserver1'];
			if ($this->input['domainNameserver2'])
				$dnsServers[] = $this->input['domainNameserver2'];
			if ($this->input['domainNameserver3'])
				$dnsServers[] = $this->input['domainNameserver3'];
			if ($this->input['domainNameserver4'])
				$dnsServers[] = $this->input['domainNameserver4'];
		} else {
			if ($this->config['domainNameserver1'])
				$dnsServers[] = $this->config['domainNameserver1'];
			if ($this->config['domainNameserver2'])
				$dnsServers[] = $this->config['domainNameserver2'];
			if ($this->config['domainNameserver3'])
				$dnsServers[] = $this->config['domainNameserver3'];
			if ($this->config['domainNameserver4'])
				$dnsServers[] = $this->config['domainNameserver4'];
		}
		return $dnsServers;
	}

	/**
	 * This method takes data from $this->input and $this->config and constructs the
	 * contacts data array to pass to the API
	 *
	 * @return	array	$contacts
	 */
	function constructContactsData()
	{
		// match these with the module config naming conventions
		$this->input['RegistrantFirstName'] = $this->input['registrantFirstName'];
		$this->input['RegistrantLastName'] = $this->input['registrantLastName'];
		$this->input['RegistrantOrganizationName'] = $this->input['registrantOrganizationName'];
		$this->input['RegistrantJobTitle'] = $this->input['registrantJobTitle'];
		$this->input['RegistrantAddress1'] = $this->input['registrantAddress1'];
		$this->input['RegistrantAddress2'] = $this->input['registrantAddress2'];
		$this->input['RegistrantCity'] = $this->input['registrantCity'];
		$this->input['RegistrantStateProvince'] = $this->input['registrantState'];
		$this->input['RegistrantPostalCode'] = $this->input['registrantPostalCode'];
		$this->input['RegistrantCountry'] = $this->input['registrantCountry'];
		$this->input['RegistrantEmailAddress'] = $this->input['registrantEmail'];
		$this->input['RegistrantPhone'] = $this->input['registrantPhone'];
		$this->input['RegistrantFax'] = $this->input['registrantFax'];
		$contacts = array();
		foreach ($this->contactTypes as $contactType) {
			if ($contactType == 'Registrant') {
				foreach ($this->nameTranslations as $contactParam=>$newName) {
					$contacts[$contactType][$newName] = $this->input[$contactType.$contactParam];
				}
			} else {
				foreach ($this->nameTranslations as $contactParam=>$newName) {
					$contacts[$contactType][$newName] = $this->config[$contactType.$contactParam];
				}
				// if this data is blank then just overwrite it with the registrant data
				// which is always populated first
				if (trim($contacts[$contactType]['fname'])=='' && trim($contacts[$contactType]['lname'])=='' && trim($contacts[$contactType]['org'])=='') {
					$contacts[$contactType] = $contacts['Registrant'];
				}
			}

		}
		return $contacts;
	}

	/**
	 * Call the registrar to fetch a domain's contact data (NOT IMPLEMENTED YET)
	 *
	 * @return  string  XML containing contact data for this domain
	 */
	function getDomainContactData()
	{
		$data = $this->getWWD()->domainContactInfo($this->input['domainSLD'].'.'.$this->input['domainTLD']);
		if (false !== $data) {
			$nameTranslations = array_flip($this->nameTranslations);
			foreach ($data as $type => $contact) {
				$newContact = array();
				if ('US' == $contact['cc'])
					$newContact['StateProvinceChoice'] = 'S';
				else
					$newContact['StateProvinceChoice'] = 'P';

				foreach ($contact as $field => $value) {
					if (isset($nameTranslations[$field]))
						$newContact[$nameTranslations[$field]] = $value;
				}
				$data[$type] = $newContact;
			}

			$this->arrayData['DomainData'] = array();
			$this->arrayData['DomainData']['domainname'] = $this->input['domainSLD'].'.'.$this->input['domainTLD'];
			$this->arrayData['RegistrantContactData'] = $data['registrant'];
			$this->arrayData['AuxBillingContactData'] = $data['billing'];
			$this->arrayData['TechContactData'] = $data['tech'];
			$this->arrayData['AdminContactData'] = $data['admin'];
			$this->arrayData['BillingContactData'] = $data['billing'];
			$this->arrayData['OtherData'] = array();

			$arrayKeyAdditions = array(
				'RegistrantContactData'	=> 'Registrant',
				'AuxBillingContactData'	=> 'AuxBilling',
				'TechContactData' 	=> 'Tech',
				'AdminContactData'	=> 'Admin',
				'BillingContactData'	=> 'Billing',
			);

			foreach ($arrayKeyAdditions as $field => $addition) {
				foreach ($this->arrayData[$field] as $key => $value) {
					unset($this->arrayData[$field][$key]);
					$this->arrayData[$field][$addition.$key] = $value;
				}
			}

			return $this->createReturn($this->arrayData);
		} else {
			return $this->createReturn($this->getWWD()->getErrors(), false);
		}
	}

	/**
	 * Send the updated domain contact data to the registrar  (NOT IMPLEMENTED YET)
	 *
	 * @return  string  XML result
	 */
	function setDomainContactData()
	{
		$phoneCodes = getCountryPhoneCodes('lookup');
		$contacts = $this->input['domainContactData'];
		foreach ($contacts as $type => $data) {

			if (!preg_match('/[\+]?[[:digit:]]+\.[[:digit:]]+/', $data[$type.'Phone']))
				$data[$type.'Phone'] = '+'.$phoneCodes[$data[$type.'PhoneDial']].'.'.preg_replace('/[^[:digit:]]/', '', $data[$type.'Phone']);
			elseif ($data[$type.'Phone'][0] != '+')
				$data[$type.'Phone'] = '+'.$data[$type.'Phone'];

			if (!preg_match('/[\+]?[[:digit:]]+\.[[:digit:]]+/', $data[$type.'Fax']))
				$data[$type.'Fax'] = '+'.$phoneCodes[$data[$type.'FaxDial']].'.'.preg_replace('/[^[:digit:]]/', '', $data[$type.'Fax']);
			elseif ($data[$type.'Fax'][0] != '+')
				$data[$type.'Fax'] = '+'.$data[$type.'Fax'];

			foreach ($data as $field => $value) {
				unset($contacts[$type][$field]);
				$actualFieldName = substr($field, strlen($type));
				if (isset($this->nameTranslations[$actualFieldName]))
					$contacts[$type][$this->nameTranslations[$actualFieldName]] = $value;
			}
		}

		$contacts['billing'] = $contacts['auxBilling'];
		unset($contacts['auxBilling']);

		$response = $this->getWWD()->setDomainContactInfo(array_merge($contacts, array(
			'sld' => $this->input['domainSLD'],
			'tld' => $this->input['domainTLD'],
		)));

		if (false !== $response) {
			return $this->createReturn(array(
				'success' => 1,
			));
		} else {
			return $this->createReturn($this->getWWD()->getErrors(), false);
		}
	}

	/**
	 * This method sets a domain's auto-renewal options.  (NOT IMPLEMENTED YET)
	 *
	 * @return  string xmlData containing results of the operation
	 */
	function setdomainRenewTypeOptions()
	{

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/**
	 * This method fetches the domain registrar-lock options.  (NOT IMPLEMENTED YET)
	 *
	 * @return  string xmlData containing results of the operation
	 */
	function getDomainRegistrarLockOptions()
	{
		return $this->createReturn(array(
			'DomainRegistrarLock' => 1,
			'DomainIsLockable' => 1,
		));
	}

	/**
	 * This method sets the domain registrar-lock options.  (NOT IMPLEMENTED YET)
	 *
	 * @return  string xmlData containing results of the operation
	 */
	function setDomainRegistrarLockOptions()
	{
		$response = $this->getWWD()->setDomainLock(array(
			'sld' => $this->input['domainSLD'],
			'tld' => $this->input['domainTLD'],
			'lock' => $this->input['domainRegistrarLock'],
		));
		if (false === $response) {
			return $this->createReturn($this->getWWD()->getErrors(), false);
		} else {
			return $this->createReturn(array(
				'success' => 1,
			));
		}
	}

	/**
	 * This method gets the domain access password.  (NOT IMPLEMENTED YET)
	 *
	 * @return  string xmlData of containing results of the operation
	 */
	function getDomainAccessPassword()
	{

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/**
	 * This method sets the domain access password.  (NOT IMPLEMENTED YET)
	 *
	 * @return  string xmlData of containing results of the operation
	 */
	function setDomainAccessPassword()
	{

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/**
	 * This method gets the domain expiration date from Wild West Domains.  (NOT IMPLEMENTED YET)
	 *
	 * @return  string xmlData containing the expiration date
	 */
	function getDomainExpDate()
	{

		$this->hasExecuted = 1;
		$this->xmlData = $this->getRegistrarReturnXML();
		return $this->xmlData;
	}

	/**
	 * This method gets the current DNS records for a domain from Wild West Domains.  (NOT IMPLEMENTED YET)
	 *
	 * @return  string xmlData containing the DNS servers and host records
	 */
	function getDomainDNS()
	{
		$this->arrayData = array(
			'OtherData' => array(),
			'listDNS' => array(),
			'UseDNS' => $this->input['useDNS'],
			'host' => array(),
			'DomainServices' => array(),
			'dnsOptions' => array(
				'A' => 'A',
				'MX' => 'MX',
				'CNAME' => 'CNAME',
				'SRV' => 'SRV',
				'TXT' => 'TXT',
			),
		);

		if (0 == $this->input['useDNS']) {
			$this->arrayData['listDNS'] = unserialize($this->input['dnsServers']);
			return $this->createReturn($this->arrayData);
		}

		$results = $this->getWWD()->domainInfo($this->input['domainSLD'].'.'.$this->input['domainTLD'], 'dns');

		if (false === $results) {
            return $this->createReturn($this->getWWD()->getErrors(), false);
		}

		$this->arrayData["OtherData"] = $results;

		if (!(is_array($results['info']) && is_array($results['info'][0]['dns']) && is_array($results['info'][0]['dns'][0]['rec']))){
			$this->addError(TRANS_REGISTRARERROR, 'An error has occurred, registrar did not return correct DNS information.');
			$this->arrayData = array(
				'error' => 'Invalid data returned from registrar.',
			);
			$this->hasExecutedSuccessfully = 0;
			$this->hasExecuted = 1;
			return $this->xmlData = $this->getRegistrarReturnXML();
		}

		$records = $results['info'][0]['dns'][0]['rec'];
		foreach ((array)$records as $rec) {
			if (is_array($rec['_attributes']) && isset($this->arrayData['dnsOptions'][$rec['_attributes']['type']]))
				$this->arrayData['host'][] = array(
					'name' => $rec['_attributes']['key'],
					'type' => $rec['_attributes']['type'],
					'address' => $rec['_attributes']['val'],
				);
		}

		return $this->createReturn($this->arrayData);
	}

	/**
	 * This method sets a domain's DNS servers and host records  (NOT IMPLEMENTED YET)
	 *
	 * @return  string xmlData containing results of the operation
	 */
	function setDomainDNS()
	{
		if (1 == (int)$this->input['domainDNS']) {
			// this means we use GoDaddy DNS servers, we need to set the servers
			// and we need to set the DNS resource records
			$records = array();
			if (isset($this->input['host'])) {
				if (isset($this->input['host']['RecordType'])) {
					$this->input['host'] = array($this->input['host']); // one record exception (caused by using collapseArrayElements() function in ProcessRegistrar command
				}
				foreach ($this->input['host'] as $record) {
					$records[] = array(
						'recType' => $record['RecordType'],
						'key' => $record['HostName'],
						'recValue' => $record['Address'],
						'ttl' => 3600,
					);
				}
			}
			if (empty($records)) {
				return $this->createReturn(array('You must provide at least one DNS record.'), false);
			}

			$nameservers = (1 == $this->config['moduleTestMode']) ?
				array('ns01.ote.domaincontrol.com', 'ns02.ote.domaincontrol.com') : array('ns02.domaincontrol.com', 'ns03.domaincontrol.com');
			$serversSetResult = $this->getWWD()->setDNS(array(
				'sld' => $this->input['domainSLD'],
				'tld' => $this->input['domainTLD'],
				'nameservers' => $nameservers,
			));

			if (false === $serversSetResult) {
				return $this->createReturn($this->getWWD()->getErrors(), false);
			}

			$recordsSetResult = $this->getWWD()->setDNS(array(
				'sld' => $this->input['domainSLD'],
				'tld' => $this->input['domainTLD'],
				'records' => $records,
			));

			if (false === $recordsSetResult) {
				return $this->createReturn($this->getWWD()->getErrors(), false);
			} else {
				return $this->createReturn(array(
					'success' => 1,
					'dnsServers' => serialize(array()),
					'useDNS' => 1,
				));
			}
		} else {
			// this means we will be using custom nameservers, set the servers
			// and return success
			$nameservers = array();
			foreach ($this->input['domainNameserver'] as $ns)
				if (!empty($ns))
					$nameservers[] = trim($ns);

			if (count($nameservers) > 0) {
				$serversSetResult = $this->getWWD()->setDNS(array(
					'sld' => $this->input['domainSLD'],
					'tld' => $this->input['domainTLD'],
					'nameservers' => $nameservers,
				));
				if (false === $serversSetResult) {
					return $this->createReturn($this->getWWD()->getErrors(), false);
				} else {
					return $this->createReturn(array(
						'success' => 1,
						'dnsServers' => serialize($nameservers),
						'useDNS' => 0,
					));
				}
			} else {
				return $this->createReturn(array(
					'You must provide at least one nameserver.'
				), false);
			}
		}
	}

	public function checkDomainAvailability()
	{
		$domains = array();
		foreach ($this->input["domains"] as $domain) {
			$domains[] = $domain["sld"] . "." . $domain["tld"];
		}
		$checkAvailabilityResult = $this->getWWD()->checkAvailability($domains);
		if (false === $checkAvailabilityResult) {
			return $this->createReturn($this->getWWD()->getErrors(), false);
		}
		$result = array();
		reset($domains);
		foreach ($checkAvailabilityResult as $domain) {
			if (isset($domain["error"])) {
				$result[current($domains)]["status"] = self::DOMAIN_AVAIL_STATUS_CANT_CHECK;
			} else {
				$result[current($domains)]["status"] = (1 == $domain["available"]) ? self::DOMAIN_AVAIL_STATUS_AVAIL : self::DOMAIN_AVAIL_STATUS_NOT_AVAIL;
			}
			next($domains);
		}
		return $this->createReturn($result);
	}

	public function addDefaultConfigParams()
	{
		//This needs to be tested with SetConfigGroups once a "add godaddy" link is added to the GUI
		$defaultConfigParams = array(
			array(NULL, NULL, 'moduleUsername', '', 'TRANS_REGISTRAR_MODULE_SETTINGS', 'TRANS_REGISTRAR_USERNAME', '', 't', 1, 3, 0, NULL),
			array(NULL, NULL, 'modulePassword', '', 'TRANS_REGISTRAR_MODULE_SETTINGS', 'TRANS_REGISTRAR_PASSWORD', '', 'p', 1, 4, 0, NULL),
			array(NULL, NULL, 'moduleTestMode', '1', 'TRANS_REGISTRAR_MODULE_SETTINGS', 'TRANS_REGISTRAR_GODADDY_TEST_MODE', '', 'b', 0, 5, 0, NULL),
			array(NULL, NULL, 'moduleCertified', 0, 'TRANS_REGISTRAR_MODULE_SETTINGS', 'TRANS_MODULE_PASSED_CERTIFICATION', '', 'h', 0, 6, 0, NULL),
			array(NULL, NULL, 'moduleAllowAutoRenew', 2, 'TRANS_REGISTRAR_MODULE_SETTINGS', 'TRANS_REGISTRAR_GODADDY_ALLOW_AUTORENEW', 'TRANS_REGISTRAR_GODADDY_ALLOW_AUTORENEW_DESC', 'r', 0, 9, 0, array
				(array(NULL, NULL, '0', 'TRANS_OBEY_TLD', 1),
				array(NULL, NULL, '1', 'TRANS_REGISTRAR_AUTORENEW_OPTION', 2),
				array(NULL, NULL, '2', 'TRANS_MODERNBILL_RENEW_OPTION', 3))),
			array(NULL, NULL, 'moduleAllowChangeDNS', '1','TRANS_REGISTRAR_MODULE_SETTINGS', 'TRANS_REGISTRAR_GODADDY_ALLOW_CHANGE_DNS', 'TRANS_REGISTRAR_GODADDY_ALLOW_CHANGE_DNS_DESC', 'b', 0,14, 0, NULL),
			array(NULL, NULL, 'moduleAllowChangeContactInfo', '1', 'TRANS_REGISTRAR_MODULE_SETTINGS','TRANS_REGISTRAR_GODADDY_ALLOW_CHANGE_CONTACT_INFO', 'TRANS_REGISTRAR_GODADDY_ALLOW_CHANGE_CONTACT_INFO_DESC', 'b', 0, 15, 0, NULL),
			array(NULL, NULL, 'moduleAllowChangeRegistrarLock', '1','TRANS_REGISTRAR_MODULE_SETTINGS', 'TRANS_REGISTRAR_GODADDY_ALLOW_CHANGE_REGISTRAR_LOCK', 'TRANS_REGISTRAR_GODADDY_ALLOW_CHANGE_REGISTRAR_LOCK_DESC', 'b', 0, 16, 0, NULL),
			array(NULL, NULL, 'moduleAllowChangeUpdateDomainPassword', '1','TRANS_REGISTRAR_MODULE_SETTINGS', 'TRANS_REGISTRAR_GODADDY_ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD', 'TRANS_REGISTRAR_GODADDY_ALLOW_CHANGE_UPDATE_DOMAIN_PASSWORD_DESC', 'b', 0, 18, 0, NULL),
			// admin contact data
			array(NULL, NULL, 'adminFirstName', '', 'TRANS_ADMINISTRATIVE_CONTACT', 'TRANS_REGISTRAR_FIRST_NAME', '', 't', 0, 20, 0, NULL),
			array(NULL, NULL, 'adminLastName', '', 'TRANS_ADMINISTRATIVE_CONTACT', 'TRANS_REGISTRAR_LAST_NAME', '', 't', 0, 21, 0, NULL),
			array(NULL, NULL, 'adminCompany', '', 'TRANS_ADMINISTRATIVE_CONTACT', 'TRANS_REGISTRAR_COMPANY', '', 't', 0, 22, 0, NULL),
			array(NULL, NULL, 'adminEmail', '', 'TRANS_ADMINISTRATIVE_CONTACT', 'TRANS_REGISTRAR_EMAIL', '', 't', 0, 23, 0, NULL),
			array(NULL, NULL, 'adminAddress1', '', 'TRANS_ADMINISTRATIVE_CONTACT', 'TRANS_REGISTRAR_ADDRESS1', '', 't', 0, 24, 0, NULL),
			array(NULL, NULL, 'adminAddress2', '', 'TRANS_ADMINISTRATIVE_CONTACT', 'TRANS_REGISTRAR_ADDRESS2', '', 't', 0, 25, 0, NULL),
			array(NULL, NULL, 'adminCity', '', 'TRANS_ADMINISTRATIVE_CONTACT', 'TRANS_REGISTRAR_CITY', '', 't', 0, 26, 0, NULL),
			array(NULL, NULL, 'adminState', '', 'TRANS_ADMINISTRATIVE_CONTACT', 'TRANS_REGISTRAR_STATE', '', 't', 0, 27, 0, NULL),
			array(NULL, NULL, 'adminZip', '', 'TRANS_ADMINISTRATIVE_CONTACT', 'TRANS_REGISTRAR_ZIP', '', 't', 0, 28, 0, NULL),
			array(NULL, NULL, 'adminCountry', '', 'TRANS_ADMINISTRATIVE_CONTACT', 'TRANS_REGISTRAR_COUNTRY', '', 't', 0, 29, 0, NULL),
			array(NULL, NULL, 'adminPhone', '', 'TRANS_ADMINISTRATIVE_CONTACT', 'TRANS_REGISTRAR_PHONE', '', 't', 0, 30, 0, NULL),
			array(NULL, NULL, 'adminFax', '', 'TRANS_ADMINISTRATIVE_CONTACT', 'TRANS_REGISTRAR_FAX', '', 't', 0, 31, 0, NULL),
			// tech contact data
			array(NULL, NULL, 'techFirstName', '', 'TRANS_TECHNICAL_CONTACT', 'TRANS_REGISTRAR_FIRST_NAME', '', 't', 0, 32, 0, NULL),
			array(NULL, NULL, 'techLastName', '', 'TRANS_TECHNICAL_CONTACT', 'TRANS_REGISTRAR_LAST_NAME', '', 't', 0, 33, 0, NULL),
			array(NULL, NULL, 'techCompany', '', 'TRANS_TECHNICAL_CONTACT', 'TRANS_REGISTRAR_COMPANY', '', 't', 0, 34, 0, NULL),
			array(NULL, NULL, 'techEmail', '', 'TRANS_TECHNICAL_CONTACT', 'TRANS_REGISTRAR_EMAIL', '', 't', 0, 35, 0, NULL),
			array(NULL, NULL, 'techAddress1', '', 'TRANS_TECHNICAL_CONTACT', 'TRANS_REGISTRAR_ADDRESS1', '', 't', 0, 36, 0, NULL),
			array(NULL, NULL, 'techAddress2', '', 'TRANS_TECHNICAL_CONTACT', 'TRANS_REGISTRAR_ADDRESS2', '', 't', 0, 37, 0, NULL),
			array(NULL, NULL, 'techCity', '', 'TRANS_TECHNICAL_CONTACT', 'TRANS_REGISTRAR_CITY', '', 't', 0, 38, 0, NULL),
			array(NULL, NULL, 'techState', '', 'TRANS_TECHNICAL_CONTACT', 'TRANS_REGISTRAR_STATE', '', 't', 0, 39, 0, NULL),
			array(NULL, NULL, 'techZip', '', 'TRANS_TECHNICAL_CONTACT', 'TRANS_REGISTRAR_ZIP', '', 't', 0, 40, 0, NULL),
			array(NULL, NULL, 'techCountry', '', 'TRANS_TECHNICAL_CONTACT', 'TRANS_REGISTRAR_COUNTRY', '', 't', 0, 41, 0, NULL),
			array(NULL, NULL, 'techPhone', '', 'TRANS_TECHNICAL_CONTACT', 'TRANS_REGISTRAR_PHONE', '', 't', 0, 42, 0, NULL),
			array(NULL, NULL, 'techFax', '', 'TRANS_TECHNICAL_CONTACT', 'TRANS_REGISTRAR_FAX', '', 't', 0, 43, 0, NULL),
			// aux billing contact data
			array(NULL, NULL, 'auxBillingFirstName', '', 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_FIRST_NAME', '', 't', 0, 44, 0, NULL),
			array(NULL, NULL, 'auxBillingLastName', '', 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_LAST_NAME', '', 't', 0, 45, 0, NULL),
			array(NULL, NULL, 'auxBillingCompany', '', 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_COMPANY', '', 't', 0, 46, 0, NULL),
			array(NULL, NULL, 'auxBillingEmail', '', 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_EMAIL', '', 't', 0, 47, 0, NULL),
			array(NULL, NULL, 'auxBillingAddress1', '', 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_ADDRESS1', '', 't', 0, 48, 0, NULL),
			array(NULL, NULL, 'auxBillingAddress2', '', 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_ADDRESS2', '', 't', 0, 49, 0, NULL),
			array(NULL, NULL, 'auxBillingCity', '', 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_CITY', '', 't', 0, 50, 0, NULL),
			array(NULL, NULL, 'auxBillingState', '', 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_STATE', '', 't', 0, 51, 0, NULL),
			array(NULL, NULL, 'auxBillingZip', '', 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_ZIP', '', 't', 0, 52, 0, NULL),
			array(NULL, NULL, 'auxBillingCountry', '', 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_COUNTRY', '', 't', 0, 53, 0, NULL),
			array(NULL, NULL, 'auxBillingPhone', '', 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_PHONE', '', 't', 0, 54, 0, NULL),
			array(NULL, NULL, 'auxBillingFax', '', 'TRANS_BILLING_CONTACT', 'TRANS_REGISTRAR_FAX', '', 't', 0, 55, 0, NULL),
			// nameservers
			array(NULL, NULL, 'domainNameserver1', '', 'TRANS_REGISTRAR_NS', 'TRANS_REGISTRAR_NS1', '', 't', 0, 56, 0, array()),
			array(NULL, NULL, 'domainNameserver2', '', 'TRANS_REGISTRAR_NS', 'TRANS_REGISTRAR_NS2', '', 't', 0, 57, 0, array()),
			array(NULL, NULL, 'domainNameserver3', '', 'TRANS_REGISTRAR_NS', 'TRANS_REGISTRAR_NS3', '', 't', 0, 58, 0, array()),
			array(NULL, NULL, 'domainNameserver4', '', 'TRANS_REGISTRAR_NS', 'TRANS_REGISTRAR_NS4', '', 't', 0, 59, 0, array())
		);
		// Grab the configs set at the registrar level
		return array_merge($this->configParams, $defaultConfigParams);
	}

    private function addAdditionalRegistrantFields(array& $input)
	{
		foreach ($input["domains"] as &$domain) {
			switch ($domain["tld"]) {
			case "us":
				$domain["nexus"]["category"] = self::$nexusCategoryMap[$this->input["us_nexus"]];
				$domain["nexus"]["use"] = self::$nexusPurposeMap[$this->input["us_purpose"]];
                if ("C31" == $this->input["us_nexus"] || "C32" == $this->input["us_nexus"]) {
					$domain["nexus"]["country"] =
						"" != $this->input["global_cc_us"] ? $this->input["global_cc_us"] : $this->input["registrantCountry"];
				}
				break;
			}
		}
	}

	private function waitDomainCreation($domain, $useDNS)
	{
		// wait until GoDaddy completes domain creation
		for ($i=0; $i < self::GET_DOMAIN_INFO_ATTEMTS_NUMBER; ++$i) {
			$result = $this->getWWD()->domainInfo($domain, 1 == $useDNS ? 'dns' : false);
			if (false === $result) {
				sleep(self::POLL_RETRY_TIME);
				continue;
			}
			break;
		}
	}

	private static $nexusCategoryMap = array(
		"C11" => "citizen of US",
		"C12" => "permanent resident of US",
		"C21" => "incorporated or organized in US",
		"C31" => "foreign entity doing business in US",
		"C32" => "foreign entity with office or property in US",
	);

	private static $nexusPurposeMap = array(
		"P1" => "business use for profit",
		"P2" => "non-profit business or organization",
		"P3" => "personal",
		"P4" => "educational purposes",
		"P5" => "government purposes",
	);
}

$GLOBALS['moduleInfo'][$className = 'WildWestDomains'] = WildWestDomains::staticGetModuleInfo();
