<?php

class OrderDetailsAction extends CartAction {
	
	const OPERATION_SKIP = 'skip';
	
	public function execute($request) {
		
		sfContext::getInstance()->getConfiguration()->loadHelpers('BillingFormat');
		sfContext::getInstance()->getConfiguration()->loadHelpers('I18N');
		$totalPurchasesCount = 0;
		$customQuestionPlanIds = ResourceLocator::getInstance()->billingContactService->getPlanIDsCustomQuestions(ResourceLocator::getInstance()->cart->getPlanIDs());
		$customQuestionPurchases = empty($customQuestionPlanIds) ? array() : ResourceLocator::getInstance()->cart->getPurchasesWithCustomQuestions($customQuestionPlanIds, $totalPurchasesCount) ;
		$domainPurchases = ResourceLocator::getInstance()->cart->getDomainPurchases(true, $totalPurchasesCount);
		$certificatePurchases = ResourceLocator::getInstance()->cart->getCertificatePurchasesAndAddons(true, $totalPurchasesCount);
		$domainPurchase = $certificatePurchase = $customQuestionPurchase = NULL;
		if (!empty($domainPurchases)) {
			$domainPurchase = array_shift($domainPurchases);
		} elseif (!empty($certificatePurchases)) {
			$certificatePurchase = array_shift($certificatePurchases);
		} elseif (!empty($customQuestionPurchases)) {
			$customQuestionPurchase = array_shift($customQuestionPurchases);
		}

		if (!$totalPurchasesCount) {
			$this->_router->next(array(), false);
		}

		if ($domainPurchase) {
			$plan = $this->_store->getPlan($domainPurchase->getPlanId());
			$price = $plan->getPriceById($domainPurchase->getPriceId());
			$variantId = $price->productVariantID;
			$domainData = $domainPurchase->getParameter('domainData');
			$SLD = $domainData['SLD'];
			$action = $domainPurchase->getParameter('type');
			$this->form = ResourceLocator::getInstance()->billingDomainService->getDomainDetailsForm($this->_store->getId(), $variantId, $SLD, $action);
			$this->form->setAttribute('variantId', $variantId);
			$this->form->setAttribute('SLD', $SLD);
			$this->form->setAttribute('action', $action);
			$this->form->setAttribute('purchase', $domainPurchase);
			$domainData = $domainPurchase->getParameter('domainData');
			$this->title = __('TRANS_INFORMATION_FOR_DOMAIN', array('%NAME%' => formatProductName($domainData['name'])));
			$this->description = 'TRANS_INFORMATION_FOR_DOMAIN_DESCRIPTION';
            if ( 1 < ( $domainCount = count(ResourceLocator::getInstance()->cart->getDomainPurchases(false))) ) {
                  $this->itemsCountDescription = __('TRANS_ONLINE_STORE_DOMAIN_PURCHASES_COUNT', array("%ALL%" => $domainCount, '%CURRENT%' => ( $domainCount - count($domainPurchases))));
            }
		} else if ($certificatePurchase) {
			$planId = $certificatePurchase instanceof Purchase ? $certificatePurchase->getPlanId() : $certificatePurchase->getAddonId();
			$this->form = ResourceLocator::getInstance()->billingCertificateService->getCertificateDetailsForm($this->_store->getId(), $planId);
			$this->form->setAttribute('planId', $planId);
			$this->form->setAttribute('purchase', $certificatePurchase);
			$this->title = __('TRANS_INFORMATION_FOR_SSL_CERTIFICATE');
			$this->description = 'TRANS_INFORMATION_FOR_SSL_CERTIFICATE_DESCRIPTION';
            if ( 1 < ( $certificateCount = count(ResourceLocator::getInstance()->cart->getCertificatePurchasesAndAddons(false))) ) {
                  $this->itemsCountDescription = __('TRANS_ONLINE_STORE_CERTIFICATE_PURCHASES_COUNT', array("%ALL%" => $certificateCount, '%CURRENT%' => ( $certificateCount - count($certificatePurchases))));
            }
		} else if ($customQuestionPurchase) {
			$planId = $customQuestionPurchase->getPlanId();
			$this->form = ResourceLocator::getInstance()->billingContactService->getPlanCustomQuestionsForm(array($customQuestionPurchase->getId() => $planId));
			$this->form->setAttribute('planId', $planId);
			$this->form->setAttribute('purchase', $customQuestionPurchase);
			$this->title = __($this->form->getSection('customQuestion')->getTitle());
			$this->description =  __('TRANS_ORDERFORM_ADDITIONAL_PRODUCT_INFORMATION_DESC', array('%PLAN_NAME%' => $customQuestionPurchase->getName()));
		}

		if ($request->isMethod('post')) {
			$operation = $request->getParameter('operation');
			switch ($operation) {
				case self::OPERATION_SKIP:
					$this->_router->next();
					break;
				default:
					$this->form->bind($request->getParameter($this->form->getName()));
					if ($this->form->isValid()) {
						if ($this->form->save()) {
							if (!is_null($domainPurchases) || !is_null($certificatePurchases) || !is_null($customQuestionPurchase)) {
								$this->_router->refresh();
							} else {
								$this->_router->next();
							}
						}

					} else {
					}
			}
			
		}
	}
}