<?php

class RegisterDomainAction extends CartAction {
	
	const ACTIVITY_CHECK = 'check';
	
	const OPERATION_REGISTER = 'register';
	const OPERATION_TRANSFER = 'transfer';
	
	public function execute($request) {
		
		if ($request->isMethod('post')) {
			
			try {
				switch ($request->getParameter('activity')) {
					case self::ACTIVITY_CHECK:
						if (
							!($domainPlanId = $request->getParameter('domainPlanId')) ||
							!($SLD = trim($request->getParameter('SLD'))) ||
							!($operation = $request->getParameter('operation'))
						) {
							throw new InputException('TRANS_YOU_MUST_SPECIFY_DOMAIN_NAME');
						}
						$masterTLD = $request->getParameter('masterTLD');
						$selectDomainParameters = new sfParameterHolder();
						$selectDomainParameters->add(
							array(
								'domainPlanId' => $domainPlanId,
								'masterTLD' => $masterTLD,
								'SLD' => $SLD,
								'operation' => $operation
							)
						);
						sfContext::getInstance()->getUser()->setAttribute('selectDomainParameters', $selectDomainParameters);
						$this->_router->refresh();
						break;
					default:
						$planId = $request->getParameter('planId');
						$plan = $this->_store->getPlan($planId);
						$duplicatedDomains = array();
						$registeredDomains = array();
						if (PurchaseManager::purchaseDomains($request, $this->_store, $this->_cart, $plan, $duplicatedDomains, $registeredDomains)) {
							$this->_router->next();
						} else {
							sfContext::getInstance()->getConfiguration()->loadHelpers('I18N');
							if ($duplicatedDomains) {
								$errorMessage = __('TRANS_DUPLICATE_DOMAINS', array('%DOMAINS%' => implode(', ', $duplicatedDomains)));
							} else if($registeredDomains) {
								$errorMessage = __('TRANS_DUPLICATE_REGISTERED_DOMAINS', array('%DOMAINS%' => implode(', ', $registeredDomains)));
							} else {
								$errorMessage = 'TRANS_YOU_MUST_SELECT_AT_LEAST_ONE_DOMAIN';
							}
							throw new InputException($errorMessage);
						}
				}
			} catch (InputException $exception) {
				$this->errorMessage = $exception->getMessage();
			}
			
		}
	
		if (!$selectDomainParameters = sfContext::getInstance()->getUser()->getAttribute('selectDomainParameters')) {
			$this->_router->previous();
		}
		
		$cartItem = $this->_cart->getCurrentItem();
		
		$plan = $this->_store->getPlan($cartItem->getPlanId(), true);
		
		if ($defaultTaxZoneId = isset($this->_store->defaultTaxZoneID) ? $this->_store->defaultTaxZoneID : false) {
			$this->taxZone = ResourceLocator::getInstance()->billingTaxZoneService->getById($defaultTaxZoneId);
		}
		
		$domainPlanId = $selectDomainParameters->get('domainPlanId');
		$domainPlan = $this->_store->getPlan($domainPlanId, true);
		
		$masterTLD = $selectDomainParameters->get('masterTLD');
		$masterTLDValid = false;
		
		$domains = $domainPlan->getDomains($this->_store->getId());
		$TLDs = array();
		foreach ($domains as $domain) {
			if ($domain->tldAutoSearch) {
				$TLDs[] = $domain->tldExtension;
			}
			if ($masterTLD == $domain->tldExtension) {
				$masterTLDValid = true;
			}
		}
		$TLDs = $masterTLDValid ? array_unique(array_merge(array($masterTLD), $TLDs)) : $TLDs;
		
		$SLD = $selectDomainParameters->get('SLD');
		
		$this->SLD = $SLD;
		$this->TLDs = $TLDs;
		$this->masterTLD = $masterTLD;
		
		$this->domainName = $SLD . ($masterTLD ? '.' . $masterTLD : '');
		
		$this->plan = $plan;
		
		$this->domainPlan = $domainPlan;
		
		$this->domainPlanId = $domainPlanId;
		
		$this->operation = $selectDomainParameters->get('operation', self::OPERATION_REGISTER);
		
	}
	
}