<?php

abstract class BillingService implements Resource  {
	
	protected $_client;
	
	private static $_totalExecutionTime = 0;
	
	protected function __construct(SoapClient $client) {
		$this->_client = $client;
	}
	
	protected static function prepareSoapClient($code) {
		$url = sfConfig::get('app_billing_service_url') . '?service=' . $code;
		return new SoapClient(
			null,
			array(
				'location' => $url,
				'uri' => $url,
				'trace' => 1
			)
		); 
	}
	
	public function __call($name, $arguments) {
		if (strpos($name, 'execute') === 0) {
			// parse name of method of SOAP service
			$command = substr($name, 7);
			$command{0} = strtolower($command{0});
			// make call of method to SOAP service
			try {
				$time = microtime(true);
				$response = new BillingServiceResponse(call_user_func_array(array($this->_client, $command), $arguments));
				$executionTime = microtime(true) - $time;
				self::$_totalExecutionTime += $executionTime;
				sfContext::getInstance()->getLogger()->debug('[profile] Duration of command "' . $command . '" is ' . $executionTime . ' ms. (total: ' . self::$_totalExecutionTime . ' ms.)');
                return $response;
            }  catch (SoapFault $exception) {
				throw new BillingServiceException($exception->getMessage());
			}
		}
	}
	
}