<?php

class BillingPlan extends BillingProduct {
	
	const TYPE_HOSTING = 1;
	const TYPE_DOMAIN = 3;
	const TYPE_SSL_CERTIFICATE = 11;
	
	private $_variants = null;
	
	private $_cycles = null;
	
	public function getVariants() {
		if (is_null($this->_variants)) {
			$this->_variants = array();
			if (isset($this->productVariants)) {
				foreach ($this->productVariants as $data) {
					$variant = new BillingVariant($data);
					$this->_variants[$variant->getId()] = $variant;
				}
			}
		}
		return $this->_variants;
	}
	
	public function getCycles() {
		if (is_null($this->_cycles)) {
			$this->_cycles = array();
			if (isset($this->cycles)) {
				foreach ($this->cycles as $data) {
					$cycle = new BillingCycle($data);
					$this->_cycles[$cycle->getId()] = $cycle;
				}
			}
		}
		return $this->_cycles;
	}
	
	public function getCycle($id) {
		$cycles = $this->getCycles();
		if (isset($cycles[$id])) {
			return $cycles[$id];
		} else {
			throw new BillingObjectException('Cycle [' . $id . '] not found for plan [' . $this->getId() . ']');
		}
	}
	
	public function getVariant($id) {
		$variants = $this->getVariants();
		if (isset($variants[$id])) {
			return $variants[$id];
		} else {
			throw new BillingObjectException('Variant [' . $id . '] not found for plan [' . $this->getId() . ']');
		}
	}
	
	public function getAddonGroups($storeId, $cycleId, $anonymously = false) {
		return ResourceLocator::getInstance()->billingAddonGroupService->getByPlan($this->getId(), $storeId, $cycleId, array(), $anonymously);
	}
	
	public function getAddonGroup($addonGroupId, $storeId, $cycleId, $anonymously = false) {
		$result = null;
		foreach (ResourceLocator::getInstance()->billingAddonGroupService->getByPlan($this->getId(), $storeId, $cycleId, array(), $anonymously) as $addonGroup) {
			if ($addonGroup->getId() == $addonGroupId) {
				$result = $addonGroup;
				break;
			}
		}
		if (!$result) {
			throw new BillingObjectException('Can not retrieve addon group [' . $addonGroupId . ']');
		}
		return $result;
	}
	
	public function getAddons($storeId, $cycleId, $anonymously = false) {
		$addons = array();
		foreach ($this->getAddonGroups($storeId, $cycleId, $anonymously) as $addonGroup) {
			foreach ($addonGroup->getAddons() as $addon) {
				$addons[$addon->getId()] = $addon;
			}
		}
		return $addons;
	}
	
	public function hasAddons($storeId, $cycleId) {
		$result = false;
		foreach ($this->getAddonGroups($storeId, $cycleId) as $addonGroup) {
			if ($result = $addonGroup->hasAddons()) {
				break;
			}
		}
		return $result;
	}
	
	public function hasMultipleVariants($currencyId) {
		$result = false;
		$count = 0;
		foreach ($this->getVariants() as $variant) {
			if ($variant->hasPrices($currencyId)) {
				if ($result = ++$count > 1) {
					break;
				}
			}
		}
		return $result;
	}
	
	public function hasPrices($currencyId, $multiple = false) {
		$comparison = $multiple ? 1 : 0;
		$count = 0;
		foreach ($this->getVariants() as $variant) {
			$count += $variant->getPricesCount($currencyId);
		}
		return $count > $comparison ? true : false;
	}
	
	public function getPricesByCurrency($currencyId) {
		$prices = array();
		foreach ($this->getVariants() as $variant) {
			foreach ($variant->getPricesByCurrency($currencyId) as $price) {
				$prices[$price->getId()] = $price;
			}
		}
		return $prices;
	}
	
	public function getPricesCount($currencyId) {
		$count = 0;
		foreach ($this->getVariants() as $variant) {
			$count += $variant->getPricesCount($currencyId);
		}
		return $count;
	}
	
	public function getPriceById($id) {
		$result = null;
		foreach ($this->getVariants() as $variant) {
			if ($result = $variant->getPriceById($id)) {
				break;
			}
		}
		if ($result) {
			return $result;
		} else {
			throw new BillingObjectException('Price [' . $id . '] not found (or zero) for plan [' . $this->getId() . ']');
		}
	}
	
	public function mayHaveAddons() {
		return $this->productAddonRequired < 2 ? true : false;
	}
	
	public function mustHaveAddons() {
		return $this->productAddonRequired == 1 ? true : false;
	}

	public function hasAddAddonAvailability($cycleId) {
		return isset($this->hasAddAddonAvailability[$cycleId]) ? $this->hasAddAddonAvailability[$cycleId] : false;
	}
	
	public function mayHaveDomains() {
		return $this->productDomainRequired < 2 ? true : false;
	}
	
	public function mustHaveDomains() {
		return $this->productDomainRequired == 1 ? true : false;
	}
	
	protected function _extractId() {
		return $this->productID;
	}
	
}