<?php

class BillingDomainService extends BillingService {
	
	protected static $instance = null;
	
	public function getDomains($planId, $storeId) {
		$domains = array();
		// domains prices we always retrieve anonymously,
		// because in current implementation no need to retrieve
		// prices for concrete client with concrete tax
		$responce = $this->executeGetDomainData($storeId, $planId, null);
		if (!$responce->hasErrors()) {
			foreach ($responce->getData() as $data) {
				$domains[] = new BillingDomain($data);
			}
		}
		return $domains;
	}
	
	public function checkWhois($SLD, $TLDs) {
		$response = $this->executeCheckWhois($SLD, $TLDs);
		if ($response->hasErrors()) {
			throw new BillingObjectException('Can not perform whois check for STD "' . $SLD . '"');
		}

		return $response->getData();
	}
	
	public function getDomainDetailsForm($storeId, $variantId, $SLD, $action) {
		$response = $this->executeGetDomainParams($storeId, $variantId, $SLD, $action, ResourceLocator::getInstance()->customer->getContactID(), ResourceLocator::getInstance()->customer->getCustomerData());
		if ($response->hasErrors()) {
			throw new BillingServiceException('Can not retrieve domain details form for Store with id "' . $storeId . '"');
		}

		return new BillingDomainDetailsForm($response->getData());
	}
	
	public function collectDomainDetailsForm($storeId, $values, $variantId, $SLD, $action, $clientId = null) {
		$response = $this->executeCollectDomainParams($storeId, $values, $variantId, $SLD, $action, $clientId);
		return new CollectFormResult($response->getErrors(), $response->getViolations(), $response->getData());
	}

	public function isDomainRegistered($SLD, $TLDs) {
		$response = $this->executeIsDomainRegistered($SLD, $TLDs);
		if ($response->hasErrors()) {
			throw new BillingServiceException("Can not check if domain {$SLD}.{$TLDs} already registered");
		}
		return $response->getData();
	}
	
	public static function getInstance() {
		if (self::$instance == null) {
			self::$instance = new self(self::prepareSoapClient('domain'));
		}
		return self::$instance;
	}
	
}