<?php

class BillingPlanService extends BillingService{

	public function getById($planId, $storeId, $couponId = false, $anonymously = false) {
		$clientId = (!$anonymously && ($customer = sfContext::getInstance()->getUser()->getAttribute('customer'))) ? $customer->getId() : null;
		$customerData = sfContext::getInstance()->getUser()->getAttribute('customerData', array());
		$responce = $this->executeGetProductsWithAttributes($storeId, array($planId), $clientId, $customerData, $couponId ? array($couponId) : array());
		if ($responce->hasErrors() || !($data = $responce->getData()) || !($item = array_shift($data))) {
			throw new BillingObjectException('Plan with id "' . $planId . '" not found');
		}
        $className = $this->_getClassnameByProductType($item['productType']);
		return new $className($item);
	}
	
	public function getByStore($storeId, $planIds = array(), $anonymously = false) {
		$result = array();
		if (!$planIds) {
			if (!isset($this->_plans[$storeId])) {
				$this->_plans[$storeId] = $this->_getByStore($storeId, array(), $anonymously);
			}
			$result = $this->_plans[$storeId];
		} else {
			if (!isset($this->_plans[$storeId])) {
				$result = $this->_getByStore($storeId, $planIds, $anonymously);
			} else {
				foreach ($planIds as $planId) {
					if (isset($this->_plans[$storeId][$planId])) {
						$result[$planId] = $this->_plans[$storeId][$planId];
					}
				}
			}
		}
		return $result;
	}

	public static function getInstance() {
		if (self::$instance == null) {
			self::$instance = new self(self::prepareSoapClient('product'));
		}
		return self::$instance;
	}


	private function _getByStore($storeId, $planIds = array(), $anonymously = false) {
		if (!$anonymously && ($customer = sfContext::getInstance()->getUser()->getAttribute('customer'))) {
			$clientId = $customer->getId();
		} else {
			$clientId = null;
		}

		$plans = array();

		$responce = $this->executeGetProductsWithAttributes($storeId, $planIds, $clientId);
		if (!$responce->hasErrors()) {
			foreach ($responce->getData() as $data) {
				$className = $this->_getClassnameByProductType($data['productType']);
				$plan = new $className($data);
				$plans[$plan->getId()] = $plan;
			}
		}

		return $plans;
	}

    private function _getClassnameByProductType($productType)
    {
        switch ($productType)
        {
            case BillingPlan::TYPE_DOMAIN:
                return 'BillingDomainPlan';
            case BillingPlan::TYPE_SSL_CERTIFICATE:
                return 'BillingSSLCertificatePlan';
            default:
                return 'BillingPlan';
        }            
    }

	protected static $instance = null;
	private $_plans = array();
}