<?php

class ResourceLocator {
	
	private static $instance;
	
	private $resources;
	
	private function __construct() {
		$this->resources = array();
	}
	
	public static function getInstance() {
		if (self::$instance == null) {
			self::$instance = new self();
		}
		return self::$instance;
	}
	
	public function __set($name, $resource) {
		if ($resource instanceof Resource) {
			$this->resources[$name] = $resource;
		} else {
			throw new ResourceException('Object of class "' . get_class($resource) . '" is not a resource');
		}
	}
	
	public function __get($name) {
		if (!isset($this->resources[$name])) {
			$className = ucfirst($name);
			$reflectionClass = new ReflectionClass($className);
			if (in_array('Resource', $reflectionClass->getInterfaceNames())) {
				$this->resources[$name] = call_user_func($className . '::getInstance');
			} else {
				throw new ResourceException('Requested resource "'. $name .'" does not exist');
			}
		}
		return $this->resources[$name];
	}
	
}