<?php

class Router {
	
	private $_pages;
	
	private $_initialPage;
	
	private $_history;
	
	private $_current;
	
	public function __construct() {
		$this->_pages = array();
		$this->_initialPage = null;
		$this->_history = array();
		$this->_current = -1;
	}
	
	public function init($pages, $initialPage = null) {
		$this->_pages = $pages;
		$this->_initialPage = $initialPage ? $initialPage : $steps[sizeof($steps) - 1];
		$this->_history = array();
		$this->_current = -1;
	}
	
	public function check($code) {
		$currentPage = $this->getCurrentPage();
		if ($currentPage->getCode() != $code) {
			sfContext::getInstance()->getController()->redirect($currentPage->getUrl());
			throw new sfStopException();
		}
	}
	
	public function checkByAction($action) {
		$currentPage = $this->getCurrentPage();
		if ($currentPage->getAction() != $action) {
			sfContext::getInstance()->getController()->redirect($currentPage->getUrl());
			throw new sfStopException();
		}
	}
	
	public function refresh() {
		sfContext::getInstance()->getController()->redirect($this->getCurrentPage()->getUrl());
		throw new sfStopException();
	}
	
	public function next($skipPages = array(), $saveInHistory = true) {
		$tmp = $this->_current;
		do {
			$nextPage = $this->_current < sizeof($this->_pages) - 1 ? $this->_pages[++$tmp] : null;
		} while ($nextPage && array_search($nextPage->getCode(), $skipPages) !== false);
		if ($nextPage) {
			if ($saveInHistory && $this->_current >= 0) {
				$this->_history[] = $this->_current;
			}
			$this->_current = $tmp;
			sfContext::getInstance()->getController()->redirect($nextPage->getUrl());
			throw new sfStopException();
		} else {
			ResourceLocator::getInstance()->routersStack->pop();
			if ($router = ResourceLocator::getInstance()->routersStack->current()) {
				$router->next($skipPages, $saveInHistory);
			} else {
				return false;
			}
		}
	}
	
	public function previous() {
		if ($this->_history) {
			$nextStepIndex = array_pop($this->_history);
			$this->redirect($this->_pages[$nextStepIndex]->getCode(), false);
		} else {
			$this->_current = -1;
			ResourceLocator::getInstance()->routersStack->pop();
			sfContext::getInstance()->getController()->redirect($this->getInitialPage()->getUrl());
			throw new sfStopException();
		}
	}
	
	public function redirect($pageCode, $saveHistory = true) {
		foreach ($this->_pages as $index => $page) {
			if ($page->getCode() == $pageCode) {
				if ($saveHistory && $this->_current >= 0) {
					$this->_history[] = $this->_current;
				}
				$this->_current = $index;
				sfContext::getInstance()->getController()->redirect($page->getUrl());
				throw new sfStopException();
				break;
			}
		}
		return false;
	}
	
	public function removeStep($pageCode) {
		if ($pageCode == $this->_pages[$this->_current]->getCode()) {
			return false;
		} else {
			$pages = array();
			foreach ($this->_pages as $page) {
				if ($page->getCode() == $pageCode) {
					continue;
				}
				$pages[] = $page;
			}
			$this->_pages = $pages;
			$history = array();
			foreach ($this->_history as $index) {
				if ($this->_pages[$index]->getCode() == $pageCode) {
					continue;
				}
				$history[] = $index;
			}
			$this->_history = $history;
			return true;
		}
	}
	
	public function getInitialPage() {
		return $this->_initialPage;
	}
	
	public function getCurrentPage() {
		if ($this->_current >= 0) {
			return $this->_pages[$this->_current];
		} else {
			return $this->_initialPage;
		}
	}
	
	public function getPreviousPage() {
		if ($this->hasPreviousPage()) {
			return $this->_pages[$this->_history[sizeof($this->_history) - 1]];
		} else {
			return $this->getInitialPage();
		}
	}
	
	public function hasPreviousPage() {
		return $this->_history ? true : false;
	}
	
}