<?php

class CartItem {
	
	private $_id;
	
	private $_planId;
	
	private $_purchases;
	
	private $_masterPurchase;
	
	public function __construct($planId) {
		$this->_id = ResourceLocator::getInstance()->cart->generateItemId();
		$this->_planId = $planId;
		$this->_purchases = array();
		$this->_masterPurchase = null;
	}
	
	public function getId() {
		return $this->_id;
	}
	
	public function getPlanId() {
		return $this->_planId;
	}
	
	public function getPurchases($recursively = false) {
		if ($recursively) {
			$result = array();
			foreach ($this->_purchases as $purchase) {
				$result[$purchase->getId()] = $purchase;
				foreach ($purchase->getPurchases(true) as $subPurchase) {
					$result[$subPurchase->getId()] = $subPurchase;
				}
			}
			return $result;
		} else {
			return $this->_purchases;
		}
	}
	
	public function getAddons() {
		$result = array();
		foreach ($this->_purchases as $purchase) {
			$result = array_merge($result, $purchase->getAddons(true));
		}
		return $result;
	}
	
	public function hasPurchase($id) {
		return isset($this->_purchases[$id]) ? true : false;
	}
	
	public function getPurchase($id) {
		return isset($this->_purchases[$id]) ? $this->_purchases[$id] : null;
	}
	
	public function addPurchase(Purchase $purchase) {
		if ($masterPurchase = $this->getMasterPurchase()) {
			$masterPurchase->addPurchase($purchase);
		} else {
			$this->_purchases[$purchase->getId()] = $purchase;
		}
	}
	
	public function removePurchase($purchaseId) {
		if (isset($this->_purchases[$purchaseId])) {
			unset($this->_purchases[$purchaseId]);
		} else {
			foreach ($this->_purchases as $purchase) {
				$purchase->removePurchase($purchaseId);
			}
		}
	}
	
	public function getMasterPurchase() {
		return $this->_masterPurchase;
	}
	
	public function hasMasterPurchase() {
		return $this->_masterPurchase ? true : false;
	}
	
	public function setMasterPurchase($id) {
		if ($this->hasPurchase($id)) {
			$this->_masterPurchase = $this->getPurchase($id); 
		}
	}
	
	public function unsetMasterPurchase() {
		$this->_masterPurchase = null;
	}
	
}