<?php

abstract class PurchaseContainer {
	
	private $_planId;
	
	private $_cycleId;
	
	private $_priceId;
	
	private $_name;
	
	private $_type;
	
	private $_coupon;
	
	private $_parameters;
	
	public function __construct($planId, $cycleId, $priceId, $name, $type) {
		$this->_planId = $planId;
		$this->_cycleId = $cycleId;
		$this->_priceId = $priceId;
		$this->_name = $name;
		$this->_type = $type;
		$this->_coupon = null;
		$this->_parameters = array();
	}
	
	public function getPlanId() {
		return $this->_planId;
	}
	
	public function getCycleId() {
		return $this->_cycleId;
	}
	
	public function getPriceId() {
		return $this->_priceId;
	}
	
	public function getName() {
		return $this->_name;
	}
	
	public function getType() {
		return $this->_type;
	}
	
	public function setCoupon(BillingCoupon $coupon) {
		if (!$this->_coupon || ($this->_coupon && !$this->_coupon->preapplied())) {
			$this->_coupon = $coupon;
			return true;
		} else {
			return false;
		}
	}
	
	public function unsetCoupon() {
		$this->_coupon = null;
	}
	
	public function getCoupon() {
		return $this->_coupon;
	}
	
	public function hasCoupon() {
		return $this->_coupon ? true : false;
	}
	
	public function hasCouponByCode($code) {
		return $this->_coupon && $this->_coupon->getCode() == $code ? true : false;
	}
	
	public function addParameter($name, $value) {
		$this->_parameters[$name] = $value;
	}
	
	public function removeParameter($name) {
		if (isset($this->_parameters[$name])) {
			unset($this->_parameters[$name]);
		}
	}
	
	public function getParameter($name, $default = null) {
		return isset($this->_parameters[$name]) ? $this->_parameters[$name] : $default;
	}
	
	public function setParameter($name, $value) {
		if (isset($this->_parameters[$name])) {
			$this->_parameters[$name] = $value;
			return true;
		} else {
			return false;
		}
	}
	
	public function hasParameter($name) {
		return isset($this->_parameters[$name]) ? true : false;
	}
	
	abstract public function getId();
	
}