<?php

abstract class BillingForm extends sfForm {
	
	const SORT_ASC = 'ASC';
	
	const SORT_DESC = 'DESC';
	
	const COMMON_FORM_INPUT_ERROR_KEY = '__form';
	
	private $_data;
	
	private $_sections;
	
	private $_groups;
	
	private $_name;
	
	private $_errors = false;
	
	private $_attributes = array();

	private $_hiddenFieldsByDefault = array();

	private $_fieldsWithJs = array();
	
	public function __construct($data, $defaults = array(), $options = array(), $CSRFSecret = null) {
		$this->_data = $data;
		$this->_sections = array();
		$this->_groups = array();
		$className = get_class($this);
		$formName = substr(substr($className, 7), 0, -4);
		$formName{0} = strtolower($formName{0});
		$this->_name = $formName;
		
		parent::__construct($defaults, $options, $CSRFSecret);
	}
	
	public function configure() {
		
		sfContext::getInstance()->getConfiguration()->loadHelpers('I18N');
		
		$stylesDescriptor = sfConfig::get('mod_cart_form_styles');
		$formatterDescriptor = sfConfig::get('mod_cart_form_formatters');
		$defaultStyle = $stylesDescriptor[$this->_name ]['default'];
		
		$widgets = array();
		$validators = array();
		$defaults = array();
        $hiddenFieldsByDefault = array();
        $fieldsWithJs = array();

		foreach ($this->_data as $key => $sectionData) {
			
			$sectionTitle = isset($sectionData['title']) ? $sectionData['title'] : false;
			$sectionDescription = isset($sectionData['description']) ? $sectionData['description'] : false;
			$section = new FormSection($key, $sectionTitle, $sectionDescription);
			
			$hasMultipalGroups = isset($sectionData['groups']) ? true : false;
			
			$groups = isset($sectionData['groups']) ? $sectionData['groups'] : array('default-' . $key => $sectionData);

			foreach ($groups as $groupName => $groupData) {
				
				$groupTitle = isset($groupData['title']) ? $groupData['title'] : false;
				$groupDescription = isset($groupData['description']) ? $groupData['description'] : false;
				$group = new FormGroup($groupName, $groupTitle, $groupDescription);
				
				$inputs = isset($groupData['inputs']) ? $groupData['inputs'] : array();
				
				foreach ($inputs as $inputData) {
					$name = $inputData['id'];
					$type = $inputData['type'];
					$label = isset($inputData['title']) ? $inputData['title'] : false;
					$required = (isset($inputData['required']) && $inputData['required']) || (isset($inputData['optional-required']) && $inputData['optional-required']) ? true : false;
					$description = isset($inputData['description']) ? $inputData['description'] : false;
					$default = isset($inputData['default']) ? $inputData['default'] : false;
					$readOnly = isset($inputData['read_only']) ? 1 == $inputData['read_only'] : false;

					if (isset($stylesDescriptor[$this->_name ]['fields'][$name])) {
						$class = $stylesDescriptor[$this->_name ]['fields'][$name] ? $stylesDescriptor[$this->_name ]['fields'][$name] : false;
					} else {
						$class = $defaultStyle ? $defaultStyle : false;
					}
					$options = array();
					if ($class) {
						$options['class'] = $class;
					}
					if (isset($inputData['autocomplete']) && !$inputData['autocomplete']) {
						$options['autocomplete'] = 'off';
					}
					
					$itemName = $hasMultipalGroups ? $group->getName() . '__' . $name : $name;
					
                    if (isset($inputData['hidden']) && $inputData['hidden']) {
                        $hiddenFieldsByDefault[] = $name;
                    }
                    
					switch ($type) {
						case FormField::TYPE_SELECT:
							$choices = array();
							$tmpItems = $inputData['items'];
							$tmp = array_shift($tmpItems);
							if (is_array($tmp)) {
								foreach ($inputData['items'] as $item) {
									$tmp = $item[1];
									if (is_array($tmp)) {
										$output = '';
										foreach ($tmp as $line) {
											$output .= ($output ? ' - ' : '') . __($line);
										}
										$tmp = $output;
									}
									$choices[$item[0]] = __($tmp);
                                    if (isset($item[2])) {
                                        $fieldsWithJs[$itemName][$item[0]] = $item[2];
                                    }
								}
							}
							if (isset($inputData['sort'])) {
								if ($inputData['sort'] == self::SORT_ASC) {
                        			asort($choices);
								} else {
									arsort($choices);
								}
							}
                        	$widget = new sfWidgetFormSelect(
								array(
									'label' => $label,
									'choices' => $choices,
								),
								$options
							);
							$validator = new sfValidatorPass();
							$readOnlyLabel = $default !== false && isset($choices[$default]) ? $choices[$default] : false;
							break;
						case FormField::TYPE_RADIO:
							$choices = $hints = array();
							$options['class'] = '';
							$defaultRadioFormatter = isset($formatterDescriptor[$this->_name ]['defaultRadioFormatter']) ? $formatterDescriptor[$this->_name ]['defaultRadioFormatter'] : 'rowRadioFormatter' ;
							foreach ($inputData['items'] as $item) {
								$choices[$item[0]] = __($item[1]);
								$hints[] = isset($item[3]) ? $item[3] : "" ;
							}
							$widget = new sfWidgetFormSelectRadio(
								array(
									'label' => $label,
									'choices' => $choices,
									'formatter' => array($this, $defaultRadioFormatter)
								),
								$options
							);
							$widget->addRequiredOption('hints');
							$widget->setOption('hints',$hints);
							$validator = new sfValidatorPass();
							$readOnlyLabel = $default !== false && isset($choices[$default]) ? $choices[$default] : false;
							break;
						case FormField::TYPE_CHECKBOX:
							$options['class'] = '';
							$value = isset($inputData['value']) ? $inputData['value'] : 1 ;
							$widget = new sfWidgetFormInputCheckbox(
								array('label' => $label, 'value_attribute_value' => $value),
								$options
							);
							$validator = new sfValidatorPass();
							$readOnlyLabel = $default ? __('TRANS_YES') : __('TRANS_NO');
							break;
						case FormField::TYPE_PASSWORD:
							$widget = new sfWidgetFormInputPassword(
								array('label' => $label),
								$options
							);
							$validator = new sfValidatorPass();
							$readOnlyLabel = __('TRANS_STORE_READONLY_PASSWORD');
							break;
                        case FormField::TYPE_TEXTAREA:
							$widget = new sfWidgetFormTextarea(
								array('label' => $label),
								$options
							);
							$validator = new sfValidatorPass();
							$readOnlyLabel = nl2br($default);
							break;
						case FormField::TYPE_HTML:
							$widget = new WidgetFormHTML();
							$validator = new sfValidatorPass();
							$readOnlyLabel = $default;
							break;
						default:
							$widget = new sfWidgetFormInput(
								array('label' => $label),
								$options
							);
							$validator = new sfValidatorPass();
							$readOnlyLabel = $default;
					}
					
					$formField = new FormField($name, $type, $description, $required);
					$formField->setValue($default);
					
					// replace actual widget to hidden widget for read-only fields
					if($readOnly) {
						$widget = new sfWidgetFormInputHidden(array('label' => $label));
						$formField->setReadOnly($readOnlyLabel);
					}
					
					$group->addField($formField);

					$widgets[$itemName] = $widget;
					$validators[$itemName] = $validator;
					$defaults[$itemName] = $default;
				}
				$section->addGroup($group);
			}
			$this->_sections[$section->getName()] = $section;
		}

        	$this->setHiddenFieldsByDefault($hiddenFieldsByDefault);
		$this->setFieldsWithJs($fieldsWithJs);
		$this->setWidgets($this->addCustomWidgets($widgets));
		$this->setValidators($this->addCustomValidators($validators));
		$this->setDefaults($this->addCustomDefaults($defaults));
		$this->widgetSchema->setNameFormat($this->_name . '[%s]');
	}
	
	public function getName() {
		return $this->_name;
	}

	public function hasMultipalSections() {
		return sizeof($this->_sections) > 1 ? true : false;
	}
	
	public function getSections() {
		return $this->_sections;
	}
	
	public function getSection($name) {
		return isset($this->_sections[$name]) ? $this->_sections[$name] : null;
	}
	
	public function hasSection($name) {
		return isset($this->_sections[$name]) ? true : false;
	}
	
	public function getGroups() {
		$result = array();
		foreach ($this->getSections() as $section) {
			$result = array_merge($result, $section->getGroups());
		}
		return $result;
	}
	
	public function getGroup($name) {
		$result = null;
		foreach ($this->getSections() as $section) {
			if ($result = $section->getGroup($name)) {
				break;
			}
		}
		return $result;
	}
	
	public function getFields() {
		$result = array();
		foreach ($this->getSections() as $section) {
			foreach ($section->getGroups() as $group) {
				$result = array_merge($result, $group->getFields());
			}
		}
		return $result;
	}
	
	public function getField($name) {
		$result = null;
		foreach ($this->getSections() as $section) {
			foreach ($section->getGroups() as $group) {
				if ($result = $group->getField($name)) {
					break;
				}
			}
			if ($result) {
				break;
			}
		}
		return $result;
	}
	
	public function hasHTMLFields() {
		$result = false;
		foreach ($this->getFields() as $field) {
			if ($result = ($field->getType() == FormField::TYPE_HTML)) {
				break;
			}
		}
		return $result;
	}
	
	public function getHTMLFields() {
		$result = array();
		foreach ($this->getFields() as $field) {
			if ($field->getType() == FormField::TYPE_HTML) {
				$result[] = $field;
			}
		}
		return $result;
	}
	
	public function getFieldsByName($name) {
		$result = array();
		foreach ($this->getSections() as $section) {
			foreach ($section->getGroups() as $group) {
				if ($tmp = $group->getField($name)) {
					$result[] = $tmp;
				}
			}
		}
		return $result;
	}
	
	public function getFieldInGroup($fieldName, $groupName) {
		$result = null;
		foreach ($this->getGroups() as $group) {
			if ($group->getName() == $groupName && $result = $group->getField($fieldName)) {
				break;
			}
		}
		return $result;
	}
	
	public function colRadioFormatter($widget, $inputs) {
		return $this->radioFormatter($widget, $inputs, '<br>');
	}

	public function rowRadioFormatter($widget, $inputs) {
		return $this->radioFormatter($widget, $inputs, '&nbsp;');
	}

	public function setErrors($errors) {
		$this->_errors = $errors;
	}

	public function getErrors() {
		return $this->_errors;
	}

	public function setError($code, $message) {
		$this->_errors[] = array($code => $message);

	}

    public function setHiddenFieldsByDefault($fields) {
        $this->_hiddenFieldsByDefault = $fields;
    }

    public function getHiddenFieldsByDefault() {
        return $this->_hiddenFieldsByDefault;
    }

    public function setFieldsWithJs($fields) {
		$this->_fieldsWithJs = $fields;
	}

	public function getFieldsWithJs() {
		return $this->_fieldsWithJs;
	}

	public function setAttribute($name, $value) {
		$this->_attributes[$name] = $value;
	}
	
	public function getAttribute($name, $default = null) {
		return isset($this->_attributes[$name]) ? $this->_attributes[$name] : $default;
	}
	
	protected function addCustomWidgets($widgets) {
		return $widgets;
	}
	
	protected function addCustomValidators($validators) {
		return $validators;
	}
	
	protected function addCustomDefaults($defaults) {
		return $defaults;
	}

	protected function validate(CollectFormResult $result, $group = NULL) {
		$returnValue = true;
		if ($result->hasErrors()) {
			$returnValue = false;
			$this->setErrors($result->getErrors());
		}
		if ($result->hasViolations()) {
			$returnValue = false;
			foreach ($result->getViolations() as $name => $description) {
				if (self::COMMON_FORM_INPUT_ERROR_KEY == $name) {
					throw new BillingFormException($description);
				}
				$field = is_null($group) ? $this->getField($name) : $this->getFieldInGroup($name, $group) ;
				if ($field) {
					$field->setViolation($description);
				}
			}
		}
		return $returnValue;
	}


	private function radioFormatter($widget, $inputs, $inputSeparator) {
		$result = '';
		$i = 0;
		$hints = $widget->getOption('hints');
		foreach ($inputs as $input) {
			$result .= ($result ? $inputSeparator : '') . '<label>' . $input['input'] . ' ' . $input['label'] . '</label>';
			if (!is_null($hints) && "" != ($hint = $hints[$i])) {
				$result .= '<div class="hint">'.$hint.'</div>';
			}
			$i++;
		}
		return $result;
	}

	abstract public function save();
	
}